/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.dao.impl;

import java.util.List;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.kosmos.commentaire.bean.CommentaireBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;

public class CommentaireDAO extends AbstractFicheDAO<CommentaireBean> {

    public CommentaireDAO() {
        tableName = "COMMENTAIRE";
    }

    /**
     * Récupère en base la liste des commentaires par l'id meta de la fiche commentée.
     * @param idMeta l'id meta de la fiche commentée
     * @return une {@link List<CommentaireBean>} correspondant à l'idMeta
     * @throws DataSourceException
     */
    public List<CommentaireBean> getByIdMeta(final Long idMeta) throws DataSourceException {
        initColumns();
        MapSqlParameterSource parameterSource = new MapSqlParameterSource();
        StringBuilder request = new StringBuilder(" WHERE T1.ID_META = :idMeta ");
        parameterSource.addValue("idMeta", idMeta);
        String query = String.format(SELECT_DISTINCT, columnsNameForRequest, tableName) + request.toString();
        return select(query, parameterSource);
    }
}
