/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.misc;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.AbstractRechercheExterne;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.utils.RechercheFicheHelper;

import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereDate;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVideFormater;

@Deprecated
public class RechercheCommentaire extends AbstractRechercheExterne {

    private static final Logger LOG = LoggerFactory.getLogger(RechercheCommentaire.class);

    private final ServiceMetatag serviceMetatag;

    public RechercheCommentaire() {
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
    }

    /**
     * Prepare le formulaire de recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if preparer recherche
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public boolean preparerRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
        boolean rechercheEffectuee = false;
        final String objet = StringUtils.defaultString(infoBean.getString("OBJET"), "").toUpperCase();
        if (objet.equals("COMMENTAIRE")) {
            rechercheEffectuee = true;
        }
        return rechercheEffectuee;
    }

    /**
     * Traite la recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if traiter recherche
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public boolean traiterRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
        boolean rechercheEffectuee = false;
        final String objet = infoBean.getString("OBJET");
        if (objet.equalsIgnoreCase("COMMENTAIRE")) {
            final List<CritereRecherche> criteres = new ArrayList<>();
            criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "TITRE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereDate(infoBean, "DATE_COMMENTAIRE"));
            final String idMeta = infoBean.getString("ID_META");
            if (StringUtils.isNotBlank(idMeta) && StringUtils.isNumeric(idMeta)) {
                String libelleIdMeta = StringUtils.EMPTY;
                final MetatagBean metatag = serviceMetatag.getById(Long.valueOf(idMeta));
                if (metatag != null) {
                    try {
                        libelleIdMeta = metatag.getMetaLibelleFiche();
                    } catch (final Exception e) {
                        LOG.error("impossible de récuperer la fiche d'idmeta " + idMeta, e);
                    } finally {
                        criteres.add(new CritereRecherche("ID_META", idMeta, libelleIdMeta));
                        criteres.add(new CritereRecherche("LIBELLE_ID_META", libelleIdMeta));
                    }
                }
            }
            rechercheEffectuee = true;
            // attribut commun à toute les fiches
            criteres.addAll(getCritereDefaut(infoBean));
            infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
            gestionAncienFront(infoBean, criteres);
        }
        return rechercheEffectuee;
    }

    @Override
    public List<String> getCriteresRequete(String objet, boolean listeIncluse) {
        return new ArrayList<>();
    }
}
