/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.module;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.kosmos.commentaire.om.Commentaire;
import com.kosmos.commentaire.util.CommentaireUtil;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheObjet;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceUser;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;

public class PluginFicheCommentaire extends PluginFicheObjet {

    @Override
    public void supprimerObjets(final FicheUniv ficheUniv, final MetatagBean meta, final String classeObjet) throws Exception {
        if (!(ficheUniv instanceof Commentaire) && !"0".equals(meta.getMetaForum())) {
            super.supprimerObjets(ficheUniv, meta, Commentaire.class.getName());
        }
    }

    @Override
    public void synchroniserObjets(final FicheUniv ficheUniv, final MetatagBean meta, final String classeObjet) throws Exception {
        if (!(ficheUniv instanceof Commentaire) && !"0".equals(meta.getMetaForum())) {
            super.synchroniserObjets(ficheUniv, meta, Commentaire.class.getName());
        }
    }

    @Override
    public void traiterPrincipal(final Map<String, Object> infoBean, final FicheUniv ficheUniv, final MetatagBean meta) throws Exception {
        // dans le cas d'un commentaire si la saisie était anonyme on supprime la fiche
        if (ficheUniv instanceof Commentaire && ficheUniv.getCodeRedacteur().equals(ServiceUser.UTILISATEUR_ANONYME)) {
            ficheUniv.setEtatObjet("0004");
            ficheUniv.update();
            serviceMetatag.synchroniser(meta, ficheUniv.getPersistanceBean(), true);
            serviceMetatag.save(meta);
            serviceMetatag.addHistory(meta, MetatagUtils.HISTORIQUE_SUPPRESSION, ContexteUtil.getContexteUniv().getCode(), "0004");
        }
    }

    @Override
    public void preparerPrincipal(final Map<String, Object> infoBean, final FicheUniv ficheUniv, final MetatagBean meta) throws Exception {
        /*
         * pour les fiches avec forum, preparation de la case a cocher
         * de contribution
         */
        if (!(ficheUniv instanceof Commentaire)) {
            final String codeTypeObjet = ReferentielObjets.getCodeObjetParClasse(ficheUniv.getClass().getName());
            /* optionforum vaut " ","0","1","2" : desactive, inactif, affiche, actif et affiche */
            final String optionforum = Commentaire.getModeForum(codeTypeObjet);
            Boolean actif = StringUtils.isNotEmpty(optionforum);
            String typeforum = CommentaireUtil.getProperty("commentaire.anonyme");
            if (typeforum == null) {
                typeforum = "1";
            }
            //on dit si la case a cocher doit etre affichee ou non
            infoBean.put("FORUM_ACTIF", actif);
            //on donne la valeur initial de la case a cocher
            if (infoBean.get(InfoBean.ETAT_OBJET).equals(InfoBean.ETAT_OBJET_CREATION)) {
                infoBean.put("FORUM", optionforum);
                infoBean.put("FORUM_ANO", typeforum);
            } else {
                infoBean.put("FORUM", meta.getMetaForum());
                String metaAno = "";
                if (meta.getMetaForumAno() != null) {
                    metaAno = meta.getMetaForumAno();
                }
                if (metaAno.length() > 0) {
                    infoBean.put("FORUM_ANO", metaAno);
                } else {
                    infoBean.put("FORUM_ANO", typeforum);
                }
            }
        } else {
            infoBean.put("FORUM_ACTIF", false);
        }
    }
}
