/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.module;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.commentaire.om.Commentaire;
import com.kosmos.commentaire.util.CommentaireUtil;
import com.kportal.extension.module.plugin.objetspartages.DefaultPluginSaisieFiche;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.services.ServiceMetatag;

public class PluginSaisieCommentaire extends DefaultPluginSaisieFiche {

    private ServiceMetatag serviceMetatag;

    public void setServiceMetatag(final ServiceMetatag serviceMetatag) {
        this.serviceMetatag = serviceMetatag;
    }

    @Override
    public void preTraiterAction(final Map<String, Object> infoBean, final FicheUniv ficheUniv, MetatagBean meta) throws Exception {
        if (ficheUniv instanceof Commentaire) {
            if (infoBean.get("ID_META") != null && StringUtils.isNotEmpty((String) infoBean.get("ID_META"))) {
                try {
                    meta = serviceMetatag.getById(new Long((String) infoBean.get("ID_META")));
                } catch (final Exception e) {
                    throw new ErreurApplicative(CommentaireUtil.getMessage("ERREUR_SAISIE_COMMENTAIRE_FICHE_INEXISTANTE"));
                }
            }
            //si la fiche parent a spécifié le caractère anonyme/pas anonyme, on le récupère
            boolean saisieAnonyme = false;
            if (meta != null && StringUtils.isNotBlank(meta.getMetaForumAno())) {
                saisieAnonyme = "1".equals(meta.getMetaForumAno());
            } else {
                //sinon on regarde ce qui est indiqué dans le jtf.properties
                //par defaut, les commentaires sont anonymes (contrairement aux autres objets)
                final String commentAnonyme = CommentaireUtil.getProperty("commentaire.anonyme");
                saisieAnonyme = !"0".equals(commentAnonyme);
            }
            infoBean.put("SAISIE_ANONYME", saisieAnonyme);
        }
    }
}
