/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.module.service.beanexport;

import java.util.ArrayList;
import java.util.List;

import com.kosmos.commentaire.bean.CommentaireBean;
import com.kosmos.commentaire.service.ServiceCommentaire;
import com.kportal.extension.module.bean.BeanExportMap;
import com.kportal.extension.module.bean.FicheBeanExport;
import com.kportal.extension.module.dto.ServiceBeanExportDto;
import com.kportal.extension.module.service.impl.beanexport.ServiceBeanExportFiche;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.ContexteDao;

/**
 * Service spécialisé dans l'export des fiche commentaires.
 * A l'export, les fiches commentaires sont considérées comme des plugin de fiche pour le périmètre.
 * Il faut donc n'extraire que les commentaires des fiches commentées extraites.
 * Le reste de l'export se base sur l'export classique d'une fiche.
 */
public class ServiceBeanExportCommentaire extends ServiceBeanExportFiche {

    private ServiceCommentaire serviceCommentaire;

    @Override
    public BeanExportMap<FicheBeanExport<?>> getBeansByRubrique(final ServiceBeanExportDto serviceBeanExportDto) throws Exception {
        final BeanExportMap<FicheBeanExport<?>> fichesBeans = new BeanExportMap<>();
        final String codeObject = ReferentielObjets.getCodeObjetParClasseBean(CommentaireBean.class);
        try (ContexteDao ctx = new ContexteDao()) {
            List<MetatagBean> treatedMetas = new ArrayList<>();
            for (MetatagBean meta : serviceBeanExportDto.getTreatedMeta()) {
                List<CommentaireBean> comments = serviceCommentaire.getByIdMeta(meta.getId());
                for (CommentaireBean comment : comments) {
                    MetatagBean metatag = serviceMetatag.getByCodeAndIdFiche(codeObject, comment.getId());
                    processContent(serviceBeanExportDto, metatag, fichesBeans);
                    treatedMetas.add(meta);
                }
            }
            serviceBeanExportDto.getTreatedMeta().addAll(treatedMetas);
        }
        return fichesBeans;
    }

    public void setServiceCommentaire(final ServiceCommentaire serviceCommentaire) {
        this.serviceCommentaire = serviceCommentaire;
    }
}