/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.om;

import java.util.Collection;
import java.util.Date;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.commentaire.bean.CommentaireBean;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.kportal.extension.module.plugin.objetspartages.om.ObjetPluginContenu;
import com.kportal.extension.module.plugin.objetspartages.om.ObjetPluginContenuHelper;
import com.kportal.extension.module.plugin.objetspartages.om.SousObjet;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheObjet;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.Chaine;
import com.univ.utils.DateUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;
import com.univ.utils.sql.operande.TypeOperande;

/**
 * The Class Commentaire.
 */
@FicheAnnotation(isLienInterne = false, isLienRequete = false, isEncadreRecherche = false, isEncadreRechercheEmbarquable = false, isFicheFrontOffice = false)
public class Commentaire extends AbstractFicheRestriction<CommentaireBean> implements FicheObjet, Cloneable {

    private static final Logger LOG = LoggerFactory.getLogger(Commentaire.class);

    /**
     * Gets the libelle affichable.
     *
     * @param ctx    the ctx
     * @param codes  the codes
     * @param langue the langue
     * @return the libelle affichable
     * @throws Exception the exception
     * @deprecated Méthode plus utilisée, à supprimer
     */
    @Deprecated
    public static String getLibelleAffichable(final OMContext ctx, final String codes, final String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (codes == null || codes.length() == 0) {
            return res;
        }
        final Commentaire commentaire = new Commentaire();
        commentaire.init();
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = commentaire.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = commentaire.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (commentaire.nextItem()) {
                    res += commentaire.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Détermine si un forum est actif.
     *
     * @param codeTypeObjet the code type objet
     * @return -1,0,1,2 : desactive, inactif, affiche, actif et affiche
     */
    public static String getModeForum(final String codeTypeObjet) {
        final String nomObjet = ReferentielObjets.getNomObjet(codeTypeObjet);
        String optionforum = ReferentielObjets.getObjetByCode(codeTypeObjet).getExtension().getProperty("forum." + nomObjet);
        if (optionforum == null) {
            optionforum = StringUtils.EMPTY;
        } else if (optionforum.equals("-1")) {
            optionforum = StringUtils.EMPTY;
        }
        return optionforum;
    }

    /**
     * Initialisation de l'objet.
     */
    @Override
    public void init() {
        super.init();
        setIdMeta((long) 0);
        setIdParent((long) 0);
        setTexte(StringUtils.EMPTY);
        setAuteur(StringUtils.EMPTY);
        setEmailAuteur(StringUtils.EMPTY);
        setNomSite(StringUtils.EMPTY);
        setUrlSite("http://");
        setCodeRattachementAutres(StringUtils.EMPTY);
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.om.DescriptionCommentaire#getContenu()
     * @deprecated Méthode plus utilisée, appeler getTexte()
     */
    @Deprecated
    public String getContenu() {
        return getTexte();
    }

    /**
     * A partir d'un commentaire, renvoie le parent direct sous forme de ficheUniv (peut etre un autre commentaire, ou un article...)
     *
     * @return the fiche univ
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public FicheUniv renvoyerFicheParente() throws Exception {
        return ObjetPluginContenuHelper.renvoyerFicheParente(this);
    }

    /**
     * A partir d'un commentaire, renvoie le parent direct
     *
     * @return the fiche univ
     *
     * @throws Exception
     *             the exception
     * @deprecated Méthode plus utilisée, les commentaires sont affichés à plat
     */
    @Deprecated
    public FicheUniv getCommentaireParent() throws Exception {
        FicheUniv res = null;
        if (!getIdParent().equals(new Long(0))) {
            final CommentaireBean commentaireBean = commonDao.getById(getIdParent());
            final Commentaire commentaire = new Commentaire();
            commentaire.setPersistenceBean(commentaireBean);
            res = commentaire;
        }
        return res;
    }

    @Override
    public int selectByMeta(final Long idMeta) {
        final RequeteSQL requeteSelect = new RequeteSQL();
        final ClauseWhere where = new ClauseWhere();
        where.setPremiereCondition(ConditionHelper.egal("ID_META", idMeta, TypeOperande.LONG));
        where.and(ConditionHelper.egalVarchar("ETAT_OBJET", EtatFiche.EN_LIGNE.getEtat()));
        requeteSelect.where(where);
        // ajout de l'ordre de tri
        requeteSelect.orderBy(new ClauseOrderBy("DATE_CREATION", SensDeTri.DESC));
        return super.select(requeteSelect.formaterRequete());
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     * @deprecated Méthode plus utilisée, à supprimer
     */
    @Deprecated
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Récupération du libellé à afficher
     * @see com.univ.objetspartages.om.FicheUniv#getLibelleAffichable()
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Traitement d'une requête sur l'objet
     * @param requete La requête
     * @see com.univ.objetspartages.om.FicheUniv#traiterRequete(java.lang.String)
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Récupération des paramètres de la requete
        final String titre = RequeteUtil.renvoyerParametre(requete, "TITRE");
        final Date dateCommentaire = DateUtil.parseDate(RequeteUtil.renvoyerParametre(requete, "DATE_COMMENTAIRE"));
        final String idMeta = RequeteUtil.renvoyerParametre(requete, "ID_META");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        return select(titre, dateCommentaire, idMeta, langue, codeRedacteur, etatObjet, "");
    }

    /**
     * Selection d'une instance de l'objet Commentaire à partir de l'ensemble des critères combinés
     *
     * @param titre the titre
     * @param dateCommentaire the date commentaire
     * @param idMeta the id meta
     * @param langue the langue
     * @param codeRedacteur the code redacteur
     * @param etatObjet the etat objet
     * @param code the code
     * @return the number of results
     * @throws Exception the exception
     */
    public int select(final String titre, final Date dateCommentaire, final String idMeta, final String langue, final String codeRedacteur, final String etatObjet, final String code) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (StringUtils.isNotEmpty(titre)) {
            where.setPremiereCondition(ConditionHelper.rechercheMots("TITRE", titre));
        }
        if (StringUtils.isNotEmpty(idMeta)) {
            where.and(ConditionHelper.egalVarchar("ID_META", idMeta));
        }
        // TODO : faire évoluer en proposant une fourchette de dates
        if (Formateur.estSaisie(dateCommentaire)) {
            where.and(ConditionHelper.egal("DATE(DATE_CREATION)", dateCommentaire, TypeOperande.DATE));
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("CODE", code));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, "");
        requeteSelect.orderBy(new ClauseOrderBy("DATE_CREATION", SensDeTri.ASC));
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, "");
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    public Long getIdCommentaire(){
        return persistenceBean.getId();
    }

    public void setIdCommentaire(Long id){
        persistenceBean.setId(id);
    }

    @Deprecated
    public String getFormatedTexte() {
        return Chaine.formatToHtml(persistenceBean.getTexte());
    }

    @Override
    public Long getIdObjet() {
        return persistenceBean.getId();
    }

    @Override
    public void setIdObjet(final Long idObjet) {
        persistenceBean.setId(idObjet);
    }

    @Override
    public String getCle() {
        return getCode();
    }

    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    @Override
    public ObjetPluginContenu cloneObjet() {
        try {
            return (ObjetPluginContenu) super.clone();
        } catch (final CloneNotSupportedException e) {
        }
        return null;
    }

    @Override
    public Collection<SousObjet> getSousObjets() {
        return null;
    }

    /**
     * Gets the code parent.
     *
     * @return the code parent
     */
    public Long getIdMeta() {
        return persistenceBean.getIdMeta();
    }

    /**
     * Gets the code principal.
     *
     * @return the code principal
     */
    @Deprecated
    public Long getIdParent() {
        return persistenceBean.getIdParent();
    }

    /**
     * Gets the texte.
     *
     * @return the texte
     */
    public String getTexte() {
        return persistenceBean.getTexte();
    }

    /**
     * Gets the auteur.
     *
     * @return the auteur
     */
    public String getAuteur() {
        return persistenceBean.getAuteur();
    }

    /**
     * Gets the email auteur.
     *
     * @return the email auteur
     */
    public String getEmailAuteur() {
        return persistenceBean.getEmailAuteur();
    }

    /**
     * Gets the nom site.
     *
     * @return the nom site
     */
    @Deprecated
    public String getNomSite() {
        return persistenceBean.getNomSite();
    }

    /**
     * Gets the url site.
     *
     * @return the url site
     */
    public String getUrlSite() {
        return persistenceBean.getUrlSite();
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    /**
     * Sets the code parent.
     *
     * @param _idMeta
     *            the new code parent
     */
    public void setIdMeta(Long _idMeta) {
        this.persistenceBean.setIdMeta(_idMeta);
    }

    /**
     * Sets the code principal.
     *
     * @param _idParent
     *            the new code principal
     */
    public void setIdParent(Long _idParent) {
        this.persistenceBean.setIdParent(_idParent);
    }

    /**
     * Sets the texte.
     *
     * @param _texte
     *            the new texte
     */
    public void setTexte(String _texte) {
        this.persistenceBean.setTexte(_texte);
    }

    /**
     * Sets the auteur.
     *
     * @param _auteur
     *            the new auteur
     */
    public void setAuteur(String _auteur) {
        this.persistenceBean.setAuteur(_auteur);
    }

    /**
     * Sets the email auteur.
     *
     * @param _emailAuteur
     *            the new email auteur
     */
    public void setEmailAuteur(String _emailAuteur) {
        this.persistenceBean.setEmailAuteur(_emailAuteur);
    }

    /**
     * Sets the nom site.
     *
     * @param _nomSite
     *            the new nom site
     */
    public void setNomSite(String _nomSite) {
        this.persistenceBean.setNomSite(_nomSite);
    }

    /**
     * Sets the url site.
     *
     * @param _urlSite
     *            the new url site
     */
    public void setUrlSite(String _urlSite) {
        this.persistenceBean.setUrlSite(_urlSite);
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param _codeRattachementAutres
     *            the new code rattachement autres
     */
    public void setCodeRattachementAutres(String _codeRattachementAutres) {
        this.persistenceBean.setCodeRattachementAutres(_codeRattachementAutres);
    }

}
