/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.service;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.commentaire.bean.CommentaireBean;
import com.kosmos.commentaire.dao.impl.CommentaireDAO;
import com.kosmos.service.ServiceContentRedirect;
import com.kosmos.service.impl.ServiceFiche;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.FicheUnivHelper;

/**
 * Created on 04/10/17.
 */
public class ServiceCommentaire extends ServiceFiche<CommentaireBean, CommentaireDAO, FicheDefaultViewModel> implements ServiceContentRedirect<CommentaireBean> {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceCommentaire.class);

    @Override
    public String getLibelleAffichable(CommentaireBean bean) {
        FicheUniv fiche = null;
        try {
            fiche = FicheUnivHelper.getFicheParIdMeta(bean.getIdMeta());
        } catch (Exception e) {
            LOG.error("Fiche non trouvée : " + bean.getIdMeta(), e);
        }
        String libelle = (fiche != null) ? fiche.getLibelleAffichable() : "Commentaire fiche " + bean.getIdMeta();
        final DateFormat df = new SimpleDateFormat("yyyy/MM/dd - HH:mm:ss");
        libelle += " (" + df.format(bean.getDateCreation()) + ")";
        return libelle;
    }

    @Override
    public String getUrlRedirect(CommentaireBean commentaireBean) {
        final ServiceMetatag serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
        final UrlProvider urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
        final MetatagBean metaParent = serviceMetatag.getById(commentaireBean.getIdMeta());
        if (metaParent != null) {
            return urlProvider.provideAbsoluteUrl(metaParent) + "#commentaire" + commentaireBean.getId();
        }
        return StringUtils.EMPTY;
    }

    @Override
    public boolean isContentVisible(CommentaireBean bean) {
        return false;
    }

    /**
     * Récupère la liste des Commentaires d'une fiche commentée.
     * @param idMeta l'id meta de la fiche commentée
     * @return une {@link List<CommentaireBean>} correspondant à l'idMeta
     */
    public List<CommentaireBean> getByIdMeta(Long idMeta) {
        return dao.getByIdMeta(idMeta);
    }
}
