/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: commentaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.commentaire.util;

import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.extension.ExtensionHelper;
import com.kportal.extension.IExtension;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.Metatag;
import com.univ.objetspartages.om.PermissionBean;

public class CommentaireUtil {

    public static final String ID_EXTENSION = "commentaire";

    private static final String AJOUT_COMMENTAIRE_ACTIF = "2";

    private static final String AFFICHAGE_COMMENTAIRE_ACTIF = "1";

    public static boolean isGestionnaireForum(final AutorisationBean autorisations) {
        boolean res = Boolean.FALSE;
        final PermissionBean permission = new PermissionBean("TECH", "gfo", "");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = Boolean.TRUE;
        }
        return res;
    }

    public static IExtension getExtension() {
        return ExtensionHelper.getExtensionManager().getExtension(ID_EXTENSION);
    }

    public static String getProperty(final String key) {
        return PropertyHelper.getProperty(ID_EXTENSION, key);
    }

    public static String getMessage(final String key) {
        return MessageHelper.getMessage(ID_EXTENSION, key);
    }

    /**
     * @deprecated Utilisez {@link #isCommentaireSaisissable(MetatagBean)}
     */
    @Deprecated
    public static boolean isCommentaireSaisissable(final Metatag metaCourant) {
        return metaCourant != null && AJOUT_COMMENTAIRE_ACTIF.equals(metaCourant.getMetaForum());
    }

    /**
     * @deprecated Utilisez {@link #isCommentaireAffichable(MetatagBean)}
     */
    @Deprecated
    public static boolean isCommentaireAffichable(final Metatag metaCourant) {
        return metaCourant != null && (AJOUT_COMMENTAIRE_ACTIF.equals(metaCourant.getMetaForum()) || AFFICHAGE_COMMENTAIRE_ACTIF.equals(metaCourant.getMetaForum()));
    }

    public static boolean isCommentaireSaisissable(final MetatagBean metaCourant) {
        return metaCourant != null && AJOUT_COMMENTAIRE_ACTIF.equals(metaCourant.getMetaForum());
    }

    public static boolean isCommentaireAffichable(final MetatagBean metaCourant) {
        return metaCourant != null && (AJOUT_COMMENTAIRE_ACTIF.equals(metaCourant.getMetaForum()) || AFFICHAGE_COMMENTAIRE_ACTIF.equals(metaCourant.getMetaForum()));
    }
}
