/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.components.tag;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.components.model.EditableViewModel;
import com.kosmos.components.tags.AbstractComponentTag;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.frontoffice.util.JSPIncludeHelper;

public class EditableTag extends AbstractComponentTag<EditableViewModel> {

    private String value;

    private boolean editAccess;

    private String endPath;

    public EditableTag() {
        this.path = String.format("/extensions/%s/adminsite/components/editable/editable_start.jsp", OffreFormationUtil.ID_EXTENSION);
        this.endPath = String.format("/extensions/%s/adminsite/components/editable/editable_end.jsp", OffreFormationUtil.ID_EXTENSION);
    }

    public void setValue(String value) {
        this.value = value;
    }

    public void setEditAccess(boolean editAccess) {
        this.editAccess = editAccess;
    }

    @Override
    public int doStartTag() throws JspException {
        super.doStartTag();
        return editOption!=-1?EVAL_BODY_INCLUDE:SKIP_BODY;
    }

    @Override
    protected EditableViewModel buildModel() {
        EditableViewModel model = new EditableViewModel();
        final String name = StringUtils.defaultIfBlank(OffreFormationUtil.getMessage(label), label);
        model.setLabel(name);
        model.setValue(value);
        model.setEditAccess(editAccess);
        return model;
    }

    @Override
    public int doEndTag() throws JspException {
        if ( editOption!=-1 ) {
            JSPIncludeHelper.includeJsp(pageContext.getOut(), pageContext.getServletContext(), (HttpServletRequest) pageContext.getRequest(), (HttpServletResponse) pageContext.getResponse(), endPath);
        }
        return SKIP_BODY;
    }
}
