/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.dao.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.jsbsoft.jtf.datasource.dao.impl.mysql.AbstractLegacyDAO;
import com.jsbsoft.jtf.datasource.exceptions.AddToDataSourceException;
import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.jsbsoft.jtf.datasource.utils.DaoUtils;
import com.kosmos.fil.bean.FilBean;

/**
 * Created on 06/01/15.
 */
public class FilDAO extends AbstractLegacyDAO<FilBean> {

    private static final Logger LOG = LoggerFactory.getLogger(FilDAO.class);

    public FilDAO() {
        this.tableName = "FIL";
    }

    public FilBean getByCode(String code) {
        FilBean enrollmentModelMetaBean = null;
        try {
            SqlParameterSource params = new MapSqlParameterSource("code", code);
            enrollmentModelMetaBean = namedParameterJdbcTemplate.queryForObject("SELECT * FROM FIL WHERE CODE = :code", params, rowMapper);
        } catch (EmptyResultDataAccessException e) {
            LOG.debug(String.format("empty resultset size for code  %s on table %s", code, tableName), e);
        } catch (IncorrectResultSizeDataAccessException e) {
            LOG.debug("incorrect resultset size",e);
            LOG.error(String.format("Incorrect resultset size for fil with code %s", code));
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for FIL row for fil with code %s", code), dae);
        }
        return enrollmentModelMetaBean;
    }

    public List<FilBean> getAll() {
        try {
            return namedParameterJdbcTemplate.query("SELECT * FROM FIL ORDER BY TYPE_FIL,INTITULE", rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting for all ENROLLMENT_MODEL_META row in the database", dae);
        }
    }

    public List<FilBean> getByCodesRubrique(List<String> codesRubrique) {
        final List<FilBean> results = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(codesRubrique)) {
            try {
                final MapSqlParameterSource params = new MapSqlParameterSource("codes", codesRubrique);
                results.addAll(namedParameterJdbcTemplate.query("SELECT * FROM FIL WHERE CODE_RUBRIQUE IN (:codes)", params, rowMapper));
            } catch (DataAccessException dae) {
                throw new DataSourceException("An error occured requesting for all ENROLLMENT_MODEL_META row in the database", dae);
            }
        }
        return results;
    }

    public List<FilBean> getByType(String type) {
        List<FilBean> results;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("type", type);
            results = namedParameterJdbcTemplate.query("SELECT * FROM FIL WHERE TYPE_FIL = :type", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting for all ENROLLMENT_MODEL_META row in the database", dae);
        }
        return results;
    }

    public FilBean addWithForcedId(FilBean bean) {
        try {
            namedParameterJdbcTemplate.update(DaoUtils.getAddWithForcedIdQuery(tableName, getColumns(), columnNamingStrategy), getParameters(bean));
        } catch (DataAccessException dae) {
            throw new AddToDataSourceException(String.format("Unable to add [%s] to table \"%s\"", bean.toString(), tableName), dae);
        }
        return bean;
    }
}
