/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.datagrid;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.fil.bean.FilBean;
import com.kosmos.fil.service.impl.ServiceFil;
import com.univ.datagrid.bean.ResultatDatagrid;
import com.univ.datagrid.processus.AbstractServiceDatagrid;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.utils.SessionUtil;

public class ManualListToolboxDatagrid extends AbstractServiceDatagrid {

    /** l'id Spring du bean. */
    public static final String ID_BEAN = "manualListToolboxDatagrid";

    private static final Logger LOG = LoggerFactory.getLogger(ManualListToolboxDatagrid.class);

    private ServiceFil serviceFil;

    public void setServiceFil(final ServiceFil serviceFil) {
        this.serviceFil = serviceFil;
    }

    public List<ResultatDatagrid> mapperResultatRecherche(final List<FilBean> fils) {
        final Map<String, String> types = serviceFil.getTypesFils();
        final List<ResultatDatagrid> resultats = new ArrayList<>();
        for(FilBean currentFilBean : fils) {
            final ResultManualListToolboxDataGrid result = new ResultManualListToolboxDataGrid();
            result.setId(currentFilBean.getId());
            result.setLibelle(currentFilBean.getIntitule());
            result.setRubrique(currentFilBean.getCodeRubrique());
            result.setTypeObjet(types.get(currentFilBean.getTypeFil()));
            result.setValue("[traitement;requete;objet=" + currentFilBean.getTypeFil() + "#FIL=" + currentFilBean.getCode());
            result.setCodeFil(currentFilBean.getCode());
            resultats.add(result);
        }
        return resultats;
    }

    @Override
    public List<ResultatDatagrid> traiterRechercheDepuisRequete(final HttpServletRequest req) {
        final AutorisationBean autorisations = (AutorisationBean) SessionUtil.getInfosSession(req).get(SessionUtilisateur.AUTORISATIONS);
        if (autorisations == null) {
            return Collections.emptyList();
        }
        final String type = req.getParameter("TYPE");
        List<ResultatDatagrid> resultats;
        if(StringUtils.isNotBlank(type)) {
            resultats = mapperResultatRecherche(serviceFil.getManualListByType(type));
        } else {
            resultats = mapperResultatRecherche(serviceFil.getAll());
        }
        return resultats;
    }
}
