/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.module;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.textsearch.ResultatRecherche;
import com.kosmos.fil.bean.FilBean;
import com.kosmos.fil.service.impl.ServiceFil;
import com.kportal.extension.module.plugin.objetspartages.DefaultPluginRecherche;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.RequeteUtil;

public class PluginRechercheFil extends DefaultPluginRecherche {

    private ServiceFil serviceFil;

    public void setServiceFil(ServiceFil serviceFil) {
        this.serviceFil = serviceFil;
    }

    @Override
    public boolean annuleCritereNombre(final String requete) {
        return isActif(requete);
    }

    @Override
    public boolean isActif(final String requete) {
        final String codeFil = RequeteUtil.renvoyerParametre(requete, "FIL");
        return codeFil.length() > 0;
    }

    @Override
    public List<ResultatRecherche> traiterRequete(final OMContext ctx, final String requete, final List<ResultatRecherche> res) {
        String listeFiches = "";
        final String objet = RequeteUtil.renvoyerParametre(requete, "OBJET");
        //on va stocker dans le hashtable les fiches du fil et leur ordre
        final Hashtable<String, Integer> hListeFichesFil = new Hashtable<>();
        final String codeFil = RequeteUtil.renvoyerParametre(requete, "FIL");
        if (codeFil != null && codeFil.length() > 0) {
            /* recup du fil */
            final FilBean fil = serviceFil.getByCode(codeFil);
            if (fil != null) {
                listeFiches = fil.getListeFiches();
            }
            final StringTokenizer stListeFiches = new StringTokenizer(listeFiches, ";");
            /* on stocke dans l'ordre les fiches du fil */
            int iListe = 0;
            while (stListeFiches.hasMoreTokens()) {
                hListeFichesFil.put(stListeFiches.nextToken(), iListe);
                iListe++;
            }
            final TreeMap<Integer, Long> tMapListeFiches = new TreeMap<>();
            final TreeSet<Long> tSetListeFiches = new TreeSet<>();
            for (final ResultatRecherche resultatRecherche : res) {
                // gestion des doublons
                if (!tSetListeFiches.contains(Long.valueOf(resultatRecherche.getIdFiche()))) {
                    tSetListeFiches.add(Long.valueOf(resultatRecherche.getIdFiche()));
                }
                final String codelangue = resultatRecherche.getCodeFiche() + ",LANGUE=" + resultatRecherche.getLangue();
                if (hListeFichesFil.containsKey(codelangue)) {
                    tMapListeFiches.put(hListeFichesFil.get(codelangue), Long.valueOf(resultatRecherche.getIdFiche()));
                }
            }
            if (tMapListeFiches.size() > 0) {
                final Iterator<Integer> it = tMapListeFiches.keySet().iterator();
                int limit = -1;
                if (ctx.getDatas().get("optimizedLimit") != null) {
                    limit = Integer.parseInt((String) ctx.getDatas().get("optimizedLimit"));
                }
                // on a d'abord réordonné les fiches via la boucle précédente,
                // maintenant on peut se limiter au nombre de fiches souhaité (critère limit)
                while (it.hasNext() && (limit == -1 || res.size() < limit)) {
                    final ResultatRecherche resultat = new ResultatRecherche();
                    resultat.setObjet(objet);
                    resultat.setIdFiche(String.valueOf(tMapListeFiches.get(it.next())));
                    res.add(resultat);
                }
            }
        }
        return gestionOrdonnencementFil(requete, res);
    }

    /*
     * pour réordonnée les résultats, on créé une map contenant l'id et le résultat de la fiche
     * puis on parcours les ids du fils selectionné pour recréer une list dans le bon ordre.
     *
     */
    private List<ResultatRecherche> gestionOrdonnencementFil(final String requete, final List<ResultatRecherche> res) {
        List<ResultatRecherche> resultatOrdonne = new ArrayList<>();
        final String idsFicheDansFil = RequeteUtil.renvoyerParametre(requete, "IDS");
        final TreeMap<String, ResultatRecherche> resultatParId = new TreeMap<>();
        if (StringUtils.isNotBlank(idsFicheDansFil) && !"0000".equals(idsFicheDansFil)) {
            for (final ResultatRecherche resultatRecherche : res) {
                resultatParId.put(resultatRecherche.getIdFiche(), resultatRecherche);
            }
            final String[] idsFiches = StringUtils.split(idsFicheDansFil, ";");
            for (final String idFiche : idsFiches) {
                final ResultatRecherche resultatDeLaFiche = resultatParId.get(idFiche);
                if (resultatDeLaFiche != null) {
                    resultatOrdonne.add(resultatDeLaFiche);
                }
            }
        } else {
            resultatOrdonne = res;
        }
        return resultatOrdonne;
    }

    @Override
    public String preparerRequete(final OMContext ctx, final String requete) {
        final String codeFil = RequeteUtil.renvoyerParametre(requete, "FIL");
        if (StringUtils.isNotEmpty(codeFil) && !"0000".equals(codeFil)) {
            final List<String> codes = serviceFil.getIdFichesByFil(codeFil);
            String replace = "";
            for (final String string : codes) {
                if (replace.length() > 0) {
                    replace += ";";
                }
                replace += string;
            }
            if (StringUtils.isNotBlank(replace)) {
                return StringUtils.replace(requete, "FIL=" + codeFil, "IDS=" + replace);
            }
        }
        return StringUtils.EMPTY;
    }

    @Override
    public Map<String, Object> preparerRecherche(final OMContext ctx, final Map<String, Object> infoBean) {
        final String objet = (String) infoBean.get("CODE_OBJET");
        infoBean.put("LISTE_FILS", serviceFil.getListFilsByType(ReferentielObjets.getNomObjet(objet)));
        return infoBean;
    }

    @Override
    public Map<String, Object> taiterRecherche(final OMContext ctx, final Map<String, Object> infoBean) {
        final String codeFil = (String) infoBean.get("FIL");
        if (StringUtils.isNotEmpty(codeFil) && !"0000".equals(codeFil)) {
            final List<String> codes = serviceFil.getIdFichesByFil(codeFil);
            String replace = "";
            for (final String string : codes) {
                if (replace.length() > 0) {
                    replace += ";";
                }
                replace += string;
            }
            final FilBean fil = serviceFil.getByCode(codeFil);
            if (fil != null && fil.getId() != null) {
                infoBean.put("LIBELLE_FIL", fil.getIntitule());
            }
            if (StringUtils.isBlank(replace)) {
                replace = String.valueOf(Integer.MAX_VALUE);
            }
            infoBean.put("IDS", replace);
        }
        return infoBean;
    }

    @Override
    public List<String> getCriteresRequete() {
        List<String> list = super.getCriteresRequete();
        list.add("FIL");
        return list;
    }
}
