/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.om;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.fil.bean.FilBean;
import com.kosmos.fil.dao.impl.FilDAO;
import com.univ.objetspartages.om.AbstractOm;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;

/**
 * The Class Fil.
 * @deprecated Utilisez {@link com.kosmos.fil.bean.FilBean} et {@link com.kosmos.fil.service.impl.ServiceFil}
 */
@Deprecated
public class Fil extends AbstractOm<FilBean, FilDAO> implements Cloneable {

    /**
     * Instantiates a new fil.
     */
    public Fil() {
        super();
    }

    /**
     * implementation de la methode clone() (utilisée dans le module catalogue)
     */
    @Override
    public Fil clone() throws CloneNotSupportedException {
        return (Fil) super.clone();
    }

    /**
     * Inits the.
     */
    public void init() {
        setId(System.currentTimeMillis());
        String timeStamp = Long.toString(System.currentTimeMillis());
        setCode(timeStamp.substring(timeStamp.length() - 8, timeStamp.length()));
        setLangue("0");
        setIntitule(StringUtils.EMPTY);
        setTypeFil(StringUtils.EMPTY);
        setListeFiches(StringUtils.EMPTY);
        setCodeRattachement(StringUtils.EMPTY);
        setCodeRubrique(StringUtils.EMPTY);
    }

    /**
     * Sélection d'un groupe DSI à partir de l'ensemble des critères combinés.
     *
     * @param code             the _code
     * @param codeRattachement the _code rattachement
     * @param type             the _type
     * @param langue           the _langue
     * @param intitule         the _intitule
     * @param codeRubrique     the _code rubrique
     * @return the int
     * @throws Exception the exception
     */
    public int select(String code, String codeRattachement, String type, String langue, String intitule, String codeRubrique) throws Exception {
        RequeteSQL requeteSelect = new RequeteSQL();
        ClauseOrderBy orderBy = new ClauseOrderBy("INTITULE", SensDeTri.ASC);
        ClauseWhere where = new ClauseWhere();
        if (StringUtils.isNotEmpty(code)) {
            where.setPremiereCondition(ConditionHelper.egalVarchar("CODE", code));
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(type) && !"0000".equals(type)) {
            where.and(ConditionHelper.egalVarchar("TYPE_FIL", type));
        }
        if (StringUtils.isNotEmpty(intitule)) {
            where.and(ConditionHelper.rechercheMots("INTITULE", intitule));
        }
        if (StringUtils.isNotEmpty(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("CODE_RATTACHEMENT", codeRattachement));
        }
        if (StringUtils.isNotEmpty(codeRubrique)) {
            where.and(ConditionHelper.getConditionRubrique("CODE_RUBRIQUE", codeRubrique));
        }
        requeteSelect.where(where).orderBy(orderBy);
        return select(requeteSelect.formaterRequete());
    }

    public Long getId() {
        return persistenceBean.getId();
    }

    public void setId(Long id) {
        persistenceBean.setId(id);
    }

    public String getCode() {
        return persistenceBean.getCode();
    }

    public void setCode(String code) {
        this.persistenceBean.setCode(code);
    }

    public String getCodeRattachement() {
        return persistenceBean.getCodeRattachement();
    }

    public void setCodeRattachement(String codeRattachement) {
        this.persistenceBean.setCodeRattachement(codeRattachement);
    }

    public String getCodeRubrique() {
        return persistenceBean.getCodeRubrique();
    }

    public void setCodeRubrique(String codeRubrique) {
        this.persistenceBean.setCodeRubrique(codeRubrique);
    }

    public String getLangue() {
        return persistenceBean.getLangue();
    }

    public void setLangue(String langue) {
        this.persistenceBean.setLangue(langue);
    }

    public String getIntitule() {
        return persistenceBean.getIntitule();
    }

    public void setIntitule(String intitule) {
        this.persistenceBean.setIntitule(intitule);
    }

    public String getTypeFil() {
        return persistenceBean.getTypeFil();
    }

    public void setTypeFil(String typeFil) {
        this.persistenceBean.setTypeFil(typeFil);
    }

    public String getListeFiches() {
        return persistenceBean.getListeFiches();
    }

    public void setListeFiches(String listeFiches) {
        this.persistenceBean.setListeFiches(listeFiches);
    }
}
