/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.reference;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import com.kosmos.fil.bean.FilBean;
import com.kosmos.fil.bean.FilBeanExport;
import com.kosmos.fil.service.impl.ServiceFil;
import com.kosmos.usinesite.exception.ErreurReferenceException;
import com.kosmos.usinesite.reference.BeanReference;
import com.kosmos.usinesite.reference.DelimitationPattern;
import com.kosmos.usinesite.reference.ServiceBeanReference;
import com.kosmos.usinesite.reference.dto.ServiceBeanReferenceDto;
import com.univ.utils.ContexteDao;

public class ServiceFilBeanReference extends ServiceBeanReference<Serializable> {

    private ServiceFil serviceFil;

    public void setServiceFil(ServiceFil serviceFil) {
        this.serviceFil = serviceFil;
    }

    @Override
    public Collection<BeanReference> check(final ServiceBeanReferenceDto<Serializable> serviceBeanReferenceDto) throws ErreurReferenceException {
        final Collection<BeanReference> referencesToUpdate = new ArrayList<>();
        FilBeanExport filBeanExport = (FilBeanExport) serviceBeanReferenceDto.getBeansOrig().values().iterator().next();
        final Collection<String> moduleToSearch = new ArrayList<>();
        moduleToSearch.add(filBeanExport.getIdModule());
        final BeanReference referencesByCode = new BeanReference();
        // mise à jour du code sur le module fil
        referencesByCode.addModulesBySearchString("\"code\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        // mise à jour des toolboxs tags traitement requete ex [traitement;requete;objet=actualite#TRI_DATE=DATE_ASC#FIL=62319083]
        referencesByCode.addModulesBySearchString("FIL=" + DelimitationPattern.SHARP_SEMICOLON_BRACKETS_DELIMITER + "\\]", Collections.emptyList());
        // mise à jour de l'id fil
        final BeanReference referencesById = new BeanReference();
        referencesById.addModulesBySearchString("\"id_fil\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        for (final String codeFil : serviceBeanReferenceDto.getBeansOrig().keySet()) {
            final ContexteDao ctx = new ContexteDao();
            try {
                // Gestion des rubriques
                final FilBean fil = serviceFil.getByCode(codeFil);
                // si la rubrique existe on duplique son code
                if (fil != null) {
                    referencesByCode.addNewValueByOldValue(codeFil, Long.toString(System.nanoTime()));
                    referencesById.addNewValueByOldValue(fil.getId().toString(), "0");
                }
            } catch (final Exception e) {
                throw new ErreurReferenceException("Erreur lors de la récupération des formulaires en bdd", e);
            } finally {
                ctx.close();
            }
        }
        referencesToUpdate.add(referencesByCode);
        referencesToUpdate.add(referencesById);
        return referencesToUpdate;
    }
}
