/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.fil.bean.FilBean;
import com.kosmos.fil.dao.impl.FilDAO;
import com.kosmos.fil.module.ModuleFil;
import com.kosmos.service.impl.AbstractServiceBean;
import com.kportal.extension.module.ModuleHelper;
import com.univ.dto.ReferenceFicheDTO;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.ContexteDao;
import com.univ.utils.FicheUnivHelper;

/**
 * Created on 13/05/15.
 */
public class ServiceFil extends AbstractServiceBean<FilBean, FilDAO> {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceFil.class);

    public FilBean getByCode(String code) {
        return dao.getByCode(code);
    }

    public List<FilBean> getAll() {
        return dao.getAll();
    }

    public List<FilBean> getByCodesRubrique(List<String> codesRubrique) {
        return dao.getByCodesRubrique(codesRubrique);
    }

    public String getLibelleTypeFil(String nomObjet) {
        String libelleObjet = "";
        if (nomObjet != null) {
            final String codeObjet = ReferentielObjets.getCodeObjet(nomObjet);
            libelleObjet = ReferentielObjets.getLibelleObjet(codeObjet);
        }
        return libelleObjet;
    }

    public Map<String, String> getTypesFils() {
        final Map<String, String> h = new HashMap<>();
        FicheUniv ficheUniv;
        final ModuleFil module = (ModuleFil) ModuleHelper.getModule(ModuleFil.ID_EXTENSION, ModuleFil.ID_BEAN);
        for (final String codeObjet : ReferentielObjets.getListeCodesObjet()) {
            ficheUniv = ReferentielObjets.instancierFiche(codeObjet);
            if (ficheUniv != null && module.getPluginFiche().getClasses().contains(ficheUniv.getClass().getName())) {
                h.put(ReferentielObjets.getNomObjet(codeObjet), ReferentielObjets.getLibelleObjet(codeObjet));
            }
        }
        return h;
    }

    public Map<String, String> getListFilsWithFiche(String code, String langue) {
        final Map<String, String> listefilsfiche = new HashMap<>();
        if (code != null && langue != null) {
            final String codelangue = code + ",LANGUE=" + langue;
            // selection de tous les fils
            final List<FilBean> fils = getAll();
            if (CollectionUtils.isNotEmpty(fils)) {
                // boucle sur les fils
                for (FilBean fil : fils) {
                    // liste des fiches du fil
                    final String listeFiches = fil.getListeFiches();
                    if (listeFiches.length() > 0) {
                        final StringTokenizer st = new StringTokenizer(listeFiches, ";");
                        while (st.hasMoreTokens()) {
                            // si le code et la langue se trouve dans la liste
                            // des fiches du fil
                            // alors on ajoute le fil dans la table
                            if (st.nextToken().equals(codelangue)) {
                                listefilsfiche.put(fil.getCode(), fil.getIntitule());
                            }
                        }
                    }
                }
            }
        }
        return listefilsfiche;
    }

    public List<FilBean> getManualListByType(String type) {
        return dao.getByType(type);
    }

    public Map<String, String> getListFilsByType(String type) {
        final Map<String, String> listeFils = new HashMap<>();
        final List<FilBean> fils = dao.getByType(type);
        for (FilBean fil : fils) {
            listeFils.put(fil.getCode(), fil.getIntitule());
        }
        return listeFils;
    }

    public List<String> getIdFichesByFil(String codeFil) {
        final List<String> idDesFichesUnivs = new ArrayList<>();
        final FilBean fil = getByCode(codeFil);
        if (fil == null || StringUtils.isEmpty(fil.getListeFiches())) {
            return idDesFichesUnivs;
        }
        final String listeFiches = fil.getListeFiches();
        final String listeFichesSplite[] = listeFiches.split(";");
        final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(fil.getTypeFil());
        try (ContexteDao ctx = new ContexteDao()) {
            ficheUniv.init();
            ficheUniv.setCtx(ctx);
            String code;
            String langue;
            for (final String codelangue : listeFichesSplite) {
                if (StringUtils.isNotEmpty(codelangue)) {
                    final int indexVirgule = codelangue.indexOf(",");
                    if (indexVirgule != -1) {
                        code = codelangue.substring(0, indexVirgule);
                        langue = codelangue.substring(indexVirgule + 8);
                    } else {
                        code = codelangue;
                        langue = "0";
                    }
                    if (StringUtils.isNotEmpty(code)) {
                        int nbFiche = ficheUniv.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
                        if (nbFiche == 0) {
                            ficheUniv.selectCodeLangueEtat(code, langue, "");
                        }
                        if (ficheUniv.nextItem()) {
                            idDesFichesUnivs.add(String.valueOf(ficheUniv.getIdFiche()));
                        }
                    }
                }
            }
        } catch (Exception e) {
            LOG.error(String.format("Une erreur est survenue lors de la récupération des ids de fiche associés au code fil %s", codeFil), e);
        }
        return idDesFichesUnivs;
    }

    public void addWithForcedId(FilBean filBean) throws Exception {
        dao.addWithForcedId(filBean);
    }

    /**
     * Désérialise une liste manuelle de fiches.
     * @param serialisation Valeur sérialisée de la liste de fiches (séparées par des ; )
     * @return La liste des fiches de la liste manuelle.
     */
    public List<ReferenceFicheDTO> deserialiserListeFiches(String serialisation) {
        List<ReferenceFicheDTO> fiches = new ArrayList<>();
        if (StringUtils.isNotBlank(serialisation)) {
            String[] listeFiches = StringUtils.split(serialisation, ";");
            for (String serialFiche: listeFiches) {
                ReferenceFicheDTO referenceFiche = FicheUnivHelper.deserialiserFiche(serialFiche);
                fiches.add(referenceFiche);
            }
        }
        return fiches;
    }

    /**
     * Renvoie la liste des fiches d'une liste manuelle à partir de son code et la classe manipulée.
     * @param codeFil Code de la liste manuelle.
     * @param <F> Type de la liste de retour.
     * @return La liste des fiches de la liste manuelle.
     */
    public <F extends FicheUniv> List<F> getListeFichesDepuisFil(String codeFil, Class<F> classFiche) {
        List<F> fiches = new ArrayList<>();
        if (StringUtils.isBlank(codeFil)) {
            return fiches;
        }
        FilBean fil = getByCode(codeFil);
        if (fil != null) {
            String serialListe = fil.getListeFiches();
            List<ReferenceFicheDTO> references = deserialiserListeFiches(serialListe);
            for (ReferenceFicheDTO referenceFiche: references) {
                try {
                    FicheUniv fiche = FicheUnivHelper.getFiche(fil.getTypeFil(), referenceFiche.getCode(), referenceFiche.getLangue());
                    fiches.add(classFiche.cast(fiche));
                } catch (Exception e) {
                    LOG.error(String.format("Erreur lors de la lecture de la fiche %s (langue = %s)", referenceFiche.getCode(), referenceFiche.getLangue()), e);
                }
            }
        }
        return fiches;
    }
}
