/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.toolbox.provider;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.kosmos.toolbox.bean.TagValue;
import com.kosmos.toolbox.provider.impl.AbstractTagDataProvider;


public class TagManualListProvider extends AbstractTagDataProvider {

    // Récupère la valeur de objet dans un tag de type [traitement;requete;objet=article#FIL=1533104755254#STYLE=0001]    
    private static final Pattern MANUAL_LIST_TAG_PATTERN_OBJET =  Pattern.compile(".*objet=(\\w*).*");

    // Récupère la valeur de FIL dans un tag de type [traitement;requete;objet=article#FIL=1533104755254#STYLE=0001]
    private static final Pattern MANUAL_LIST_TAG_PATTERN_FIL =  Pattern.compile(".*FIL=(\\w*).*");

    // Récupère la valeur de STYLE dans un tag de type [traitement;requete;objet=article#FIL=1533104755254#STYLE=0001]
    private static final Pattern MANUAL_LIST_TAG_PATTERN_STYLE =  Pattern.compile(".*STYLE=(\\w*).*");
    
    public static final String KEY_MAP_OBJET = "objet";

    public static final String KEY_MAP_FIL = "fil";

    public static final String KEY_MAP_STYLE = "style";

    /**
     * Récupération des valeurs du tag pour la prévalorisation des données en cas d'edition du tag
     * Obliger de découper en 3 REGEX car seul le paramètre FIL est obligatoire
     * @param data
     * @param tagValue
     */
    @Override
    public void populateData(final Map<String, String> data, final TagValue tagValue) {
        
        final Matcher matcherObjet = MANUAL_LIST_TAG_PATTERN_OBJET.matcher(tagValue.getValue());
        if (matcherObjet.find()) {
            data.put(KEY_MAP_OBJET, matcherObjet.group(1));
        }
        final Matcher matcherFil = MANUAL_LIST_TAG_PATTERN_FIL.matcher(tagValue.getValue());
        if (matcherFil.find()) {
            data.put(KEY_MAP_FIL, matcherFil.group(1));
        }
        final Matcher matcherStyle = MANUAL_LIST_TAG_PATTERN_STYLE.matcher(tagValue.getValue());
        if (matcherStyle.find()) {
            data.put(KEY_MAP_STYLE, matcherStyle.group(1));
        }
    }
}
