/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: listemanuelle
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.fil.util;

import java.util.Hashtable;
import java.util.List;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.fil.bean.FilBean;
import com.kosmos.fil.module.ModuleFil;
import com.kosmos.fil.om.Fil;
import com.kosmos.fil.service.impl.ServiceFil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.PermissionBean;


/**
 * Classe utile pour la gestion des fils.
 */
public class FilUtil {

    public static ServiceFil getServiceFil() {
        return ServiceManager.getServiceForBean(FilBean.class);
    }

    /**
     * Retourne un message défini dans les fichiers de langues de l'extension Fil ou du core si non trouvé
     *
     * @param key
     *            la chaine à chercher
     * @return la valeur ou null si non trouvé
     */
    public static String getMessage(final String key) {
        return MessageHelper.getMessage(ModuleFil.ID_EXTENSION, key);
    }

    /**
     * Renvoie le libellé du type de fil correspondant au nom de l'objet càd renvoie le libellé de l'objet = type de fil.
     *
     * @param nomObjet
     *            the nom objet
     *
     * @return String libellé objet par exemple 'Articles'
     * @deprecated Utilisez {@link com.kosmos.fil.service.impl.ServiceFil#getLibelleTypeFil(String)}
     */
    @Deprecated
    public static String getLibelleTypeFil(final String nomObjet) {
        return getServiceFil().getLibelleTypeFil(nomObjet);
    }

    /**
     * Renvoie la liste des types de fils qui existent, càd la liste des objets qui étendent l'interface FicheFil.
     *
     * @return Hashtable la liste des types clé = nomobjet (ex 'article'), valeur = libelleobjet (ex : 'Articles')
     * @deprecated Utilisez {@link com.kosmos.fil.service.impl.ServiceFil#getTypesFils()}
     */
    @Deprecated
    public static Hashtable<String, String> renvoyerTypesFils() {
        final Hashtable<String, String> h = new Hashtable<>();
        h.putAll(getServiceFil().getTypesFils());
        return h;
    }

    /**
     * Renvoie la liste des fils dans lesquels se trouve la fiche.
     *
     * @param ctx
     *            : contexte pour accès à la base
     * @param code
     *            the code
     * @param langue
     *            the langue
     *
     * @return hashtable liste des fils avec cle = fil.getCode() + i et valeur = fil.getIntitule()
     *
     * @throws Exception
     *             the exception
     * @deprecated Utilisez {@link com.kosmos.fil.service.impl.ServiceFil#getListFilsWithFiche(String, String)}
     */
    @Deprecated
    public static Hashtable<String, String> getListeFilsParFiche(final OMContext ctx, final String code, final String langue) throws Exception {
        final Hashtable<String, String> h = new Hashtable<>();
        h.putAll(getServiceFil().getListFilsWithFiche(code, langue));
        return h;
    }

    /**
     * Renvoie la liste des fils par type d'objet.
     *
     * @param ctx
     *            contexte pour accès à la base
     * @param nomObjet
     *            the nom objet
     *
     * @return Hashtable cle = fil.getCode(), valeur = fil.getIntitule()
     *
     * @throws Exception
     *             the exception
     * @deprecated Utilisez {@link com.kosmos.fil.service.impl.ServiceFil#getListFilsByType(String)}
     */
    @Deprecated
    public static Hashtable<String, String> getListeFilsParType(final OMContext ctx, final String nomObjet) throws Exception {
        Hashtable<String, String> listefils = new Hashtable<>();
        listefils.putAll(getServiceFil().getListFilsByType(nomObjet));
        return listefils;
    }

    /**
     * Permet de calculer la liste des id des ficheUniv lié au fil fourni en paramètre.
     *
     * @param ctx
     *            nécessaire pour intéroger la bdd
     * @param codeFil
     *            le code du fil dont on veut connaitre les ficheUniv
     * @return la liste des Id des fiches liés.
     * @throws Exception
     * @deprecated
     */
    @Deprecated
    public static List<String> getIdFichesUnivParFil(final OMContext ctx, final String codeFil) throws Exception {
        return getServiceFil().getIdFichesByFil(codeFil);
    }

    /**
     * Retourne le fil ayant pour code le code fourni en paramètre
     *
     * @param ctx
     * @param codeFil
     * @return
     * @throws Exception
     * @deprecated Utilisez {@link com.kosmos.fil.service.impl.ServiceFil#getByCode(String)}
     */
    @Deprecated
    public static Fil getFilParCode(final OMContext ctx, final String codeFil) throws Exception {
        final Fil fil = new Fil();
        fil.select(codeFil, "", "", "", "", "");
        fil.nextItem();
        return fil;
    }

    public static boolean isAdministrateurFil(final AutorisationBean autorisations) {
        boolean res = Boolean.FALSE;
        PermissionBean permission = new PermissionBean("TECH", "fil", "C");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        permission = new PermissionBean("TECH", "fil", "S");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        permission = new PermissionBean("TECH", "fil", "M");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        return res;
    }

    public static boolean isCreateurFil(final AutorisationBean autorisations) {
        final PermissionBean permission = new PermissionBean("TECH", "fil", "C");
        return autorisations != null && autorisations.getListePermissions() != null && autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null;
    }
}
