/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.formulaire.util.FormulaireUtil;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.utils.ContexteUtil;

public class ExtractionDonneesPersonnelles {

    public static final Logger LOG = LoggerFactory.getLogger(ExtractionDonneesPersonnelles.class);
    /**
     * Extraction des données personnelles dans un fichier txt.
     * @param nameForm
     * @param dateForm
     * @param mapDonneesPersonnelles
     * @return
     * @throws IOException
     */
    public static String extraire(final String nameForm, final Date dateForm, final Map<String, String> mapDonneesPersonnelles) throws ErreurApplicative {
        final StringBuilder str = new StringBuilder();
        final Format formatter = new SimpleDateFormat("dd/MM/yyyy", ContexteUtil.getContexteUniv().getLocale());
        str.append(String.format(FormulaireUtil.getMessage("EXPORT_DP_INTRODUCTION"), nameForm, formatter.format(dateForm))).append(System.lineSeparator());
        mapDonneesPersonnelles.forEach((keyForm, valueForm) -> str.append(keyForm + "=" + valueForm + System.lineSeparator()));
        final String fileName = String.format(FormulaireUtil.getMessage("PATTERN_FILE_NAME_EXPORT_DP"), System.currentTimeMillis());
        final String path = WebAppUtil.getDownloadPrivatePath() + File.separator + fileName;
        try {
            FileUtils.writeStringToFile(new File(path), str.toString(), CharEncoding.DEFAULT);
        } catch (IOException e) {
            LOG.error("Erreur dans l'écriture de l'extraction des données personnelles",e);
            throw new ErreurApplicative(FormulaireUtil.getMessage("ERROR_EXPORT_DP"));
        }
        return path;
    }
}
