/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.StringWriter;
import java.util.Properties;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.formulaire.om.Contenuformulaire;
import com.kosmos.formulaire.om.Formulaire;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;

public class ExtractionFormulaireImpl implements ExtractionFormulaire {

    /** The Constant FORMAT_HTML. */
    public static final int FORMAT_HTML = 0;

    /** The Constant FORMAT_CSV. */
    public static final int FORMAT_CSV = 1;

    public static final int MODELE_SIMPLE = 0;

    /** The Constant MODELE_SONDAGE. */
    public static final int MODELE_SONDAGE = 1;

    /** The format extraction. */
    protected int formatExtraction = FORMAT_HTML;

    /** The modele extraction. */
    protected int modeleExtraction = MODELE_SIMPLE;

    /** The writer. */
    protected StringWriter writer = new StringWriter();

    @Override
    public void setTypeExtraction(final String typeExtraction) throws Exception {
        if ((typeExtraction.equals("CSV")) || (typeExtraction.equals("CSV_SONDAGE"))) {
            formatExtraction = FORMAT_CSV;
        }
        if ((typeExtraction.equals("CSV_SONDAGE")) || (typeExtraction.equals("SONDAGE"))) {
            modeleExtraction = MODELE_SONDAGE;
        }
    }

    @Override
    public String getContentType() throws Exception {
        if (formatExtraction == FORMAT_CSV) {
            return "application/csv";
        } else {
            return "text/html";
        }
    }

    public String getFileDay() {
        if (formatExtraction == FORMAT_CSV) {
            return "formulaire-" + System.currentTimeMillis() + ".csv";
        } else {
            return "formulaire-" + System.currentTimeMillis() + ".html";
        }
    }

    @Override
    public String extraireFormulaire(final Formulaire formulaire, final InfoBean infoBean, final OMContext ctx) throws Exception {
        final String fileName = WebAppUtil.getDownloadPrivatePath() + File.separator + getFileDay();
        formaterDebutExtraction();
        if (modeleExtraction == MODELE_SIMPLE) {
            final Contenuformulaire contenu = new Contenuformulaire();
            contenu.init();
            contenu.select("WHERE CODE_FORMULAIRE='" + formulaire.getCode() + "' ORDER BY ID_CONTENUFORMULAIRE DESC");
            formaterDebutTable();
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            int nbColonne = 7;
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                final String typeChamp = infoBean.getString("CHAMP" + numeroChamp + "_TYPE");
                if (!"EVENT".equals(typeChamp)) {
                    nbColonne++;
                }
            }
            final String[] ligne = new String[nbColonne];
            int numeroColonne = 6;
            ligne[0] = "Code";
            ligne[1] = "Date saisie";
            ligne[2] = "Code utilisateur";
            ligne[3] = "Nom utilisateur";
            ligne[4] = "Prénom utilisateur";
            ligne[5] = "Email utilisateur";
            ligne[6] = "Dossier des pièces jointes";
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                final String typeChamp = infoBean.getString("CHAMP" + numeroChamp + "_TYPE");
                if (!"EVENT".equals(typeChamp)) {
                    numeroColonne++;
                    ligne[numeroColonne] = infoBean.getString("CHAMP" + numeroChamp + "_LIBELLE");
                }
            }
            formaterLigne(ligne);
            while (contenu.nextItem()) {
                ligne[0] = "";
                ligne[1] = "";
                ligne[2] = "";
                ligne[3] = "";
                ligne[4] = "";
                ligne[5] = "";
                ligne[6] = "";
                final String instance = contenu.getInstance();
                final Properties proprietes = new Properties();
                proprietes.load(new ByteArrayInputStream(instance.getBytes()));
                ligne[0] = contenu.getCodeUnique();
                if (proprietes.getProperty("SAISIE_DATE") != null) {
                    ligne[1] = proprietes.getProperty("SAISIE_DATE");
                }
                if (proprietes.getProperty("SAISIE_UTILISATEUR_CODE") != null) {
                    ligne[2] = proprietes.getProperty("SAISIE_UTILISATEUR_CODE");
                }
                if (proprietes.getProperty("SAISIE_UTILISATEUR_NOM") != null) {
                    ligne[3] = proprietes.getProperty("SAISIE_UTILISATEUR_NOM");
                }
                if (proprietes.getProperty("SAISIE_UTILISATEUR_PRENOM") != null) {
                    ligne[4] = proprietes.getProperty("SAISIE_UTILISATEUR_PRENOM");
                }
                if (proprietes.getProperty("SAISIE_UTILISATEUR_MAIL") != null) {
                    ligne[5] = proprietes.getProperty("SAISIE_UTILISATEUR_MAIL");
                }
                if (proprietes.getProperty("SAISIE_PATH_PJ") != null) {
                    ligne[6] = proprietes.getProperty("SAISIE_PATH_PJ");
                }
                numeroColonne = 6;
                for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                    final String codeChamp = infoBean.getString("CHAMP" + numeroChamp + "_CODE");
                    final String typeChamp = infoBean.getString("CHAMP" + numeroChamp + "_TYPE");
                    if (!"EVENT".equals(typeChamp)) {
                        numeroColonne++;
                        String valeurExtraction = "";
                        if ("INPUT".equals(typeChamp) || "TEXTAREA".equals(typeChamp) || "FILE".equals(typeChamp) || "IDENTITY".equals(typeChamp)) {
                            valeurExtraction = proprietes.getProperty(codeChamp);
                        } else if ("SELECT".equals(typeChamp)) {
                            final int nbItem = (Integer) infoBean.get("CHAMP" + numeroChamp + "_NB_ITEM");
                            for (int j = 0; j < nbItem; j++) {
                                final String codeItem = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                                final String libelleItem = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                                if (proprietes.getProperty(codeChamp) != null) {
                                    final String[] lstCodeSelectionne = proprietes.getProperty(codeChamp).split(",", -2);
                                    for (final String element : lstCodeSelectionne) {
                                        if (element.equals(codeItem)) {
                                            if (valeurExtraction.length() == 0) {
                                                valeurExtraction = libelleItem;
                                            } else {
                                                valeurExtraction += "\n";
                                                valeurExtraction += libelleItem;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        ligne[numeroColonne] = valeurExtraction;
                    }
                }
                formaterLigne(ligne);
            }
            formaterFinTable();
        } else if (modeleExtraction == MODELE_SONDAGE) {
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                final String codeChamp = infoBean.getString("CHAMP" + numeroChamp + "_CODE");
                final String typeChamp = infoBean.getString("CHAMP" + numeroChamp + "_TYPE");
                if ("SELECT".equals(typeChamp)) {
                    formaterDebutTable();
                    final int nbItem = (Integer) infoBean.get("CHAMP" + numeroChamp + "_NB_ITEM");
                    final String[] ligne = new String[nbItem + 1];
                    final Integer[] lstTotaux = new Integer[nbItem];
                    final String[] lstCode = new String[nbItem];
                    ligne[0] = "Total";
                    for (int j = 0; j < nbItem; j++) {
                        lstCode[j] = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                        ligne[j + 1] = infoBean.getString("CHAMP" + numeroChamp + "_LIBELLE") + " : " + infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                    }
                    formaterLigne(ligne);
                    for (int j = 0; j < nbItem; j++) {
                        lstTotaux[j] = 0;
                    }
                    final Contenuformulaire contenu = new Contenuformulaire();
                    contenu.init();
                    final RequeteSQL requete = new RequeteSQL();
                    final ClauseWhere whereCodeFormulaireEtCodeUnique = new ClauseWhere(ConditionHelper.egalVarchar("CODE_FORMULAIRE", formulaire.getCode()));
                    final ClauseOrderBy orderBy = new ClauseOrderBy("ID_CONTENUFORMULAIRE", SensDeTri.ASC);
                    requete.where(whereCodeFormulaireEtCodeUnique).orderBy(orderBy);
                    final int nbSaisie = contenu.select(requete.formaterRequete());
                    while (contenu.nextItem()) {
                        final String instance = contenu.getInstance();
                        final Properties proprietes = new Properties();
                        proprietes.load(new ByteArrayInputStream(instance.getBytes()));
                        for (int j = 0; j < nbItem; j++) {
                            final String property = StringUtils.defaultString(proprietes.getProperty(codeChamp));
                            final String[] lstCodeSaisis = property.split(",", -2);
                            for (final String lstCodeSaisi : lstCodeSaisis) {
                                if (lstCodeSaisi.equals(lstCode[j])) {
                                    lstTotaux[j] = lstTotaux[j] + 1;
                                    break;
                                }
                            }
                        }
                    }
                    ligne[0] = String.valueOf(nbSaisie);
                    for (int j = 0; j < nbItem; j++) {
                        ligne[j + 1] = lstTotaux[j].toString();
                    }
                    formaterLigne(ligne);
                    formaterFinTable();
                }
            }
        }
        formaterFinExtraction();
        FileUtils.writeStringToFile(new File(fileName), writer.toString(), CharEncoding.DEFAULT);
        return fileName;
    }

    /**
     * Formater ligne.
     *
     * @param ligne
     *            the ligne
     *
     * @throws Exception
     *             the exception
     */
    protected void formaterLigne(final String[] ligne) throws Exception {
        if (formatExtraction == FORMAT_CSV) {
            for (int i = 0; i < ligne.length; i++) {
                if (i != 0) {
                    writer.write(';');
                }
                final String champ = ligne[i];
                if (champ != null) {
                    if (!champ.contains(";") && !champ.contains("\"") && !champ.contains("\n") && !champ.contains("\r")) {
                        writer.write(champ);
                    } else {
                        writer.write("\"");
                        writer.write(champ.replaceAll("\"", "\"\"").replaceAll("\r", " ").replaceAll("\n", " "));
                        writer.write("\"");
                    }
                }
            }
            writer.write("\r\n");
        } else {
            writer.write("<tr>");
            for (final String champ : ligne) {
                writer.write("<td>&nbsp;");
                if (champ != null) {
                    writer.write(champ.replaceAll("\n", "<br/>"));
                }
                writer.write("</td>\r\n");
            }
            writer.write("</tr>\r\n");
        }
    }

    /**
     * Formater debut extraction.
     *
     * @throws Exception
     *             the exception
     */
    protected void formaterDebutExtraction() throws Exception {
        // insertion du BOM utf8
        writer.write('\ufeff');
        if (formatExtraction == FORMAT_HTML) {
            writer.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"><html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"fr\"><head><title>Extraction</title><meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\" /></head><body>");
        }
    }

    /**
     * Formater fin extraction.
     *
     * @throws Exception
     *             the exception
     */
    protected void formaterFinExtraction() throws Exception {
        if (formatExtraction == FORMAT_HTML) {
            writer.write("</body></html>  ");
        }
    }

    /**
     * Formater debut table.
     *
     * @throws Exception
     *             the exception
     */
    protected void formaterDebutTable() throws Exception {
        if (formatExtraction == FORMAT_HTML) {
            writer.write("<table width=\"90%\" border=\"1\">");
        }
    }

    /**
     * Formater fin table.
     *
     * @throws Exception
     *             the exception
     */
    protected void formaterFinTable() throws Exception {
        if (formatExtraction == FORMAT_HTML) {
            writer.write("</table>");
        } else {
            writer.write("\n");
        }
    }
}
