/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.text.DecimalFormat;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.FormateurJSP;
import com.jsbsoft.jtf.core.InfoBean;
import com.kosmos.controllers.SGController;
import com.kosmos.formulaire.om.Contenuformulaire;
import com.kosmos.formulaire.processus.TraitementFormulaire;
import com.kosmos.formulaire.util.FormulaireUtil;
import com.kportal.captcha.CaptchaUtils;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.univ.utils.ContexteUniv;
import com.univ.utils.URLResolver;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
// TODO: Auto-generated Javadoc

/**
 * The Class FormatageFormulaireImpl.
 *
 * @author Administrateur
 *
 *         To change the template for this generated type comment go to Window - Preferences - Java - Code Generation - Code and Comments
 */
public class FormatageFormulaireImpl implements FormatageFormulaire {

    private static final Logger LOGGER = LoggerFactory.getLogger(FormatageFormulaireImpl.class);

    /**
     * The info bean formulaire.
     */
    protected InfoBean infoBeanFormulaire = null;

    /**
     * The fmt.
     */
    protected FormateurJSP fmt = null;

    /**
     * The proprietes contenu.
     */
    protected Properties proprietesContenu = null;

    /**
     * The code unique.
     */
    protected String codeUnique = null;

    /** The style. */
    protected String style = null;

    /** The code. */
    protected String code = null;

    /** The element. */
    protected String element = null;

    /** The ctx. */
    protected ContexteUniv ctx = null;

    /** The sw. */
    protected StringWriter sw = null;

    /** The saisie. */
    protected boolean saisie = true;

    /**
     * The df.
     */
    protected DecimalFormat df = new DecimalFormat();

    // infoBean utilisé pour renvoyer un message d'erreur
    protected InfoBean infoBean = null;

    //page referer, transmis en champ caché du formulaire, pour revenir à la page en cas d'erreur
    protected String pageReferer = null;

    /*
     * (non-Javadoc)
     *
     * @see
     * com.kosmos.formulaire.FormatageFormulaire#formaterHtml(com.univ.utils
     * .ContexteUniv, java.lang.String, java.lang.String, java.lang.String)
     */
    @Override
    public String formaterHtml(final ContexteUniv _ctx, final String _code, final String _style, final String _element) {
        ctx = _ctx;
        code = _code;
        style = _style;
        element = _element;
        try {
            sw = new StringWriter();
            initialiserInfosFormatage();
            if (infoBeanFormulaire != null) {
                if (style.startsWith("DISPOSITION")) {
                    // 1 colonnes
                    formaterDebutFormulaire(style.toLowerCase(), _ctx.getCodeRubriquePageCourante());
                    formaterHtmlUnBloc();
                    formaterFinFormulaire();
                } else {
                    // elements
                    if ("DEBUT".equals(element)) {
                        formaterDebutFormulaire("", _ctx.getCodeRubriquePageCourante());
                    } else if ("FIN".equals(element)) {
                        formaterFinFormulaire();
                    } else {
                        // CHAMP
                        final int nbChamp = (Integer) infoBeanFormulaire.get("LISTE_NB_CHAMPS");
                        for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                            final String codeChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_CODE");
                            if (codeChamp.equals(element)) {
                                // si utilisateur anonyme, activation du champ
                                // anti spam avant le premier bouton
                                if (ctx.getCode().length() == 0 && "EVENT".equals(infoBeanFormulaire.getString("CHAMP" + (numeroChamp) + "_TYPE")) && !"EVENT".equals(infoBeanFormulaire.getString("CHAMP" + (numeroChamp - 1) + "_TYPE"))) {
                                    formaterChampCaptcha();
                                }
                                formaterChampHtml(numeroChamp);
                            }
                        }
                    }
                }
            }
        } catch (final Exception e) {
            LOGGER.error("erreur lors du formatage du formulaire", e);
            sw = new StringWriter();
        }
        return sw.toString();
    }

    /**
     * Initialiser infos formatage.
     *
     * @throws Exception the exception
     */
    protected void initialiserInfosFormatage() throws Exception {
        df.applyPattern("000");
        if ("DISPOSITION2".equals(style) || "DISPOSITION1".equals(style) || "E".equals(style) && "DEBUT".equals(element)) {
            if (ctx.getRequeteHTTP() != null && ctx.getRequeteHTTP().getSession(false) != null) {
                infoBean = (InfoBean) ctx.getRequeteHTTP().getSession(false).getAttribute("infoBean");
                if (infoBean != null) {
                    pageReferer = (String) infoBean.get("PAGE_REFERER");
                    ctx.getRequeteHTTP().getSession(false).removeAttribute("infoBean");
                }
            }
            if (pageReferer == null && StringUtils.isNotBlank(ctx.getUrlPageCourante())) {
                pageReferer = ctx.getUrlPageCourante();
            }
            infoBeanFormulaire = TraitementFormulaire.getInfoBeanFormulaire(ctx, code);
            fmt = new FormateurJSP();
            // Prise en compte de l'enregistrement unique soit à partir de
            // l'utilisateur soit à partir du contexte
            if ("1".equals(infoBeanFormulaire.getString("ENREGISTREMENT_UNIQUE"))) {
                codeUnique = ctx.getCode();
            } else {
                codeUnique = (String) ctx.getDatas().get("CODE_UNIQUE");
            }
            if ((codeUnique != null) && (codeUnique.length() > 0)) {
                final Contenuformulaire contenu = new Contenuformulaire();
                contenu.init();
                final ClauseWhere whereCodeFormulaireEtCodeUnique = new ClauseWhere(ConditionHelper.egalVarchar("CODE_FORMULAIRE", code));
                whereCodeFormulaireEtCodeUnique.and(ConditionHelper.egalVarchar("CODE_UNIQUE", codeUnique));
                contenu.select(whereCodeFormulaireEtCodeUnique.formaterSQL());
                if (contenu.nextItem()) {
                    final String instance = contenu.getInstance();
                    proprietesContenu = new Properties();
                    proprietesContenu.load(new ByteArrayInputStream(instance.getBytes()));
                }
            }
            // recuperation des valeurs pré-saisies si retour suite a un message
            // d'erreur (champ obligatoire par ex.)
            if (infoBean != null && StringUtils.isNotEmpty(infoBean.getMessageErreur())) {
                proprietesContenu = new Properties();
                for (int numeroChamp = 0; numeroChamp < (Integer) infoBeanFormulaire.get("LISTE_NB_CHAMPS"); numeroChamp++) {
                    String codeChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_CODE");
                    if (infoBean.get(codeChamp) != null && infoBean.getString(codeChamp).length() > 0) {
                        proprietesContenu.put(codeChamp, infoBean.get(codeChamp));
                    }
                    final int nbItem = (Integer) infoBeanFormulaire.get("CHAMP" + numeroChamp + "_NB_ITEM");
                    // listes radio
                    final String codeChampItem = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + infoBean.get(codeChamp) + "_CODE");
                    if (infoBean.get(codeChamp) != null && codeChampItem != null && "RD".equals(infoBeanFormulaire.get("CHAMP" + numeroChamp + "_TYPE_LISTE"))) {
                        proprietesContenu.put(codeChamp, infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + infoBean.get(codeChamp) + "_CODE"));
                    } else if ("CB".equals(infoBeanFormulaire.get("CHAMP" + numeroChamp + "_TYPE_LISTE"))) {
                        String value = "";
                        // listes cases à cocher
                        for (int j = 0; j < nbItem; j++) {
                            if ("1".equals(infoBean.get(codeChamp + "_" + j))) {
                                if (value.length() > 0) {
                                    value += ",";
                                }
                                value += infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                            }
                        }
                        proprietesContenu.put(codeChamp, value);
                    }
                }
            }
            if ("E".equals(style) && ("DEBUT".equals(element))) {
                infoBeanFormulaire.set("STYLE_AFFICHAGE_FORMULAIRE", style);
                ctx.getDatas().put("FORMULAIRE_FORMATEUR" + code, fmt);
                ctx.getDatas().put("FORMULAIRE_INFOBEAN" + code, infoBeanFormulaire);
                if (proprietesContenu != null) {
                    ctx.getDatas().put("FORMULAIRE_CONTENU" + code, proprietesContenu);
                }
            }
        } else {
            fmt = (FormateurJSP) ctx.getDatas().get("FORMULAIRE_FORMATEUR" + code);
            infoBeanFormulaire = (InfoBean) ctx.getDatas().get("FORMULAIRE_INFOBEAN" + code);
            proprietesContenu = (Properties) ctx.getDatas().get("FORMULAIRE_CONTENU" + code);
            if ("E".equals(style) && ("FIN".equals(element))) {
                ctx.getDatas().remove("FORMULAIRE_FORMATEUR" + code);
                ctx.getDatas().remove("FORMULAIRE_INFOBEAN" + code);
                ctx.getDatas().remove("FORMULAIRE_CONTENU" + code);
            }
        }
        if (ctx.getDatas().get("AFFICHAGE_FORMULAIRE") != null && ctx.getDatas().get("AFFICHAGE_FORMULAIRE").equals("1")) {
            saisie = false;
        }
    }

    /**
     * Formater html une colonne.
     *
     * @throws Exception the exception
     */
    protected void formaterHtmlUnBloc() throws Exception {
        final int nbChamp = (Integer) infoBeanFormulaire.get("LISTE_NB_CHAMPS");
        boolean firstEvent = true;
        for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
            final String typeChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_TYPE");
            final String libelleChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_LIBELLE");
            final String typeListe = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_TYPE_LISTE");
            final boolean obligatoire = ("1").equals(infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_OBLIGATOIRE"));
            final String codeChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_CODE");
            if ("EVENT".equals(typeChamp)) {
                if (saisie) {
                    if (firstEvent) {
                        // si utilisateur anonyme, activation du champ anti spam
                        if (ctx.getCode().length() == 0) {
                            formaterChampCaptcha();
                        }
                        sw.write("<p class=\"validation\">");
                    }
                    formaterChampHtml(numeroChamp);
                    firstEvent = false;
                }
            } else {
                if (!firstEvent) {
                    sw.write("</p>");
                    firstEvent = true;
                }
                if (typeListe.equals("RD") || typeListe.equals("CB")) {
                    formaterChampHtml(numeroChamp);
                } else {
                    sw.write("<p " + (obligatoire ? "class=\"obligatoire\"" : "") + ">");
                    sw.write("<label for=\"" + codeChamp + "\">");
                    sw.write(libelleChamp);
                    if (obligatoire) {
                        sw.write(" (*)");
                    }
                    sw.write("</label>");
                    formaterChampHtml(numeroChamp);
                    sw.write("</p>");
                }
            }
        }
        if (!firstEvent) {
            sw.write("</p>");
        }
    }

    private void formaterChampCaptcha() {
        sw.write("<p class=\"validation_saisie_honeypot\">");
        sw.write("<label for=\"VALIDATION_ANONYME\">" + MessageHelper.getCoreMessage("ST_VALIDATION_HONEYPOT") + "</label>");
        sw.write("<input form=\"" + code + "\" type=\"text\" id=\"VALIDATION_ANONYME\" name=\"VALIDATION_ANONYME\" value=\"\" tabindex=\"-1\">");
        sw.write("</p>");
        sw.write(CaptchaUtils.generateHtml(code, FormulaireUtil.getProperty("VALIDATION_CAPTCHA")));
    }

    /**
     * Formater debut formulaire.
     *
     * @param disposition TODO
     * @throws Exception the exception
     */
    protected void formaterDebutFormulaire(final String disposition, final String codeRubriqueCourante) throws Exception {
        if (infoBean != null && infoBean.getMessageErreur().length() > 0) {
            sw.write("<p class=\"erreur\">" + infoBean.getMessageErreur() + "</p>");
        }
        sw.write("<form id=\"" + code + "\" method=\"post\" action=\"" + URLResolver.getAbsoluteUrl(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY), ctx) + "\" class=\"formulaire_redacteur js-form-redacteur " + disposition + "\" " + ((infoBeanFormulaire.getString("FICHIER") != null) ? "enctype=\"multipart/form-data\"" : "") + ">");
        sw.write("<div>");
        sw.write("<input type=\"hidden\" name=\"PAGE_REFERER\" value=\"" + pageReferer + "\"/>");
        sw.write("<input type=\"hidden\" name=\"PROC\" value=\"SAISIE_FORMULAIRE\"/>");
        sw.write("<input type=\"hidden\" name=\"EXT\" value=\"" + "formulaire" + "\"/>");
        sw.write("<input type=\"hidden\" name=\"ACTION\" value=\"SAISIE_ENREGISTRER\"/>");
        sw.write("<input type=\"hidden\" name=\"CODE\" value=\"" + infoBeanFormulaire.getString("CODE") + "\"/>");
        sw.write("<input type=\"hidden\" name=\"ECRAN_LOGIQUE\" value=\"FIN_SAISIE\"/>");
        if (StringUtils.isNotBlank(codeUnique)) {
            sw.write("<input type=\"hidden\" name=\"CODE_UNIQUE\" value=\"" + codeUnique + "\"/>");
        }
        if (StringUtils.isNotBlank(codeRubriqueCourante)) {
            sw.write("<input type=\"hidden\" name=\"RH\" value=\"" + codeRubriqueCourante + "\"/>");
        }
        sw.write("</div>");
    }

    /**
     * Formater fin formulaire.
     *
     * @throws Exception
     *             the exception
     */
    protected void formaterFinFormulaire() throws Exception {
        final String mentions = infoBeanFormulaire.getString("MENTIONS_LEGALES");
        // Insertion des mentions uniquement en mode colonne
        if (!"E".equals(style) && StringUtils.isNotBlank(mentions)) {
            sw.write("<p>");
            sw.write(mentions);
            sw.write("</p>");
        }
        sw.write("</form>");
    }

    /**
     * Formater champ html.
     *
     * @param numeroChamp
     *            the numero champ
     *
     * @throws Exception
     *             the exception
     */
    protected void formaterChampHtml(final int numeroChamp) throws Exception {
        final String codeChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_CODE");
        final String typeChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_TYPE");
        final String libelleChamp = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_LIBELLE");
        final boolean obligatoire = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_OBLIGATOIRE").equals("1");
        final String styleAffichageListe = infoBeanFormulaire.getString("STYLE_AFFICHAGE_FORMULAIRE");

        infoBeanFormulaire.set("ID_FORM", code);
        if ("EVENT".equals(typeChamp) && saisie) {
            final String typeAction = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_TYPE_ACTION");
            if ("ANNULER".equals(typeAction)) {
                sw.write("<input type=\"reset\" class=\"reset\" value=\"" + libelleChamp + "\" form=\"" + code + "\" /> ");
            } else if (typeAction.equals("RESET")) {
                sw.write("<input type=\"reset\" class=\"reset\" value=\"" + libelleChamp + "\" form=\"" + code + "\" /> ");
            } else {
                sw.write("<input type=\"hidden\" name=\"NUMERO_ACTION\" value=\"" + numeroChamp + "\" form=\"" + code + "\" />");
                sw.write("<input type=\"submit\" class=\"submit\" value=\"" + libelleChamp + "\" form=\"" + code + "\" /> ");
            }
        } else if ("INPUT".equals(typeChamp) || "IDENTITY".equals(typeChamp)) {
            int nbCarmax = 32;
            try {
                nbCarmax = Integer.parseInt(infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_NBCARMAX"));
            } catch (final Exception e) {
                // exception
            }
            int formatSaisie = FormateurJSP.SAISIE_FACULTATIF;
            if (infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_OBLIGATOIRE").equals("1")) {
                formatSaisie = FormateurJSP.SAISIE_OBLIGATOIRE;
            }
            if (!saisie) {
                formatSaisie = FormateurJSP.SAISIE_AFFICHAGE;
            }
            // présaisie
            if (proprietesContenu != null) {
                if (proprietesContenu.getProperty(codeChamp) != null) {
                    infoBeanFormulaire.set(codeChamp, proprietesContenu.getProperty(codeChamp));
                }
            } else {
                infoBeanFormulaire.set(codeChamp, infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_INSTANCE"));
            }
            if ("MAIL".equals(codeChamp)) {
                fmt.insererChampSaisie(sw, infoBeanFormulaire, codeChamp, formatSaisie, FormateurJSP.FORMAT_EMAIL, 0, nbCarmax, null, "LIB=" + libelleChamp);
            } else if ("ADRESSE".equals(codeChamp)) {
                fmt.insererChampSaisie(sw, infoBeanFormulaire, codeChamp, formatSaisie, FormateurJSP.FORMAT_MULTI_LIGNE, 0, nbCarmax, null, "LIB=" + libelleChamp);
            } else {
                fmt.insererChampSaisie(sw, infoBeanFormulaire, codeChamp, formatSaisie, FormateurJSP.FORMAT_TEXTE, 0, nbCarmax, null, "LIB=" + libelleChamp);
            }
        } else if ("TEXTAREA".equals(typeChamp)) {
            int nbCarmax = 8000;
            try {
                nbCarmax = Integer.parseInt(infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_NBCARMAX"));
            } catch (final Exception e) {
                //nothing
            }
            int formatSaisie = FormateurJSP.SAISIE_FACULTATIF;
            if (infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_OBLIGATOIRE").equals("1")) {
                formatSaisie = FormateurJSP.SAISIE_OBLIGATOIRE;
            }
            if (!saisie) {
                formatSaisie = FormateurJSP.SAISIE_AFFICHAGE;
            }
            // présaisie
            if (proprietesContenu != null) {
                if (proprietesContenu.getProperty(codeChamp) != null) {
                    infoBeanFormulaire.set(codeChamp, proprietesContenu.getProperty(codeChamp));
                }
            } else {
                infoBeanFormulaire.set(codeChamp, infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_INSTANCE"));
            }
            fmt.insererChampSaisie(sw, infoBeanFormulaire, codeChamp, formatSaisie, FormateurJSP.FORMAT_MULTI_LIGNE, 0, nbCarmax, null, "LIB=" + libelleChamp);
        } else if (typeChamp.equals("FILE")) {
            int nbCarmax = 200;
            try {
                nbCarmax = Integer.parseInt(infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_NBCARMAX"));
            } catch (final Exception e) {
                // nothing
            }
            int formatSaisie = FormateurJSP.SAISIE_FACULTATIF;
            if (infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_OBLIGATOIRE").equals("1")) {
                formatSaisie = FormateurJSP.SAISIE_OBLIGATOIRE;
            }
            if (!saisie) {
                formatSaisie = FormateurJSP.SAISIE_AFFICHAGE;
            }
            //présaisie
            if (proprietesContenu != null) {
                if (proprietesContenu.getProperty(codeChamp) != null) {
                    infoBeanFormulaire.set(codeChamp, proprietesContenu.getProperty(codeChamp));
                }
            } else {
                infoBeanFormulaire.set(codeChamp, infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_INSTANCE"));
            }
            fmt.insererChampSaisie(sw, infoBeanFormulaire, codeChamp, formatSaisie, FormateurJSP.FORMAT_FICHIER, 0, nbCarmax, null, "LIB=" + libelleChamp);
        } else if ("SELECT".equals(typeChamp)) {
            int formatSaisie = FormateurJSP.SAISIE_FACULTATIF;
            if (infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_OBLIGATOIRE").equals("1")) {
                formatSaisie = FormateurJSP.SAISIE_OBLIGATOIRE;
            }
            if (!saisie) {
                formatSaisie = FormateurJSP.SAISIE_AFFICHAGE;
            }
            final String typeListe = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_TYPE_LISTE");
            if ("CB".equals(typeListe)) {
                final int nbItem = (Integer) infoBeanFormulaire.get("CHAMP" + numeroChamp + "_NB_ITEM");
                if (nbItem > 0) {
                    String cssClass = StringUtils.EMPTY;
                    if (formatSaisie == FormateurJSP.SAISIE_OBLIGATOIRE && nbItem > 1) {
                        cssClass = " js-checkbox-mandatory";
                    }
                    sw.write("<fieldset " + (obligatoire ? "class=\"obligatoire\"" : "") + ">");
                    sw.write(getLegend(libelleChamp, obligatoire, styleAffichageListe));
                    sw.write("<span class=\"checkbox" + cssClass + "\"");
                    sw.write(" data-libelle=\"" + libelleChamp + "\"");
                    sw.write(">");
                }
                for (int j = 0; j < nbItem; j++) {
                    final String codeItem = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                    final String libelleItem = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                    boolean selectionne = false;
                    // présaisie
                    if (proprietesContenu != null) {
                        if (proprietesContenu.getProperty(codeChamp) != null) {
                            final String[] lstCodeSelectionne = proprietesContenu.getProperty(codeChamp).split(",", -2);
                            for (final String element2 : lstCodeSelectionne) {
                                if (element2.equals(codeItem)) {
                                    selectionne = true;
                                }
                            }
                        }
                    }
                    if (!selectionne && (proprietesContenu == null || proprietesContenu.getProperty(codeChamp) == null) && infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_SELECTION").equals("1")) {
                        selectionne = true;
                    }
                    if (selectionne) {
                        infoBeanFormulaire.set(codeChamp + "_" + j, "1");
                    }
                    sw.write("<span>");
                    if (saisie) {
                        int modeSaisie = FormateurJSP.SAISIE_FACULTATIF;
                        if (formatSaisie == FormateurJSP.SAISIE_OBLIGATOIRE && nbItem == 1) {
                            modeSaisie = FormateurJSP.SAISIE_OBLIGATOIRE;
                        }
                        fmt.insererChampSaisie(sw, infoBeanFormulaire, codeChamp + "_" + j, modeSaisie, FormateurJSP.FORMAT_CHECKBOX, 0, 255, null, "LIB=" + libelleChamp);
                        sw.write("<label for=\"" + codeChamp + "_" + j + "\" class=\"checkbox\"> ");
                        sw.write(libelleItem);
                        sw.write("</label>");
                    } else if (selectionne) {
                        sw.write("<label for=\"\"><strong>");
                        sw.write(libelleItem);
                        sw.write("</strong></label>");
                    }
                    sw.write("<br/></span>");
                }
                if (nbItem > 0) {
                    sw.write("</span>");
                    sw.write("</fieldset>");
                }
            } else if ("RD".equals(typeListe)) {
                final int nbItem = (Integer) infoBeanFormulaire.get("CHAMP" + numeroChamp + "_NB_ITEM");
                if (nbItem > 0) {
                    sw.write("<fieldset " + (obligatoire ? "class=\"obligatoire\"" : "") + ">");
                    sw.write(getLegend(libelleChamp, obligatoire, styleAffichageListe));
                    sw.write("<span class=\"radio\">");
                }
                for (int j = 0; j < nbItem; j++) {
                    final String codeItem = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                    final String libelleItem = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                    boolean selectionne = false;
                    // présaisie
                    if (proprietesContenu != null) {
                        if ((proprietesContenu.getProperty(codeChamp) != null) && (proprietesContenu.getProperty(codeChamp).equals(codeItem))) {
                            selectionne = true;
                        }
                    }
                    if (!selectionne && (proprietesContenu == null || proprietesContenu.getProperty(codeChamp) == null) && infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_SELECTION").equals("1")) {
                        selectionne = true;
                    }
                    if (selectionne) {
                        infoBeanFormulaire.set(codeChamp, String.valueOf(j));
                    }
                    sw.write("<span>");
                    if (saisie) {
                        infoBeanFormulaire.set("DONOT_PRESELECT_" + codeChamp, "1");
                        fmt.insererChampSaisie(sw, infoBeanFormulaire, codeChamp, formatSaisie, FormateurJSP.FORMAT_RADIO, 0, 255);
                        sw.write("<label for=\"" + codeChamp + "_" + fmt.getIndiceRadio() + "\" class=\"radio\"> ");
                        sw.write(libelleItem);
                        sw.write("</label>");
                    } else if (selectionne) {
                        sw.write("<label for=\"" + codeChamp + "_" + fmt.getIndiceRadio() + "\"><strong>");
                        sw.write(libelleItem);
                        sw.write("</strong></label>");
                    }
                    sw.write("<br/></span>");
                }
                if (nbItem > 0) {
                    sw.write("</span>");
                    sw.write("</fieldset>");
                }
            } else if ("S1".equals(typeListe)) {
                final int nbItem = (Integer) infoBeanFormulaire.get("CHAMP" + numeroChamp + "_NB_ITEM");
                final Hashtable<String, String> lstItem = new Hashtable<>();
                for (int j = 0; j < nbItem; j++) {
                    infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                    final String codeItem = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                    final String libelleItem = infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                    final String codeValeur = df.format(j);
                    lstItem.put(codeValeur, libelleItem);
                    // présaisie
                    if (proprietesContenu != null) {
                        if ((proprietesContenu.getProperty(codeChamp) != null) && (proprietesContenu.getProperty(codeChamp).equals(codeItem))) {
                            infoBeanFormulaire.set(codeChamp, codeValeur);
                        }
                    } else if ((proprietesContenu == null || proprietesContenu.getProperty(codeChamp) == null) && infoBeanFormulaire.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_SELECTION").equals("1")) {
                        infoBeanFormulaire.set(codeChamp, codeValeur);
                    }
                }
                infoBeanFormulaire.set("CHAMP" + numeroChamp + "_LISTE", lstItem);
                Map<String, String> table = infoBeanFormulaire.getMap("CHAMP" + numeroChamp + "_LISTE");
                if (table == null) {
                    table = new Hashtable<>();
                }
                fmt.insererComboMultiple(sw, infoBeanFormulaire, codeChamp, formatSaisie, table, "TRI_CLE,LIB=" + libelleChamp, null);
            }
        }
    }

    /**
     * RGAA : Dans chaque formulaire, les informations de même nature doivent-être regroupées :
     * les listes doivent être regroupées dans un fieldset + legend. Mais en mode "élément par élément (E)",
     * le label de la légende est ajouté manuellement dans la toolbox par le contributeur.
     * @param libelleChamp le label de la liste à renseigner dans la legend.
     * @param obligatoire obligation de saisie
     * @param styleAffichageListe le style d'affichage du formulaire : E si mode "élément par élément".
     * @return la legend si la l'affichage de la liste de valeurs n'est pas "élément par élémént".
     */
    private static String getLegend(final String libelleChamp, final boolean obligatoire, final String styleAffichageListe) {
        String ret = StringUtils.EMPTY;
        if (!("E").equals(styleAffichageListe)) {
            ret += "<legend class=\"label\">";
            ret += libelleChamp;
            if (obligatoire) {
                ret += " (*)";
            }
            ret += "</legend>";
        }
        return ret;
    }
}
