/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.email.JSBMailbox;
import com.jsbsoft.jtf.upload.UploadedFile;
import com.kosmos.formulaire.om.Contenuformulaire;
import com.kosmos.formulaire.om.Formulaire;
import com.kosmos.formulaire.util.FormulaireUtil;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.utils.Chaine;
import com.univ.utils.EscapeString;
import com.univ.utils.URLResolver;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;

/**
 * The Class SaisieFormulaireImpl.
 */
public class SaisieFormulaireImpl implements SaisieFormulaire {

    private static final Logger LOG = LoggerFactory.getLogger(SaisieFormulaireImpl.class);

    public static final String REGEX_NON_ALPHA_NUMERIC = "[^A-Za-z1-9]+";

    /* (non-Javadoc)
     * @see com.kosmos.formulaire.SaisieFormulaire#traiterSaisieFormulaire(com.univ.objetspartages.om.Formulaire, com.jsbsoft.jtf.core.InfoBean, com.jsbsoft.jtf.database.OMContext, com.univ.objetspartages.om.Utilisateur, java.lang.String, java.lang.String)
     */
    @Override
    public void traiterSaisieFormulaire(final Formulaire formulaire, final InfoBean infoBean, final OMContext ctx, final UtilisateurBean utilisateur, final String typeAction, final String mail) throws Exception {
        final String now = new SimpleDateFormat("dd/MM/yyyy").format(new Date());
        final DecimalFormat df = new DecimalFormat();
        df.applyPattern("000");
        final long idFile = System.currentTimeMillis();
        final String nomDossierPj = this.genererNomDossierPieceJointe(formulaire, idFile);
        if ((typeAction.equals("ENREGISTRER")) || (typeAction.equals("ENREGISTRER_MAIL"))) {
            final Contenuformulaire contenu = new Contenuformulaire();
            contenu.init();
            boolean contenuExistant = false;
            final Properties proprietes = new Properties();
            final String codeUnique = infoBean.getString("CODE_UNIQUE");
            if (codeUnique != null && codeUnique.length() > 0) {
                final ClauseWhere whereCodeFormulaireEtCodeUnique = new ClauseWhere(ConditionHelper.egalVarchar("CODE_FORMULAIRE", formulaire.getCode()));
                whereCodeFormulaireEtCodeUnique.and(ConditionHelper.egalVarchar("CODE_UNIQUE", codeUnique));
                contenu.select(whereCodeFormulaireEtCodeUnique.formaterSQL());
                if (contenu.nextItem()) {
                    final String instance = contenu.getInstance();
                    proprietes.load(new ByteArrayInputStream(instance.getBytes()));
                    contenuExistant = true;
                }
                contenu.setCodeUnique(codeUnique);
            }
            if (!contenuExistant) {
                proprietes.setProperty("SAISIE_DATE", now);
                proprietes.setProperty("SAISIE_UTILISATEUR_CODE", "");
                proprietes.setProperty("SAISIE_UTILISATEUR_NOM", "");
                proprietes.setProperty("SAISIE_UTILISATEUR_PRENOM", "");
                proprietes.setProperty("SAISIE_UTILISATEUR_MAIL", "");
            }
            // Formulaire non anonyme et utilisateur existant
            if (!"0".equals(formulaire.getEnregistrementUnique()) && utilisateur != null) {
                proprietes.setProperty("SAISIE_UTILISATEUR_CODE", utilisateur.getCode());
                proprietes.setProperty("SAISIE_UTILISATEUR_NOM", utilisateur.getNom());
                proprietes.setProperty("SAISIE_UTILISATEUR_PRENOM", utilisateur.getPrenom());
                proprietes.setProperty("SAISIE_UTILISATEUR_MAIL", utilisateur.getAdresseMail());
            }
            if (StringUtils.isNotEmpty(nomDossierPj)) {
                proprietes.setProperty("SAISIE_PATH_PJ", nomDossierPj);
            }
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                final String typeChamp = infoBean.getString("CHAMP" + numeroChamp + "_TYPE");
                final String codeChamp = infoBean.getString("CHAMP" + numeroChamp + "_CODE");
                if ((typeChamp.equals("INPUT")) || (typeChamp.equals("TEXTAREA")) || (typeChamp.equals("IDENTITY"))) {
                    if (infoBean.getString(codeChamp) != null) {
                        proprietes.setProperty(codeChamp, infoBean.getString(codeChamp));
                    }
                }
                if (typeChamp.equals("FILE")) {
                    final String fileCodeChamp = codeChamp + "_FILE";
                    final UploadedFile file = (UploadedFile) infoBean.get(fileCodeChamp);
                    if (file != null && file.getContentFilename().length() != 0) {
                        final String libelleChamp = infoBean.getString("CHAMP" + numeroChamp + "_LIBELLE");
                        final String filename = generateName(file.getContentFilename(), idFile);
                        proprietes.setProperty(codeChamp, libelleChamp + "/" + filename);
                    }
                }
                if (typeChamp.equals("SELECT")) {
                    final String typeListe = infoBean.getString("CHAMP" + numeroChamp + "_TYPE_LISTE");
                    String valeurListe = null;
                    final int nbItem = (Integer) infoBean.get("CHAMP" + numeroChamp + "_NB_ITEM");
                    for (int j = 0; j < nbItem; j++) {
                        if (typeListe.equals("CB")) {
                            final String codeItem = codeChamp + "_" + j;
                            if (infoBean.getString(codeItem) != null) {
                                if (infoBean.getString(codeItem).equals("1")) {
                                    if ((valeurListe == null) || (valeurListe.length() == 0)) {
                                        valeurListe = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                                    } else {
                                        valeurListe += ",";
                                        valeurListe += infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                                    }
                                } else {
                                    //pour gérer l'update d'enregistrement précédent, on prend en compte la case décochée
                                    if (valeurListe == null) {
                                        valeurListe = "";
                                    }
                                }
                            }
                        } else if (typeListe.equals("RD")) {
                            if ((infoBean.getString(codeChamp) != null) && (infoBean.get(codeChamp).equals(String.valueOf(j)))) {
                                valeurListe = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                            }
                        } else if (typeListe.equals("S1")) {
                            if ((infoBean.getString(codeChamp) != null) && (infoBean.get(codeChamp).equals(df.format(j)))) {
                                valeurListe = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                            }
                        }
                    }
                    if (valeurListe != null) {
                        proprietes.setProperty(codeChamp, valeurListe);
                    }
                }
            }
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            proprietes.store(baos, null);
            if (contenu.getCodeUnique().length() == 0) {
                contenu.setCodeUnique(String.valueOf(idFile));
            }
            contenu.setCodeFormulaire(formulaire.getCode());
            contenu.setInstance(baos.toString());
            if (contenuExistant) {
                contenu.update();
            } else {
                contenu.add();
            }
        }
        if (typeAction.equals("MAIL") || typeAction.equals("ENREGISTRER_MAIL") || typeAction.equals("ENREGISTRER_MAIL_AUTO")) {
            final String separator = FormulaireUtil.getMessage("FORMULAIRE_SAISIE_SEPARATEUR");
            final StringWriter sw = new StringWriter();
            sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_SITE"), URLResolver.getAbsoluteUrl("", false, ctx.getInfosSite())));
            sw.write(separator);
            sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_DATE"), now));
            sw.write(separator);
            // Formulaire non anonyme et utilisateur existant
            if (!"0".equals(formulaire.getEnregistrementUnique()) && utilisateur != null) {
                sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_UTILISATEUR_CODE"), utilisateur.getCode()));
                sw.write(separator);
                sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_UTILISATEUR_NOM"), utilisateur.getNom()));
                sw.write(separator);
                sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_UTILISATEUR_PRENOM"), utilisateur.getPrenom()));
                sw.write(separator);
                sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_UTILISATEUR_MAIL"), utilisateur.getAdresseMail()));
                sw.write(separator);
            }
            sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_INTITULE_FORMULAIRE"), formulaire.getTitre()));
            sw.write(separator);
            sw.write(String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_PATH_PJ"), nomDossierPj));
            sw.write(separator);
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            final ArrayList<String> files = new ArrayList<>();
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                final String codeChamp = infoBean.getString("CHAMP" + numeroChamp + "_CODE");
                final String typeChamp = infoBean.getString("CHAMP" + numeroChamp + "_TYPE");
                String valeurSaisie = "";
                if ((typeChamp.equals("INPUT")) || (typeChamp.equals("TEXTAREA")) || typeChamp.equals("FILE") || typeChamp.equals("IDENTITY")) {
                    if (infoBean.getString(codeChamp) != null) {
                        valeurSaisie = infoBean.getString(codeChamp);
                    }
                }
                if (typeChamp.equals("SELECT")) {
                    final String typeListe = infoBean.getString("CHAMP" + numeroChamp + "_TYPE_LISTE");
                    String valeurListe = null;
                    final int nbItem = (Integer) infoBean.get("CHAMP" + numeroChamp + "_NB_ITEM");
                    for (int j = 0; j < nbItem; j++) {
                        if (typeListe.equals("CB")) {
                            final String codeItem = codeChamp + "_" + j;
                            if (infoBean.getString(codeItem) != null) {
                                if (infoBean.getString(codeItem).equals("1")) {
                                    infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                                    if ((valeurListe == null) || (valeurListe.length() == 0)) {
                                        valeurListe = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                                    } else {
                                        valeurListe += ", ";
                                        valeurListe += infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                                    }
                                } else {
                                    //pour gérer l'update d'enregistrement précédent, on prend en compte la case décochée
                                    if (valeurListe == null) {
                                        valeurListe = "";
                                    }
                                }
                            }
                        } else if (typeListe.equals("RD")) {
                            if ((infoBean.getString(codeChamp) != null) && (infoBean.get(codeChamp).equals(String.valueOf(j)))) {
                                valeurListe = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                                infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                            }
                        } else if (typeListe.equals("S1")) {
                            if ((infoBean.getString(codeChamp) != null) && (infoBean.get(codeChamp).equals(df.format(j)))) {
                                valeurListe = infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                                infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                            }
                        }
                        if (valeurListe != null) {
                            valeurSaisie = valeurListe;
                        }
                    }
                }
                if (typeChamp.equals("FILE")) {
                    final String fileCodeChamp = codeChamp + "_FILE";
                    final UploadedFile file = (UploadedFile) infoBean.get(fileCodeChamp);
                    final String nomFichier = "";
                    if (file != null && file.getContentFilename().length() != 0) {
                        final String pathFichier = this.enregistrerPieceJointe(file, idFile, formulaire, nomDossierPj);
                        files.add(pathFichier.toString());
                    }
                }
                if (!typeChamp.equals("EVENT")) {
                    sw.write(infoBean.getString("CHAMP" + numeroChamp + "_LIBELLE") + " : " + valeurSaisie);
                    sw.write(separator);
                }
            }
            sw.write(separator);
            if (typeAction.equals("ENREGISTRER_MAIL")) {
                sw.write(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_ACTION_ENREGISTER"));
            }
            final String texteMail = Chaine.formatToHtml(EscapeString.unescapeHtml(sw.getBuffer().toString()));
            final JSBMailbox mailbox = new JSBMailbox(false);
            final String sujet = String.format(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_SUJET_MAIL"), formulaire.getTitre());
            // Initialisation du champs from avec le mail saisie dans le formulaire
            if (files.size() > 0) {
                final String[] tabPath = new String[files.size()];
                files.toArray(tabPath);
                mailbox.sendHtmlMsgWithAttachedFiles(null, mail, sujet, texteMail, tabPath);
            } else {
                mailbox.sendHtmlMsg(null, mail, sujet, texteMail);
            }
        }
        if (typeAction.equals("ENREGISTRER")) {
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                final String typeChamp = infoBean.getString("CHAMP" + numeroChamp + "_TYPE");
                if (typeChamp.equals("FILE")) {
                    final String codeChamp = infoBean.getString("CHAMP" + numeroChamp + "_CODE");
                    final String libelleChamp = infoBean.getString("CHAMP" + numeroChamp + "_LIBELLE");
                    final String fileCodeChamp = codeChamp + "_FILE";
                    final UploadedFile file = (UploadedFile) infoBean.get(fileCodeChamp);
                    final String nomFichier = "";
                    if (file != null && file.getContentFilename().length() != 0) {
                        this.enregistrerPieceJointe(file, idFile, formulaire, nomDossierPj);
                    }
                }
            }
        }
    }

    public String generateName(final String nomFichier, final long id) {
        final String nomFichierSansExtension = StringUtils.defaultString(StringUtils.substringBeforeLast(nomFichier, "."));
        return Chaine.formatString(nomFichierSansExtension) + "_" + id + "." + getExtension(nomFichier);
    }

    public String getExtension(final String source) {
        String res = "";
        final int i = source.lastIndexOf(".");
        if (i != -1) {
            res = source.substring(i + 1);
        }
        return res;
    }

    /**
     * Méthode permettant de persister les pièces jointes sur le système de fichier
     * @param file Le fichier à enregistrer
     * @param idFile l'id unique génerer pour un formulaire
     * @param form Le formulaire courant
     * @param pathDossierFormulairePJ Le nom du dossier dans lequel on doit enregistrer les pièces jointes
     * @return Le path du fichier sauvegardé
     * @throws IOException
     */
    private String enregistrerPieceJointe(final UploadedFile file, final long idFile, final Formulaire form, final String pathDossierFormulairePJ) throws IOException {
        String nomFichier = file.getContentFilename();
        LOG.debug("FileName : {}", nomFichier);
        final File source = file.getTemporaryFile();
        nomFichier = generateName(file.getContentFilename(), idFile);
        LOG.debug("Nom du fichier généré {}", nomFichier);
        final String pathFichier = FormulaireUtil.getFormsDefaultPath() + File.separator +form.getCode() +File.separator + pathDossierFormulairePJ;
        LOG.debug("Path du fichier {}", pathFichier);
        final File formDir = new File(pathFichier);
        formDir.mkdirs();
        final File destination = new File(pathFichier,nomFichier);
        LOG.debug("Répertoire destination {}", StringUtils.defaultString(formDir.getPath()));
        FileUtils.copyFile(source, destination);
        return destination.getPath();
    }

    /**
     * Méhode permettant de générer le nom du dossier des pièces jointes par formulaire
     * @param formulaire Afin de récuperer le code du formulaire
     * @param idFile L'id des fichiers
     * @return Le nom du dossier en tenant compte du code formulaire et l'id du fichier
     */
    private String genererNomDossierPieceJointe(final Formulaire formulaire, final long idFile) {
        final StringBuilder str = new StringBuilder();
        String titreFormulaire = StringUtils.defaultString(formulaire.getTitre());
        titreFormulaire = titreFormulaire.toUpperCase();
        // On remplace tous les carctère non alphanumérique par "", afin d'avoir un nom de dossier cohérent
        titreFormulaire = titreFormulaire.replaceAll(REGEX_NON_ALPHA_NUMERIC, "");
        LOG.debug("Titre formulaire : {}", titreFormulaire);
        str.append(formulaire.getCode());
        str.append("_");
        str.append(titreFormulaire);
        str.append("_");
        str.append(idFile);
        LOG.debug("Nom dossier final {}", str.toString());
        return str.toString();
    }
}