/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.dao.impl;

import java.util.List;

import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.jsbsoft.jtf.datasource.dao.impl.mysql.AbstractLegacyDAO;
import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.kosmos.formulaire.bean.ContenuformulaireBean;

/**
 * DAO permettant de gérer le contenu d'un formulaire kportal.
 */
public class ContenuformulaireDAO extends AbstractLegacyDAO<ContenuformulaireBean> {

    public ContenuformulaireDAO() {
        tableName = "CONTENUFORMULAIRE";
    }

    /**
     * Méthode permettant de récupérer tous les objets de la table CONTENUFORMULAIRE.
     * @return La liste des ContenuformulaireBean de la table CONTENUFORMULAIRE
     */
    public List<ContenuformulaireBean> getAll() {
        final List<ContenuformulaireBean> contenus;
        try {
            contenus = namedParameterJdbcTemplate.query("SELECT * FROM CONTENUFORMULAIRE", rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting CONTENUFORMULAIRE rows", dae);
        }
        return contenus;
    }

    /**
     * Méthode permettant de récupérer les objets de la table CONTENUFORMULAIRE pour un formulaire donné.
     * @param code le code formulaire pour lequel on veut retrouver les contenus
     * @return La liste des ContenuformulaireBean d'un formulaire de la table CONTENUFORMULAIRE
     */
    public List<ContenuformulaireBean> getAllByFormCode(final String code) {
        final List<ContenuformulaireBean> contenus;
        try {
            final SqlParameterSource params = new MapSqlParameterSource("code", code);
            contenus = namedParameterJdbcTemplate.query("SELECT * FROM CONTENUFORMULAIRE WHERE CODE_FORMULAIRE = :code", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting CONTENUFORMULAIRE rows", dae);
        }
        return contenus;
    }
}