/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.migration;

import java.sql.Connection;
import java.text.ParseException;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.flywaydb.core.api.migration.MigrationChecksumProvider;
import org.flywaydb.core.api.migration.jdbc.JdbcMigration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.formulaire.bean.ContenuformulaireBean;
import com.kosmos.formulaire.bean.FormulaireBean;
import com.kosmos.formulaire.service.impl.ContenuFormulaireService;
import com.kosmos.formulaire.service.impl.FormulaireService;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.PropertyHelper;

/**
 * Script exécuté après la création de la colonne DATE_CREATION dans la table CONTENUFORMULAIRE,
 * Et la création de la colonne DUREE_CONSERVATION dans la table FORMULAIRE
 * DATE_CREATION : Récupère le champ INSTANCE de tous les contenus et y récupère la date présente pour la copier dans la nouvelle colonne crée.
 * DUREE_CONSERVATION : A la création de la colonne, passe la valeur de tous les formulaires à celle définie par défaut dans la propriété.
 */
public class V1_7_0_1__alimentation_colonne_date_creation implements JdbcMigration, MigrationChecksumProvider {

    private static final Logger LOG = LoggerFactory.getLogger(V1_7_0_1__alimentation_colonne_date_creation.class);

    @Override
    public Integer getChecksum() {
        return 1535102969;
    }

    @Override
    public void migrate(final Connection connection) {
        FormulaireService formulaireService = ServiceManager.getService(FormulaireService.class);
        //On met le champ DUREE_CONSERVATION à la valeur définie défaut dans la propriété.
        List<FormulaireBean> listForms = formulaireService.getAllForms("ID_FORMULAIRE");
        for (FormulaireBean formulaire : listForms) {
            formulaire.setDureeConservation(PropertyHelper.getCorePropertyAsInt("formulaire.purge.delay.months", 12));
            formulaireService.save(formulaire);
        }

        ContenuFormulaireService service = ServiceManager.getService(ContenuFormulaireService.class);
        List<ContenuformulaireBean> listContenus = service.getAllContenus();
        for (ContenuformulaireBean contenu : listContenus) {
            //On récupère la date du contenu stockée dans le champ Instance pour l'ajouter au à la colonne DATE_CREATION.
            String instance = contenu.getInstance().replaceAll(StringUtils.CR + StringUtils.LF, StringUtils.LF);
            String dateString = StringUtils.substringBetween(instance, "#", StringUtils.LF);
            Date dateCreation = new Date();
            try {
                dateCreation = DateUtils.parseDateStrictly(dateString, Locale.US, "EEE MMM dd HH:mm:ss z yyyy");
            } catch (ParseException e) {
                LOG.error("Une erreur est survenue lors du formattage de la date", e);
            }
            contenu.setInstance(instance);
            contenu.setDateCreation(dateCreation);
            service.save(contenu);
        }
    }
}
