/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.om;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.formulaire.bean.FormulaireBean;
import com.kosmos.formulaire.dao.impl.FormulaireDAO;
import com.kosmos.formulaire.util.FormulaireUtil;
import com.univ.objetspartages.om.AbstractOm;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;

/**
 * The Class Formulaire.
 */
public class Formulaire extends AbstractOm<FormulaireBean, FormulaireDAO> implements Cloneable {

    /**
     * Instantiates a new formulaire.
     */
    public Formulaire() {
        super();
    }

    /**
     * implementation de la methode clone() (utilisée dans le module catalogue)
     */
    @Override
    public Formulaire clone() throws CloneNotSupportedException {
        return (Formulaire) super.clone();
    }

    /**
     * Initialisation de l'objet.
     */
    public void init() {
        setIdFormulaire(System.currentTimeMillis());
        setCode(StringUtils.EMPTY);
        setCodeRubrique(StringUtils.EMPTY);
        setCodeRattachement(StringUtils.EMPTY);
        setModele(StringUtils.EMPTY);
        setTitre(StringUtils.EMPTY);
        setEnregistrementUnique("0");
        setMentionsLegales(StringUtils.EMPTY);
        setDureeConservation(FormulaireUtil.getPurgeDelay());
    }
    // Sélection d'une instance de l'objet Formulaire à partir de l'ensemble des
    // critères combinés
    // !!!! Complèter le code !!!!

    /**
     * Select.
     *
     * @param code
     *            the _code
     * @param titre
     *            the _titre
     *
     * @return the int
     *
     * @throws Exception
     *             the exception
     */
    public int select(String code, String titre) throws Exception {
        ClauseWhere whereCodeEtTitre = new ClauseWhere();
        if (StringUtils.isNotEmpty(code)) {
            whereCodeEtTitre.setPremiereCondition(ConditionHelper.egalVarchar("CODE", code));
        }
        if (StringUtils.isNotEmpty(titre)) {
            whereCodeEtTitre.and(ConditionHelper.egalVarchar("TITRE", titre));
        }
        return select(whereCodeEtTitre.formaterSQL());
    }

    public Long getIdFormulaire() {
        return persistenceBean.getId();
    }

    public void setIdFormulaire(Long idFormulaire) {
        persistenceBean.setId(idFormulaire);
    }

    public String getCode() {
        return persistenceBean.getCode();
    }

    public void setCode(String code) {
        persistenceBean.setCode(code);
    }

    public String getTitre() {
        return persistenceBean.getTitre();
    }

    public void setTitre(String titre) {
        persistenceBean.setTitre(titre);
    }

    public String getEnregistrementUnique() {
        return persistenceBean.getEnregistrementUnique();
    }

    public void setEnregistrementUnique(String enregistrementUnique) {
        persistenceBean.setEnregistrementUnique(enregistrementUnique);
    }

    public String getCodeRubrique() {
        return persistenceBean.getCodeRubrique();
    }

    public void setCodeRubrique(String codeRubrique) {
        persistenceBean.setCodeRubrique(codeRubrique);
    }

    public String getCodeRattachement() {
        return persistenceBean.getCodeRattachement();
    }

    public void setCodeRattachement(String codeRattachement) {
        persistenceBean.setCodeRattachement(codeRattachement);
    }

    public String getMentionsLegales() {
        return persistenceBean.getMentionsLegales();
    }

    public void setMentionsLegales(String mentionsLegales) {
        persistenceBean.setMentionsLegales(mentionsLegales);
    }

    public String getModele() {
        return persistenceBean.getModele();
    }

    public void setModele(String modele) {
        persistenceBean.setModele(modele);
    }

    public int getDureeConservation() {
        return persistenceBean.getDureeConservation();
    }

    public void setDureeConservation(int duree) {
        persistenceBean.setDureeConservation(duree);
    }
}
