/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.processus;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.text.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.TypeMessage;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.controllers.SGController;
import com.kosmos.formulaire.ExtractionDonneesPersonnelles;
import com.kosmos.formulaire.ExtractionFormulaire;
import com.kosmos.formulaire.FormatageFormulaire;
import com.kosmos.formulaire.SaisieFormulaire;
import com.kosmos.formulaire.bean.ContenuformulaireBean;
import com.kosmos.formulaire.bean.FormulaireBean;
import com.kosmos.formulaire.dto.ContenuFormulaireDto;
import com.kosmos.formulaire.dto.FormulaireDto;
import com.kosmos.formulaire.om.Contenuformulaire;
import com.kosmos.formulaire.om.Formulaire;
import com.kosmos.formulaire.service.impl.ContenuFormulaireService;
import com.kosmos.formulaire.service.impl.FormulaireService;
import com.kosmos.formulaire.util.FormulaireUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.utils.SaisieUtils;
import com.kportal.captcha.CaptchaUtils;
import com.kportal.captcha.exception.EmptyCaptchaException;
import com.kportal.captcha.exception.ExpiredCaptchaException;
import com.kportal.captcha.exception.InvalidCaptchaException;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.Perimetre;
import com.univ.objetspartages.om.PermissionBean;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.InfoImplementation;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.services.ServiceUser;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;

/**
 * processus gérant le formulaire. Ce processus est un mystère.
 */
public class TraitementFormulaire extends ProcessusBean {

    public static final PermissionBean PERMISSION_ECRAN = new PermissionBean("TECH", "for", "M");

    private static final Logger LOG = LoggerFactory.getLogger(TraitementFormulaire.class);

    private static final char[] hexDigit = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

    /** The Constant ECRAN_LISTE. */
    private static final String ECRAN_LISTE = "LISTE";

    /** The Constant ECRAN_RECHERCHE_DONNEES_PERSONNELLES. */
    private static final String ECRAN_RECHERCHE_DONNEES_PERSONNELLES = "RECHERCHE_DONNEES_PERSONNELLES";

    /** The Constant ECRAN_RESULTAT_RECHERCHE. */
    private static final String ECRAN_RESULTAT_RECHERCHE = "ECRAN_RESULTAT_RECHERCHE";

    /** The Constant ECRAN_DETAIL. */
    private static final String ECRAN_DETAIL = "DETAIL";

    /** The Constant ECRAN_DETAIL_CHAMP. */
    private static final String ECRAN_DETAIL_CHAMP = "DETAIL_CHAMP";

    /** The Constant ECRAN_DETAIL_ITEM. */
    private static final String ECRAN_DETAIL_ITEM = "DETAIL_ITEM";

    /** The Constant ECRAN_EXTRACTION. */
    private static final String ECRAN_EXTRACTION = "EXTRAIRE";

    /** The Constant ECRAN_ACCUEIL. */
    private static final String ECRAN_ACCUEIL = "ACCUEIL";

    /** The Constant ECRAN_DETAIL_EXTRACTION. */
    private static final String ECRAN_DETAIL_EXTRACTION = "DETAIL_EXTRACTION";

    /** The Constant ECRAN_DETAIL_INSERTION. */
    private static final String ECRAN_DETAIL_INSERTION = "DETAIL_INSERTION";

    private static final String ECRAN_REDIRECTION = "REDIRECTION";

    private static final String ECRAN_CONFIRMATION = "CONFIRMATION";

    private static final String ACTION_EXTRAIRE_DONNEES_PERSONNELLES = "EXTRAIRE_DONNEES_PERSONNELLES";

    /** The lst type disposition. */
    private static Map<String, String> lstTypeDisposition = null;

    /** The lst type liste. */
    private static Map<String, String> lstTypeListe = null;

    /** The lst type champ. */
    private static Map<String, String> lstTypeChamp = null;

    /** The lst type action. */
    private static Map<String, String> lstTypeAction = null;

    /** The lst type identite. */
    private static HashMap<String, String> lstTypeIdentite = null;

    /** The lst type extraction. */
    private static Map<String, String> lstTypeExtraction = null;

    /** The lst action formulaire. */
    private static Map<String, InfoImplementation> lstActionFormulaire = null;

    /** The lst identite formulaire. */
    private static Map<String, InfoImplementation> lstIdentiteFormulaire = null;

    /** The lst extraction formulaire. */
    private static Map<String, InfoImplementation> lstExtractionFormulaire = null;

    /** The lst disposition formulaire. */
    private static Map<String, InfoImplementation> lstDispositionFormulaire = null;

    private final ServiceUser serviceUser;

    private final ServiceRubrique serviceRubrique;

    private final ServiceStructure serviceStructure;

    private final ServiceMetatag serviceMetatag;

    private final FormulaireService serviceFormulaire;

    private final ContenuFormulaireService serviceContenuFormulaire;

    private final UrlProvider urlProvider;

    /** The formulaire. */
    protected Formulaire formulaire = null;

    /** The autorisations. */
    private AutorisationBean autorisations = null;

    /**
     * processus saisie Libelle.
     *
     * @param ciu
     *            com.jsbsoft.jtf.core.InfoBean
     */
    public TraitementFormulaire(final InfoBean ciu) {
        super(ciu);
        formulaire = new Formulaire();
        checkInit();
        infoBean.set("LISTE_TYPE_LISTE", lstTypeListe);
        infoBean.set("LISTE_TYPE_CHAMP", lstTypeChamp);
        infoBean.set("LISTE_TYPE_ACTION", lstTypeAction);
        infoBean.set("LISTE_TYPE_IDENTITE", lstTypeIdentite);
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
        serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        serviceFormulaire = ServiceManager.getServiceForBean(FormulaireBean.class);
        urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
        serviceContenuFormulaire = ServiceManager.getServiceForBean(ContenuformulaireBean.class);
    }

    /**
     * Instantiates a new traitement formulaire.
     */
    public TraitementFormulaire() {
        super(new InfoBean());
        checkInit();
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
        serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        serviceFormulaire = ServiceManager.getServiceForBean(FormulaireBean.class);
        urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
        serviceContenuFormulaire = ServiceManager.getServiceForBean(ContenuformulaireBean.class);
    }

    /**
     * Check null.
     *
     * @param o
     *            the o
     *
     * @return the string
     */
    private static String checkNull(final Object o) {
        if (o == null) {
            return "";
        }
        return o.toString();
    }

    /**
     * Check init.
     */
    private static void checkInit() {
        if (lstTypeChamp == null) {
            lstTypeChamp = new HashMap<>();
            lstTypeChamp.put("EVENT", "Action");
            lstTypeChamp.put("INPUT", "Texte simple");
            lstTypeChamp.put("TEXTAREA", "Texte multiligne");
            lstTypeChamp.put("SELECT", "Liste de valeurs");
            lstTypeChamp.put("FILE", "Pièce jointe");
            lstTypeChamp.put("IDENTITY", "Identité");
        }
        if (lstTypeListe == null) {
            lstTypeListe = new HashMap<>();
            lstTypeListe.put("CB", "Cases à cocher");
            lstTypeListe.put("RD", "Boutons radios");
            lstTypeListe.put("S1", "Liste déroulante");
        }
        if (lstTypeAction == null) {
            lstTypeAction = new HashMap<>();
            lstActionFormulaire = new HashMap<>();
            int i = 1;
            String param;
            while ((param = PropertyHelper.getProperty("formulaire", "formulaire.action" + i)) != null) {
                try {
                    fillLists(lstTypeAction, lstActionFormulaire, param, SaisieFormulaire.class);
                    i++;
                } catch (final ClassNotFoundException e) {
                    LOG.error("Erreur dans le paramétrage des actions formulaire : " + param, e);
                }
            }
        }
        if (lstTypeIdentite == null) {
            lstTypeIdentite = new HashMap<>();
            lstIdentiteFormulaire = new HashMap<>();
            int i = 1;
            String param;
            while ((param = PropertyHelper.getProperty("formulaire", "formulaire.identite" + i)) != null) {
                try {
                    fillLists(lstTypeIdentite, lstIdentiteFormulaire, param, SaisieFormulaire.class);
                    i++;
                } catch (ClassNotFoundException e) {
                    LOG.error("Erreur dans le paramétrage des identités : " + param, e);
                }
            }
        }
        if (lstTypeExtraction == null) {
            lstTypeExtraction = new HashMap<>();
            lstExtractionFormulaire = new HashMap<>();
            int i = 1;
            String param;
            while ((param = PropertyHelper.getProperty("formulaire", "formulaire.extraction" + i)) != null) {
                try {
                    fillLists(lstTypeExtraction, lstExtractionFormulaire, param, ExtractionFormulaire.class);
                    i++;
                } catch (final ClassNotFoundException e) {
                    LOG.error("Erreur dans le paramétrage des extractions formulaire : " + param, e);
                }
            }
        }
        if (lstTypeDisposition == null) {
            lstTypeDisposition = new HashMap<>();
            lstDispositionFormulaire = new HashMap<>();
            int i = 1;
            String param;
            while ((param = PropertyHelper.getProperty("formulaire", "formulaire.disposition" + i)) != null) {
                try {
                    fillLists(lstTypeDisposition, lstDispositionFormulaire, param, FormatageFormulaire.class);
                    i++;
                } catch (final ClassNotFoundException e) {
                    LOG.error("Erreur dans le paramétrage des dispositions formulaire : " + param, e);
                }
            }
        }
    }

    /**
     * Alimente les listes de type de champ et d'implémentation des champs
     * @param listType Lise contenant les différents types du champ
     * @param listForm Liste contenant le code des champs et leur implémentation
     * @param param Paramètres des champs récupérés dans les properties
     * @param clazz Classe à implémenter
     * @throws ClassNotFoundException
     */
    private static void fillLists(Map<String, String> listType, Map<String, InfoImplementation> listForm, String param, Class<?> clazz) throws ClassNotFoundException {
        final String[] lstParam = StringUtils.split(param, ';');
        final InfoImplementation infoImplementation = new InfoImplementation();
        if (ArrayUtils.isNotEmpty(lstParam)) {
            infoImplementation.setCode(lstParam[0]);
            infoImplementation.setLibelle(lstParam[1]);
            infoImplementation.setNomClasse(lstParam[2]);
            if (lstParam.length == 4) { //cas des "formulaire.identite" dans formulaire.properties (4 champs)
                infoImplementation.setAffichageRecherche(Boolean.parseBoolean(lstParam[3]));
            }
            //Vérification qu'on implémente la bonne classe
            if (clazz.isAssignableFrom(Class.forName(infoImplementation.getNomClasse()))) {
                listType.put(infoImplementation.getCode(), infoImplementation.getLibelle());
                listForm.put(infoImplementation.getCode(), infoImplementation);
            }
        } else {
            LOG.warn("Mauvaise configuration de ligne dans le fichier formulaire.properties");
        }
    }

    /**
     * Formater html.
     *
     * @param _ctx
     *            the _ctx
     * @param code
     *            the code
     * @param style
     *            the style
     * @param element
     *            the element
     *
     * @return the string
     */
    public static String formaterHtml(final ContexteUniv _ctx, final String code, final String style, final String element) {
        String out = "";
        try {
            final InfoImplementation implementation = TraitementFormulaire.getLstDispositionFormulaire().get(style);
            if (implementation != null) {
                final FormatageFormulaire formatageFormulaire = (FormatageFormulaire) (Class.forName(implementation.getNomClasse()).newInstance());
                out = formatageFormulaire.formaterHtml(_ctx, code, style, element);
            }
        } catch (final Exception e) {
            LOG.error("Exception lors du formatage d'un champ de formulaire", e);
        }
        return out;
    }

    /**
     * Renseigner infobean.
     *
     * @param unInfoBean
     *            the un info bean
     * @param unFormulaire
     *            the un formulaire
     *
     * @throws Exception
     *             the exception
     */
    private static void renseignerInfobean(final InfoBean unInfoBean, final Formulaire unFormulaire) throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        unInfoBean.set("CODE", unFormulaire.getCode());
        unInfoBean.set("TITRE", unFormulaire.getTitre());
        unInfoBean.set("CODE_RATTACHEMENT", unFormulaire.getCodeRattachement());
        unInfoBean.set("ENREGISTREMENT_UNIQUE", unFormulaire.getEnregistrementUnique());
        unInfoBean.set("MENTIONS_LEGALES", unFormulaire.getMentionsLegales());
        unInfoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(unFormulaire.getCodeRattachement(), "0"));
        unInfoBean.set("CODE_RUBRIQUE", unFormulaire.getCodeRubrique());
        unInfoBean.set("DUREE_CONSERVATION", unFormulaire.getDureeConservation());
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(unFormulaire.getCodeRubrique());
        if (rubriqueBean != null) {
            unInfoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueBean.getIntitule());
        }
        String modele = unFormulaire.getModele();
        if (modele == null) {
            modele = "";
        }
        final Properties proprietes = new Properties();
        proprietes.load(new ByteArrayInputStream(modele.getBytes()));
        unInfoBean.set("LISTE_NB_CHAMPS", new Integer(proprietes.getProperty("LISTE_NB_CHAMPS")));
        final int nbChamp = (Integer) unInfoBean.get("LISTE_NB_CHAMPS");
        for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
            if ("FILE".equals(proprietes.getProperty("CHAMP" + numeroChamp + "_TYPE"))) {
                unInfoBean.set("FICHIER", "oui");
            }
            unInfoBean.set("CHAMP" + numeroChamp + "_CODE", proprietes.getProperty("CHAMP" + numeroChamp + "_CODE"));
            unInfoBean.set("CHAMP" + numeroChamp + "_TYPE", proprietes.getProperty("CHAMP" + numeroChamp + "_TYPE"));
            unInfoBean.set("CHAMP" + numeroChamp + "_LIBELLE", proprietes.getProperty("CHAMP" + numeroChamp + "_LIBELLE"));
            unInfoBean.set("CHAMP" + numeroChamp + "_TYPE_ACTION", proprietes.getProperty("CHAMP" + numeroChamp + "_TYPE_ACTION"));
            unInfoBean.set("CHAMP" + numeroChamp + "_MAIL", proprietes.getProperty("CHAMP" + numeroChamp + "_MAIL"));
            unInfoBean.set("CHAMP" + numeroChamp + "_DESTINATION", proprietes.getProperty("CHAMP" + numeroChamp + "_DESTINATION"));
            unInfoBean.set("LIBELLE_CHAMP" + numeroChamp + "_DESTINATION", proprietes.getProperty("LIBELLE_CHAMP" + numeroChamp + "_DESTINATION"));
            unInfoBean.set("CHAMP" + numeroChamp + "_INSTANCE", proprietes.getProperty("CHAMP" + numeroChamp + "_INSTANCE"));
            unInfoBean.set("CHAMP" + numeroChamp + "_OBLIGATOIRE", proprietes.getProperty("CHAMP" + numeroChamp + "_OBLIGATOIRE"));
            unInfoBean.set("CHAMP" + numeroChamp + "_NBCARMAX", checkNull(proprietes.getProperty("CHAMP" + numeroChamp + "_NBCARMAX")));
            unInfoBean.set("CHAMP" + numeroChamp + "_TYPE_LISTE", proprietes.getProperty("CHAMP" + numeroChamp + "_TYPE_LISTE"));
            unInfoBean.set("CHAMP" + numeroChamp + "_NB_ITEM", new Integer(proprietes.getProperty("CHAMP" + numeroChamp + "_NB_ITEM")));
            unInfoBean.set("CHAMP" + numeroChamp + "_TYPE", proprietes.getProperty("CHAMP" + numeroChamp + "_TYPE"));
            final int nbItem = (Integer) unInfoBean.get("CHAMP" + numeroChamp + "_NB_ITEM");
            for (int j = 0; j < nbItem; j++) {
                final String codeItem = proprietes.getProperty("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                final String libelleItem = proprietes.getProperty("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                unInfoBean.set("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE", codeItem);
                unInfoBean.set("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE", libelleItem);
                unInfoBean.set("CHAMP" + numeroChamp + "_ITEM" + j + "_SELECTION", proprietes.getProperty("CHAMP" + numeroChamp + "_ITEM" + j + "_SELECTION"));
            }
        }
    }

    /**
     * Gets the info bean formulaire.
     *
     * @param ctx
     *            the ctx
     * @param codeFormulaire
     *            the code formulaire
     *
     * @return the info bean formulaire
     *
     * @throws Exception
     *             the exception
     */
    public static InfoBean getInfoBeanFormulaire(final ContexteUniv ctx, final String codeFormulaire) throws Exception {
        final Formulaire unFormulaire = new Formulaire();
        unFormulaire.init();
        final ClauseWhere whereCode = whereCodeEgal(codeFormulaire);
        if (unFormulaire.select(whereCode.formaterSQL()) == 0) {
            return null;
        }
        unFormulaire.nextItem();
        final InfoBean unInfoBean = new InfoBean();
        renseignerInfobean(unInfoBean, unFormulaire);
        return unInfoBean;
    }

    /**
     * Gets the lst disposition formulaire.
     *
     * @return Returns the lstDispositionFormulaire.
     */
    private static Map<String, InfoImplementation> getLstDispositionFormulaire() {
        checkInit();
        return lstDispositionFormulaire;
    }

    /**
     * Migrer code.
     *
     * @param ctx
     *            the ctx
     *
     * @return the string
     *
     * @throws Exception
     *             the exception
     */
    public static String migrerCode(final OMContext ctx) throws Exception {
        final Hashtable<String, Properties> lstPropertyFormulaire = new Hashtable<>();
        final Formulaire form = new Formulaire();
        form.init();
        form.select("");
        while (form.nextItem()) {
            String modele = form.getModele();
            if (modele == null) {
                modele = "";
            }
            final Properties proprietes = new Properties();
            proprietes.load(new ByteArrayInputStream(modele.getBytes()));
            final int nbChamp = new Integer(proprietes.getProperty("LISTE_NB_CHAMPS"));
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                String codeChamp = proprietes.getProperty("CHAMP" + numeroChamp + "_CODE");
                final String typeChamp = proprietes.getProperty("CHAMP" + numeroChamp + "_TYPE");
                if (codeChamp == null) {
                    codeChamp = proprietes.getProperty("CHAMP" + numeroChamp + "_LIBELLE");
                }
                proprietes.setProperty("CHAMP" + numeroChamp + "_CODE", codeChamp);
                if (typeChamp.equals("SELECT")) {
                    final int nbItem = Integer.parseInt(proprietes.getProperty("CHAMP" + numeroChamp + "_NB_ITEM"));
                    for (int j = 0; j < nbItem; j++) {
                        String codeItem = proprietes.getProperty("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                        if (codeItem == null) {
                            codeItem = proprietes.getProperty("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE");
                        }
                        proprietes.setProperty("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE", codeItem);
                    }
                }
            }
            lstPropertyFormulaire.put(form.getCode(), proprietes);
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            proprietes.store(baos, null);
            form.setModele(modele);
            form.update();
        }
        final Contenuformulaire contenu = new Contenuformulaire();
        contenu.init();
        contenu.select("");
        while (contenu.nextItem()) {
            final Properties proprietes = lstPropertyFormulaire.get(contenu.getCodeFormulaire());
            String instance = contenu.getInstance();
            if (instance == null) {
                instance = "";
            }
            final Properties propInstance = new Properties();
            propInstance.load(new ByteArrayInputStream(instance.getBytes()));
            final int nbChamp = new Integer(proprietes.getProperty("LISTE_NB_CHAMPS"));
            for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
                final String codeChamp = proprietes.getProperty("CHAMP" + numeroChamp + "_CODE");
                final String typeChamp = proprietes.getProperty("CHAMP" + numeroChamp + "_TYPE");
                if ((typeChamp.equals("INPUT")) || (typeChamp.equals("TEXTAREA")) || (typeChamp.equals("FILE")) || (typeChamp.equals("IDENTITY"))) {
                    final String valeurChamp = propInstance.getProperty("CHAMP" + numeroChamp + "_INSTANCE");
                    if (valeurChamp != null) {
                        propInstance.remove("CHAMP" + numeroChamp + "_INSTANCE");
                        propInstance.setProperty(codeChamp, valeurChamp);
                    }
                }
                if (typeChamp.equals("SELECT")) {
                    String valeurChamp = "";
                    final int nbItem = Integer.parseInt(proprietes.getProperty("CHAMP" + numeroChamp + "_NB_ITEM"));
                    for (int j = 0; j < nbItem; j++) {
                        final String codeItem = proprietes.getProperty("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE");
                        if (propInstance.getProperty("CHAMP" + numeroChamp + "_ITEM" + j).equals("1")) {
                            if (valeurChamp.length() > 0) {
                                valeurChamp += ",";
                            }
                            valeurChamp += codeItem;
                        }
                    }
                    propInstance.setProperty(codeChamp, valeurChamp);
                }
            }
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            propInstance.store(baos, null);
            contenu.setInstance(instance);
            contenu.update();
        }
        return "OK";
    }

    private static ClauseWhere whereCodeEgal(final String code) {
        return new ClauseWhere(ConditionHelper.egalVarchar("CODE", code));
    }

    private static String toUnicode(String in) {
        StringBuilder sb = new StringBuilder();
        for (char c : in.toCharArray()) {
            if (c < 0x0020 || c > 0x007E) {
                sb.append("\\u").append(toHex((c >> 12) & 0xF)).append(toHex((c >> 8) & 0xF)).append(toHex((c >> 4) & 0xF)).append(toHex(c & 0xF));
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }

    private static char toHex(int nibble) {
        return hexDigit[nibble & 0xF];
    }

    /**
     * Preparer liste.
     *
     */
    protected void preparerLISTE() {
        int i = 0;
        ecranLogique = ECRAN_LISTE;
        final Hashtable<Long, String> idFormulaires = new Hashtable<Long, String>();
        final boolean duplication = (infoBean.get("MODE_FORMULAIRE") != null && (infoBean.getString("MODE_FORMULAIRE").equals("DUPLICATION")));
        final boolean modification = infoBean.get("MODE_FORMULAIRE") != null && infoBean.getString("MODE_FORMULAIRE").equals("MODIFICATION");
        final boolean suppression = (infoBean.get("MODE_FORMULAIRE") != null && (infoBean.getString("MODE_FORMULAIRE").equals("SUPPRESSION")));
        final boolean extraction = (infoBean.get("MODE_FORMULAIRE") != null && (infoBean.getString("MODE_FORMULAIRE").equals("EXTRACTION")));
        final boolean purge = infoBean.get("MODE_FORMULAIRE") != null && infoBean.getString("MODE_FORMULAIRE").equals("PURGE");
        final boolean insertion = infoBean.get("MODE_FORMULAIRE") != null && infoBean.getString("MODE_FORMULAIRE").equals("INSERTION");
        /*
        Dans le cas d'une toolbox, on affiche d'abord les fiches
        de la langue de l'article
         */
        formulaire.init();
        if (formulaire.select("ORDER BY TITRE") > 0) {
            while (formulaire.nextItem()) {
                if (idFormulaires.get(formulaire.getIdFormulaire()) == null) {
                    boolean inserer = true;
                    if (modification || duplication || suppression || purge) {
                        if (!autorisations.possedePermission(new PermissionBean("TECH", "for", "M"), new Perimetre(formulaire.getCodeRattachement(), formulaire.getCodeRubrique(), "*", "*", ""))) {
                            inserer = false;
                        }
                    }
                    if (insertion) {
                        if (!autorisations.possedePermission(new PermissionBean("TECH", "for", "I"), new Perimetre(formulaire.getCodeRattachement(), formulaire.getCodeRubrique(), "*", "*", ""))) {
                            inserer = false;
                        }
                    }
                    if (extraction) {
                        if (!autorisations.possedePermission(new PermissionBean("TECH", "for", "C"), new Perimetre(formulaire.getCodeRattachement(), formulaire.getCodeRubrique(), "*", "*", ""))) {
                            inserer = false;
                        }
                    }
                    if (inserer) {
                        idFormulaires.put(formulaire.getIdFormulaire(), "");
                        infoBean.set("ID_FORMULAIRE" + i, formulaire.getIdFormulaire());
                        infoBean.set("CODE" + i, formulaire.getCode());
                        infoBean.set("LIBELLE" + i, formulaire.getTitre());
                        i++;
                    }
                }
            }
        }
        infoBean.set("LISTE_NB_ITEMS", i);
    }

    /**
     * Preparer l'écran d'accueil.
     *
     */
    private void preparerACCUEIL() {
        final List<FormulaireDto> forms = serviceFormulaire.getAllWithRecordsCount();
        final Pattern patternIdentityField = Pattern.compile("^(.*)_TYPE=IDENTITY$", Pattern.MULTILINE);
        final List<FormulaireDto> visibleForms = new ArrayList<>();
        for (FormulaireDto form : forms) {
            preparerFormulaireDto(form);
            if (form.hasDroitGestionModele() || form.hasDroitGestionContenu()) {
                visibleForms.add(form);
                extraireDonneesPersonnelle(form, patternIdentityField);
            }
        }
        infoBean.set("LISTE_FORMS", visibleForms);
    }

    private void preparerFormulaireDto(FormulaireDto form) {
        RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(form.getCodeRubrique());
        form.setLibelleRubrique(rubriqueBean != null ? rubriqueBean.getIntitule() : MessageHelper.getCoreMessage("RUBRIQUE_INEXISTANTE"));
        form.setLibelleRattachement(serviceStructure.getDisplayableLabel(form.getCodeRattachement(), ContexteUtil.getContexteUniv().getLangue()));
        Perimetre perimetre = new Perimetre(form.getCodeRattachement(), form.getCodeRubrique(), "*", "*", "");
        form.setDroitGestionModele(autorisations.isWebMaster() || autorisations.possedePermission(new PermissionBean("TECH", "for", "M"), perimetre));
        form.setDroitGestionContenu(autorisations.isWebMaster() || autorisations.possedePermission(new PermissionBean("TECH", "for", "C"), perimetre));
    }

    private void extraireDonneesPersonnelle(FormulaireDto form, Pattern pattern) {
        try {
            final String modele = form.getModele();
            final Matcher fieldMatcher = pattern.matcher(modele);
            final List<String> identityFields = new ArrayList<>();
            while (fieldMatcher.find()) {
                identityFields.add(fieldMatcher.group(1));
            }
            final Properties proprietes = new Properties();
            proprietes.load(new ByteArrayInputStream(modele.getBytes()));
            final List<String> donneesPersonnelles = new ArrayList<>();
            identityFields.forEach(field -> donneesPersonnelles.add(proprietes.getProperty(field + "_LIBELLE")));
            form.setDonneesPersonnelles(donneesPersonnelles);
        } catch (IOException ioe) {
            LOG.debug("Impossible d'analyser le modèle :", ioe);
            LOG.error("Impossible d'analyser le modèle :" + ioe.getMessage());
        }
    }

    /**
     * Preparer l'écran de recherche des données personnelles dans les formulaire.
     *
     */
    private void preparerRECHERCHEdonneespersonnelles() {
        infoBean.set("LISTE_CHAMPS_IDENTITE", lstIdentiteFormulaire);
        ecranLogique = ECRAN_RECHERCHE_DONNEES_PERSONNELLES;
    }

    /**
     * Preparer detail.
     *
     * @param mode
     *            the mode
     *
     * @throws Exception
     *             the exception
     */
    private void preparerDETAIL(final String mode) throws Exception {
        if (mode.equals("CREATION")) {
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
            infoBean.set("CODE", String.valueOf(System.currentTimeMillis()));
            infoBean.set("TITRE", "");
            infoBean.set("CODE_RATTACHEMENT", "");
            infoBean.set("LIBELLE_CODE_RATTACHEMENT", "");
            infoBean.set("CODE_RUBRIQUE", "");
            infoBean.set("LIBELLE_CODE_RUBRIQUE", "");
            infoBean.set("LISTE_NB_CHAMPS", 0);
            infoBean.set("DUREE_CONSERVATION", FormulaireUtil.getPurgeDelay());
        } else if (mode.equals("MODIFICATION")) {
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_MODIF);
            formulaire.init();
            final ClauseWhere whereCode = whereCodeEgal(infoBean.getString("CODE"));
            formulaire.select(whereCode.formaterSQL());
            formulaire.nextItem();
            renseignerInfobean(infoBean, formulaire);
        } else if (mode.equals("DUPLICATION")) {
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
            formulaire.init();
            final ClauseWhere whereCode = whereCodeEgal(infoBean.getString("CODE"));
            formulaire.select(whereCode.formaterSQL());
            formulaire.nextItem();
            renseignerInfobean(infoBean, formulaire);
            infoBean.set("CODE", String.valueOf(System.currentTimeMillis()));
            infoBean.addMessageConfirmation(FormulaireUtil.getMessage("BO.FORMULAIRE.DUPLICATION_CONFIRMATION"));
        }
        infoBean.set("NUMERO_CHAMP", 0);
        infoBean.set("GRS_FILTRE_ARBRE_NOM_CODE_RUBRIQUE", "CODE_RUBRIQUE");
        infoBean.set("GRS_PERMISSION_TYPE", "TECH");
        infoBean.set("GRS_PERMISSION_OBJET", "for");
        infoBean.set("GRS_PERMISSION_ACTION", "M");
        ecranLogique = ECRAN_DETAIL;
        if (StringUtils.isNotBlank(formulaire.getTitre())) {
            infoBean.setTitreEcran(formulaire.getTitre());
        }
    }

    /**
     * Preparer detail champ.
     *
     * @throws ErreurApplicative
     *             the exception
     */
    private void preparerDetailChamp() throws ErreurApplicative {
        final Integer i = (Integer) infoBean.get("LISTE_NB_CHAMPS");
        final Integer j = new Integer(infoBean.getString("NUMERO_CHAMP"));
        if (i.equals(j))//création
        {
            if (infoBean.getString("TYPE_CHAMP").equals("0000")) {
                throw new ErreurApplicative("Vous devez spécifier le type de champ !");
            }
            infoBean.set("CHAMP" + i + "_TYPE", infoBean.get("TYPE_CHAMP"));
            infoBean.set("CHAMP" + i + "_CODE", SaisieUtils.determinerCode());
            infoBean.set("CHAMP" + i + "_LIBELLE", "");
            infoBean.set("CHAMP" + i + "_TYPE_ACTION", "");
            infoBean.set("CHAMP" + i + "_MAIL", PropertyHelper.getProperty(infoBean.getNomExtension(), "mail.webmaster"));
            infoBean.set("CHAMP" + i + "_DESTINATION", "");
            infoBean.set("LIBELLE_CHAMP" + i + "_DESTINATION", "");
            infoBean.set("CHAMP" + i + "_NB_ITEM", 0);
            infoBean.set("CHAMP" + i + "_OBLIGATOIRE", "");
            infoBean.set("CHAMP" + i + "_NBCARMAX", 32);
            if (infoBean.getString("TYPE_CHAMP").equals("TEXTAREA")) {
                infoBean.set("CHAMP" + i + "_NBCARMAX", 8000);
            }
            infoBean.set("CHAMP" + i + "_INSTANCE", "");
            infoBean.set("CHAMP" + i + "_TYPE_LISTE", "");
        }
        ecranLogique = ECRAN_DETAIL_CHAMP;
    }

    /**
     * Traiter detail champ.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterDetailChamp() throws Exception {
        if (action.equals("VALIDER")) {
            final Integer i = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            final Integer j = new Integer(infoBean.getString("NUMERO_CHAMP"));
            if (i.equals(j)) //création
            {
                final String codeChamp = infoBean.getString("CHAMP" + i + "_CODE");
                for (int numeroChamp = 0; numeroChamp < i; numeroChamp++) {
                    if (infoBean.getString("CHAMP" + numeroChamp + "_CODE").equals(codeChamp)) {
                        throw new ErreurApplicative("Le code " + codeChamp + " existe déjà, veuillez en spécifier un nouveau.");
                    }
                }
                infoBean.set("LISTE_NB_CHAMPS", i + 1);
            }
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(infoBean.getString("CODE_RUBRIQUE"));
            if (rubriqueBean != null) {
                infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueBean.getIntitule());
            }
            final String champLibelle = "CHAMP" + j + "_LIBELLE";
            if (StringUtils.contains(infoBean.get(champLibelle, String.class), ";")) {
                throw new ErreurApplicative(FormulaireUtil.getMessage("FORMULAIRE_SAISIE_CHAMP_LIBELLE_ERREUR_POINT_VIRGULE"));
            }
            infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(infoBean.getString("CODE_RATTACHEMENT"), "0"));
            ecranLogique = ECRAN_DETAIL;
        } else if (action.equals("AFFICHER_ITEM")) {
            preparerDetailItem();
        } else if (action.equals("SUPPRIMER_ITEM")) {
            final int numChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
            final int numItem = new Integer(infoBean.getString("NUMERO_ITEM"));
            final int nbItem = (Integer) infoBean.get("CHAMP" + numChamp + "_NB_ITEM");
            infoBean.set("CHAMP" + numChamp + "_NB_ITEM", nbItem - 1);
            for (int i = numItem + 1; i < nbItem; i++) {
                deplacerItem(numChamp, i, i - 1);
            }
            ecranLogique = ECRAN_DETAIL_CHAMP;
        } else if (action.equals("DESCENDRE_ITEM")) {
            //on met temporairement dans le dernier champ (non existant)
            final int numChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
            final int numItem = new Integer(infoBean.getString("NUMERO_ITEM"));
            final int nbItem = (Integer) infoBean.get("CHAMP" + numChamp + "_NB_ITEM");
            if (numItem + 1 != nbItem) {
                deplacerItem(numChamp, numItem + 1, nbItem);
                deplacerItem(numChamp, numItem, numItem + 1);
                deplacerItem(numChamp, nbItem, numItem);
            }
            ecranLogique = ECRAN_DETAIL_CHAMP;
        } else if (action.equals("MONTER_ITEM")) {
            final int numChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
            final int numItem = new Integer(infoBean.getString("NUMERO_ITEM"));
            final int nbItem = (Integer) infoBean.get("CHAMP" + numChamp + "_NB_ITEM");
            if (numItem != 0) {
                deplacerItem(numChamp, numItem - 1, nbItem);
                deplacerItem(numChamp, numItem, numItem - 1);
                deplacerItem(numChamp, nbItem, numItem);
            }
            ecranLogique = ECRAN_DETAIL_CHAMP;
        }
    }

    /**
     * Traiter detail.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterDetail() throws Exception {
        if (action.equals("VALIDER")) {
            if (!autorisations.isWebMaster() && !autorisations.possedePermission(new PermissionBean("TECH", "for", "M"), new Perimetre(infoBean.getString("CODE_RATTACHEMENT"), infoBean.getString("CODE_RUBRIQUE"), "*", "*", ""))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_ENREGISTREMENT_HORS_PERIMETRE"));
            }
            if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
                formulaire.init();
                renseignerFormulaire();
                formulaire.add();
            } else {
                formulaire.init();
                final ClauseWhere whereCode = whereCodeEgal(infoBean.getString("CODE"));
                formulaire.select(whereCode.formaterSQL());
                formulaire.nextItem();
                renseignerFormulaire();
                formulaire.update();
            }
            final String confirmation = String.format(FormulaireUtil.getMessage("CONFIRMATION_VALIDATION_FORMULAIRE"), formulaire.getTitre());
            infoBean.removeMessage(TypeMessage.CONFIRMATION.getCode());
            infoBean.addMessageConfirmation(confirmation);
            etat = FIN;
        } else if (action.equals("MODIFIER_CHAMP")) {
            preparerDetailChamp();
        } else if (action.equals("AFFICHER_CHAMP")) {
            preparerDetailChamp();
        } else if (action.equals("SUPPRIMER_CHAMP")) {
            final int numeroChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            infoBean.set("LISTE_NB_CHAMPS", nbChamp - 1);
            for (int i = numeroChamp + 1; i < nbChamp; i++) {
                deplacerChamp(i, i - 1);
            }
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(infoBean.getString("CODE_RUBRIQUE"));
            if (rubriqueBean != null) {
                infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueBean.getIntitule());
            }
            infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(infoBean.getString("CODE_RATTACHEMENT"), "0"));
            ecranLogique = ECRAN_DETAIL;
        } else if (action.equals("DESCENDRE_CHAMP")) {
            //on met temporairement dans le dernier champ (non existant)
            final int numeroChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            if (numeroChamp + 1 != nbChamp) {
                deplacerChamp(numeroChamp + 1, nbChamp);
                deplacerChamp(numeroChamp, numeroChamp + 1);
                deplacerChamp(nbChamp, numeroChamp);
            }
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(infoBean.getString("CODE_RUBRIQUE"));
            if (rubriqueBean != null) {
                infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueBean.getIntitule());
            }
            infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(infoBean.getString("CODE_RATTACHEMENT"), "0"));
            ecranLogique = ECRAN_DETAIL;
        } else if (action.equals("MONTER_CHAMP")) {
            //on met temporairement dans le dernier champ (non existant)
            final int numeroChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
            final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
            if (numeroChamp != 0) {
                deplacerChamp(numeroChamp - 1, nbChamp);
                deplacerChamp(numeroChamp, numeroChamp - 1);
                deplacerChamp(nbChamp, numeroChamp);
            }
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(infoBean.getString("CODE_RUBRIQUE"));
            if (rubriqueBean != null) {
                infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueBean.getIntitule());
            }
            infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(infoBean.getString("CODE_RATTACHEMENT"), "0"));
            ecranLogique = ECRAN_DETAIL;
        }
    }

    /**
     * Deplacer champ.
     *
     * @param indexDebut
     *            the index debut.
     * @param indexFin
     *            the index fin.
     *
     */
    private void deplacerChamp(final int indexDebut, final int indexFin) {
        infoBean.set("CHAMP" + indexFin + "_CODE", infoBean.get("CHAMP" + indexDebut + "_CODE"));
        infoBean.set("CHAMP" + indexFin + "_TYPE", infoBean.get("CHAMP" + indexDebut + "_TYPE"));
        infoBean.set("CHAMP" + indexFin + "_LIBELLE", infoBean.get("CHAMP" + indexDebut + "_LIBELLE"));
        infoBean.set("CHAMP" + indexFin + "_TYPE_ACTION", infoBean.get("CHAMP" + indexDebut + "_TYPE_ACTION"));
        infoBean.set("CHAMP" + indexFin + "_MAIL", infoBean.get("CHAMP" + indexDebut + "_MAIL"));
        infoBean.set("CHAMP" + indexFin + "_DESTINATION", infoBean.get("CHAMP" + indexDebut + "_DESTINATION"));
        infoBean.set("LIBELLE_CHAMP" + indexFin + "_DESTINATION", infoBean.get("LIBELLE_CHAMP" + indexDebut + "_DESTINATION"));
        infoBean.set("CHAMP" + indexFin + "_INSTANCE", infoBean.get("CHAMP" + indexDebut + "_INSTANCE"));
        infoBean.set("CHAMP" + indexFin + "_OBLIGATOIRE", infoBean.get("CHAMP" + indexDebut + "_OBLIGATOIRE"));
        infoBean.set("CHAMP" + indexFin + "_NBCARMAX", infoBean.get("CHAMP" + indexDebut + "_NBCARMAX"));
        infoBean.set("CHAMP" + indexFin + "_TYPE_LISTE", infoBean.get("CHAMP" + indexDebut + "_TYPE_LISTE"));
        infoBean.set("CHAMP" + indexFin + "_NB_ITEM", infoBean.get("CHAMP" + indexDebut + "_NB_ITEM"));
        final int nbItem = (Integer) infoBean.get("CHAMP" + indexDebut + "_NB_ITEM");
        for (int j = 0; j < nbItem; j++) {
            infoBean.set("CHAMP" + indexFin + "_ITEM" + j + "_CODE", infoBean.get("CHAMP" + indexDebut + "_ITEM" + j + "_CODE"));
            infoBean.set("CHAMP" + indexFin + "_ITEM" + j + "_LIBELLE", infoBean.get("CHAMP" + indexDebut + "_ITEM" + j + "_LIBELLE"));
            infoBean.set("CHAMP" + indexFin + "_ITEM" + j + "_SELECTION", infoBean.get("CHAMP" + indexDebut + "_ITEM" + j + "_SELECTION"));
        }
    }

    /**
     * Deplacer item.
     *
     * @param indexChamp
     *            the index champ
     * @param indexDebut
     *            the index debut
     * @param indexFin
     *            the index fin
     *
     */
    private void deplacerItem(final int indexChamp, final int indexDebut, final int indexFin) {
        infoBean.set("CHAMP" + indexChamp + "_ITEM" + indexFin + "_CODE", infoBean.get("CHAMP" + indexChamp + "_ITEM" + indexDebut + "_CODE"));
        infoBean.set("CHAMP" + indexChamp + "_ITEM" + indexFin + "_LIBELLE", infoBean.get("CHAMP" + indexChamp + "_ITEM" + indexDebut + "_LIBELLE"));
        infoBean.set("CHAMP" + indexChamp + "_ITEM" + indexFin + "_SELECTION", infoBean.get("CHAMP" + indexChamp + "_ITEM" + indexDebut + "_SELECTION"));
    }

    /**
     * Preparer detail item.
     *
     */
    private void preparerDetailItem() {
        ecranLogique = ECRAN_DETAIL_ITEM;
        final Integer numChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
        final Integer numItem = new Integer(infoBean.getString("NUMERO_ITEM"));
        final Integer nbItem = (Integer) infoBean.get("CHAMP" + numChamp + "_NB_ITEM");
        if (numItem.equals(nbItem)) {
            infoBean.set("CHAMP" + numChamp + "_ITEM" + numItem + "_CODE", SaisieUtils.determinerCode());
            infoBean.set("CHAMP" + numChamp + "_ITEM" + numItem + "_LIBELLE", "");
            infoBean.set("CHAMP" + numChamp + "_ITEM" + numItem + "_SELECTION", "");
        }
    }

    /**
     * Traiter detail item.
     *
     */
    private void traiterDetailItem() {
        final Integer numChamp = new Integer(infoBean.getString("NUMERO_CHAMP"));
        final Integer numItem = new Integer(infoBean.getString("NUMERO_ITEM"));
        final Integer nbItem = (Integer) infoBean.get("CHAMP" + numChamp + "_NB_ITEM");
        if (numItem.equals(nbItem))//Création
        {
            infoBean.set("CHAMP" + numChamp + "_NB_ITEM", nbItem + 1);
        }
        ecranLogique = ECRAN_DETAIL_CHAMP;
    }

    /**
     * Traiter saisie formulaire.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterSaisieFormulaire() throws Exception {
        //anti spam sur la saisie anonyme
        if (autorisations == null) {
            final String honeyPot = infoBean.getString("VALIDATION_ANONYME");
            if (StringUtils.isNotEmpty(honeyPot)) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage(this.getLocale(), "JTF_ERR_VALIDATION_ANONYME"));
            }

            final String captchaId = infoBean.getSessionHttp().getId();
            final String captchaResponse = infoBean.getString(FormulaireUtil.getProperty("VALIDATION_CAPTCHA"));

            try {
                CaptchaUtils.validate(captchaId, captchaResponse);
            } catch(EmptyCaptchaException empty) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage(this.getLocale(), "JTF_ERR_VALIDATION_ANONYME"), empty);
            } catch (InvalidCaptchaException invalide) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage(this.getLocale(), "ST_FORMULAIRE_SAISIE_INVALIDE"), invalide);
            } catch (ExpiredCaptchaException expired) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage(this.getLocale(), "JTF_ERR_CAPTCHA_EXPIRE"), expired);
            }
        }
        final String codeFormulaire = infoBean.getString("CODE");
        formulaire.init();
        final ClauseWhere whereCode = whereCodeEgal(codeFormulaire);
        formulaire.select(whereCode.formaterSQL());
        formulaire.nextItem();
        renseignerInfobean(infoBean, formulaire);
        // controle de la session de l'utilisateur en cas de saisie en mode connecté
        if (!formulaire.getEnregistrementUnique().equals("0")) {
            if (autorisations == null) {
                throw new ErreurApplicative(MessageHelper.getMessage(infoBean.getNomExtension(), this.getLocale(), "ST_FORMULAIRE_SAISIE_INVALIDE2"));
            }
        }
        final String numeroAction = infoBean.getString("NUMERO_ACTION");
        final String typeAction = infoBean.getString("CHAMP" + numeroAction + "_TYPE_ACTION");
        final String destination = infoBean.getString("CHAMP" + numeroAction + "_DESTINATION");
        final String mail = infoBean.getString("CHAMP" + numeroAction + "_MAIL");
        final InfoImplementation implementation = lstActionFormulaire.get(typeAction);
        // traitement de la saisie
        final SaisieFormulaire saisieFormulaire = (SaisieFormulaire) (Class.forName(implementation.getNomClasse()).newInstance());
        UtilisateurBean utilisateur = null;
        if (autorisations != null) {
            utilisateur = serviceUser.getByCode(autorisations.getCode());
        }
        saisieFormulaire.traiterSaisieFormulaire(formulaire, infoBean, this, utilisateur, typeAction, mail);
        // redirection
        if ((destination != null) && (destination.length() > 0)) {
            String codeFiche;
            String langueFiche;
            if (!destination.contains(",LANGUE=")) {
                codeFiche = destination;
                langueFiche = "0";
            } else {
                codeFiche = destination.substring(0, destination.indexOf(",LANGUE="));
                langueFiche = destination.substring(destination.indexOf(",LANGUE=") + 8);
            }
            final MetatagBean metatagBean = serviceMetatag.getByMetaCodeObjetCodeLangueEtat(ReferentielObjets.getCodeObjet("pagelibre"), codeFiche, langueFiche, EtatFiche.EN_LIGNE.getEtat());
            final String urlDemandee = urlProvider.provideAbsoluteUrl(metatagBean);
            final Long timeUrlDemandee = System.currentTimeMillis();
            infoBean.getSessionHttp().setAttribute("URL_DEMANDEE", urlDemandee);
            infoBean.getSessionHttp().setAttribute("TIME_URL_DEMANDEE", timeUrlDemandee);
            infoBean.setEcranRedirection(urlDemandee);
            ecranLogique = ECRAN_REDIRECTION;
        } else {
            ecranLogique = ECRAN_CONFIRMATION;
        }
    }

    /**
     * Point d'entrée du processus.
     *
     * @return true, if traiter action
     *
     */
    @Override
    public boolean traiterAction() {
        try {
            ecranLogique = infoBean.getEcranLogique();
            action = infoBean.getActionUtilisateur();
            autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
            //pas besoin d'être authentifie pour saisir
            if (action.equals("SAISIE_ENREGISTRER")) {
                traiterSaisieFormulaire();
            } else if (autorisations == null) {
                infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
                ecranLogique = "LOGIN";
            } else if (!autorisations.possedePermission(PERMISSION_ECRAN)) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE"));
            } else {
                if (action.equals("ACCUEIL")) {
                    preparerACCUEIL();
                    ecranLogique = ECRAN_ACCUEIL;
                } else if (StringUtils.equals(action, "RECHERCHER")) {
                    traiterRecherche();
                } else if (action.equals("AFFICHER_FORMULAIRE")) {
                    ecranLogique = ECRAN_DETAIL;
                } else if (action.equals("AFFICHER_CHAMP")) {
                    //ecranLogique = ECRAN_DETAIL_CHAMP;
                    preparerDetailChamp();
                } else if (ecranLogique == null) {
                    if (action.equals("AJOUTER")) {
                        preparerDETAIL("CREATION");
                    } else if (action.equals("MODIFIER")) {
                        preparerDETAIL("MODIFICATION");
                    } else if (action.equals("DUPLIQUER")) {
                        preparerDETAIL("DUPLICATION");
                    } else if (action.equals("EXTRAIRE")) {
                        preparerExtraction();
                    } else if (action.equals("SUPPRIMER")) {
                        FormulaireBean formBean = serviceFormulaire.getByCode(infoBean.getString("CODE"));
                        final String titreFormulaire = formBean.getTitre();
                        serviceFormulaire.delete(formBean);
                        //Supprime les fichiers si le formulaire contient des pièces jointes
                        final String pathFichier = FormulaireUtil.getFormsDefaultPath().concat(File.separator).concat(infoBean.getString("CODE"));
                        FileUtils.deleteDirectory(new File(pathFichier));
                        final String confirmation = String.format(FormulaireUtil.getMessage("CONFIRMATION_SUPPRESSION_FORMULAIRE"), titreFormulaire);
                        infoBean.addMessageConfirmation(confirmation);
                        etat = FIN;
                    } else if (action.equals("VIDER")) {
                        formulaire.init();
                        final ClauseWhere whereCode = whereCodeEgal(infoBean.getString("CODE"));
                        formulaire.select(whereCode.formaterSQL());
                        formulaire.nextItem();
                        final Contenuformulaire contenu = new Contenuformulaire();
                        contenu.init();
                        final ClauseWhere whereCodeFormulaire = new ClauseWhere(ConditionHelper.egalVarchar("CODE_FORMULAIRE", infoBean.getString("CODE")));
                        contenu.select(whereCodeFormulaire.formaterSQL());
                        while (contenu.nextItem()) {
                            serviceContenuFormulaire.delete(contenu.getIdContenuformulaire());
                        }
                        //Supprime les fichiers si le formulaire contient des pièces jointes
                        final String pathFichier = FormulaireUtil.getFormsDefaultPath().concat(File.separator).concat(infoBean.getString("CODE"));
                        FileUtils.deleteDirectory(new File(pathFichier));
                        final String confirmation = String.format(FormulaireUtil.getMessage("CONFIRMATION_PURGE_FORMULAIRE"), formulaire.getTitre());
                        infoBean.addMessageConfirmation(confirmation);
                        etat = FIN;
                    } else if (action.equals("AFFICHER_LISTE")) {
                        preparerLISTE();
                    } else if (StringUtils.equals(action, "DONNEES_PERSONNELLES")) {
                        preparerRECHERCHEdonneespersonnelles();
                    } else if (StringUtils.equals(action, "VIDER_INSTANCE")) {
                        supprimerDonneePersonnelle();
                    } else if (StringUtils.equals(action, ACTION_EXTRAIRE_DONNEES_PERSONNELLES)) {
                        traiterExtractionDonneesPersonnelles();
                    }
                } else {
                    if (ecranLogique.equals(ECRAN_DETAIL)) {
                        traiterDetail();
                    } else if (ecranLogique.equals(ECRAN_DETAIL_CHAMP)) {
                        traiterDetailChamp();
                    } else if (ecranLogique.equals(ECRAN_DETAIL_ITEM)) {
                        traiterDetailItem();
                    } else if (ecranLogique.equals(ECRAN_DETAIL_EXTRACTION)) {
                        traiterExtraction();
                    }
                }
            }
            //placer l'état dans le composant d'infoBean
            infoBean.setEcranLogique(ecranLogique);
        } catch (final Exception e) {
            LOG.error("Erreur lors du traitement des formulaires", e);
            infoBean.addMessageErreur(e.toString());
        }
        // On continue si on n'est pas à la FIN !!!
        return etat == FIN;
    }

    private void traiterRecherche() throws ErreurApplicative {
        final Map<String, String> params = new HashMap<>();
        final List<CritereRecherche> criteres = new ArrayList<>();
        for (final String fieldCode : lstIdentiteFormulaire.keySet()) {
            final String value = infoBean.getString(fieldCode);
            if (StringUtils.isNotBlank(value)) {
                final String encodedValue = StringEscapeUtils.escapeJava(toUnicode(value));
                params.put(fieldCode, encodedValue);
                criteres.add(new CritereRecherche(fieldCode, encodedValue, value));
            }
        }
        if (params.size() == 0) {
            throw new ErreurApplicative(FormulaireUtil.getMessage("BO_ERREUR_PARAM_OBLIGATOIRE"));
        }
        final List<ContenuFormulaireDto> list = serviceFormulaire.getAllWithContent(params);
        final List<ContenuFormulaireDto> filteredList = new ArrayList<>();
        for (final ContenuFormulaireDto userInput : list) {
            final Properties userData = extractContent(userInput.getInstance());
            //Extrait la liste des données personnelles dans l'instance saisie sous la forme "libelle : valeur"
            userInput.setDonneesPersonnelles(lstIdentiteFormulaire.entrySet().stream().filter(k -> userData.getProperty(k.getKey()) != null).map(k -> k.getValue().getLibelle() + " : " + userData.getProperty(k.getKey())).collect(Collectors.toList()));
            preparerContenuFormulaireDto(userInput);
            filteredList.add(userInput);
        }
        infoBean.set("LISTE_CHAMPS_IDENTITE", lstIdentiteFormulaire);
        infoBean.set("LISTE_CRITERES", criteres);
        infoBean.set("LISTE_FORMS", filteredList);
        ecranLogique = ECRAN_RESULTAT_RECHERCHE;
    }

    private void preparerContenuFormulaireDto(ContenuFormulaireDto form) {
        Perimetre perimetre = new Perimetre(form.getCodeRattachement(), form.getCodeRubrique(), "*", "*", "");
        form.setDroitGestionContenu(autorisations.isWebMaster() || autorisations.possedePermission(new PermissionBean("TECH", "for", "C"), perimetre));
    }

    private Properties extractContent(final String rawData) throws ErreurApplicative {
        try {
            final Properties proprietes = new Properties();
            proprietes.load(new ByteArrayInputStream(rawData.getBytes()));
            return proprietes;
        } catch (IOException ioe) {
            LOG.error("Erreur lors de la lecture du contenu du formulaire", ioe);
            throw new ErreurApplicative(FormulaireUtil.getMessage("BO_ERREUR_LECTURE_CONTENU"));
        }
    }

    /**
     * Renseigner formulaire.
     *
     * @throws Exception
     *             the exception
     */
    private void renseignerFormulaire() throws Exception {
        final Properties proprietes = new Properties();
        proprietes.put("LISTE_NB_CHAMPS", ((Integer) infoBean.get("LISTE_NB_CHAMPS")).toString());
        final int nbChamp = (Integer) infoBean.get("LISTE_NB_CHAMPS");
        for (int numeroChamp = 0; numeroChamp < nbChamp; numeroChamp++) {
            proprietes.put("CHAMP" + numeroChamp + "_CODE", infoBean.getString("CHAMP" + numeroChamp + "_CODE"));
            proprietes.put("CHAMP" + numeroChamp + "_TYPE", infoBean.getString("CHAMP" + numeroChamp + "_TYPE"));
            proprietes.put("CHAMP" + numeroChamp + "_LIBELLE", infoBean.getString("CHAMP" + numeroChamp + "_LIBELLE"));
            proprietes.put("CHAMP" + numeroChamp + "_TYPE_ACTION", infoBean.getString("CHAMP" + numeroChamp + "_TYPE_ACTION"));
            proprietes.put("CHAMP" + numeroChamp + "_MAIL", infoBean.getString("CHAMP" + numeroChamp + "_MAIL"));
            proprietes.put("CHAMP" + numeroChamp + "_DESTINATION", infoBean.getString("CHAMP" + numeroChamp + "_DESTINATION"));
            proprietes.put("LIBELLE_CHAMP" + numeroChamp + "_DESTINATION", infoBean.getString("LIBELLE_CHAMP" + numeroChamp + "_DESTINATION"));
            proprietes.put("CHAMP" + numeroChamp + "_INSTANCE", infoBean.getString("CHAMP" + numeroChamp + "_INSTANCE"));
            proprietes.put("CHAMP" + numeroChamp + "_OBLIGATOIRE", infoBean.getString("CHAMP" + numeroChamp + "_OBLIGATOIRE"));
            proprietes.put("CHAMP" + numeroChamp + "_NBCARMAX", checkNull(infoBean.get("CHAMP" + numeroChamp + "_NBCARMAX")));
            proprietes.put("CHAMP" + numeroChamp + "_TYPE_LISTE", infoBean.getString("CHAMP" + numeroChamp + "_TYPE_LISTE"));
            proprietes.put("CHAMP" + numeroChamp + "_NB_ITEM", ((Integer) infoBean.get("CHAMP" + numeroChamp + "_NB_ITEM")).toString());
            final int nbItem = (Integer) infoBean.get("CHAMP" + numeroChamp + "_NB_ITEM");
            for (int j = 0; j < nbItem; j++) {
                proprietes.put("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE", infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_CODE"));
                proprietes.put("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE", infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_LIBELLE"));
                proprietes.put("CHAMP" + numeroChamp + "_ITEM" + j + "_SELECTION", infoBean.getString("CHAMP" + numeroChamp + "_ITEM" + j + "_SELECTION"));
            }
        }
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        proprietes.store(baos, null);
        final String modele = baos.toString();
        formulaire.setCode(infoBean.getString("CODE"));
        formulaire.setTitre(infoBean.getString("TITRE"));
        formulaire.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        formulaire.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        formulaire.setEnregistrementUnique(infoBean.getString("ENREGISTREMENT_UNIQUE"));
        formulaire.setMentionsLegales(infoBean.getString("MENTIONS_LEGALES"));
        formulaire.setModele(modele);
        formulaire.setDureeConservation(infoBean.getInt("DUREE_CONSERVATION"));
    }

    /**
     * Preparer extraction.
     *
     */
    private void preparerExtraction() {
        formulaire.init();
        final ClauseWhere whereCode = whereCodeEgal(infoBean.getString("CODE"));
        formulaire.select(whereCode.formaterSQL());
        formulaire.nextItem();
        infoBean.set("TITRE", formulaire.getTitre());
        infoBean.set("LISTE_TYPE_EXTRACTION", lstTypeExtraction);
        ecranLogique = ECRAN_DETAIL_EXTRACTION;
        if (StringUtils.isNotBlank(formulaire.getTitre())) {
            infoBean.setTitreEcran(formulaire.getTitre());
        }
    }

    private void supprimerDonneePersonnelle() throws ErreurApplicative {
        final Long instanceId = Long.valueOf(infoBean.getString("ID_INSTANCE"));
        ContenuformulaireBean contenu = serviceContenuFormulaire.getById(instanceId);
        FormulaireBean formulaireBean = serviceFormulaire.getByContenuId(contenu.getId());
        Perimetre perimetre = new Perimetre(formulaireBean.getCodeRattachement(), formulaireBean.getCodeRubrique(), "*", "*", "");
        if (!autorisations.isWebMaster() && !autorisations.possedePermission(new PermissionBean("TECH", "for", "C"), perimetre)) {
            throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_PAS_DROIT_EXTRACTION_FORMULAIRE"));
        }
        serviceContenuFormulaire.delete(instanceId);
        final String confirmation = String.format(FormulaireUtil.getMessage("CONFIRMATION_SUPPRESSION_CONTENU"), formulaireBean.getTitre());
        infoBean.addMessageConfirmation(confirmation);
        traiterRecherche();
        ecranLogique = ECRAN_RESULTAT_RECHERCHE;
    }

    /**
     * Traiter extraction.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterExtraction() throws Exception {
        formulaire.init();
        final ClauseWhere whereCode = whereCodeEgal(infoBean.getString("CODE"));
        formulaire.select(whereCode.formaterSQL());
        formulaire.nextItem();
        if (!autorisations.isWebMaster() && !autorisations.possedePermission(new PermissionBean("TECH", "for", "C"), new Perimetre(formulaire.getCodeRattachement(), formulaire.getCodeRubrique(), "*", "*", ""))) {
            throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_PAS_DROIT_EXTRACTION_FORMULAIRE"));
        }
        renseignerInfobean(infoBean, formulaire);
        final String typeExtraction = infoBean.getString("TYPE_EXTRACTION");
        final InfoImplementation implementation = lstExtractionFormulaire.get(typeExtraction);
        if (implementation == null) {
            throw new ErreurApplicative("Veuillez choisir un type d'extraction");
        }
        final ExtractionFormulaire extractionFormulaire = (ExtractionFormulaire) (Class.forName(implementation.getNomClasse()).newInstance());
        extractionFormulaire.setTypeExtraction(typeExtraction);
        final String path = extractionFormulaire.extraireFormulaire(formulaire, infoBean, this);
        infoBean.set("EXTRACTION_FILE", StringUtils.substringAfterLast(path, File.separator));
        infoBean.set("EXTRACTION_TYPE", extractionFormulaire.getContentType());
        infoBean.set("CODE", infoBean.getString("CODE"));
        final String urlCallback = PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?EXT=formulaire&PROC=TRAITEMENT_FORMULAIRE&ACTION=EXTRAIRE&CODE=" + infoBean.getString("CODE");
        infoBean.set("URL_EXTRACT_CALLBACK", urlCallback);
        ecranLogique = ECRAN_EXTRACTION;
    }

    private void traiterExtractionDonneesPersonnelles() throws ErreurApplicative {
        final Long instanceId = Long.valueOf(infoBean.getString("ID_INSTANCE"));
        final ContenuformulaireBean contenu = serviceContenuFormulaire.getById(instanceId);
        final FormulaireBean formulaireBean = serviceFormulaire.getByContenuId(contenu.getId());
        Perimetre perimetre = new Perimetre(formulaireBean.getCodeRattachement(), formulaireBean.getCodeRubrique(), "*", "*", "");
        if (!autorisations.isWebMaster() && !autorisations.possedePermission(new PermissionBean("TECH", "for", "C"), perimetre)) {
            throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_PAS_DROIT_EXTRACTION_FORMULAIRE"));
        }
        final Properties userData = extractContent(contenu.getInstance());
        //Extrait la liste des données personnelles dans l'instance saisie
        Map<String, String> mapDonnesPersonnelles = lstIdentiteFormulaire.keySet().stream().filter(item -> Objects.nonNull(userData.getProperty(item))).collect(Collectors.toMap(item -> item, item -> userData.getProperty(item), (oldVal, newVal) -> newVal));
        String path = ExtractionDonneesPersonnelles.extraire(formulaireBean.getTitre(), contenu.getDateCreation(), mapDonnesPersonnelles);
        infoBean.set("URL_EXTRACT_CALLBACK", String.format(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?ACTION=RECHERCHER&BEANPU=%s", infoBean.get("BEANPU", String.class)));
        infoBean.set("EXTRACTION_TYPE", "txt");
        infoBean.set("EXTRACTION_FILE", StringUtils.substringAfterLast(path, File.separator));
        ecranLogique = ECRAN_EXTRACTION;
    }
}
