/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.reference;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Map.Entry;

import org.apache.commons.collections4.MapUtils;

import com.kosmos.formulaire.bean.ContenuformulaireBean;
import com.kosmos.formulaire.bean.FormulaireBeanExport;
import com.kosmos.usinesite.exception.ErreurReferenceException;
import com.kosmos.usinesite.reference.BeanReference;
import com.kosmos.usinesite.reference.DelimitationPattern;
import com.kosmos.usinesite.reference.ServiceBeanReference;
import com.kosmos.usinesite.reference.dto.ServiceBeanReferenceDto;

public class ServiceFormulaireBeanReference extends ServiceBeanReference<FormulaireBeanExport> {

    @Override
    public Collection<BeanReference> check(final ServiceBeanReferenceDto<FormulaireBeanExport> serviceBeanReferenceDto) throws ErreurReferenceException {
        final Collection<BeanReference> referencesToUpdate = new ArrayList<>();
        FormulaireBeanExport formulaireBean = serviceBeanReferenceDto.getBeansOrig().values().iterator().next();
        final Collection<String> moduleToSearch = new ArrayList<>();
        moduleToSearch.add(formulaireBean.getIdModule());
        final BeanReference referencesByCode = new BeanReference();
        // mise à jour du code sur le module formulaire
        referencesByCode.addModulesBySearchString("\"code\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        referencesByCode.addModulesBySearchString("\"code_formulaire\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        // mise à jour du code des formulaires dans tous les contenus ex [formulaire;CODE=1120733860111&amp;STYLE=DISPOSITION1]
        referencesByCode.addModulesBySearchString("\\[formulaire;CODE=" + DelimitationPattern.AMPERSAND_DELIMITER + "&", Collections.emptyList());
        // mise à jour de l'id formulaire
        final BeanReference referencesById = new BeanReference();
        referencesById.addModulesBySearchString("\"id_formulaire\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        // mise à jour de l'id contenu formulaire
        final BeanReference referencesByIdContenu = new BeanReference();
        referencesByIdContenu.addModulesBySearchString("\"id_contenuformulaire\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        for (final Entry<String, FormulaireBeanExport> entry : serviceBeanReferenceDto.getBeansOrig().entrySet()) {
            formulaireBean = executeProcessors(entry.getValue(), serviceBeanReferenceDto);
            if (formulaireBean == null) {
                continue;
            }
            referencesByCode.addNewValueByOldValue(formulaireBean.getBean().getCode(), Long.toString(System.nanoTime()));
            referencesById.addNewValueByOldValue(formulaireBean.getBean().getId().toString(), "0");
            // Gestion des contenus de formulaire
            if (MapUtils.isNotEmpty(formulaireBean.getContenusFormulaire())) {
                for (final ContenuformulaireBean contenuBean : formulaireBean.getContenusFormulaire().values()) {
                    referencesByIdContenu.addNewValueByOldValue(contenuBean.getId().toString(), "0");
                }
            }
        }
        referencesToUpdate.add(referencesByCode);
        referencesToUpdate.add(referencesById);
        referencesToUpdate.add(referencesByIdContenu);
        return referencesToUpdate;
    }
}
