/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.service;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.formulaire.bean.ContenuformulaireBean;
import com.kosmos.formulaire.dto.FormulaireDto;
import com.kosmos.formulaire.service.impl.ContenuFormulaireService;
import com.kosmos.formulaire.service.impl.FormulaireService;
import com.kosmos.service.impl.ServiceManager;
import com.univ.batch.service.DeleteObsoleteDataService;

/**
 * Job de purge des données de formulaire.
 * Récupère les contenus de formulaire avec une date dépassant la durée de conservation maximum, et les supprime.
 */
public class DeleteObsoleteFormulaireData implements DeleteObsoleteDataService {

    private static final Logger LOG = LoggerFactory.getLogger(DeleteObsoleteFormulaireData.class);

    private int expungeOldContents(final String formCode, final Date expungeDate) {
        int count = 0;
        final ContenuFormulaireService contenuFormulaireService = ServiceManager.getService(ContenuFormulaireService.class);
        final List<ContenuformulaireBean> contents = contenuFormulaireService.getAllByFormCode(formCode);
        for (final ContenuformulaireBean content : contents) {
            if (content.getDateCreation().before(expungeDate)) {
                contenuFormulaireService.delete(content.getId());
                count++;
            }
        }
        return count;
    }

    @Override
    public void execute() {
        LOG.info("Exécution du script de nettoyage des données de formulaire.");
        final FormulaireService formulaireService = ServiceManager.getService(FormulaireService.class);
        final List<FormulaireDto> forms = formulaireService.getAllWithRecordsCount();
        for (final FormulaireDto form : forms) {
            if (form.getNombreEnregistrements() > 0) {
                final int months = form.getDureeConservation();
                final Date date = DateUtils.addMonths(new Date(), -months);
                final int count = expungeOldContents(form.getCode(), date);
                LOG.info("{} contenu(s) supprimé(s) sur {} pour le formulaire '{}/{}'", count, form.getNombreEnregistrements(), form.getCode(), form.getTitre());
            }
        }
    }
}
