/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.service.impl;

import java.util.List;
import java.util.Map;

import com.kosmos.formulaire.bean.ContenuformulaireBean;
import com.kosmos.formulaire.bean.FormulaireBean;
import com.kosmos.formulaire.dao.impl.FormulaireDAO;
import com.kosmos.formulaire.dto.ContenuFormulaireDto;
import com.kosmos.formulaire.dto.FormulaireDto;
import com.kosmos.service.impl.AbstractServiceBean;
import com.univ.objetspartages.bean.MediaBean;

/**
 * Service de gestion des formulaires.
 */
public class FormulaireService extends AbstractServiceBean<FormulaireBean, FormulaireDAO> {

    /**
     * Service d'accès aux données de formulaire.
     */
    private ContenuFormulaireService contenuFormulaireService;

    public List<FormulaireBean> getAllForms(String orderByColumn) {
        return dao.getAll(orderByColumn);
    }

    /**
     * Méthode permettant de récupérer le formulaire correspondant à un CONTENUFORMULAIRE, par l'ID du CONTENUFORMULAIRE.
     * @param Id L'ID du CONTENUFORMULAIRE dont on souhaite récupérer le FORMULAIRE parent.
     * @return Le formulaire associé à l'ID du CONTENUFORMULAIRE
     */
    public FormulaireBean getByContenuId(long Id) {
        return dao.getByContenuId(Id);
    }


    /**
     * Méthode permettant de récupérer la liste des formulaires avec le nombre de réponses
     * @return Une liste de {@link FormulaireDto}
     */
    public List<FormulaireDto> getAllWithRecordsCount() {
        return dao.getAllWithRecordsCount();
    }

    /**
     * Méthode permettant de récupérer la liste des formulaires avec les contenus contenant les valeurs en paramètre
     * @param params une MAP de code / valuer à rechercher dans les contenus de formaulaire
     * @return Une liste de {@link ContenuFormulaireDto}
     */
    public List<ContenuFormulaireDto> getAllWithContent(final Map<String, String> params) {
        return dao.getAllForContent(params);
    }

    /**
     * Ajoute une entrée avec un id forcé.
     * @param formulaireBean le formulaire à ajouter
     * @return le bean enregistré
     */
    public FormulaireBean addWithForceId(final FormulaireBean formulaireBean) {
        return dao.addWithForcedId(formulaireBean);
    }

    /**
     * Récupère la liste des formulaires attachés à une rubrique.
     * @param sectionCode le code de la rubrique
     * @return la liste des formulaires correspondant
     */
    public List<FormulaireBean> getBySection(String sectionCode) {
        return dao.getBySectionCode(sectionCode);
    }

    /**
     * Récupère un {@link FormulaireBean} par son code.
     * @param code le code du formulaire recherché
     * @return le formulaire
     */
    public FormulaireBean getByCode(final String code) {
        return dao.getByCode(code);
    }

    /**
     * Supprime un formulaire ainsi que son contenu lié.
     * @param formBean le formulaire à supprimer
     */
    public void delete(final FormulaireBean formBean) {
        dao.delete(formBean.getId());
        contenuFormulaireService.getAllByFormCode(formBean.getCode()).stream()
            .map(ContenuformulaireBean::getId)
            .forEach(contenuFormulaireService::delete);
    }

    public void setContenuFormulaireService(final ContenuFormulaireService contenuFormulaireService) {
        this.contenuFormulaireService = contenuFormulaireService;
    }
}
