/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.service.impl.beanexport;

import java.util.HashMap;
import java.util.Map;

import com.kosmos.formulaire.bean.ContenuformulaireBean;
import com.kosmos.formulaire.bean.FormulaireBean;
import com.kosmos.formulaire.bean.FormulaireBeanExport;
import com.kosmos.formulaire.om.Contenuformulaire;
import com.kosmos.formulaire.om.Formulaire;
import com.kosmos.formulaire.service.impl.ContenuFormulaireService;
import com.kosmos.formulaire.service.impl.FormulaireService;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.utils.UASOmHelper;
import com.kportal.extension.module.bean.BeanExportMap;
import com.kportal.extension.module.bean.MetadataExport;
import com.kportal.extension.module.dto.ServiceBeanExportDto;
import com.kportal.extension.module.service.ServiceBeanExport;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;

public class ServiceBeanExportFormulaire extends ServiceBeanExport<FormulaireBeanExport> {

    @Override
    public BeanExportMap<FormulaireBeanExport> getBeansByRubrique(final ServiceBeanExportDto serviceBeanExportDto) throws Exception {
        final BeanExportMap<FormulaireBeanExport> formulairesBeans = new BeanExportMap<>();
        //export des formulaires de la rubrique
        final Formulaire formulaire = new Formulaire();
        formulaire.init();
        final RequeteSQL requete = new RequeteSQL();
        final ClauseWhere where = new ClauseWhere();
        where.setPremiereCondition(ConditionHelper.in("T1.CODE_RUBRIQUE", serviceBeanExportDto.getCodesRubrique()));
        requete.where(where);
        formulaire.select(requete.formaterRequete());
        //export des contenus du formulaire
        final Contenuformulaire contenuFormulaire = new Contenuformulaire();
        contenuFormulaire.init();
        while (formulaire.nextItem()) {
            if (!formulairesBeans.containsKey(formulaire.getCode())) {
                final FormulaireBean formulaireBean = new FormulaireBean();
                UASOmHelper.copyProperties(formulaireBean, formulaire);
                formulaireBean.setId(formulaire.getIdFormulaire());
                final FormulaireBeanExport formulaireBeanExport = new FormulaireBeanExport();
                //on préfixe le titre pour le retrouver facilement dans la liste
                formulaire.setTitre(formulaire.getTitre());
                formulaireBeanExport.setIdModule(serviceBeanExportDto.getModule().getId());
                formulaireBeanExport.setBean(formulaireBean);
                //si on a des données saisies pour ce formulaire, on les exporte
                if (contenuFormulaire.select("WHERE CODE_FORMULAIRE='" + formulaire.getCode() + "'") > 0) {
                    Map<Long, ContenuformulaireBean> contenusFormulaire = new HashMap<>();
                    while (contenuFormulaire.nextItem()) {
                        final ContenuformulaireBean contenuFormulaireBean = new ContenuformulaireBean();
                        UASOmHelper.copyProperties(contenuFormulaireBean, contenuFormulaire);
                        contenusFormulaire.put(contenuFormulaire.getIdContenuformulaire(), contenuFormulaireBean);
                    }
                    formulaireBeanExport.setContenusFormulaire(contenusFormulaire);
                }
                FormulaireBeanExport finalFormulaireBeanExport = executeProcessors(formulaireBeanExport, serviceBeanExportDto);
                if (finalFormulaireBeanExport != null) {
                    formulairesBeans.put(formulaire.getCode(), finalFormulaireBeanExport);
                }
            }
        }
        return formulairesBeans;
    }

    @Override
    public MetadataExport getMetadataExport(final ServiceBeanExportDto serviceBeanExportDto) {
        final MetadataExport metadataExport = new MetadataExport();
        final FormulaireService serviceFormulaire = ServiceManager.getServiceForBean(FormulaireBean.class);
        final ContenuFormulaireService serviceContenuFormulaire = ServiceManager.getServiceForBean(ContenuformulaireBean.class);
        metadataExport.addMetadataSequence("formulaire", serviceFormulaire.getAutoIncrement());
        metadataExport.addMetadataSequence("contenuFormulaire", serviceContenuFormulaire.getAutoIncrement());
        return metadataExport;
    }
}
