/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: formulaire
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.formulaire.util;

import java.io.File;

import org.apache.commons.lang3.StringUtils;

import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.PermissionBean;

public class FormulaireUtil {

    public static final String ID_EXTENSION = "formulaire";

    private static final String PROP_FORMS_DEFAULTDIR = "forms.defaultdir";

    private static final String DEFAULT_FORMS_DEFAULTDIR = "/forms";

    public static final String KEY_PROPERTIES_PURGE_DELAY = "formulaire.purge.delay.months";

    public static boolean isAdministrateurFormulaire(final AutorisationBean autorisations) {
        boolean res = Boolean.FALSE;
        PermissionBean permission = new PermissionBean("TECH", "for", "C");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        permission = new PermissionBean("TECH", "for", "M");
        if (autorisations.getListePermissions().get(permission.getChaineSerialisee()) != null) {
            res = true;
        }
        return res;
    }

    public static String getFormsDefaultPath() {
        String folder = getProperty(PROP_FORMS_DEFAULTDIR);
        if (StringUtils.isBlank(folder)) {
            folder = WebAppUtil.getStorageDir() + DEFAULT_FORMS_DEFAULTDIR;
        }
        if (StringUtils.isNotEmpty(folder) && !(new File(folder).exists())) {
            new File(folder).mkdir();
        }
        return folder;
    }

    public static int getPurgeDelay() {
        return PropertyHelper.getCorePropertyAsInt(KEY_PROPERTIES_PURGE_DELAY, 12);
    }

    public static String getProperty(final String key) {
        return PropertyHelper.getProperty(ID_EXTENSION, key);
    }

    public static String getMessage(final String key) {
        return MessageHelper.getMessage(ID_EXTENSION, key);
    }
}
