/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ged
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.ged.util;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.univ.objetspartages.bean.DocumentBean;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.om.Document;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.service.ServiceDocument;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.util.MediaUtils;

public class GedUtil {

    public static final String ID_EXTENSION = "ged";

    public static String getProperty(final String key) {
        return PropertyHelper.getProperty(ID_EXTENSION, key);
    }

    public static String getMessage(final String key) {
        return MessageHelper.getMessage(ID_EXTENSION, key);
    }

    /**
     * Détermine le title d'un lien d'une fiche document
     * @param ficheUniv {@link FicheUniv}
     * @return le title affiché.
     */
    public static String getTitleContentGed(final FicheUniv ficheUniv) {
        String title = StringUtils.EMPTY;
        final ServiceDocument serviceDocument = ServiceManager.getServiceForBean(DocumentBean.class);
        if (ficheUniv != null && ficheUniv.getClass().isAssignableFrom(Document.class)) {
            final DocumentBean documentBean = (DocumentBean) ficheUniv.getPersistanceBean();
            return getTitleContentGed(documentBean);
        }
        return title;
    }

    /**
     * Détermine le title d'un lien d'une fiche document
     * @param documentBean {@link DocumentBean}
     * @return le title affiché.
     */
    public static String getTitleContentGed(final DocumentBean documentBean) {
        String title = StringUtils.EMPTY;
        final ServiceDocument serviceDocument = ServiceManager.getServiceForBean(DocumentBean.class);
        final RessourceBean ressourceBean = serviceDocument.getSingleRessourceForDownload(documentBean);
        if (ressourceBean != null) {
            final ServiceMedia serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
            final MediaBean mediaBean = serviceMedia.getById(ressourceBean.getIdMedia());
            if (mediaBean != null) {
                title = MediaUtils.getTitleMedia(serviceDocument.getLibelleAffichable(documentBean), mediaBean);
            }
        }
        return title;
    }
}
