/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.importexport;

import com.kosmos.importexport.util.ExtensionUtil;
import com.kportal.core.autorisation.util.PermissionUtil;
import com.kportal.extension.module.IModule;
import com.kportal.extension.module.ModuleHelper;
import com.kportal.extension.module.composant.ComposantAdministration;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.PermissionBean;

public class ComposantImportExport extends ComposantAdministration {

    public static final String ID_BEAN = "composantImportExport";

    public static final String ID_PERMISSION = "IMPORTEXPORT";

    /**
     * La permission permettant d'importer/exporter des contenus
     */
    public static final String CODE_PERMISSION = "gestion";

    /**
     * La permission permettant d'importer/exporter des contenus
     */
    public static final String CODE_ACTION_GESTION = "G";

    public static IModule getModule() {
        return ModuleHelper.getModule(ExtensionUtil.getExtension().getId(), ID_BEAN);
    }

    /**
     * Retourne la permission de gestion des imports exports
     */
    public static PermissionBean getPermissionGestion() {
        return PermissionUtil.getPermissionBean(getModule(), CODE_PERMISSION, CODE_ACTION_GESTION);
    }

    /**
     * En fonction des autorisations et de l'action fourni on retourne vrai si l'utilisateur a les droits en fonction de l'action qu'il entreprend.
     */
    public static boolean isAutoriseParActionProcessus(final AutorisationBean autorisations, final String action) {
        boolean isAutorise = autorisations != null && autorisations.isWebMaster();
        if (!isAutorise && autorisations != null) {
            isAutorise = autorisations.possedePermission(getPermissionGestion());
        }
        return isAutorise;
    }

    @Override
    public boolean isVisible(final AutorisationBean autorisation) {
        boolean isVisible = Boolean.FALSE;
        if (isAccessibleBO() && autorisation != null) {
            isVisible = autorisation.possedePermission(getPermissionGestion());
        }
        return isVisible;
    }

    @Override
    public boolean isActionVisible(final AutorisationBean autorisation, final String code) {
        return isVisible(autorisation);
    }
}
