/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.importexport;

import org.quartz.JobDetail;
import org.springframework.beans.factory.annotation.Required;

import com.kportal.extension.IExtension;
import com.kportal.extension.IExtensionConfig;
import com.kportal.extension.Version;
import com.univ.batch.imports.ImportXML;
import com.univ.objetspartages.util.TraitementImportUtil;

/**
 * Classe permettant la configuration de l'extension après chargement du contexte Spring.
 * Created by josse on 22/12/16.
 */
public class ImportExportConfig implements IExtensionConfig {

	/**
	 * Job d'import XML.
	 */
	private JobDetail importXmlJob;

	@Override
	public void init(final IExtension extension, final Version old) {
		ajouteParametresImportXmlJob(extension);
	}

	@Override
	public void clean(final IExtension extension) {
		ajouteParametresImportXmlJob(extension);
	}

	/**
	 * Ajoute des paramètres au job d'import XML en fonction de la configuration de l'extension.
	 *
	 * @param extension
	 * 		l'extension en cours de (re)chargement.
	 */
	private void ajouteParametresImportXmlJob(final IExtension extension) {

		// Ajout d'un paramètre permettant la sélection d'une transformation XML si la fonctionnalité est activée
		if ("1".equals(extension.getProperty(TraitementImportUtil.CLE_ACTIVATION_TRANSFORMATION_IMPORT))) {

			// Récupération des transformations XML possibles
			final String listeTransformationPourScriptAutomatise = TraitementImportUtil.getListeTransformationPourScriptAutomatise(extension);

			// Ajout du paramètre dans la liste des paramètres du job
			importXmlJob.getJobDataMap().put(ImportXML.TRANSFORMATION_XML, listeTransformationPourScriptAutomatise);
		}

		// Récupération des patterns de fichiers définis
		final String listePatternsFichiersPourScriptAutomatise = TraitementImportUtil.getListePatternsFichiersPourScriptAutomatise(extension);

		// Ajout du paramètre dans la liste des paramètres du job
		importXmlJob.getJobDataMap().put(ImportXML.PATTERN_FICHIERS, listePatternsFichiersPourScriptAutomatise);
	}

	@Required
	public void setImportXmlJob(final JobDetail importXmlJob) {
		this.importXmlJob = importXmlJob;
	}
}
