/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.importexport.service;

import java.io.File;
import java.util.Collection;
import java.util.Date;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.importexport.util.ExtensionUtil;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.batch.service.DeleteObsoleteDataService;

public class DeleteObsoleteDataImportExport implements DeleteObsoleteDataService {

    private static final Logger LOG = LoggerFactory.getLogger(DeleteObsoleteDataImportExport.class);

    @Override
    public void execute() {
        LOG.info("Exécution du script de nettoyage du répertoire de sauvegarde d'import XML");
        int days = PropertyHelper.getPropertyAsInt(ExtensionUtil.ID_EXTENSION, "importExport.purge.delay.days", -14);
        if (days > 0) {
            days = -days;
        }
        Date date = DateUtils.addDays(new Date(), days);
        File dossierImport = new File(WebAppUtil.getSauvegardePath() + "/imports");
        long count = 0;
        if (dossierImport.exists() && dossierImport.isDirectory()) {
            Collection<File> files = FileUtils.listFiles(dossierImport, null, true);
            count = files.stream().filter(file -> FileUtils.isFileOlder(file, date)).map(FileUtils::deleteQuietly).filter(r -> r == true).count();
        }
        LOG.info("  - Nombre de fichiers supprimés dans le répertoire {} : {}", dossierImport.getAbsolutePath(), count);
    }
}
