/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.importexport.util;

import java.io.File;

import org.apache.commons.lang3.StringUtils;

import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.extension.ExtensionHelper;
import com.kportal.extension.IExtension;

public class ExtensionUtil {

    /**
     * Nom de la propriété qui permet de surcharger le chemin du répertoire contenant les fichiers XSL.
     */
    private static final String PROP_XSL_PATH = "transformation.xsl_path";

    /**
     * Emplacement relatif du répertoire contenant les fichiers XSL.
     */
    private static final String DEFAULT_XSL_PATH = "/WEB-INF/xsl/";

    public static final String HWS_WORK_DEFAULTDIR = "/hws";

    public static final String PROP_HWS_EXPORT_MEDIA = "hws.export.media";

    public static final String ID_EXTENSION = "importExport";

    private static final String PROP_HWS_WORK_DEFAULTDIR = "hws.repertoire.import";

    public static IExtension getExtension() {
        return ExtensionHelper.getExtensionManager().getExtension(ID_EXTENSION);
    }

    public static String getProperty(final String key) {
        return PropertyHelper.getProperty(ID_EXTENSION, key);
    }

    public static String getMessage(final String key) {
        return MessageHelper.getMessage(ID_EXTENSION, key);
    }

    public static String getHwsDefaultPath() {
        final String res = StringUtils.defaultIfEmpty(getProperty(PROP_HWS_WORK_DEFAULTDIR), WebAppUtil.getWorkDefaultPath() + HWS_WORK_DEFAULTDIR);
        if (StringUtils.isNotEmpty(res) && !(new File(res).exists())) {
            new File(res).mkdir();
        }
        return res;
    }

    /**
     * Retourne le chemin du répertoire contenant les fichiers XSL utilisés par l'extension.
     *
     * @return le chemin absolu du répertoire.
     */
    public static String getXslPath() {
        return StringUtils.defaultIfEmpty(ExtensionUtil.getProperty(PROP_XSL_PATH), WebAppUtil.getAbsolutePath() + getExtension().getRelativePath() + DEFAULT_XSL_PATH);
    }
}
