/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.batch;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.newsletter.util.NewsletterUtil;
import com.kportal.scheduling.spring.quartz.LogReportJob;
import com.univ.utils.ContexteUtil;

public class PurgeNewsletterJob extends LogReportJob {

    private static final Logger LOG = LoggerFactory.getLogger(PurgeNewsletterJob.class);

    private static final String DELETE_ABONNEMENT = "DELETE FROM DEMANDEABONNEMENT";

    private static final String DELETE_MAILING = "DELETE FROM MAILING WHERE DATE_PURGE <= ?";

    private static final String DELETE_MAILING_ATTACHMENT = "DELETE R FROM RESSOURCE R JOIN MAILING M ON R.CODE_PARENT = CONCAT(M.ID_MAILING,',TYPE=MAILING') AND DATE_PURGE <= ?";

    @Override
    public void perform() {
        try {
            // ajout d'un contexte dans le thread local
            ContexteUtil.setContexteSansRequete();
            if ("1".equals(NewsletterUtil.getProperty("static.purge_abonnement_newsletter"))) {
                LOG.info("Purge des demandes d'abonnement aux newsletters");
                try (
                    Connection cnx = ContexteUtil.getContexteUniv().getConnection();
                    PreparedStatement stmt = cnx.prepareStatement(DELETE_ABONNEMENT)
                ) {
                    stmt.executeUpdate();
                } catch (SQLException exc) {
                    LOG.error("Erreur SQL lors de la suppression des demandes d'abonnement", exc);
                }
            } else {
                LOG.info("Purge des demandes d'abonnement aux newsletters désactivé");
            }
            LOG.info("Suppression des archives des envois de newsletter");
            // suppression globale en une seule requête
            try (
                Connection cnx = ContexteUtil.getContexteUniv().getConnection();
                PreparedStatement stmt1 = cnx.prepareStatement(DELETE_MAILING_ATTACHMENT);
                PreparedStatement stmt2 = cnx.prepareStatement(DELETE_MAILING)
            ) {
                Date datePurge = new Date(System.currentTimeMillis());
                stmt1.setDate(1,datePurge);
                stmt1.executeUpdate();

                stmt2.setDate(1,datePurge);
                stmt2.executeUpdate();
            } catch (SQLException exc) {
                LOG.error("Erreur SQL lors de la purge des mailings obsolètes", exc);
            }
        } catch (Exception e) {
            LOG.error("Erreur d'execution du batch PurgeJob : ", e);
        } finally {
            // suppression du contexte du thread local
            ContexteUtil.releaseContexteUniv();
        }
    }
}
