/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.bean;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.univ.objetspartages.bean.AbstractPersistenceBean;

public class ModelemailBean extends AbstractPersistenceBean implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = -4339260220078697869L;

    /**
     * The type.
     */
    protected String type = StringUtils.EMPTY;

    /**
     * The etat.
     */
    protected String etat = StringUtils.EMPTY;

    /**
     * The public_interne.
     */
    protected String public_interne = "0";

    /** The numero. */
    protected String numero = StringUtils.EMPTY;

    /** The intitule. */
    protected String intitule = StringUtils.EMPTY;

    /** The code responsables. */
    protected String codeResponsables = StringUtils.EMPTY;

    /** The objet. */
    protected String objet = StringUtils.EMPTY;

    /** The contenu. */
    protected String contenu = StringUtils.EMPTY;

    /** The pied. */
    protected String pied = StringUtils.EMPTY;

    /** The code rubrique. */
    protected String codeRubrique = StringUtils.EMPTY;

    /** The code rattachement. */
    protected String codeRattachement = StringUtils.EMPTY;

    /** The id bandeau bas. */
    protected Long idBandeauBas = 0L;

    /** The id bandeau haut. */
    protected Long idBandeauHaut = 0L;

    /**
     * The diffusion public vise.
     */
    protected String diffusionPublicVise = StringUtils.EMPTY;

    /** The utilisateurs vises. */
    protected String utilisateursVises = StringUtils.EMPTY;

    /** The format envoi. */
    protected String formatEnvoi = "0";

    /** The action inscription. */
    protected String actionInscription = StringUtils.EMPTY;

    private List<String> listeDestinatairesFinaux;

    private Date datePurge;

    private String libelleResponsable;

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getEtat() {
        return etat;
    }

    public void setEtat(String etat) {
        this.etat = etat;
    }

    public String getPublicInterne() {
        return public_interne;
    }

    public void setPublicInterne(String public_interne) {
        this.public_interne = public_interne;
    }

    public String getNumero() {
        return numero;
    }

    public void setNumero(String numero) {
        this.numero = numero;
    }

    public String getIntitule() {
        return intitule;
    }

    public void setIntitule(String intitule) {
        this.intitule = intitule;
    }

    public String getCodeResponsables() {
        return codeResponsables;
    }

    public void setCodeResponsables(String codeResponsables) {
        this.codeResponsables = codeResponsables;
    }

    public String getObjet() {
        return objet;
    }

    public void setObjet(String objet) {
        this.objet = objet;
    }

    public String getContenu() {
        return contenu;
    }

    public void setContenu(String contenu) {
        this.contenu = contenu;
    }

    public String getPied() {
        return pied;
    }

    public void setPied(String pied) {
        this.pied = pied;
    }

    public String getCodeRubrique() {
        return codeRubrique;
    }

    public void setCodeRubrique(String codeRubrique) {
        this.codeRubrique = codeRubrique;
    }

    public String getCodeRattachement() {
        return codeRattachement;
    }

    public void setCodeRattachement(String codeRattachement) {
        this.codeRattachement = codeRattachement;
    }

    @GetterAnnotation(isIdMedia = true)
    public Long getIdBandeauBas() {
        return idBandeauBas;
    }

    @SetterAnnotation(isIdMedia = true)
    public void setIdBandeauBas(Long idBandeauBas) {
        this.idBandeauBas = idBandeauBas;
    }

    @GetterAnnotation(isIdMedia = true)
    public Long getIdBandeauHaut() {
        return idBandeauHaut;
    }

    @SetterAnnotation(isIdMedia = true)
    public void setIdBandeauHaut(Long idBandeauHaut) {
        this.idBandeauHaut = idBandeauHaut;
    }

    public String getDiffusionPublicVise() {
        return diffusionPublicVise;
    }

    public void setDiffusionPublicVise(String diffusionPublicVise) {
        this.diffusionPublicVise = diffusionPublicVise;
    }

    public String getUtilisateursVises() {
        return utilisateursVises;
    }

    public void setUtilisateursVises(String utilisateursVises) {
        this.utilisateursVises = utilisateursVises;
    }

    public String getFormatEnvoi() {
        return formatEnvoi;
    }

    public void setFormatEnvoi(String formatEnvoi) {
        this.formatEnvoi = formatEnvoi;
    }

    public String getActionInscription() {
        return actionInscription;
    }

    public void setActionInscription(String actionInscription) {
        this.actionInscription = actionInscription;
    }


    /**
     * Attention ce champ n'est pas persisté...
     *
     * @return the destinataires finaux
     */
    public List<String> getDestinatairesFinaux() {
        return listeDestinatairesFinaux;
    }

    /**
     * Attention ce champ n'est pas persisté...
     *
     * @param listeUtilisateurs
     *            the new destinataires finaux
     */
    public void setDestinatairesFinaux(final List<String> listeUtilisateurs) {
        listeDestinatairesFinaux = listeUtilisateurs;
    }

    /**
     * Attention ce champ n'est pas persisté...
     * @return the date purge
     */
    public Date getDatePurge() {
        return datePurge;
    }

    /**
     * Attention ce champ n'est pas persisté...
     * @param datePurge
     *            the new date purge
     */
    public void setDatePurge(final Date datePurge) {
        this.datePurge = datePurge;
    }

    /**
     * Nom de l'expéditeur de la newsletter
     * @return Nom de l'expéditeur
     */
    public String getLibelleResponsable() {
        return libelleResponsable;
    }

    /**
     * Modifie le nom de l'expéditeur de la newsletter
     * @param libelleResponsable Nom de l'expéditeur
     */
    public void setLibelleResponsable(final String libelleResponsable) {
        this.libelleResponsable = libelleResponsable;
    }
}
