/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.dao.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

import com.jsbsoft.jtf.datasource.dao.impl.mysql.AbstractLegacyDAO;
import com.jsbsoft.jtf.datasource.exceptions.AddToDataSourceException;
import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.jsbsoft.jtf.datasource.utils.DaoUtils;
import com.kosmos.newsletter.bean.ModelemailBean;

/**
 * DAO permettant de gérer les modèles de mailing
 */
public class ModelemailDAO extends AbstractLegacyDAO<ModelemailBean> {

    public ModelemailDAO() {
        tableName = "MODELEMAIL";
    }

    public List<ModelemailBean> selectByIdBandeau(long idBandeau) throws DataSourceException {
        List<ModelemailBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("id", idBandeau);
            result = namedParameterJdbcTemplate.query("SELECT * FROM MODELEMAIL WHERE ID_BANDEAU_BAS = :id OR ID_BANDEAU_HAUT = :id", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MODELEMAIL row with id %s", idBandeau), dae);
        }
        return result;
    }

    public List<ModelemailBean> selectByIds(List<Long> ids) throws DataSourceException {
        final List<ModelemailBean> result = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(ids)) {
            try {
                final MapSqlParameterSource params = new MapSqlParameterSource("ids", ids);
                result.addAll(namedParameterJdbcTemplate.query("SELECT * FROM MODELEMAIL WHERE ID_MODELEMAIL in (:ids)", params, rowMapper));
            } catch (DataAccessException dae) {
                throw new DataSourceException(String.format("An error occured requesting for MODELEMAIL row with ids %s", StringUtils.join(ids, ",")), dae);
            }
        }
        return result;
    }

    public List<ModelemailBean> selectByState(String state) throws DataSourceException {
        List<ModelemailBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("state", state);
            result = namedParameterJdbcTemplate.query("SELECT * FROM MODELEMAIL WHERE ETAT = :state", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MODELEMAIL row with state %s", state), dae);
        }
        return result;
    }

    public List<ModelemailBean> selectByCodeRubrique(String codeRubrique) throws DataSourceException {
        List<ModelemailBean> result;
        try {
            final MapSqlParameterSource params = new MapSqlParameterSource("codeRubrique", codeRubrique);
            result = namedParameterJdbcTemplate.query("SELECT * FROM MODELEMAIL WHERE CODE_RUBRIQUE = :codeRubrique", params, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for MODELEMAIL row with codeRubrique %s", codeRubrique), dae);
        }
        return result;
    }

    public List<ModelemailBean> selectByCodesRubriques(List<String> codes) throws DataSourceException {
        final List<ModelemailBean> result = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(codes)) {
            try {
                final MapSqlParameterSource params = new MapSqlParameterSource("codes", codes);
                result.addAll(namedParameterJdbcTemplate.query("SELECT * FROM MODELEMAIL WHERE CODE_RUBRIQUE in (:codes)", params, rowMapper));
            } catch (DataAccessException dae) {
                throw new DataSourceException(String.format("An error occured requesting for MODELEMAIL row with codes in (%s)", StringUtils.join(codes, ",")), dae);
            }
        }
        return result;
    }

    public List<ModelemailBean> selectAll() {
        List<ModelemailBean> result;
        try {
            result = namedParameterJdbcTemplate.query("SELECT * FROM MODELEMAIL ORDER BY INTITULE", rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting for all values of MODELEMAIL", dae);
        }
        return result;
    }

    public List<ModelemailBean> selectAllWithValidState() {
        List<ModelemailBean> result;
        try {
            result = namedParameterJdbcTemplate.query("SELECT * FROM MODELEMAIL WHERE ETAT = '' ORDER BY INTITULE", rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException("An error occured requesting for all values of MODELEMAIL", dae);
        }
        return result;
    }

    public ModelemailBean addWithForcedId(ModelemailBean bean) {
        try {
            namedParameterJdbcTemplate.update(DaoUtils.getAddWithForcedIdQuery(tableName, getColumns(), columnNamingStrategy), getParameters(bean));
        } catch (DataAccessException dae) {
            throw new AddToDataSourceException(String.format("Unable to add [%s] to table \"%s\"", bean.toString(), tableName), dae);
        }
        return bean;
    }

}
