/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.module;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.controllers.SGController;
import com.kosmos.newsletter.util.NewsletterUtil;
import com.kportal.core.config.PropertyHelper;
import com.kportal.tag.interpreteur.impl.AbstractInterpreteurTag;
import com.univ.utils.Chaine;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.URLResolver;

public class InterpreterTagLienDesabonnement extends AbstractInterpreteurTag {

    @Override
    public String interpreterTag(final String texteAInterpreter, final String baliseOuvrante, final String baliseFermante) {
        String parametres = StringUtils.substringBetween(texteAInterpreter, baliseOuvrante, baliseFermante);
        final ContexteUniv ctx = ContexteUtil.getContexteUniv();
        String contenuLien = StringUtils.EMPTY;
        parametres = Chaine.remplacerPointsVirgules(parametres);
        final String[] lstParam = parametres.split("\\*", -2);
        if (lstParam.length == 2) {
            final String idModeleMail = lstParam[0];
            final String libelleLien = lstParam[1];
            final HttpServletRequest requeteHTTP = ctx.getRequeteHTTP();
            String mode = StringUtils.EMPTY;
            String format = StringUtils.EMPTY;
            if (requeteHTTP != null) {
                mode = requeteHTTP.getParameter("MODE");
                format = requeteHTTP.getParameter("FORMAT_ENVOI");
            }
            String ampersands = "&amp;";
            if ("ENVOI".equals(mode) && "1".equals(format)) {
                ampersands = "&";
            }
            final StringBuilder urlDemandeDesabonnement = new StringBuilder()
                .append(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY))
                .append("?PROC=TRAITEMENT_DEMANDE_ABONNEMENT")
                .append(ampersands)
                .append("EXT=")
                .append(NewsletterUtil.ID_EXTENSION)
                .append(ampersands)
                .append("ACTION=DESABONNER")
                .append(ampersands)
                .append("NB_NEWSLETTERS=1")
                .append(ampersands)
                .append("ID_MODELE_MAIL0=")
                .append(idModeleMail)
                .append(ampersands)
                .append("CODE0=");
            if ( ctx.getCode()!=null) {
                urlDemandeDesabonnement.append(ctx.getCode());
            } else if (requeteHTTP != null) {
                urlDemandeDesabonnement.append(requeteHTTP.getParameter("CODE_UTILISATEUR"));
            }
            final String url = urlDemandeDesabonnement.toString();
            contenuLien = "<a href=\"" + URLResolver.getAbsoluteUrl(url, ctx) + "\" >" + libelleLien + "</a>";
        }
        return contenuLien;
    }

    @Override
    public String getReferenceTag(final String texteAInterpreter, final String baliseOuvrante, final String baliseFermante) {
        return StringUtils.EMPTY;
    }
}