/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.module;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.controllers.SGController;
import com.kosmos.newsletter.util.NewsletterUtil;
import com.kportal.core.config.PropertyHelper;
import com.kportal.tag.interpreteur.impl.DefaultInterpreteurTemplateTag;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.URLResolver;

public class InterpreteurTagFormulaireAbonnement extends DefaultInterpreteurTemplateTag {

    @Override
    public String interpreterTag(final String texteAInterpreter, final String baliseOuvrante, final String baliseFermante) {
        final String parametres = StringUtils.substringBetween(texteAInterpreter, baliseOuvrante, baliseFermante);
        if (baliseOuvrante.equals("[formabonews;")) {
            String contenuLien;
            final ContexteUniv ctx = ContexteUtil.getContexteUniv();
            List<String> allSerializedIds = Arrays.stream(parametres.split(","))
                .filter(StringUtils::isNumeric)
                .collect(Collectors.toList());

            final HttpServletRequest requeteHTTP = ctx.getRequeteHTTP();
            String ampersands = "&amp;";
            final StringBuilder urlDemandeAbonnement = new StringBuilder(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY))
                .append("?PROC=TRAITEMENT_DEMANDE_ABONNEMENT")
                .append(ampersands)
                .append("EXT=")
                .append(NewsletterUtil.ID_EXTENSION)
                .append(ampersands)
                .append("ACTION=DEMANDER_ABONNEMENT")
                .append(ampersands)
                .append("NB_NEWSLETTERS=")
                .append(allSerializedIds.size())
                .append(ampersands)
                .append("ID_MODELE_MAIL=")
                .append(String.join(",", allSerializedIds))
                .append(ampersands)
                .append("RH=")
                .append(ctx.getCodeRubriquePageCourante())
                .append(ampersands)
                .append("CODE=");
            if ( ctx.getCode()!=null) {
                urlDemandeAbonnement.append(ctx.getCode());
            } else if (requeteHTTP != null) {
                urlDemandeAbonnement.append(requeteHTTP.getParameter("CODE_UTILISATEUR"));
            }
            final String url = urlDemandeAbonnement.toString();
            contenuLien = "<a href=\"" + URLResolver.getAbsoluteUrl(url, ctx) + "\" >" + NewsletterUtil.getMessage("ST_NEWSLETTER_BUTTON_S_ABONNER") + "</a>";

            return contenuLien;
        }
        return StringUtils.EMPTY;
    }

    @Override
    public String getReferenceTag(final String texteAInterpreter, final String baliseOuvrante, final String baliseFermante) {
        return StringUtils.EMPTY;
    }

}
