/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.om;

import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.newsletter.bean.MailingBean;
import com.kosmos.newsletter.dao.impl.MailingDAO;
import com.kosmos.newsletter.service.ServiceMailing;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.om.AbstractOm;
import com.univ.utils.sql.Operateur;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.operande.TypeOperande;

/**
 * The Class Mailing.
 */
public class Mailing extends AbstractOm<MailingBean, MailingDAO> {

    private final ServiceMailing serviceMailing;

    /**
     * Instantiates a new mailing.
     */
    public Mailing() {
        super();
        serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
    }

    /**
     * Inits the.
     */
    public void init() {
        setIdMailing((long) 0);
        setEmail(StringUtils.EMPTY);
        setNomUtilisateur(StringUtils.EMPTY);
        setReplyTo(StringUtils.EMPTY);
        setObjet(StringUtils.EMPTY);
        setTypeMime(StringUtils.EMPTY);
        setMessage(StringUtils.EMPTY);
        setFichier(StringUtils.EMPTY);
        setNomFichier(StringUtils.EMPTY);
        setEtat(StringUtils.EMPTY);
        setErrorMessage(StringUtils.EMPTY);
        setIdNewsletter(StringUtils.EMPTY);
        setCodeUtilisateur(StringUtils.EMPTY);
        setDateCreation(new Date(System.currentTimeMillis()));
        setType(StringUtils.EMPTY);
    }

    /**
     * Sélectionne le mailing envoyé a un utilisateur.
     *
     * @param emailUtilisateur
     *            email de l'utilisateur
     * @param idNewsletter
     *            identifiant de la newsletter
     *
     * @return the int
     *
     * @throws Exception
     *             the exception
     */
    public int select(final String emailUtilisateur, final String idNewsletter) {
        currentSelect = serviceMailing.getByEmailAndIdNewsLetter(emailUtilisateur, idNewsletter);
        currentSelectIt = currentSelect.iterator();
        return currentSelect.size();
    }

    /**
     * Sélectionne les mailings dont la date de purge est dépassée.
     *
     * @param date
     *            the date
     *
     * @return the int
     *
     * @throws Exception
     *             the exception
     */
    public int selectParDate(final Date date) {
        final ClauseWhere whereDate = new ClauseWhere(ConditionHelper.genericConditionSurColonne("DATE_PURGE", date, TypeOperande.DATE, Operateur.GREATER_EQUALS));
        return select(whereDate.formaterSQL());
    }

    /**
     * Sélectionne les mailing par titre,type et date de création
     *
     * @param objet
     *            L' objet du mailing recherché
     * @param type
     *            Le type du mailing rechderché
     * @param dateDebut
     *            La date minimum de création du mailing
     * @param dateFin
     *            La date maximum de création du mailing
     * @param publicInterne
     *            vrai le public est interne, faux si il est externe NE SERT PLUS
     * @return Les mailings correspondant aux critères
     */
    public int selectParObjetTypeDateCreationEtPublic(final String objet, final String type, final Date dateDebut, final Date dateFin, final boolean publicInterne) {
        currentSelect = serviceMailing.getByObjectTypeCreationDatePublic(objet, type, dateDebut, dateFin);
        currentSelectIt = currentSelect.iterator();
        return currentSelect.size();
    }

    public Long getIdMailing() {
        return persistenceBean.getId();
    }

    public void setIdMailing(Long id) {
        persistenceBean.setId(id);
    }

    public String getEmail() {
        return persistenceBean.getEmail();
    }

    public void setEmail(String email) {
        persistenceBean.setEmail(email);
    }

    public String getCodeUtilisateur() {
        return persistenceBean.getCodeUtilisateur();
    }

    public void setCodeUtilisateur(String codeUtilisateur) {
        persistenceBean.setCodeUtilisateur(codeUtilisateur);
    }

    public String getNomUtilisateur() {
        return persistenceBean.getNomUtilisateur();
    }

    public void setNomUtilisateur(String nomUtilisateur) {
        persistenceBean.setNomUtilisateur(nomUtilisateur);
    }

    public String getReplyTo() {
        return persistenceBean.getReplyTo();
    }

    public void setReplyTo(String replyTo) {
        persistenceBean.setReplyTo(replyTo);
    }

    public String getObjet() {
        return persistenceBean.getObjet();
    }

    public void setObjet(String objet) {
        persistenceBean.setObjet(objet);
    }

    public String getTypeMime() {
        return persistenceBean.getTypeMime();
    }

    public void setTypeMime(String typeMime) {
        persistenceBean.setTypeMime(typeMime);
    }

    public String getMessage() {
        return persistenceBean.getMessage();
    }

    public void setMessage(String message) {
        persistenceBean.setMessage(message);
    }

    public String getFichier() {
        return persistenceBean.getFichier();
    }

    public void setFichier(String fichier) {
        persistenceBean.setFichier(fichier);
    }

    public String getNomFichier() {
        return persistenceBean.getNomFichier();
    }

    public void setNomFichier(String nomFichier) {
        persistenceBean.setNomFichier(nomFichier);
    }

    public String getEtat() {
        return persistenceBean.getEtat();
    }

    public void setEtat(String etat) {
        persistenceBean.setEtat(etat);
    }

    public String getErrorMessage() {
        return persistenceBean.getErrorMessage();
    }

    public void setErrorMessage(String errorMessage) {
        persistenceBean.setErrorMessage(errorMessage);
    }

    public String getIdNewsletter() {
        return persistenceBean.getIdNewsletter();
    }

    public void setIdNewsletter(String idNewsletter) {
        persistenceBean.setIdNewsletter(idNewsletter);
    }

    public Date getDatePurge() {
        return persistenceBean.getDatePurge();
    }

    public void setDatePurge(Date datePurge) {
        persistenceBean.setDatePurge(datePurge);
    }

    public String getType() {
        return persistenceBean.getType();
    }

    public void setType(String type) {
        persistenceBean.setType(type);
    }

    public Date getDateCreation() {
        return persistenceBean.getDateCreation();
    }

    public void setDateCreation(Date dateCreation) {
        persistenceBean.setDateCreation(dateCreation);
    }
}
