/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.om;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.dao.impl.ModelemailDAO;
import com.kosmos.newsletter.service.ServiceModelemail;
import com.kosmos.newsletter.util.ModeleMailThread;
import com.kosmos.newsletter.util.NewsletterUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.univ.mediatheque.UsingLibelleMedia;
import com.univ.objetspartages.bean.GroupeUtilisateurBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.om.AbstractOm;
import com.univ.objetspartages.om.ParagrapheBean;
import com.univ.objetspartages.om.Utilisateur;
import com.univ.objetspartages.services.ServiceGroupeUtilisateur;
import com.univ.utils.ContexteUtil;
import com.univ.utils.EscapeString;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.Operateur;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.operande.TypeOperande;
/**
 * The Class Modelemail.
 */
public class Modelemail extends AbstractOm<ModelemailBean, ModelemailDAO> implements Cloneable, UsingLibelleMedia {

    /**
     * The _pattern css.
     */
    public static Pattern _patternCSS;

    /**
     * The last injection.
     */
    private static long lastInjection = 0;

    static {
        try {
            _patternCSS = Pattern.compile("\\s");
        } catch (final Exception e) {
        }
    }


    /**
     * The liste destinataires finaux.
     */
    private ArrayList<String> listeDestinatairesFinaux = null;

    /** The date purge. */
    private Date datePurge;

    /**
     * Instantiates a new modelemail.
     */
    public Modelemail() {
        super();
    }

    /**
     * Lire fichier news letter.
     *
     * @param fichier the fichier
     * @return the string
     * @throws Exception the exception
     */
    public static String lireFichierNewsLetter(final String fichier) throws Exception {
        final String fichierFinNewsletter = NewsletterUtil.getProperty(fichier);
        String res = StringUtils.EMPTY;
        if (fichierFinNewsletter != null) {
            final File f = new File(fichierFinNewsletter);
            final InputStream is = new FileInputStream(f);
            // Get the size of the file
            final long length = f.length();
            // Create the byte array to hold the data
            final byte[] bytes = new byte[(int) length];
            // Read in the bytes
            is.read(bytes);
            // Close the input stream and return bytes
            is.close();
            res = new String(bytes);
        }
        return res;
    }

    /**
     * Retourne la liste des modèles de mail auxquels un utilisateur donné est inscrit (soit par inscription directe, soit par rattachement à un groupe).
     *
     * @param _ctx
     *            the _ctx
     * @param _u
     *            Utilisateur dont on souhaite obtenir les newsletters
     *
     * @return ArrayList liste des identifiants des modèles de mails
     *
     * @throws Exception
     *             the exception
     * @deprecated Utilisez {@link Modelemail#getListeModelesParUtilisateur(OMContext, UtilisateurBean)}
     */
    @Deprecated
    public static ArrayList<Long> getListeModelesParUtilisateur(final OMContext _ctx, final Utilisateur _u) throws Exception {
        return getListeModelesParUtilisateur(_ctx, _u.getPersistenceBean());
    }

    /**
     * Retourne la liste des modèles de mail auxquels un utilisateur donné est inscrit (soit par inscription directe, soit par rattachement à un groupe).
     *
     * @param ctx
     *            the _ctx
     * @param user
     *            Utilisateur dont on souhaite obtenir les newsletters
     *
     * @return ArrayList liste des identifiants des modèles de mails
     *
     * @throws Exception
     *             the exception
     */
    public static ArrayList<Long> getListeModelesParUtilisateur(final OMContext ctx, final UtilisateurBean user) throws Exception {
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        List<ModelemailBean> allMail = serviceModelemail.getAll();
        final ArrayList<Long> listeModeles = new ArrayList<>();
        final String codeUtilisateur = user.getCode();
        final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
        final Collection<String> vGroupesUtilisateur = serviceGroupeUtilisateur.getAllGroupsCodesByUserCode(user.getCode());
        for (ModelemailBean bean : allMail) {
            String listeUtilisateursVises = bean.getUtilisateursVises();
            if (StringUtils.isNotEmpty(listeUtilisateursVises) && (listeUtilisateursVises.equals(codeUtilisateur) || listeUtilisateursVises.endsWith(";" + codeUtilisateur) || listeUtilisateursVises.startsWith(codeUtilisateur + ";") || listeUtilisateursVises.contains(
                ";" + codeUtilisateur + ";"))) {
                listeModeles.add(bean.getId());
                continue;
            }
            // groupes cibles
            String listeGroupesVises = bean.getDiffusionPublicVise();
            if (vGroupesUtilisateur.size() > 0 && !"".equals(listeGroupesVises)) {
                for (String codeGroupeCourant : vGroupesUtilisateur) {
                    if (listeGroupesVises.contains("/" + codeGroupeCourant + "]")) {
                        listeModeles.add(bean.getId());
                        break;
                    }
                }
            }
        }
        return listeModeles;
    }

    /**
     * Détermine si le thread est actif.
     *
     * @return boolean estactif
     */
    public static boolean estActif() {
        boolean res = true;
        /* false si inactif depuis + de 10 secondes */
        if (System.currentTimeMillis() > lastInjection + 10000) {
            res = false;
        }
        return res;
    }

    public static boolean isValidEmailAddress(final String aEmailAddress) {
        if (aEmailAddress == null) {
            return false;
        }
        boolean result = true;
        try {
            new InternetAddress(aEmailAddress);
            if (!hasNameAndDomain(aEmailAddress)) {
                result = false;
            }
        } catch (final AddressException ex) {
            result = false;
        }
        return result;
    }

    private static boolean hasNameAndDomain(final String aEmailAddress) {
        final String[] tokens = aEmailAddress.split("@");
        return tokens.length == 2 && textHasContent(tokens[0]) && textHasContent(tokens[1]) && tokens[1].contains(".") && !aEmailAddress.contains(";") && !aEmailAddress.contains(" ");
    }

    public static boolean textHasContent(final String aText) {
        return (aText != null) && (aText.trim().length() > 0);
    }

    public static void setLastInjection(final long lastInjection) {
        Modelemail.lastInjection = lastInjection;
    }

    /**
     *
     * @deprecated cette méthode ne sert plus.
     * @param ctx
     */
    @Deprecated
    public void setCtx(OMContext ctx) {
        //nothing
    }

    /**
     * implementation de la methode clone() (utilisée dans le module catalogue)
     */
    @Override
    public Modelemail clone() throws CloneNotSupportedException {
        return (Modelemail) super.clone();
    }

    /**
     * Inits the.
     */
    public void init() {
        setIdModelemail((long) 0);
        setType(StringUtils.EMPTY);
        setIntitule(StringUtils.EMPTY);
        setObjet(StringUtils.EMPTY);
        setContenu(StringUtils.EMPTY);
        setPied(StringUtils.EMPTY);
        setCodeRubrique(StringUtils.EMPTY);
        setCodeRattachement(StringUtils.EMPTY);
        setCodeResponsables(StringUtils.EMPTY);
        setFormatEnvoi("0");
        setIdBandeauBas((long) 0);
        setIdBandeauHaut((long) 0);
        setDiffusionPublicVise(StringUtils.EMPTY);
        setUtilisateursVises(StringUtils.EMPTY);
        setEtat(StringUtils.EMPTY);
        setPublicInterne("0");
        setNumero(StringUtils.EMPTY);
        setActionInscription(StringUtils.EMPTY);
    }

    /**
     * Envoi d'un mailing Cette fonction fonctionne en asynchrone (lancement d'un thread).
     */
    public void envoyerMailing() {
        final Thread thread = new Thread(new ModeleMailThread(this.persistenceBean));
        thread.start();
    }

    /**
     * Renvoie la liste des paragraphes.
     *
     * @return the paragraphes
     *
     */
    public Vector<ParagrapheBean> getParagraphes() {
        final Vector<ParagrapheBean> listeParagraphes = new Vector<>();
        final String texte = getContenu();
        int indexTexte = 0;
        int indexDebutMotCle = -1;
        int indexFinParagraphe = 0;
        /* Boucle sur chaque paragraphe */
        while ((indexDebutMotCle = texte.indexOf("[paragraphe", indexTexte)) != -1) {
            ParagrapheBean paragraphe = new ParagrapheBean();
            int indexFinMotCle = texte.indexOf(']', indexDebutMotCle);
            if (indexFinMotCle != -1) {
                /* Analyse du tag paragraphe */
                final String contenuTag = texte.substring(indexDebutMotCle + 1, indexFinMotCle);
                final StringTokenizer st = new StringTokenizer(contenuTag, ";");
                int indiceToken = 0;
                while (st.hasMoreTokens()) {
                    String itemTag = st.nextToken();
                    if (indiceToken == 1) {
                        paragraphe.setLigne(Integer.parseInt(itemTag));
                    }
                    if (indiceToken == 2) {
                        paragraphe.setColonne(Integer.parseInt(itemTag));
                    }
                    if (indiceToken == 3) {
                        paragraphe.setLargeur(Integer.parseInt(itemTag));
                    }
                    indiceToken++;
                }
                /* Recherche de la fin du paragraphe */
                indexFinParagraphe = texte.indexOf("[/paragraphe]", indexDebutMotCle);
                if (indexFinParagraphe != -1) {
                    /* Stockage du paragraphe */
                    paragraphe.setContenu(texte.substring(indexFinMotCle + 1, indexFinParagraphe));
                    listeParagraphes.add(paragraphe);
                }
            }
            indexTexte = indexFinParagraphe + 12;
        }
        return listeParagraphes;
    }

    /**
     * Stocke la liste des paragraphes triÈs.
     *
     * @param listeParagraphes
     *            the _liste paragraphes
     *
     * @throws Exception
     *             the exception
     */
    public void setParagraphes(final Vector<ParagrapheBean> listeParagraphes) throws Exception {
        // Tri des paragraphes
        final TreeMap<Integer, TreeMap<Integer, ParagrapheBean>> listeLigne = new TreeMap<>();
        TreeMap<Integer, ParagrapheBean> listeColonne = null;
        for (ParagrapheBean paragraphe : listeParagraphes) {
            listeColonne = listeLigne.get(new Integer(paragraphe.getLigne()));
            if (listeColonne == null) // on crée la ligne
            {
                listeColonne = new TreeMap<Integer, ParagrapheBean>();
                listeLigne.put(paragraphe.getLigne(), listeColonne);
            }
            // controle sur l'unicite du paragraphe
            if (listeColonne.containsKey(new Integer(paragraphe.getColonne()))) {
                throw new ErreurApplicative("Ce paragraphe existe déjà, vérifiez les numéros de ligne et colonne.");
            }
            // controle sur la largeur de la ligne
            int largeurLigne = paragraphe.getLargeur();
            if (!listeColonne.isEmpty()) {
                for (ParagrapheBean col : listeColonne.values()) {
                    largeurLigne += col.getLargeur();
                }
            }
            if (largeurLigne > 100) {
                throw new ErreurApplicative("La largeur de la ligne " + paragraphe.getLigne() + " dépasse 100%.");
            }
            // on ajoute le paragraphe
            listeColonne.put(paragraphe.getColonne(), paragraphe);
        }
        // Stockage des paragraphes
        final StringBuilder contenu = new StringBuilder(4096);
        for (TreeMap<Integer, ParagrapheBean> ligne : listeLigne.values()) {
            for (ParagrapheBean paragraphe : ligne.values()) {
                contenu.append("[paragraphe;").append(paragraphe.getLigne()).append(";").append(paragraphe.getColonne()).append(";").append(paragraphe.getLargeur()).append("]").append(paragraphe.getContenu()).append("[/paragraphe]");
            }
        }
        setContenu(contenu.toString());
    }

    /**
     * Gets the destinataires finaux.
     *
     * @return the destinataires finaux
     */
    public ArrayList<String> getDestinatairesFinaux() {
        return listeDestinatairesFinaux;
    }

    /**
     * Sets the destinataires finaux.
     *
     * @param listeUtilisateurs
     *            the new destinataires finaux
     */
    public void setDestinatairesFinaux(final ArrayList<String> listeUtilisateurs) {
        listeDestinatairesFinaux = listeUtilisateurs;
    }

    /**
     * Gets the date purge.
     *
     * @return the date purge
     */
    public Date getDatePurge() {
        return datePurge;
    }

    /**
     * Sets the date purge.
     *
     * @param _datePurge
     *            the new date purge
     */
    public void setDatePurge(final Date _datePurge) {
        datePurge = _datePurge;
    }

    /**
     * sélectionne un modèle en fonction de soon état (aperçu ou pas).
     *
     * @param _etat
     *            etat : vide ou 1 pour aperçu
     *
     * @return int nombre de résultats
     *
     */
    public int selectParEtat(final String _etat) {
        return select((long) 0, "", "", "", _etat, "", "");
    }

    /**
     * selection d'un modèle de newsletter selon des critères choisis.
     *
     * @param idModelemail
     *            Long identifiant du modèle
     * @param intitule
     *            String intitulé du modèle
     * @param codeResponsable
     *            String code du responsable (code utilisateur)
     * @param type
     *            String code du type de modèle de newsletter
     * @param etat
     *            String etat de la newsletter (apercu ou pas)
     * @param codeRubrique
     *            String code de la rubrique
     * @param codeRattachement
     *            String code de rattachement
     *
     * @return int nombre de résultats
     *
     */
    public int select(final Long idModelemail, final String intitule, String codeResponsable, final String type, final String etat, final String codeRubrique, final String codeRattachement) {
        final RequeteSQL requeteSelect = new RequeteSQL();
        final ClauseWhere where = new ClauseWhere();
        final ClauseOrderBy orderBy = new ClauseOrderBy("INTITULE", SensDeTri.ASC);
        if (idModelemail != 0) {
            where.setPremiereCondition(ConditionHelper.egal("ID_MODELE_MAIL", idModelemail, TypeOperande.LONG));
        }
        if (StringUtils.isNotEmpty(intitule)) {
            where.and(ConditionHelper.rechercheMots("INTITULE", intitule));
        }
        if (StringUtils.isNotEmpty(codeResponsable) && !"0000".equals(codeResponsable)) {
            codeResponsable = EscapeString.escapeSql(codeResponsable);
            where.and(ConditionHelper.genericConditionSurColonne("CODE_RESPONSABLES", "'(^|.*;)" + codeResponsable + "($|;.*)'", TypeOperande.NON_ECHAPABLE, Operateur.REGEXP));
        }
        if (StringUtils.isNotEmpty(type) && !"0000".equals(type)) {
            where.and(ConditionHelper.egalVarchar("TYPE", type));
        }
        if (StringUtils.isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("CODE_RATTACHEMENT", codeRattachement));
        }
        if (StringUtils.isNotEmpty(codeRubrique) && !"0000".equals(codeRubrique)) {
            where.and(ConditionHelper.getConditionRubrique("CODE_RUBRIQUE", codeRubrique));
        }
        where.and(ConditionHelper.egalVarchar("ETAT", etat));
        requeteSelect.where(where).orderBy(orderBy);
        return select(requeteSelect.formaterRequete());
    }

    /**
     * Formate en HTML le champ pied
     *
     * @return le champ Pied formaté en HTML
     * @throws Exception
     * @deprecated le formateur à utiliser n'est pas le bon. Il faut utiliser ne NewsletterWebFmt
     */
    @Deprecated
    public String getFormatedPied() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPied());
    }

    @Override
    public List<String> getCodesLibelleMedia() {
        // 0103 Bandeau newsletter  est un type photo de libelle utilise dans la mediatheque
        return Collections.singletonList("0103");
    }

    @Override
    public void checkCodesLibelleMedia(final long idMedia) throws ErreurApplicative {
        // controle bandeau newsletter
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        serviceModelemail.checkCodesLibelleMedia(idMedia);
    }



    public Long getIdModelemail() {
        return persistenceBean.getId();
    }

    public void setIdModelemail(Long idModeleMail) {
        persistenceBean.setId(idModeleMail);
    }

    public String getType() {
        return persistenceBean.getType();
    }

    public void setType(String type) {
        persistenceBean.setType(type);
    }

    public String getEtat() {
        return persistenceBean.getEtat();
    }

    public void setEtat(String etat) {
        persistenceBean.setEtat(etat);
    }

    public String getPublicInterne() {
        return persistenceBean.getPublicInterne();
    }

    public void setPublicInterne(String public_interne) {
        persistenceBean.setPublicInterne(public_interne);
    }

    public String getNumero() {
        return persistenceBean.getNumero();
    }

    public void setNumero(String numero) {
        persistenceBean.setNumero(numero);
    }

    public String getIntitule() {
        return persistenceBean.getIntitule();
    }

    public void setIntitule(String intitule) {
        persistenceBean.setIntitule(intitule);
    }

    public String getCodeResponsables() {
        return persistenceBean.getCodeResponsables();
    }

    public void setCodeResponsables(String codeResponsables) {
        persistenceBean.setCodeResponsables(codeResponsables);
    }

    public String getObjet() {
        return persistenceBean.getObjet();
    }

    public void setObjet(String objet) {
        persistenceBean.setObjet(objet);
    }

    public String getContenu() {
        return persistenceBean.getContenu();
    }

    public void setContenu(String contenu) {
        persistenceBean.setContenu(contenu);
    }

    public String getPied() {
        return persistenceBean.getPied();
    }

    public void setPied(String pied) {
        persistenceBean.setPied(pied);
    }

    public String getCodeRubrique() {
        return persistenceBean.getCodeRubrique();
    }

    public void setCodeRubrique(String codeRubrique) {
        persistenceBean.setCodeRubrique(codeRubrique);
    }

    public String getCodeRattachement() {
        return persistenceBean.getCodeRattachement();
    }

    public void setCodeRattachement(String codeRattachement) {
        persistenceBean.setCodeRattachement(codeRattachement);
    }

    @GetterAnnotation(isIdMedia = true)
    public Long getIdBandeauBas() {
        return persistenceBean.getIdBandeauBas();
    }

    @SetterAnnotation(isIdMedia = true)
    public void setIdBandeauBas(Long idBandeauBas) {
        persistenceBean.setIdBandeauBas(idBandeauBas);
    }

    @GetterAnnotation(isIdMedia = true)
    public Long getIdBandeauHaut() {
        return persistenceBean.getIdBandeauHaut();
    }

    @SetterAnnotation(isIdMedia = true)
    public void setIdBandeauHaut(Long idBandeauHaut) {
        persistenceBean.setIdBandeauHaut(idBandeauHaut);
    }

    public String getDiffusionPublicVise() {
        return persistenceBean.getDiffusionPublicVise();
    }

    public void setDiffusionPublicVise(String diffusionPublicVise) {
        persistenceBean.setDiffusionPublicVise(diffusionPublicVise);
    }

    public String getUtilisateursVises() {
        return persistenceBean.getUtilisateursVises();
    }

    public void setUtilisateursVises(String utilisateursVises) {
        persistenceBean.setUtilisateursVises(utilisateursVises);
    }

    public String getFormatEnvoi() {
        return persistenceBean.getFormatEnvoi();
    }

    public void setFormatEnvoi(String formatEnvoi) {
        persistenceBean.setFormatEnvoi(formatEnvoi);
    }

    public String getActionInscription() {
        return persistenceBean.getActionInscription();
    }

    public void setActionInscription(String actionInscription) {
        persistenceBean.setActionInscription(actionInscription);
    }
}
