/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.processus;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.kosmos.newsletter.util.NewsletterUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kportal.extension.ExtensionHelper;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.utils.ContexteUtil;

public class RenderModeleMail extends ProcessusBean {

    /** The Constant ACTION_NEWSLETTER. */
    private static final String ACTION_NEWSLETTER = "NEWSLETTER";

    /** The Constant ECRAN_NEWSLETTER. */
    private static final String ECRAN_NEWSLETTER = "NEWSLETTER";

    /** The Constant ACTION_MAILING. */
    private static final String ACTION_MAILING = "MAILING";

    /** The Constant ECRAN_MAILING. */
    private static final String ECRAN_MAILING = "MAILING";

    public RenderModeleMail(final InfoBean infoBean) {
        super(infoBean);
    }

    @Override
    protected boolean traiterAction() throws Exception {
        // initalisation objets métiers
        etat = EN_COURS;
        ecranLogique = infoBean.getEcranLogique();
        action = infoBean.getActionUtilisateur();
        if (action.equals(ACTION_NEWSLETTER)) {
            ecranLogique = ECRAN_NEWSLETTER;
            // pour forcer l'affichage dans le site de la newsletter
            if (StringUtils.isNotEmpty(infoBean.getString("CODE_RUBRIQUE"))) {
                final ServiceInfosSiteProcessus serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
                final InfosSite infosSite = serviceInfosSite.determineSite(infoBean.getString("CODE_RUBRIQUE"), true);
                ContexteUtil.getContexteUniv().setInfosSite(infosSite);
            }
            infoBean.setEcranPhysique(ExtensionHelper.getTemplateExtension(NewsletterUtil.ID_EXTENSION, NewsletterUtil.getProperty("newsletter.template")));
        }
        if (action.equals(ACTION_MAILING)) {
            ecranLogique = ECRAN_MAILING;
            infoBean.setEcranPhysique(ExtensionHelper.getTemplateExtension(NewsletterUtil.ID_EXTENSION, NewsletterUtil.getProperty("mailing.template")));
        }
        infoBean.setEcranLogique(ecranLogique);
        return etat == FIN;
    }
}
