/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.processus;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;
import java.util.regex.Pattern;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.jsbsoft.jtf.upload.UploadedFile;
import com.kosmos.controllers.SGController;
import com.kosmos.newsletter.bean.MailingBean;
import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.om.Modelemail;
import com.kosmos.newsletter.service.ServiceMailing;
import com.kosmos.newsletter.service.ServiceModelemail;
import com.kosmos.newsletter.util.GestionnaireMailing;
import com.kosmos.newsletter.util.GestionnaireMailing.Newsletter;
import com.kosmos.newsletter.util.NewsletterUtil;
import com.kosmos.newsletter.util.ThreadMailing;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.bean.GroupeDsiBean;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.ParagrapheBean;
import com.univ.objetspartages.om.ParagrapheContenuHelper;
import com.univ.objetspartages.om.Perimetre;
import com.univ.objetspartages.om.PermissionBean;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.services.ServiceGroupeDsi;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.services.ServiceUser;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MediaUtils;
import com.univ.objetspartages.util.RessourceUtils;
import com.univ.utils.Chaine;

/**
 * processus saisie ModeleMail.
 */
public class SaisieModeleMail extends ProcessusBean {

    private static final Logger LOG = LoggerFactory.getLogger(SaisieModeleMail.class);

    /** The Constant MODE_AJOUT. */
    private static final int MODE_AJOUT = 0;

    /** The Constant MODE_MODIFICATION. */
    private static final int MODE_MODIFICATION = 1;

    /** The Constant MODE_SUPPRESSION. */
    private static final int MODE_SUPPRESSION = 2;

    /** The Constant MODE_RECHERCHE. */
    private static final int MODE_RECHERCHE = 3;

    /** The Constant MODE_ENVOI. */
    private static final int MODE_ENVOI = 4;

    /** The Constant MODE_SUIVI. */
    private static final int MODE_SUIVI = 5;

    /** The Constant ECRAN_PRINCIPAL. */
    private static final String ECRAN_PRINCIPAL = "PRINCIPAL";

    /** The Constant ECRAN_INSERTION. */
    private static final String ECRAN_INSERTION = "INSERTION";

    /** The Constant ECRAN_LISTE. */
    private static final String ECRAN_LISTE = "LISTE";

    /** The Constant ECRAN_MENU. */
    private static final String ECRAN_MENU = "MENU";

    /** The Constant ECRAN_ARCHIVE. */
    private static final String ECRAN_ARCHIVE = "ARCHIVE";

    /** The Constant ECRAN_CONFIRMATION. */
    private static final String ECRAN_CONFIRMATION = "CONFIRMATION";

    /** The Constant ECRAN_ACCUEIL. */
    private static final String ECRAN_ACCUEIL = "ACCUEIL";

    /** The Constant ECRAN_LISTEDESTINATAIRES. */
    private static final String ECRAN_LISTEDESTINATAIRES = "LISTEDESTINATAIRES";

    /** The Constant ECRAN_ENVOI. */
    private static final String ECRAN_ENVOI = "ENVOI";

    /** The Constant ECRAN_SUIVI. */
    private static final String ECRAN_SUIVI = "SUIVI";

    private final ServiceUser serviceUser;

    private final ServiceMailing serviceMailing;

    private final ServiceMedia serviceMedia;

    private final ServiceRessource serviceRessource;

    private final ServiceModelemail serviceModelemail;

    /** The mode. */
    private int mode = -1;

    /** The autorisations. */
    private AutorisationBean autorisations;

    /** The modele mail. */
    private ModelemailBean modeleMail = null;

    /**
     * Constructeur.
     *
     * @param ciu
     *            com.jsbsoft.jtf.core.InfoBean
     */
    public SaisieModeleMail(final InfoBean ciu) {
        super(ciu);
        serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
        serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
        serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
    }

    /**
     * Initialisation du processus.
     *
     * @throws Exception
     *             the exception
     */
    private void initialisation() throws Exception {
        // initalisation objets métiers
        etat = EN_COURS;
        this.getDatas().put("AUTORISATIONS", autorisations);
        if ("AJOUTER".equals(action)) {
            if (!autorisations.possedePermission(new PermissionBean("TECH", "new", "C"))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE"));
            }
            mode = MODE_AJOUT;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
            preparerPRINCIPAL();
        } else if ("MODIFIER".equals(action)) {
            mode = MODE_MODIFICATION;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_MODIF);
            preparerMENU();
        } else if ("SUPPRIMER".equals(action)) {
            mode = MODE_SUPPRESSION;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_SUPPRESSION);
            preparerMENU();
        } else if ("INSERER".equals(action)) {
            mode = MODE_RECHERCHE;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_RECHERCHE);
            preparerLISTE();
        } else if ("ARCHIVE_TOOLBOX".equals(action)) {
            mode = MODE_RECHERCHE;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_RECHERCHE);
            ecranLogique = "ARCHIVE_TOOLBOX";
        } else if ("ENVOYER".equals(action)) {
            mode = MODE_ENVOI;
            action = InfoBean.ACTION_SUITE;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_RECHERCHE);
            infoBean.set("REMPLACER_MODELE", "1");
            preparerMENU();
            final String titreEcran = NewsletterUtil.getMessage("BO_ENVOI_NEWSLETTER_1");
            infoBean.setTitreEcran(String.format(titreEcran, modeleMail.getIntitule()));
        } else if ("LISTEDESTINATAIRES".equals(action)) {
            preparerLISTEDESTINATAIRES();
        } else if ("SUIVRE".equals(action)) {
            mode = MODE_SUIVI;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_RECHERCHE);
            preparerSUIVI();
        } else if ("ACCUEIL".equals(action) || "RECHERCHER".equals(action)) {
            ecranLogique = ECRAN_ACCUEIL;
        }
    }

    /**
     * Affichage de la liste des ModeleMails.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerLISTE() throws Exception {
        int i = 0;
        ecranLogique = ECRAN_LISTE;
        if (infoBean.get("MODE") != null) {
            try {
                mode = Integer.parseInt(infoBean.getString("MODE"));
            } catch (final Exception ignored) {
            }
        }
        //recherche pour insertion toolbox
        if (mode == MODE_RECHERCHE) {
            ecranLogique = ECRAN_INSERTION;
        }
        final String intitule = StringUtils.defaultString(infoBean.getString("INTITULE"));
        final String codeResponsables = StringUtils.defaultString(infoBean.getString("CODE_RESPONSABLES"));
        final String codeRattachement = StringUtils.defaultString(infoBean.getString("CODE_RATTACHEMENT"));
        final String codeRubrique = StringUtils.defaultString(infoBean.getString("CODE_RUBRIQUE"));
        final String type = StringUtils.defaultString(infoBean.getString("TYPE"));
        List<ModelemailBean> searchResult = serviceModelemail.select((long) 0, intitule, codeResponsables, type, "", codeRubrique, codeRattachement);
        final int count = searchResult.size();
        /* The liste identifiant. */
        final Long[] listeIdentifiant = new Long[count];
        for (ModelemailBean currentModel : searchResult) {
            boolean inserer = true;
            if (mode == MODE_SUPPRESSION || infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_SUPPRESSION)) {
                inserer = autorisations.possedePermission(new PermissionBean("TECH", "new", "S"), new Perimetre(currentModel.getCodeRattachement(), currentModel.getCodeRubrique(), "*", "*", ""));
            } else if (mode == MODE_MODIFICATION || infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_MODIF)) {
                inserer = autorisations.possedePermission(new PermissionBean("TECH", "new", "M"), new Perimetre(currentModel.getCodeRattachement(), currentModel.getCodeRubrique(), "*", "*", ""));
            } else if (mode == MODE_ENVOI) {
                inserer = autorisations.possedePermission(new PermissionBean("TECH", "new", "E"), new Perimetre(currentModel.getCodeRattachement(), currentModel.getCodeRubrique(), "*", "*", ""));
            }
            if (inserer) {
                infoBean.set("INTITULE#" + i, currentModel.getIntitule());
                listeIdentifiant[i] = currentModel.getId();
                i++;
            }
        }
        if (i == 0) {
            infoBean.set("MESSAGE_ERREUR", "Aucun enregistrement sélectionné");
            preparerMENU();
        }
        infoBean.set("LISTE_NB_ITEMS", i);
        infoBean.set("LISTE_IDENTIFIANTS", listeIdentifiant);
    }

    /**
     * Affichage de l'écran de saisie d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerPRINCIPAL() throws Exception {
        final ServiceGroupeDsi serviceGroupeDsi = ServiceManager.getServiceForBean(GroupeDsiBean.class);
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        if (mode == MODE_ENVOI) {
            ecranLogique = ECRAN_ENVOI;
        } else {
            ecranLogique = ECRAN_PRINCIPAL;
        }
        infoBean.set("FORMAT_ENVOI", modeleMail.getFormatEnvoi());
        infoBean.set("RADIO0", "HTML");
        infoBean.set("RADIO1", "Texte");
        if (mode == MODE_AJOUT) {
            modeleMail.setPublicInterne("0");
        }
        infoBean.set("PUBLIC_INTERNE", modeleMail.getPublicInterne());
        infoBean.set("NUMERO", modeleMail.getNumero());
        final String action_inscription = modeleMail.getActionInscription();
        if (StringUtils.isNotEmpty(action_inscription)) {
            final String[] listeActions = action_inscription.split(";");
            for (final String listeAction : listeActions) {
                if ("U".equals(listeAction)) {
                    infoBean.set("INSCRIPTION_1", "1");
                } else if (listeAction.startsWith("G") && listeAction.contains(",")) {
                    infoBean.set("INSCRIPTION_2", "1");
                    final String codegroupe = listeAction.substring(listeAction.indexOf(",") + 1);
                    final String libellegroupe = serviceGroupeDsi.getIntitules(codegroupe);
                    infoBean.set("LIBELLE_GROUPE_INSCRIPTION_2", libellegroupe);
                    infoBean.set("GROUPE_INSCRIPTION_2", codegroupe);
                }
            }
        }
        infoBean.set("LISTE_TYPES_NEWSLETTER", LabelUtils.getLabelCombo("01", LangueUtil.getDefaultLocale()));
        infoBean.set("TYPE", modeleMail.getType());
        if (StringUtils.isNotEmpty(modeleMail.getCodeRubrique())) {
            infoBean.set("LIBELLE_TYPE", LabelUtils.getLibelle("01", modeleMail.getType(), LangueUtil.getDefaultLocale()));
        } else {
            infoBean.set("LIBELLE_TYPE", LabelUtils.getLibelle("01", modeleMail.getType(), LangueUtil.getDefaultLocale()));
        }
        infoBean.set("INTITULE", modeleMail.getIntitule());
        infoBean.set("CODE_RESPONSABLES", modeleMail.getCodeResponsables());
        infoBean.set("LIBELLE_RESPONSABLE", modeleMail.getLibelleResponsable());
        infoBean.set("LIBELLE_CODE_RESPONSABLES", StringUtils.join(NewsletterUtil.getAllLibelleUtilisateurFromCode(modeleMail.getCodeResponsables()),";"));
        infoBean.set("UTILISATEURS_VISES", modeleMail.getUtilisateursVises());
        infoBean.set("LIBELLE_UTILISATEURS_VISES", StringUtils.join(NewsletterUtil.getAllLibelleUtilisateurFromCode(modeleMail.getUtilisateursVises()),";"));
        infoBean.set("OBJET", modeleMail.getObjet());
        infoBean.set("LISTE_BANDEAUX", MediaUtils.getMediasAsMap(serviceMedia.getByTypeAndTypeMedia("photo", "0103")));
        infoBean.set("ID_BANDEAU_BAS", modeleMail.getIdBandeauBas());
        infoBean.set("ID_BANDEAU_HAUT", modeleMail.getIdBandeauHaut());
        final MediaBean mediaBandeauHaut = serviceMedia.getById(modeleMail.getIdBandeauHaut());
        final MediaBean mediaBandeauBas = serviceMedia.getById(modeleMail.getIdBandeauBas());
        if (mediaBandeauHaut != null) {
            infoBean.set("LIBELLE_BANDEAU_HAUT", StringUtils.defaultString(mediaBandeauHaut.getTitre()));
        }
        if (mediaBandeauBas != null) {
            infoBean.set("LIBELLE_BANDEAU_BAS", StringUtils.defaultString(mediaBandeauBas.getTitre()));
        }
        infoBean.set("CODE_RUBRIQUE", modeleMail.getCodeRubrique());
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(modeleMail.getCodeRubrique());
        String rubriqueLabel = StringUtils.EMPTY;
        if (rubriqueBean != null) {
            rubriqueLabel = rubriqueBean.getIntitule();
        }
        infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueLabel);
        //     Affichage des restrictions dans l'arbre des rubriques et structures
        // Utilisé dans les fonctions UnivFmt.insererSaisieZone rubrique et structure
        infoBean.set("GRS_PERMISSION_TYPE", "TECH");
        infoBean.set("GRS_PERMISSION_OBJET", "new");
        if (InfoBean.ETAT_OBJET_CREATION.equals(infoBean.getEtatObjet())) {
            infoBean.set("GRS_PERMISSION_ACTION", "C");
        }
        if (InfoBean.ETAT_OBJET_MODIF.equals(infoBean.getEtatObjet())) {
            infoBean.set("GRS_PERMISSION_ACTION", "M");
        }
        infoBean.set("GRS_FILTRE_ARBRE_NOM_CODE_RATTACHEMENT", "CODE_RATTACHEMENT");
        infoBean.set("GRS_FILTRE_ARBRE_NOM_CODE_RUBRIQUE", "CODE_RUBRIQUE");
        // Sauvegarde des valeurs avant modification
        infoBean.set("GRS_SAUVEGARDE_CODE_RUBRIQUE", modeleMail.getCodeRubrique());
        infoBean.set("GRS_SAUVEGARDE_CODE_RATTACHEMENT", modeleMail.getCodeRattachement());
        infoBean.set("NOM_CODE_RATTACHEMENT", "");
        infoBean.set("CODE_RATTACHEMENT", modeleMail.getCodeRattachement());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(modeleMail.getCodeRattachement(), LangueUtil.getLangueLocale(LangueUtil.getDefaultLocale())));
        /* Fichiers joints */
        String totalFichierJoint = "";
        String modeFichier = "";
        final List<RessourceBean> listeFichier = serviceRessource.getByCodeParent(modeleMail.getId() + ",TYPE=NEWSLETTER");
        if (listeFichier.size() > 0) {
            for (int j = 0; j < listeFichier.size(); j++) {
                final RessourceBean f = listeFichier.get(j);
                if (j == 0) {
                    modeFichier = "multiple_" + RessourceUtils.getTypeRessource(f);
                }
                if (totalFichierJoint.length() > 0) {
                    totalFichierJoint += "|";
                }
                totalFichierJoint += f.getId().toString() + ";" + RessourceUtils.getSource(f) + ";" + RessourceUtils.getFormat(f) + ";" + RessourceUtils.getLegende(f) + ";" + DateFormat.getDateInstance(DateFormat.SHORT, this.getLocale()).format(RessourceUtils.getDateCreation(f)) + ";" + RessourceUtils.getPoids(f);
            }
            infoBean.set("TOTAL_FICHIER_MULTIPLE_1", totalFichierJoint);
            infoBean.set("MODE_FICHIER_MULTIPLE_1", modeFichier);
        }
        // MAX_SIZE_FILE
        String poidsFichiers = NewsletterUtil.getProperty("newsletter.fichiers_joints.poids_max");
        if (poidsFichiers == null) {
            poidsFichiers = "1024";
        }
        infoBean.set(UploadedFile.KEY_MAX_FILE_SIZE, poidsFichiers);
        /* Contenu */
        //    en mode création, on créé un premier paragraphe, ou bien quand aucun paragraphe n'a été saisi.
        if (InfoBean.ETAT_OBJET_CREATION.equals(infoBean.getEtatObjet()) || StringUtils.isEmpty(modeleMail.getContenu())) {
            final int nbItems = 0;
            infoBean.set("CONTENU_" + nbItems, "");
            infoBean.setInt("LIGNE_" + nbItems, nbItems + 1);
            infoBean.setInt("COLONNE_" + nbItems, 1);
            infoBean.setInt("LARGEUR_" + nbItems, 100);
            infoBean.set("PARAGRAPHES_NB_ITEMS", "" + (nbItems + 1));
        } else {
            int i = 0;
            for (final ParagrapheBean paragraphe : ParagrapheContenuHelper.getParagraphes(modeleMail.getContenu())) {
                infoBean.set("CONTENU_" + i, paragraphe.getContenu());
                infoBean.setInt("LIGNE_" + i, paragraphe.getLigne());
                infoBean.setInt("COLONNE_" + i, paragraphe.getColonne());
                infoBean.setInt("LARGEUR_" + i, paragraphe.getLargeur());
                i++;
            }
            infoBean.set("PARAGRAPHES_NB_ITEMS", "" + i);
        }
        infoBean.set("PIED", modeleMail.getPied());
        rafraichirPRINCIPAL();
        String chainePublicVise =StringUtils.defaultString(modeleMail.getDiffusionPublicVise());
        if (chainePublicVise.startsWith("[*]")) {
            // les valeurs [*]; ne devraient plus exister, elles ont été supprimées avec la sortie du collab
            chainePublicVise = chainePublicVise.substring(3);
            if (chainePublicVise.startsWith(";")) {
                chainePublicVise = chainePublicVise.substring(1);
            }
            LOG.warn("Erreur sur modeleMail.getDiffusionPublicVise() : " + chainePublicVise);
        }
        ControleurUniv.formaterPublicViseDsiDansInfoBean(infoBean, this, chainePublicVise, "PUBLIC_VISE_DSI");
        if (StringUtils.isNotBlank(modeleMail.getIntitule())) {
            infoBean.setTitreEcran(modeleMail.getIntitule());
        }
    }

    /**
     * Point d'entrée du processus.
     *
     * @return true, if traiter action
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public boolean traiterAction() throws Exception {
        try {
            ecranLogique = infoBean.getEcranLogique();
            action = infoBean.getActionUtilisateur();
            autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
            if (autorisations == null || !NewsletterUtil.isAdministrateurNewsletter(autorisations)) {
                infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
                infoBean.setEcranLogique("LOGIN");
            } else {
                modeleMail = new ModelemailBean();
                if (ecranLogique == null) {
                    initialisation();
                } else if (ECRAN_LISTE.equals(ecranLogique)) {
                    traiterLISTE();
                } else if (ECRAN_MENU.equals(ecranLogique)) {
                    preparerLISTE();
                } else if (ECRAN_PRINCIPAL.equals(ecranLogique) || ECRAN_ENVOI.equals(ecranLogique) || "1".equals(infoBean.getString("APERCU"))) {
                    //on renseigne le mode envoi pour que cette info soit conservée dans le cas où il y a un ajout de zone de contenu
                    if (ecranLogique.equals(ECRAN_ENVOI)) {
                        mode = MODE_ENVOI;
                    }
                    //on doit renseigner le mode pour les contrôles sur périmètres
                    if (InfoBean.ETAT_OBJET_CREATION.equals(infoBean.getEtatObjet())) {
                        mode = MODE_AJOUT;
                    } else if (InfoBean.ETAT_OBJET_MODIF.equals(infoBean.getEtatObjet())) {
                        mode = MODE_MODIFICATION;
                    }
                    traiterPRINCIPAL();
                } else if (ECRAN_CONFIRMATION.equals(ecranLogique)) {
                    if (action.equals("LISTEDESTINATAIRES")) {
                        preparerLISTEDESTINATAIRES();
                    } else {
                        traiterCONFIRMATION();
                    }
                } else if (ECRAN_SUIVI.equals(ecranLogique)) {
                    traiterSUIVI();
                }
                //placer l'état dans le composant d'infoBean
                infoBean.setEcranLogique(ecranLogique);
                infoBean.set("MODE", "" + mode);
            }
        } catch (final Exception e) {
            LOG.error("erreur lors du traitement du processus", e);
            infoBean.addMessageErreur(e.toString());
        }
        // On continue si on n'est pas à la FIN !!!
        return etat == FIN;
    }

    /**
     * Traitement de l'écran de sélection d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterLISTE() throws Exception {
        final String szIndice = infoBean.getString("LISTE_INDICE");
        Long idM;
        // on vient de la recherche
        if (szIndice != null) {
            idM = new Long(infoBean.getString("indice" + szIndice));
            // on vient de la liste
        } else {
            idM = new Long(infoBean.getString("ID_MODELEMAIL"));
        }
        modeleMail = serviceModelemail.getById(idM);
        if (InfoBean.ACTION_SUITE.equals(action) || InfoBean.ACTION_MODIFIER.equals(action)) {
            if (InfoBean.ACTION_SUITE.equals(action)) {
                mode = MODE_ENVOI;
            }
            if (InfoBean.ACTION_MODIFIER.equals(action)) {
                mode = MODE_MODIFICATION;
            }
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_MODIF);
            preparerPRINCIPAL();
        } else if (InfoBean.ACTION_SUPPRIMER.equals(action)) {
            deleteModeleMail();
        } else if (InfoBean.ACTION_ANNULER.equals(action)) {
            infoBean.set("ID_MODELE_MAIL", null);
            etat = FIN;
        }
    }

    private void deleteModeleMail() throws Exception {
        mode = MODE_SUPPRESSION;
        //suppression des modeles temporaires utilises lors des envois
        final List<ModelemailBean> modelesByState = serviceModelemail.getByState("2_" + modeleMail.getId());
        for (ModelemailBean modeleToDelete : modelesByState) {
            serviceRessource.deleteFiles(modeleToDelete.getId() + ",TYPE=NEWSLETTER");
            serviceModelemail.delete(modeleToDelete.getId());
        }
        //suppression du modele original
        serviceModelemail.delete(modeleMail.getId());
        final String confirmation = String.format(MessageHelper.getMessage(infoBean.getNomExtension(), "CONFIRMATION_SUPPRESSION_MODELEMAIL"), modeleMail.getIntitule());
        infoBean.addMessageConfirmation(confirmation);
        etat = FIN;
    }

    /**
     * Preparer menu.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerMENU() throws Exception {
        if (infoBean.get("ID_MODELEMAIL") != null && !infoBean.get("ID_MODELEMAIL").equals("")) {
            ecranLogique = ECRAN_LISTE;
            infoBean.setEcranLogique(ecranLogique);
            traiterLISTE();
        } else {
            infoBean.set("INTITULE", "");
            ecranLogique = ECRAN_MENU;
            preparerDonneesRecherche();
        }
    }

    /**
     * Gestion de l'écran de saisie d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterPRINCIPAL() throws Exception {
        final boolean apercu = "1".equals(infoBean.getString("APERCU"));
        //JSS 20040419 : test permissions
        if (mode == MODE_AJOUT && !apercu) {
            if (!autorisations.possedePermission(new PermissionBean("TECH", "new", "C"), new Perimetre(infoBean.getString("CODE_RATTACHEMENT"), infoBean.getString("CODE_RUBRIQUE"), "*", "*", ""))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_CREATION_HORS_PERIMETRE"));
            }
        }
        if (mode == MODE_MODIFICATION && !apercu) {
            if (!autorisations.possedePermission(new PermissionBean("TECH", "new", "M"), new Perimetre(infoBean.getString("CODE_RATTACHEMENT"), infoBean.getString("CODE_RUBRIQUE"), "*", "*", ""))) {
                throw new ErreurApplicative(MessageHelper.getCoreMessage("BO_OPERATION_INTERDITE") + " : " + MessageHelper.getCoreMessage("AUTORISATION_MODIFICATION_HORS_PERIMETRE"));
            }
        }
        if (!InfoBean.ETAT_OBJET_CREATION.equals(infoBean.getEtatObjet())) {
            modeleMail = serviceModelemail.getById(new Long(infoBean.getString("ID_MODELEMAIL")));
        }
        if (action.indexOf("SUPPRIMER_PARAGRAPHE") == 0) {
            /* Suppression de l'élément courant */
            final int indiceDiese = action.indexOf("#");
            final int indice = Integer.parseInt(action.substring(indiceDiese + 1));
            final int nbItems = Integer.parseInt(infoBean.getString("PARAGRAPHES_NB_ITEMS"));
            for (int j = indice; j < nbItems - 1; j++) {
                infoBean.set("CONTENU_" + j, infoBean.getString("CONTENU_" + (j + 1)));
                // Style magazine
                infoBean.setInt("LIGNE_" + j, infoBean.getInt("LIGNE_" + (j + 1)));
                infoBean.setInt("COLONNE_" + j, infoBean.getInt("COLONNE_" + (j + 1)));
                infoBean.setInt("LARGEUR_" + j, infoBean.getInt("LARGEUR_" + (j + 1)));
            }
            infoBean.set("PARAGRAPHES_NB_ITEMS", "" + (nbItems - 1));
            infoBean.setActionUtilisateur("SUPPRIMER_PARAGRAPHE");
            rafraichirPRINCIPAL();
        } else if ("AJOUTER_PARAGRAPHE".equals(action)) {
            final int nbItems = Integer.parseInt(infoBean.getString("PARAGRAPHES_NB_ITEMS"));
            infoBean.set("CONTENU_" + nbItems, "");
            // Style magazine
            infoBean.setInt("LIGNE_" + nbItems, nbItems + 1);
            infoBean.setInt("COLONNE_" + nbItems, 1);
            infoBean.setInt("LARGEUR_" + nbItems, 100);
            infoBean.set("PARAGRAPHES_NB_ITEMS", "" + (nbItems + 1));
            rafraichirPRINCIPAL();
        } else if (InfoBean.ACTION_VALIDER.equals(action)
            //si l'utilisateur n'a pas le droit de modifier, aucune données n'a été saisie
            && (InfoBean.ETAT_OBJET_CREATION.equals(infoBean.getEtatObjet()) || autorisations.possedePermission(new PermissionBean("TECH", "new", "M"), new Perimetre(modeleMail.getCodeRattachement(), modeleMail.getCodeRubrique(), "*", "*", "")))) {
            /* Stockage des paragraphes dans un vecteur */
            final int nbItems = Integer.parseInt(infoBean.getString("PARAGRAPHES_NB_ITEMS"));
            final Collection<ParagrapheBean> listeParagraphes = new ArrayList<>();
            for (int i = 0; i < nbItems; i++) {
                final ParagrapheBean paragraphe = new ParagrapheBean();
                // Le passage en mode magazine implique
                // une réinitialisation
                if (infoBean.get("LIGNE_" + i) != null) {
                    paragraphe.setLigne(infoBean.getInt("LIGNE_" + i));
                } else {
                    paragraphe.setLigne(i + 1);
                }
                if (infoBean.get("COLONNE_" + i) != null) {
                    paragraphe.setColonne(infoBean.getInt("COLONNE_" + i));
                } else {
                    paragraphe.setColonne(1);
                }
                if (infoBean.get("LARGEUR_" + i) != null) {
                    paragraphe.setLargeur(infoBean.getInt("LARGEUR_" + i));
                } else {
                    paragraphe.setLargeur(100);
                }
                paragraphe.setContenu(infoBean.getString("CONTENU_" + i));
                listeParagraphes.add(paragraphe);
            }
            modeleMail.setContenu(ParagrapheContenuHelper.formateParagraphes(listeParagraphes));
            modeleMail.setPied((String) infoBean.get("PIED"));
            if (!apercu && StringUtils.isBlank(infoBean.getString("INTITULE"))) {
                throw new ErreurApplicative("La zone Intitulé est obligatoire");
            }
            modeleMail.setIntitule((String) infoBean.get("INTITULE"));
            modeleMail.setObjet((String) infoBean.get("OBJET"));
            modeleMail.setCodeRattachement((String) infoBean.get("CODE_RATTACHEMENT"));
            modeleMail.setCodeRubrique((String) infoBean.get("CODE_RUBRIQUE"));
            modeleMail.setIdBandeauBas(new Long(infoBean.getString("ID_BANDEAU_BAS")));
            modeleMail.setIdBandeauHaut(new Long(infoBean.getString("ID_BANDEAU_HAUT")));
            final String mailFrom = (String) infoBean.get("CODE_RESPONSABLES");
            if (mailFrom.length() > 0) {
                if (!Pattern.matches("[_A-Za-z0-9-]+(\\.[_A-Za-z0-9-]+)*@[A-Za-z0-9-]+(\\.[A-Za-z0-9-]+)*(\\.[_A-Za-z0-9-]+)", mailFrom)) {
                    throw new ErreurApplicative(NewsletterUtil.getMessage("JTF_ERR_FMT_EMAIL"));
                }
            }
            final String mailFromLibelle = StringUtils.defaultString((String) infoBean.get("LIBELLE_RESPONSABLE"));
            modeleMail.setCodeResponsables(mailFrom);
            modeleMail.setLibelleResponsable(mailFromLibelle);
            modeleMail.setType((String) infoBean.get("TYPE"));
            modeleMail.setPublicInterne((String) infoBean.get("PUBLIC_INTERNE"));
            modeleMail.setNumero((String) infoBean.get("NUMERO"));
            modeleMail.setUtilisateursVises((String) infoBean.get("UTILISATEURS_VISES"));
            if (infoBean.get("FORMAT_ENVOI") == null) {
                modeleMail.setFormatEnvoi("0");
            } else {
                modeleMail.setFormatEnvoi((String) infoBean.get("FORMAT_ENVOI"));
            }
            String action_inscription = "";
            final String action_1 = infoBean.getString("INSCRIPTION_1");
            if ("1".equals(action_1)) {
                action_inscription = "U";
            }
            final String action_2 = infoBean.getString("INSCRIPTION_2");
            String groupeaction = "";
            if ("1".equals(action_2)) {
                groupeaction = infoBean.getString("GROUPE_INSCRIPTION_2");
                if (StringUtils.isNotEmpty(groupeaction)) {
                    if (groupeaction.startsWith("/")) {
                        groupeaction = groupeaction.substring(1);
                    }
                    if (action_inscription.equals("U")) {
                        action_inscription += ";";
                    }
                    action_inscription += "G," + groupeaction;
                }
            }
            //si aucune action définie, par défaut, l'utilisateur sera ajouté dans la liste des utilisateurs
            if (StringUtils.isEmpty(action_inscription)) {
                action_inscription = "U";
            }
            modeleMail.setActionInscription(action_inscription);
            String chainePublicVise = infoBean.getString("PUBLIC_VISE_DSI");
            final Vector<String> vPublicVise = Chaine.getVecteurPointsVirgules(chainePublicVise);
            if (StringUtils.isNotEmpty(groupeaction) && !vPublicVise.contains(groupeaction)) {
                if (chainePublicVise.length() > 0) {
                    chainePublicVise += ";";
                }
                chainePublicVise += groupeaction;
            }
            String chainePublicViseFormatee = ControleurUniv.formaterPublicViseDsiPourEnregistrement(chainePublicVise);
            modeleMail.setDiffusionPublicVise(chainePublicViseFormatee);
        }
        if (apercu || InfoBean.ACTION_VALIDER.equals(action)) {
            if (!apercu) {
                if (InfoBean.ETAT_OBJET_CREATION.equals(infoBean.getEtatObjet())) {
                    final String confirmation = String.format(MessageHelper.getMessage(infoBean.getNomExtension(), "CONFIRMATION_CREATION_MODELEMAIL"), modeleMail.getIntitule());
                    infoBean.addMessageConfirmation(confirmation);
                } else if ((ECRAN_PRINCIPAL.equals(ecranLogique) && InfoBean.ETAT_OBJET_MODIF.equals(infoBean.getEtatObjet())) || (ECRAN_ENVOI.equals(ecranLogique) && "1".equals(infoBean.getString("REMPLACER_MODELE")))) {
                    final String confirmation = String.format(MessageHelper.getMessage(infoBean.getNomExtension(), "CONFIRMATION_MODIFICATION_MODELEMAIL"), modeleMail.getIntitule());
                    infoBean.addMessageConfirmation(confirmation);
                } else if (ECRAN_ENVOI.equals(ecranLogique)) {
                    // On enregistre les modifications pour l'envoi (etat=2_idmodelemailparent)
                    final long idModeleOriginal = modeleMail.getId();
                    modeleMail.setEtat("2_" + idModeleOriginal);
                    modeleMail.setId((long) 0);
                }
                serviceModelemail.save(modeleMail);
                infoBean.set("ID_MODELEMAIL", modeleMail.getId().toString());
                final String codeparent = modeleMail.getId() + ",TYPE=NEWSLETTER";
                /* Fichiers joints */
                if (infoBean.get("TOTAL_FICHIER_MULTIPLE_1") != null) {
                    serviceRessource.syncFiles(codeparent, (String) infoBean.get("TOTAL_FICHIER_MULTIPLE_1"));
                }
                final List<RessourceBean> listeFichier = serviceRessource.getByCodeParent(modeleMail.getId() + ",TYPE=NEWSLETTER");
                int iPoidsTotal = 0;
                if (listeFichier.size() > 0) {
                    for (final RessourceBean fichier : listeFichier) {
                        iPoidsTotal += RessourceUtils.getPoids(fichier) / 1024;
                    }
                    String poidsFichiers = NewsletterUtil.getProperty("newsletter.fichiers_joints.poids_max");
                    if (poidsFichiers == null) {
                        poidsFichiers = "1024";
                    }
                    if (Integer.parseInt(poidsFichiers) < iPoidsTotal) {
                        throw new ErreurApplicative(String.format(NewsletterUtil.getMessage("BO_NEWSLETTER_MAX_FILE_SIZE"), poidsFichiers));
                    }
                }
                if (ECRAN_ENVOI.equals(ecranLogique)) {
                    preparerCONFIRMATION();
                } else {
                    etat = FIN;
                }
            } else {
                // suppression des anciens aperçus
                final List<ModelemailBean> modelsByState = serviceModelemail.getByState("1");
                for (ModelemailBean modelToDelete : modelsByState) {
                    serviceRessource.deleteFiles(modelToDelete.getId() + ",TYPE=NEWSLETTER");
                    serviceModelemail.delete(modelToDelete.getId());
                }
                // insertion d'un nouvel apercu seulement en édition
                if (!ECRAN_CONFIRMATION.equals(ecranLogique)) {
                    modeleMail.setId((long) 0);
                    modeleMail.setEtat("1");
                    serviceModelemail.save(modeleMail);
                    final String codeparent = modeleMail.getId() + ",TYPE=NEWSLETTER";
                    /* Fichiers joints */
                    if (infoBean.get("TOTAL_FICHIER_MULTIPLE_1") != null) {
                        serviceRessource.syncFiles(codeparent, (String) infoBean.get("TOTAL_FICHIER_MULTIPLE_1"));
                    }
                }
                String urlApercu = PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?PROC=RENDER_MODELE_MAIL&MODE=APERCU&ACTION=NEWSLETTER&FORMAT_ENVOI=" + infoBean.get("FORMAT_ENVOI_APERCU") + "&ID_MODELE_MAIL=" + modeleMail.getId() + "&CODE_RUBRIQUE=" + modeleMail.getCodeRubrique();
                if ("1".equals(infoBean.get("PUBLIC_INTERNE")) && autorisations != null) {
                    urlApercu += "&CODE_UTILISATEUR=" + autorisations.getCode();
                }
                infoBean.setEcranRedirection(urlApercu);
                ecranLogique = "APERCU";
            }
        } else if (InfoBean.ACTION_SUPPRIMER.equals(action)) {
            deleteModeleMail();
        }
    }

    /**
     * Préparation des données à renvoyer pour une rechercher ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerDonneesRecherche() throws Exception {
        infoBean.set("LISTE_TYPES_NEWSLETTER", LabelUtils.getLabelCombo("01", LangueUtil.getDefaultLocale()));
        final Hashtable<String, String> listeResponsables = new Hashtable<>();
        final List<ModelemailBean> allModels = serviceModelemail.getAll();
        final Collection<String> listeCodes = new ArrayList<>();
        for (ModelemailBean currentModel : allModels) {
            listeCodes.add(currentModel.getCodeResponsables());
        }
        Collection<UtilisateurBean> users = serviceUser.getByCodes(listeCodes.toArray(new String[listeCodes.size()]));
        for (UtilisateurBean responsable : users) {
            listeResponsables.put(responsable.getCode(), String.format("%s %s", responsable.getNom(), responsable.getPrenom()));
        }
        infoBean.set("LISTE_CODES_RESPONSABLES", listeResponsables);
    }

    /**
     * Gestion de l'écran de saisie d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerLISTEDESTINATAIRES() throws Exception {
        ecranLogique = ECRAN_LISTEDESTINATAIRES;
        infoBean.set("LIBELLE_DESTINATAIRES", infoBean.get("LIBELLE_DESTINATAIRES"));
        infoBean.set("NB_DESTINATAIRES", infoBean.get("NB_DESTINATAIRES"));
    }

    /**
     * Gestion de l'écran de saisie d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerCONFIRMATION() throws Exception {
        ecranLogique = ECRAN_CONFIRMATION;
        final String titreEcran = NewsletterUtil.getMessage("BO_ENVOI_NEWSLETTER_2");
        infoBean.setTitreEcran(String.format(titreEcran, modeleMail.getIntitule()));
        /* Parcours des utilisateurs */
        // RP 20050301
        final TreeMap<String, String> destinataires = new TreeMap<>();
        final StringBuilder listeCodesDest = new StringBuilder();
        final String dPv = modeleMail.getDiffusionPublicVise();
        final String utV = modeleMail.getUtilisateursVises();
        int i = 0;
        if (StringUtils.isEmpty(dPv) && StringUtils.isEmpty(utV)) {
            infoBean.addMessageErreur(NewsletterUtil.getMessage("BO_NEWSLETTER_ERREUR_AUCUN_DESTINATAIRE"));
            mode = MODE_ENVOI;
            action = InfoBean.ACTION_SUITE;
            infoBean.setEtatObjet(InfoBean.ETAT_OBJET_RECHERCHE);
            infoBean.set("REMPLACER_MODELE", "1");
            preparerMENU();
        } else {
            if (!dPv.equals("")) {
                final List<UtilisateurBean> users = serviceUser.getByChaineDiffusion(dPv);
                for (UtilisateurBean user : users) {
                    String szKey = String.format("%s %s (%s)", user.getNom(), user.getPrenom(), user.getAdresseMail());
                    if (destinataires.containsKey(szKey)) {
                        szKey = i + "_" + szKey;
                        i++;
                    }
                    destinataires.put(szKey, user.getCode());
                    listeCodesDest.append(user.getCode()).append(";");
                }
            }
            final List<UtilisateurBean> users = serviceUser.getByCodes(utV.split(";"));
            if (CollectionUtils.isNotEmpty(users)) {
                for (UtilisateurBean currentUser : users) {
                    if (!destinataires.containsValue(currentUser.getCode())) {
                        String szKey = String.format("%s %s (%s)", currentUser.getNom(), currentUser.getPrenom(), currentUser.getAdresseMail());
                        if (destinataires.containsKey(szKey)) {
                            szKey = String.format("%d_%s", i, szKey);
                            i++;
                        }
                        destinataires.put(szKey, currentUser.getCode());
                        listeCodesDest.append(currentUser.getCode()).append(";");
                    }
                }
            }
            if (destinataires.size() > 50) {
                infoBean.set("ATT_DESTINATAIRES", NewsletterUtil.getMessage("BO_NEWSLETTER_MAX_USER"));
            }
            infoBean.setInt("NB_DESTINATAIRES", destinataires.size());
            infoBean.set("DESTINATAIRES", destinataires);
            infoBean.set("CODES_DESTINATAIRES", listeCodesDest.toString());
        }
        if (StringUtils.isNotEmpty(modeleMail.getCodeResponsables())) {
            infoBean.set("CODE_RESPONSABLES", modeleMail.getCodeResponsables());
        } else {
            infoBean.set("CODE_RESPONSABLES", NewsletterUtil.getProperty("mail.from"));
        }
        if (StringUtils.isNotBlank(modeleMail.getLibelleResponsable())) {
            infoBean.set("LIBELLE_RESPONSABLE", modeleMail.getLibelleResponsable());
        } else {
            infoBean.set("LIBELLE_RESPONSABLE", StringUtils.EMPTY);
        }
        infoBean.set("FORMAT_ENVOI_APERCU", modeleMail.getFormatEnvoi());
        if ("0".equals(modeleMail.getFormatEnvoi())) {
            infoBean.set("LABEL_FORMAT_ENVOI", NewsletterUtil.getMessage("BO_NEWSLETTER_FORMAT_ENVOI_HTML"));
        } else {
            infoBean.set("LABEL_FORMAT_ENVOI", NewsletterUtil.getMessage("BO_NEWSLETTER_FORMAT_ENVOI_TEXT"));
        }
        final Calendar cal = Calendar.getInstance();
        //Dans le cas d'une newsletter non public interne (non personnalisée) on ne purge pas par défaut
        if ("0".equals(infoBean.getString("PUBLIC_INTERNE"))) {
            cal.set(Calendar.YEAR, 2999);
            cal.set(Calendar.MONTH, cal.getActualMaximum(Calendar.MONTH));
            cal.set(Calendar.DATE, cal.getActualMaximum(Calendar.DATE));
            cal.set(Calendar.HOUR, cal.getActualMaximum(Calendar.HOUR));
            cal.set(Calendar.MINUTE, cal.getActualMaximum(Calendar.MINUTE));
            cal.set(Calendar.SECOND, cal.getActualMaximum(Calendar.SECOND));
            cal.set(Calendar.MILLISECOND, cal.getActualMaximum(Calendar.MILLISECOND));
        } else {
            cal.add(Calendar.DAY_OF_YEAR, Integer.valueOf(NewsletterUtil.getProperty("newsletter.purge.delai")));
        }
        infoBean.set("DATE_PURGE", new Date(cal.getTimeInMillis()));
    }

    private void preparerListeHistoriqueDesEnvois(final Map<Long, Boolean> hAcces, final GestionnaireMailing gesMail, final Collection<Newsletter> listeNewletterEnCours) throws Exception {
        final GregorianCalendar gCal = new GregorianCalendar();
        final DateFormat dFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
        final List<MailingBean> mailingBeans = serviceMailing.getAllMailings();
        final Map<String, Map<String, String>> hm = new HashMap<>();
        for (MailingBean currentMailing : mailingBeans) {
            final Long idModeleMail = Long.parseLong(StringUtils.substringBefore(currentMailing.getIdNewsletter(), "_"));
            if (hAcces.get(idModeleMail) == null) {
                final ModelemailBean modelemailBean = serviceModelemail.getById(idModeleMail);
                if (modelemailBean == null || hAcces.containsKey(idModeleMail) && !hAcces.get(idModeleMail)) {
                    continue;
                }
                if (!autorisations.possedePermission(new PermissionBean("TECH", "new", "E"), new Perimetre(modelemailBean.getCodeRattachement(), modelemailBean.getCodeRubrique(), "*", "*", ""))) {
                    hAcces.put(idModeleMail, false);
                    continue;
                }
                hAcces.put(idModeleMail, true);
            }
            final String idNewsletter = currentMailing.getIdNewsletter();
            if (!listeNewletterEnCours.contains(gesMail.new Newsletter(idNewsletter, "", 0))) {
                String[] tokens = StringUtils.split(idNewsletter, "_");
                gCal.setTimeInMillis(Long.parseLong(tokens[1]));
                final Map<String, String> vals = new Hashtable<>();
                vals.put("OBJET", currentMailing.getObjet());
                vals.put("DATE_ENVOI", dFormat.format(gCal.getTime()));
                hm.put(idNewsletter, vals);
            }
        }
        infoBean.set("LISTE_OLD_NEWSLETTER", hm);
    }

    /**
     * Affichage de l'écran de saisie d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerSUIVI() throws Exception {
        ecranLogique = ECRAN_SUIVI;
        // variable static qui permet de tester si le thread d'envoi de mail est actif
        final boolean emailThread = ThreadMailing.estActif();
        final boolean injectionThread = Modelemail.estActif();
        // envoi en cours
        final GestionnaireMailing gesMail = GestionnaireMailing.getInstance();
        final Collection<Newsletter> listeNewletterEnCours = new ArrayList<>();
        int i = 0;
        if (emailThread && gesMail.getNewsletterEnCours() != null) {
            final ThreadMailing threadEnCours = gesMail.getThreadEnvoiEncours();
            if (threadEnCours != null) {
                infoBean.set("ETAT_NEWSLETTER_0", "2");
                infoBean.set("ID_NEWSLETTER_0", threadEnCours.getIdNewsletter());
                infoBean.set("LIBELLE_NEWSLETTER_0", threadEnCours.getLibelleNewsletter());
                infoBean.setInt("NB_MAILS_TOTAL_0", threadEnCours.getNbMailsTotal());
                infoBean.setInt("NB_MAILS_RESTANTS_0", threadEnCours.getNbMailsRestants());
                infoBean.setInt("NB_MAILS_ERREURS_0", threadEnCours.getListeMailsErreur().size());
                i++;
                listeNewletterEnCours.add(gesMail.getNewsletterEnCours());
            }
            if (gesMail.getListeNewsletterEnAttente().size() > 0) {
                for (final Newsletter newsletter : gesMail.getListeNewsletterEnAttente()) {
                    infoBean.set("ETAT_NEWSLETTER_" + i, "1");
                    infoBean.set("ID_NEWSLETTER_" + i, newsletter.getId());
                    infoBean.set("LIBELLE_NEWSLETTER_" + i, newsletter.getLibelle());
                    int nb = newsletter.getNbDestinataires();
                    if (nb == 0) {
                        final List<MailingBean> mailingBeans = serviceMailing.getByIdNewsLetterAndState(newsletter.getId(), StringUtils.EMPTY);
                        nb = mailingBeans.size();
                    }
                    infoBean.setInt("NB_MAILS_TOTAL_" + i, nb);
                    infoBean.setInt("NB_MAILS_RESTANTS_" + i, -1);
                    infoBean.setInt("NB_MAILS_ERREURS_" + i, -1);
                    i++;
                }
                listeNewletterEnCours.addAll(gesMail.getListeNewsletterEnAttente());
            }
        }
        final List<MailingBean> mailingBeans = serviceMailing.getByState(StringUtils.EMPTY);
        final Map<Long, Boolean> hAcces = new HashMap<>();
        for (MailingBean currentMailing : mailingBeans) {
            final Long idModeleMail = Long.parseLong(StringUtils.substringBefore(currentMailing.getIdNewsletter(), "_"));
            if (hAcces.get(idModeleMail) == null) {
                final ModelemailBean modelemailBean = serviceModelemail.getById(idModeleMail);
                if (modelemailBean == null || hAcces.containsKey(idModeleMail) && !hAcces.get(idModeleMail)) {
                    continue;
                }
                if (!autorisations.possedePermission(new PermissionBean("TECH", "new", "E"), new Perimetre(modelemailBean.getCodeRattachement(), modelemailBean.getCodeRubrique(), "*", "*", ""))) {
                    hAcces.put(idModeleMail, false);
                    continue;
                }
                hAcces.put(idModeleMail, true);
            }
            if (!listeNewletterEnCours.contains(gesMail.new Newsletter(currentMailing.getIdNewsletter(), "", 0))) {
                if (injectionThread) {
                    infoBean.set("ETAT_NEWSLETTER_" + i, "0");
                } else {
                    infoBean.set("ETAT_NEWSLETTER_" + i, "3");
                }
                infoBean.set("ID_NEWSLETTER_" + i, currentMailing.getIdNewsletter());
                infoBean.set("LIBELLE_NEWSLETTER_" + i, currentMailing.getObjet());
                infoBean.setInt("NB_MAILS_TOTAL_" + i, -1);
                infoBean.setInt("NB_MAILS_RESTANTS_" + i, -1);
                infoBean.setInt("NB_MAILS_ERREURS_" + i, -1);
                i++;
            }
        }
        infoBean.setInt("NB_NEWSLETTERS", i);
        // Historique des envois
        preparerListeHistoriqueDesEnvois(hAcces, gesMail, listeNewletterEnCours);
    }

    /**
     * Gestion de l'écran de saisie d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterCONFIRMATION() throws Exception {
        if (InfoBean.ACTION_VALIDER.equals(action)) {
            modeleMail = serviceModelemail.getById(new Long(infoBean.getString("ID_MODELEMAIL")));
            final String listeDestinataires = infoBean.getString("CODES_DESTINATAIRES");
            final StringTokenizer st = new StringTokenizer(listeDestinataires, ";");
            final ArrayList<String> aListe = new ArrayList<>();
            while (st.hasMoreTokens()) {
                aListe.add(st.nextToken());
            }
            final int nbDestinataires = Integer.parseInt(infoBean.getString("NB_DESTINATAIRES"));
            for (int i = 0; i < nbDestinataires; i++) {
                if (infoBean.get("des" + i) != null) {
                    aListe.remove(infoBean.get("des" + i, String.class));
                }
            }
            modeleMail.setDestinatairesFinaux(aListe);
            modeleMail.setDatePurge((Date) infoBean.get("DATE_PURGE"));
            serviceModelemail.envoyerMailing(modeleMail);
            etat = FIN;
        }
    }

    /**
     * Gestion de l'écran de saisie d'un ModeleMail.
     *
     * @throws Exception
     *             the exception
     */
    private void traiterSUIVI() throws Exception {
        if (action.equals("ACTIVER")) {
            GestionnaireMailing.getInstance().envoiNewsletter(this, "", "", 0);
            while (!ThreadMailing.estActif()) {
                Thread.sleep(100);
            }
            preparerSUIVI();
        }
        if (action.equals("RAFRAICHIR")) {
            preparerSUIVI();
        }
        if (action.equals("VOIR")) {
            preparerARCHIVE();
        }
    }

    /**
     * Preparer archive.
     *
     * @throws Exception
     *             the exception
     */
    private void preparerARCHIVE() throws Exception {
        ecranLogique = ECRAN_ARCHIVE;
        final String idNewsletter = infoBean.getString("ID_OLD_NEWSLETTER");
        if ("0000".equals(idNewsletter)) {
            preparerSUIVI();
        } else {
            final List<MailingBean> mailingBeans = serviceMailing.getByIdNewsLetter(idNewsletter);
            int nbMailsTotal = mailingBeans.size();
            for (MailingBean currentMailing : mailingBeans) {
                String destinataires;
                if( "0".equals(currentMailing.getPublicInterne()) ) {
                    destinataires = currentMailing.getEmail();
                } else {
                    destinataires = currentMailing.getCodeUtilisateur();
                }
                if (StringUtils.contains(destinataires, "[")) {
                    nbMailsTotal += StringUtils.countMatches(destinataires, "[");
                }
            }
            final MailingBean firstMailing = mailingBeans.get(0);
            final GregorianCalendar gCal = new GregorianCalendar();
            String[] tokens = StringUtils.split(idNewsletter, "_");
            gCal.setTimeInMillis(Long.parseLong(tokens[1]));
            final DateFormat dFormat = DateFormat.getDateInstance(DateFormat.LONG);
            final DateFormat dFormatTime = DateFormat.getTimeInstance(DateFormat.SHORT);
            infoBean.set("DATE_ENVOI", "le " + dFormat.format(gCal.getTime()) + " à " + dFormatTime.format(gCal.getTime()));
            String codeResponsables = StringUtils.substringBefore(firstMailing.getReplyTo(), "#");
            String libelleResponsables = StringUtils.substringAfter(firstMailing.getReplyTo(), "#");
            infoBean.set("CODE_RESPONSABLES", codeResponsables);
            infoBean.set("LIBELLE_RESPONSABLE", libelleResponsables);
            infoBean.setInt("NB_DESTINATAIRES", nbMailsTotal);
            infoBean.set("LIBELLE_NEWSLETTER", firstMailing.getObjet());
            infoBean.set("ID_NEWSLETTER", idNewsletter);
            //Si la newsletter n'est pas destinée à un public interne alors il y a plusieurs destinataires dans le
            //champ email. Ces destinataires sont alors encadrés par des [].
            //Donc si le champ email  contient le caractere [ alors la newsletter n'est pas destinée à un public interne
            String urlApercu = PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?PROC=RENDER_MODELE_MAIL&ACTION=NEWSLETTER&EXT=" + NewsletterUtil.ID_EXTENSION + "&FROMARCHIVE=TRUE&ID_NEWSLETTER=" + infoBean.get("ID_NEWSLETTER");
            infoBean.set("URL_ARCHIVE", urlApercu);
        }
    }

    /**
     * Affichage de l'écran de saisie d'un PageLibre.
     *
     * @throws Exception
     *             the exception
     */
    private void rafraichirPRINCIPAL() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        if (mode == MODE_ENVOI) {
            ecranLogique = ECRAN_ENVOI;
        } else {
            ecranLogique = ECRAN_PRINCIPAL;
        }
        // Données applicatives non saisissables
        infoBean.set("ID_MODELEMAIL", modeleMail.getId().toString());
        infoBean.set("LISTE_TYPES_NEWSLETTER", new Hashtable<>(LabelUtils.getLabelCombo("01", LangueUtil.getDefaultLocale())));
        infoBean.set("LISTE_BANDEAUX", MediaUtils.getMediasAsMap(serviceMedia.getByTypeAndTypeMedia("photo", "0103")));
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(modeleMail.getCodeRubrique());
        String rubriqueLabel = StringUtils.EMPTY;
        if (rubriqueBean != null) {
            rubriqueLabel = rubriqueBean.getIntitule();
        }
        infoBean.set("LIBELLE_CODE_RUBRIQUE", rubriqueLabel);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(modeleMail.getCodeRattachement(), LangueUtil.getLangueLocale(LangueUtil.getDefaultLocale())));
        infoBean.set("RADIO0", "HTML");
        infoBean.set("RADIO1", "Texte");
        if (StringUtils.isNotEmpty(modeleMail.getCodeRubrique())) {
            infoBean.set("LIBELLE_TYPE", LabelUtils.getLibelleWithSection("01", modeleMail.getType(), LangueUtil.getIndiceLocaleDefaut(), modeleMail.getCodeRubrique()));
        } else {
            infoBean.set("LIBELLE_TYPE", LabelUtils.getLibelle("01", modeleMail.getType(), LangueUtil.getDefaultLocale()));
        }
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION) || autorisations.possedePermission(new PermissionBean("TECH", "new", "M"), new Perimetre(modeleMail.getCodeRattachement(), modeleMail.getCodeRubrique(), "*", "*", ""))) {
            infoBean.set("DROIT_MODIFICATION", true);
        } else {
            infoBean.set("DROIT_MODIFICATION", false);
        }
        // 20060130 : FBI : filtre l'arbre des groupes en fonction des autorisations
        final String activationDsi = NewsletterUtil.getProperty("dsi.activation");
        if ("1".equals(activationDsi)) {
            infoBean.set("GRS_FILTRE_ARBRE_GROUPE", "1");
            infoBean.set("GRS_FILTRE_ARBRE_GROUPE_TYPE", "TECH");
            infoBean.set("GRS_FILTRE_ARBRE_GROUPE_OBJET", "dsi");
            infoBean.set("GRS_FILTRE_ARBRE_GROUPE_ACTION", "");
        }
        if (StringUtils.isNotBlank(modeleMail.getIntitule())) {
            infoBean.setTitreEcran(modeleMail.getIntitule());
        }
    }
}
