/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.reference;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;

import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.bean.NewsletterBeanExport;
import com.kosmos.newsletter.service.ServiceModelemail;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.exception.ErreurReferenceException;
import com.kosmos.usinesite.reference.BeanReference;
import com.kosmos.usinesite.reference.DelimitationPattern;
import com.kosmos.usinesite.reference.ServiceBeanReference;
import com.kosmos.usinesite.reference.dto.ServiceBeanReferenceDto;
import com.kosmos.usinesite.reference.impl.MediaServiceBeanReference;
import com.kosmos.usinesite.reference.impl.RessourceServiceBeanReference;
import com.univ.objetspartages.bean.RessourceBean;

public class ServiceNewsletterBeanReference extends ServiceBeanReference<NewsletterBeanExport> {

    @Override
    public Collection<BeanReference> check(final ServiceBeanReferenceDto<NewsletterBeanExport> serviceBeanReferenceDto) throws ErreurReferenceException {
        final Collection<BeanReference> referencesToUpdate = new ArrayList<>();
        NewsletterBeanExport newsletterBean = serviceBeanReferenceDto.getBeansOrig().values().iterator().next();
        final Collection<String> moduleToSearch = new ArrayList<>();
        moduleToSearch.add(newsletterBean.getIdModule());
        final BeanReference referencesByCode = new BeanReference();
        // mise à jour du code sur le module newsletter
        referencesByCode.addModulesBySearchString("\"code\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        // mise à jour de l'id newsletter
        final BeanReference referencesById = new BeanReference();
        referencesById.addModulesBySearchString("\"id_modelemail\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        referencesById.addModulesBySearchString("\"id\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        referencesById.addModulesBySearchString(DelimitationPattern.COMMA_DELIMITER + ",TYPE=NEWSLETTER", moduleToSearch);
        // mise à jour des tags d'insertion toolbox
        // Ici on cherche à remplacer quelque chose pouvant contenir les informations : [formabonews;id1,id2,dd/mm/yyyy]
        referencesById.addModulesBySearchStrings("\\[formabonews;" + DelimitationPattern.CLOSINGBRACKET_DELIMITER + "\\]", "[;,]?"+ DelimitationPattern.COMMA_DELIMITER, Collections.emptyList());
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        long maxIdModeleMail = shiftSequence(serviceModelemail, serviceBeanReferenceDto, "newsletter");
        for (final Entry<String, NewsletterBeanExport> entry : serviceBeanReferenceDto.getBeansOrig().entrySet()) {
            newsletterBean = executeProcessors(entry.getValue(), serviceBeanReferenceDto);
            if (newsletterBean == null) {
                continue;
            }
            try {
                referencesById.addNewValueByOldValue(entry.getKey(), String.valueOf(maxIdModeleMail));
                if (MapUtils.isNotEmpty(newsletterBean.getMedias())) {
                    final MediaServiceBeanReference mediaServiceBeanRef = new MediaServiceBeanReference();
                    mediaServiceBeanRef.addSearchStringForBean(newsletterBean, moduleToSearch);
                    referencesToUpdate.addAll(mediaServiceBeanRef.check(new ServiceBeanReferenceDto()
                        .setProcessor(serviceBeanReferenceDto.getProcessor())
                        .setBeansOrig(newsletterBean.getMedias())));
                    referencesToUpdate.addAll(prepareRessources(serviceBeanReferenceDto, moduleToSearch));
                }
                maxIdModeleMail--;
            } catch (final Exception e) {
                throw new ErreurReferenceException("Erreur lors de la récupération des newsletter en bdd", e);
            }
        }
        referencesToUpdate.add(referencesByCode);
        referencesToUpdate.add(referencesById);
        return referencesToUpdate;
    }

    /**
     * Prépare le changement de références sur les ressources de la newsletter
     * @param serviceBeanReferenceDto : le Dto avec les informations du bean à importer
     * @return une liste de {@link BeanReference} avec les modifications de référence sur les ressources
     * @throws ErreurReferenceException
     */
    private Collection<BeanReference> prepareRessources(ServiceBeanReferenceDto<NewsletterBeanExport> serviceBeanReferenceDto, final Collection<String> moduleToSearch) throws ErreurReferenceException {
        final Map<String, RessourceBean> ressources = new HashMap<>();
        for (final NewsletterBeanExport beanExport : serviceBeanReferenceDto.getBeansOrig().values()) {
            if (beanExport != null && CollectionUtils.isNotEmpty(beanExport.getRessources())) {
                for (final RessourceBean res : beanExport.getRessources()) {
                    ressources.put(res.getId().toString(), res);
                }
            }
        }
        final RessourceServiceBeanReference ressourceServiceBeanRef = new RessourceServiceBeanReference();
        final Collection<BeanReference> references = ressourceServiceBeanRef.check(new ServiceBeanReferenceDto()
            .setProcessor(serviceBeanReferenceDto.getProcessor())
            .setBeansOrig(ressources));
        for (final BeanReference beanRef : references) {
            beanRef.addModulesBySearchString("\"id_ressource\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", moduleToSearch);
        }
        return references;
    }
}
