/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.service;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.newsletter.bean.MailingBean;
import com.kosmos.newsletter.dao.impl.MailingDAO;
import com.kosmos.service.impl.AbstractServiceBean;

/**
 * Created on 23/04/15.
 */
public class ServiceMailing extends AbstractServiceBean<MailingBean, MailingDAO> {

    /**
     * Retourne la liste des newsletter correspondant à l'id fourni en paramètre et si le champ email est renseigné, on filtre aussi sur lui
     * @param email l'email du destinataire, peut etre null
     * @param idNewsletter l'id de la newsletter que l'on cherche, ne doit pas etre null
     * @return la liste des mailing correspondant, ou une liste vide sinon
     */
    public List<MailingBean> getByEmailAndIdNewsLetter(String email, String idNewsletter) {
        List<MailingBean> results;
        if (StringUtils.isNotBlank(email)) {
            results = dao.getByEmailAndIdNewsLetter(email, idNewsletter);
        } else {
            results = dao.getByIdNewsletter(idNewsletter);
        }
        return results;
    }

    /**
     * Retourne la liste des newsletter correspondant à l'id fourni en paramètre et si le champ codeUtilisateur est renseigné, on filtre aussi sur lui
     * @param codeUtilisateur code de l'utilisateur
     * @param idNewsletter l'id de la newsletter que l'on cherche, ne doit pas etre null
     * @return la liste des mailing correspondant, ou une liste vide sinon
     */
    public List<MailingBean> getByCodeAndIdNewsLetter(String codeUtilisateur, String idNewsletter) {
        List<MailingBean> results;
        if (StringUtils.isNotBlank(codeUtilisateur)) {
            results = dao.getByCodeUtilisateurAndIdNewsLetter(codeUtilisateur, idNewsletter);
        } else {
            results = dao.getByIdNewsletter(idNewsletter);
        }
        return results;
    }

    public List<MailingBean> getByObjectTypeCreationDatePublic(String object, String type, Date startDate, Date endDate) {
        return dao.getByObjectTypeCreationDatePublic(object, type, startDate, endDate);
    }

    public List<MailingBean> getByIdNewsLetter(String idNewsletter) {
        return dao.getByIdNewsletter(idNewsletter);
    }

    public List<MailingBean> getAllMailings() {
        return dao.getAll();
    }

    public List<MailingBean> getByIdNewsLetterAndState(String id, String state) {
        return dao.getByIdNewsletterAndState(id, state);
    }

    public Integer getCountOfIdNewsLetterAndState(String id, String state) {
        return dao.getCountOfIdNewsLetterAndState(id, state);
    }

    public List<MailingBean> getByIdNewsLetterAndState(String id, String state, Integer offset, Integer range) {
        return dao.getByIdNewsletterAndState(id, state, offset, range);
    }

    public List<MailingBean> getByState(String state) {
        return dao.getByState(state);
    }

    public List<MailingBean> getByIdNewsletterAndStateOrdered(String idNewsletter, String state, String ordered) {
        return dao.getByIdNewsletterAndStateOrdered(idNewsletter, state, ordered);
    }
}
