/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.service.impl.beanexport;

import java.util.List;
import java.util.Map;

import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.bean.NewsletterBeanExport;
import com.kosmos.newsletter.service.ServiceModelemail;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.utils.UASOmHelper;
import com.kosmos.usinesite.utils.UASServicesHelper;
import com.kportal.extension.module.bean.BeanExportMap;
import com.kportal.extension.module.bean.MetadataExport;
import com.kportal.extension.module.dto.ServiceBeanExportDto;
import com.kportal.extension.module.service.ServiceBeanExport;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceRessource;

public class ServiceBeanExportNewsletter extends ServiceBeanExport<NewsletterBeanExport> {

    private ServiceRessource serviceRessource;

    public void setServiceRessource(ServiceRessource serviceRessource) {
        this.serviceRessource = serviceRessource;
    }

    @Override
    public BeanExportMap<NewsletterBeanExport> getBeansByRubrique(final ServiceBeanExportDto serviceBeanExportDto) throws Exception {
        final BeanExportMap<NewsletterBeanExport> modelesMailsBeans = new BeanExportMap<>();
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        List<ModelemailBean> newsletterByRubrique = serviceModelemail.getByCodesRubriques(serviceBeanExportDto.getCodesRubrique());
        for (ModelemailBean modeleMail : newsletterByRubrique) {
            if (!modelesMailsBeans.containsKey(modeleMail.getId())) {
                final ModelemailBean modeleMailBean = new ModelemailBean();
                UASOmHelper.copyProperties(modeleMailBean, modeleMail);
                final NewsletterBeanExport modeleBean = new NewsletterBeanExport();
                modeleBean.setIdModule(serviceBeanExportDto.getModule().getId());
                modeleMail.setIntitule(modeleMail.getIntitule());
                modeleBean.setBean(modeleMailBean);
                final List<RessourceBean> vRessources = serviceRessource.getFilesOrderBy(modeleMail.getId() + ",TYPE=NEWSLETTER", "CODE_PARENT");
                modeleBean.setRessources(vRessources);
                final Map<String, MediaBean> medias = UASServicesHelper.getMediasByRessources(serviceBeanExportDto.getCodeSite(), vRessources, serviceBeanExportDto.getPathExport());
                medias.putAll(medias);
                modeleBean.setMedias(medias);
                NewsletterBeanExport finalNewsletterBeanExport = executeProcessors(modeleBean, serviceBeanExportDto);
                if (finalNewsletterBeanExport != null) {
                    modelesMailsBeans.put(String.valueOf(modeleMail.getId()), finalNewsletterBeanExport);
                }
            }
        }
        return modelesMailsBeans;
    }

    @Override
    public MetadataExport getMetadataExport(final ServiceBeanExportDto serviceBeanExportDto) {
        final MetadataExport metadataExport = new MetadataExport();
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        final ServiceMedia serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
        metadataExport.addMetadataSequence("newsletter", serviceModelemail.getAutoIncrement());
        metadataExport.addMetadataSequence("ressource", serviceRessource.getAutoIncrement());
        metadataExport.addMetadataSequence("media", serviceMedia.getAutoIncrement());
        return metadataExport;
    }
}
