/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.service.impl.beanimport;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.bean.NewsletterBeanExport;
import com.kosmos.newsletter.service.ServiceModelemail;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.utils.UASOmHelper;
import com.kportal.extension.module.dto.ServiceBeanImportDto;
import com.kportal.extension.module.service.ServiceBeanImport;
import com.univ.utils.ContexteDao;

public class ServiceBeanImportNewsletter extends ServiceBeanImport<NewsletterBeanExport> {

    /** Logger available to subclasses. */
    private static final Logger LOGGER = LoggerFactory.getLogger(ServiceBeanImportNewsletter.class);

    @Override
    public void importBeanExport(final ServiceBeanImportDto serviceBeanImportDto) {
        final NewsletterBeanExport newsLetterBeanExport = (NewsletterBeanExport) serviceBeanImportDto.getBeanExport();
        NewsletterBeanExport finalNewsletterBeanExport = executeProcessors(newsLetterBeanExport, serviceBeanImportDto);
        if (finalNewsletterBeanExport == null) {
            return;
        }
        final ContexteDao contexteDao = new ContexteDao();
        final ServiceModelemail serviceModelemail = ServiceManager.getServiceForBean(ModelemailBean.class);
        try {
            serviceModelemail.addWithForcedId(finalNewsletterBeanExport.getBean());
            UASOmHelper.addAllRessourceMediaBean(finalNewsletterBeanExport.getRessources(), finalNewsletterBeanExport.getMedias(), serviceBeanImportDto.getPathImport());
        } catch (final Exception e) {
            LOGGER.error(String.format("Une erreur est survenue lors de l'import de la newsletter \"%s\"", finalNewsletterBeanExport), e);
        } finally {
            contexteDao.close();
        }
    }

}
