/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.tag;

import org.apache.commons.lang3.StringUtils;

import com.kportal.tag.interpreteur.impl.InterpreteurMailto;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.RequeteUtil;

public class InterpreteurMailtoNewsletter extends InterpreteurMailto {

    @Override
    public String interpreterTag(final String texteAInterpreter, final String baliseOuvrante, final String baliseFermante) {
        final ContexteUniv ctx = ContexteUtil.getContexteUniv();
        final String contenuTagMailto = StringUtils.substringBetween(texteAInterpreter, baliseInterneOuvrante, baliseInterneFermante);
        String texteInterprete = StringUtils.EMPTY;
        String adresseMail = StringUtils.EMPTY;
        // nouveau tag de la forme [mailto]email=annuaire;549805613&subject=[...]&body=[...][/mailto]
        if (StringUtils.startsWith(contenuTagMailto, "email=")) {
            adresseMail = RequeteUtil.renvoyerParametre(contenuTagMailto, "email");
            if (adresseMail.contains(";")) {
                adresseMail = getAdresseMailFromAnnuaire(ctx, adresseMail);
            }
        } else {
            adresseMail = getAdresseMailFromAnnuaire(ctx, contenuTagMailto);
        }
        if (StringUtils.isNotBlank(adresseMail)) {
            final StringBuilder lienMailto = new StringBuilder("mailto:");
            lienMailto.append(adresseMail);
            final String sujet = RequeteUtil.renvoyerParametre(contenuTagMailto, "subject");
            if (StringUtils.isNotBlank(sujet)) {
                lienMailto.append("?subject=");
                lienMailto.append(sujet);
            }
            final String corps = RequeteUtil.renvoyerParametre(contenuTagMailto, "body");
            if (StringUtils.isNotBlank(corps)) {
                lienMailto.append("&amp;body=");
                lienMailto.append(corps);
            }
            texteInterprete = StringUtils.replace(texteAInterpreter, baliseInterneOuvrante + contenuTagMailto + baliseInterneFermante, lienMailto.toString());
        }
        return texteInterprete;
    }
}
