/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.util;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.mail.SendFailedException;
import javax.mail.internet.AddressException;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.email.JSBMailbox;
import com.kosmos.controllers.SGController;
import com.kosmos.newsletter.bean.MailingBean;
import com.kosmos.newsletter.service.ServiceMailing;
import com.kosmos.newsletter.util.GestionnaireMailing.Newsletter;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.PropertyHelper;
import com.univ.utils.ContexteUniv;
import com.univ.utils.URLResolver;

/**
 * Envoi de mailing, avec gestion en thread de chaque mail envoyé
 */
public class EmailThread implements Runnable {

    private static final Logger LOG = LoggerFactory.getLogger(EmailThread.class);

    /** The last update. */
    private static long lastUpdate = 0;

    private ServiceMailing serviceMailing;

    /** The liste_mails_erreur. */
    private final Hashtable<String, String> liste_mails_erreur = new Hashtable<>();

    /** The id_newsletter. */
    private String id_newsletter = "";

    /** The libelle_newsletter. */
    private String libelle_newsletter = "";

    /** The nb mails total. */
    private int nbMailsTotal = 0;

    /** The nb mails restants. */
    private int nbMailsRestants = 0;

    /**
     * constructeur.
     *
     * @param _idNewsletter
     *            the _id newsletter
     * @param _libelleNewsletter
     *            the _libelle newsletter
     * @param _nbMailsTotal
     *            the _nb mails total
     */
    public EmailThread(final String _idNewsletter, final String _libelleNewsletter, final int _nbMailsTotal) {
        super();
        id_newsletter = _idNewsletter;
        libelle_newsletter = _libelleNewsletter;
        nbMailsTotal = _nbMailsTotal;
        serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
    }

    /**
     * constructeur.
     *
     * @param newsletter
     *            the newsletter
     */
    public EmailThread(final Newsletter newsletter) {
        super();
        id_newsletter = newsletter.getId();
        libelle_newsletter = newsletter.getLibelle();
        nbMailsTotal = newsletter.getNbDestinataires();
        serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
    }

    /**
     * Détermine si le thread est actif.
     *
     * @return boolean estactif
     */
    public static boolean estActif() {
        boolean res = true;
        /* false si inactif depuis + de 10 secondes */
        if (System.currentTimeMillis() > lastUpdate + 10000) {
            res = false;
        }
        return res;
    }

    /**
     * lancement.
     */
    @Override
    public void run() {
        String texteAnomalies = StringUtils.EMPTY;
        String objetMailing = StringUtils.EMPTY;
        String mimeType = StringUtils.EMPTY;
        final ArrayList<String[]> arrayUtilisateur = new ArrayList<>();
        boolean init = true;
        try {
            LOG.debug("Lancement thread email");
            if (nbMailsTotal == 0) {
                nbMailsTotal = serviceMailing.getCountOfIdNewsLetterAndState(id_newsletter, StringUtils.EMPTY);
            }
            nbMailsRestants = nbMailsTotal;
            final String[] user = new String[2];
            String[] tabFichiers = null, tabNomFichiers = null;
            String fichiers;
            String nomFichiers;
            long sleep = 100;
            List<MailingBean> mailingBeans = serviceMailing.getByIdNewsLetterAndState(id_newsletter, StringUtils.EMPTY, 0, 100);
            while (mailingBeans.size() > 0) {
                for (MailingBean currentMailing : mailingBeans) {
                    if(StringUtils.isEmpty(mimeType)) {
                        mimeType = currentMailing.getTypeMime();
                    }
                    if (NewsletterUtil.getProperty("mail.sleep") != null) {
                        try {
                            sleep = Long.parseLong(NewsletterUtil.getProperty("mail.sleep"));
                        } catch (final Exception e) {
                        }
                    }
                    if (sleep > 0) {
                        Thread.sleep(sleep);
                    }
                    if (libelle_newsletter.length() == 0) {
                        libelle_newsletter = currentMailing.getObjet();
                    }
                    try {
                        user[0] = currentMailing.getEmail();
                        user[1] = currentMailing.getNomUtilisateur();
                        if (init) {
                            objetMailing = currentMailing.getObjet();
                            fichiers = currentMailing.getFichier();
                            if (fichiers.contains(";")) {
                                tabFichiers = fichiers.split(";");
                            } else if (!fichiers.equals("")) {
                                tabFichiers = new String[1];
                                tabFichiers[0] = fichiers;
                            }
                            nomFichiers = currentMailing.getNomFichier();
                            if (nomFichiers.contains(";")) {
                                tabNomFichiers = nomFichiers.split(";");
                            } else if (!nomFichiers.equals("")) {
                                tabNomFichiers = new String[1];
                                tabNomFichiers[0] = nomFichiers;
                            }
                        }
                        LOG.debug("send mail to " + currentMailing.getEmail());
                        //envoi du mail
                        final JSBMailbox mailbox = new JSBMailbox(false);
                        mailbox.sendMsgWithAttachedFiles(currentMailing.getReplyTo(), user, currentMailing.getObjet(), currentMailing.getMessage(), currentMailing.getTypeMime(), tabFichiers, tabNomFichiers);
                        nbMailsRestants--;
                        arrayUtilisateur.add(new String[] {user[0], user[1]});
                        // suppression enregistrement
                        currentMailing.setEtat("SENT");
                        serviceMailing.save(currentMailing);
                        LOG.debug("sent");
                        //mailing.delete();
                    } catch (final AddressException e) {
                        LOG.debug("unsent");
                        texteAnomalies = addErrorMessage(e, currentMailing, texteAnomalies);
                    } catch (final SendFailedException e) {
                        LOG.debug("unsent");
                        //    AM 2004/11/15 : on ne doit pas stopper le mailing si le SMTP nous renvoie une erreur de type "invalid addresses"
                        // si l'erreur provient d'une adresse invalide (exemple : tt@kosmos.fr n'existe pas)
                        //AGA 2010/04/09 ajout de la gestion des erreurs 450 (email valide mais probleme lors de l'envoi)
                        final boolean hasInvalidAddresses = e.getInvalidAddresses() != null && e.getInvalidAddresses().length > 0;
                        final boolean hasValidUnsentAddresses = e.getValidUnsentAddresses() != null && e.getValidUnsentAddresses().length > 0;
                        if (hasInvalidAddresses || hasValidUnsentAddresses) {
                            final String invalidEmails = ArrayUtils.toString(ArrayUtils.add(e.getInvalidAddresses(), e.getValidUnsentAddresses()));
                            texteAnomalies += "\n ERREUR NON BLOQUANTE (le mail n'est pas envoyé à " + invalidEmails + ") : " + e.toString();
                            serviceMailing.delete(currentMailing.getId());
                        } else {
                            LOG.error("Erreur renvoyée par le SMTP", e);
                            //sinon, on stoppe le mailing, car l'erreur ne provient pas d'une mauvaise adresse
                            throw new Exception(e);
                        }
                    } catch (final Exception e) {
                        LOG.debug("unsent");
                        texteAnomalies = addErrorMessage(e, currentMailing, texteAnomalies);
                    }
                    lastUpdate = System.currentTimeMillis();
                    if (init) {
                        init = false;
                    }
                }// fin while
                mailingBeans = serviceMailing.getByIdNewsLetterAndState(id_newsletter, StringUtils.EMPTY, 0, 100);
            }//fin while
            LOG.debug("fin normale thread");
        } catch (final javax.mail.SendFailedException e) {
            LOG.debug("stop mailing");
            // Erreur renvoyée par le SMTP
            // Le problème peut être lié aux autorisation globales du SMTP
            // ('Relay not allowed' quelle que soit l'adresse du destinataire)
            // ou à une adresse incorrecte,
            // on ne detruit donc pas le mail car il peut etre correct
            LOG.error("Erreur renvoyée par le SMTP", e);
            texteAnomalies += "\n ERREUR  BLOQUANTE (arrêt du mailing) " + e.getMessage();
            texteAnomalies += "\n  - Si le problème relève du serveur SMTP ,  relancez l'envoi après y avoir remédié. ";
            texteAnomalies += "\n  - Si le problème relève d'une adresse refusée par le serveur SMTP, supprimer l'enregistrement dans la table MAILING puis relancer l'envoi";
        } catch (final Exception e) {
            LOG.debug("stop mailing");
            LOG.error("arret du mailing", e);
            texteAnomalies += "\n ERREUR BLOQUANTE (arrêt du mailing) :" + e.getMessage();
            texteAnomalies += "\n Relancez le l'envoi après avoir remédié au problème. ";
        }
        /* Envoi du rapport */
        try {
            final JSBMailbox mailbox = new JSBMailbox(false);
            final String sujet = "[ENVOI MAILING] Compte-rendu";
            final String to = NewsletterUtil.getProperty("mail.webmaster");
            String rapport = "";
            if (texteAnomalies.length() == 0) {
                rapport += "Le mailing s'est bien déroulé.";
            } else {
                rapport += "Des anomalies sont apparues : " + texteAnomalies;
            }
            rapport += "\n\nNombre de mails envoyés : " + (nbMailsTotal - nbMailsRestants);
            rapport += "\n\nObjet du mail : " + objetMailing;
            String idModeleMail = id_newsletter;
            if (idModeleMail.contains("_")) {
                idModeleMail = idModeleMail.substring(0, idModeleMail.indexOf("_"));
            }
            final String urlNewsletter = URLResolver.getAbsoluteUrl(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?PROC=RENDER_MODELE_MAIL&ACTION=NEWSLETTER&EXT=" + NewsletterUtil.ID_EXTENSION + "&MODE=APERCU&ID_MODELE_MAIL=" + idModeleMail + "&FORMAT_ENVOI=" + mimeType, new ContexteUniv(""));
            rapport += "\n\nContenu du mail : " + urlNewsletter;
            rapport += "\n\nListe des destinataires : ";
            for (final String[] user : arrayUtilisateur) {
                rapport += "\n" + user[1] + "[" + user[0] + "]";
            }
            rapport += "\n\nCordialement.\n";
            mailbox.sendSystemMsg(to, sujet, rapport);
        } catch (final Exception e) {
            LOG.error("impossible d'envoyer le rapport", e);
        }
    }

    /**
     * En cas d'erreur sur une adresse mail en particulier, on mémorise l'erreur dans l'enregistrement de la base et on ajoute l'erreur au contenu du mail de compte-rendu.
     *
     * @param e
     *            the e
     * @param mailing
     *            the mailing
     * @param texteAnomalies
     *            the texte anomalies
     *
     * @return the string
     *
     * @throws Exception
     *             the exception
     */
    private String addErrorMessage(final Exception e, final MailingBean mailing, String texteAnomalies) throws Exception {
        LOG.error("probleme non connu", e);
        // Suppression du mail
        //     AM 2004/11/15 : en bonus, on affiche l'adresse mail qui bloque
        texteAnomalies += "\n ERREUR NON BLOQUANTE (le mail n'est pas envoyé à " + mailing.getEmail() + ") : " + e.toString();
        //on sauvegarde les adresses qui ne fonctionnent pas
        mailing.setEtat("UNSENT");
        mailing.setErrorMessage(e.getMessage());
        liste_mails_erreur.put(mailing.getEmail(), mailing.getErrorMessage());
        serviceMailing.save(mailing);
        return texteAnomalies;
    }

    /**
     * Gets the id newsletter.
     *
     * @return the id newsletter
     */
    public String getIdNewsletter() {
        return id_newsletter;
    }

    /**
     * Gets the libelle newsletter.
     *
     * @return the libelle newsletter
     */
    public String getLibelleNewsletter() {
        return libelle_newsletter;
    }

    /**
     * Gets the liste mails erreur.
     *
     * @return the liste mails erreur
     */
    public Hashtable<String, String> getListeMailsErreur() {
        return liste_mails_erreur;
    }

    /**
     * Gets the nb mails restants.
     *
     * @return the nb mails restants
     */
    public int getNbMailsRestants() {
        return nbMailsRestants;
    }

    /**
     * Gets the nb mails total.
     *
     * @return the nb mails total
     */
    public int getNbMailsTotal() {
        return nbMailsTotal;
    }
}
