/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Created on 15 nov. 2006
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package com.kosmos.newsletter.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.newsletter.bean.MailingBean;
import com.kosmos.newsletter.service.ServiceMailing;
import com.kosmos.service.impl.ServiceManager;
// TODO: Auto-generated Javadoc

/**
 * The Class GestionnaireMailing.
 */
public class GestionnaireMailing {

    private static final Logger LOG = LoggerFactory.getLogger(GestionnaireMailing.class);

    /** The _instance. */
    private static GestionnaireMailing _instance;

    /** The _liste newsletter. */
    private final Liste _listeNewsletter = new Liste();

    private final ServiceMailing serviceMailing;

    /** The _newsletter en cours. */
    private Newsletter _newsletterEnCours;

    /** The _thread envoi encours. */
    private ThreadMailing _threadEnvoiEncours;

    /** The b envoi. */
    private boolean bEnvoi;

    /**
     * Constructeur prive. La classe est en fait un singleton.
     */
    private GestionnaireMailing() {
        bEnvoi = false;
        serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
    }

    /**
     * Gets the single instance of GestionnaireMailing.
     *
     * @return single instance of GestionnaireMailing
     */
    public synchronized static GestionnaireMailing getInstance() {
        if (_instance == null) {
            _instance = new GestionnaireMailing();
        }
        return _instance;
    }

    /**
     * Inits the envois.
     *
     * @param ctx
     *            the ctx
     * @param idNewsletter
     *            the id newsletter
     *
     * @throws Exception
     *             the exception
     */
    public synchronized void initEnvois(final OMContext ctx, final String idNewsletter) throws Exception {
        // Si pas de mailing en cours, on va vérifier qu'il n'y a pas de newsletters bloquées
        if (!bEnvoi) {
            // On récupère les newsletters non envoyées
            final List<MailingBean> mailingBeans = serviceMailing.getByIdNewsletterAndStateOrdered(idNewsletter, StringUtils.EMPTY, "DESC");
            for (MailingBean currentMailingBean : mailingBeans) {
                final Newsletter news = new Newsletter(currentMailingBean.getIdNewsletter(), currentMailingBean.getObjet(), 0);
                if (!_listeNewsletter.contains(news)) {
                    _listeNewsletter.addFirst(news);
                }
            }
            lanceEnvois();
            bEnvoi = true;
        }
    }

    /**
     * Inits the envois.
     * @param idNewsletter
     *            the id newsletter
     *
     * @throws Exception
     *             the exception
     */
    public synchronized void initEnvois(final String idNewsletter) throws Exception {
        // Si pas de mailing en cours, on va vérifier qu'il n'y a pas de newsletters bloquées
        if (!bEnvoi) {
            // On récupère les newsletters non envoyées
            final List<MailingBean> mailingBeans = serviceMailing.getByIdNewsletterAndStateOrdered(idNewsletter, StringUtils.EMPTY, "DESC");
            for (MailingBean currentMailingBean : mailingBeans) {
                final Newsletter news = new Newsletter(currentMailingBean.getIdNewsletter(), currentMailingBean.getObjet(), 0);
                if (!_listeNewsletter.contains(news)) {
                    _listeNewsletter.addFirst(news);
                }
            }
            lanceEnvois();
            bEnvoi = true;
        }
    }

    /**
     * Indexe l'ensemble des sites en base.
     *
     * @param ctx
     *            contexte
     * @param idNewsletter
     *            the id newsletter
     * @param libelleNewletter
     *            the libelle newletter
     * @param nbDestinataires
     *            the nb destinataires
     *
     * @throws Exception
     *             levee si probleme de chargement
     */
    public void envoiNewsletter(final OMContext ctx, final String idNewsletter, final String libelleNewletter, final int nbDestinataires) throws Exception {
        // On ajoute la newsletter à envoyer si elle n'est pas déjà en pile
        final Newsletter news = new Newsletter(idNewsletter, libelleNewletter, nbDestinataires);
        if (idNewsletter.length() > 0 && !_listeNewsletter.contains(news)) {
            _listeNewsletter.addLast(news);
        }
        initEnvois(ctx, idNewsletter);
    }

    /**
     * Indexe l'ensemble des sites en base.
     *
     * @param idNewsletter
     *            the id newsletter
     * @param libelleNewletter
     *            the libelle newletter
     * @param nbDestinataires
     *            the nb destinataires
     *
     * @throws Exception
     *             levee si probleme de chargement
     */
    public void envoiNewsletter(final String idNewsletter, final String libelleNewletter, final int nbDestinataires) throws Exception {
        // On ajoute la newsletter à envoyer si elle n'est pas déjà en pile
        final Newsletter news = new Newsletter(idNewsletter, libelleNewletter, nbDestinataires);
        if (idNewsletter.length() > 0 && !_listeNewsletter.contains(news)) {
            _listeNewsletter.addLast(news);
        }
        initEnvois(idNewsletter);
    }

    /**
     * Lance le thread dedié a l'envoi. Permet un traitement en tâche de fond.
     */
    protected void lanceEnvois() {
        final Thread threadEnvoi = new Thread() {

            @Override
            public void run() {
                try {
                    if (_listeNewsletter.size() > 0) {
                        Date dateDeb, dateFin;
                        while (_listeNewsletter.size() > 0) {
                            _newsletterEnCours = _listeNewsletter.removeFirst();
                            _threadEnvoiEncours = new ThreadMailing(_newsletterEnCours);
                            final Thread threadMail = new Thread(_threadEnvoiEncours);
                            threadMail.start();
                            dateDeb = new Date();
                            LOG.info("%%%% Début de l'envoi de la newsletter \"" + _newsletterEnCours.getLibelle() + "\" (" + _newsletterEnCours.getId() + ") à " + dateDeb);
                            threadMail.join();
                            dateFin = new Date();
                            LOG.info("%%%% Fin de l'envoi de la newsletter \"" + _newsletterEnCours.getLibelle() + "\" (" + _newsletterEnCours.getId() + ") en " + ((dateFin.getTime() - dateDeb.getTime()) / 1000) + "s");
                        }
                    }
                    bEnvoi = false;
                    _newsletterEnCours = null;
                    _threadEnvoiEncours = null;
                } catch (final Exception e) {
                    LOG.error("erreur dans le thread d'envoi d'email", e);
                }
            }
        };
        threadEnvoi.start();
    }

    /**
     * Gets the liste newsletter en attente.
     *
     * @return the liste newsletter en attente
     */
    public Collection<Newsletter> getListeNewsletterEnAttente() {
        return _listeNewsletter.getCollection();
    }

    /**
     * Gets the thread envoi encours.
     *
     * @return the thread envoi encours
     */
    public ThreadMailing getThreadEnvoiEncours() {
        return _threadEnvoiEncours;
    }

    /**
     * Gets the newsletter en cours.
     *
     * @return the newsletter en cours
     */
    public Newsletter getNewsletterEnCours() {
        return _newsletterEnCours;
    }

    /**
     * The Class Newsletter.
     */
    public class Newsletter {

        /** The id newletter. */
        private String idNewletter = "";

        /** The libelle newletter. */
        private String libelleNewletter = "";

        /** The nb destinataires. */
        private int nbDestinataires = 0;

        /**
         * Instantiates a new newsletter.
         *
         * @param id
         *            the id
         * @param libelle
         *            the libelle
         * @param nb
         *            the nb
         */
        public Newsletter(final String id, final String libelle, final int nb) {
            this.idNewletter = id;
            this.libelleNewletter = libelle;
            this.nbDestinataires = nb;
        }

        /* (non-Javadoc)
         * @see java.lang.Object#equals(java.lang.Object)
         */
        @Override
        public boolean equals(final Object o) {
            if (getId().equals(((Newsletter) o).getId())) {
                return true;
            }
            return false;
        }

        /**
         * Gets the id.
         *
         * @return the id
         */
        public String getId() {
            return idNewletter;
        }

        /**
         * Gets the libelle.
         *
         * @return the libelle
         */
        public String getLibelle() {
            return libelleNewletter;
        }

        /**
         * Gets the nb destinataires.
         *
         * @return the nb destinataires
         */
        public int getNbDestinataires() {
            return nbDestinataires;
        }
    }

    /**
     * The Class Liste.
     */
    protected class Liste {

        /** The liste. */
        private final LinkedList<Newsletter> liste = new LinkedList<Newsletter>();

        /**
         * Adds the last.
         *
         * @param news
         *            the news
         */
        public synchronized void addLast(final Newsletter news) {
            liste.addLast(news);
        }

        /**
         * Adds the first.
         *
         * @param news
         *            the news
         */
        public synchronized void addFirst(final Newsletter news) {
            liste.addFirst(news);
        }

        /**
         * Contains.
         *
         * @param news
         *            the news
         *
         * @return true, if successful
         */
        public synchronized boolean contains(final Newsletter news) {
            return liste.contains(news);
        }

        /**
         * Size.
         *
         * @return the int
         */
        public synchronized int size() {
            return liste.size();
        }

        /**
         * Removes the first.
         *
         * @return the newsletter
         */
        public synchronized Newsletter removeFirst() {
            return liste.removeFirst();
        }

        /**
         * Gets the collection.
         *
         * @return the collection
         */
        public synchronized Collection<Newsletter> getCollection() {
            return new ArrayList<>(liste);
        }
    }
}
