/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.security.MessageDigest;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.controllers.SGController;
import com.kosmos.newsletter.bean.MailingBean;
import com.kosmos.newsletter.bean.ModelemailBean;
import com.kosmos.newsletter.exception.NewsletterException;
import com.kosmos.newsletter.om.Modelemail;
import com.kosmos.newsletter.service.ServiceMailing;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kportal.core.config.PropertyHelper;
import com.univ.batch.LogReportThread;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.services.ServiceUser;
import com.univ.objetspartages.util.RessourceUtils;
import com.univ.utils.ContexteUniv;
import com.univ.utils.URLResolver;

public class ModeleMailThread extends LogReportThread {

    private static final Logger LOG = LoggerFactory.getLogger(ModeleMailThread.class);

    public static final String PROP_DESCRIPTION_LOG_REPORT = "NEWSLETTER.DESCRIPTION_LOG_REPORT";

    public static final String MAILING = "mailing";

    private final ServiceUser serviceUser;

    private final ServiceMailing serviceMailing;

    private final ServiceRessource serviceRessource;

    private final ServiceInfosSiteProcessus serviceInfosSite;

    private ModelemailBean modele;

    private String cheminsFichiers = StringUtils.EMPTY;

    private String libelleFichiers = StringUtils.EMPTY;

    private String id_newsletter = StringUtils.EMPTY;

    private InfosSite site = null;

    public ModeleMailThread() {
        this.serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
        this.serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        this.serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
        this.serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
    }

    public ModeleMailThread(final ModelemailBean modele) {
        this.modele = modele;
        this.id_newsletter = modele.getId() + "_" + System.currentTimeMillis();
        this.serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
        this.serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        this.serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
        this.serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
    }

    private static boolean isValidEmailAddress(final String aEmailAddress) {
        if (aEmailAddress == null) {
            return false;
        }
        boolean result = true;
        try {
            new InternetAddress(aEmailAddress);
            if (!hasNameAndDomain(aEmailAddress)) {
                result = false;
            }
        } catch (final AddressException ex) {
            result = false;
        }
        return result;
    }

    private static boolean hasNameAndDomain(final String aEmailAddress) {
        final String[] tokens = aEmailAddress.split("@");
        return tokens.length == 2 && textHasContent(tokens[0]) && textHasContent(tokens[1]) && tokens[1].contains(".") && !aEmailAddress.contains(";") && !aEmailAddress.contains(" ");
    }

    private static boolean textHasContent(final String aText) {
        return (aText != null) && (aText.trim().length() > 0);
    }

    @Override
    public String getLogFileName() {
        return MAILING + "_" + id_newsletter;
    }

    @Override
    public String getLogDescription() {
        return NewsletterUtil.getMessage(PROP_DESCRIPTION_LOG_REPORT);
    }

    private List<RessourceBean> traiterFichiersJointsModele() {
        StringBuilder listeCheminsFichiers = new StringBuilder(), listeLibelleFichiers = new StringBuilder();
        final List<RessourceBean> vFichiers = serviceRessource.getByCodeParent(modele.getId() + ",TYPE=NEWSLETTER");
        final Iterator<RessourceBean> itFichiers = vFichiers.iterator();
        RessourceBean f;
        while (itFichiers.hasNext()) {
            f = itFichiers.next();
            String extensionFichier = "";
            final String nomFichierJoint = RessourceUtils.getSource(f).toLowerCase();
            int posPoint = -1;
            while (nomFichierJoint.indexOf(".", posPoint + 1) != -1) {
                posPoint = nomFichierJoint.indexOf(".", posPoint + 1);
            }
            if (posPoint != -1) {
                extensionFichier = nomFichierJoint.substring(posPoint + 1);
            }
            if (listeCheminsFichiers.length()>0) {
                listeCheminsFichiers.append(";");
            }
            listeCheminsFichiers.append(RessourceUtils.getPathAbsolu(f));
            if (!"".equals(RessourceUtils.getPathAbsolu(f))) {
                if (listeLibelleFichiers.length()>0) {
                    listeLibelleFichiers.append(";");
                }
                if (!"".equals(RessourceUtils.getTitre(f))) {
                    listeLibelleFichiers.append(RessourceUtils.getTitre(f)).append(".").append(extensionFichier);
                } else if (!RessourceUtils.getSource(f).equals("")) {
                    listeLibelleFichiers.append(nomFichierJoint);
                } else {
                    listeLibelleFichiers.append("fichier.").append(extensionFichier);
                }
            }
        }
        this.libelleFichiers = listeLibelleFichiers.toString();
        this.cheminsFichiers = listeCheminsFichiers.toString();
        return vFichiers;
    }

    private void enregistrerFichiersJointsMailing(MailingBean mailing, List<RessourceBean> vFichiers) {
        for (RessourceBean vFichier : vFichiers) {
            vFichier.setId(null);
            vFichier.setCodeParent(mailing.getId() + ",TYPE=MAILING");
            serviceRessource.save(vFichier);
        }
    }

    @Override
    public void perform() {
        final ContexteUniv ctx = new ContexteUniv("");
        // identifiant unique pour cet envoi de newsletter
        new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
        final String libelle_newletter = modele.getObjet();
        int nbDestinataires = 0;
        int nbErreursEmail = 0;
        int nbErreursCreationMailing = 0;
        try {
            if ( modele != null && StringUtils.defaultString(modele.getCodeRubrique()).length() > 0) {
                site = serviceInfosSite.determineSite(modele.getCodeRubrique(), true);
            } else {
                site = ctx.getInfosSite();
            }

            LOG.info("Enregistrement du mailing newsletter ID : " + id_newsletter + " Libelle : " + libelle_newletter);
            // récupération des fichiers joints de cette newsletter
            List<RessourceBean> vFichiers = traiterFichiersJointsModele();
            /* Parcours des utilisateurs */
            // JSS 20040409 : relation groupes utilisateurs
            final List<String> listeDestinataires = modele.getDestinatairesFinaux();
            if (!listeDestinataires.isEmpty()) {
                // boucle sur les utilisateurs destinataires
                final List<UtilisateurBean> users = serviceUser.getByCodes(listeDestinataires.toArray(new String[listeDestinataires.size()]));
                if (CollectionUtils.isNotEmpty(users)) {
                    // Dans le cas d'une newsletter non destinée à un public
                    // interne (newsletter anonyme),on ne crée qu'un seul objet mailing dans lequel on concatene
                    // les adresse mails de tous les destinataires.
                    if ("0".equals(modele.getPublicInterne())) {
                        final StringBuilder listeEmailHTML = new StringBuilder();
                        final StringBuilder listeEmailTexte = new StringBuilder();
                        for (UtilisateurBean currentUser : users) {
                            if (isValidEmailAddress(currentUser.getAdresseMail())) {
                                // on regarde d'abord le format d'envoi préféré de l'utilisateur puis le format du modele
                                if ("1".equals(currentUser.getFormatEnvoi()) || ("".equals(currentUser.getFormatEnvoi()) && "1".equals(modele.getFormatEnvoi()))) {
                                    listeEmailTexte.append("[").append(currentUser.getAdresseMail()).append("]");
                                } else {
                                    listeEmailHTML.append("[").append(currentUser.getAdresseMail()).append("]");
                                }
                                nbDestinataires++;
                            }
                        }
                        if (listeEmailHTML.length() > 0) {
                            final MailingBean mailing = createMailing(NewsletterUtil.FORMAT_HTML, null, modele.getType());
                            mailing.setEmail(listeEmailHTML.toString());
                            serviceMailing.save(mailing);
                            enregistrerFichiersJointsMailing(mailing, vFichiers);
                        }
                        if (listeEmailTexte.length() > 0) {
                            final MailingBean mailing = createMailing("1", null, modele.getType());
                            mailing.setEmail(listeEmailTexte.toString());
                            serviceMailing.save(mailing);
                            enregistrerFichiersJointsMailing(mailing, vFichiers);
                        }
                        Modelemail.setLastInjection(System.currentTimeMillis());
                    // Dans le cas d'une newsletter destinée à un public
                    // interne (newsletter personalisée),on crée qu'un objet mailing par contenu différent généré dans lequel on concatene
                    // les codes utilisateur de tous les destinataires qui ont le même contenu (hors données personnelles [nom, prenom, email, code]).
                    } else {
                        final MessageDigest digester = MessageDigest.getInstance("MD5");
                        final Map<String,Long> mapMd5Hex= new HashMap<>();
                        final Map<Long,String> mapDestinataires= new HashMap<>();

                        for (UtilisateurBean currentUser : users) {
                            try {
                                final String sAdresseEmail = currentUser.getAdresseMail();
                                // si l'adresse mail est bien renseignée
                                if (isValidEmailAddress(sAdresseEmail)) {
                                    String codeDestinataire = "[".concat(currentUser.getCode()).concat("]");

                                    String formatEnvoi = currentUser.getFormatEnvoi();
                                    if ( StringUtils.isBlank(formatEnvoi) ) {
                                        formatEnvoi = modele.getFormatEnvoi();
                                    }
                                    final MailingBean mailing = createMailing(formatEnvoi, currentUser, modele.getType());
                                    final byte[] md5 =  digester.digest(mailing.getMessage().getBytes());
                                    final String md5Hex = NewsletterUtil.toHexString(md5);
                                    Long idMailing = mapMd5Hex.get(md5Hex);
                                    if ( idMailing==null ) {
                                        serviceMailing.save(mailing);
                                        mapMd5Hex.put(md5Hex, mailing.getId());
                                        mapDestinataires.put(mailing.getId(), codeDestinataire);
                                        enregistrerFichiersJointsMailing(mailing, vFichiers);
                                    } else {
                                        String codeDestinataires = mapDestinataires.get(idMailing).concat(codeDestinataire);
                                        mapDestinataires.put(idMailing, codeDestinataires);
                                    }
                                    Modelemail.setLastInjection(System.currentTimeMillis());
                                    nbDestinataires++;
                                } else {
                                    LOG.info("Adresse mail invalide pour l'utilisateur identifiant : " + currentUser.getCode() + "  mail : " + currentUser.getAdresseMail());
                                    nbErreursEmail++;
                                }
                            } catch (final Exception e) {
                                LOG.info("Erreur sur la generation du mailing pour l'utilisateur identifiant : " + currentUser.getCode() + ", le traitement continue", e.getCause());
                                nbErreursCreationMailing++;
                            }
                            LOG.debug(String.format("Injection mail %s %s", currentUser.getNom(), currentUser.getPrenom()));
                        }// fin while
                        //On enregistre les destinataires en une fois pour limiter les accès BDD.
                        //(Un accès par newsletter au lieu d'un accès par destinataire)
                        for (Map.Entry<Long, String> entry : mapDestinataires.entrySet()) {
                            final MailingBean mailing  = serviceMailing.getById( entry.getKey() );
                            mailing.setCodeUtilisateur(entry.getValue());
                            serviceMailing.save(mailing);
                        }
                    }
                }
            }
            LOG.info("Adresse(s) mail erronée(s) : " + nbErreursEmail);
            LOG.info("Erreur(s) creation mailing : " + nbErreursCreationMailing);
            LOG.info("Enregistrement(s) crée(s) : " + nbDestinataires);
            /* Activation du gestionnaire d'envoi */
            GestionnaireMailing.getInstance().envoiNewsletter(id_newsletter, libelle_newletter, nbDestinataires);
        } catch (final Exception e) {
            LOG.error("Erreur sur l'envoi du mailing : le traitement a été annulé", e);
        } finally {
            ctx.release();
        }
    }

    /**
     * Créé un objet mailing personnalisé à partir des paramètres
     *
     * @param formatEnvoi le format d'envoi de la newsletter
     * @param utilisateur l'utilisateur destinataire
     * @param typeNewsletter le type de newsletter
     * @return MailingBean
     * @throws NewsletterException
     */
    private MailingBean createMailing(final String formatEnvoi, final UtilisateurBean utilisateur, final String typeNewsletter) throws NewsletterException {
        // récupération du contenu de la newsletter par appel de la jsp
        StringBuilder urlNewsletter = new StringBuilder(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY))
            .append("?PROC=RENDER_MODELE_MAIL&ACTION=NEWSLETTER&EXT=").append(NewsletterUtil.ID_EXTENSION)
            .append("&MODE=ENVOI&ID_MODELE_MAIL=").append(modele.getId())
            .append("&FORMAT_ENVOI=").append(formatEnvoi)
            .append("&ID_NEWSLETTER=").append(id_newsletter)
            .append("&CODE_RUBRIQUE=").append(modele.getCodeRubrique());
        if ( utilisateur!=null ) {
            urlNewsletter.append("&EMAIL=").append(utilisateur.getAdresseMail()).append("&CODE_UTILISATEUR=").append(utilisateur.getCode());
        } else {
            urlNewsletter.append("&EMAIL=").append("&CODE_UTILISATEUR=");
        }
        String urlJSp = URLResolver.getAbsoluteUrl(urlNewsletter.toString(), site);
        String corpsMail = generateNewsletter(urlJSp);
        // enregistrement de toutes les infos du mail dans mailing
        final MailingBean mailing = new MailingBean();
        if ( NewsletterUtil.FORMAT_HTML.equals(formatEnvoi)) {
            mailing.setMessage( NewsletterCSSUtil.externalCssToInlineStyles(corpsMail) );
        } else {
            mailing.setMessage( corpsMail.replaceAll("<br />", "\n") );
        }
        mailing.setReplyTo(getReplyTo());
        mailing.setDateCreation(new Date());
        mailing.setObjet(modele.getObjet());
        mailing.setFichier(cheminsFichiers);
        mailing.setNomFichier(libelleFichiers);
        mailing.setIdNewsletter(id_newsletter);
        mailing.setType(typeNewsletter);
        mailing.setTypeMime(formatEnvoi);
        mailing.setDatePurge(modele.getDatePurge());
        mailing.setPublicInterne(modele.getPublicInterne());
        return mailing;
    }

    /**
     * Génère le paramètre replyTo sérialisé.
     * @return Chaîne formatée sous la forme "mail#libellé"
     */
    private String getReplyTo() {
        String replyTo;
        if (StringUtils.isNotBlank(modele.getCodeResponsables())) {
            replyTo = modele.getCodeResponsables();
            String libelle = modele.getLibelleResponsable();
            if (StringUtils.isNotBlank(libelle)) {
                replyTo+= "#" + libelle;
            }
        } else {
            replyTo = NewsletterUtil.getProperty("mail.from");
        }
        return replyTo;
    }

    private String generateNewsletter(String urlJSp) throws NewsletterException {
        try {
            final URL url = new URL(urlJSp);
            try ( final InputStream is = url.openStream();
                  final BufferedReader in = new BufferedReader(new InputStreamReader(is)) ) {
                final StringBuilder buffer = new StringBuilder();
                String line;
                while ((line = in.readLine()) != null) {
                    buffer.append(line);
                }
                return buffer.toString();
            }
        } catch (IOException e) {
            throw new NewsletterException("Erreur lors de la génération de la newsletter ["+urlJSp+"]",e);
        }
    }
}
