/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: newsletter
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.newsletter.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import javax.mail.MessagingException;
import javax.mail.SendFailedException;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.kosmos.controllers.SGController;
import com.univ.multisites.InfosSite;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.services.ServiceUser;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.mail.EmailException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.email.JSBMailbox;
import com.kosmos.newsletter.bean.MailingBean;
import com.kosmos.newsletter.service.ServiceMailing;
import com.kosmos.newsletter.util.GestionnaireMailing.Newsletter;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.batch.LogReportThread;
import com.univ.utils.ContexteUniv;
import com.univ.utils.URLResolver;

/**
 * Envoi de mailing, avec gestion en thread de chaque mail envoyé Date de création : (29/03/00 15:42:15).
 *
 * @author : Jean-Sébastien
 */
public class ThreadMailing extends LogReportThread {

    private static final Logger LOG = LoggerFactory.getLogger(ThreadMailing.class);

    private final ServiceMailing serviceMailing;

    private final ServiceUser serviceUser;

    /** The last update. */
    private static long lastUpdate = 0;

    /** The liste_mails_erreur. */
    private final Hashtable<String, String> liste_mails_erreur = new Hashtable<>();

    /** The id_newsletter. */
    private String id_newsletter = "";

    /** The libelle_newsletter. */
    private String libelle_newsletter = "";

    /** The nb mails total. */
    private int nbMailsTotal = 0;

    /** The nb mails restants. */
    private int nbMailsRestants = 0;

    private int nbErreurs = 0;

    public ThreadMailing() {
        serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
    }

    /**
     * constructeur.
     *
     * @param newsletter
     *            the newsletter
     */
    public ThreadMailing(final Newsletter newsletter) {
        super();
        id_newsletter = newsletter.getId();
        libelle_newsletter = newsletter.getLibelle();
        nbMailsTotal = newsletter.getNbDestinataires();
        serviceMailing = ServiceManager.getServiceForBean(MailingBean.class);
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
    }

    /**
     * Détermine si le thread est actif.
     *
     * @return boolean estactif
     */
    public static boolean estActif() {
        boolean res = true;
        /* false si inactif depuis + de 10 secondes */
        if (System.currentTimeMillis() > lastUpdate + 10000) {
            res = false;
        }
        return res;
    }

    private List<String> prepareDestinataires(MailingBean mailing) {
        final List<String> listDestinataires = new ArrayList<>();

        String destinataires;
        if ( "0".equals(mailing.getPublicInterne()) ) {
            destinataires = mailing.getEmail();
        } else {
            destinataires = mailing.getCodeUtilisateur();
        }

        destinataires = destinataires.replaceAll("\\[", "");
        listDestinataires.addAll(Arrays.asList(destinataires.split("]")));

        return listDestinataires;
    }

    /**
     * lancement.
     */
    @SuppressWarnings("unchecked")
    @Override
    public void perform() {
        StringBuilder texteAnomalies = new StringBuilder();
        final ContexteUniv ctxEnvoi = new ContexteUniv("");
        ctxEnvoi.setConnectionLongue(true);
        final DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
        StringBuilder rapport = new StringBuilder(200);
        final String sFileUsers = WebAppUtil.getLogsPath() + File.separator + getLogFileName() + ".users";
        try {
            final List<String[]> sentUsers = new ArrayList<>();
            List<String[]> oldSentUsers = chargerAvancement(sFileUsers);
            List<String> oldSentMails = new ArrayList<>();
            for (final String[] user : oldSentUsers) {
                oldSentMails.add(user[0]);
            }
            LOG.info("[ThreadMailing " + df.format(new Date(System.currentTimeMillis())) + "] Début envoi newsletter ID : " + id_newsletter);
            final List<MailingBean> mailingBeans = serviceMailing.getByIdNewsLetterAndState(id_newsletter, StringUtils.EMPTY);
            nbMailsTotal = 0;
            for ( MailingBean mailing : mailingBeans) {
                nbMailsTotal += StringUtils.countMatches(mailing.getEmail(), "[");
                nbMailsTotal += StringUtils.countMatches(mailing.getCodeUtilisateur(), "[");
            }
            nbMailsRestants = nbMailsTotal - oldSentMails.size();
            boolean isInitialise = false;
            final int limit = 100;
            final String[] user = new String[2];
            long sleep = NewsletterUtil.getPropertyAsLong("mail.sleep",100);

            // envoi du mailing par tranche 100
            int bouclePause = limit;
            try {
                final JSBMailbox mailbox = new JSBMailbox(false);
                for (MailingBean currentMailing : mailingBeans) {
                    if (libelle_newsletter.length() == 0) {
                        libelle_newsletter = currentMailing.getObjet();
                    }

                    currentMailing.setEtat("UNSENT");
                    serviceMailing.save(currentMailing);

                    String corpsMessage = currentMailing.getMessage();

                    List<String> listDestinataires = prepareDestinataires(currentMailing);
                    String emailDestinataire = StringUtils.EMPTY;

                    for (final String destinataire : listDestinataires) {
                        try {
                            String nomUtilisateur = StringUtils.EMPTY;
                            //Affichage personnalisé : on ne connait que le code. On va donc charger l'utilisateur et personnaliser le mail
                            if ( "1".equals(currentMailing.getPublicInterne()) ) {
                                final UtilisateurBean utilisateur = serviceUser.getByCode(destinataire);
                                emailDestinataire = utilisateur.getAdresseMail();
                                nomUtilisateur = utilisateur.getNom() + " " + utilisateur.getPrenom();
                                corpsMessage = NewsletterUtil.personnaliserMessage(currentMailing.getMessage(), utilisateur);
                            } else {
                                emailDestinataire = destinataire;
                            }
                            // si le mail n'a pas déjà été envoyé
                            if (!oldSentMails.contains(emailDestinataire)) {
                                user[0] = emailDestinataire;
                                user[1] = nomUtilisateur;
                                String[] tabFichiers = null;
                                String[] tabNomFichiers = null;
                                if (!isInitialise) {
                                    String fichiers = currentMailing.getFichier();
                                    if (fichiers.contains(";")) {
                                        tabFichiers = fichiers.split(";");
                                    } else if (!fichiers.equals("")) {
                                        tabFichiers = new String[1];
                                        tabFichiers[0] = fichiers;
                                    }
                                    String nomFichiers = currentMailing.getNomFichier();
                                    if (nomFichiers.contains(";")) {
                                        tabNomFichiers = nomFichiers.split(";");
                                    } else if (!nomFichiers.equals("")) {
                                        tabNomFichiers = new String[1];
                                        tabNomFichiers[0] = nomFichiers;
                                    }
                                    isInitialise = true;
                                }
                                //envoi du mail
                                String logMessage = String.format("au format %s à <%s>", (currentMailing.getTypeMime().equals("0") ? "html" : "texte"), emailDestinataire);
                                LOG.debug("Envoi du mail ".concat(logMessage));
                                mailbox.sendMsgWithAttachedFiles(currentMailing.getReplyTo(), user, currentMailing.getObjet(), corpsMessage, currentMailing.getTypeMime(), tabFichiers, tabNomFichiers);
                                LOG.info("Mail envoyé ".concat(logMessage));
                                bouclePause--;
                                sentUsers.add(new String[] {user[0], user[1]});
                                nbMailsRestants--;
                            }
                        } catch (final SendFailedException e) {
                            // on ne doit pas stopper le mailing si le SMTP nous renvoie une erreur de type "invalid addresses"
                            final boolean hasInvalidAddresses = e.getInvalidAddresses()!=null && e.getInvalidAddresses().length>0;
                            final boolean hasValidUnsentAddresses = e.getValidUnsentAddresses()!=null && e.getValidUnsentAddresses().length>0;
                            if (hasInvalidAddresses || hasValidUnsentAddresses) {
                                texteAnomalies.append(addErrorMessage(e, currentMailing, emailDestinataire));
                                nbErreurs++;
                            }
                            // sinon, on stoppe le mailing, car l'erreur provient du SMTP
                            else {
                                throw e;
                            }
                        } catch (final MessagingException | EmailException e) {
                            texteAnomalies.append(addErrorMessage(e, currentMailing, emailDestinataire));
                            nbErreurs++;
                        }
                        if (bouclePause == 0) {
                            if (sleep > 0) {
                                Thread.sleep(sleep);
                            }
                            bouclePause = limit;
                        }
                        lastUpdate = System.currentTimeMillis();
                    } // fin boucle for
                    currentMailing.setEtat("SENT");
                    serviceMailing.save(currentMailing);

                    // reinitialisation pour le mailing suivant
                    isInitialise = false;
                } // fin for
                LOG.debug("fin normale thread");
                rapport.append("Le mailing s'est terminé normalement.");
                LOG.info(rapport.toString());
            } catch (final InterruptedException | SendFailedException e ) {
                LOG.error("Arrêt du mailing", e);
                rapport.append("Le mailing ne s'est pas terminé normalement.\n\nUne erreur bloquante est survenue : ").append(e.getMessage());
                // Erreur renvoyée par le SMTP
                // Le problème peut être lié aux autorisation globales du SMTP ('Relay not allowed' quelle que soit l'adresse du destinataire) ou à une adresse incorrecte,
                // On ne detruit donc pas le mail car il peut etre correct
                if (e instanceof javax.mail.SendFailedException) {
                    rapport.append("\nSi le problème relève du serveur SMTP, relancez l'envoi après y avoir remédié.");
                    rapport.append("\nSi le problème relève d'une adresse refusée par le serveur SMTP, supprimez l'enregistrement dans la table MAILING puis relancer l'envoi.");
                } else {
                    rapport.append("\nRelancez l'envoi après avoir remédié au problème.");
                }
                // sauvegarde la liste des utilisateurs et des erreurs pour relance
                try {
                    // maj de l'état pour proposer une relance manuelle
                    for(MailingBean currentMailing : mailingBeans) {
                        currentMailing.setEtat("");
                        serviceMailing.save(currentMailing);
                    }
                } catch (final DataSourceException ex) {
                    LOG.error("Erreur de sauvegarde de l'état du mailing");
                }
                sauvegarderAvancement(sFileUsers,oldSentUsers, sentUsers);
            }
            envoyerRapport(rapport.toString(), texteAnomalies.toString(), oldSentUsers, sentUsers, mailingBeans.get(0));
        } finally {
            // libération contexte
            ctxEnvoi.release();
        }
    }

    private List<String[]> chargerAvancement(String sFileUsers) {
        List<String[]> oldSentUsers = new ArrayList<>();
        if (new File(sFileUsers).exists()) {
            try (
                final FileInputStream fis = new FileInputStream(sFileUsers);
                final ObjectInputStream ois = new ObjectInputStream(fis)
            ){
                oldSentUsers = (ArrayList<String[]>)ois.readObject();
                LOG.info("Chargement de la liste des destinataires traités");
                boolean result = new File(sFileUsers).delete();
                if ( !result ) {
                    LOG.error("Impossible de supprimer le fichier des destinataires traités");
                }
            } catch (final IOException | ClassNotFoundException e) {
                LOG.error("Erreur de chargement de la liste des destinataires traités", e);
            }
        }
        return oldSentUsers;
    }

    private void sauvegarderAvancement(String sFileUsers, List<String[]> oldSentUsers, List<String[]> sentUsers ) {
        try (
            FileOutputStream fos = new FileOutputStream(sFileUsers);
            ObjectOutputStream oos = new ObjectOutputStream(fos)
        ) {
            final ArrayList<String[]> allSentUsers = new ArrayList<>();
            allSentUsers.addAll(oldSentUsers);
            allSentUsers.addAll(sentUsers);
            // on ajoute les erreurs aux mails traités
            if (liste_mails_erreur.size() > 0) {
                for (final String mail : liste_mails_erreur.keySet()) {
                    allSentUsers.add(new String[] {mail, ""});
                }
            }
            oos.writeObject(allSentUsers);

            LOG.info("Sauvegarde des destinataires traités");
        } catch (final IOException e) {
            LOG.error("Erreur de sauvegarde des utilisateurs traités", e);
        }
    }

    private void envoyerRapport(String texte, String texteAnomalies, List<String[]> oldSentUsers, List<String[]> sentUsers, MailingBean mailing ) {
        LOG.info("Envoi du compte rendu par mail");

        final boolean reprise = oldSentUsers !=null && oldSentUsers.size() > 0;
        final String sujet = "[Mailing] Compte-rendu " + (reprise ? "reprise " : "") + "envoi newsletter : " + mailing.getObjet();
        LOG.info("Sujet : " + sujet);
        StringBuilder rapport = new StringBuilder(texte);
        String logMessage = "Nombre de mail total : " + nbMailsTotal;
        LOG.info(logMessage);
        rapport.append("\n\n").append(logMessage);
        if (reprise) {
            logMessage = "Nombre de mail restant à traiter : " + (nbMailsTotal - oldSentUsers.size());
            LOG.info(logMessage);
            rapport.append("\n\n").append(logMessage);
        }
        logMessage = "Nombre de mail envoyé : " + sentUsers.size();
        LOG.info(logMessage);
        rapport.append("\n").append(logMessage);
        if (nbErreurs > 0) {
            logMessage = "Nombre d'erreurs survenues : " + nbErreurs;
            LOG.info(logMessage);
            rapport.append("\n").append(logMessage).append(texteAnomalies);
        }
        rapport.append("\n\nObjet du mail : ").append(mailing.getObjet());
        String idModeleMail = id_newsletter;
        if (idModeleMail.contains("_")) {
            idModeleMail = idModeleMail.substring(0, idModeleMail.indexOf("_"));
        }
        final String urlNewsletter = URLResolver.getAbsoluteUrl(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?PROC=RENDER_MODELE_MAIL&ACTION=NEWSLETTER&EXT=" + NewsletterUtil.ID_EXTENSION + "&MODE=APERCU&ID_MODELE_MAIL=" + idModeleMail + "&FORMAT_ENVOI=" + mailing.getTypeMime(), (InfosSite) null);
        rapport.append("\n\nContenu du mail : ")
            .append(urlNewsletter)
            .append("\n\nListe des destinataires ayant reçu le mail :\n");
        for (final String[] currentUser : sentUsers) {
            if ( StringUtils.isNotBlank(currentUser[1]) ) {
                rapport.append(currentUser[1]).append(" ");
            }
            rapport.append("<").append(currentUser[0]).append(">\n");
        }
        rapport.append("\nCordialement.\n");
        final JSBMailbox mailbox = new JSBMailbox(false);
        String to;
        if (StringUtils.isNotBlank(mailing.getReplyTo())) {
            to = mailing.getReplyTo();
        } else {
            to = PropertyHelper.getCoreProperty("mail.webmaster");
        }
        try {
            mailbox.sendSystemMsg(to, sujet, rapport.toString());
        } catch (final EmailException | MessagingException e) {
            LOG.error("Erreur dans l'envoi du compte rendu par mail", e);
        }
        LOG.info("Fin envoi compte rendu");
    }

    /**
     * En cas d'erreur sur une adresse mail en particulier, on mémorise l'erreur dans l'enregistrement de la base et on ajoute l'erreur au contenu du mail de compte-rendu.
     *
     * @param e
     *            the e
     * @param mailing
     *            the mailing
     *
     * @return the string
     *
     */
    private String addErrorMessage(final Exception e, final MailingBean mailing, String sDestinataire) {
        LOG.error("mail non envoyé à " + sDestinataire, e);
        String message = "";
        if (e.getMessage() != null) {
            message = e.getMessage();
        }
        // mailing personnalisé on note l'état
        if ( "1".equals(mailing.getPublicInterne()) ) {
            sDestinataire = mailing.getEmail();
            mailing.setEtat("UNSENT");
            mailing.setErrorMessage(message);
            serviceMailing.save(mailing);
        }
        String result = String.format("\n - mail non envoyé à <%s> : %s",sDestinataire,e.toString());
        //on sauvegarde les adresses qui ne fonctionnent pas
        liste_mails_erreur.put(sDestinataire, message);
        return result;
    }

    /**
     * Gets the id newsletter.
     *
     * @return the id newsletter
     */
    public String getIdNewsletter() {
        return id_newsletter;
    }

    /**
     * Gets the libelle newsletter.
     *
     * @return the libelle newsletter
     */
    public String getLibelleNewsletter() {
        return libelle_newsletter;
    }

    /**
     * Gets the liste mails erreur.
     *
     * @return the liste mails erreur
     */
    public Hashtable<String, String> getListeMailsErreur() {
        return liste_mails_erreur;
    }

    /**
     * Gets the nb mails restants.
     *
     * @return the nb mails restants
     */
    public int getNbMailsRestants() {
        return nbMailsRestants;
    }

    /**
     * Gets the nb mails total.
     *
     * @return the nb mails total
     */
    public int getNbMailsTotal() {
        return nbMailsTotal;
    }

    @Override
    public String getLogFileName() {
        return ModeleMailThread.MAILING + "_" + id_newsletter;
    }

    @Override
    public String getLogDescription() {
        return NewsletterUtil.getMessage(ModeleMailThread.PROP_DESCRIPTION_LOG_REPORT);
    }
}
