/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.offreformation.cdm.processus;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.email.JSBMailbox;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.offreformation.cdm.module.ModuleCDM;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.extension.ExtensionHelper;
import com.kportal.extension.module.IModule;
import com.ksup.objetspartages.om.Cours;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.Ueup;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.ContexteUtil;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.UnivWebFmt;

/**
 * Classe permettant d'exporter via un processus du back-office:
 * <ul>
 * <li>les structures</li>
 * <li>les formations (diplômes et programmes)</li>
 * <li>les éléments pédagogiques</li>
 * <li>les cours</li>
 * <li>les personnes responsables de ces contenus et provenant de l'annuaire</li>
 * </ul>
 * depuis K-Sup vers un format compatible CDM (Course Description Metadata).<br />
 * <br />
 * Le format CDM retenu correspond à la version CDM-2.0.2 et au profil CDM-FR_0.8.<br />
 * <br />
 * L'export est réservé aux utilisateurs possédant le rôle webmaster<br />
 * <br />
 * Le fichier XML généré peut-être soit téléchargé directement, soit envoyé par email à une adresse choisie par l'utilisateur.<br />
 * <br />
 * Cet export se base sur SAX pour des raisons d'utilisation mémoire, le fichier XML généré pouvant être extrémement volumineux.
 *
 * @see <a href="http://www2.educnet.education.fr/sections/superieur/soutien/normes/juillet_2005__publi">EducNet</a>
 * @author Julien ROZO
 * @version 2.0
 *
 * @deprecated
 * Code legacy qui sera supprimé lors de la refonte. 
 *
 */
@Deprecated
public class Exportcdm extends ProcessusBean {

    private static final Logger LOG = LoggerFactory.getLogger(Exportcdm.class);

    private static final String ECRAN_EXPORT = "ECRAN_EXPORT";

    private static final String ECRAN_RESULTAT = "RESULTAT";

    private static final String EN_LIGNE = "0003";

    private static final long MEGA_OCTET = 1048576;

    private static final long KILO_OCTET = 1024;

    private static final String prefixeExportcdm = "exportcdm.";

    private static final String suffixeExportActif = ".actif";

    private final HashSet<String> hsCodesStructures = new HashSet<>();

    private final HashSet<AnnuaireModele> hsAnnuairesResponsables = new HashSet<>();

    private final HashSet<String> hsCodesUeupsDansParcours = new HashSet<>();

    private final HashSet<String> hsCodesCoursDansUeup = new HashSet<>();

    private boolean exportActif = true;

    private boolean exporterDefaultstructureksup = true;

    private boolean exporterLaboratoire = false;

    private boolean exporterFormation = true;

    private boolean exporterUeup = true;

    private boolean exporterCours = false;

    private boolean exporterAnnuaireksup = true;

    private long valeurCritiqueEnvoi = MEGA_OCTET;

    private IModule moduleCDM = null;

    /**
     * constructeur
     *
     * @param ciu
     *            com.jsbsoft.jtf.core.InfoBean
     */
    public Exportcdm(final InfoBean ciu) {
        super(ciu);
    }

    /**
     * Point d'entrée du processus
     */
    @Override
    public boolean traiterAction() {
        try {
            moduleCDM = OffreFormationUtil.getModule(ModuleCDM.ID_BEAN);
            ecranLogique = infoBean.getEcranLogique();
            action = infoBean.getActionUtilisateur();
            final AutorisationBean autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
            this.getDatas().get(OMContext.CLE_SERVLET_SG_REQUETE_HTTP);
            if (autorisations == null || !autorisations.isWebMaster()) {
                infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
                infoBean.setEcranLogique("LOGIN");
            } else {
                if (ecranLogique == null) {
                    preparerEXPORT();
                } else if (ecranLogique.equals(ECRAN_EXPORT)) {
                    traiterEXPORT();
                } else {
                    LOG.error("Action non prise en charge par l'export CDM !!");
                }
                //placer l'état dans le composant d'infoBean
                infoBean.setEcranLogique(ecranLogique);
            }
        } catch (final Exception e) {
            LOG.error("erreur lors du traitemetn de l'action", e);
            infoBean.addMessageErreur(e.toString());
        }
        /* On continue si on n'est pas à la FIN !!! */
        return etat == FIN;
    }

    /**
     * Prépare l'écran logique pour l'envoi de l'export via email
     */
    private void preparerEXPORT() {
        ecranLogique = ECRAN_EXPORT;
    }

    /**
     * Traitement de l'export CDM
     *
     * @throws Exception
     */
    private void traiterEXPORT() {
        try {
            if (action.equals(InfoBean.ACTION_VALIDER)) {
                String envoi = infoBean.getString("ENVOI");
                if ("1".equals(envoi) && infoBean.getString("EMAIL").length() == 0) {
                    envoi = "0";
                }
                /* on vérifie que le module est activé */
                exportActif = isExportActif();
                if (exportActif) {
                    /* on vérifie quels exports sont activés */
                    exporterDefaultstructureksup = ("1".equals(infoBean.get("STRUCTURE")));
                    exporterLaboratoire = ("1".equals(infoBean.get("LABORATOIRE")));
                    exporterFormation = ("1".equals(infoBean.get("FORMATION")));
                    exporterUeup = ("1".equals(infoBean.get("UEUP")));
                    exporterCours = ("1".equals(infoBean.get("COURS")));
                    exporterAnnuaireksup = ("1".equals(infoBean.get("ANNUAIRE")));
                    /* on ne lance l'export global que s'il y a au moins un type de contenus à exporter */
                    if (exporterDefaultstructureksup || exporterLaboratoire || exporterFormation || exporterUeup || exporterCours || exporterAnnuaireksup) {
                        final String dataSource = this.getInfosSite().getIntitule() + " - " + getApplicationName();
                        final String date = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
                        final String time = new SimpleDateFormat("HH-mm-ss").format(new Date());
                        /* Le fichier XML généré est sauvegardé dans un fichier temporaire, qui est ensuite
                         * soit attaché au mail envoyé, soit proposé au téléchargement */
                        final String nomFichierXml = "exportcdm_" + date + "_" + time + ".xml";
                        final String cheminFichierXml = WebAppUtil.getDownloadPrivatePath() + File.separator + nomFichierXml;
                        final File fichierXml = new File(cheminFichierXml);
                        try (FileOutputStream osFichierXml = new FileOutputStream(fichierXml)) {
                            final OutputStreamWriter osWriter = new OutputStreamWriter(osFichierXml, CharEncoding.DEFAULT);
                            final StreamResult srFichierXml = new StreamResult(osWriter);
                            long tailleFichier = 0;
                        /* on exporte les contenus dans ce fichier temporaire */
                            export(srFichierXml, dataSource, date, time);
                            osWriter.close();
                        /* On a fini d'écrire le flux dans le fichier, on ferme le pointeur */
                            osFichierXml.close();
                        /* on peut récupérer la taille du fichier généré */
                            tailleFichier = fichierXml.length();
                        /* 2 traitements exclusifs : soit le fichier généré est envoyé par email,
                         * soit il est proposé au téléchargement */
                            if (envoi.equals("1")) {
                                envoyerFichierParEmail(cheminFichierXml);
                            /* on récupère dans le fichier .properties la taille critique à partir de laquelle le fichier joint risque d'être filtré */
                                if (StringUtils.isNotEmpty(moduleCDM.getProperty(prefixeExportcdm + "envoi.taille_critique"))) {
                                    valeurCritiqueEnvoi = Long.parseLong(moduleCDM.getProperty(prefixeExportcdm + "envoi.taille_critique"));
                                    valeurCritiqueEnvoi = valeurCritiqueEnvoi * KILO_OCTET; // on convertit la valeur saisie en octets pour une comparaison avec la taille du fichier
                                }
                                if (tailleFichier > valeurCritiqueEnvoi) {
                                    String uniteTaille = "octets";
                                    if (tailleFichier > MEGA_OCTET) {
                                        tailleFichier = tailleFichier / MEGA_OCTET;
                                        uniteTaille = "Mo";
                                    } else if (tailleFichier > KILO_OCTET) {
                                        tailleFichier = tailleFichier / KILO_OCTET;
                                        uniteTaille = "Ko";
                                    }
                                    infoBean.set("RESULTAT", OffreFormationUtil.getMessage("EXPORT_CDM.RESULTAT.EXPORT_OK") + "<br />" + String.format(OffreFormationUtil.getMessage("EXPORT_CDM.RESULTAT.EXPORT_OK_TAILLE_MAX"), tailleFichier, uniteTaille));
                                } else {
                                    infoBean.set("RESULTAT", OffreFormationUtil.getMessage("EXPORT_CDM.RESULTAT.EXPORT_OK"));
                                }
                            } else {
                                infoBean.set("NOMFICHIERXML", nomFichierXml);
                                infoBean.set("RESULTAT", OffreFormationUtil.getMessage("EXPORT_CDM.RESULTAT.EXPORT_OK"));
                            }
                        }
                    } else{
                        infoBean.set("RESULTAT", OffreFormationUtil.getMessage("EXPORT_CDM.MODULE.CONTENU_INACTIF"));
                    }
                } else {
                    infoBean.set("RESULTAT", OffreFormationUtil.getMessage("EXPORT_CDM.MODULE.INACTIF"));
                }
                infoBean.set("ENVOI", envoi);
                ecranLogique = ECRAN_RESULTAT;
            }
        } catch (final Exception e) {
            LOG.error("Erreur lors du traitement de l'export CDM", e);
        }
    }

    /**
     * l'export des contenus doit toujours se faire dans cet ordre :
     * <ul>
     * <li>structures = orgUnit</li>
     * <li>formations = program</li>
     * <li>ueup = course</li>
     * <li>cours = course</li>
     * <li>annuaire = person</li>
     * </ul>
     *
     * @param sr
     *            le StreamResult recevant le flux XML généré
     * @param dataSource
     *            le nom de l'application générant le flux XML
     * @param date
     *            la date de l'export au format yyyy-MM-dd
     * @param time
     *            l'heure de l'export au format HH:mm:ss
     */
    private void export(final StreamResult sr, final String dataSource, final String date, final String time) {
        try {
            /* La génération du flux XML se fait avec SAX pour restreindre la consommation de ressources */
            final SAXTransformerFactory tf = (SAXTransformerFactory) TransformerFactory.newInstance();
            final TransformerHandler hd = tf.newTransformerHandler();
            final Transformer transformer = hd.getTransformer();
            final AttributesImpl attsImpl = new AttributesImpl();
            /* On positionne quelques propriétés du flux XML */
            transformer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            /* le flux généré est écrit dans le StreamResult passé en paramètre de cette méthode */
            hd.setResult(sr);
            /* La génération du flux XML débute */
            hd.startDocument();
            /* La racine de ce flux est le noeud CDM */
            attsImpl.addAttribute("", "", "language", "language", "fr-FR");
            attsImpl.addAttribute("", "", "version", "token", "CDM-2.0.2");
            attsImpl.addAttribute("", "", "profile", "token", "CDM-fr_0.8");
            attsImpl.addAttribute("", "", "xmlns:xsi", "text", "http://www.w3.org/2001/XMLSchema-instance");
            attsImpl.addAttribute("", "", "xsi:schemaLocation", "text", "http://cdm-fr.fr/2006/CDM-frSchema http://cdm-fr.fr/2006/schemas/CDM-fr.xsd");
            attsImpl.addAttribute("", "", "xmlns", "text", "http://cdm-fr.fr/2006/CDM-frSchema");
            hd.startElement("", "", "CDM", attsImpl);
            attsImpl.clear();
            /* on indique quelques propriétés du flux CDM généré */
            hd.startElement("", "", "properties", attsImpl);
            hd.startElement("", "", "datasource", attsImpl);
            hd.characters(dataSource.toCharArray(), 0, dataSource.length());
            hd.endElement("", "", "datasource");
            attsImpl.addAttribute("", "", "date", "token", date);
            attsImpl.addAttribute("", "", "time", "token", time);
            hd.startElement("", "", "datetime", attsImpl);
            hd.endElement("", "", "datetime");
            attsImpl.clear();
            hd.endElement("", "", "properties");
            /* on décrit les structures : defaultstructureksup et laboratoire */
            if (exporterDefaultstructureksup || exporterLaboratoire) {
                initialiserStructures();
                ajouterNoeudsStructures(hd);
            }
            /* on décrit les formations et les parcours associés à ces formations */
            if (exporterFormation) {
                ajouterNoeudsFormations(hd);
            }
            /* on décrit les UEUP */
            if (exporterUeup) {
                ajouterNoeudsUeups(hd);
            }
            /* on décrit les cours */
            if (exporterCours) {
                ajouterNoeudsCours(hd);
            }
            /* on décrit les personnes responsables des fiches qui ont été précédemment décrites */
            if (exporterAnnuaireksup) {
                ajouterNoeudsAnnuaires(hd);
            }
            /* on termine le flux CDM */
            hd.endElement("", "", "CDM");
            /* la génération du flux XML est terminée */
            hd.endDocument();
        } catch (final Exception e) {
            LOG.error("Erreur à la génération de l'export CDM", e);
        }
    }

    /**
     * Garde en référence tous les defaultstructureksup et laboratoires qui vont être exportés, si les exports pour ces types de contenu sont activés.<br />
     * Cela permet par la suite de savoir quelles fiches de ce type seront utilisés comme refOrgUnit dans les contenus exportés.
     *
     */
    private void initialiserStructures() {
        try {
            if (exporterDefaultstructureksup) {
                for (final String codeFiche : FicheUnivHelper.getListeFiche("DEFAULTSTRUCTUREKSUP", EN_LIGNE, "").keySet()) {
                    hsCodesStructures.add(codeFiche);
                }
            }
            if (exporterLaboratoire) {
                for (final String codeFiche : FicheUnivHelper.getListeFiche("LABORATOIRE", EN_LIGNE, "").keySet()) {
                    hsCodesStructures.add(codeFiche);
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'initialisation des structures à récupérer", e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des defaultstructureksup et laboratoires exportées depuis K-Sup (noeud orgUnit)
     *
     * @param _hd
     *            hd le TransformerHandler dans lequel les noeuds orgUnit doivent être ajoutés
     * @throws Exception
     */
    private void ajouterNoeudsStructures(final TransformerHandler _hd) {
        if (exporterDefaultstructureksup) {
            ajouterNoeudsDefaultstructureksups(_hd);
        }
        if (exporterLaboratoire) {
            ajouterNoeudsLaboratoires(_hd);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des defaultstructureksup exportées depuis K-Sup dans un format compatible CDM (noeud orgUnit)
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds orgUnit doivent être ajoutés
     */
    private void ajouterNoeudsDefaultstructureksups(final TransformerHandler hd) {
        try {
            final AttributesImpl _attsImpl = new AttributesImpl();
            String valeurNoeud = "";
            final StructureModele defaultstructureksup = (StructureModele) ReferentielObjets.instancierFiche("DEFAULTSTRUCTUREKSUP");
            defaultstructureksup.setCtx(this);
            defaultstructureksup.init();
            /* On récupère toutes les structures existantes, puis on mappe des champs du modèle K-Sup
             * vers le modèle CDM */
            if (defaultstructureksup.selectCodeLangueEtat("", "", EN_LIGNE) > 0) {
                while (defaultstructureksup.nextItem()) {
                    _attsImpl.addAttribute("", "", "ident", "token", defaultstructureksup.getCode());
                    _attsImpl.addAttribute("", "", "language", "language", getXsLanguage(defaultstructureksup.getLangue()));
                    hd.startElement("", "", "orgUnit", _attsImpl);
                    _attsImpl.clear();
                    valeurNoeud = defaultstructureksup.getCode();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "orgUnitID", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitID");
                    }
                    valeurNoeud = defaultstructureksup.getLibelleLong();
                    hd.startElement("", "", "orgUnitName", _attsImpl);
                    hd.startElement("", "", "text", _attsImpl);
                    hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                    hd.endElement("", "", "text");
                    hd.endElement("", "", "orgUnitName");
                    valeurNoeud = defaultstructureksup.getLibelleCourt();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "orgUnitAcronym", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitAcronym");
                    }
                    valeurNoeud = defaultstructureksup.getCode();
                    if (valeurNoeud.length() > 0) {
                        _attsImpl.addAttribute("", "", "codeSet", "orgUnitCodeValueSet", "userDefined");
                        hd.startElement("", "", "orgUnitCode", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitCode");
                        _attsImpl.clear();
                    }
                    valeurNoeud = defaultstructureksup.getTypeStructure();
                    if (valeurNoeud.length() > 0 && !"0000".equals(valeurNoeud)) {
                        if (FicheUnivHelper.getLibelle(defaultstructureksup, "cdm_orgunitkind", defaultstructureksup.getTypeStructure()).length() > 0) {
                            _attsImpl.addAttribute("", "", "orgType", "orgUnitKindValueType", FicheUnivHelper.getLibelle(defaultstructureksup, "cdm_orgunitkind", defaultstructureksup.getTypeStructure()));
                        }
                        valeurNoeud = LabelUtils.getLibelle("60", valeurNoeud, LangueUtil.getLocale(defaultstructureksup.getLangue()));
                        hd.startElement("", "", "orgUnitKind", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitKind");
                        _attsImpl.clear();
                    }
                    ajouterUrlFiche(hd, defaultstructureksup);
                    valeurNoeud = formaterEnHtml(defaultstructureksup.getActivite());
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "orgUnitDescription", _attsImpl);
                        hd.startElement("", "", "subBlock", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "subBlock");
                        hd.endElement("", "", "orgUnitDescription");
                    }
                    if (defaultstructureksup.getAdresse().length() > 0 || defaultstructureksup.getCodePostal().length() > 0 || defaultstructureksup.getVille().length() > 0 || defaultstructureksup.getTelephone().length() > 0 || defaultstructureksup.getTelecopie().length() > 0 || defaultstructureksup.getAdresseMail().length() > 0 || Formateur.estSaisi(defaultstructureksup.getSiteWeb()) || defaultstructureksup.getComplementCoordonnees().length() > 0) {
                        hd.startElement("", "", "contacts", _attsImpl);
                        hd.startElement("", "", "contactData", _attsImpl);
                        if (defaultstructureksup.getAdresse().length() > 0 || defaultstructureksup.getCodePostal().length() > 0 || defaultstructureksup.getVille().length() > 0) {
                            hd.startElement("", "", "adr", _attsImpl);
                            valeurNoeud = defaultstructureksup.getAdresse();
                            if (valeurNoeud.length() > 0) {
                                hd.startElement("", "", "street", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "street");
                            }
                            valeurNoeud = defaultstructureksup.getVille();
                            if (valeurNoeud.length() > 0) {
                                hd.startElement("", "", "locality", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "locality");
                            }
                            valeurNoeud = defaultstructureksup.getCodePostal();
                            if (valeurNoeud.length() > 0) {
                                hd.startElement("", "", "pcode", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "pcode");
                            }
                            hd.endElement("", "", "adr");
                        }
                        valeurNoeud = defaultstructureksup.getTelephone();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "telephone", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "telephone");
                        }
                        valeurNoeud = defaultstructureksup.getTelecopie();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "fax", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "fax");
                        }
                        valeurNoeud = defaultstructureksup.getAdresseMail();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "email", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "email");
                        }
                        valeurNoeud = defaultstructureksup.getSiteWeb();
                        if (Formateur.estSaisi(valeurNoeud)) {
                            hd.startElement("", "", "webLink", _attsImpl);
                            hd.startElement("", "", "href", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "href");
                            hd.endElement("", "", "webLink");
                        }
                        /*if (defaultstructureksup.getComplementCoordonnees().length() > 0 ||
                                defaultstructureksup.getContacts().length() > 0) {*/
                        if (defaultstructureksup.getComplementCoordonnees().length() > 0) {
                            hd.startElement("", "", "infoBlock", _attsImpl);
                            valeurNoeud = formaterEnHtml(defaultstructureksup.getComplementCoordonnees());
                            if (valeurNoeud.length() > 0) {
                                hd.startElement("", "", "subBlock", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "subBlock");
                            }
                            /*
                                valeurNoeud = formaterEnHtml(defaultstructureksup.getContacts());
                                if (valeurNoeud.length() > 0) {
                                    hd.startElement("", "", "subBlock", _attsImpl);
                                    hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                    hd.endElement("", "", "subBlock");
                                }*/
                            hd.endElement("", "", "infoBlock");
                        }
                        hd.endElement("", "", "contactData");
                        hd.endElement("", "", "contacts");
                    }
                    ajouterResponsables(hd, defaultstructureksup.getCodeResponsable());
                    ajouterStructuresRattachement(hd, defaultstructureksup);
                    valeurNoeud = defaultstructureksup.getMetaKeywords();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "searchword", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "searchword");
                    }
                    hd.endElement("", "", "orgUnit");
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des defaultstructureksup dans l'export CDM", e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des laboratoires exportés depuis K-Sup dans un format compatible CDM (noeud orgUnit)
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds orgUnit doivent être ajoutés
     */
    private void ajouterNoeudsLaboratoires(final TransformerHandler hd) {
        try {
            final AttributesImpl _attsImpl = new AttributesImpl();
            String valeurNoeud = "";
            final StructureModele laboratoire = (StructureModele) ReferentielObjets.instancierFiche("LABORATOIRE");
            laboratoire.setCtx(this);
            laboratoire.init();
            /* On récupère tous les laboratoires existants, puis on mappe des champs du modèle K-Sup
             * vers le modèle CDM */
            if (laboratoire.selectCodeLangueEtat("", "", EN_LIGNE) > 0) {
                while (laboratoire.nextItem()) {
                    _attsImpl.addAttribute("", "", "ident", "token", laboratoire.getCode());
                    _attsImpl.addAttribute("", "", "language", "language", getXsLanguage(laboratoire.getLangue()));
                    hd.startElement("", "", "orgUnit", _attsImpl);
                    _attsImpl.clear();
                    valeurNoeud = laboratoire.getCode();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "orgUnitID", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitID");
                    }
                    valeurNoeud = laboratoire.getLibelleLong();
                    hd.startElement("", "", "orgUnitName", _attsImpl);
                    hd.startElement("", "", "text", _attsImpl);
                    hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                    hd.endElement("", "", "text");
                    hd.endElement("", "", "orgUnitName");
                    valeurNoeud = laboratoire.getLibelleCourt();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "orgUnitAcronym", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitAcronym");
                    }
                    valeurNoeud = laboratoire.getCode();
                    if (valeurNoeud.length() > 0) {
                        _attsImpl.addAttribute("", "", "codeSet", "orgUnitCodeValueSet", "userDefined");
                        hd.startElement("", "", "orgUnitCode", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitCode");
                        _attsImpl.clear();
                    }
                    valeurNoeud = laboratoire.getTypeStructure();
                    if (valeurNoeud.length() > 0 && !valeurNoeud.equals("0000")) {
                        if (FicheUnivHelper.getLibelle(laboratoire, "cdm_orgunitkind", laboratoire.getTypeStructure()).length() > 0) {
                            _attsImpl.addAttribute("", "", "orgType", "orgUnitKindValueType", FicheUnivHelper.getLibelle(laboratoire, "cdm_orgunitkind", laboratoire.getTypeStructure()));
                        }
                        valeurNoeud = LabelUtils.getLibelle("60", valeurNoeud, LangueUtil.getLocale(laboratoire.getLangue()));
                        hd.startElement("", "", "orgUnitKind", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "orgUnitKind");
                        _attsImpl.clear();
                    }
                    ajouterUrlFiche(hd, laboratoire);
                    if (laboratoire.getAdresse().length() > 0 || laboratoire.getCodePostal().length() > 0 || laboratoire.getVille().length() > 0 || laboratoire.getTelephone().length() > 0 || laboratoire.getTelecopie().length() > 0 || laboratoire.getAdresseMail().length() > 0 || Formateur.estSaisi(laboratoire.getSiteWeb()) || laboratoire.getComplementCoordonnees().length() > 0) {
                        hd.startElement("", "", "contacts", _attsImpl);
                        hd.startElement("", "", "contactData", _attsImpl);
                        if (laboratoire.getAdresse().length() > 0 || laboratoire.getCodePostal().length() > 0 || laboratoire.getVille().length() > 0) {
                            hd.startElement("", "", "adr", _attsImpl);
                            valeurNoeud = laboratoire.getAdresse();
                            if (valeurNoeud.length() > 0) {
                                hd.startElement("", "", "street", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "street");
                            }
                            valeurNoeud = laboratoire.getVille();
                            if (valeurNoeud.length() > 0) {
                                hd.startElement("", "", "locality", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "locality");
                            }
                            valeurNoeud = laboratoire.getCodePostal();
                            if (valeurNoeud.length() > 0) {
                                hd.startElement("", "", "pcode", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "pcode");
                            }
                            hd.endElement("", "", "adr");
                        }
                        valeurNoeud = laboratoire.getTelephone();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "telephone", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "telephone");
                        }
                        valeurNoeud = laboratoire.getTelecopie();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "fax", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "fax");
                        }
                        valeurNoeud = laboratoire.getAdresseMail();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "email", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "email");
                        }
                        valeurNoeud = laboratoire.getSiteWeb();
                        if (Formateur.estSaisi(valeurNoeud)) {
                            hd.startElement("", "", "webLink", _attsImpl);
                            hd.startElement("", "", "href", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "href");
                            hd.endElement("", "", "webLink");
                        }
                        valeurNoeud = formaterEnHtml(laboratoire.getComplementCoordonnees());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "infoBlock", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "infoBlock");
                        }
                        hd.endElement("", "", "contactData");
                        hd.endElement("", "", "contacts");
                    }
                    ajouterResponsables(hd, laboratoire.getCodeResponsable());
                    ajouterStructuresRattachement(hd, laboratoire);
                    valeurNoeud = laboratoire.getMetaKeywords();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "searchword", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "searchword");
                    }
                    hd.endElement("", "", "orgUnit");
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des laboratoires dans l'export CDM", e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des formations exportées depuis K-Sup dans un format compatible CDM (noeud program). Ajoute également les parcours associés
     * à ces formations.
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds program doivent être ajoutés
     */
    private void ajouterNoeudsFormations(final TransformerHandler hd) {
        try {
            final AttributesImpl _attsImpl = new AttributesImpl();
            String valeurNoeud = "";
            StringTokenizer stValeursNoeud = null;
            final Formation formation = new Formation();
            /* On récupère toutes les formations existantes, puis on mappe des champs du modèle K-Sup
             * vers le modèle CDM */
            if (formation.selectCodeLangueEtat("", "", EN_LIGNE) > 0) {
                while (formation.nextItem()) {
                    _attsImpl.addAttribute("", "", "ident", "token", formation.getCode());
                    _attsImpl.addAttribute("", "", "language", "language", getXsLanguage(formation.getLangue()));
                    hd.startElement("", "", "program", _attsImpl);
                    _attsImpl.clear();
                    valeurNoeud = formation.getCode();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "programID", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "programID");
                    }
                    valeurNoeud = formation.getIntituleLong();
                    hd.startElement("", "", "programName", _attsImpl);
                    hd.startElement("", "", "text", _attsImpl);
                    hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                    hd.endElement("", "", "text");
                    hd.endElement("", "", "programName");
                    valeurNoeud = formation.getCode();
                    if (valeurNoeud.length() > 0) {
                        _attsImpl.addAttribute("", "", "codeSet", "programCodeValueSet", "userDefined");
                        hd.startElement("", "", "programCode", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "programCode");
                        _attsImpl.clear();
                    }
                    ajouterUrlFiche(hd, formation);
                    valeurNoeud = formaterEnHtml(formation.getResume());
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "programDescription", _attsImpl);
                        hd.startElement("", "", "subBlock", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "subBlock");
                        hd.endElement("", "", "programDescription");
                    }
                    if (formation.getIntituleLong().length() > 0 || formation.getResume().length() > 0 || formation.getDebouches().length() > 0 || formation.getTypeDiplome().length() > 0) {
                        hd.startElement("", "", "qualification", _attsImpl);
                        valeurNoeud = formation.getIntituleLong();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "qualificationName", _attsImpl);
                            hd.startElement("", "", "text", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "text");
                            hd.endElement("", "", "qualificationName");
                        }
                        valeurNoeud = formaterEnHtml(formation.getResume());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "qualificationDescription", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "qualificationDescription");
                        }
                        stValeursNoeud = new StringTokenizer(formation.getTypeDiplome(), ";");
                        if (stValeursNoeud.hasMoreTokens()) {
                            valeurNoeud = stValeursNoeud.nextToken();
                            if (FicheUnivHelper.getLibelle(formation, "cdm_degreevaluetype", valeurNoeud).length() > 0) {
                                _attsImpl.addAttribute("", "", "degree", "degreeValueType", FicheUnivHelper.getLibelle(formation, "cdm_degreevaluetype", valeurNoeud));
                            }
                            hd.startElement("", "", "degree", _attsImpl);
                            _attsImpl.clear();
                            valeurNoeud = LabelUtils.getLibelle("20", valeurNoeud, LangueUtil.getLocale(formation.getLangue()));
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "degree");
                        }
                        valeurNoeud = formaterEnHtml(formation.getDebouches());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "profession", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "profession");
                        }
                        hd.endElement("", "", "qualification");
                    }
                    valeurNoeud = formation.getLibelleGrade();
                    if (valeurNoeud.length() > 0) {
                        if (FicheUnivHelper.getLibelle(formation, "cdm_levelvaluetype", formation.getGrade()).length() > 0) {
                            _attsImpl.addAttribute("", "", "level", "levelValueType", FicheUnivHelper.getLibelle(formation, "cdm_levelvaluetype", formation.getGrade()));
                        }
                        hd.startElement("", "", "level", _attsImpl);
                        _attsImpl.clear();
                        hd.startElement("", "", "subBlock", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "subBlock");
                        hd.endElement("", "", "level");
                    }
                    valeurNoeud = formaterEnHtml(formation.getDescription());
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "learningObjectives", _attsImpl);
                        hd.startElement("", "", "subBlock", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "subBlock");
                        hd.endElement("", "", "learningObjectives");
                    }
                    valeurNoeud = formaterEnHtml(formation.getConditionsAdmission());
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "admissionInfo", _attsImpl);
                        hd.startElement("", "", "admissionDescription", _attsImpl);
                        hd.startElement("", "", "subBlock", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "subBlock");
                        hd.endElement("", "", "admissionDescription");
                        hd.endElement("", "", "admissionInfo");
                    }
                    valeurNoeud = formaterEnHtml(formation.getFormationsRequises());
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "formalPrerequisites", _attsImpl);
                        hd.startElement("", "", "subBlock", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "subBlock");
                        hd.endElement("", "", "formalPrerequisites");
                    }
                    if (formation.getCodeRattachement().length() > 0 || formation.getCodeRattachementAutres().length() > 0 || formation.getResponsablesFiche().length() > 0) {
                        hd.startElement("", "", "infoBlock", _attsImpl);
                        ajouterStructuresRattachement(hd, formation);
                        stValeursNoeud = new StringTokenizer(formation.getResponsablesFiche(), ";");
                        valeurNoeud = "";
                        while (stValeursNoeud.hasMoreTokens()) {
                            valeurNoeud += stValeursNoeud.nextToken() + "," + formation.getLangue() + ",responsable;";
                        }
                        ajouterResponsables(hd, valeurNoeud);
                        hd.endElement("", "", "infoBlock");
                    }
                    valeurNoeud = formation.getMetaKeywords();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "searchword", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "searchword");
                    }
                    hd.endElement("", "", "program");
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des formations dans l'export CDM", e);
        }
    }

    private String formaterEnHtml(final String texte) throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), texte);
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des UEUP exportées depuis K-Sup, s'ils font partie d'un parcours exporté, dans un format compatible CDM (noeud course)
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds course doivent être ajoutés
     * @throws Exception
     */
    private void ajouterNoeudsUeups(final TransformerHandler hd) {
        try {
            final AttributesImpl _attsImpl = new AttributesImpl();
            String valeurNoeud = "";
            StringTokenizer stValeursNoeud = null;
            final Ueup ueup = new Ueup();
            /* On récupère toutes les UEUP existantes, on ne garde que celles présentes dans les programmes,
             * puis on mappe des champs du modèle K-Sup vers le modèle CDM */
            if (ueup.selectCodeLangueEtat("", "", EN_LIGNE) > 0) {
                while (ueup.nextItem()) {
                    if (hsCodesUeupsDansParcours.contains(ueup.getCode())) {
                        _attsImpl.addAttribute("", "", "ident", "token", ueup.getCode());
                        _attsImpl.addAttribute("", "", "language", "language", getXsLanguage(ueup.getLangue()));
                        hd.startElement("", "", "course", _attsImpl);
                        _attsImpl.clear();
                        valeurNoeud = ueup.getCode();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "courseID", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "courseID");
                        }
                        valeurNoeud = ueup.getLibelleLong();
                        hd.startElement("", "", "courseName", _attsImpl);
                        hd.startElement("", "", "text", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "text");
                        hd.endElement("", "", "courseName");
                        valeurNoeud = ueup.getCode();
                        if (valeurNoeud.length() > 0) {
                            _attsImpl.addAttribute("", "", "codeSet", "courseCodeValueSet", "userDefined");
                            hd.startElement("", "", "courseCode", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "courseCode");
                            _attsImpl.clear();
                        }
                        ajouterUrlFiche(hd, ueup);
                        valeurNoeud = formaterEnHtml(ueup.getContenu());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "courseDescription", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "courseDescription");
                        }
                        valeurNoeud = formaterEnHtml(ueup.getCalendrier());
                        if (valeurNoeud.length() > 0) {
                            _attsImpl.addAttribute("", "", "term", "teachingTermValueType", "userDefined");
                            hd.startElement("", "", "teachingTerm", _attsImpl);
                            _attsImpl.clear();
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "teachingTerm");
                        }
                        valeurNoeud = ueup.getCreditEcts();
                        if (valeurNoeud.length() > 0) {
                            _attsImpl.addAttribute("", "", "ECTScredits", "nonNegativeInteger", nettoyerPourEcts(valeurNoeud));
                            hd.startElement("", "", "credits", _attsImpl);
                            _attsImpl.clear();
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "credits");
                        }
                        valeurNoeud = formaterEnHtml(ueup.getObjectifs());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "learningObjectives", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "learningObjectives");
                        }
                        valeurNoeud = formaterEnHtml(ueup.getPrerequis());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "formalPrerequisites", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "formalPrerequisites");
                        }
                        valeurNoeud = ueup.getLibelleLanguesEnseignement();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "instructionLanguage", _attsImpl);
                            stValeursNoeud = new StringTokenizer(valeurNoeud, ";");
                            while (stValeursNoeud.hasMoreTokens()) {
                                valeurNoeud = stValeursNoeud.nextToken();
                                hd.startElement("", "", "subBlock", _attsImpl);
                                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                                hd.endElement("", "", "subBlock");
                            }
                            hd.endElement("", "", "instructionLanguage");
                        }
                        valeurNoeud = formaterEnHtml(ueup.getBibliographie());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "syllabus", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "syllabus");
                        }
                        valeurNoeud = formaterEnHtml(ueup.getControleConnaissances());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "exam", _attsImpl);
                            hd.startElement("", "", "examKind", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "examKind");
                            hd.endElement("", "", "exam");
                        }
                        valeurNoeud = formaterEnHtml(ueup.getCoordonnees());
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "contacts", _attsImpl);
                            hd.startElement("", "", "infoBlock", _attsImpl);
                            hd.startElement("", "", "subBlock", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "subBlock");
                            hd.endElement("", "", "infoBlock");
                            hd.endElement("", "", "contacts");
                        }
                        if (ueup.getCodeRattachement().length() > 0 || ueup.getCodeRattachementAutres().length() > 0 || ueup.getUeupfils().length() > 0 || ueup.getCours().length() > 0 || ueup.getCodeResponsables().length() > 0) {
                            hd.startElement("", "", "infoBlock", _attsImpl);
                            ajouterStructuresRattachement(hd, ueup);
                            ajouterUeupsRattaches(hd, ueup);
                            ajouterCoursRattaches(hd, ueup);
                            stValeursNoeud = new StringTokenizer(ueup.getCodeResponsables(), ";");
                            valeurNoeud = "";
                            while (stValeursNoeud.hasMoreTokens()) {
                                valeurNoeud += stValeursNoeud.nextToken() + "," + ueup.getLangue() + ",responsable;";
                            }
                            ajouterResponsables(hd, valeurNoeud);
                            hd.endElement("", "", "infoBlock");
                        }
                        valeurNoeud = ueup.getMetaKeywords();
                        if (valeurNoeud.length() > 0) {
                            hd.startElement("", "", "searchword", _attsImpl);
                            hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            hd.endElement("", "", "searchword");
                        }
                        hd.endElement("", "", "course");
                    }
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des UEUP dans l'export CDM", e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des cours exportées depuis K-Sup, s'ils sont rattachés à une UEUP exportée, dans un format compatible CDM (noeud course)
     *
     * @param _hd
     *            le TransformerHandler dans lequel les noeuds course doivent être ajoutés
     */
    private void ajouterNoeudsCours(final TransformerHandler _hd) {
        try {
            final AttributesImpl _attsImpl = new AttributesImpl();
            String valeurNoeud = "";
            StringTokenizer stValeursNoeud = null;
            final Cours cours = new Cours();
            /* On récupère tous les cours existants, on ne garde que ceux rattachés à une UEUP précédemment exportée,
             * puis on mappe des champs du modèle K-Sup vers le modèle CDM */
            if (cours.selectCodeLangueEtat("", "", EN_LIGNE) > 0) {
                while (cours.nextItem()) {
                    if (hsCodesCoursDansUeup.contains(cours.getCode())) {
                        _attsImpl.addAttribute("", "", "ident", "token", cours.getCode());
                        _attsImpl.addAttribute("", "", "language", "language", getXsLanguage(cours.getLangue()));
                        _hd.startElement("", "", "course", _attsImpl);
                        _attsImpl.clear();
                        valeurNoeud = cours.getCode();
                        if (valeurNoeud.length() > 0) {
                            _hd.startElement("", "", "courseID", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "courseID");
                        }
                        valeurNoeud = cours.getLibelleLong();
                        _hd.startElement("", "", "courseName", _attsImpl);
                        _hd.startElement("", "", "text", _attsImpl);
                        _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        _hd.endElement("", "", "text");
                        _hd.endElement("", "", "courseName");
                        valeurNoeud = cours.getCode();
                        if (valeurNoeud.length() > 0) {
                            _attsImpl.addAttribute("", "", "codeSet", "courseCodeValueSet", "userDefined");
                            _hd.startElement("", "", "courseCode", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "courseCode");
                            _attsImpl.clear();
                        }
                        ajouterUrlFiche(_hd, cours);
                        valeurNoeud = formaterEnHtml(cours.getContenu());
                        if (valeurNoeud.length() > 0) {
                            _hd.startElement("", "", "courseDescription", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "courseDescription");
                        }
                        valeurNoeud = formaterEnHtml(cours.getCalendrier());
                        if (valeurNoeud.length() > 0) {
                            _attsImpl.addAttribute("", "", "term", "teachingTermValueType", "userDefined");
                            _hd.startElement("", "", "teachingTerm", _attsImpl);
                            _attsImpl.clear();
                            _hd.startElement("", "", "subBlock", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "subBlock");
                            _hd.endElement("", "", "teachingTerm");
                        }
                        valeurNoeud = cours.getCreditEcts();
                        if (valeurNoeud.length() > 0) {
                            _attsImpl.addAttribute("", "", "ECTScredits", "nonNegativeInteger", nettoyerPourEcts(valeurNoeud));
                            _hd.startElement("", "", "credits", _attsImpl);
                            _attsImpl.clear();
                            _hd.startElement("", "", "subBlock", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "subBlock");
                            _hd.endElement("", "", "credits");
                        }
                        valeurNoeud = formaterEnHtml(cours.getObjectifs());
                        if (valeurNoeud.length() > 0) {
                            _hd.startElement("", "", "learningObjectives", _attsImpl);
                            _hd.startElement("", "", "subBlock", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "subBlock");
                            _hd.endElement("", "", "learningObjectives");
                        }
                        valeurNoeud = formaterEnHtml(cours.getBibliographie());
                        if (valeurNoeud.length() > 0) {
                            _hd.startElement("", "", "syllabus", _attsImpl);
                            _hd.startElement("", "", "subBlock", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "subBlock");
                            _hd.endElement("", "", "syllabus");
                        }
                        valeurNoeud = formaterEnHtml(cours.getControleConnaissances());
                        if (valeurNoeud.length() > 0) {
                            _hd.startElement("", "", "exam", _attsImpl);
                            _hd.startElement("", "", "examKind", _attsImpl);
                            _hd.startElement("", "", "subBlock", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "subBlock");
                            _hd.endElement("", "", "examKind");
                            _hd.endElement("", "", "exam");
                        }
                        valeurNoeud = formaterEnHtml(cours.getCoordonnees());
                        if (valeurNoeud.length() > 0) {
                            _hd.startElement("", "", "contacts", _attsImpl);
                            _hd.startElement("", "", "infoBlock", _attsImpl);
                            _hd.startElement("", "", "subBlock", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "subBlock");
                            _hd.endElement("", "", "infoBlock");
                            _hd.endElement("", "", "contacts");
                        }
                        if (cours.getCodeRattachement().length() > 0 || cours.getCodeRattachementAutres().length() > 0 || cours.getCodeResponsable().length() > 0) {
                            _hd.startElement("", "", "infoBlock", _attsImpl);
                            ajouterStructuresRattachement(_hd, cours);
                            stValeursNoeud = new StringTokenizer(cours.getCodeResponsable(), ";");
                            valeurNoeud = "";
                            while (stValeursNoeud.hasMoreTokens()) {
                                valeurNoeud += stValeursNoeud.nextToken() + "," + cours.getLangue() + ",responsable;";
                            }
                            ajouterResponsables(_hd, valeurNoeud);
                            _hd.endElement("", "", "infoBlock");
                        }
                        valeurNoeud = cours.getMetaKeywords();
                        if (valeurNoeud.length() > 0) {
                            _hd.startElement("", "", "searchword", _attsImpl);
                            _hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                            _hd.endElement("", "", "searchword");
                        }
                        _hd.endElement("", "", "course");
                    }
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des cours dans l'export CDM", e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des personnes responsables des autres contenus exportés depuis K-Sup dans un format compatible CDM (noeud person)
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds person doivent être ajoutés
     */
    private void ajouterNoeudsAnnuaires(final TransformerHandler hd) {
        try {
            final AttributesImpl _attsImpl = new AttributesImpl();
            String valeurNoeud = "";
            final Iterator<AnnuaireModele> itAnnuairesResponsables = hsAnnuairesResponsables.iterator();
            AnnuaireModele annuaireModele = null;
            /* Pour toutes les fiches annuaireksup récoltées, on mappe des champs du modèle K-Sup
             * vers le modèle CDM */
            while (itAnnuairesResponsables.hasNext()) {
                annuaireModele = itAnnuairesResponsables.next();
                _attsImpl.addAttribute("", "", "ident", "token", annuaireModele.getCode());
                _attsImpl.addAttribute("", "", "language", "language", getXsLanguage(annuaireModele.getLangue()));
                hd.startElement("", "", "person", _attsImpl);
                _attsImpl.clear();
                valeurNoeud = annuaireModele.getCode();
                if (valeurNoeud.length() > 0) {
                    hd.startElement("", "", "personID", _attsImpl);
                    hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                    hd.endElement("", "", "personID");
                }
                hd.startElement("", "", "name", _attsImpl);
                valeurNoeud = annuaireModele.getPrenom();
                hd.startElement("", "", "given", _attsImpl);
                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                hd.endElement("", "", "given");
                valeurNoeud = annuaireModele.getNom();
                hd.startElement("", "", "family", _attsImpl);
                hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                hd.endElement("", "", "family");
                hd.endElement("", "", "name");
                valeurNoeud = LabelUtils.getLibelle("02", annuaireModele.getTypePopulation(), LangueUtil.getLocale(annuaireModele.getLangue()));
                if (valeurNoeud.length() > 0) {
                    hd.startElement("", "", "title", _attsImpl);
                    hd.startElement("", "", "text", _attsImpl);
                    hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                    hd.endElement("", "", "text");
                    hd.endElement("", "", "title");
                }
                valeurNoeud = annuaireModele.getCorps();
                if (valeurNoeud.length() > 0) {
                    hd.startElement("", "", "role", _attsImpl);
                    hd.startElement("", "", "text", _attsImpl);
                    hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                    hd.endElement("", "", "text");
                    hd.endElement("", "", "role");
                }
                if (annuaireModele.getAdresse().length() > 0 || annuaireModele.getTelephone1().length() > 0 || annuaireModele.getTelecopie1().length() > 0 || annuaireModele.getAdresseMail().length() > 0 || Formateur.estSaisi(annuaireModele.getAdresseSitePerso())) {
                    hd.startElement("", "", "contactData", _attsImpl);
                    valeurNoeud = annuaireModele.getAdresse();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "adr", _attsImpl);
                        hd.startElement("", "", "street", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "street");
                        hd.endElement("", "", "adr");
                    }
                    valeurNoeud = annuaireModele.getTelephone1();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "telephone", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "telephone");
                    }
                    valeurNoeud = annuaireModele.getTelephone2();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "telephone", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "telephone");
                    }
                    valeurNoeud = annuaireModele.getTelecopie1();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "fax", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "fax");
                    }
                    valeurNoeud = annuaireModele.getAdresseMail();
                    if (valeurNoeud.length() > 0) {
                        hd.startElement("", "", "email", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "email");
                    }
                    valeurNoeud = annuaireModele.getAdresseSitePerso();
                    if (Formateur.estSaisi(valeurNoeud)) {
                        hd.startElement("", "", "webLink", _attsImpl);
                        hd.startElement("", "", "href", _attsImpl);
                        hd.characters(valeurNoeud.toCharArray(), 0, valeurNoeud.length());
                        hd.endElement("", "", "href");
                        hd.endElement("", "", "webLink");
                    }
                    hd.endElement("", "", "contactData");
                }
                if (annuaireModele.getCodeRattachement().length() > 0 || annuaireModele.getCodeRattachementAutres().length() > 0) {
                    hd.startElement("", "", "infoBlock", _attsImpl);
                    ajouterStructuresRattachement(hd, annuaireModele);
                    hd.endElement("", "", "infoBlock");
                }
                hd.endElement("", "", "person");
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des fiches annuaire dans l'export CDM", e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant le lien vers la fiche que l'on est en train d'exporter (=> noeud webLink)
     *
     * @param hd
     *            le TransformerHandler dans lequel le noeud webLink doit être ajouté
     * @param ficheUniv
     *            la fiche pour laquelle on génère ce noeud
     */
    private void ajouterUrlFiche(final TransformerHandler hd, final FicheUniv ficheUniv) {
        try {
            final AttributesImpl attsImpl = new AttributesImpl();
            UrlProvider urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
            final String urlFiche = urlProvider.provideAbsoluteUrl(ficheUniv.getPersistanceBean());
            if (urlFiche.length() > 0) {
                hd.startElement("", "", "webLink", attsImpl);
                hd.startElement("", "", "href", attsImpl);
                hd.characters(urlFiche.toCharArray(), 0, urlFiche.length());
                hd.endElement("", "", "href");
                final String libelleFiche = ficheUniv.getLibelleAffichable();
                if (libelleFiche.length() > 0) {
                    hd.startElement("", "", "linkName", attsImpl);
                    hd.characters(libelleFiche.toCharArray(), 0, libelleFiche.length());
                    hd.endElement("", "", "linkName");
                }
                hd.endElement("", "", "webLink");
            }
        } catch (final SAXException e) {
            LOG.error("Erreur à l'ajout de l'URL d'une fiche", e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des structures auxquelles est rattachée la fiche que l'on est en train d'exporter (=> noeud refOrgUnit)
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds refOrgUnit doivent être ajoutés
     * @param ficheUniv
     *            la fiche pour laquelle on génère ces noeuds
     */
    private void ajouterStructuresRattachement(final TransformerHandler hd, final FicheUniv ficheUniv) {
        try {
            if (exporterDefaultstructureksup || exporterLaboratoire) {
                final AttributesImpl attsImpl = new AttributesImpl();
                String valeurNoeud = "";
                StringTokenizer stValeursNoeud = null;
                /* rattachement principal */
                valeurNoeud = ficheUniv.getCodeRattachement();
                if (valeurNoeud.length() > 0 && hsCodesStructures.contains(valeurNoeud)) {
                    attsImpl.addAttribute("", "", "ref", "token", valeurNoeud);
                    hd.startElement("", "", "refOrgUnit", attsImpl);
                    hd.endElement("", "", "refOrgUnit");
                    attsImpl.clear();
                }
                /* rattachements secondaires s'ils existent */
                if (ficheUniv instanceof FicheRattachementsSecondaires) {
                    stValeursNoeud = new StringTokenizer(((FicheRattachementsSecondaires) ficheUniv).getCodeRattachementAutres(), ";");
                    while (stValeursNoeud.hasMoreTokens()) {
                        valeurNoeud = stValeursNoeud.nextToken();
                        if (hsCodesStructures.contains(valeurNoeud)) {
                            attsImpl.addAttribute("", "", "ref", "token", valeurNoeud);
                            hd.startElement("", "", "refOrgUnit", attsImpl);
                            hd.endElement("", "", "refOrgUnit");
                            attsImpl.clear();
                        }
                    }
                }
            }
        } catch (final SAXException e) {
            LOG.error("Erreur à l'ajout des rattachements structures dans l'export CDM : " + e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des personnes responsables de la fiche que l'on est en train d'exporter (=> noeud refPerson)
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds refPerson doivent être ajoutés
     * @param codesResponsables
     *            une liste de responsables de la forme suivante : codefiche,langue,titre;codefiche,langue,titre
     */
    private void ajouterResponsables(final TransformerHandler hd, final String codesResponsables) {
        try {
            if (exporterAnnuaireksup) {
                final AttributesImpl attsImpl = new AttributesImpl();
                String valeurNoeud = "";
                final StringTokenizer stValeursNoeud = new StringTokenizer(codesResponsables, ";");
                StringTokenizer stValeursDansNoeud = null;
                String codeAnnuaire = "";
                String langueAnnuaire = "";
                AnnuaireModele annuaireModele = null;
                while (stValeursNoeud.hasMoreTokens()) {
                    stValeursDansNoeud = new StringTokenizer(stValeursNoeud.nextToken(), ",");
                    while (stValeursDansNoeud.hasMoreTokens()) {
                        /* code de la fiche */
                        codeAnnuaire = stValeursDansNoeud.nextToken();
                        /* langue de la fiche */
                        if (stValeursDansNoeud.hasMoreTokens()) {
                            langueAnnuaire = stValeursDansNoeud.nextToken();
                        }
                        /* on vérifie que la fiche annuaire existe toujours, ie. qu'il ne s'agit pas d'un rattachement obsolète,
                         * et on ne traite que les fiches qui sont en ligne, car les fiches dans un autre état
                         * ne sont pas exportées */
                        annuaireModele = FicheAnnuaire.getFicheAnnuaire(codeAnnuaire, langueAnnuaire);
                        if (annuaireModele != null && annuaireModele.getEtatObjet().equals(EN_LIGNE)) {
                            attsImpl.addAttribute("", "", "ref", "token", codeAnnuaire);
                            /* titre de la personne */
                            if (stValeursDansNoeud.hasMoreTokens()) {
                                valeurNoeud = stValeursDansNoeud.nextToken();
                                if (!valeurNoeud.equals("responsable")) {
                                    valeurNoeud = LabelUtils.getLibelle("64", valeurNoeud, LangueUtil.getLocale((new Integer(langueAnnuaire).intValue())));
                                }
                                attsImpl.addAttribute("", "", "role", "token", valeurNoeud);
                            }
                            /* on ajoute le noeud */
                            hd.startElement("", "", "refPerson", attsImpl);
                            hd.endElement("", "", "refPerson");
                            attsImpl.clear();
                            /* on sauvegarde la fiche annuaire correspondant à cette personne, pour exporter dans un second temps
                             * la liste des responsables sous la forme de noeuds person */
                            hsAnnuairesResponsables.add(annuaireModele);
                        }
                    }
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des responsables dans l'export CDM : " + e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des ueups rattachées à la fiche que l'on est en train d'exporter (=> noeud refCourse)
     *
     * @param hd
     *            le TransformerHandler dans lequel les noeuds refCourse doivent être ajoutés
     * @param ueup
     *            la fiche UEUP dont on souhaite récupérer les rattachements UEUP
     */
    private void ajouterUeupsRattaches(final TransformerHandler hd, final Ueup ueup) {
        try {
            final AttributesImpl attsImpl = new AttributesImpl();
            final StringTokenizer stValeursNoeud = new StringTokenizer(ueup.getUeupfils(), ";");
            String valeurNoeud = "";
            while (stValeursNoeud.hasMoreTokens()) {
                valeurNoeud = stValeursNoeud.nextToken();
                if (hsCodesUeupsDansParcours.contains(valeurNoeud)) {
                    /* on ajoute le noeud */
                    attsImpl.addAttribute("", "", "ref", "token", valeurNoeud);
                    hd.startElement("", "", "refCourse", attsImpl);
                    hd.endElement("", "", "refCourse");
                    attsImpl.clear();
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des cours rattachés à l'UEUP " + ueup.getLibelleAffichable() + " (" + ueup.getCode() + ") : " + e);
        }
    }

    /**
     * Ajoute à un TransformerHandler existant la liste des cours rattachés à la fiche que l'on est en train d'exporter (=> noeud refCourse)
     *
     * @param _hd
     *            le TransformerHandler dans lequel les noeuds refCourse doivent être ajoutés
     * @param _ueup
     *            la fiche UEUP dont on souhaite récupérer les rattachements cours
     */
    private void ajouterCoursRattaches(final TransformerHandler _hd, final Ueup _ueup) {
        try {
            if (exporterCours) {
                final AttributesImpl _attsImpl = new AttributesImpl();
                final StringTokenizer stValeursNoeud = new StringTokenizer(_ueup.getCours(), ";");
                String valeurNoeud = "";
                Cours _cours = null;
                while (stValeursNoeud.hasMoreTokens()) {
                    valeurNoeud = stValeursNoeud.nextToken();
                    String codeCours = valeurNoeud;
                    String langueCours = _ueup.getLangue();
                    if (valeurNoeud.contains(",LANGUE=")) {
                        codeCours = valeurNoeud.substring(0, valeurNoeud.indexOf(",LANGUE="));
                        langueCours = valeurNoeud.substring(valeurNoeud.indexOf(",LANGUE=") + 8);
                    }
                    /* on ne traite que les fiches cours qui sont en ligne, les fiches
                     * dans un autre état ne sont pas exportées */
                    _cours = Cours.getFicheCours(codeCours, langueCours);
                    if (_cours != null && _cours.getEtatObjet().equals(EN_LIGNE)) {
                        /* on ajoute le noeud */
                        _attsImpl.addAttribute("", "", "ref", "token", codeCours);
                        _hd.startElement("", "", "refCourse", _attsImpl);
                        _hd.endElement("", "", "refCourse");
                        _attsImpl.clear();
                        /* on stocke le code du cours rattaché pour référence ultérieure */
                        hsCodesCoursDansUeup.add(codeCours);
                    }
                }
            }
        } catch (final Exception e) {
            LOG.error("Erreur à l'ajout des cours rattachés à l'UEUP " + _ueup.getLibelleAffichable() + " (" + _ueup.getCode() + ") : " + e);
        }
    }

    /**
     * Envoie un fichier résultant de l'export par email, à l'adresse saisie dans l'écran d'export
     *
     * @param _pieceJointe
     *            une chaîne identifiant le chemin du fichier à joindre à l'email
     */
    protected void envoyerFichierParEmail(final String _pieceJointe) {
        try {
            final JSBMailbox mailbox = new JSBMailbox(false);
            final String corpsMail = "L'export CDM s'est correctement déroulé, vous trouverez le résultat de l'export en pièce jointe";
            mailbox.sendSystemMsgWithAttachedFiles(infoBean.getString("EMAIL"), "Export CDM", corpsMail, new String[] {_pieceJointe});
        } catch (final Exception e) {
            LOG.error("Erreur à l'envoi de l'export par email : " + e);
        }
    }

    /**
     * Récupère le nom et la version de l'application lus depuis le fichier .properties
     *
     * @return le nom de l'application sous forme de chaîne
     */
    protected String getApplicationName() {
        String res = "";
        final String applicationName = ExtensionHelper.getCoreExtension().getApplicationName();
        if (applicationName != null && applicationName.length() > 2) {
            res = applicationName.substring(0, 1).toUpperCase() + "-" + applicationName.substring(1, 2).toUpperCase() + applicationName.substring(2).toLowerCase();
        }
        final String applicationVersion = ExtensionHelper.getCoreExtension().getApplicationVersion();
        if (applicationVersion != null && applicationVersion.length() > 0) {
            res += " " + applicationVersion;
        }
        return res;
    }

    /**
     * Permet d'obtenir depuis un code de langue K-Portal/K-Sup la langue au format xs:language
     *
     * @param _indiceLangue
     *            le code K-Portal/K-Sup de la langue dont on souhaite obtenir la valeur xs:language
     * @return la valeur au format xs:language de l'indice de la langue passé en paramètre
     */
    protected String getXsLanguage(final String _indiceLangue) {
        return LangueUtil.getLocale(_indiceLangue).toString().replaceAll("_", "-");
    }

    /**
     * permet de garder uniquement la partie entière de la valeur d'un crédit ECTS
     *
     * @param _aNettoyer
     *            la chaîne à nettoyer
     * @return la chaîne nettoyée si nécessaire
     */
    protected String nettoyerPourEcts(final String _aNettoyer) {
        String res = _aNettoyer;
        /* le séparateur pour les chiffres décimaux est le point, pas la virgule */
        if (res.indexOf(',') != -1) {
            res = res.replaceAll(",", "\\.");
        }
        /* on retourne la partie entière du crédit ECTS s'il s'agit d'un chiffre décimal */
        if (res.indexOf(".") != -1) {
            res = String.valueOf((int) Float.parseFloat(res));
        }
        return res;
    }

    /**
     * permet de valider que le module d'export est activé
     *
     * @return true si l'export est actif
     */
    protected boolean isExportActif() {
        return "1".equals(moduleCDM.getProperty(prefixeExportcdm + "activation"));
    }

    /**
     * permet de valider que l'export des defaultstructure est activé
     *
     * @return true si l'export est actif
     */
    protected boolean isExportDefaultstructureksupActif() {
        return "1".equals(moduleCDM.getProperty(prefixeExportcdm + "defaultstructureksup" + suffixeExportActif));
    }

    /**
     * permet de valider que l'export des laboratoires est activé
     *
     * @return true si l'export est actif
     */
    protected boolean isExportLaboratoireActif() {
        return "1".equals(moduleCDM.getProperty(prefixeExportcdm + "laboratoire" + suffixeExportActif));
    }

    /**
     * permet de valider que l'export des formations est activé
     *
     * @return true si l'export est actif
     */
    protected boolean isExportFormationActif() {
        return "1".equals(moduleCDM.getProperty(prefixeExportcdm + "formation" + suffixeExportActif));
    }

    /**
     * permet de valider que l'export des UEUP est activé
     *
     * @return true si l'export est actif
     */
    protected boolean isExportUeupActif() {
        return "1".equals(moduleCDM.getProperty(prefixeExportcdm + "ueup" + suffixeExportActif));
    }

    /**
     * permet de valider que l'export des cours est activé
     *
     * @return true si l'export est actif
     */
    protected boolean isExportCoursActif() {
        return "1".equals(moduleCDM.getProperty(prefixeExportcdm + "cours" + suffixeExportActif));
    }

    /**
     * permet de valider que l'export des fiches annuaireksup est activé
     *
     * @return true si l'export est actif
     */
    protected boolean isExportAnnuaireksupActif() {
        return "1".equals(moduleCDM.getProperty(prefixeExportcdm + "annuaireksup" + suffixeExportActif));
    }
}
