/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.offreformation.layout.card.view.builder.impl;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import com.kosmos.layout.Layout;
import com.kosmos.layout.card.view.builder.impl.AbstractViewModelBuilder;
import com.kosmos.layout.slot.Slot;
import com.kosmos.offreformation.layout.card.bean.ProfessionnalisationCardBean;
import com.kosmos.offreformation.layout.card.view.model.ProfessionnalisationCardView;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.service.ServiceFormation;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.utils.ContexteUtil;

public class ProfessionnalisationCardViewModelBuilder extends AbstractViewModelBuilder<ProfessionnalisationCardBean, ProfessionnalisationCardView> {

    private ServiceLabel serviceLabel;

    private ServiceFormation serviceFormation;

    @Override
    protected ProfessionnalisationCardView buildViewModel(final ProfessionnalisationCardView viewModel, final ProfessionnalisationCardBean card, final Slot slot, final Layout layout) {
        MetatagBean metatagBean = ContexteUtil.getContexteUniv().getMetaCourant();
        if (metatagBean != null) {
            Optional<FormationBean> formation = serviceFormation.getFiche(metatagBean.getIdMetatag());
            if (formation.isPresent()) {
                viewModel.setMetiers(getWorksLabel(formation.get()));
            }
        }
        return viewModel;
    }

    private List<String> getWorksLabel(FormationBean formationBean) {
        return serviceLabel.getFromCodes(formationBean.getMetiers(), formationBean.getLangue(), ";", MetierSerializer.TYPE_LIBELLE_METIER).stream()
            .map(labelDto -> labelDto.getLibelle())
            .collect(Collectors.toList());
    }

    public void setServiceLabel(final ServiceLabel serviceLabel) {
        this.serviceLabel = serviceLabel;
    }

    public void setServiceFormation(final ServiceFormation serviceFormation) {
        this.serviceFormation = serviceFormation;
    }
}
