/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.offreformation.servlet;

import java.io.IOException;

import javax.mail.MessagingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.mail.EmailException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.jsbsoft.jtf.email.JSBMailbox;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.core.config.MessageHelper;
import com.ksup.objetspartages.om.Formation;

@Controller
public class FormationContactServlet {

    private static final Logger LOG = LoggerFactory.getLogger(FormationContactServlet.class);

    public final static String URL_PROPERTY = "formation_contact.mapping";

    public FormationContactServlet() {}

    @RequestMapping("${" + URL_PROPERTY + ":/servlet/offreformation/ajax/contact}")
    protected void doPost(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
        String output = StringUtils.EMPTY;
        if (!isHoneyPotted(httpServletRequest)) {
            if (!sendEmail(httpServletRequest)) {
                output = "Une erreur est survenue pendant l'envoi du mail.";
            }
        } else {
            output = "WE DO NOT WANT SPAM ROBOTS HERE!";
        }
        httpServletResponse.getOutputStream().write(output.getBytes());
    }

    private boolean sendEmail(HttpServletRequest request) {
        // Récupération des données du mail
        String code = request.getParameter("CODE");
        String langue = request.getParameter("LANGUE");
        Formation formation;
        try {
            formation = Formation.getFicheFormation(code, langue);
        }catch (Exception e){
            LOG.error(String.format("Impossible de récupérer la formation avec le code %s et la langue %s", code, langue));
            return false;
        }
        String from = request.getParameter("you");
        String question = request.getParameter("question");
        // Si la question n'a pas été saisie, on envoie la question par défait
        if(StringUtils.isBlank(question)){
            question = String.format(MessageHelper.getMessage(OffreFormationUtil.ID_EXTENSION, "FORMATION.FO.__EXEMPLE_QUESTION"), formation.getIntituleCourt());
        }
        // Si l'un des champs est null, impossible d'envoyer le mail
        String to = formation.getContactEmail();
        if (StringUtils.isAnyBlank(from, question, code, langue) || StringUtils.isAnyBlank(to)) {
            return false;
        }

        JSBMailbox mailbox = new JSBMailbox(false);
        String subject = retrieveSubject(question);
        // Envoi du message
        try {
            mailbox.sendHtmlMsg(from, to, subject, question);
            return true;
        } catch (MessagingException | EmailException e) {
            LOG.error("Erreur à l'envoi de mail", e);
        }
        return false;
    }

    private String retrieveSubject(String fullMessage) {
        String[] splitedQuestion = StringUtils.split(fullMessage, StringUtils.SPACE);
        String extract = StringUtils.join(ArrayUtils.subarray(splitedQuestion, 0, Math.min(splitedQuestion.length, 12)), StringUtils.SPACE);
        return String.format(MessageHelper.getMessage(OffreFormationUtil.ID_EXTENSION, "ST_FORMATION_CONTACT_MAIL_SUBJECT"), extract);
    }

    private boolean isHoneyPotted(HttpServletRequest request) {
        String email = request.getParameter("EMAIL");
        return StringUtils.isNotBlank(email);
    }
}
