/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: offrestageemploi
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.offrestageemploi.migration;

import java.sql.Connection;

import org.flywaydb.core.api.migration.MigrationChecksumProvider;
import org.flywaydb.core.api.migration.jdbc.JdbcMigration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.migration.FlywayMigrationUtils;
import com.kosmos.offrestageemploi.util.OffrestageemploiUtil;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.services.ServiceLabel;

public class V1_7_0_2__data implements JdbcMigration, MigrationChecksumProvider {

    private ServiceLabel serviceLabel;

    private static final String PREVIOUS_VERSION = "1.2.4.1";

    private static final Logger LOGGER = LoggerFactory.getLogger(V1_7_0_2__data.class);

    @Override
    public Integer getChecksum() {
        return -978275368;
    }

    @Override
    public void migrate(final Connection connection) throws Exception {
        if (!FlywayMigrationUtils.isScriptWithVersionAlreadyExecuted(connection, OffrestageemploiUtil.ID_EXTENSION, PREVIOUS_VERSION)) {
            serviceLabel = ServiceManager.getService(ServiceLabel.class);
            saveLabels();
        } else {
            LOGGER.info("Migration non effectuée car le traitement est déjà présent dans le script {}", PREVIOUS_VERSION);
        }
    }

    private void saveLabels() {
        // 32 : (Stage/Emploi) Durée
        serviceLabel.save(new LabelDto("32", "MD2MOIS", "- de 2 mois", "0"));
        serviceLabel.save(new LabelDto("32", "2A6MOIS", "de 2 à 6 mois", "0"));
        serviceLabel.save(new LabelDto("32", "PD6MOIS", "+ de 6 mois", "0"));
        // 33 : (Stage/Emploi) Début
        serviceLabel.save(new LabelDto("33", "00DQP", "Dès que possible", "0"));
        serviceLabel.save(new LabelDto("33", "01JANV", "Janvier", "0"));
        serviceLabel.save(new LabelDto("33", "02FEVR", "Février", "0"));
        serviceLabel.save(new LabelDto("33", "03MARS", "Mars", "0"));
        serviceLabel.save(new LabelDto("33", "04AVRI", "Avril", "0"));
        serviceLabel.save(new LabelDto("33", "05MAI", "Mai", "0"));
        serviceLabel.save(new LabelDto("33", "06JUIN", "Juin", "0"));
        serviceLabel.save(new LabelDto("33", "07JUIL", "Juillet", "0"));
        serviceLabel.save(new LabelDto("33", "08AOUT", "Août", "0"));
        serviceLabel.save(new LabelDto("33", "09SEPT", "Septembre", "0"));
        serviceLabel.save(new LabelDto("33", "10OCTO", "Octobre", "0"));
        serviceLabel.save(new LabelDto("33", "11NOVE", "Novembre", "0"));
        serviceLabel.save(new LabelDto("33", "12DECE", "Décembre", "0"));
        // 35 : (Stage/Emploi) Type stage
        serviceLabel.save(new LabelDto("35", "ALTER", "Stage en alternance", "0"));
        serviceLabel.save(new LabelDto("35", "ETRANG", "Stage à l'étranger", "0"));
            // 36 : (Stage/Emploi) Type job
            serviceLabel.save(new LabelDto("36", "PLEIN", "Temps plein", "0"));
        serviceLabel.save(new LabelDto("36", "PARTIEL", "Temps partiel", "0"));
        serviceLabel.save(new LabelDto("36", "PONCT", "Mission ponctuelle", "0"));
        // 37 : (Stage/Emploi) Type emploi
        serviceLabel.save(new LabelDto("37", "CDI", "CDI", "0"));
        serviceLabel.save(new LabelDto("37", "CDD", "CDD", "0"));
        serviceLabel.save(new LabelDto("37", "ALTER", "Alternance", "0"));
        serviceLabel.save(new LabelDto("37", "INTER", "Interim", "0"));
    }
}
