/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.ofin.migration;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.flywaydb.core.api.migration.MigrationChecksumProvider;
import org.flywaydb.core.api.migration.jdbc.JdbcMigration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.migration.FlywayMigrationUtils;
import com.kosmos.migration.exception.MigrationException;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.bean.RoleBean;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.PermissionBean;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceRole;

public class V2_7_0_6__data implements JdbcMigration, MigrationChecksumProvider {

    private ServiceLabel serviceLabel;

    private ServiceRole serviceRole;

    private final static Logger LOG = LoggerFactory.getLogger(V2_7_0_6__data.class);

    private final static String LANGUE_METIER = "0";

    private static final String PREVIOUS_VERSION = "2.1.0.1";

    @Override
    public Integer getChecksum() {
        return -154436676;
    }

    @Override
    public void migrate(final Connection connection) throws Exception {
        if (!FlywayMigrationUtils.isScriptWithVersionAlreadyExecuted(connection, OffreFormationUtil.ID_EXTENSION, PREVIOUS_VERSION)) {
            serviceLabel = ServiceManager.getService(ServiceLabel.class);
            serviceRole = ServiceManager.getService(ServiceRole.class);
            saveLabels();
            saveRole();
            saveMetiers();
        } else {
            LOG.info("Migration non effectuée car le traitement est déjà présent dans le script {}", PREVIOUS_VERSION);
        }
    }

    private void saveRole() {
        RoleBean roleBean = new RoleBean();
        roleBean.setCode("G_FORMATION");
        roleBean.setLibelle("Gestionnaire de formation");
        roleBean.setPerimetre("////");
        Collection<PermissionBean> permissionBeans = new ArrayList<>();
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0002", "C"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0002", "D"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0002", "M"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0002", "V"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0002", "S"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0003", "C"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0003", "D"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0003", "M"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0003", "V"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0003", "S"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0004", "C"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0004", "D"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0004", "M"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0004", "V"));
        permissionBeans.add(new PermissionBean(PermissionBean.TYPE_FICHE, "0004", "S"));
        roleBean.setPermissions(permissionBeans);
        serviceRole.save(roleBean);
    }

    private void saveLabels() {
        // 20 : (Formation) Type de diplôme
        serviceLabel.save(new LabelDto("20", "#10000#", "Niveau Licence", "0"));
        serviceLabel.save(new LabelDto("20", "11000", "DEUST", "0"));
        serviceLabel.save(new LabelDto("20", "12000", "Diplôme Universitaire de Technologie", "0"));
        serviceLabel.save(new LabelDto("20", "13000", "Licence", "0"));
        serviceLabel.save(new LabelDto("20", "14000", "Licence Professionnelle", "0"));
        serviceLabel.save(new LabelDto("20", "#20000#", "Niveau Master", "0"));
        serviceLabel.save(new LabelDto("20", "21000", "Ingénieur", "0"));
        serviceLabel.save(new LabelDto("20", "23000", "Master professionnel", "0"));
        serviceLabel.save(new LabelDto("20", "24000", "Master recherche", "0"));
        serviceLabel.save(new LabelDto("20", "#30000#", "Niveau Doctorat", "0"));
        serviceLabel.save(new LabelDto("20", "31000", "Attestation de Formation Spécialisée Approfondie", "0"));
        serviceLabel.save(new LabelDto("20", "32000", "Capacité médecine", "0"));
        serviceLabel.save(new LabelDto("20", "33000", "DES médecine", "0"));
        serviceLabel.save(new LabelDto("20", "34000", "DES pharmacie", "0"));
        serviceLabel.save(new LabelDto("20", "35000", "Dip Etudes Spéc Compl médecine", "0"));
        serviceLabel.save(new LabelDto("20", "36000", "Dip Etudes Spéc Complémentaire pharmacie", "0"));
        serviceLabel.save(new LabelDto("20", "37000", "Diplôme d'Etat de docteur en Médecine", "0"));
        serviceLabel.save(new LabelDto("20", "38000", "Diplôme d'Etat de docteur en Pharmacie", "0"));
        serviceLabel.save(new LabelDto("20", "39000", "Diplôme de recherche technologique (DRT)", "0"));
        serviceLabel.save(new LabelDto("20", "39100", "Doctorat d'université", "0"));
        serviceLabel.save(new LabelDto("20", "39200", "Doctorat d'état", "0"));
        serviceLabel.save(new LabelDto("20", "39300", "Habilitation à Diriger des Recherches", "0"));
        serviceLabel.save(new LabelDto("20", "#40000#", "Concours", "0"));
        serviceLabel.save(new LabelDto("20", "42000", "Concours administratifs", "0"));
        serviceLabel.save(new LabelDto("20", "41000", "Concours de l'enseignement", "0"));
        serviceLabel.save(new LabelDto("20", "43000", "Concours de la magistrature", "0"));
        serviceLabel.save(new LabelDto("20", "#50000#", "Autres formations", "0"));
        serviceLabel.save(new LabelDto("20", "51000", "Capacité", "0"));
        serviceLabel.save(new LabelDto("20", "52000", "Certificat", "0"));
        serviceLabel.save(new LabelDto("20", "53000", "DAEU", "0"));
        serviceLabel.save(new LabelDto("20", "54000", "Diplôme d'université", "0"));
        serviceLabel.save(new LabelDto("20", "55000", "Formation continue non diplômante", "0"));
        serviceLabel.save(new LabelDto("20", "56000", "Mastères spécialisés", "0"));
        // 50 : (Formation) Domaines de formation LMD
        serviceLabel.save(new LabelDto("2004", "D1", "Arts, lettres, langues", "0"));
        serviceLabel.save(new LabelDto("2004", "D2", "Droit, économie, gestion", "0"));
        serviceLabel.save(new LabelDto("2004", "D3", "Sciences humaines et sociales", "0"));
        serviceLabel.save(new LabelDto("2004", "D4", "Sciences, technologie, santé", "0"));
        // 54 : (Elément pédagogique) Nature
        serviceLabel.save(new LabelDto("54", "SEM", "Semestre", "0"));
        serviceLabel.save(new LabelDto("54", "UE", "UE", "0"));
        serviceLabel.save(new LabelDto("54", "COUR", "Cours", "0"));
        serviceLabel.save(new LabelDto("54", "CHOI", "Choix", "0"));
        serviceLabel.save(new LabelDto("54", "STAG", "Stage", "0"));
        serviceLabel.save(new LabelDto("54", "MEM", "Mémoire", "0"));
        serviceLabel.save(new LabelDto("54", "PRJ", "Projet", "0"));
        // 55 : (K-Sup) Disciplines de formation
        serviceLabel.save(new LabelDto("55", "D01", "Arts", "0"));
        serviceLabel.save(new LabelDto("55", "D02", "Information-Communication", "0"));
        serviceLabel.save(new LabelDto("55", "D03", "Commerce", "0"));
        serviceLabel.save(new LabelDto("55", "D04", "Economie-Gestion", "0"));
        serviceLabel.save(new LabelDto("55", "D05", "Droit", "0"));
        serviceLabel.save(new LabelDto("55", "D06", "Administration", "0"));
        serviceLabel.save(new LabelDto("55", "D07", "Sciences politiques", "0"));
        serviceLabel.save(new LabelDto("55", "D08", "Lettres", "0"));
        serviceLabel.save(new LabelDto("55", "D09", "Langues", "0"));
        serviceLabel.save(new LabelDto("55", "D10", "Sciences humaines et sociales", "0"));
        serviceLabel.save(new LabelDto("55", "D11", "Sciences et technologies", "0"));
        serviceLabel.save(new LabelDto("55", "D12", "Médecine et Santé", "0"));
        serviceLabel.save(new LabelDto("55", "D13", "Sport et activités sportives", "0"));
        // 56 : (Elément pédagogique) Type d"élément
        serviceLabel.save(new LabelDto("56", "O", "Obligatoire", "0"));
        serviceLabel.save(new LabelDto("56", "X", "Obligatoire à choix", "0"));
        serviceLabel.save(new LabelDto("56", "F", "Facultatif", "0"));
        // 57 : (Elément pédagogique) Semestre
        serviceLabel.save(new LabelDto("57", "S1", "Semestre 1", "0"));
        serviceLabel.save(new LabelDto("57", "S2", "Semestre 2", "0"));
        serviceLabel.save(new LabelDto("57", "S3", "Semestre 3", "0"));
        serviceLabel.save(new LabelDto("57", "S4", "Semestre 4", "0"));
        serviceLabel.save(new LabelDto("57", "S5", "Semestre 5", "0"));
        serviceLabel.save(new LabelDto("57", "S6", "Semestre 6", "0"));
        // 58 : (Elément pédagogique) Langues d"enseignement
        serviceLabel.save(new LabelDto("58", "FR", "Français", "0"));
        serviceLabel.save(new LabelDto("58", "EN", "Anglais", "0"));
        serviceLabel.save(new LabelDto("2034", "SSCOND", "Sous conditions", "0"));
    }

    private void saveMetiers() throws IOException {
        String filePath = WebAppUtil.getAbsoluteExtensionsPath() + OffreFormationUtil.ID_EXTENSION + "/WEB-INF/classes/migration/metiers.csv";
        File metierCsvFile = new File(filePath);
        List<String> untreatedLines = new ArrayList<>();
        if (metierCsvFile.exists() && metierCsvFile.canRead()) {
            untreatedLines = loadWorkFile(metierCsvFile);
        }
        for(String untreatedLine : untreatedLines){
            getBeanFromLine(untreatedLine)
                .ifPresent(labelDto -> serviceLabel.save(labelDto));
        }
    }

    /**
     * Charge le fichier contenant les métiers
     * @param metierCsvFile le fichier csv
     * @return la liste des lignes non traitées
     * @throws IOException
     */
    private List<String> loadWorkFile(final File metierCsvFile) throws IOException {
        final List<String> untreatedLines = new ArrayList<>();
        try (BufferedReader bufferedReader = new BufferedReader(new FileReader(metierCsvFile))) {
            String currentLine;
            while ((currentLine = bufferedReader.readLine()) != null) {
                boolean ok = manageWorkLine(currentLine);
                if(!ok){
                    untreatedLines.add(currentLine);
                }
            }
        }
        return untreatedLines;
    }

    /**
     * Enregistre une ligne métier.
     * @param currentLine la ligne à enregistrer
     * @return false si la ligne n'a pas été enregistrée par manque de données connexe
     */
    private boolean manageWorkLine(final String currentLine) {
        boolean treatmentOk = false;
        try {
            Optional<LabelDto> labelDto = getBeanFromLine(currentLine);
            if(labelDto.isPresent()){
                serviceLabel.save(labelDto.get());
                treatmentOk = true;
            }
        } catch (MigrationException e){
            LOG.error("Ligne incomplète", e);
        }
        return treatmentOk;
    }

    private Optional<LabelDto> getBeanFromLine(String csvLine) throws MigrationException {
        if (StringUtils.isEmpty(csvLine) || StringUtils.countMatches(csvLine, ';') != 2) {
            throw new MigrationException("Uncomplete line : " + csvLine);
        }
        String[] metier = csvLine.split(";");
        String codeParent = "NULL".equals(metier[2]) ? null : metier[2];
        Long parentLabelId = null;
        if(codeParent != null) {
            LabelDto parentLabel = serviceLabel.getByTypeCodeLanguage(MetierSerializer.TYPE_LIBELLE_METIER, codeParent, LANGUE_METIER);
            if(parentLabel != null) {
                parentLabelId = parentLabel.getId();
            }
        }
        if(StringUtils.isNotEmpty(codeParent) && parentLabelId == null){
            // Dans ce cas le label parent n'est pas encore créé on relègue donc à plus tard la création du label courant
            return Optional.empty();
        }

        String code = metier[0];
        LabelDto labelDto = new LabelDto(MetierSerializer.TYPE_LIBELLE_METIER, code, metier[1], LANGUE_METIER);
        labelDto.setIdLibelleParent(parentLabelId);
        return Optional.of(labelDto);
    }
}
