/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.ofin.module.service.beanexport;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.usinesite.utils.UASServicesHelper;
import com.kportal.extension.module.bean.FicheBeanExport;
import com.kportal.extension.module.dto.ServiceBeanExportDto;
import com.kportal.extension.module.service.impl.beanexport.ServiceBeanExportFiche;
import com.ksup.objetspartages.om.Parcours;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.ContexteDao;

/**
 * Service d'export de {@link com.ksup.objetspartages.bean.ParcoursBean}.
 * Demande une gestion spéciale pour éviter les boucles infinies liées aux liens "parcours amont" et "parcours aval".
 */
public class ServiceBeanExportParcours extends ServiceBeanExportFiche {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceBeanExportParcours.class);

    /**
     * Permet la génération d'un {@link FicheBeanExport} de {@link com.ksup.objetspartages.bean.ParcoursBean} sans les liens parcoursAmont/parcoursAval.
     * @param serviceBeanExportDto le dto contenant les informations d'export
     * @param metatag le metatag à exporter
     * @return le {@link FicheBeanExport} correspondant
     * @throws Exception
     */
    public FicheBeanExport buildBeanExportWithoutChildren(final ServiceBeanExportDto serviceBeanExportDto, final MetatagBean metatag) throws Exception {
        final String typeObjet = ReferentielObjets.getNomObjet(metatag.getMetaCodeObjet());
        final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(typeObjet);
        if (ficheUniv == null) {
            return null;
        }
        final String codeFiche = metatag.getMetaCode();
        final String langue = metatag.getMetaLangue();
        FicheBeanExport finalBeanExport = null;
        try (final ContexteDao ctxConnection = new ContexteDao()) {
            ficheUniv.setCtx(ctxConnection);
            ficheUniv.init();
            if (ficheUniv.selectCodeLangueEtat(codeFiche, langue, metatag.getMetaEtatObjet()) > 0) {
                ficheUniv.nextItem();
                if (ficheUniv instanceof Parcours) {
                    Parcours parcours = getParcoursFromFicheWithoutChildren(ficheUniv);
                    final FicheBeanExport<?> beanExport = UASServicesHelper.getFicheBeanExport(parcours, UASServicesHelper.getBeanClass(parcours));
                    if (beanExport.getBean() != null) {
                        populateBeanExport(beanExport, serviceBeanExportDto, metatag, parcours);
                        //ajout du contenu de la fiche dans le tableau des contenus à exporter
                        finalBeanExport = executeProcessors(beanExport, serviceBeanExportDto);
                    } else {
                        LOG.warn("Le bean de la fiche ({}) est null pour le metatag [{}] : contenu ignoré", typeObjet, metatag.getId());
                    }
                }
            }
        }
        return finalBeanExport;
    }

    /**
     * Transforme la {@link FicheUniv} en {@link Parcours} et supprime le lien avec les enfants (parcours amont/aval).
     * Ce lien est supprimé pour éviter la boucle infinie (A rattaché à B, B rattaché à A).
     * Cette méthode est appelée dans le contexte de serialisation d'un ParcoursLink.
     * Donc ce n'est qu'aux enfants de la fiche sérialisé à l'origine qu'on supprime les enfants.
     * @param ficheUniv
     * @return
     */
    private Parcours getParcoursFromFicheWithoutChildren(final FicheUniv ficheUniv) {
        Parcours parcours = (Parcours) ficheUniv;
        parcours.setParcoursAmontBeans(null);
        parcours.setParcoursAvalBeans(null);
        return parcours;
    }
}
