/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.ofin.serialization;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import com.kportal.extension.module.bean.FicheBeanExport;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ParcoursLink;

/**
 * Désérialiseur de classe {@link ParcoursLink}.
 */
public class ParcoursLinkDeserializer extends StdDeserializer<ParcoursLink> {

    private static final Logger LOG = LoggerFactory.getLogger(ParcoursLinkDeserializer.class);

    protected ParcoursLinkDeserializer() {
        this(null);
    }

    protected ParcoursLinkDeserializer(final Class<?> vc) {
        super(vc);
    }

    @Override
    public ParcoursLink deserialize(final JsonParser jp, final DeserializationContext ctxt) throws IOException {
        JsonNode rootNode = jp.getCodec().readTree(jp);
        return new ParcoursLink(getParcoursFromRoot(jp, rootNode), rootNode.get("right").asText());
    }

    /**
     * Récupère la fiche parcours contenue dans le noeud json.
     * @param jp le parser
     * @param root le noeud contenant
     * @return le {@link ParcoursBean} trouvé ou null
     */
    private ParcoursBean getParcoursFromRoot(final JsonParser jp, final JsonNode root) {
        JsonNode leftJsonNode = root.get("left");
        FicheBeanExport ficheBeanExport = null;
        try {
            ficheBeanExport = jp.getCodec().treeToValue(leftJsonNode, FicheBeanExport.class);
        } catch (JsonProcessingException e) {
            LOG.error("Erreur lors de la récupération du parcours", e);
        }
        ParcoursBean result = null;
        if (ficheBeanExport != null && ficheBeanExport.getBean() instanceof ParcoursBean) {
            result = (ParcoursBean) ficheBeanExport.getBean();
        }
        return result;
    }
}
