/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.ofin.serialization;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kosmos.ofin.module.service.beanexport.ServiceBeanExportParcours;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.extension.module.ModuleHelper;
import com.kportal.extension.module.bean.FicheBeanExport;
import com.kportal.extension.module.dto.ServiceBeanExportDto;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ParcoursLink;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceMetatag;

/**
 * Sérialiseur de classe {@link ParcoursLink}.
 */
public class ParcoursLinkSerializer extends StdSerializer<ParcoursLink> {

    private static final Logger LOG = LoggerFactory.getLogger(ParcoursLinkSerializer.class);

    private ServiceBeanExportParcours serviceBeanExportParcours;

    private ServiceMetatag serviceMetatag;

    protected ParcoursLinkSerializer() {
        this(null);
    }

    protected ParcoursLinkSerializer(final Class<ParcoursLink> t) {
        super(t);
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
    }

    @Override
    public void serialize(final ParcoursLink value, final JsonGenerator gen, final SerializerProvider provider) throws IOException {
        MetatagBean metatagBean = serviceMetatag.getByMetaCodeObjetCodeLangueEtat(ReferentielObjets.getCodeObjetParClasseBean(ParcoursBean.class), value.getParcours().getCode(), value.getParcours().getLangue(), value.getParcours().getEtatObjet());
        if (metatagBean == null) {
            return;
        }
        ServiceBeanExportDto serviceBeanExportDto = buildServiceBeanExportDto();
        FicheBeanExport ficheBeanExport = null;
        try {
            ficheBeanExport = getServiceBeanExportParcours().buildBeanExportWithoutChildren(serviceBeanExportDto, metatagBean);
        } catch (Exception e) {
            LOG.error("Erreur lors de la récupération du bean export", e);
        }
        if (ficheBeanExport != null) {
            gen.writeStartObject();
            gen.writeObjectField("left", ficheBeanExport);
            gen.writeObjectField("right", value.getRight());
            gen.writeEndObject();
        }
    }

    /**
     * Génère le dto nécessaire à l'export de la fiche {@link ParcoursBean}.
     */
    private ServiceBeanExportDto buildServiceBeanExportDto() {
        ServiceBeanExportDto serviceBeanExportDto = new ServiceBeanExportDto();
        serviceBeanExportDto.setModule(ModuleHelper.getModule("offreformationObjetParcours"));
        return serviceBeanExportDto;
    }

    private ServiceBeanExportParcours getServiceBeanExportParcours() {
        if (serviceBeanExportParcours == null) {
            serviceBeanExportParcours = ApplicationContextManager.getBean(OffreFormationUtil.ID_EXTENSION, "defaultServiceBeanExportParcours", ServiceBeanExportParcours.class);
        }
        return serviceBeanExportParcours;
    }
}
