/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.search.index.mapper.context;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.kportal.cms.objetspartages.annotation.Metier;

/**
 * The Class MetierContextSerializer.
 */
public class MetierContextSerializer implements ContextualSerializer {

    /**
     * The Constant LOG.
     */
    private static final Logger LOG = LoggerFactory.getLogger(MetierContextSerializer.class);

    /**
     * {@inheritDoc}
     */
    @Override
    public JsonSerializer<?> createContextual(final SerializerProvider paramSerializerProvider, final BeanProperty paramBeanProperty) throws JsonMappingException {
        final String nomParam = paramBeanProperty.getName();
        Metier metierAnnotation = paramBeanProperty.getAnnotation(Metier.class);
        if (null == metierAnnotation) {
            metierAnnotation = paramBeanProperty.getContextAnnotation(Metier.class);
        }
        if (null != metierAnnotation) {
            LOG.debug("Utilisation de la sérialisation Métier pour l'attribut '{}'", nomParam);
            return MetierSerializer.instance;
        }
        return null;
    }
}
