/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.search.index.mapper.serializer;

import java.io.IOException;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.services.ServiceLabel;

/**
 * The Class MetierSerializer.
 */
public class MetierSerializer extends AbstractLanguageSerializer<String> {

    public final static String TYPE_LIBELLE_METIER = "2035";

    /**
     * The Constant instance.
     */
    public static final MetierSerializer instance = new MetierSerializer();

    /**
     * The Constant serialVersionUID.
     */
    private static final long serialVersionUID = -9193858579287338368L;

    /**
     * The Constant LOG.
     */
    private static final Logger LOG = LoggerFactory.getLogger(MetierSerializer.class);

    /**
     * The serviceMetier.
     */
    private final transient ServiceLabel serviceLabel;

    /**
     * The Constructor MetierSerializer.
     */
    public MetierSerializer() {
        super(String.class);
        this.serviceLabel = ServiceManager.getService(ServiceLabel.class);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void serialize(final String metierCodes, final JsonGenerator jsonGen, final SerializerProvider provider) throws IOException {
        if (StringUtils.isBlank(metierCodes)) {
            jsonGen.writeNull();
            return;
        }
        String langue = processLanguage(jsonGen, provider);
        jsonGen.writeStartArray();
        for (final String frg : StringUtils.split(metierCodes, ';')) {
            final LabelDto labelDto = serviceLabel.getByTypeCodeLanguage(TYPE_LIBELLE_METIER, frg, langue);
            if (null == labelDto) {
                LOG.warn("Le métier recherché est introuvable avec les données suivantes (code='{}')", frg);
                continue;
            }
            jsonGen.writeStartObject();
            // spécificités du métier
            jsonGen.writeStringField("metier_code", labelDto.getCode());
            jsonGen.writeStringField("metier_appelation", labelDto.getLibelle());
            jsonGen.writeEndObject();
        }
        jsonGen.writeEndArray();
    }
}
