/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.seo.sitemap;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.seo.util.SEOUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.utils.ContexteUtil;
import com.univ.utils.DateUtil;
import com.univ.utils.URLResolver;

/**
 * Processus permettant de générer le fichier sitemap.xml à la racine du site
 */
public class GenererSitemap extends ProcessusBean {

    private static final Logger LOG = LoggerFactory.getLogger(GenererSitemap.class);

    private static final String ECRAN_GESTION = "GESTION";

    /**
     * @param infoBean
     */
    public GenererSitemap(final InfoBean infoBean) {
        super(infoBean);
    }

    /**
     * Point d'entrée du processus
     */
    @Override
    protected boolean traiterAction() {
        try {
            ecranLogique = infoBean.getEcranLogique();
            action = infoBean.getActionUtilisateur();
            final AutorisationBean autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
            if (autorisations == null || !autorisations.isWebMaster()) {
                infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
                infoBean.setEcranLogique("LOGIN");
            } else {
                if (ecranLogique == null) {
                    if (action.equals("GESTION")) {
                        preparerGESTION();
                    }
                } else {
                    LOG.error("Action non prise en charge par le module de génération du sitemap !!");
                }
                // placer l'état dans le composant d'infoBean
                infoBean.setEcranLogique(ecranLogique);
            }
        } catch (final Exception e) {
            LOG.error("Une erreur est survenue lors de l'appel au processus", e);
            infoBean.addMessageErreur(SEOUtil.getMessage("BO.SEO.GENERER_SITEMAP.ERREUR.APPEL_PROCESSUS"));
        }
        /* On continue si on n'est pas à la FIN !!! */
        return etat == FIN;
    }

    private void preparerGESTION() throws UnsupportedEncodingException {
        final Collection<HashMap<String, String>> sites = new ArrayList<>();
        final ServiceInfosSiteProcessus serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
        for (final InfosSite currentSite : serviceInfosSite.getListeInfosSitesActifs()) {
            if(!currentSite.isNonIndexe()) {
                final HashMap<String, String> currentMap = new HashMap<>();
                currentMap.put("alias", currentSite.getIntitule());
                currentMap.put("code", currentSite.getAlias());
                final String urlSiteMap = URLEncoder.encode(URLResolver.getAbsoluteUrl(StringUtils.EMPTY, ContexteUtil.getContexteUniv(), currentSite) + SEOUtil.getProperty(SiteMapServlet.URL_PROPERTY), CharEncoding.DEFAULT);
                currentMap.put("link", urlSiteMap);
                File siteMapFile = SEOUtil.getSiteMapFile(currentSite.getHttpHostname());
                if(siteMapFile.exists()){
                    currentMap.put("lastModified", DateUtil.formatDate(new Date(siteMapFile.lastModified())));
                }
                sites.add(currentMap);
            }
        }
        infoBean.set("SITES", sites);
        infoBean.setTitreEcran(SEOUtil.getMessage("BO.SEO.GESTION.TITRE"));
        ecranLogique = ECRAN_GESTION;
    }
}
