/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.seo.sitemap;

import java.io.File;
import java.util.Arrays;
import java.util.Collection;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.seo.util.SEOUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.scheduling.spring.quartz.LogReportJob;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;
import com.univ.utils.ContexteUtil;

/**
 * @author olivier.camon
 */
public class SiteMapJob extends LogReportJob {

	private static final Logger LOG = LoggerFactory.getLogger(SiteMapJob.class);

	private ServiceInfosSite serviceInfosSite;

	public SiteMapJob() {
		serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
	}

	@Override
	public void perform() {
		try {
			// ajout d'un contexte dans le thread local
			ContexteUtil.setContexteSansRequete();
			// génération des sitemaps par site déclaré
			LOG.info("***** Génération des sitemaps *****");
			deleteOldSitemaps();
			SitemapGenerator sitemapGenerator = SEOUtil.getModuleSiteMap().getGenerator();
			final Collection<InfosSite> listeInfosSite = serviceInfosSite.getListeInfosSitesActifs();
			for (final InfosSite infosSite: listeInfosSite) {
				sitemapGenerator.generate(infosSite.getAlias());
			}
		} finally {
			// suppression du contexte du thread local
			ContexteUtil.releaseContexteUniv();
		}
	}

	/**
	 * Suppression des anciens sitemaps.
	 */
	private void deleteOldSitemaps() {
		File siteMapsDirectory = new File(SEOUtil.getSitemapDir());
		File[] files = siteMapsDirectory.listFiles(file -> StringUtils.endsWith(file.getName(), SEOUtil.SITE_MAP_SUFFIX));
		if (ArrayUtils.isNotEmpty(files)) {
			Arrays.stream(files).forEach(File::delete);
		}
	}
}
