/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.seo.sitemap;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.kosmos.seo.util.SEOUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.scheduling.spring.quartz.LogReportJob;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;

/**
 * @author olivier.camon
 */
@Controller
public class SiteMapServlet {

	private static final Logger LOG = LoggerFactory.getLogger(LogReportJob.class);

	public final static String URL_PROPERTY = "sitemap.mapping";

	/**
	 * Méthode standard, sous-classée si besoin.
	 *
	 * @param req
	 *            la requete http en GET
	 * @param res
	 *            la réponse à renvoyer
	 *
	 */
	@RequestMapping(value = "${" + URL_PROPERTY + ":/sitemap.xml}", method = RequestMethod.GET)
	public void doGet(final HttpServletRequest req, final HttpServletResponse res) {
		final ServiceInfosSite serviceInfosSite = ServiceManager.getServiceForBean(InfosSiteImpl.class);
		final InfosSite infosSite = serviceInfosSite.getActiveSiteByHost(req.getServerName());
		File file = SEOUtil.getSiteMapFile(infosSite.getHttpHostname());
		try {
			if(!infosSite.isNonIndexe() && file.exists()){
				res.setContentType("text/xml");
				res.setContentLength((int) file.length());
				Files.copy(file.toPath(), res.getOutputStream());
			}else{
				LOG.warn("Unable to retieve sitemap file {}", file.getName());
				res.sendError(HttpServletResponse.SC_NOT_FOUND);
			}
		} catch (IOException e) {
			LOG.error("Unable to forward the request", e);
		}
	}
}
