/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.seo.sitemap;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.seo.util.SEOUtil;
import com.univ.multisites.service.ServiceInfosSite;

public class SitemapGenerator {

    private static final Logger LOGGER = LoggerFactory.getLogger(SitemapGenerator.class);

    private ServiceInfosSite serviceInfosSite;

    public void setServiceInfosSite(final ServiceInfosSite serviceInfosSite) {
        this.serviceInfosSite = serviceInfosSite;
    }


    /**
     * Génère une fichier sitemap pour site donné.
     * @param alias l'alias du site
     * @return le chemin vers le fichier
     */
    public String generate(String alias) {
        String cheminFichierXml = SEOUtil.getSitemapDir() + File.separator + serviceInfosSite.getInfosSite(alias).getHttpHostname() + SEOUtil.SITE_MAP_SUFFIX;
        try {
            FileUtils.write(new File(cheminFichierXml), genererSitemap(alias), CharEncoding.DEFAULT);
        } catch (IOException e) {
            LOGGER.error("unable to generate a sitemap for the site " + alias, e);
        }
        return cheminFichierXml;
    }

    /**
     * Génère le contenu du fichier sitemap.xml
     * @param alias l'alias du site
     * @return le contenu du fichier
     */
    public String genererSitemap(final String alias) {
        SiteMapBuilder siteMapBuilder = new SiteMapBuilder(alias);
        return siteMapBuilder.generateSiteMap();
    }

    /**
     * Génère une fichier sitemap pour site donné.
     * @param ctx le contexte
     * @param alias l'alias du site
     * @return le chemin vers le fichier
     * @deprecated utiliser la signature sans le contexte
     */
    @Deprecated
    public String generate(OMContext ctx, String alias) {
        return generate(alias);
    }

    /**
     * Génère le contenu du fichier sitemap.xml
     * @param ctx le contexte
     * @param alias l'alias du site
     * @return le contenu du fichier
     * @deprecated utiliser la signature sans le contexte
     */
    @Deprecated
    public String genererSitemap(final OMContext ctx, final String alias) {
        return genererSitemap(alias);
    }
}
