/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.seo.util;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.seo.sitemap.ModuleSiteMap;
import com.kosmos.seo.sitemap.SitemapGenerator;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.extension.ExtensionHelper;
import com.kportal.extension.IExtension;
import com.kportal.extension.module.IModule;
import com.univ.objetspartages.om.ReferentielObjets;

public class SEOUtil {

    private static final String ID_EXTENSION = "seo";

    private static final String ID_GENERATOR = "seoSitemapGenerator";

    private static final String SITEMAP_DIR = "sitemap.path";

    private static final String DEFAULT_SITEMAP_DIR = "sitemap";

    public static final String SITE_MAP_SUFFIX = "-sitemap.xml";

    public static final String SITE_PROPERTY_CODE_VERIFICATION = "CODE_VERIFICATION";

    public static IExtension getExtension() {
        return ExtensionHelper.getExtensionManager().getExtension(ID_EXTENSION);
    }

    public static String getProperty(String key) {
        return PropertyHelper.getProperty(ID_EXTENSION, key);
    }

    public static String getMessage(String key) {
        return MessageHelper.getMessage(ID_EXTENSION, key);
    }

    public static SitemapGenerator getGenerator() {
        return ApplicationContextManager.getBean(ID_EXTENSION, ID_GENERATOR, SitemapGenerator.class);
    }

    public static ModuleSiteMap getModuleSiteMap() {
        for (IModule module : getExtension().getModules()) {
            if (module instanceof ModuleSiteMap) {
                return (ModuleSiteMap) module;
            }
        }
        return null;
    }

    /**
     * Calcule le chemin d'upload des fichiers
     *
     * @return
     */
    public static String getSitemapDir() {
        final String res = StringUtils.defaultIfEmpty(PropertyHelper.getExtensionProperty(ID_EXTENSION, SITEMAP_DIR), WebAppUtil.getStorageDir() + DEFAULT_SITEMAP_DIR);
        if (StringUtils.isNotEmpty(res) && !new File(res).exists()) {
            new File(res).mkdirs();
        }
        return res;
    }

    /**
     * Retourne l'ensemble des codes objets pour lequels il existe une propriété fiche.[objet].sitemap=1.
     * @return la liste des objets devant être présents dans le fichier sitemap
     */
    public static List<String> getSitemapObjects() {
        List<String> codeObjetsForSitemap = new ArrayList<>();

        for(final String codeObjet : ReferentielObjets.getListeCodesObjet()){
            final String nomObjet = ReferentielObjets.getNomObjet(codeObjet);
            final String property = PropertyHelper.getProperty(ReferentielObjets.getExtension(codeObjet), "fiche." + nomObjet.toUpperCase() + ".sitemap");
            if(StringUtils.isNotEmpty(property) && "1".equals(property)){
                codeObjetsForSitemap.add(codeObjet);
            }
        }
        return codeObjetsForSitemap;
    }

    /**
     * Renvoie le fichier sitemap correspondant au site donné.
     * @param hostname le host du site
     * @return le fichier sitemap
     */
    public static File getSiteMapFile(String hostname){
        return new File(SEOUtil.getSitemapDir() + File.separator + hostname + "-sitemap.xml");
    }

}
