/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: dam
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.toolbox.provider.impl.tag;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.toolbox.bean.TagValue;
import com.kportal.core.config.MessageHelper;
import com.univ.mediatheque.Mediatheque;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.utils.ContexteUtil;
import com.univ.utils.media.TypeRessourceTypeMediaHelper;

/**
 * Created on 03/08/15.
 */
public class KTagGalleryDataProvider extends AbstractKTagDataProvider {

    private static final Logger LOG = LoggerFactory.getLogger(KTagGalleryDataProvider.class);

    @Override
    public void populateData(final Map<String, String> data, TagValue tagValue) {
        final String value = tagValue.getValue().replace("[traitement;media_galerie;", StringUtils.EMPTY).replace("]", StringUtils.EMPTY);
        splitValues(data, value);
        handleNoArbo(data);
        handleDynamik(data);
        handleLabels(data);
    }

    private void handleNoArbo(Map<String, String> data) {
        if(data.containsKey("CODE_RUBRIQUE")) {
            final String codeRubrique = data.get("CODE_RUBRIQUE");
            if(StringUtils.isNotBlank(codeRubrique) && codeRubrique.contains("NOARBO")) {
                data.put("CODE_RUBRIQUE", data.get("CODE_RUBRIQUE").replace("_NOARBO", StringUtils.EMPTY));
                data.put("CODE_RUBRIQUE_NOARBO", "1");
            }
        }
        if(data.containsKey("CODE_RATTACHEMENT")) {
            final String codeStructure = data.get("CODE_RATTACHEMENT");
            if(StringUtils.isNotBlank(codeStructure) && codeStructure.contains("NOARBO")) {
                data.put("CODE_RATTACHEMENT", data.get("CODE_RATTACHEMENT").replace("_NOARBO", StringUtils.EMPTY));
                data.put("CODE_RATTACHEMENT_NOARBO", "1");
            }
        }
    }

    private void handleDynamik(Map<String, String> data) {
        if(data.containsKey("CODE_RUBRIQUE")) {
            final String codeRubrique = data.get("CODE_RUBRIQUE");
            if("DYNAMIK".equals(codeRubrique)) {
                data.remove("CODE_RUBRIQUE");
                data.put("CODE_RUBRIQUE_DYNAMIK", "1");
            }
        }
        if(data.containsKey("CODE_REDACTEUR")) {
            final String codeRedacteur = data.get("CODE_REDACTEUR");
            if("DYNAMIK".equals(codeRedacteur)) {
                data.remove("CODE_REDACTEUR");
                data.put("CODE_REDACTEUR_DYNAMIK", "1");
            }
        }
    }

    private void handleLabels(Map<String, String> data) {
        if(data.containsKey("CODE_RUBRIQUE")) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final String codeRubrique = data.get("CODE_RUBRIQUE");
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(codeRubrique);
            String label = MessageHelper.getCoreMessage("RUBRIQUE_INEXISTANTE");
            if (rubriqueBean != null) {
                label = rubriqueBean.getIntitule();
            }
            data.put("LIBELLE_CODE_RUBRIQUE", label);
        }
        if(data.containsKey("CODE_RATTACHEMENT")) {
            final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
            final String codeStructure = data.get("CODE_RATTACHEMENT");
            final StructureModele structure = serviceStructure.getByCodeLanguage(codeStructure, LangueUtil.getIndiceLocaleDefaut());
            if(structure != null) {
                data.put("LIBELLE_CODE_RATTACHEMENT", structure.getLibelleAffichable());
            }
        }
        if(data.containsKey("CODE_REDACTEUR")) {
            final String codeRedacteur = data.get("CODE_REDACTEUR");
            try {
                data.put("LIBELLE_CODE_RATTACHEMENT", FicheAnnuaire.getLibelleAffichable(codeRedacteur, StringUtils.EMPTY));
            } catch (Exception e) {
                LOG.debug("unable to find the label", e);
                data.put("LIBELLE_CODE_RATTACHEMENT", MessageHelper.getCoreMessage("BO_LIBELLE_VIDE"));
            }
        }
        if(data.containsKey("TYPE_MEDIA")) {
            final List<String> labels = new ArrayList<>();
            final String[] mediaTypes = data.get("TYPE_MEDIA").split(";");
            final Map<String, String> types = Mediatheque.getInstance().getTypesRessourcesAffichables();
            final TypeRessourceTypeMediaHelper typeRessourceTypeMediaHelper = (TypeRessourceTypeMediaHelper) ApplicationContextManager.getCoreContextBean(TypeRessourceTypeMediaHelper.ID_BEAN);
            for(String mediaType : mediaTypes) {
                for (final Map.Entry<String, String> currentType : types.entrySet()) {
                    final Map<String, String> realTypes = typeRessourceTypeMediaHelper.getCodeEtLibelleTypeRessourceEtTypeMedia(ContexteUtil.getContexteUniv(), currentType.getKey());
                    if (realTypes.containsKey(mediaType)) {
                        labels.add(realTypes.get(mediaType));
                    }
                }
            }
            data.put("LIBELLE_TYPE_MEDIA", StringUtils.join(labels, ";"));
        }
    }
}
