/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.composant;

import java.util.Arrays;
import java.util.Collection;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.utils.UASUtil;
import com.kportal.core.autorisation.util.PermissionUtil;
import com.kportal.extension.module.IModule;
import com.kportal.extension.module.ModuleHelper;
import com.kportal.extension.module.composant.Composant;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.PermissionBean;

public class UsineASiteComposant extends Composant {

    public static final String ID_BEAN = "composantUsineASite";

    /**
     * La permission permettant de modifier des sites
     */
    public static final String CODE_ACTION_MODIFICATION = "M";

    public static final String CODE_PERMISSION = "site";

    /**
     * Autorité d'accès aux fonctionnalités avancées.
     */
    private Autorizer advancedFeaturesAutorizer;

    /**
     * La permission permettant de gérer les sites (Create Update Delete)
     */
    public static final String CODE_ACTION_GESTION = "G";

    /**
     * Code de l'action d'import.
     */
    public static final String CODE_ACTION_IMPORTER = "IMPORTER";

    /**
     * Les actions possible avec les droits de modification (lister & modifier)
     */
    private static final Collection<String> ACTIONS_MODIFICATION = Arrays.asList("MODIFIER", "ACCUEIL", "LISTE", "VALIDER_MODIFICATION");

    /**
     * En fonction des autorisations et de l'action fourni on retourne vrai si l'utilisateur a les droits en fonction de l'action qu'il entreprend.
     */
    public static boolean isAutoriseParActionProcessus(final AutorisationBean autorisations, final String action) {
        boolean isAutorise = autorisations != null && autorisations.isWebMaster();
        if (!isAutorise && autorisations != null) {
            isAutorise = autorisations.possedePermission(getPermissionGestion());
            if (!isAutorise && ACTIONS_MODIFICATION.contains(action)) {
                isAutorise = autorisations.possedePermission(getPermissionModification());
            }
        }
        return isAutorise;
    }

    public static IModule getModule() {
        return ModuleHelper.getModule(ApplicationContextManager.DEFAULT_CORE_CONTEXT, ID_BEAN);
    }

    /**
     * Retourne la permission de gestion des sites.
     */
    public static PermissionBean getPermissionGestion() {
        return PermissionUtil.getPermissionBean(UASUtil.getModule(), CODE_PERMISSION, CODE_ACTION_GESTION);
    }

    /**
     * Retourne la permission de modification des sites.
     */
    public static PermissionBean getPermissionModification() {
        return PermissionUtil.getPermissionBean(UASUtil.getModule(), CODE_PERMISSION, CODE_ACTION_MODIFICATION);
    }

    @Override
    public boolean isVisible(final AutorisationBean autorisation) {
        boolean isVisible = Boolean.FALSE;
        if (accessibleBO && autorisation != null) {
            isVisible = autorisation.possedePermission(getPermissionGestion()) || autorisation.possedePermission(getPermissionModification());
        }
        return isVisible;
    }

    @Override
    public boolean isActionVisible(final AutorisationBean autorisation, final String code) {
        boolean isVisible = super.isVisible(autorisation);
        if (!isVisible && autorisation != null) {
            isVisible = autorisation.possedePermission(getPermissionGestion());
        }
        if (isVisible && CODE_ACTION_IMPORTER.equals(code)) {
            isVisible = advancedFeaturesAutorizer.canAccess(autorisation, Autorizer.ACTION_PERMISSION_IMPORTER);
        }
        return isVisible;
    }

    public void setAdvancedFeaturesAutorizer(final Autorizer advancedFeaturesAutorizer) {
        this.advancedFeaturesAutorizer = advancedFeaturesAutorizer;
    }
}
