/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution;

import com.kosmos.uas.execution.dto.SiteImportDto;
import com.kosmos.uas.execution.errorcase.ImportSiteErrorCase;
import com.kosmos.uas.execution.step.ImportSaveSiteStep;
import com.kosmos.uas.execution.step.ImportSiteUnzipStep;
import com.kosmos.uas.execution.step.ReferenceToModuleStep;
import com.kosmos.uas.execution.step.SiteImportStep;
import com.kosmos.uas.execution.step.SiteReferenceComputerStep;
import com.kosmos.usinesite.execution.AbstractImportExportHandler;
import com.kosmos.usinesite.execution.ImportSchedulerImpl;
import com.kportal.extension.module.service.ServiceContenuModuleContext;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Traitement d'import de site.
 */
public class ImportSiteHandler extends AbstractImportExportHandler<String, SiteImportDto> {

    private InfosSiteImpl newSite;

    /**
     * Constructeur de la classe.
     */
    public ImportSiteHandler() {
        super();
        addStep(new ImportSiteUnzipStep(this));
        addStep(new SiteReferenceComputerStep(this));
        addStep(new ReferenceToModuleStep(this));
        addStep(new SiteImportStep(this, new ImportSchedulerImpl(ServiceContenuModuleContext.SITE_EXECUTION_CONTEXT)));
        addStep(new ImportSaveSiteStep(this));
        setErrorCase(new ImportSiteErrorCase(this));
    }

    @Override
    protected Class<String> getInputType() {
        return String.class;
    }

    @Override
    protected Class<? extends SiteImportDto> getOutputType() {
        return SiteImportDto.class;
    }

    public InfosSiteImpl getNewSite() {
        return newSite;
    }

    public void setNewSite(final InfosSiteImpl newSite) {
        this.newSite = newSite;
    }
}
