/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution.errorcase;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.execution.ErrorCase;
import com.kosmos.execution.ExecutionHandler;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.uas.execution.ImportSiteHandler;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;

/**
 * Cas d'erreur pour l'import de site.
 */
public class ImportSiteErrorCase extends ErrorCase<ImportSiteHandler> {

    private static final Logger LOG = LoggerFactory.getLogger(ImportSiteErrorCase.class);

    /**
     * Service de gestion des {@link com.univ.multisites.InfosSite}
     */
    private ServiceInfosSiteProcessus serviceInfosSite;

    /**
     * Constructeur de la classe.

     * @param handler {@link ExecutionHandler} contenant la classe
     */
    public ImportSiteErrorCase(final ImportSiteHandler handler) {
        super(handler);
        serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
    }

    @Override
    public void execute() {
        if (handler.getWorkFolder() != null) {
            handler.getWorkFolder().compressFile();
        } else {
            LOG.error("Répertoire de travail non défini, impossible de le compresser.");
        }
        try {
            if (handler.getNewSite() != null) {
                serviceInfosSite.supprimer(handler.getNewSite().getAlias());
            } else {
                LOG.error("Nouveau site non défini, impossible de le supprimer.");
            }
        } catch (Exception e) {
            LOG.error("Erreur de suppression du site", e);
        }
    }
}
