/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution.step;

import com.kosmos.execution.ExecutionStep;
import com.kosmos.execution.StepExecutionException;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.uas.execution.ImportSiteHandler;
import com.kosmos.uas.execution.dto.SiteImportDto;
import com.kosmos.usinesite.service.PreSaveSiteCheck;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Étape de sauvegarde de site pour le cas de l'importe de site.
 */
public class ImportSaveSiteStep extends ExecutionStep<SiteImportDto, SiteImportDto, ImportSiteHandler> {

    /**
     * Service de gestion des {@link com.univ.multisites.InfosSite}
     */
    private ServiceInfosSiteProcessus serviceInfosSite;

    /**
     * Constructeur de la classe.

     * @param handler handler exécutant l'étape courante.
     */
    public ImportSaveSiteStep(final ImportSiteHandler handler) {
        super(handler);
        serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
    }

    @Override
    public void execute() throws StepExecutionException {
        InfosSiteImpl siteToSave = handler.getNewSite();
        siteToSave.setCodeRubrique(input.getSiteSection());
        siteToSave.setSitePrincipal(Boolean.FALSE);
        siteToSave.setActif(false);
        try {
            PreSaveSiteCheck.getChecker().dontCheckHostUnicity().check(siteToSave);
            serviceInfosSite.modifier(siteToSave);
        } catch (Exception e) {
            throw new StepExecutionException(ImportSaveSiteStep.class, "Erreur lors de la sauvegarde du site", e);
        }
        output = input;
    }

    @Override
    protected Class<? extends SiteImportDto> getInputType() {
        return SiteImportDto.class;
    }

    @Override
    protected Class<? extends SiteImportDto> getOutputType() {
        return SiteImportDto.class;
    }
}
