/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution.step;

import java.io.File;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.execution.ExecutionStep;
import com.kosmos.execution.StepExecutionException;
import com.kosmos.uas.execution.ImportSiteHandler;
import com.kosmos.uas.execution.dto.SiteReferenceDto;
import com.kosmos.uas.utils.PluginSiteHelper;
import com.kosmos.usinesite.reference.ExtensionFileFilter;
import com.kosmos.usinesite.workfolder.WorkFolder;
import com.kosmos.usinesite.workfolder.WorkFolderType;
import com.kportal.core.webapp.WebAppUtil;
import com.kportal.util.compress.Zip;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Étape de préparation avant import de site. Dézippage du fichier à importer.
 */
public class ImportSiteUnzipStep extends ExecutionStep<String, SiteReferenceDto, ImportSiteHandler> {

    private static final Logger LOG = LoggerFactory.getLogger(ImportSiteUnzipStep.class);

    /**
     * Constructeur de la classe.

     * @param handler handler exécutant l'étape courante.
     */
    public ImportSiteUnzipStep(final ImportSiteHandler handler) {
        super(handler);
    }

    @Override
    public void execute() throws StepExecutionException {
        final File zipFile = new File(String.format("%s%s%s%s", WebAppUtil.getSauvegardePath(), WebAppUtil.DEFAULT_FICHIERS_UAS, File.separator, input));
        final String duplicationPath = String.format("%s%s%s%s", zipFile.getParent(), File.separator, input.replace("." + ExtensionFileFilter.ZIP, StringUtils.EMPTY), File.separator);
        LOG.info("Debut de l'import du site {}", zipFile.getName());
        try {
            // Décompression du fichier zip dans un dossier portant le nom du zip
            final File duplicationDirectory = new File(duplicationPath);
            Zip.decompress(zipFile, duplicationDirectory, false);
            // Copie du fichier de configuration de site source afin qu'il soit lu
            // Import des propriétés et récupération de l'infosSite correspondant
            final String importPathSite = PluginSiteHelper.getPropertiesPath(duplicationDirectory);
            final InfosSiteImpl newSite = PluginSiteHelper.importProperties(importPathSite);
            File siteDestTemp = new File(WebAppUtil.getConfigurationSitesPath() + File.separator + new File(importPathSite).getName());
            if (!siteDestTemp.exists()) {
                FileUtils.copyFile(new File(importPathSite), siteDestTemp);
            }
            PluginSiteHelper.importTemplateDatas(newSite, duplicationPath);
            File destWorkDir = PluginSiteHelper.moveFileToDuplicationDirectory(newSite, duplicationDirectory);
            output = new SiteReferenceDto();
            output.setOldSite(newSite);
            output.setNewSite(newSite);
            handler.setNewSite(newSite);
            output.setFolderPath(destWorkDir.getPath());
            handler.setWorkFolder(new WorkFolder(destWorkDir.getPath(), WorkFolderType.SITE));
        } catch (final Exception e) {
            throw new StepExecutionException(ImportSiteUnzipStep.class, e);
        }
    }

    @Override
    protected Class<String> getInputType() {
        return String.class;
    }

    @Override
    protected Class<SiteReferenceDto> getOutputType() {
        return SiteReferenceDto.class;
    }
}
