/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution.step;

import com.kosmos.execution.StepExecutionException;
import com.kosmos.uas.execution.dto.ModuleSiteDuplicationDto;
import com.kosmos.uas.execution.dto.SiteReferenceDto;
import com.kosmos.usinesite.exception.ErreurExportException;
import com.kosmos.usinesite.execution.AbstractImportExportHandler;
import com.kosmos.usinesite.execution.step.AbstractExportStep;
import com.kosmos.usinesite.utils.TransformationUtils;
import com.kosmos.usinesite.workfolder.WorkFolder;
import com.kosmos.usinesite.workfolder.WorkFolderType;
import com.kportal.extension.module.service.ServiceContenuModuleContext;

/**
 * Étape d'export de site.
 */
public class SiteExportStep extends AbstractExportStep<ModuleSiteDuplicationDto, SiteReferenceDto, AbstractImportExportHandler> {

    /**
     * Constructeur de la classe.
     */
    public SiteExportStep(final AbstractImportExportHandler handler) {
        super(ServiceContenuModuleContext.SITE_EXECUTION_CONTEXT, handler);
    }

    @Override
    public void execute() throws StepExecutionException {
        String savePath = TransformationUtils.getPathDossierDuplication(input.getNewSite());
        try {
            process(input.getOldSite().getAlias(), savePath, input.getOldSite().getCodeRubrique(), input.getModules());
        } catch (ErreurExportException e) {
            throw new StepExecutionException(SiteExportStep.class, e);
        }
        output = new SiteReferenceDto();
        output.setNewSite(input.getNewSite());
        output.setOldSite(input.getOldSite());
        output.setFolderPath(savePath);
        handler.setWorkFolder(new WorkFolder(savePath, input.getOldSite().getAlias(), WorkFolderType.SITE));
    }

    @Override
    protected Class<? extends ModuleSiteDuplicationDto> getInputType() {
        return ModuleSiteDuplicationDto.class;
    }

    @Override
    protected Class<? extends SiteReferenceDto> getOutputType() {
        return SiteReferenceDto.class;
    }
}
