/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution.step;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.exception.ErreurImportException;
import com.kosmos.execution.StepExecutionException;
import com.kosmos.uas.execution.dto.SiteImportDto;
import com.kosmos.uas.utils.PluginSiteHelper;
import com.kosmos.usinesite.execution.AbstractImportExportHandler;
import com.kosmos.usinesite.execution.ImportScheduler;
import com.kosmos.usinesite.execution.step.AbstractImportStep;
import com.kosmos.usinesite.workfolder.WorkFolder;
import com.kosmos.usinesite.workfolder.WorkFolderType;

/**
 * Étape d'import de site.
 */
public class SiteImportStep extends AbstractImportStep<SiteImportDto, SiteImportDto, AbstractImportExportHandler> {

    private static final Logger LOG = LoggerFactory.getLogger(SiteImportStep.class);

    /**
     * Constructeur de la classe.
     */
    public SiteImportStep(final AbstractImportExportHandler handler, ImportScheduler scheduler) {
        super(handler, scheduler);
    }

    @Override
    public void execute() throws StepExecutionException {
        if (handler.getWorkFolder() == null) {
            handler.setWorkFolder(new WorkFolder(input.getFolderPath(), WorkFolderType.SITE));
        }
        try {
            ImportScheduler scheduler = processImport(input.getFolderPath());
            output = new SiteImportDto();
            output.setFolderPath(input.getFolderPath());
            output.setSiteSection(scheduler.getOutputRootSection());
        } catch (ErreurImportException e) {
            throw new StepExecutionException(SiteImportStep.class, e);
        }
        try {
            PluginSiteHelper.cleanUpDirectory(input.getFolderPath(), false);
        } catch (final IOException e) {
            LOG.error("une erreur est survenue lors du nettoyage du répertoire de traitement", e);
        }
    }

    @Override
    protected Class<? extends SiteImportDto> getInputType() {
        return SiteImportDto.class;
    }

    @Override
    protected Class<? extends SiteImportDto> getOutputType() {
        return SiteImportDto.class;
    }
}
