/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.execution.step;

import java.util.Collection;

import com.kosmos.uas.execution.dto.ModuleSiteDuplicationDto;
import com.kosmos.uas.execution.dto.SiteDuplicationDto;
import com.kosmos.usinesite.execution.AbstractImportExportHandler;
import com.kosmos.usinesite.execution.filter.InfoBeanModuleFilter;
import com.kosmos.usinesite.execution.step.AbstractModuleGathererStep;
import com.kportal.extension.module.IModule;

/**
 * Étape de rassemblement des modules de l'application dans le cas d'une duplication de site.
 * @see AbstractModuleGathererStep
 */
public class SiteModuleGathererStep extends AbstractModuleGathererStep<SiteDuplicationDto, ModuleSiteDuplicationDto, AbstractImportExportHandler> {

    /**
     * Constructeur de la classe.
     * @param handler
     */
    public SiteModuleGathererStep(final AbstractImportExportHandler handler) {
        super(handler);
    }

    @Override
    public void execute() {
        filters.add(new InfoBeanModuleFilter(input.getInfoBean()));
        Collection<IModule> modules = getModulesATraiter();
        ModuleSiteDuplicationDto out = new ModuleSiteDuplicationDto();
        out.setModules(modules);
        out.setOldSite(input.getOldSite());
        out.setNewSite(input.getNewSite());
        output = out;
    }

    @Override
    public Class<? extends SiteDuplicationDto> getInputType() {
        return SiteDuplicationDto.class;
    }

    @Override
    public Class<? extends ModuleSiteDuplicationDto> getOutputType() {
        return ModuleSiteDuplicationDto.class;
    }
}
