/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.jsbsoft.jtf.exception.ErreurDonneeNonTrouve;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.utils.SaisieInfosSiteInfoBeanHelper;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.exception.ErreursSaisieInfosSite;
import com.kosmos.usinesite.service.PreSaveSiteCheck;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Action d'ajout de site.
 */
class Add extends SaisieInfoSiteAction {

    private static final Logger LOG = LoggerFactory.getLogger(Add.class);

    @Override
    public void doAction(final ActionDto actionDto) throws Exception {
        final InfosSiteImpl infosSite = new InfosSiteImpl();
        final TemplateSite template = getTemplateDepuisInfosBean(actionDto.getInfoBean());
        SaisieInfosSiteInfoBeanHelper.peuplerInfosSiteDepuisInfoBean(actionDto.getInfoBean(), infosSite, template);
        verifierDonneesInfosSite(infosSite, template);
        if (!possedePermissionGestionPourSite(infosSite, actionDto.getAutorisations())) {
            throw new ErreurActionNonAutorisee();
        }
        final String codeUtilisateur = actionDto.getAutorisations().getCode();
        try {
            PreSaveSiteCheck.getChecker().check(infosSite);
            serviceInfosSite.creer(infosSite, codeUtilisateur);
            lancementTraitementsComplementairesTemplate(infosSite, template, actionDto.getInfoBean());
        } catch (final ErreurDonneeNonTrouve e) {
            LOG.debug("Donnée non trouvée", e);
            throw new ErreursSaisieInfosSite(e.getMessage(), infosSite, template);
        }
        actionDto.getInfoBean().addMessageConfirmation(String.format(UASUtil.getMessage("CONFIRMATION_CREATION_SITE"), infosSite.getIntitule()));
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.VALIDER_AJOUTER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.FIN.name();
    }
}
