/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.taskscheduler.bean.Lock;
import com.kosmos.taskscheduler.manager.TaskSchedulerManager;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.task.DuplicateSiteUasTask;
import com.kosmos.uas.task.DuplicateSiteUasTaskParameters;
import com.kosmos.uas.utils.SaisieInfosUASUtil;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.service.PreSaveSiteCheck;
import com.kosmos.usinesite.task.UasTaskScheduler;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.kportal.core.config.MessageHelper;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Action de doublonnage d'un site.
 */
class Duplicate extends AdvancedSaisieInfoSiteAction {

    @Override
    public String getCodeActionPermission() {
        return Autorizer.ACTION_PERMISSION_DUPLIQUER;
    }

    @Override
    public void doAdvancedAction(final ActionDto actionDto) throws Exception {
        final InfosSiteImpl infosSite = recuperationSite(actionDto.getInfoBean(), actionDto.getAutorisations());
        final Lock currentLock = lockManager.getLockStatus(UasTaskScheduler.LOCK_KEY);
        if (currentLock!=null && !currentLock.isIdling()) {
            actionDto.getInfoBean().addMessageAlerte(String.format(UASUtil.getMessage("ERREUR.DUPLICATE.TASK_IN_PROGRESS"), infosSite.getAlias()));
            return;
        }
        final TemplateSite template = serviceTemplateSite.getTemplateSiteParCode(infosSite.getCodeTemplate());
        SaisieInfosUASUtil.alimenteDonneeDuplicationDepuisInfoBean(infosSite, template, actionDto.getInfoBean());
        verifierDonneesInfosSite(infosSite, template);
        checkFuturSiteUnicity(infosSite);
        PreSaveSiteCheck.getChecker().check(infosSite);
        InfosSiteImpl createdSite = serviceInfosSite.createWithoutSaving(infosSite, actionDto.getAutorisations().getCode());
        validerDuplication(actionDto.getInfoBean(), createdSite, actionDto.getAutorisations().getCode());
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.VALIDER_DUPLIQUER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.FIN.name();
    }

    /**
     * Lance le traitement de duplication
     *
     * @param infoBean
     *            les données saisie
     * @throws Exception
     *             ...
     */
    private void validerDuplication(final InfoBean infoBean, InfosSiteImpl futurInfoSite, String userCode) throws Exception {
        final String oldCodeSite = (String) infoBean.get(SaisieInfosSite.INFOBEAN_CODE_ANCIEN_SITE);
        final String newCodeSite = (String) infoBean.get(SaisieInfosSite.INFOBEAN_CODE);
        final DuplicateSiteUasTaskParameters parameters = new DuplicateSiteUasTaskParameters();
        parameters.setSourceSiteCode(oldCodeSite);
        parameters.setTargetSiteCode(newCodeSite);
        parameters.setInfoBean(infoBean.getValues());
        parameters.setNewInfoSite(futurInfoSite);
        parameters.setUserCode(userCode);
        final DuplicateSiteUasTask copyTask = new DuplicateSiteUasTask();
        copyTask.setParameters(parameters);
        final TaskSchedulerManager taskSchedulerManager = (TaskSchedulerManager) ApplicationContextManager.getCoreContextBean("taskSchedulerManager");
        taskSchedulerManager.execute(copyTask);
    }
}