/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.kosmos.taskscheduler.bean.Lock;
import com.kosmos.taskscheduler.manager.TaskSchedulerManager;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.task.ExportSiteUasTask;
import com.kosmos.uas.task.ExportSiteUasTaskParameters;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.task.UasTaskScheduler;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Action d'export d'un site.
 */
class Export extends AdvancedSaisieInfoSiteAction {

    @Override
    public String getCodeActionPermission() {
        return Autorizer.ACTION_PERMISSION_EXPORTER;
    }

    @Override
    public void doAdvancedAction(final ActionDto actionDto) throws Exception {
        final InfosSiteImpl infosSite = recuperationSite(actionDto.getInfoBean(), actionDto.getAutorisations());
        final Lock currentLock = lockManager.getLockStatus(UasTaskScheduler.LOCK_KEY);
        if (currentLock!=null && !currentLock.isIdling()) {
            actionDto.getInfoBean().addMessageAlerte(String.format(UASUtil.getMessage("ERREUR.EXPORT.TASK_IN_PROGRESS"), infosSite.getAlias()));
            return;
        }
        validerExport(actionDto.getInfoBean());
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.VALIDER_EXPORTER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.FIN.name();
    }

    /**
     * Lance le traitement d'export de site à partir de l'infobean (contenant le site & les modules sélectionnés).
     * @param infoBean les données saisie
     * @throws Exception
     */
    private void validerExport(final InfoBean infoBean) throws Exception {
        final String codeSite = (String) infoBean.get(SaisieInfosSite.INFOBEAN_CODE_ANCIEN_SITE);
        final ExportSiteUasTaskParameters parameters = new ExportSiteUasTaskParameters();
        parameters.setCodeSite(codeSite);
        parameters.setInfoBean(infoBean.getValues());
        final ExportSiteUasTask exportTask = new ExportSiteUasTask();
        exportTask.setParameters(parameters);
        final TaskSchedulerManager taskSchedulerManager = (TaskSchedulerManager) ApplicationContextManager.getCoreContextBean("taskSchedulerManager");
        taskSchedulerManager.execute(exportTask);
    }
}
