/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.TypeMessage;
import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.kosmos.controllers.SGController;
import com.kosmos.taskscheduler.bean.Lock;
import com.kosmos.taskscheduler.manager.TaskSchedulerManager;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.task.ImportSiteUasTask;
import com.kosmos.uas.task.ImportSiteUasTaskParameters;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.task.UasTaskScheduler;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;

/**
 * Action d'import d'un site.
 */
class Import extends AdvancedSaisieInfoSiteAction {

    private String ecranLogique;

    @Override
    public String getCodeActionPermission() {
        return Autorizer.ACTION_PERMISSION_IMPORTER;
    }

    @Override
    public void doAdvancedAction(final ActionDto actionDto) throws Exception {
        final String fileName = actionDto.getInfoBean().getString(SaisieInfosSite.INFOBEAN_IMPORT_ZIP_FILE);
        actionDto.getInfoBean().removeMessage(TypeMessage.ALERTE.getCode());
        final Lock currentLock = lockManager.getLockStatus(UasTaskScheduler.LOCK_KEY);
        if (currentLock != null && !currentLock.isIdling()) {
            actionDto.getInfoBean().addMessageAlerte(String.format(UASUtil.getMessage("ERREUR.IMPORT.TASK_IN_PROGRESS"), fileName));
            final String urlListImport = new StringBuilder(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY))
                .append("?PROC=").append("USINE_SITE")
                .append("&ACTION=").append(SaisieSiteActionUtilisateur.IMPORTER.name()).toString();
            actionDto.getInfoBean().setEcranRedirection(urlListImport);
            ecranLogique = EcranLogique.REDIRECT.name();
            return;
        }
        if (StringUtils.isBlank(fileName)) {
            throw new ErreurActionNonAutorisee(MessageHelper.getCoreMessage("BO_USINESITE_ERREUR_AUCUN_FICHIER_ZIP"));
        }
        validerImport(actionDto.getInfoBean());
        ecranLogique = EcranLogique.FIN.name();
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.VALIDER_IMPORTER.name();
    }

    @Override
    public String getEcranLogique() {
        return ecranLogique;
    }

    /**
     * Lance le traitement d'import.
     * @param infoBean les données saisie
     */
    private void validerImport(final InfoBean infoBean) {
        final ImportSiteUasTaskParameters parameters = new ImportSiteUasTaskParameters();
        parameters.setFileName((String) infoBean.get(SaisieInfosSite.INFOBEAN_IMPORT_ZIP_FILE));
        final ImportSiteUasTask importTask = new ImportSiteUasTask();
        importTask.setParameters(parameters);
        final TaskSchedulerManager taskSchedulerManager = (TaskSchedulerManager) ApplicationContextManager.getCoreContextBean("taskSchedulerManager");
        taskSchedulerManager.execute(importTask);
    }
}