/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.service.impl.ServiceManager;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.utils.UASUtil;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.services.ServiceRubrique;

/**
 * Préparation de l'écran de suppression d'un site.
 */
class PrepareDelete extends SaisieInfoSiteAction {

    @Override
    public void doAction(final ActionDto actionDto) throws Exception {
        final String code = getCodeDepuisInfoBean(actionDto.getInfoBean());
        final InfosSiteImpl infosSiteASupprimer = (InfosSiteImpl) serviceInfosSite.getInfosSite(code);
        if (!possedePermissionGestionPourSite(infosSiteASupprimer, actionDto.getAutorisations())) {
            setDonneesPourSaisieDansInfoBean(new InfosSiteImpl(), null, actionDto.getInfoBean());
        } else {
            final String codeRubrique = infosSiteASupprimer.getCodeRubrique();
            String chemin = StringUtils.EMPTY;
            if (StringUtils.isNotEmpty(codeRubrique)) {
                final List<String> cheminRubrique = new ArrayList<>();
                final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
                RubriqueBean rubriqueMereSite = serviceRubrique.getRubriqueByCode(codeRubrique);
                while (rubriqueMereSite != null) {
                    cheminRubrique.add(rubriqueMereSite.getIntitule());
                    rubriqueMereSite = serviceRubrique.getRubriqueByCode(rubriqueMereSite.getCodeRubriqueMere());
                }
                chemin = StringUtils.join(cheminRubrique, " > ");
            }
            setDonneesPourSaisieDansInfoBean(infosSiteASupprimer, getTemplateDepuisInfosBean(actionDto.getInfoBean()), actionDto.getInfoBean());
            setListeObjetsDansInfoBean(actionDto.getInfoBean());
            actionDto.getInfoBean().setTitreEcran(UASUtil.getMessage("USINESITE_SUPPRESSION") + " " + infosSiteASupprimer.getIntitule());
            actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_SUPPRESSION_RUBRIQUE, chemin);
        }
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.SUPPRIMER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.SUPPRESSION.name();
    }
}
