/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import com.jsbsoft.jtf.exception.ErreurActionNonAutorisee;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.utils.PluginSiteHelper;
import com.kosmos.uas.utils.UASUtil;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Préparation de l'écran d'export de site.
 */
class PrepareExport extends AdvancedSaisieInfoSiteAction {

    @Override
    public String getCodeActionPermission() {
        return Autorizer.ACTION_PERMISSION_EXPORTER;
    }

    @Override
    public void doAdvancedAction(final ActionDto actionDto) throws Exception {
        final String code = getCodeDepuisInfoBean(actionDto.getInfoBean());
        final InfosSiteImpl infosSiteAExporter = (InfosSiteImpl) serviceInfosSite.getInfosSite(code);
        if (!possedePermissionGestionPourSite(infosSiteAExporter, actionDto.getAutorisations())) {
            throw new ErreurActionNonAutorisee();
        }
        setListeObjetsDansInfoBean(actionDto.getInfoBean());
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_CODE_ANCIEN_SITE, getCodeDepuisInfoBean(actionDto.getInfoBean()));
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_PLUGIN_SITE_DISPO, PluginSiteHelper.getAllPluginsSite());
        actionDto.getInfoBean().setTitreEcran(UASUtil.getMessage("USINESITE_EXPORT") + " " + infosSiteAExporter.getIntitule());
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.EXPORTER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.EXPORT.name();
    }
}
