/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.reference.ExtensionFileFilter;
import com.kportal.core.webapp.WebAppUtil;

/**
 * Préparation de l'écran d'import de site.
 */
class PrepareImport extends AdvancedSaisieInfoSiteAction {

    @Override
    public String getCodeActionPermission() {
        return Autorizer.ACTION_PERMISSION_IMPORTER;
    }

    @Override
    public void doAdvancedAction(final ActionDto actionDto) throws Exception {
        Collection<File> zipFiles = new ArrayList<>();
        final File importDirectory = new File(String.format("%s%s", WebAppUtil.getSauvegardePath(), WebAppUtil.DEFAULT_FICHIERS_UAS));
        if (importDirectory.exists()) {
            zipFiles = Arrays.asList(importDirectory.listFiles(new ExtensionFileFilter(ExtensionFileFilter.ZIP)));
        }
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_IMPORT_ZIP_LIST, zipFiles);
        actionDto.getInfoBean().setTitreEcran(UASUtil.getMessage("USINESITE_IMPORT"));
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.IMPORTER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.IMPORT.name();
    }
}
