/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.taskscheduler.bean.AbstractTaskParameters;
import com.kosmos.taskscheduler.bean.Lock;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.EtatInterfaceSaisie;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.task.UasTaskParameters;
import com.kosmos.uas.utils.SaisieInfosUASUtil;
import com.kosmos.usinesite.task.UasTaskScheduler;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.univ.multisites.InfosSite;

/**
 * Préparation de l'écran de modification de site.
 */
class PrepareModification extends SaisieInfoSiteAction {

    @Override
    public void doAction(final ActionDto actionDto) throws Exception {
        final String code = getCodeDepuisInfoBean(actionDto.getInfoBean());
        final InfosSite infosSite = serviceInfosSite.getInfosSite(code);
        if (!possedePermissionGestionPourSite(infosSite, actionDto.getAutorisations()) && !possedePermissionModificationPourSite(infosSite, actionDto.getAutorisations())) {
            setDonneesPourSaisieDansInfoBean(infosSite, null, actionDto.getInfoBean());
        } else {
            final TemplateSite template = serviceTemplateSite.getTemplateSiteParCodeSite(code);
            setDonneesPourSaisieDansInfoBean(infosSite, template, actionDto.getInfoBean());
        }
        actionDto.getInfoBean().set("particularSiteLock", SaisieInfosUASUtil.isLockAboutSite(code));
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_ETAT_INTERFACE_SAISIE, EtatInterfaceSaisie.MODIFICATION);
        actionDto.getInfoBean().setTitreEcran(infosSite.getIntitule());
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.MODIFIER.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.SAISIE.name();
    }
}
