/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections4.MapUtils;

/**
 * Gestionnaire des actions.
 * Contruit la liste des actions disponibles et fournis l'action voulue.
 */
public class SaisieInfoSiteActionManager {

    /**
     * Map des actions.
     */
    protected Map<String, SaisieInfoSiteAction> actions;

    /**
     * Id du bean en base de données.
     */
    public static final String ID_BEAN = "saisieInfoSiteActionManager";

    /**
     * Fourni l'action voulue ou null si inexistante.
     * @param action le nom de l'action recherchée
     * @return l'action voulue
     */
    public SaisieInfoSiteAction getSaisieInfoSiteAction(final String action) {
        if (MapUtils.isEmpty(actions)) {
            initActions();
        }
        return actions.get(action);
    }

    /**
     * Initialise les actions.
     */
    protected void initActions() {
        Map<String, SaisieInfoSiteAction> tempMap = new HashMap<>();
        addAction(tempMap, new Add());
        addAction(tempMap, new Delete());
        addAction(tempMap, new Duplicate());
        addAction(tempMap, new Export());
        addAction(tempMap, new Import());
        addAction(tempMap, new Modify());
        addAction(tempMap, new PrepareAdd());
        addAction(tempMap, new PrepareDelete());
        addAction(tempMap, new PrepareDuplicate());
        addAction(tempMap, new PrepareExport());
        addAction(tempMap, new PrepareImport());
        addAction(tempMap, new PrepareModification());
        addAction(tempMap, new SiteList());
        actions = tempMap;
    }

    /**
     * Ajoute une action dans une map temporaire.
     * @param tempMap la map temporaire
     * @param saisieInfoSiteAction l'action à ajouter
     */
    protected void addAction(final Map<String, SaisieInfoSiteAction> tempMap, final SaisieInfoSiteAction saisieInfoSiteAction) {
        tempMap.put(saisieInfoSiteAction.getAction(), saisieInfoSiteAction);
    }
}
