/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.processus.actions;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.composant.UsineASiteComposant;
import com.kosmos.uas.processus.EcranLogique;
import com.kosmos.uas.processus.SaisieInfosSite;
import com.kosmos.uas.processus.SaisieSiteActionUtilisateur;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.univ.multisites.InfosSite;

/**
 * Action de listage des sites, l'accueil.
 */
class SiteList extends SaisieInfoSiteAction {

    private static final Logger LOG = LoggerFactory.getLogger(SiteList.class);

    @Override
    public void doAction(final ActionDto actionDto) throws Exception {
        Autorizer autorizer = ApplicationContextManager.getBean(UASUtil.ID_EXTENSION, Autorizer.ID_BEAN, Autorizer.class);
        final Collection<InfosSite> listeInfosSites =  new LinkedList<>(serviceInfosSite.getListeTousInfosSites());
        if (!actionDto.getAutorisations().isWebMaster()) {
            CollectionUtils.filter(listeInfosSites, siteCourant -> possedePermissionGestionPourSite(siteCourant, actionDto.getAutorisations()) || possedePermissionModificationPourSite(siteCourant, actionDto.getAutorisations()));
        }
        final Map<String, TemplateSite> templatesParCodesTemplates = new HashMap<>();
        final Collection<String> codesTemplate = CollectionUtils.collect(listeInfosSites, InfosSite::getCodeTemplate);
        for (final String codeTemplate : codesTemplate) {
            try {
                templatesParCodesTemplates.put(codeTemplate, serviceTemplateSite.getTemplateSiteParCode(codeTemplate));
            } catch (final Exception e) {
                LOG.debug("Erreur durant la récupération de template", e);
                actionDto.getInfoBean().addMessageAlerte(e.getMessage());
            }
        }
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_CAN_DUPLICATE, autorizer.canAccess(actionDto.getAutorisations(), Autorizer.ACTION_PERMISSION_DUPLIQUER));
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_CAN_EXPORT, autorizer.canAccess(actionDto.getAutorisations(), Autorizer.ACTION_PERMISSION_EXPORTER));
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_IS_GESTIONNAIRE_SITE, actionDto.getAutorisations().possedePermission(UsineASiteComposant.getPermissionGestion()) || actionDto.getAutorisations().isWebMaster());
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_VALEURS_LISTE_TEMPLATES_INFOSSITE, templatesParCodesTemplates);
        actionDto.getInfoBean().set(SaisieInfosSite.INFOBEAN_LISTE_INFOSSITES, listeInfosSites.stream()
            .sorted((site1, site2) -> site1.getIntitule().compareToIgnoreCase(site2.getIntitule()))
            .collect(Collectors.toList()));
    }

    @Override
    public String getAction() {
        return SaisieSiteActionUtilisateur.ACCUEIL.name();
    }

    @Override
    public String getEcranLogique() {
        return EcranLogique.LISTE.name();
    }
}
