/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.service;

import java.io.File;
import java.util.Collection;
import java.util.Date;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.batch.service.DeleteObsoleteDataService;

public class DeleteObsoleteSiteCopyData implements DeleteObsoleteDataService {

    private static final Logger LOG = LoggerFactory.getLogger(DeleteObsoleteSiteCopyData.class);

    @Override
    public void execute() {
        LOG.info("Exécution du script de nettoyage du répertoire de sauvegarde des sites");
        int days = PropertyHelper.getPropertyAsInt("uas", "uas.purge.delay.days", -14);
        if (days > 0) {
            days = -days;
        }
        Date date = DateUtils.addDays(new Date(), days);
        File dossierUAS = new File(WebAppUtil.getSauvegardePath() + WebAppUtil.DEFAULT_FICHIERS_UAS);
        long count = 0;
        if (dossierUAS.exists() && dossierUAS.isDirectory()) {
            Collection<File> files = FileUtils.listFiles(dossierUAS, null, true);
            count = files.stream().filter(file -> FileUtils.isFileOlder(file, date)).map(FileUtils::deleteQuietly).filter(r -> r == true).count();
        }
        LOG.info("  > Nombre de fichiers supprimés dans le répertoire {} : {}", dossierUAS.getAbsolutePath(), count);
    }
}
