/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.task;

import java.util.Collection;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.service.impl.ServiceManager;
import com.kosmos.usinesite.service.ServiceDeletion;
import com.kosmos.uas.utils.PluginSiteHelper;
import com.kosmos.uas.utils.UASUtil;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kosmos.usinesite.task.AbstractUasTask;
import com.kportal.extension.module.IModule;
import com.kportal.extension.module.utils.ServiceContenuModuleHelper;
import com.univ.multisites.InfosSite;
import com.univ.multisites.service.ServiceInfosSite;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.services.ServiceRubrique;

/**
 * Traitement avec verrou ({@see com.kosmos.taskscheduler.bean.LockableTask} spécialisé dans la suppression d'un site.
 */
public class DeleteSiteUasTask extends AbstractUasTask<DeleteSiteUasTaskParameters> {

    private static final Logger LOG = LoggerFactory.getLogger(DeleteSiteUasTask.class);

    @Override
    protected void execute() {
        final String codeSite = parameters.getCodeSite();
        LOG.info("Lancement de la suppression du site \"{}\"", codeSite);
        final String deleteContent = parameters.getDeleteContent();
        final ServiceDeletion serviceDeletion = ApplicationContextManager.getBean(UASUtil.ID_EXTENSION, "defaultServiceDeletion", ServiceDeletion.class);
        final ServiceInfosSite serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
        final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
        try {
            final InfosSite siteToDelete = serviceInfosSite.getInfosSite(codeSite);
            final RubriqueBean rubrique = serviceRubrique.getRubriqueByCode(siteToDelete.getCodeRubrique());
            if (rubrique != null && StringUtils.isNoneBlank(deleteContent, siteToDelete.getCodeRubrique()) && UASUtil.isAdvancedFeatureEnabled()) {
                final Collection<IModule> modulesToDelete = ServiceContenuModuleHelper.getModulesExportables();
                serviceDeletion.deleteContent(siteToDelete.getCodeRubrique(), modulesToDelete);
            }
            serviceInfosSite.supprimerTousFichiers(siteToDelete);
            serviceInfosSite.supprimer(siteToDelete.getAlias());
        } catch (final Exception e) {
            LOG.error("une erreur s'est produite lors de la suppression du site", e);
        }
    }
}