/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.task;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.service.impl.ServiceManager;
import com.kosmos.uas.execution.DuplicationSiteHandler;
import com.kosmos.uas.execution.dto.SiteDuplicationDto;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kosmos.usinesite.task.AbstractUasTask;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;
import com.univ.multisites.service.ServiceInfosSite;

/**
 * Traitement avec verrou ({@see com.kosmos.taskscheduler.bean.LockableTask} spécialisé dans la duplication d'un site.
 */
public class DuplicateSiteUasTask extends AbstractUasTask<DuplicateSiteUasTaskParameters> {

    private static final Logger LOG = LoggerFactory.getLogger(DuplicateSiteUasTask.class);

    @Override
    protected void execute() {
        LOG.info("Lancement de la duplication du site \"{}\", nouveau code du site : \"{}\"", parameters.getSourceSiteCode(), parameters.getTargetSiteCode());
        final ServiceInfosSite serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
        try {
            final InfosSite oldSite = serviceInfosSite.getInfosSite(parameters.getSourceSiteCode());
            final InfosSiteImpl newSite = parameters.getNewInfoSite();
            final DuplicationSiteHandler handler = new DuplicationSiteHandler(parameters);
            final SiteDuplicationDto inputDto = new SiteDuplicationDto();
            inputDto.setOldSite(oldSite);
            inputDto.setNewSite(newSite);
            inputDto.setInfoBean(parameters.getInfoBean());
            handler.setInput(inputDto);
            handler.execute();
        } catch (final Exception e) {
            LOG.error("une erreur s'est produite lors de la duplication du site", e);
        }
    }
}