/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.task;

import java.io.IOException;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.service.impl.ServiceManager;
import com.kosmos.uas.execution.ExportSiteHandler;
import com.kosmos.uas.execution.dto.SiteDuplicationDto;
import com.kosmos.uas.utils.PluginSiteHelper;
import com.kosmos.usinesite.service.impl.ServiceInfosSiteProcessus;
import com.kosmos.usinesite.task.AbstractUasTask;
import com.univ.multisites.InfosSite;
import com.univ.multisites.service.ServiceInfosSite;

/**
 * Traitement avec verrou ({@see com.kosmos.taskscheduler.bean.LockableTask} spécialisé dans l'export d'un site.
 */
public class ExportSiteUasTask extends AbstractUasTask<ExportSiteUasTaskParameters> {

    private static final Logger LOG = LoggerFactory.getLogger(ExportSiteUasTask.class);

    @Override
    protected void execute() {
        LOG.info("Lancement de l'export du site \"{}\"", parameters.getCodeSite());
        final ServiceInfosSite serviceInfosSite = ServiceManager.getService(ServiceInfosSiteProcessus.class);
        String savePath = null;
        try {
            final InfosSite oldSite = serviceInfosSite.getInfosSite(parameters.getCodeSite());
            final ExportSiteHandler handler = new ExportSiteHandler();
            final SiteDuplicationDto input = new SiteDuplicationDto();
            input.setOldSite(oldSite);
            input.setNewSite(oldSite);
            input.setInfoBean(parameters.getInfoBean());
            handler.setInput(input);
            handler.execute();
            if (handler.getOutput() != null && StringUtils.isNotEmpty(handler.getOutput().getFolderPath())) {
                savePath = handler.getOutput().getFolderPath();
                PluginSiteHelper.duplicateProperties(oldSite, savePath);
                PluginSiteHelper.duplicateTemplateDatas(oldSite, savePath);
                PluginSiteHelper.compressExportFolder(oldSite, savePath);
            }
        } catch (final Exception e) {
            LOG.error("une erreur s'est produite lors de la duplication du site", e);
        } finally {
            if (StringUtils.isNotBlank(savePath)) {
                try {
                    PluginSiteHelper.cleanUpDirectory(savePath, false);
                } catch (final IOException e) {
                    LOG.error("une erreur est survenue lors du nettoyage du répertoire de traitement", e);
                }
            }
        }
    }
}