/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.utils;

import java.util.ArrayList;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.InfoBean;
import com.kosmos.taskscheduler.bean.AbstractTaskParameters;
import com.kosmos.taskscheduler.bean.Lock;
import com.kosmos.taskscheduler.manager.LockManager;
import com.kosmos.uas.task.UasTaskParameters;
import com.kosmos.usinesite.exception.ErreursSaisieInfosSite;
import com.kosmos.usinesite.task.UasTaskScheduler;
import com.kosmos.usinesite.template.bean.TemplateSite;
import com.univ.multisites.InfosSite;
import com.univ.multisites.bean.impl.InfosSiteImpl;

/**
 * Classe utilitaire pour les propriétés suplémentaires nécessaires au fonctionnement de la duplication de site.
 *
 * @author olivier.camon
 *
 */
public class SaisieInfosUASUtil {

    private static final String CLE_ID_BEAN_DUPLICATION_SITE = "idBeanDuplicationSite";

    public static void alimenteDonneeDuplicationDepuisInfoBean(final InfosSiteImpl infosSite, final TemplateSite template, final InfoBean infoBean) throws ErreursSaisieInfosSite {
        final ArrayList<String> fluxMessagesErreurs = new ArrayList<String>();
        SaisieInfosSiteInfoBeanHelper.setCodeInfosSite(infosSite, infoBean, fluxMessagesErreurs);
        SaisieInfosSiteInfoBeanHelper.setIntituleInfosSite(infosSite, infoBean, fluxMessagesErreurs);
        SaisieInfosSiteInfoBeanHelper.setHostnameHttpInfosSite(infosSite, infoBean, fluxMessagesErreurs);
        infosSite.setCodeRubrique(StringUtils.EMPTY);
        infosSite.setActif(Boolean.FALSE);
        infosSite.setSitePrincipal(Boolean.FALSE);
        SaisieInfosUASUtil.setIdBeanDuplicationInfosSite(infosSite, infoBean.getString("PLUGIN_ID"));
        if (!fluxMessagesErreurs.isEmpty()) {
            throw new ErreursSaisieInfosSite(fluxMessagesErreurs, infosSite, template);
        }
    }

    /**
     * Rècupère la valeur de l'id du bean de duplication.
     *
     * @param site
     *            le site à analyser
     * @return l'id du bean ou null si non renseigné.
     */
    public static String getIdBeanDuplication(final InfosSite site) {
        return site.getProprieteComplementaireString(CLE_ID_BEAN_DUPLICATION_SITE);
    }

    public static void setIdBeanDuplicationInfosSite(final InfosSiteImpl site, final String idBean) {
        site.putProperty(CLE_ID_BEAN_DUPLICATION_SITE, idBean);
    }

    public static String getClassStatusInfosSite(final InfosSite site) {
        String classCSS = StringUtils.EMPTY;
        if (isSiteInProgress(site)) {
            classCSS = "lock";
        }
        return classCSS;
    }

    /**
     * Détermine s'il est possible de faire un export ou une duplication du site.
     * @param site le site en question
     * @param isGestionnaire true si l'utilisateur courant est gestionnaire
     * @return true s'il est possible d'exporter ou dupliquer le site
     */
    public static boolean isExportOrDuplicationAllowed(final InfosSite site, final boolean isGestionnaire) {
        return isGestionnaire && site.isActif() && !isSiteInProgress(site);
    }

    /**
     * Détermine si un processus est en cours sur le site fournit.
     * @param site le site que l'on cherche à vérifier
     * @return true si le site a un job en cours de traitement
     */
    public static boolean isSiteInProgress(final InfosSite site) {
        return isLockAboutSite(site.getAlias());
    }

    /**
     * Check if current Lock concerne a particular site.
     * @param codeSite a {@link String}
     * @return true if the current Lock concern the site code
     */
    public static boolean isLockAboutSite(final String codeSite) {
        LockManager lockManager = ApplicationContextManager.getCoreContextBean(LockManager.ID_BEAN, LockManager.class);
        boolean result = false;
        final Lock currentLock = lockManager.getLockStatus(UasTaskScheduler.LOCK_KEY);
        if (currentLock != null && !currentLock.isIdling()) {
            AbstractTaskParameters parameters = currentLock.getParameters();
            if (parameters instanceof UasTaskParameters) {
                result = StringUtils.equals(codeSite, ((UasTaskParameters) parameters).getCodeSite());
            }
        }
        return result;
    }
}
