/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.utils;

import java.io.File;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.io.filefilter.IOFileFilter;

import com.kosmos.usinesite.utils.TransformationUtils;
import com.kosmos.usinesite.utils.UASOmHelper;
import com.kosmos.usinesite.utils.UASServicesHelper;
import com.univ.multisites.InfosSite;

public class UASDirectoriesFilter implements IOFileFilter {

    private final Collection<String> uasDirectories = new ArrayList<>();

    public UASDirectoriesFilter(InfosSite site) {
        uasDirectories.add(new File(TransformationUtils.getPathDossierDuplication(site)).getName());
        uasDirectories.add(UASServicesHelper.SUB_FOLDER_NAME);
        uasDirectories.add(new File(TransformationUtils.getPathDossierExport(site)).getName());
    }

    @Override
    public boolean accept(File pathname) {
        return pathname.isDirectory() && !uasDirectories.contains(pathname.getName());
    }

    @Override
    public boolean accept(File dir, String name) {
        return dir.isDirectory() && !uasDirectories.contains(name);
    }
}
