/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: uas
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.uas.utils;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.kosmos.uas.advancedFeatures.Autorizer;
import com.kosmos.uas.composant.UsineASiteComposant;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.config.PropertyHelper;
import com.kportal.extension.module.IModule;
import com.kportal.extension.module.ModuleHelper;
import com.kportal.extension.module.service.ServiceContenuModule.TypeContenu;

public final class UASUtil {

    public static final String ID_EXTENSION = "uas";

    public static IModule getModule() {
        return ModuleHelper.getModule(UASUtil.ID_EXTENSION, UsineASiteComposant.ID_BEAN);
    }

    /**
     * Recherche une valeur dans les messages
     *
     * @param cle la clé à chercherdans les messages
     * @return la valeur du libellé
     */
    public static String getMessage(final String cle) {
        return MessageHelper.getMessage(ID_EXTENSION, cle);
    }

    /**
     * Recherche une valeur dans les properties de l'extension puis de l'application
     *
     * @param cle
     *            la clé de la property à chercher
     * @return la valeur de la property ou null si non trouvé
     */
    public static String getProperty(final String cle) {
        return PropertyHelper.getProperty(ID_EXTENSION, cle);
    }

    public static String getMessageForType(final TypeContenu type) {
        return MessageHelper.getMessage(ID_EXTENSION, String.format("USINESITE_TYPE_CONTENU_%s", type.toString()));
    }

    /**
     * @see Autorizer#isAdvancedFeatureEnabled()
     */
    public static boolean isAdvancedFeatureEnabled() {
        Autorizer autorizer = ApplicationContextManager.getBean(UASUtil.ID_EXTENSION, Autorizer.ID_BEAN, Autorizer.class);
        return autorizer!= null && autorizer.isAdvancedFeatureEnabled();
    }
}
