/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.bean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.cms.objetspartages.annotation.Aggregation;
import com.kportal.cms.objetspartages.annotation.Autocompletion;
import com.kportal.cms.objetspartages.annotation.Fiche;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.Label;
import com.kportal.cms.objetspartages.annotation.Metier;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.ksup.objetspartages.util.FicheBeanUtils;
import com.univ.objetspartages.bean.AbstractRestrictionFicheBean;

/**
 * Bean associé à l'objet Formation.
 */
public class FormationBean extends AbstractRestrictionFicheBean implements Serializable {

    /**
     * The Constant serialVersionUID.
     */
    private static final long serialVersionUID = 3413287769398018365L;

    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "structuremodele")
    @Aggregation(name = "structures")
    private String codeRattachementAutres = null;

    private String codeSiScolarite = null;

    @Autocompletion(name = "titre_completion")
    private String intituleCourt = null;

    @Autocompletion(name = "titre_completion")
    private String intituleLong = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2001")
    private String catalogue = null;

    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "formation")
    private String codeFormationMere = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2002")
    private String natureFormation = null;

    private String formationDiplomante = null;

    private String formationLmd = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "20")
    private String typeDiplome = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2003")
    private String grade = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2004")
    private String domaineLmd = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "55")
    private String discipline = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2005")
    private String mention = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2006")
    private String parcours = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2007")
    private String niveauRecrutement = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2008")
    private String anneePostbac = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2009")
    private String niveauSortie = null;

    private String formationInitiale = null;

    private String repriseEtudes = null;

    private String tempsPlein = null;

    private String compatibleActivitePro = null;

    private String tempsAmenage = null;

    private String alternance = null;

    private String horsContratAlternance = null;

    private String contratApprentissage = null;

    private String contratPro = null;

    private String presentiel = null;

    private String partiellementDistance = null;

    private String totalementDistance = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2010")
    private String lieux = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2011")
    private String languesEnseignement = null;

    private String selectif = null;

    private String capaciteAccueilLimitee = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2012")
    private String dureeListe = null;

    private String autreDuree = null;

    private String stages = null;

    private String stagesOptionnel = null;

    private String stagesOptionnelMultiple = null;

    private String stagesOptionnelEtranger = null;

    private String stagesObligatoire = null;

    private String stagesObligatoireMultiple = null;

    private String stagesObligatoireEtranger = null;

    private String sejoursEtranger = null;

    private String sejoursEtrangerOptionnel = null;

    private String sejoursEtrangerOptionnelMultiples = null;

    private String sejoursEtrangerObligatoire = null;

    private String sejoursEtrangerObligatoireMultiples = null;

    private String projetsTutores = null;

    private String projetsTutoresOptionnel = null;

    private String projetsTutoresOptionnelMultiples = null;

    private String projetsTutoresObligatoire = null;

    private String projetsTutoresObligatoireMultiples = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "65")
    private String secteursActivites = null;

    @Metier
    private String metiers = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2014")
    private String principalesMatieres = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2015")
    private String centresInteret = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2016")
    private String serieBac = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2017")
    private String domaineErasmus = null;

    private String accroche = null;

    private String resume = null;

    private String description = null;

    private String specificites = null;

    private String lieuxComplement = null;

    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "annuaireksup")
    private String responsablesFiche = null;

    private String responsablesComplement = null;

    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "annuaireanciensetudiants")
    private String temoignages = null;

    private String laboratoiresPartenaires = null;

    private String etablissementsPartenaires = null;

    private String entreprisesPartenaires = null;

    private String conventionEtablissements = null;

    private String formationsRequises = null;

    private List<FormationBean> formationsLiees = new ArrayList<>();

    private String publicCible = null;

    private String modalitesCandidatures = null;

    private String modalitesCandidaturesSpecifiques = null;

    private String conditionsAdmission = null;

    private String programme = null;

    private List<ParcoursBean> parcoursFiche = new ArrayList<>();

    private String contenuFormation = null;

    private String rythmePremiereAnnee = null;

    private String equipePedagogique = null;

    private String tempsPleinDescription = null;

    private String compatibleProfessionnelleDescription = null;

    private String tempsAmenageDescription = null;

    private String alternanceDescription = null;

    private String presentielDescription = null;

    private String partiellementDistanceDescription = null;

    private String totalementDistanceDescription = null;

    private String stagesDescription = null;

    private String sejoursEtrangerDescription = null;

    private String projetsTutoresComplement = null;

    private String modalitesEvaluation = null;

    private String ficheRncp = null;

    private String competencesAttestees = null;

    private String poursuiteEtudes = null;

    private List<ParcoursBean> parcoursPoursuite = new ArrayList<>();

    private List<FormationBean> formationPoursuite = new ArrayList<>();

    private String debouches = null;

    private String insertionProfessionnelle = null;

    private String tarifComplement = null;

    private String modalitesInscription = null;

    private String plaquettesCrm = null;

    private String contactEmail = null;

    private String contactTelephone = null;

    private String contactCrm = null;

    private String urlExterneFormation = null;

    private String renseignements = null;

    private String urlInscription = null;

    private String afficherProgramme = null;

    private String activerConstruireParcours = null;

    private String formationContinue = null;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2018")
    private String typeFormationContinue = null;

    /**
     * Instantiates a new bean.
     */
    public FormationBean() {
        super();
    }

    /**
     * Getter and Setter
     */
    public void setCodeRattachementAutres(final String codeRattachementAutres) {
        this.codeRattachementAutres = codeRattachementAutres;
    }

    public String getCodeRattachementAutres() {
        return this.codeRattachementAutres;
    }

    public void setCodeSiScolarite(final String codeSiScolarite) {
        this.codeSiScolarite = codeSiScolarite;
    }

    public String getCodeSiScolarite() {
        return this.codeSiScolarite;
    }

    public void setIntituleCourt(final String intituleCourt) {
        this.intituleCourt = intituleCourt;
    }

    public String getIntituleCourt() {
        return this.intituleCourt;
    }

    public void setIntituleLong(final String intituleLong) {
        this.intituleLong = intituleLong;
    }

    public String getIntituleLong() {
        return this.intituleLong;
    }

    public void setCatalogue(final String catalogue) {
        this.catalogue = catalogue;
    }

    public String getCatalogue() {
        return this.catalogue;
    }

    public void setCodeFormationMere(final String codeFormationMere) {
        this.codeFormationMere = codeFormationMere;
    }

    public String getCodeFormationMere() {
        return this.codeFormationMere;
    }

    public void setNatureFormation(final String natureFormation) {
        this.natureFormation = natureFormation;
    }

    public String getNatureFormation() {
        return this.natureFormation;
    }

    public void setFormationDiplomante(final String formationDiplomante) {
        this.formationDiplomante = formationDiplomante;
    }

    public String getFormationDiplomante() {
        return this.formationDiplomante;
    }

    public void setFormationLmd(final String formationLmd) {
        this.formationLmd = formationLmd;
    }

    public String getFormationLmd() {
        return this.formationLmd;
    }

    public void setTypeDiplome(final String typeDiplome) {
        this.typeDiplome = typeDiplome;
    }

    public String getTypeDiplome() {
        return this.typeDiplome;
    }

    public void setGrade(final String grade) {
        this.grade = grade;
    }

    public String getGrade() {
        return this.grade;
    }

    public void setDomaineLmd(final String domaineLmd) {
        this.domaineLmd = domaineLmd;
    }

    public String getDomaineLmd() {
        return this.domaineLmd;
    }

    public void setDiscipline(final String discipline) {
        this.discipline = discipline;
    }

    public String getDiscipline() {
        return this.discipline;
    }

    public void setMention(final String mention) {
        this.mention = mention;
    }

    public String getMention() {
        return this.mention;
    }

    public void setParcours(final String parcours) {
        this.parcours = parcours;
    }

    public String getParcours() {
        return this.parcours;
    }

    public void setNiveauRecrutement(final String niveauRecrutement) {
        this.niveauRecrutement = niveauRecrutement;
    }

    public String getNiveauRecrutement() {
        return this.niveauRecrutement;
    }

    public void setAnneePostbac(final String anneePostbac) {
        this.anneePostbac = anneePostbac;
    }

    public String getAnneePostbac() {
        return this.anneePostbac;
    }

    public void setNiveauSortie(final String niveauSortie) {
        this.niveauSortie = niveauSortie;
    }

    public String getNiveauSortie() {
        return this.niveauSortie;
    }

    public void setFormationInitiale(final String formationInitiale) {
        this.formationInitiale = formationInitiale;
    }

    public String getFormationInitiale() {
        return this.formationInitiale;
    }

    public void setRepriseEtudes(final String repriseEtudes) {
        this.repriseEtudes = repriseEtudes;
    }

    public String getRepriseEtudes() {
        return this.repriseEtudes;
    }

    public void setTempsPlein(final String tempsPlein) {
        this.tempsPlein = tempsPlein;
    }

    public String getTempsPlein() {
        return this.tempsPlein;
    }

    public void setCompatibleActivitePro(final String compatibleActivitePro) {
        this.compatibleActivitePro = compatibleActivitePro;
    }

    public String getCompatibleActivitePro() {
        return this.compatibleActivitePro;
    }

    public void setTempsAmenage(final String tempsAmenage) {
        this.tempsAmenage = tempsAmenage;
    }

    public String getTempsAmenage() {
        return this.tempsAmenage;
    }

    public void setAlternance(final String alternance) {
        this.alternance = alternance;
    }

    public String getAlternance() {
        return this.alternance;
    }

    public void setHorsContratAlternance(final String horsContratAlternance) {
        this.horsContratAlternance = horsContratAlternance;
    }

    public String getHorsContratAlternance() {
        return this.horsContratAlternance;
    }

    public void setContratApprentissage(final String contratApprentissage) {
        this.contratApprentissage = contratApprentissage;
    }

    public String getContratApprentissage() {
        return this.contratApprentissage;
    }

    public void setContratPro(final String contratPro) {
        this.contratPro = contratPro;
    }

    public String getContratPro() {
        return this.contratPro;
    }

    public void setPresentiel(final String presentiel) {
        this.presentiel = presentiel;
    }

    public String getPresentiel() {
        return this.presentiel;
    }

    public void setPartiellementDistance(final String partiellementDistance) {
        this.partiellementDistance = partiellementDistance;
    }

    public String getPartiellementDistance() {
        return this.partiellementDistance;
    }

    public void setTotalementDistance(final String totalementDistance) {
        this.totalementDistance = totalementDistance;
    }

    public String getTotalementDistance() {
        return this.totalementDistance;
    }

    public void setLieux(final String lieux) {
        this.lieux = lieux;
    }

    public String getLieux() {
        return this.lieux;
    }

    public void setLanguesEnseignement(final String languesEnseignement) {
        this.languesEnseignement = languesEnseignement;
    }

    public String getLanguesEnseignement() {
        return this.languesEnseignement;
    }

    public void setSelectif(final String selectif) {
        this.selectif = selectif;
    }

    public String getSelectif() {
        return this.selectif;
    }

    public void setCapaciteAccueilLimitee(final String capaciteAccueilLimitee) {
        this.capaciteAccueilLimitee = capaciteAccueilLimitee;
    }

    public String getCapaciteAccueilLimitee() {
        return this.capaciteAccueilLimitee;
    }

    public void setDureeListe(final String dureeListe) {
        this.dureeListe = dureeListe;
    }

    public String getDureeListe() {
        return this.dureeListe;
    }

    public void setAutreDuree(final String autreDuree) {
        this.autreDuree = autreDuree;
    }

    public String getAutreDuree() {
        return this.autreDuree;
    }

    public void setStages(final String stages) {
        this.stages = stages;
    }

    public String getStages() {
        return this.stages;
    }

    public void setStagesOptionnel(final String stagesOptionnel) {
        this.stagesOptionnel = stagesOptionnel;
    }

    public String getStagesOptionnel() {
        return this.stagesOptionnel;
    }

    public void setStagesOptionnelMultiple(final String stagesOptionnelMultiple) {
        this.stagesOptionnelMultiple = stagesOptionnelMultiple;
    }

    public String getStagesOptionnelMultiple() {
        return this.stagesOptionnelMultiple;
    }

    public void setStagesOptionnelEtranger(final String stagesOptionnelEtranger) {
        this.stagesOptionnelEtranger = stagesOptionnelEtranger;
    }

    public String getStagesOptionnelEtranger() {
        return this.stagesOptionnelEtranger;
    }

    public void setStagesObligatoire(final String stagesObligatoire) {
        this.stagesObligatoire = stagesObligatoire;
    }

    public String getStagesObligatoire() {
        return this.stagesObligatoire;
    }

    public void setStagesObligatoireMultiple(final String stagesObligatoireMultiple) {
        this.stagesObligatoireMultiple = stagesObligatoireMultiple;
    }

    public String getStagesObligatoireMultiple() {
        return this.stagesObligatoireMultiple;
    }

    public void setStagesObligatoireEtranger(final String stagesObligatoireEtranger) {
        this.stagesObligatoireEtranger = stagesObligatoireEtranger;
    }

    public String getStagesObligatoireEtranger() {
        return this.stagesObligatoireEtranger;
    }

    public void setSejoursEtranger(final String sejoursEtranger) {
        this.sejoursEtranger = sejoursEtranger;
    }

    public String getSejoursEtranger() {
        return this.sejoursEtranger;
    }

    public void setSejoursEtrangerOptionnel(final String sejoursEtrangerOptionnel) {
        this.sejoursEtrangerOptionnel = sejoursEtrangerOptionnel;
    }

    public String getSejoursEtrangerOptionnel() {
        return this.sejoursEtrangerOptionnel;
    }

    public void setSejoursEtrangerOptionnelMultiples(final String sejoursEtrangerOptionnelMultiples) {
        this.sejoursEtrangerOptionnelMultiples = sejoursEtrangerOptionnelMultiples;
    }

    public String getSejoursEtrangerOptionnelMultiples() {
        return this.sejoursEtrangerOptionnelMultiples;
    }

    public void setSejoursEtrangerObligatoire(final String sejoursEtrangerObligatoire) {
        this.sejoursEtrangerObligatoire = sejoursEtrangerObligatoire;
    }

    public String getSejoursEtrangerObligatoire() {
        return this.sejoursEtrangerObligatoire;
    }

    public void setSejoursEtrangerObligatoireMultiples(final String sejoursEtrangerObligatoireMultiples) {
        this.sejoursEtrangerObligatoireMultiples = sejoursEtrangerObligatoireMultiples;
    }

    public String getSejoursEtrangerObligatoireMultiples() {
        return this.sejoursEtrangerObligatoireMultiples;
    }

    public void setProjetsTutores(final String projetsTutores) {
        this.projetsTutores = projetsTutores;
    }

    public String getProjetsTutores() {
        return this.projetsTutores;
    }

    public void setProjetsTutoresOptionnel(final String projetsTutoresOptionnel) {
        this.projetsTutoresOptionnel = projetsTutoresOptionnel;
    }

    public String getProjetsTutoresOptionnel() {
        return this.projetsTutoresOptionnel;
    }

    public void setProjetsTutoresOptionnelMultiples(final String projetsTutoresOptionnelMultiples) {
        this.projetsTutoresOptionnelMultiples = projetsTutoresOptionnelMultiples;
    }

    public String getProjetsTutoresOptionnelMultiples() {
        return this.projetsTutoresOptionnelMultiples;
    }

    public void setProjetsTutoresObligatoire(final String projetsTutoresObligatoire) {
        this.projetsTutoresObligatoire = projetsTutoresObligatoire;
    }

    public String getProjetsTutoresObligatoire() {
        return this.projetsTutoresObligatoire;
    }

    public void setProjetsTutoresObligatoireMultiples(final String projetsTutoresObligatoireMultiples) {
        this.projetsTutoresObligatoireMultiples = projetsTutoresObligatoireMultiples;
    }

    public String getProjetsTutoresObligatoireMultiples() {
        return this.projetsTutoresObligatoireMultiples;
    }

    public void setSecteursActivites(final String secteursActivites) {
        this.secteursActivites = secteursActivites;
    }

    public String getSecteursActivites() {
        return this.secteursActivites;
    }

    public void setMetiers(final String metiers) {
        this.metiers = metiers;
    }

    public String getMetiers() {
        return this.metiers;
    }

    public void setPrincipalesMatieres(final String principalesMatieres) {
        this.principalesMatieres = principalesMatieres;
    }

    public String getPrincipalesMatieres() {
        return this.principalesMatieres;
    }

    public void setCentresInteret(final String centresInteret) {
        this.centresInteret = centresInteret;
    }

    public String getCentresInteret() {
        return this.centresInteret;
    }

    public void setSerieBac(final String serieBac) {
        this.serieBac = serieBac;
    }

    public String getSerieBac() {
        return this.serieBac;
    }

    public void setDomaineErasmus(final String domaineErasmus) {
        this.domaineErasmus = domaineErasmus;
    }

    public String getDomaineErasmus() {
        return this.domaineErasmus;
    }

    public void setAccroche(final String accroche) {
        this.accroche = accroche;
    }

    public String getAccroche() {
        return this.accroche;
    }

    @SetterAnnotation(isToolbox = true)
    public void setResume(final String resume) {
        this.resume = resume;
    }

    @GetterAnnotation(isToolbox = true)
    public String getResume() {
        return this.resume;
    }

    @SetterAnnotation(isToolbox = true)
    public void setDescription(final String description) {
        this.description = description;
    }

    @GetterAnnotation(isToolbox = true)
    public String getDescription() {
        return this.description;
    }

    @SetterAnnotation(isToolbox = true)
    public void setSpecificites(final String specificites) {
        this.specificites = specificites;
    }

    @GetterAnnotation(isToolbox = true)
    public String getSpecificites() {
        return this.specificites;
    }

    @SetterAnnotation(isToolbox = true)
    public void setLieuxComplement(final String lieuxComplement) {
        this.lieuxComplement = lieuxComplement;
    }

    @GetterAnnotation(isToolbox = true)
    public String getLieuxComplement() {
        return this.lieuxComplement;
    }

    public void setResponsablesFiche(final String responsablesFiche) {
        this.responsablesFiche = responsablesFiche;
    }

    public String getResponsablesFiche() {
        return this.responsablesFiche;
    }

    @SetterAnnotation(isToolbox = true)
    public void setResponsablesComplement(final String responsablesComplement) {
        this.responsablesComplement = responsablesComplement;
    }

    @GetterAnnotation(isToolbox = true)
    public String getResponsablesComplement() {
        return this.responsablesComplement;
    }

    public void setTemoignages(final String temoignages) {
        this.temoignages = temoignages;
    }

    public String getTemoignages() {
        return this.temoignages;
    }

    @SetterAnnotation(isToolbox = true)
    public void setLaboratoiresPartenaires(final String laboratoiresPartenaires) {
        this.laboratoiresPartenaires = laboratoiresPartenaires;
    }

    @GetterAnnotation(isToolbox = true)
    public String getLaboratoiresPartenaires() {
        return this.laboratoiresPartenaires;
    }

    @SetterAnnotation(isToolbox = true)
    public void setEtablissementsPartenaires(final String etablissementsPartenaires) {
        this.etablissementsPartenaires = etablissementsPartenaires;
    }

    @GetterAnnotation(isToolbox = true)
    public String getEtablissementsPartenaires() {
        return this.etablissementsPartenaires;
    }

    @SetterAnnotation(isToolbox = true)
    public void setEntreprisesPartenaires(final String entreprisesPartenaires) {
        this.entreprisesPartenaires = entreprisesPartenaires;
    }

    @GetterAnnotation(isToolbox = true)
    public String getEntreprisesPartenaires() {
        return this.entreprisesPartenaires;
    }

    @SetterAnnotation(isToolbox = true)
    public void setConventionEtablissements(final String conventionEtablissements) {
        this.conventionEtablissements = conventionEtablissements;
    }

    @GetterAnnotation(isToolbox = true)
    public String getConventionEtablissements() {
        return this.conventionEtablissements;
    }

    @SetterAnnotation(isToolbox = true)
    public void setFormationsRequises(final String formationsRequises) {
        this.formationsRequises = formationsRequises;
    }

    @GetterAnnotation(isToolbox = true)
    public String getFormationsRequises() {
        return this.formationsRequises;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @return
     */
    public String getFormationsLiees() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(this.formationsLiees);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @param formationsLiees
     */
    public void setFormationsLiees(String formationsLiees) {
        this.formationsLiees = FicheBeanUtils.convertStringCodesLanguesToListBean(formationsLiees, FormationBean.class);
    }

    public List<FormationBean> getFormationsLieesBeans() {
        return this.formationsLiees;
    }

    public void setFormationsLieesBeans(List<FormationBean> formationsLiees) {
        this.formationsLiees = formationsLiees;
    }

    @SetterAnnotation(isToolbox = true)
    public void setPublicCible(final String publicCible) {
        this.publicCible = publicCible;
    }

    @GetterAnnotation(isToolbox = true)
    public String getPublicCible() {
        return this.publicCible;
    }

    @SetterAnnotation(isToolbox = true)
    public void setModalitesCandidatures(final String modalitesCandidatures) {
        this.modalitesCandidatures = modalitesCandidatures;
    }

    @GetterAnnotation(isToolbox = true)
    public String getModalitesCandidatures() {
        return this.modalitesCandidatures;
    }

    @SetterAnnotation(isToolbox = true)
    public void setModalitesCandidaturesSpecifiques(final String modalitesCandidaturesSpecifiques) {
        this.modalitesCandidaturesSpecifiques = modalitesCandidaturesSpecifiques;
    }

    @GetterAnnotation(isToolbox = true)
    public String getModalitesCandidaturesSpecifiques() {
        return this.modalitesCandidaturesSpecifiques;
    }

    @SetterAnnotation(isToolbox = true)
    public void setConditionsAdmission(final String conditionsAdmission) {
        this.conditionsAdmission = conditionsAdmission;
    }

    @GetterAnnotation(isToolbox = true)
    public String getConditionsAdmission() {
        return this.conditionsAdmission;
    }

    @SetterAnnotation(isToolbox = true)
    public void setProgramme(final String programme) {
        this.programme = programme;
    }

    @GetterAnnotation(isToolbox = true)
    public String getProgramme() {
        return this.programme;
    }

    public void setParcoursFiche(final String parcoursFiche) {
        this.parcoursFiche = FicheBeanUtils.convertStringCodesLanguesToListBean(parcoursFiche, ParcoursBean.class);
    }

    public String getParcoursFiche() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(this.parcoursFiche);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @param parcoursFiche
     */
    public void setParcoursFicheBeans(List<ParcoursBean> parcoursFiche) {
        this.parcoursFiche = parcoursFiche;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @return
     */
    public List<ParcoursBean> getParcoursFicheBeans() {
        return this.parcoursFiche;
    }

    @SetterAnnotation(isToolbox = true)
    public void setContenuFormation(final String contenuFormation) {
        this.contenuFormation = contenuFormation;
    }

    @GetterAnnotation(isToolbox = true)
    public String getContenuFormation() {
        return this.contenuFormation;
    }

    @SetterAnnotation(isToolbox = true)
    public void setRythmePremiereAnnee(final String rythmePremiereAnnee) {
        this.rythmePremiereAnnee = rythmePremiereAnnee;
    }

    @GetterAnnotation(isToolbox = true)
    public String getRythmePremiereAnnee() {
        return this.rythmePremiereAnnee;
    }

    @SetterAnnotation(isToolbox = true)
    public void setEquipePedagogique(final String equipePedagogique) {
        this.equipePedagogique = equipePedagogique;
    }

    @GetterAnnotation(isToolbox = true)
    public String getEquipePedagogique() {
        return this.equipePedagogique;
    }

    @SetterAnnotation(isToolbox = true)
    public void setTempsPleinDescription(final String tempsPleinDescription) {
        this.tempsPleinDescription = tempsPleinDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getTempsPleinDescription() {
        return this.tempsPleinDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setCompatibleProfessionnelleDescription(final String compatibleProfessionnelleDescription) {
        this.compatibleProfessionnelleDescription = compatibleProfessionnelleDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getCompatibleProfessionnelleDescription() {
        return this.compatibleProfessionnelleDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setTempsAmenageDescription(final String tempsAmenageDescription) {
        this.tempsAmenageDescription = tempsAmenageDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getTempsAmenageDescription() {
        return this.tempsAmenageDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setAlternanceDescription(final String alternanceDescription) {
        this.alternanceDescription = alternanceDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getAlternanceDescription() {
        return this.alternanceDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setPresentielDescription(final String presentielDescription) {
        this.presentielDescription = presentielDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getPresentielDescription() {
        return this.presentielDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setPartiellementDistanceDescription(final String partiellementDistanceDescription) {
        this.partiellementDistanceDescription = partiellementDistanceDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getPartiellementDistanceDescription() {
        return this.partiellementDistanceDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setTotalementDistanceDescription(final String totalementDistanceDescription) {
        this.totalementDistanceDescription = totalementDistanceDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getTotalementDistanceDescription() {
        return this.totalementDistanceDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setStagesDescription(final String stagesDescription) {
        this.stagesDescription = stagesDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getStagesDescription() {
        return this.stagesDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setSejoursEtrangerDescription(final String sejoursEtrangerDescription) {
        this.sejoursEtrangerDescription = sejoursEtrangerDescription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getSejoursEtrangerDescription() {
        return this.sejoursEtrangerDescription;
    }

    @SetterAnnotation(isToolbox = true)
    public void setProjetsTutoresComplement(final String projetsTutoresComplement) {
        this.projetsTutoresComplement = projetsTutoresComplement;
    }

    @GetterAnnotation(isToolbox = true)
    public String getProjetsTutoresComplement() {
        return this.projetsTutoresComplement;
    }

    @SetterAnnotation(isToolbox = true)
    public void setModalitesEvaluation(final String modalitesEvaluation) {
        this.modalitesEvaluation = modalitesEvaluation;
    }

    @GetterAnnotation(isToolbox = true)
    public String getModalitesEvaluation() {
        return this.modalitesEvaluation;
    }

    @SetterAnnotation(isToolbox = true)
    public void setFicheRncp(final String ficheRncp) {
        this.ficheRncp = ficheRncp;
    }

    @GetterAnnotation(isToolbox = true)
    public String getFicheRncp() {
        return this.ficheRncp;
    }

    @SetterAnnotation(isToolbox = true)
    public void setCompetencesAttestees(final String competencesAttestees) {
        this.competencesAttestees = competencesAttestees;
    }

    @GetterAnnotation(isToolbox = true)
    public String getCompetencesAttestees() {
        return this.competencesAttestees;
    }

    @SetterAnnotation(isToolbox = true)
    public void setPoursuiteEtudes(final String poursuiteEtudes) {
        this.poursuiteEtudes = poursuiteEtudes;
    }

    @GetterAnnotation(isToolbox = true)
    public String getPoursuiteEtudes() {
        return this.poursuiteEtudes;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     * @return
     */
    public String getParcoursPoursuite() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(this.parcoursPoursuite);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @param parcoursPoursuite
     */
    public void setParcoursPoursuite(final String parcoursPoursuite) {
        this.parcoursPoursuite = FicheBeanUtils.convertStringCodesLanguesToListBean(parcoursPoursuite, ParcoursBean.class);
    }

    public List<ParcoursBean> getParcoursPoursuiteBeans() {
        return this.parcoursPoursuite;
    }

    public void setParcoursPoursuiteBeans(final List<ParcoursBean> parcoursPoursuite) {
        this.parcoursPoursuite = parcoursPoursuite;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     * @return
     */
    public String getFormationPoursuite() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(this.formationPoursuite);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     * @param formationPoursuite
     */
    public void setFormationPoursuite(final String formationPoursuite) {
        this.formationPoursuite = FicheBeanUtils.convertStringCodesLanguesToListBean(formationPoursuite, FormationBean.class);
    }

    public List<FormationBean> getFormationPoursuiteBeans() {
        return this.formationPoursuite;
    }

    public void setFormationPoursuiteBeans(List<FormationBean> formationPoursuite) {
        this.formationPoursuite = formationPoursuite;
    }

    @SetterAnnotation(isToolbox = true)
    public void setDebouches(final String debouches) {
        this.debouches = debouches;
    }

    @GetterAnnotation(isToolbox = true)
    public String getDebouches() {
        return this.debouches;
    }

    @SetterAnnotation(isToolbox = true)
    public void setInsertionProfessionnelle(final String insertionProfessionnelle) {
        this.insertionProfessionnelle = insertionProfessionnelle;
    }

    @GetterAnnotation(isToolbox = true)
    public String getInsertionProfessionnelle() {
        return this.insertionProfessionnelle;
    }

    @SetterAnnotation(isToolbox = true)
    public void setTarifComplement(final String tarifComplement) {
        this.tarifComplement = tarifComplement;
    }

    @GetterAnnotation(isToolbox = true)
    public String getTarifComplement() {
        return this.tarifComplement;
    }

    @SetterAnnotation(isToolbox = true)
    public void setModalitesInscription(final String modalitesInscription) {
        this.modalitesInscription = modalitesInscription;
    }

    @GetterAnnotation(isToolbox = true)
    public String getModalitesInscription() {
        return this.modalitesInscription;
    }

    public void setPlaquettesCrm(final String plaquettesCrm) {
        this.plaquettesCrm = plaquettesCrm;
    }

    public String getPlaquettesCrm() {
        return this.plaquettesCrm;
    }

    public void setContactEmail(final String contactEmail) {
        this.contactEmail = contactEmail;
    }

    public String getContactEmail() {
        return this.contactEmail;
    }

    public void setContactTelephone(final String contactTelephone) {
        this.contactTelephone = contactTelephone;
    }

    public String getContactTelephone() {
        return this.contactTelephone;
    }

    public void setContactCrm(final String contactCrm) {
        this.contactCrm = contactCrm;
    }

    public String getContactCrm() {
        return this.contactCrm;
    }

    public void setUrlExterneFormation(final String urlExterneFormation) {
        this.urlExterneFormation = urlExterneFormation;
    }

    public String getUrlExterneFormation() {
        return this.urlExterneFormation;
    }

    @SetterAnnotation(isToolbox = true)
    public void setRenseignements(final String renseignements) {
        this.renseignements = renseignements;
    }

    @GetterAnnotation(isToolbox = true)
    public String getRenseignements() {
        return this.renseignements;
    }

    public void setUrlInscription(final String urlInscription) {
        this.urlInscription = urlInscription;
    }

    public String getUrlInscription() {
        return this.urlInscription;
    }

    public String getAfficherProgramme() {
        return afficherProgramme;
    }

    public void setAfficherProgramme(final String afficherProgramme) {
        this.afficherProgramme = afficherProgramme;
    }

    public String getActiverConstruireParcours() {
        return activerConstruireParcours;
    }

    public void setActiverConstruireParcours(final String activerConstruireParcours) {
        this.activerConstruireParcours = activerConstruireParcours;
    }

    public String getFormationContinue() {
        return formationContinue;
    }

    public void setFormationContinue(final String formationContinue) {
        this.formationContinue = formationContinue;
    }

    public String getTypeFormationContinue() {
        return typeFormationContinue;
    }

    public void setTypeFormationContinue(final String typeFormationContinue) {
        this.typeFormationContinue = typeFormationContinue;
    }

    @Override
    public String getTitre() {
        return intituleLong;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || !(o.getClass() == this.getClass())) {
            return false;
        }
        FormationBean that = (FormationBean) o;
        return code.equals(that.getCode()) && langue.equals(that.getLangue()) && etatObjet.equals(that.getEtatObjet());
    }

    @Override
    public int hashCode() {
        return 10000 * code.hashCode() + 100 * langue.hashCode() + etatObjet.hashCode();
    }
}

