/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: offrestageemploi
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.bean;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import com.kportal.cms.objetspartages.annotation.Aggregation;
import com.kportal.cms.objetspartages.annotation.Fiche;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.Label;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.kportal.cms.objetspartages.annotation.strategy.DatLabelStrategy;
import com.univ.objetspartages.bean.AbstractRestrictionFicheBean;

/**
 * The Class OffresstagesemploisBean.
 */
public class OffresstagesemploisBean extends AbstractRestrictionFicheBean implements Serializable {

    /**
     * The Constant serialVersionUID.
     */
    private static final long serialVersionUID = 3710162723279550084L;

    /**
     * The type offre.
     */
    @Label(contexte = "offrestageemploi", type = "type_offre", strategy = DatLabelStrategy.class)
    private String typeOffre = StringUtils.EMPTY;

    /** The type stage. */
    @Aggregation(name = "nature_offre")
    @Label(contexte = "offrestageemploi", type = "35")
    private String typeStage = StringUtils.EMPTY;

    /** The type emploi. */
    @Aggregation(name = "nature_offre")
    @Label(contexte = "offrestageemploi", type = "37")
    private String typeEmploi = StringUtils.EMPTY;

    /** The type job. */
    @Aggregation(name = "nature_offre")
    @Label(contexte = "offrestageemploi", type = "36")
    private String typeJob = StringUtils.EMPTY;

    /**
     * The code rattachement autres.
     */
    @Fiche(contexte = "offrestageemploi", nomObjet = "structuremodele")
    private String codeRattachementAutres = StringUtils.EMPTY;

    /** The secteurs activites. */
    @Label(contexte = "offrestageemploi", type = "65")
    private String secteursActivites = StringUtils.EMPTY;

    /** The descriptif. */
    private String descriptif = StringUtils.EMPTY;

    /**
     * The duree.
     */
    @Label(contexte = "offrestageemploi", type = "32")
    private String duree = StringUtils.EMPTY;

    /** The commentaire duree. */
    private String commentaireDuree = StringUtils.EMPTY;

    /**
     * The debut.
     */
    @Label(contexte = "offrestageemploi", type = "33")
    private String debut = StringUtils.EMPTY;

    /**
     * The commentaire debut.
     */
    private String commentaireDebut = StringUtils.EMPTY;

    /** The lieu. */
    private String lieu = StringUtils.EMPTY;

    /** The pays. */
    private String pays = StringUtils.EMPTY;

    /**
     * The villes.
     */
    private String villes = StringUtils.EMPTY;

    /**
     * The remuneration stage.
     */
    private String remunerationStage = StringUtils.EMPTY;

    /**
     * The commentaire remuneration stage.
     */
    private String commentaireRemunerationStage = StringUtils.EMPTY;

    /** The date limite offre. */
    private Date dateLimiteOffre = null;

    /**
     * The url detail offre.
     */
    private String urlDetailOffre = StringUtils.EMPTY;

	/**
	 * The filiere.
	 */
	@Fiche(contexte = "offrestageemploi", nomObjet = "defaultstructureksup")
	private String filiere = StringUtils.EMPTY;

    /**
     * The niveau etude.
     */
    @Label(type = "niveau_recrutement", strategy = DatLabelStrategy.class)
    private String niveauEtude = StringUtils.EMPTY;

    /**
     * The formations requises.
     */
    private String formationsRequises = StringUtils.EMPTY;

    /**
     * The competences requises.
     */
    private String competencesRequises = StringUtils.EMPTY;

    /** The raison sociale. */
    private String raisonSociale = StringUtils.EMPTY;

    /** The activite. */
    private String activite = StringUtils.EMPTY;

    /** The responsable. */
    private String responsable = StringUtils.EMPTY;

    /**
     * The coordonnees.
     */
    private String coordonnees = StringUtils.EMPTY;

    /**
     * The code postal.
     */
    private String codePostal = StringUtils.EMPTY;

    /**
     * The ville.
     */
    private String ville = StringUtils.EMPTY;

    /**
     * The telephone.
     */
    private String telephone = StringUtils.EMPTY;

    /**
     * The telecopie.
     */
    private String telecopie = StringUtils.EMPTY;

    /**
     * The email responsable.
     */
    private String emailResponsable = StringUtils.EMPTY;

    /**
     * The site web.
     */
    private String siteWeb = StringUtils.EMPTY;

    /**
     * The activation reponse.
     */
    private String activationReponse = StringUtils.EMPTY;

    /**
     * The complements.
     */
    private String complements = StringUtils.EMPTY;

    /**
     * The theme.
     */
    private String theme = StringUtils.EMPTY;

    /**
     * The date suppression.
     */
    private Date dateSuppression = null;

    /**
     * Gets the type offre.
     *
     * @return the type offre
     */
    public String getTypeOffre() {
        return typeOffre;
    }

    /**
     * Sets the type offre.
     *
     * @param typeOffre
     *            the new type offre
     */
    public void setTypeOffre(final String typeOffre) {
        this.typeOffre = typeOffre;
    }

    /**
     * Gets the type stage.
     *
     * @return the type stage
     */
    public String getTypeStage() {
        return typeStage;
    }

    /**
     * Sets the type stage.
     *
     * @param typeStage
     *            the new type stage
     */
    public void setTypeStage(final String typeStage) {
        this.typeStage = typeStage;
    }

    /**
     * Gets the type emploi.
     *
     * @return the type emploi
     */
    public String getTypeEmploi() {
        return typeEmploi;
    }

    /**
     * Sets the type emploi.
     *
     * @param typeEmploi
     *            the new type emploi
     */
    public void setTypeEmploi(final String typeEmploi) {
        this.typeEmploi = typeEmploi;
    }

    /**
     * Gets the type job.
     *
     * @return the type job
     */
    public String getTypeJob() {
        return typeJob;
    }

    /**
     * Sets the type job.
     *
     * @param typeJob the new type job
     */
    public void setTypeJob(final String typeJob) {
        this.typeJob = typeJob;
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    public String getCodeRattachementAutres() {
        return codeRattachementAutres;
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param codeRattachementAutres the new code rattachement autres
     */
    public void setCodeRattachementAutres(final String codeRattachementAutres) {
        this.codeRattachementAutres = codeRattachementAutres;
    }

    /**
     * Gets the secteurs activites.
     *
     * @return the secteurs activites
     */
    public String getSecteursActivites() {
        return secteursActivites;
    }

    /**
     * Sets the secteurs activites.
     *
     * @param secteursActivites the new secteurs activites
     */
    public void setSecteursActivites(final String secteursActivites) {
        this.secteursActivites = secteursActivites;
    }

    /**
     * Gets the descriptif.
     *
     * @return the descriptif
     */
    public String getDescriptif() {
        return descriptif;
    }

    /**
     * Sets the descriptif.
     *
     * @param descriptif
     *            the new descriptif
     */
    public void setDescriptif(final String descriptif) {
        this.descriptif = descriptif;
    }

    /**
     * Gets the duree.
     *
     * @return the duree
     */
    public String getDuree() {
        return duree;
    }

    /**
     * Sets the duree.
     *
     * @param duree the new duree
     */
    public void setDuree(final String duree) {
        this.duree = duree;
    }

    /**
     * Gets the commentaire duree.
     *
     * @return the commentaire duree
     */
    public String getCommentaireDuree() {
        return commentaireDuree;
    }

    /**
     * Sets the commentaire duree.
     *
     * @param commentaireDuree
     *            the new commentaire duree
     */
    public void setCommentaireDuree(final String commentaireDuree) {
        this.commentaireDuree = commentaireDuree;
    }

    /**
     * Gets the debut.
     *
     * @return the debut
     */
    public String getDebut() {
        return debut;
    }

    /**
     * Sets the debut.
     *
     * @param debut the new debut
     */
    public void setDebut(final String debut) {
        this.debut = debut;
    }

    /**
     * Gets the commentaire debut.
     *
     * @return the commentaire debut
     */
    public String getCommentaireDebut() {
        return commentaireDebut;
    }

    /**
     * Sets the commentaire debut.
     *
     * @param commentaireDebut
     *            the new commentaire debut
     */
    public void setCommentaireDebut(final String commentaireDebut) {
        this.commentaireDebut = commentaireDebut;
    }

    /**
     * Gets the lieu.
     *
     * @return the lieu
     */
    public String getLieu() {
        return lieu;
    }

    /**
     * Sets the lieu.
     *
     * @param lieu
     *            the new lieu
     */
    public void setLieu(final String lieu) {
        this.lieu = lieu;
    }

    /**
     * Gets the pays.
     *
     * @return the pays
     */
    public String getPays() {
        return pays;
    }

    /**
     * Sets the pays.
     *
     * @param pays
     *            the new pays
     */
    public void setPays(final String pays) {
        this.pays = pays;
    }

    /**
     * Gets the villes.
     *
     * @return the villes
     */
    public String getVilles() {
        return villes;
    }

    /**
     * Sets the villes.
     *
     * @param villes the new villes
     */
    public void setVilles(final String villes) {
        this.villes = villes;
    }

    /**
     * Gets the remuneration stage.
     *
     * @return the remuneration stage
     */
    public String getRemunerationStage() {
        return remunerationStage;
    }

    /**
     * Sets the remuneration stage.
     *
     * @param remunerationStage
     *            the new remuneration stage
     */
    public void setRemunerationStage(final String remunerationStage) {
        this.remunerationStage = remunerationStage;
    }

    /**
     * Gets the commentaire remuneration stage.
     *
     * @return the commentaire remuneration stage
     */
    public String getCommentaireRemunerationStage() {
        return commentaireRemunerationStage;
    }

    /**
     * Sets the commentaire remuneration stage.
     *
     * @param commentaireRemunerationStage
     *            the new commentaire remuneration stage
     */
    public void setCommentaireRemunerationStage(final String commentaireRemunerationStage) {
        this.commentaireRemunerationStage = commentaireRemunerationStage;
    }

    /**
     * Gets the date limite offre.
     *
     * @return the date limite offre
     */
    public Date getDateLimiteOffre() {
        return dateLimiteOffre;
    }

    /**
     * Sets the date limite offre.
     *
     * @param dateLimiteOffre
     *            the new date limite offre
     */
    public void setDateLimiteOffre(final Date dateLimiteOffre) {
        this.dateLimiteOffre = dateLimiteOffre;
    }

    /**
     * Gets the url detail offre.
     *
     * @return the url detail offre
     */
    public String getUrlDetailOffre() {
        return urlDetailOffre;
    }

    /**
     * Sets the url detail offre.
     *
     * @param urlDetailOffre the new url detail offre
     */
    public void setUrlDetailOffre(final String urlDetailOffre) {
        this.urlDetailOffre = urlDetailOffre;
    }

    /**
     * Gets the filiere.
     *
     * @return the filiere
     */
    public String getFiliere() {
        return filiere;
    }

    /**
     * Sets the filiere.
     *
     * @param filiere the new filiere
     */
    public void setFiliere(final String filiere) {
        this.filiere = filiere;
    }

    /**
     * Gets the niveau etude.
     *
     * @return the niveau etude
     */
    public String getNiveauEtude() {
        return niveauEtude;
    }

    /**
     * Sets the niveau etude.
     *
     * @param niveauEtude
     *            the new niveau etude
     */
    public void setNiveauEtude(final String niveauEtude) {
        this.niveauEtude = niveauEtude;
    }

    /**
     * Gets the formations requises.
     *
     * @return the formations requises
     */
    public String getFormationsRequises() {
        return formationsRequises;
    }

    /**
     * Sets the formations requises.
     *
     * @param formationsRequises
     *            the new formations requises
     */
    public void setFormationsRequises(final String formationsRequises) {
        this.formationsRequises = formationsRequises;
    }

    /**
     * Gets the competences requises.
     *
     * @return the competences requises
     */
    public String getCompetencesRequises() {
        return competencesRequises;
    }

    /**
     * Sets the competences requises.
     *
     * @param competencesRequises
     *            the new competences requises
     */
    public void setCompetencesRequises(final String competencesRequises) {
        this.competencesRequises = competencesRequises;
    }

    /**
     * Gets the raison sociale.
     *
     * @return the raison sociale
     */
    public String getRaisonSociale() {
        return raisonSociale;
    }

    /**
     * Sets the raison sociale.
     *
     * @param raisonSociale
     *            the new raison sociale
     */
    public void setRaisonSociale(final String raisonSociale) {
        this.raisonSociale = raisonSociale;
    }

    /**
     * Gets the activite.
     *
     * @return the activite
     */
    public String getActivite() {
        return activite;
    }

    /**
     * Sets the activite.
     *
     * @param activite the new activite
     */
    public void setActivite(final String activite) {
        this.activite = activite;
    }

    /**
     * Gets the responsable.
     *
     * @return the responsable
     */
    public String getResponsable() {
        return responsable;
    }

    /**
     * Sets the responsable.
     *
     * @param responsable
     *            the new responsable
     */
    public void setResponsable(final String responsable) {
        this.responsable = responsable;
    }

    /**
     * Gets the coordonnees.
     *
     * @return the coordonnees
     */
    public String getCoordonnees() {
        return coordonnees;
    }

    /**
     * Sets the coordonnees.
     *
     * @param coordonnees
     *            the new coordonnees
     */
    public void setCoordonnees(final String coordonnees) {
        this.coordonnees = coordonnees;
    }

    /**
     * Gets the code postal.
     *
     * @return the code postal
     */
    public String getCodePostal() {
        return codePostal;
    }

    /**
     * Sets the code postal.
     *
     * @param codePostal the new code postal
     */
    public void setCodePostal(final String codePostal) {
        this.codePostal = codePostal;
    }

    /**
     * Gets the ville.
     *
     * @return the ville
     */
    public String getVille() {
        return ville;
    }

    /**
     * Sets the ville.
     *
     * @param ville the new ville
     */
    public void setVille(final String ville) {
        this.ville = ville;
    }

    /**
     * Gets the telephone.
     *
     * @return the telephone
     */
    public String getTelephone() {
        return telephone;
    }

    /**
     * Sets the telephone.
     *
     * @param telephone
     *            the new telephone
     */
    public void setTelephone(final String telephone) {
        this.telephone = telephone;
    }

    /**
     * Gets the telecopie.
     *
     * @return the telecopie
     */
    public String getTelecopie() {
        return telecopie;
    }

    /**
     * Sets the telecopie.
     *
     * @param telecopie
     *            the new telecopie
     */
    public void setTelecopie(final String telecopie) {
        this.telecopie = telecopie;
    }

    /**
     * Gets the email responsable.
     *
     * @return the email responsable
     */
    public String getEmailResponsable() {
        return emailResponsable;
    }

    /**
     * Sets the email responsable.
     *
     * @param emailResponsable
     *            the new email responsable
     */
    public void setEmailResponsable(final String emailResponsable) {
        this.emailResponsable = emailResponsable;
    }

    /**
     * Gets the site web.
     *
     * @return the site web
     */
    public String getSiteWeb() {
        return siteWeb;
    }

    /**
     * Sets the site web.
     *
     * @param siteWeb
     *            the new site web
     */
    public void setSiteWeb(final String siteWeb) {
        this.siteWeb = siteWeb;
    }

    /**
     * Gets the activation reponse.
     *
     * @return the activation reponse
     */
    public String getActivationReponse() {
        return activationReponse;
    }

    /**
     * Sets the activation reponse.
     *
     * @param activationReponse the new activation reponse
     */
    public void setActivationReponse(final String activationReponse) {
        this.activationReponse = activationReponse;
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    @GetterAnnotation(isToolbox = true)
    public String getComplements() {
        return complements;
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplements(final String complements) {
        this.complements = complements;
    }

    /**
     * Gets the theme.
     *
     * @return the theme
     */
    public String getTheme() {
        return theme;
    }

    /**
     * Sets the theme.
     *
     * @param theme
     *            the new theme
     */
    public void setTheme(final String theme) {
        this.theme = theme;
    }

    /**
     * Gets the date suppression.
     *
     * @return the date suppression
     */
    public Date getDateSuppression() {
        return dateSuppression;
    }

    /**
     * Sets the date suppression.
     *
     * @param dateSuppression
     *            the new date suppression
     */
    public void setDateSuppression(final Date dateSuppression) {
        this.dateSuppression = dateSuppression;
    }
}
