/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.bean;

import java.io.Serializable;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.MutablePair;
import org.apache.commons.lang3.tuple.Pair;

import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.cms.objetspartages.annotation.Autocompletion;
import com.kportal.cms.objetspartages.annotation.Fiche;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.ksup.objetspartages.util.FicheBeanUtils;
import com.univ.objetspartages.bean.AbstractRestrictionFicheBean;

/**
 * Bean associé à l'objet Parcours.
 */
public class ParcoursBean extends AbstractRestrictionFicheBean implements Serializable {

    private static final long serialVersionUID = 0L; // TODO : mettre à jour l'UID

    private String codeSiScolarite;

    @Autocompletion(name = "titre_completion")
    private String intituleCourt;

    @Autocompletion(name = "titre_completion")
    private String intituleLong;

    private String intituleParcoursDiplomant;

    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "structuremodele")
    private String codeRattachementAutres = null;

    private String parcoursFictif;

    private String diplomantEtape;

    private String niveauRecrutement;

    private String niveauDuDiplome;

    private String accesSousCondition;

    private String libelleParcoursSemestre;

    private String tempsPlein = null;

    private String compatibleActivitePro = null;

    private String tempsAmenage = null;

    private String alternance = null;

    private String horsContratAlternance = null;

    private String contratApprentissage = null;

    private String contratPro = null;
    private String annee;

    private List<UeupBean> ueups;

    private String presentation;

    private String secteur;

    private List<ParcoursLink> parcoursAmont;

    private List<ParcoursLink> parcoursAval;

    private int dureeSemestre = 0;

    private int numeroPremierSemestre = 0;

    private String libelleParcoursAnnee;

    /**
     * Instantiates a new bean.
     */
    public ParcoursBean() {
        super();
    }

    /**
     * Getter and Setter
     */
    public void setCodeSiScolarite(final String codeSiScolarite) {
        this.codeSiScolarite = codeSiScolarite;
    }

    public String getCodeSiScolarite() {
        return this.codeSiScolarite;
    }

    public void setIntituleCourt(final String intituleCourt) {
        this.intituleCourt = intituleCourt;
    }

    public String getIntituleCourt() {
        return this.intituleCourt;
    }

    public void setIntituleLong(final String intituleLong) {
        this.intituleLong = intituleLong;
    }

    public String getIntituleLong() {
        return this.intituleLong;
    }

    public void setParcoursFictif(final String parcoursFictif) {
        this.parcoursFictif = parcoursFictif;
    }

    public String getParcoursFictif() {
        return this.parcoursFictif;
    }

    public void setDiplomantEtape(final String diplomantEtape) {
        this.diplomantEtape = diplomantEtape;
    }

    public String getDiplomantEtape() {
        return this.diplomantEtape;
    }

    public void setNiveauRecrutement(final String niveauRecrutement) {
        this.niveauRecrutement = niveauRecrutement;
    }

    public String getNiveauRecrutement() {
        return this.niveauRecrutement;
    }

    public void setNiveauDuDiplome(final String niveauDuDiplome) {
        this.niveauDuDiplome = niveauDuDiplome;
    }

    public String getNiveauDuDiplome() {
        return this.niveauDuDiplome;
    }

    public void setAccesSousCondition(final String accesSousCondition) {
        this.accesSousCondition = accesSousCondition;
    }

    public String getAccesSousCondition() {
        return this.accesSousCondition;
    }

    public String getTempsPlein() {
        return tempsPlein;
    }

    public void setTempsPlein(final String tempsPlein) {
        this.tempsPlein = tempsPlein;
    }

    public String getCompatibleActivitePro() {
        return compatibleActivitePro;
    }

    public void setCompatibleActivitePro(final String compatibleActivitePro) {
        this.compatibleActivitePro = compatibleActivitePro;
    }

    public String getTempsAmenage() {
        return tempsAmenage;
    }

    public void setTempsAmenage(final String tempsAmenage) {
        this.tempsAmenage = tempsAmenage;
    }

    public String getAlternance() {
        return alternance;
    }

    public void setAlternance(final String alternance) {
        this.alternance = alternance;
    }

    public String getHorsContratAlternance() {
        return horsContratAlternance;
    }

    public void setHorsContratAlternance(final String horsContratAlternance) {
        this.horsContratAlternance = horsContratAlternance;
    }

    public String getContratApprentissage() {
        return contratApprentissage;
    }

    public void setContratApprentissage(final String contratApprentissage) {
        this.contratApprentissage = contratApprentissage;
    }

    public String getContratPro() {
        return contratPro;
    }

    public void setContratPro(final String contratPro) {
        this.contratPro = contratPro;
    }

    public String getCodeRattachementAutres() {
        return codeRattachementAutres;
    }

    public void setCodeRattachementAutres(final String codeRattachementAutres) {
        this.codeRattachementAutres = codeRattachementAutres;
    }

    public void setAnnee(final String annee) {
        this.annee = annee;
    }

    public String getAnnee() {
        return this.annee;
    }

    @SetterAnnotation(isToolbox = true)
    public void setPresentation(final String presentation) {
        this.presentation = presentation;
    }

    @GetterAnnotation(isToolbox = true)
    public String getPresentation() {
        return this.presentation;
    }

    public void setSecteur(final String secteur) {
        this.secteur = secteur;
    }

    public String getSecteur() {
        return this.secteur;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @param parcoursAmont
     */
    public void setParcoursAmont(final String parcoursAmont) {
        this.parcoursAmont = FicheBeanUtils.convertStringCodesLanguesOptionsToListOptionAndBeans(parcoursAmont, ParcoursBean.class).stream()
            .map(ParcoursLink::new)
            .collect(Collectors.toList());
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @return
     */
    public String getParcoursAmont() {
        if (CollectionUtils.isEmpty(parcoursAmont)) {
            return StringUtils.EMPTY;
        }
        List<Pair<ParcoursBean, String>> parcoursLinkPairList = parcoursAmont.stream()
            .map(link -> Pair.of(link.getLeft(), link.getRight()))
            .collect(Collectors.toList());
        return FicheBeanUtils.convertListOptionAndBeanToStringCodesLanguesOptions(parcoursLinkPairList);
    }

    public void setParcoursAmontBeans(List<ParcoursLink> parcoursAmont) {
        this.parcoursAmont = parcoursAmont;
    }

    public List<ParcoursLink>  getParcoursAmontBeans() {
        return this.parcoursAmont;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @param parcoursAval
     */
    public void setParcoursAval(final String parcoursAval) {
        this.parcoursAval = FicheBeanUtils.convertStringCodesLanguesOptionsToListOptionAndBeans(parcoursAval, ParcoursBean.class).stream()
            .map(ParcoursLink::new)
            .collect(Collectors.toList());
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @return
     */
    public String getParcoursAval() {
        if (CollectionUtils.isEmpty(parcoursAval)) {
            return StringUtils.EMPTY;
        }
        List<Pair<ParcoursBean, String>> parcoursLinkPairList = parcoursAval.stream()
            .map(link -> Pair.of(link.getLeft(), link.getRight()))
            .collect(Collectors.toList());
        return FicheBeanUtils.convertListOptionAndBeanToStringCodesLanguesOptions(parcoursLinkPairList);
    }

    public void setParcoursAvalBeans(List<ParcoursLink> parcoursAval) {
        this.parcoursAval = parcoursAval;
    }

    public List<ParcoursLink> getParcoursAvalBeans() {
        return this.parcoursAval;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @return
     */
    public String getUeups() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(ueups);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @param ueups
     */
    public void setUeups(String ueups) {
        this.ueups = FicheBeanUtils.convertStringCodesLanguesToListBean(ueups, UeupBean.class);
    }

    public List<UeupBean> getUeupsBeans() {
        return this.ueups;
    }

    public void setUeupsBeans(List<UeupBean> ueups) {
        this.ueups = ueups;
    }

    public String getLibelleParcoursSemestre() {
        return libelleParcoursSemestre;
    }

    public void setLibelleParcoursSemestre(String libelleParcoursSemestre) {
        this.libelleParcoursSemestre = libelleParcoursSemestre;
    }

    public int getDureeSemestre() {
        return dureeSemestre;
    }

    public void setDureeSemestre(int dureeSemestre) {
        this.dureeSemestre = dureeSemestre;
    }

    public int getNumeroPremierSemestre() {
        return numeroPremierSemestre;
    }

    public void setNumeroPremierSemestre(int numeroPremierSemestre) {
        this.numeroPremierSemestre = numeroPremierSemestre;
    }

    public String getIntituleParcoursDiplomant() {
        return intituleParcoursDiplomant;
    }

    public void setIntituleParcoursDiplomant(String intituleParcoursDiplomant) {
        this.intituleParcoursDiplomant = intituleParcoursDiplomant;
    }

    public String getLibelleParcoursAnnee() {
        return libelleParcoursAnnee;
    }

    public void setLibelleParcoursAnnee(String libelleParcoursAnnee) {
        this.libelleParcoursAnnee = libelleParcoursAnnee;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || !(o.getClass() == this.getClass())) {
            return false;
        }
        ParcoursBean that = (ParcoursBean) o;
        return code.equals(that.getCode()) && langue.equals(that.getLangue()) && etatObjet.equals(that.getEtatObjet());
    }

    @Override
    public int hashCode() {
        return 10000 * code.hashCode() + 100 * langue.hashCode() + etatObjet.hashCode();
    }
}

