/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.bean;

import java.io.Serializable;
import java.util.Objects;

import org.apache.commons.lang3.tuple.Pair;

import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.cms.objetspartages.annotation.Fiche;
import com.kportal.cms.objetspartages.annotation.Label;

/**
 * Représentation d'un lien entre parcours.
 */
public class ParcoursLink extends Pair<ParcoursBean, String> implements Serializable {

    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "parcours")
    private ParcoursBean parcours;

    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2034")
    private String access;

    public ParcoursLink(final Pair<ParcoursBean, String> mutablePair) {
        this.parcours = mutablePair.getLeft();
        this.access = mutablePair.getRight();
    }

    public ParcoursLink(final ParcoursBean parcours, final String access) {
        this.parcours = parcours;
        this.access = access;
    }

    @Override
    public ParcoursBean getLeft() {
        return parcours;
    }

    @Override
    public String getRight() {
        return access;
    }

    @Override
    public String setValue(final String access) {
        this.access = access;
        return access;
    }

    public void setLeft(final ParcoursBean left) {
        this.parcours = left;
    }

    public void setRight(final String right) {
        this.access = right;
    }

    public ParcoursBean getParcours() {
        return parcours;
    }

    public void setParcours(final ParcoursBean parcours) {
        this.parcours = parcours;
    }

    public String getAccess() {
        return access;
    }

    public void setAccess(final String access) {
        this.access = access;
    }

    @Override
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }
        final ParcoursLink that = (ParcoursLink) o;
        return parcours.getId().equals(that.parcours.getId()) && access.equals(that.access);
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), parcours, access);
    }
}
