/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.bean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.cms.objetspartages.annotation.Autocompletion;
import com.kportal.cms.objetspartages.annotation.Fiche;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.Label;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.ksup.objetspartages.util.FicheBeanUtils;
import com.univ.objetspartages.bean.AbstractRestrictionFicheBean;

/**
 * The Class UeupBean.
 */
public class UeupBean extends AbstractRestrictionFicheBean implements Serializable {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -1058528589518565727L;

    /**
     * Code SI Scolarite
     */
    private String codeSiScolarite;

    /** The libelle court. */
    @Autocompletion(name = "titre_completion")
    private String libelleCourt = null;

    /** The libelle long. */
    @Autocompletion(name = "titre_completion")
    private String libelleLong = null;

    /** The credit ects. */
    private String creditEcts = null;

    /** The semestre. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "57")
    private String semestre = null;

    /** The type. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "56")
    private String type = null;

    /** The nature. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "54")
    private String nature = null;

    /** The nb maxi achoix. */
    private String nbMaxiAchoix = null;

    /** The nb mini achoix. */
    private String nbMiniAchoix = null;

    /** The disciplines. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "55")
    private String disciplines = null;

    /** The code rattachement autres. */
    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "structuremodele")
    private String codeRattachementAutres = null;

    /** The ville. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "22")
    private String ville = null;

    /** The a distance. */
    private String aDistance = null;

    /** The mixte distance. */
    private String mixteDistance = null;

    /** The code responsables. */
    @Fiche(contexte = OffreFormationUtil.ID_EXTENSION, nomObjet = "annuaireksup")
    private String codeResponsables = null;

    /** The complements responsable. */
    private String complementsResponsable = null;

    /** The coordonnees. */
    private String coordonnees = null;

    /** The volume horaire total. */
    private String volumeHoraireTotal = null;

    /** The volume horaire cm. */
    private String volumeHoraireCm = null;

    /** The volume horaire td. */
    private String volumeHoraireTd = null;

    /** The volume horaire tp. */
    private String volumeHoraireTp = null;

    /** The volume horaire projet. */
    private String volumeHoraireProjet = null;

    /** The volume horaire stage. */
    private String volumeHoraireStage = null;

    /** The volume horaire ds. */
    private String volumeHoraireDs = null;

    /** The volume horaire formation distance. */
    private String volumeHoraireFormationDistance = null;

    /** The volume horaire presentiel. */
    private String volumeHorairePresentiel = null;

    /** The prerequis. */
    private String prerequis = null;

    /** The code ueups requises. */
    private List<UeupBean> ueupsRequises = new ArrayList<>();

    /** The objectifs. */
    private String objectifs = null;

    /** The calendrier. */
    private String calendrier = null;

    /** The contenu. */
    private String contenu = null;

    /** The code ueupfils. */
    private List<UeupBean> ueupfils = new ArrayList<>();

    /** The code cours. */
    private List<CoursBean> cours = new ArrayList<>();

    /** The bibliographie. */
    private String bibliographie = null;

    /** The controle connaissances. */
    private String controleConnaissances = null;

    /** The domaines. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2004")
    private String domaines = null;

    /** The domaine erasmus. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2017")
    private String domaineErasmus = null;

    /** The grade. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2003")
    private String grade = null;

    /** The type diplome. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "20")
    private String typeDiplome = null;

    /** The langues enseignement. */
    @Label(contexte = OffreFormationUtil.ID_EXTENSION, type = "2011")
    private String languesEnseignement = null;

    /** The complements. */
    private String complements = null;

    /** The rattachement bandeau. */
    private String rattachementBandeau = null;

    public String getCodeSiScolarite() {
        return codeSiScolarite;
    }

    public void setCodeSiScolarite(final String codeSiScolarite) {
        this.codeSiScolarite = codeSiScolarite;
    }

    /**
     * Gets the libelle court.
     *
     * @return the libelle court
     */
    public String getLibelleCourt() {
        return libelleCourt;
    }

    /**
     * Sets the libelle court.
     *
     * @param libelleCourt
     *            the new libelle court
     */
    public void setLibelleCourt(final String libelleCourt) {
        this.libelleCourt = libelleCourt;
    }

    /**
     * Gets the libelle long.
     *
     * @return the libelle long
     */
    public String getLibelleLong() {
        return libelleLong;
    }

    /**
     * Sets the libelle long.
     *
     * @param libelleLong
     *            the new libelle long
     */
    public void setLibelleLong(final String libelleLong) {
        this.libelleLong = libelleLong;
    }

    /**
     * Gets the credit ects.
     *
     * @return the credit ects
     */
    public String getCreditEcts() {
        return creditEcts;
    }

    /**
     * Sets the credit ects.
     *
     * @param creditEcts
     *            the new credit ects
     */
    public void setCreditEcts(final String creditEcts) {
        this.creditEcts = creditEcts;
    }

    /**
     * Gets the semestre.
     *
     * @return the semestre
     */
    public String getSemestre() {
        return semestre;
    }

    /**
     * Sets the semestre.
     *
     * @param semestre
     *            the new semestre
     */
    public void setSemestre(final String semestre) {
        this.semestre = semestre;
    }

    /**
     * Gets the type.
     *
     * @return the type
     */
    public String getType() {
        return type;
    }

    /**
     * Sets the type.
     *
     * @param type
     *            the new type
     */
    public void setType(final String type) {
        this.type = type;
    }

    /**
     * Gets the nature.
     *
     * @return the nature
     */
    public String getNature() {
        return nature;
    }

    /**
     * Sets the nature.
     *
     * @param nature
     *            the new nature
     */
    public void setNature(final String nature) {
        this.nature = nature;
    }

    /**
     * Gets the nb maxi achoix.
     *
     * @return the nb maxi achoix
     */
    public String getNbMaxiAchoix() {
        return nbMaxiAchoix;
    }

    /**
     * Sets the nb maxi achoix.
     *
     * @param nbMaxiAchoix
     *            the new nb maxi achoix
     */
    public void setNbMaxiAchoix(final String nbMaxiAchoix) {
        this.nbMaxiAchoix = nbMaxiAchoix;
    }

    /**
     * Gets the nb mini achoix.
     *
     * @return the nb mini achoix
     */
    public String getNbMiniAchoix() {
        return nbMiniAchoix;
    }

    /**
     * Sets the nb mini achoix.
     *
     * @param nbMiniAchoix
     *            the new nb mini achoix
     */
    public void setNbMiniAchoix(final String nbMiniAchoix) {
        this.nbMiniAchoix = nbMiniAchoix;
    }

    /**
     * Gets the disciplines.
     *
     * @return the disciplines
     */
    public String getDisciplines() {
        return disciplines;
    }

    /**
     * Sets the disciplines.
     *
     * @param disciplines
     *            the new disciplines
     */
    public void setDisciplines(final String disciplines) {
        this.disciplines = disciplines;
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    public String getCodeRattachementAutres() {
        return codeRattachementAutres;
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param codeRattachementsAutres
     *            the new code rattachement autres
     */
    public void setCodeRattachementAutres(final String codeRattachementsAutres) {
        this.codeRattachementAutres = codeRattachementsAutres;
    }

    /**
     * Gets the ville.
     *
     * @return the ville
     */
    public String getVille() {
        return ville;
    }

    /**
     * Sets the ville.
     *
     * @param ville
     *            the new ville
     */
    public void setVille(final String ville) {
        this.ville = ville;
    }

    /**
     * Gets the a distance.
     *
     * @return the a distance
     */
    public String getaDistance() {
        return aDistance;
    }

    /**
     * Sets the a distance.
     *
     * @param aDistance
     *            the new a distance
     */
    public void setaDistance(final String aDistance) {
        this.aDistance = aDistance;
    }

    /**
     * Gets the mixte distance.
     *
     * @return the mixte distance
     */
    public String getMixteDistance() {
        return mixteDistance;
    }

    /**
     * Sets the mixte distance.
     *
     * @param mixteDistance
     *            the new mixte distance
     */
    public void setMixteDistance(final String mixteDistance) {
        this.mixteDistance = mixteDistance;
    }

    /**
     * Gets the code responsables.
     *
     * @return the code responsables
     */
    public String getCodeResponsables() {
        return codeResponsables;
    }

    /**
     * Sets the code responsables.
     *
     * @param codeResponsables
     *            the new code responsables
     */
    public void setCodeResponsables(final String codeResponsables) {
        this.codeResponsables = codeResponsables;
    }

    /**
     * Gets the complements responsable.
     *
     * @return the complements responsable
     */
    @GetterAnnotation(isToolbox = true)
    public String getComplementsResponsable() {
        return complementsResponsable;
    }

    /**
     * Sets the complements responsable.
     *
     * @param complementsResponsable
     *            the new complements responsable
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplementsResponsable(final String complementsResponsable) {
        this.complementsResponsable = complementsResponsable;
    }

    /**
     * Gets the coordonnees.
     *
     * @return the coordonnees
     */
    @GetterAnnotation(isToolbox = true)
    public String getCoordonnees() {
        return coordonnees;
    }

    /**
     * Sets the coordonnees.
     *
     * @param coordonnees
     *            the new coordonnees
     */
    @SetterAnnotation(isToolbox = true)
    public void setCoordonnees(final String coordonnees) {
        this.coordonnees = coordonnees;
    }

    /**
     * Gets the volume horaire total.
     *
     * @return the volume horaire total
     */
    public String getVolumeHoraireTotal() {
        return volumeHoraireTotal;
    }

    /**
     * Sets the volume horaire total.
     *
     * @param volumeHoraireTotal
     *            the new volume horaire total
     */
    public void setVolumeHoraireTotal(final String volumeHoraireTotal) {
        this.volumeHoraireTotal = volumeHoraireTotal;
    }

    /**
     * Gets the volume horaire cm.
     *
     * @return the volume horaire cm
     */
    public String getVolumeHoraireCm() {
        return volumeHoraireCm;
    }

    /**
     * Sets the volume horaire cm.
     *
     * @param volumeHoraireCm
     *            the new volume horaire cm
     */
    public void setVolumeHoraireCm(final String volumeHoraireCm) {
        this.volumeHoraireCm = volumeHoraireCm;
    }

    /**
     * Gets the volume horaire td.
     *
     * @return the volume horaire td
     */
    public String getVolumeHoraireTd() {
        return volumeHoraireTd;
    }

    /**
     * Sets the volume horaire td.
     *
     * @param volumeHoraireTd
     *            the new volume horaire td
     */
    public void setVolumeHoraireTd(final String volumeHoraireTd) {
        this.volumeHoraireTd = volumeHoraireTd;
    }

    /**
     * Gets the volume horaire tp.
     *
     * @return the volume horaire tp
     */
    public String getVolumeHoraireTp() {
        return volumeHoraireTp;
    }

    /**
     * Sets the volume horaire tp.
     *
     * @param volumeHoraireTp
     *            the new volume horaire tp
     */
    public void setVolumeHoraireTp(final String volumeHoraireTp) {
        this.volumeHoraireTp = volumeHoraireTp;
    }

    /**
     * Gets the volume horaire projet.
     *
     * @return the volume horaire projet
     */
    public String getVolumeHoraireProjet() {
        return volumeHoraireProjet;
    }

    /**
     * Sets the volume horaire projet.
     *
     * @param volumeHoraireProjet
     *            the new volume horaire projet
     */
    public void setVolumeHoraireProjet(final String volumeHoraireProjet) {
        this.volumeHoraireProjet = volumeHoraireProjet;
    }

    /**
     * Gets the volume horaire stage.
     *
     * @return the volume horaire stage
     */
    public String getVolumeHoraireStage() {
        return volumeHoraireStage;
    }

    /**
     * Sets the volume horaire stage.
     *
     * @param volumeHoraireStage
     *            the new volume horaire stage
     */
    public void setVolumeHoraireStage(final String volumeHoraireStage) {
        this.volumeHoraireStage = volumeHoraireStage;
    }

    /**
     * Gets the volume horaire ds.
     *
     * @return the volume horaire ds
     */
    public String getVolumeHoraireDs() {
        return volumeHoraireDs;
    }

    /**
     * Sets the volume horaire ds.
     *
     * @param volumeHoraireDs
     *            the new volume horaire ds
     */
    public void setVolumeHoraireDs(final String volumeHoraireDs) {
        this.volumeHoraireDs = volumeHoraireDs;
    }

    /**
     * Gets the volume horaire formation distance.
     *
     * @return the volume horaire formation distance
     */
    public String getVolumeHoraireFormationDistance() {
        return volumeHoraireFormationDistance;
    }

    /**
     * Sets the volume horaire formation distance.
     *
     * @param volumeHoraireFormationDistance
     *            the new volume horaire formation distance
     */
    public void setVolumeHoraireFormationDistance(final String volumeHoraireFormationDistance) {
        this.volumeHoraireFormationDistance = volumeHoraireFormationDistance;
    }

    /**
     * Gets the volume horaire presentiel.
     *
     * @return the volume horaire presentiel
     */
    public String getVolumeHorairePresentiel() {
        return volumeHorairePresentiel;
    }

    /**
     * Sets the volume horaire presentiel.
     *
     * @param volumeHorairePresentiel
     *            the new volume horaire presentiel
     */
    public void setVolumeHorairePresentiel(final String volumeHorairePresentiel) {
        this.volumeHorairePresentiel = volumeHorairePresentiel;
    }

    /**
     * Gets the prerequis.
     *
     * @return the prerequis
     */
    @GetterAnnotation(isToolbox = true)
    public String getPrerequis() {
        return prerequis;
    }

    /**
     * Sets the prerequis.
     *
     * @param prerequis
     *            the new prerequis
     */
    @SetterAnnotation(isToolbox = true)
    public void setPrerequis(final String prerequis) {
        this.prerequis = prerequis;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     * @return the code ueups requises
     */
    public String getUeupsRequises() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(this.ueupsRequises);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     *
     */
    public void setUeupsRequises(final String ueupsRequises) {
        this.ueupsRequises = FicheBeanUtils.convertStringCodesLanguesToListBean(ueupsRequises, UeupBean.class);
    }

    public List<UeupBean> getUeupsRequisesBeans() {
        return this.ueupsRequises;
    }

    public void setUeupsRequisesBeans(List<UeupBean> ueupsRequises) {
        this.ueupsRequises = ueupsRequises;
    }

    /**
     * Gets the objectifs.
     *
     * @return the objectifs
     */
    @GetterAnnotation(isToolbox = true)
    public String getObjectifs() {
        return objectifs;
    }

    /**
     * Sets the objectifs.
     *
     * @param objectifs
     *            the new objectifs
     */
    @SetterAnnotation(isToolbox = true)
    public void setObjectifs(final String objectifs) {
        this.objectifs = objectifs;
    }

    /**
     * Gets the calendrier.
     *
     * @return the calendrier
     */
    @GetterAnnotation(isToolbox = true)
    public String getCalendrier() {
        return calendrier;
    }

    /**
     * Sets the calendrier.
     *
     * @param calendrier
     *            the new calendrier
     */
    @SetterAnnotation(isToolbox = true)
    public void setCalendrier(final String calendrier) {
        this.calendrier = calendrier;
    }

    /**
     * Gets the contenu.
     *
     * @return the contenu
     */
    @GetterAnnotation(isToolbox = true)
    public String getContenu() {
        return contenu;
    }

    /**
     * Sets the contenu.
     *
     * @param contenu
     *            the new contenu
     */
    @SetterAnnotation(isToolbox = true)
    public void setContenu(final String contenu) {
        this.contenu = contenu;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     */
    public String getUeupfils() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(this.ueupfils);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     */
    public void setUeupfils(final String codeUeupfils) {
        this.ueupfils = FicheBeanUtils.convertStringCodesLanguesToListBean(codeUeupfils, UeupBean.class);
    }

    public List<UeupBean> getUeupfilsBeans() {
        return this.ueupfils;
    }

    public void setUeupfilsBeans(List<UeupBean> ueupfils) {
        this.ueupfils = ueupfils;
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     */
    public String getCours() {
        return FicheBeanUtils.convertListBeanToStringCodesLangues(this.cours);
    }

    /**
     * En attendant correctif : IMPORTXML-45, cette méthode sert pour la compatibilité KSUP.
     * Pour utiliser la méthode avec des beans, ajouter 'Beans' au nom de la méthode.
     *
     */
    public void setCours(final String codeCours) {
        this.cours = FicheBeanUtils.convertStringCodesLanguesToListBean(codeCours, CoursBean.class);
    }

    public List<CoursBean> getCoursBeans() {
        return this.cours;
    }

    public void setCoursBeans(List<CoursBean> cours) {
        this.cours = cours;
    }

    /**
     * Gets the bibliographie.
     *
     * @return the bibliographie
     */
    @GetterAnnotation(isToolbox = true)
    public String getBibliographie() {
        return bibliographie;
    }

    /**
     * Sets the bibliographie.
     *
     * @param bibliographie
     *            the new bibliographie
     */
    @SetterAnnotation(isToolbox = true)
    public void setBibliographie(final String bibliographie) {
        this.bibliographie = bibliographie;
    }

    /**
     * Gets the controle connaissances.
     *
     * @return the controle connaissances
     */
    @GetterAnnotation(isToolbox = true)
    public String getControleConnaissances() {
        return controleConnaissances;
    }

    /**
     * Sets the controle connaissances.
     *
     * @param controleConnaissances
     *            the new controle connaissances
     */
    @SetterAnnotation(isToolbox = true)
    public void setControleConnaissances(final String controleConnaissances) {
        this.controleConnaissances = controleConnaissances;
    }

    /**
     * Gets the domaines.
     *
     * @return the domaines
     */
    public String getDomaines() {
        return domaines;
    }

    /**
     * Sets the domaines.
     *
     * @param domaines
     *            the new domaines
     */
    public void setDomaines(final String domaines) {
        this.domaines = domaines;
    }

    /**
     * Gets the domaine erasmus.
     *
     * @return the domaine erasmus
     */
    public String getDomaineErasmus() {
        return domaineErasmus;
    }

    /**
     * Sets the domaine erasmus.
     *
     * @param domaineErasmus
     *            the new domaine erasmus
     */
    public void setDomaineErasmus(final String domaineErasmus) {
        this.domaineErasmus = domaineErasmus;
    }

    /**
     * Gets the grade.
     *
     * @return the grade
     */
    public String getGrade() {
        return grade;
    }

    /**
     * Sets the grade.
     *
     * @param grade
     *            the new grade
     */
    public void setGrade(final String grade) {
        this.grade = grade;
    }

    /**
     * Gets the type diplome.
     *
     * @return the type diplome
     */
    public String getTypeDiplome() {
        return typeDiplome;
    }

    /**
     * Sets the type diplome.
     *
     * @param typeDiplome
     *            the new type diplome
     */
    public void setTypeDiplome(final String typeDiplome) {
        this.typeDiplome = typeDiplome;
    }

    /**
     * Gets the langues enseignement.
     *
     * @return the langues enseignement
     */
    public String getLanguesEnseignement() {
        return languesEnseignement;
    }

    /**
     * Sets the langues enseignement.
     *
     * @param languesEnseignement
     *            the new langues enseignement
     */
    public void setLanguesEnseignement(final String languesEnseignement) {
        this.languesEnseignement = languesEnseignement;
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    @GetterAnnotation(isToolbox = true)
    public String getComplements() {
        return complements;
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplements(final String complements) {
        this.complements = complements;
    }

    /**
     * Gets the rattachement bandeau.
     *
     * @return the rattachement bandeau
     */
    public String getRattachementBandeau() {
        return rattachementBandeau;
    }

    /**
     * Sets the rattachement bandeau.
     *
     * @param rattachementBandeau
     *            the new rattachement bandeau
     */
    public void setRattachementBandeau(final String rattachementBandeau) {
        this.rattachementBandeau = rattachementBandeau;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || !(o.getClass() == this.getClass())) {
            return false;
        }
        UeupBean that = (UeupBean) o;
        return code.equals(that.getCode()) && langue.equals(that.getLangue()) && etatObjet.equals(that.getEtatObjet());
    }

    @Override
    public int hashCode() {
        return 10000 * code.hashCode() + 100 * langue.hashCode() + etatObjet.hashCode();
    }
}
