/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.util.List;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

import com.ksup.objetspartages.bean.AnnuaireksupBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;

/**
 * DAO de l'objet {@link AnnuaireksupBean}
 */
public class AnnuaireksupDAO extends AbstractFicheDAO<AnnuaireksupBean> {

    public AnnuaireksupDAO() {
        tableName = "ANNUAIREKSUP";
        orderClause = "ORDER BY T1.NOM ASC";
    }

    /**
     * Récupère la liste des fiches annuaire selon la structure.
     * @param codeStructure le code de la structure recherchée
     * @return la liste trouvée
     */
    public List<AnnuaireksupBean> getByStructure(final String codeStructure) {
        final MapSqlParameterSource namedParameters = new MapSqlParameterSource("structUnique", codeStructure)
            .addValue("structSt", codeStructure + ";%")
            .addValue("structMi", "%;" + codeStructure + ";%")
            .addValue("structEn", "%;" + codeStructure );
        final String request = "select * from " + tableName + " WHERE CODE_RATTACHEMENT = :structUnique"
            + " OR CODE_RATTACHEMENT_AUTRES = :structUnique"
            + " OR CODE_RATTACHEMENT_AUTRES LIKE :structSt"
            + " OR CODE_RATTACHEMENT_AUTRES LIKE :structMi"
            + " OR CODE_RATTACHEMENT_AUTRES LIKE :structEn";
        return namedParameterJdbcTemplate.query(request, namedParameters, rowMapper);
    }

}
