/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.ksup.objetspartages.bean.DefaultstructureksupBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;
import com.univ.objetspartages.om.EtatFiche;

/**
 * 5 : STRUCTURE_ENSEIGNEMENT
 * 21 : CONTACTS
 * 24 : DOMAINE_FORMATION_ETABLISSEMENT
 * 25 : FORMATIONS
 * 26 : RECHERCHE
 * 27 : PARTENARIATS
 *
 *
 *
 */
public class DefaultstructureksupDAO extends AbstractFicheDAO<DefaultstructureksupBean> {

    private final static Logger LOG = LoggerFactory.getLogger(DefaultstructureksupDAO.class);

    public DefaultstructureksupDAO() {
        tableName = "DEFAULTSTRUCTUREKSUP";
        orderClause = "ORDER BY T1.LIBELLE_LONG ASC";
    }

    /**
     * Récupère la liste des structures correspondant à un code parent et un état.
     * @param codeRattachement le code de la structure parente
     * @param etatFiche état de la fiche
     * @return la liste des structure correspondantes
     */
    public List<DefaultstructureksupBean> getByCodeRattachementAndState(String codeRattachement, final EtatFiche etatFiche) {
        MapSqlParameterSource parameterSource = new MapSqlParameterSource("codeRattachement", codeRattachement)
            .addValue("etat", etatFiche.getEtat());
        String request = "SELECT * from DEFAULTSTRUCTUREKSUP WHERE CODE_RATTACHEMENT = :codeRattachement AND ETAT_OBJET = :etat";
        return select(request, parameterSource);
    }

    /**
     * Récupère une structure par son code et son état.
     * @param code le code de la structure recherchée
     * @param etatFiche L'état de la fiche
     * @return la structure ou null si non existante
     */
    public DefaultstructureksupBean getByCodeAndState(String code, EtatFiche etatFiche) {
        MapSqlParameterSource parameterSource = new MapSqlParameterSource("code", code)
            .addValue("etat", etatFiche.getEtat());
        String request = "SELECT * from DEFAULTSTRUCTUREKSUP WHERE CODE = :code AND ETAT_OBJET = :etat";
        try {
            return namedParameterJdbcTemplate.queryForObject(request, parameterSource, rowMapper);
        } catch (EmptyResultDataAccessException e) {
            LOG.debug(String.format("empty resultset size for code %s on table %s", code, tableName), e);
        } catch (IncorrectResultSizeDataAccessException e) {
            LOG.debug("incorrect result size for query", e);
            LOG.info(String.format("incorrect resultset size for code %s", code));
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured requesting for ESPACECOLLABORATIF row with code %s", code), dae);
        }
        return null;
    }
}
