/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.ksup.objetspartages.bean.FormationBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;

/**
 * Classe d'accès aux données pour Formation.
 */
public class FormationDAO extends AbstractFicheDAO<FormationBean> {

    private FormationJointuresDAO formationJointuresDAO;

    public FormationDAO() {
        tableName = "FORMATION";
        orderClause = "ORDER BY T1.INTITULE_LONG ASC";
    }

    public void setFormationJointuresDAO(FormationJointuresDAO formationJointureDAO) {
        this.formationJointuresDAO = formationJointureDAO;
    }

    @Override
    public List<FormationBean> select(String request) throws DataSourceException {
        List<FormationBean> results = super.select(request);
        for (FormationBean formationBean : results) {
            selectJoinedDatas(formationBean);
        }
        return results;
    }

    @Override
    public List<FormationBean> select(String request, SqlParameterSource parameters) {
        List<FormationBean> results = super.select(request, parameters);
        for (FormationBean formationBean : results) {
            selectJoinedDatas(formationBean);
        }
        return results;
    }

    @Override
    public FormationBean getById(Long id) throws DataSourceException {
        FormationBean result = super.getById(id);
        if(result != null) {
            selectJoinedDatas(result);
        }
        return result;
    }

    protected void selectJoinedDatas(FormationBean formationBean) {
        formationBean.setFormationsLieesBeans(formationJointuresDAO.selectFormationsLiees(formationBean));
        formationBean.setFormationPoursuiteBeans(formationJointuresDAO.selectFormationsPoursuites(formationBean));
        formationBean.setParcoursFicheBeans(formationJointuresDAO.selectParcoursDiplomant(formationBean));
        formationBean.setParcoursPoursuiteBeans(formationJointuresDAO.selectParcoursPoursuites(formationBean));
    }

    public String chooseMyProgram(final String id) {
        if (StringUtils.isNotBlank(id)) {
            String query = "SELECT ACTIVER_CONSTRUIRE_PARCOURS FROM FORMATION WHERE ID_FORMATION = :idFormation";
            return this.namedParameterJdbcTemplate.queryForObject(query, new MapSqlParameterSource("idFormation", id), String.class);
        } else {
            throw new DataIntegrityViolationException("L'id ne peut pas être vide ou null");
        }
    }
}

