/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.OffreFormationTypeLien;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.univ.objetspartages.om.EtatFiche;

/**
 * Created by alexandre.baillif on 06/04/17.
 */
public class FormationJointuresDAO {

    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    private RowMapper<FormationBean> jointureFormationLieRowMapper = new RowMapper<FormationBean>() {

        @Override
        public FormationBean mapRow(ResultSet rs, int rowNum) throws SQLException {
            FormationBean myBean = new FormationBean();
            myBean.setCode(rs.getString("CODE_FORMATION_LIEE"));
            myBean.setLangue(rs.getString("LANGUE_FORMATION_LIEE"));
            myBean.setEtatObjet(rs.getString("ETAT_FORMATION_LIEE"));
            return myBean;
        }
    };

    private RowMapper<ParcoursBean> jointureParcoursLieRowMapper = new RowMapper<ParcoursBean>() {

        @Override
        public ParcoursBean mapRow(ResultSet rs, int rowNum) throws SQLException {
            ParcoursBean myBean = new ParcoursBean();
            myBean.setCode(rs.getString("CODE_PARCOURS_LIE"));
            myBean.setLangue(rs.getString("LANGUE_PARCOURS_LIE"));
            myBean.setEtatObjet(rs.getString("ETAT_PARCOURS_LIE"));
            return myBean;
        }
    };

    public void setDataSource(DataSource dataSource) {
        this.namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
    }

    public void deleteFormationToFormationsLiens(FormationBean formation) {
        if (formation != null && formation.getId() != null && formation.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM FORMATION_TO_FORMATIONS WHERE ID_FORMATION = :idFormation";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idFormation", formation.getId()));
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public void saveFormationsLiees(FormationBean formationBean) {
        if (formationBean == null) {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter un bean null");
        }
        saveFormationsToFormationsLiens(formationBean, formationBean.getFormationsLieesBeans(), OffreFormationTypeLien.FORMATION_LIEE.name());
    }

    public void saveFormationsPoursuite(FormationBean formationBean) {
        if (formationBean == null) {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter un bean null");
        }
        saveFormationsToFormationsLiens(formationBean, formationBean.getFormationPoursuiteBeans(), OffreFormationTypeLien.FORMATION_POURSUITE.name());
    }

    public void saveFormationsToFormationsLiens(FormationBean formationBean, List<FormationBean> formations, String typeLien) {
        if (formationBean != null && formationBean.getId() != null && formationBean.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM FORMATION_TO_FORMATIONS WHERE ID_FORMATION = :idFormation AND TYPE_LIEN = :typeLien";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idFormation", formationBean.getId()).addValue("typeLien", typeLien));
            String insertAfterDeleteQuery = "INSERT INTO FORMATION_TO_FORMATIONS(ID_FORMATION,CODE_FORMATION,LANGUE_FORMATION,ETAT_FORMATION,CODE_FORMATION_LIEE,LANGUE_FORMATION_LIEE,ETAT_FORMATION_LIEE,TYPE_LIEN) VALUES(:idFormation,:codeFormation,:langueFormation,:etatFormation,:codeFormationLiee,:langueFormationLiee,:etatFormationLiee,:typeLien)";
            for (FormationBean formationRequise : formations) {
                MapSqlParameterSource params = new MapSqlParameterSource("codeFormation", formationBean.getCode());
                params.addValue("idFormation", formationBean.getId());
                params.addValue("langueFormation", formationBean.getLangue());
                params.addValue("etatFormation", formationBean.getEtatObjet());
                params.addValue("codeFormationLiee", formationRequise.getCode());
                params.addValue("langueFormationLiee", formationRequise.getLangue());
                params.addValue("etatFormationLiee", StringUtils.defaultIfBlank(formationRequise.getEtatObjet(), EtatFiche.EN_LIGNE.getEtat()));
                params.addValue("typeLien", typeLien);
                this.namedParameterJdbcTemplate.update(insertAfterDeleteQuery, params);
            }
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public void saveParcoursDiplomant(FormationBean formationBean) {
        if (formationBean == null) {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter un bean null");
        }
        saveFormationToParcoursLiens(formationBean, formationBean.getParcoursFicheBeans(), OffreFormationTypeLien.PARCOURS_DIPLOMANT.name());
    }

    public void saveParcoursPoursuite(FormationBean formationBean) {
        if (formationBean == null) {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter un bean null");
        }
        saveFormationToParcoursLiens(formationBean, formationBean.getParcoursPoursuiteBeans(), OffreFormationTypeLien.PARCOURS_POURSUITE.name());
    }

    protected void saveFormationToParcoursLiens(FormationBean formation, List<ParcoursBean> parcours, String typeLien) {
        if (formation != null && formation.getId() != null && formation.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM FORMATION_TO_PARCOURS WHERE ID_FORMATION = :idFormation AND TYPE_LIEN = :typeLien";
            String insertAfterDeleteQuery = "INSERT INTO FORMATION_TO_PARCOURS(ID_FORMATION,CODE_FORMATION,LANGUE_FORMATION,ETAT_FORMATION,CODE_PARCOURS_LIE,LANGUE_PARCOURS_LIE,ETAT_PARCOURS_LIE,TYPE_LIEN) VALUES(:idFormation,:codeFormation,:langueFormation,:etatFormation,:codeParcoursLie,:langueParcoursLie,:etatParcoursLie,:typeLien)";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idFormation", formation.getId()).addValue("typeLien", typeLien));
            for (ParcoursBean parcoursDiplomant : parcours) {
                MapSqlParameterSource params = new MapSqlParameterSource("codeFormation", formation.getCode());
                params.addValue("idFormation", formation.getId());
                params.addValue("langueFormation", formation.getLangue());
                params.addValue("etatFormation", formation.getEtatObjet());
                params.addValue("codeParcoursLie", parcoursDiplomant.getCode());
                params.addValue("langueParcoursLie", parcoursDiplomant.getLangue());
                params.addValue("etatParcoursLie", StringUtils.defaultIfBlank(parcoursDiplomant.getEtatObjet(), EtatFiche.EN_LIGNE.getEtat()));
                params.addValue("typeLien", typeLien);
                this.namedParameterJdbcTemplate.update(insertAfterDeleteQuery, params);
            }
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public void deleteFormationToParcoursLiens(FormationBean formation) {
        if (formation != null && formation.getId() != null && formation.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM FORMATION_TO_PARCOURS WHERE ID_FORMATION = :idFormation";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idFormation", formation.getId()));
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    protected List<FormationBean> selectFormationsToFormations(FormationBean formationBean, String typeLien) {
        String query = "SELECT * FROM FORMATION_TO_FORMATIONS WHERE CODE_FORMATION = :codeFormation AND LANGUE_FORMATION = :langueFormation AND ETAT_FORMATION = :etatFormation AND TYPE_LIEN = :typeLien ORDER BY ID_LIEN";
        MapSqlParameterSource params = new MapSqlParameterSource("codeFormation", formationBean.getCode());
        params.addValue("langueFormation", formationBean.getLangue());
        params.addValue("etatFormation", formationBean.getEtatObjet());
        params.addValue("typeLien", typeLien);
        return this.namedParameterJdbcTemplate.query(query, params, jointureFormationLieRowMapper);
    }

    public List<FormationBean> selectFormationsLiees(FormationBean formationBean) {
        return selectFormationsToFormations(formationBean, OffreFormationTypeLien.FORMATION_LIEE.name());
    }

    public List<FormationBean> selectFormationsPoursuites(FormationBean formationBean) {
        return selectFormationsToFormations(formationBean, OffreFormationTypeLien.FORMATION_POURSUITE.name());
    }

    protected List<ParcoursBean> selectFormationsToParcours(FormationBean formationBean, String typeLien) {
        String query = "SELECT * FROM FORMATION_TO_PARCOURS WHERE CODE_FORMATION = :codeFormation AND LANGUE_FORMATION = :langueFormation AND ETAT_FORMATION = :etatFormation AND TYPE_LIEN = :typeLien ORDER BY ID_LIEN";
        MapSqlParameterSource params = new MapSqlParameterSource("codeFormation", formationBean.getCode());
        params.addValue("langueFormation", formationBean.getLangue());
        params.addValue("etatFormation", formationBean.getEtatObjet());
        params.addValue("typeLien", typeLien);
        return this.namedParameterJdbcTemplate.query(query, params, jointureParcoursLieRowMapper);
    }

    public List<ParcoursBean> selectParcoursDiplomant(FormationBean formationBean) {
        return selectFormationsToParcours(formationBean, OffreFormationTypeLien.PARCOURS_DIPLOMANT.name());
    }

    public List<ParcoursBean> selectParcoursPoursuites(FormationBean formationBean) {
        return selectFormationsToParcours(formationBean, OffreFormationTypeLien.PARCOURS_POURSUITE.name());
    }
}
