/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.sql.DataSource;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import com.jsbsoft.jtf.datasource.naming.impl.DefaultColumnNamingStategy;
import com.ksup.objetspartages.bean.FormationBean;
import com.univ.objetspartages.dao.mapper.CustomIdBeanPropertyRowMapper;

/**
 * Created by alexandre.baillif on 10/04/17.
 */
public class OffreFormationDAO {

    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    public void setDataSource(DataSource dataSource) {
        this.namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
    }

    public List<String> findCodeParcoursByUeupCode(List<String> codesUeup, String langue) {
        if (CollectionUtils.isEmpty(codesUeup)) {
            return Collections.emptyList();
        }
        String query = "SELECT CODE_PARCOURS FROM PARCOURS_TO_UEUPS WHERE CODE_UEUP_LIE IN (:codesUeup) AND LANGUE_UEUP_LIE = :langue";
        List<String> codesParcours = this.namedParameterJdbcTemplate.queryForList(query, new MapSqlParameterSource("codesUeup", codesUeup).addValue("langue", langue), String.class);
        if (codesParcours.size() > 0) {
            List<String> codesResultat = new ArrayList<>();
            List<String> codesParcoursAval = new ArrayList<>(codesParcours);
            int safetyCount = 7;
            do {
                codesResultat.addAll(codesParcoursAval);
                String queryParcours = "SELECT CODE_PARCOURS_AVAL FROM PARCOURS_TO_PARCOURS WHERE CODE_PARCOURS_AMONT IN (:codesParcours) AND LANGUE_PARCOURS_AMONT = :langue";
                codesParcoursAval = this.namedParameterJdbcTemplate.queryForList(queryParcours, new MapSqlParameterSource("codesParcours", codesParcoursAval).addValue("langue", langue), String.class);
            } while (CollectionUtils.isNotEmpty(codesParcoursAval) && --safetyCount > 0);
            return codesResultat;
        } else {
            return Collections.emptyList();
        }
    }

    public List<FormationBean> findFormationBeansByCodeParcours(List<String> codesParcours, String langue) {
        if (CollectionUtils.isEmpty(codesParcours)) {
            return Collections.emptyList();
        }
        RowMapper<FormationBean> rowMapper = new CustomIdBeanPropertyRowMapper<>(FormationBean.class, new DefaultColumnNamingStategy());
        String query = "SELECT F.* FROM FORMATION F LEFT JOIN FORMATION_TO_PARCOURS FP ON F.CODE = FP.CODE_FORMATION AND F.LANGUE = FP.LANGUE_FORMATION WHERE FP.CODE_PARCOURS_LIE IN (:codesParcours) AND FP.LANGUE_PARCOURS_LIE = :langue";
        return this.namedParameterJdbcTemplate.query(query, new MapSqlParameterSource("codesParcours", codesParcours).addValue("langue", langue), rowMapper);
    }

    public List<FormationBean> findFormationBeansByUeupCodes(List<String> codesUeup, String langue) {
        return findFormationBeansByCodeParcours(findCodeParcoursByUeupCode(codesUeup, langue), langue);
    }

    public List<String> findUeupsByCodeCours(String codeCours, String langue) {
        if (StringUtils.isBlank(codeCours)) {
            return Collections.emptyList();
        }
        String query = "SELECT CODE_UEUP FROM UEUP_TO_COURS WHERE CODE_COURS_LIE = :codeCours AND LANGUE_COURS_LIE = :langue";
        return this.namedParameterJdbcTemplate.queryForList(query, new MapSqlParameterSource("codeCours", codeCours).addValue("langue", langue), String.class);
    }
}
