/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;

/**
 * Classe d'accès aux données pour Parcours.
 */
public class ParcoursDAO extends AbstractFicheDAO<ParcoursBean> {

    private static final Logger LOG = LoggerFactory.getLogger(ParcoursDAO.class);

    private ParcoursJointuresDAO parcoursJointuresDAO;

    public ParcoursDAO() {
        tableName = "PARCOURS";
        orderClause = "ORDER BY T1.INTITULE_LONG ASC";
    }

    public void setParcoursJointuresDAO(ParcoursJointuresDAO parcoursJointuresDAO) {
        this.parcoursJointuresDAO = parcoursJointuresDAO;
    }

    @Override
    public List<ParcoursBean> select(String request) throws DataSourceException {
        List<ParcoursBean> parcoursBeans = super.select(request);
        parcoursBeans.forEach(this::selectJoinedDatas);
        return parcoursBeans;
    }

    @Override
    public List<ParcoursBean> select(String request, SqlParameterSource parameters) {
        List<ParcoursBean> parcoursBeans = super.select(request, parameters);
        parcoursBeans.forEach(this::selectJoinedDatas);
        return parcoursBeans;
    }

    @Override
    public ParcoursBean getById(Long id) throws DataSourceException {
        ParcoursBean parcoursBean = super.getById(id);
        selectJoinedDatas(parcoursBean);
        return parcoursBean;
    }

    @Override
    public List<ParcoursBean> selectCodeLangueEtat(String code, String langue, String etat) throws DataSourceException {
        List<ParcoursBean> parcoursBeans = super.selectCodeLangueEtat(code, langue, etat);
        parcoursBeans.forEach(this::selectJoinedDatas);
        return parcoursBeans;
    }

    protected void selectJoinedDatas(ParcoursBean parcoursBean) {
        if (parcoursBean != null) {
            parcoursBean.setParcoursAmontBeans(parcoursJointuresDAO.selectParcoursAmont(parcoursBean));
            parcoursBean.setParcoursAvalBeans(parcoursJointuresDAO.selectParcoursAval(parcoursBean));
            parcoursBean.setUeupsBeans(parcoursJointuresDAO.selectParcoursUeups(parcoursBean).stream()
                .filter(Objects::nonNull)
                .filter(ueup -> StringUtils.isNotEmpty(ueup.getCode()))
                .collect(Collectors.toList()));
        }
    }
}
