/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.MutablePair;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ParcoursLink;
import com.ksup.objetspartages.bean.UeupBean;
import com.univ.objetspartages.om.EtatFiche;

/**
 * Created by alexandre.baillif on 07/04/17.
 */
public class ParcoursJointuresDAO {

    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    private RowMapper<ParcoursLink> rowMapperAmont = (rs, rowNum) -> {
        ParcoursBean parcoursBean = new ParcoursBean();
        parcoursBean.setCode(rs.getString("CODE_PARCOURS_AMONT"));
        parcoursBean.setLangue(rs.getString("LANGUE_PARCOURS_AMONT"));
        parcoursBean.setEtatObjet(rs.getString("ETAT_PARCOURS_AMONT"));
        String lien = rs.getString("CODE_OPTION_LIEN");
        return new ParcoursLink(parcoursBean, lien);
    };

    private RowMapper<ParcoursLink> rowMapperAval = (rs, rowNum) -> {
        ParcoursBean parcoursBean = new ParcoursBean();
        parcoursBean.setCode(rs.getString("CODE_PARCOURS_AVAL"));
        parcoursBean.setLangue(rs.getString("LANGUE_PARCOURS_AVAL"));
        parcoursBean.setEtatObjet(rs.getString("ETAT_PARCOURS_AVAL"));
        String lien = rs.getString("CODE_OPTION_LIEN");
        return new ParcoursLink(parcoursBean, lien);
    };

    private RowMapper<UeupBean> rowMapperUeup = new RowMapper<UeupBean>() {

        @Override
        public UeupBean mapRow(ResultSet rs, int rowNum) throws SQLException {
            UeupBean ueupBean = new UeupBean();
            ueupBean.setCode(rs.getString("CODE_UEUP_LIE"));
            ueupBean.setLangue(rs.getString("LANGUE_UEUP_LIE"));
            return ueupBean;
        }
    };

    public void setDataSource(DataSource dataSource) {
        this.namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
    }

    public void saveParcoursToParcoursLiens(ParcoursBean parcours) {
        if (parcours != null && parcours.getId() != null && parcours.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM PARCOURS_TO_PARCOURS WHERE (ID_PARCOURS_AMONT = :idParcours) OR (ID_PARCOURS_AVAL = :idParcours)";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idParcours", parcours.getId()));
            String insertAfterDeleteQuery = "INSERT INTO PARCOURS_TO_PARCOURS VALUES(:idParcoursAmont,:codeParcoursAmont,:langueParcoursAmont,:etatParcoursAmont,:idParcoursAval,:codeParcoursAval,:langueParcoursAval,:etatParcoursAval,:codeOptionLien)";
            if(CollectionUtils.isNotEmpty(parcours.getParcoursAmontBeans())) {
                for (Pair<ParcoursBean, String> parcoursAmont : parcours.getParcoursAmontBeans()) {
                    MapSqlParameterSource params = new MapSqlParameterSource("codeParcoursAval", parcours.getCode());
                    params.addValue("idParcoursAval", parcours.getId());
                    params.addValue("langueParcoursAval", parcours.getLangue());
                    params.addValue("etatParcoursAval", parcours.getEtatObjet());
                    params.addValue("idParcoursAmont", parcoursAmont.getLeft().getId());
                    params.addValue("codeParcoursAmont", parcoursAmont.getLeft().getCode());
                    params.addValue("langueParcoursAmont", parcoursAmont.getLeft().getLangue());
                    params.addValue("etatParcoursAmont", StringUtils.defaultIfBlank(parcoursAmont.getLeft().getEtatObjet(), EtatFiche.EN_LIGNE.getEtat()));
                    params.addValue("codeOptionLien", parcoursAmont.getRight());
                    this.namedParameterJdbcTemplate.update(insertAfterDeleteQuery, params);
                }
            }
            if(CollectionUtils.isNotEmpty(parcours.getParcoursAvalBeans())) {
                for (Pair<ParcoursBean, String> parcoursAval : parcours.getParcoursAvalBeans()) {
                    MapSqlParameterSource params = new MapSqlParameterSource("codeParcoursAmont", parcours.getCode());
                    params.addValue("idParcoursAmont", parcours.getId());
                    params.addValue("langueParcoursAmont", parcours.getLangue());
                    params.addValue("etatParcoursAmont", parcours.getEtatObjet());
                    params.addValue("idParcoursAval", parcoursAval.getLeft().getId());
                    params.addValue("codeParcoursAval", parcoursAval.getLeft().getCode());
                    params.addValue("langueParcoursAval", parcoursAval.getLeft().getLangue());
                    params.addValue("etatParcoursAval", StringUtils.defaultIfBlank(parcoursAval.getLeft().getEtatObjet(), EtatFiche.EN_LIGNE.getEtat()));
                    params.addValue("codeOptionLien", parcoursAval.getRight());
                    this.namedParameterJdbcTemplate.update(insertAfterDeleteQuery, params);
                }
            }
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public void deleteParcoursToParcoursLiens(ParcoursBean parcours) {
        if (parcours != null && parcours.getId() != null && parcours.getId() > 0) {
            String deleteFirstQuery = "DELETE FROM PARCOURS_TO_PARCOURS WHERE (ID_PARCOURS_AMONT = :idParcours) OR (ID_PARCOURS_AVAL = :idParcours)";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idParcours", parcours.getId()));
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public List<ParcoursLink> selectParcoursAmont(ParcoursBean parcoursBean) {
        String query = "SELECT * FROM PARCOURS_TO_PARCOURS WHERE CODE_PARCOURS_AVAL = :codeParcours AND LANGUE_PARCOURS_AVAL = :langueParcours AND ETAT_PARCOURS_AVAL = :etatParcours";
        return this.namedParameterJdbcTemplate.query(query, new MapSqlParameterSource("codeParcours", parcoursBean.getCode()).addValue("langueParcours", parcoursBean.getLangue()).addValue("etatParcours", parcoursBean.getEtatObjet()), rowMapperAmont);
    }

    public List<ParcoursLink> selectParcoursAval(ParcoursBean parcoursBean) {
        String query = "SELECT * FROM PARCOURS_TO_PARCOURS WHERE CODE_PARCOURS_AMONT = :codeParcours AND LANGUE_PARCOURS_AMONT = :langueParcours AND ETAT_PARCOURS_AMONT = :etatParcours";
        return this.namedParameterJdbcTemplate.query(query, new MapSqlParameterSource("codeParcours", parcoursBean.getCode()).addValue("langueParcours", parcoursBean.getLangue()).addValue("etatParcours", parcoursBean.getEtatObjet()), rowMapperAval);
    }

    public void saveParcoursToUeups(ParcoursBean parcoursBean) {
        if (parcoursBean != null && parcoursBean.getId() != null && parcoursBean.getId() > 0) {
            String deleteFirstQuery = "DELETE FROM PARCOURS_TO_UEUPS WHERE ID_PARCOURS = :idParcours";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idParcours", parcoursBean.getId()));
            String insertAfterDeleteQuery = "INSERT INTO PARCOURS_TO_UEUPS(ID_PARCOURS,CODE_PARCOURS,LANGUE_PARCOURS,ETAT_PARCOURS,CODE_UEUP_LIE,LANGUE_UEUP_LIE,ETAT_UEUP_LIE) VALUES(:idParcours,:codeParcours,:langueParcours,:etatParcours,:codeUeup,:langueUeup,:etatUeup)";
            if(CollectionUtils.isNotEmpty(parcoursBean.getUeupsBeans())) {
                for (UeupBean ueup : parcoursBean.getUeupsBeans()) {
                    MapSqlParameterSource params = new MapSqlParameterSource("codeParcours", parcoursBean.getCode());
                    params.addValue("idParcours", parcoursBean.getId());
                    params.addValue("langueParcours", parcoursBean.getLangue());
                    params.addValue("etatParcours", parcoursBean.getEtatObjet());
                    params.addValue("codeUeup", ueup.getCode());
                    params.addValue("langueUeup", ueup.getLangue());
                    params.addValue("etatUeup", StringUtils.defaultIfBlank(ueup.getEtatObjet(), EtatFiche.EN_LIGNE.getEtat()));
                    this.namedParameterJdbcTemplate.update(insertAfterDeleteQuery, params);
                }
            }
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public void deleteParcoursToUeups(ParcoursBean parcours) {
        if (parcours != null && parcours.getId() != null && parcours.getId() > 0) {
            String deleteFirstQuery = "DELETE FROM PARCOURS_TO_UEUPS WHERE ID_PARCOURS = :idParcours";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idParcours", parcours.getId()));
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public List<UeupBean> selectParcoursUeups(ParcoursBean parcoursBean) {
        String query = "SELECT * FROM PARCOURS_TO_UEUPS WHERE CODE_PARCOURS = :codeParcours AND LANGUE_PARCOURS = :langueParcours AND ETAT_PARCOURS = :etatParcours ORDER BY ID_LIEN";
        return this.namedParameterJdbcTemplate.query(query, new MapSqlParameterSource("codeParcours", parcoursBean.getCode()).addValue("langueParcours", parcoursBean.getLangue()).addValue("etatParcours", parcoursBean.getEtatObjet()), rowMapperUeup);
    }
}

