/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.ksup.objetspartages.bean.UeupBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;
import com.univ.objetspartages.om.EtatFiche;

/**
 * DAO gérant les fiches cours.
 */
public class UeupDAO extends AbstractFicheDAO<UeupBean> {

    private static final Logger LOG = LoggerFactory.getLogger(UeupDAO.class);

    private UeupJointuresDAO ueupJointuresDAO;

    public UeupDAO() {
        tableName = "UEUP";
        orderClause = "ORDER BY T1.LIBELLE_LONG ASC";
    }

    public void setUeupJointuresDAO(UeupJointuresDAO ueupJointuresDAO) {
        this.ueupJointuresDAO = ueupJointuresDAO;
    }

    public Set<UeupBean> selectByCodesAndLangueAndEtat(Set<String> codes, String langue, String etat) {
        Set<UeupBean> result = new HashSet<>();
        if (CollectionUtils.isNotEmpty(codes)) {
            MapSqlParameterSource params = new MapSqlParameterSource("codes", codes);
            params.addValue("langue", langue);
            params.addValue("etat", StringUtils.defaultIfEmpty(etat, EtatFiche.EN_LIGNE.getEtat()));
            String query = "SELECT * FROM UEUP WHERE CODE in (:codes) AND ETAT_OBJET = :etat AND LANGUE = :langue";
            List<UeupBean> resultats = this.namedParameterJdbcTemplate.query(query, params, rowMapper);
            if (CollectionUtils.isNotEmpty(resultats)) {
                for (UeupBean ueupBean : resultats) {
                    selectJoinedDatas(ueupBean);
                }
                result.addAll(resultats);
            }
        }
        return result;
    }

    @Override
    public List<UeupBean> select(String request) throws DataSourceException {
        List<UeupBean> ueupBeans = super.select(request);
        for (UeupBean ueupBean : ueupBeans) {
            selectJoinedDatas(ueupBean);
        }
        return ueupBeans;
    }

    @Override
    public List<UeupBean> select(String request, SqlParameterSource parameters) {
        List<UeupBean> ueupBeans = super.select(request, parameters);
        for (UeupBean ueupBean : ueupBeans) {
            selectJoinedDatas(ueupBean);
        }
        return ueupBeans;
    }

    @Override
    public UeupBean getById(Long id) throws DataSourceException {
        UeupBean ueupBean = super.getById(id);
        selectJoinedDatas(ueupBean);
        return ueupBean;
    }

    protected void selectJoinedDatas(UeupBean ueupBean) {
        ueupBean.setUeupsRequisesBeans(ueupJointuresDAO.selectUeupsRequises(ueupBean));
        ueupBean.setUeupfilsBeans(ueupJointuresDAO.selectUeupsFils(ueupBean));
        ueupBean.setCoursBeans(ueupJointuresDAO.selectUeupToCours(ueupBean));
    }
}
