/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.dao.impl;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.sql.DataSource;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import com.ksup.objetspartages.bean.CoursBean;
import com.ksup.objetspartages.bean.OffreFormationTypeLien;
import com.ksup.objetspartages.bean.UeupBean;
import com.univ.objetspartages.om.EtatFiche;

/**
 * Created by alexandre.baillif on 07/04/17.
 */
public class UeupJointuresDAO {

    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    private RowMapper<CoursBean> rowMapperCours = new RowMapper<CoursBean>() {

        @Override
        public CoursBean mapRow(ResultSet rs, int rowNum) throws SQLException {
            CoursBean coursBean = new CoursBean();
            coursBean.setCode(rs.getString("CODE_COURS_LIE"));
            coursBean.setLangue(rs.getString("LANGUE_COURS_LIE"));
            coursBean.setEtatObjet(rs.getString("ETAT_COURS_LIE"));
            return coursBean;
        }
    };

    private RowMapper<UeupBean> rowMapperUeup = new RowMapper<UeupBean>() {

        @Override
        public UeupBean mapRow(ResultSet rs, int rowNum) throws SQLException {
            UeupBean ueupBean = new UeupBean();
            ueupBean.setCode(rs.getString("CODE_UEUP_LIE"));
            ueupBean.setLangue(rs.getString("LANGUE_UEUP_LIE"));
            ueupBean.setEtatObjet(rs.getString("ETAT_UEUP_LIE"));
            return ueupBean;
        }
    };

    public void setDataSource(DataSource dataSource) {
        this.namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
    }

    public void saveUeupsRequises(UeupBean ueupBean) {
        if (ueupBean == null) {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter un bean null");
        }
        saveUeupsToUeups(ueupBean, ueupBean.getUeupsRequisesBeans(), OffreFormationTypeLien.UEUP_REQUISE.name());
    }

    public void saveUeupFils(UeupBean ueupBean) {
        if (ueupBean == null) {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter un bean null");
        }
        saveUeupsToUeups(ueupBean, ueupBean.getUeupfilsBeans(), OffreFormationTypeLien.UEUP_FILS.name());
    }

    protected void saveUeupsToUeups(UeupBean ueupBean, List<UeupBean> ueupsBeansLies, String typeLien) {
        if (ueupBean != null && ueupBean.getId() != null && ueupBean.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM UEUP_TO_UEUPS WHERE ID_UEUP = :idUeup AND TYPE_LIEN = :typeLien";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idUeup", ueupBean.getId()).addValue("typeLien", typeLien));
            String insertAfterDeleteQuery = "INSERT INTO UEUP_TO_UEUPS(ID_UEUP,CODE_UEUP,LANGUE_UEUP,ETAT_UEUP,CODE_UEUP_LIE,LANGUE_UEUP_LIE,ETAT_UEUP_LIE,TYPE_LIEN) VALUES(:idUeup,:codeUeup,:langueUeup,:etatUeup,:codeUeupLie,:langueUeupLie,:etatUeupLie,:typeLien)";
            for (UeupBean ueup : ueupsBeansLies) {
                MapSqlParameterSource params = new MapSqlParameterSource("codeUeup", ueupBean.getCode());
                params.addValue("idUeup", ueupBean.getId());
                params.addValue("langueUeup", ueupBean.getLangue());
                params.addValue("etatUeup", ueupBean.getEtatObjet());
                params.addValue("codeUeupLie", ueup.getCode());
                params.addValue("langueUeupLie", ueup.getLangue());
                params.addValue("etatUeupLie", StringUtils.defaultIfBlank(ueup.getEtatObjet(), EtatFiche.EN_LIGNE.getEtat()));
                params.addValue("typeLien", typeLien);
                this.namedParameterJdbcTemplate.update(insertAfterDeleteQuery, params);
            }
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public void deleteUeupToUeups(UeupBean ueupBean) {
        if (ueupBean != null && ueupBean.getId() != null && ueupBean.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM UEUP_TO_UEUPS WHERE ID_UEUP = :idUeup";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idUeup", ueupBean.getId()));
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public List<UeupBean> selectUeupsRequises(UeupBean ueupBean) {
        return selectUeupToUeups(ueupBean, OffreFormationTypeLien.UEUP_REQUISE.name());
    }

    public List<UeupBean> selectUeupsFils(UeupBean ueupBean) {
        return selectUeupToUeups(ueupBean, OffreFormationTypeLien.UEUP_FILS.name());
    }

    protected List<UeupBean> selectUeupToUeups(UeupBean ueupBean, String typeLien) {
        String query = "SELECT * FROM UEUP_TO_UEUPS WHERE CODE_UEUP = :codeUeup AND LANGUE_UEUP = :langueUeup AND ETAT_UEUP = :etatUeup AND TYPE_LIEN = :typeLien ORDER BY ID_LIEN";
        return this.namedParameterJdbcTemplate.query(query, new MapSqlParameterSource("codeUeup", ueupBean.getCode()).addValue("langueUeup", ueupBean.getLangue()).addValue("etatUeup", ueupBean.getEtatObjet()).addValue("typeLien", typeLien), rowMapperUeup);
    }

    public void saveUeupsToCours(UeupBean ueupBean) {
        if (ueupBean != null && ueupBean.getId() != null && ueupBean.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM UEUP_TO_COURS WHERE  ID_UEUP = :idUeup";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idUeup", ueupBean.getId()));
            String insertAfterDeleteQuery = "INSERT INTO UEUP_TO_COURS(ID_UEUP,CODE_UEUP,LANGUE_UEUP,ETAT_UEUP,CODE_COURS_LIE,LANGUE_COURS_LIE,ETAT_COURS_LIE) VALUES(:idUeup,:codeUeup,:langueUeup,:etatUeup,:codeCoursLie,:langueCoursLie,:etatCoursLie)";
            for (CoursBean coursBean : ueupBean.getCoursBeans()) {
                MapSqlParameterSource params = new MapSqlParameterSource("codeUeup", ueupBean.getCode());
                params.addValue("idUeup", ueupBean.getId());
                params.addValue("langueUeup", ueupBean.getLangue());
                params.addValue("etatUeup", ueupBean.getEtatObjet());
                params.addValue("codeCoursLie", coursBean.getCode());
                params.addValue("langueCoursLie", coursBean.getLangue());
                params.addValue("etatCoursLie", StringUtils.defaultIfBlank(coursBean.getEtatObjet(), EtatFiche.EN_LIGNE.getEtat()));
                this.namedParameterJdbcTemplate.update(insertAfterDeleteQuery, params);
            }
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    public void deleteUeupToCours(UeupBean ueupBean) {
        if (ueupBean != null && ueupBean.getId() != null && ueupBean.getId() > 0L) {
            String deleteFirstQuery = "DELETE FROM UEUP_TO_COURS  WHERE  ID_UEUP = :idUeup";
            this.namedParameterJdbcTemplate.update(deleteFirstQuery, new MapSqlParameterSource("idUeup", ueupBean.getId()));
        } else {
            throw new DataIntegrityViolationException("Il n'est pas possible d'ajouter des jointures sur un bean qui n'a pas d'id");
        }
    }

    protected List<CoursBean> selectUeupToCours(UeupBean ueupBean) {
        String query = "SELECT * FROM UEUP_TO_COURS WHERE CODE_UEUP = :codeUeup AND LANGUE_UEUP = :langueUeup AND ETAT_UEUP = :etatUeup ORDER BY ID_LIEN";
        return this.namedParameterJdbcTemplate.query(query, new MapSqlParameterSource("codeUeup", ueupBean.getCode()).addValue("langueUeup", ueupBean.getLangue()).addValue("etatUeup", ueupBean.getEtatObjet()), rowMapperCours);
    }

    public List<String> findCodesUeupParents(String codeUeup, String langue) {
        //Cette méthode n'est généralement utilisée que en Front Office, de plus les liens entre fiches, se font généralement sur des fiches en ligne
        String etatObjet = EtatFiche.EN_LIGNE.getEtat();
        String query = "SELECT CODE_UEUP FROM UEUP_TO_UEUPS WHERE CODE_UEUP_LIE = :codeUeup AND LANGUE_UEUP_LIE = :langueUeup AND ETAT_UEUP_LIE = :etatUeup AND TYPE_LIEN = :typeLien";
        List<String> parents = this.namedParameterJdbcTemplate.queryForList(query, new MapSqlParameterSource("codeUeup", codeUeup).addValue("langueUeup", langue).addValue("etatUeup", etatObjet).addValue("typeLien", OffreFormationTypeLien.UEUP_FILS.name()), String.class);
        if (parents.size() > 0) {
            Collection<String> ancestors = new ArrayList<>(parents);
            List<String> resultat = new ArrayList<>();
            do {
                resultat.addAll(ancestors);
                String queryAncestors = "SELECT CODE_UEUP FROM UEUP_TO_UEUPS WHERE CODE_UEUP_LIE IN (:codesUeup) AND LANGUE_UEUP_LIE = :langueUeup AND ETAT_UEUP_LIE = :etatUeup AND TYPE_LIEN = :typeLien";
                ancestors = this.namedParameterJdbcTemplate.queryForList(queryAncestors, new MapSqlParameterSource("codesUeup", ancestors).addValue("langueUeup", langue).addValue("etatUeup", etatObjet).addValue("typeLien", OffreFormationTypeLien.UEUP_FILS.name()), String.class);
                //On enleve les ancetres dejà présent
                ancestors = CollectionUtils.subtract(ancestors, resultat);
            } while (ancestors.size() > 0);
            return resultat;
        } else {
            return Collections.emptyList();
        }
    }

    public List<String> findAllChildsCode(String codeUeup, String langue) {
        //Cette méthode n'est généralement utilisée que en Front Office, de plus les liens entre fiches, se font généralement sur des fiches en ligne
        String etatObjet = EtatFiche.EN_LIGNE.getEtat();
        String query = "SELECT CODE_UEUP_LIE FROM UEUP_TO_UEUPS WHERE CODE_UEUP = :codeUeup AND LANGUE_UEUP = :langueUeup AND ETAT_UEUP = :etatUeup AND TYPE_LIEN = :typeLien";
        List<String> firstChilds = this.namedParameterJdbcTemplate.queryForList(query, new MapSqlParameterSource("codeUeup", codeUeup).addValue("langueUeup", langue).addValue("etatUeup", etatObjet).addValue("typeLien", OffreFormationTypeLien.UEUP_FILS.name()), String.class);
        if (firstChilds.size() > 0) {
            Collection<String> currentChilds = new ArrayList<>(firstChilds);
            List<String> resultat = new ArrayList<>();
            do {
                resultat.addAll(currentChilds);
                String queryAncestors = "SELECT CODE_UEUP_LIE FROM UEUP_TO_UEUPS WHERE CODE_UEUP IN (:codesUeup) AND LANGUE_UEUP = :langueUeup AND ETAT_UEUP = :etatUeup AND TYPE_LIEN = :typeLien";
                currentChilds = this.namedParameterJdbcTemplate.queryForList(queryAncestors, new MapSqlParameterSource("codesUeup", currentChilds).addValue("langueUeup", langue).addValue("etatUeup", etatObjet).addValue("typeLien", OffreFormationTypeLien.UEUP_FILS.name()), String.class);
                //On enleve les ancetres dejà présent
                currentChilds = CollectionUtils.subtract(currentChilds, resultat);
            } while (currentChilds.size() > 0);
            return resultat;
        } else {
            return Collections.emptyList();
        }
    }
}
