/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.jsputils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.apache.commons.lang3.NotImplementedException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kosmos.search.index.mapper.serializer.MetierSerializer;
import com.kosmos.service.ServiceBean;
import com.kportal.ihm.utils.ClasseFichier;
import com.kportal.ihm.utils.FrontUtil;
import com.ksup.objetspartages.bean.ImageViewBean;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.bean.ResponseRechercheParcoursBean;
import com.ksup.objetspartages.jspbeans.FormationJspBean;
import com.ksup.objetspartages.jspbeans.FormationViewBean;
import com.ksup.objetspartages.jspbeans.sub.CallToActionJspBean;
import com.ksup.objetspartages.jspbeans.sub.FichierJspBean;
import com.ksup.objetspartages.jspbeans.sub.FormationAdmissionJspBean;
import com.ksup.objetspartages.jspbeans.sub.FormationEtApresJspBean;
import com.ksup.objetspartages.jspbeans.sub.FormationInscriptionsJspBean;
import com.ksup.objetspartages.jspbeans.sub.FormationListParcoursJspBean;
import com.ksup.objetspartages.jspbeans.sub.FormationPoursuiteJspBean;
import com.ksup.objetspartages.jspbeans.sub.FormationPresentationJspBean;
import com.ksup.objetspartages.jspbeans.sub.FormationProgrammeJspBean;
import com.ksup.objetspartages.jspbeans.sub.ParcoursDetailViewBean;
import com.ksup.objetspartages.jspbeans.sub.ParcoursPoursuiteJspBean;
import com.ksup.objetspartages.jspbeans.sub.ParcoursViewBean;
import com.ksup.objetspartages.jspbeans.sub.ProgrammeJspBean;
import com.ksup.objetspartages.jspbeans.sub.TemoignageJspBean;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.ksup.objetspartages.om.Parcours;
import com.ksup.objetspartages.om.Ueup;
import com.ksup.objetspartages.service.ServiceRechercheParcours;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.RessourceUtils;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.URLResolver;

public class ServiceFormationViewBuilder implements ServiceBean<FormationJspBean> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ServiceFormationViewBuilder.class);

    private ServiceRessource serviceRessource;

    private ServiceRechercheParcours serviceRechercheParcours;

    private ServiceLabel serviceLabel;

    private UrlProvider urlProvider;

    public ServiceFormationViewBuilder() {
    }

    public FormationJspBean process() {
        ContexteUniv ctx = ContexteUtil.getContexteUniv();
        Formation formation = (Formation) ctx.getFicheCourante();
        FormationJspBean bean = new FormationJspBean();
        if (formation != null) {
            bean.setAccroche(formation.getAccroche());
            bean.setCode(formation.getCode());
            bean.setId(formation.getIdFiche());
            bean.setLangue(formation.getLangue());
            bean.setUrlMonParcours(OffreFormationUtil.urlParcours(formation.getIdFiche()));
            bean.setCallToAction(processCallToActions(formation));
            bean.setFormationPresentation(processPresentation(formation));
            bean.setFormationAdmission(processAdmission(formation));
            bean.setFormationProgramme(processProgramme(formation));
            bean.setFormationEtApres(processEtApres(formation));
            bean.setFormationInscriptions(processInscriptions(formation));
            bean.setFormationListParcours(processFormationParcours(formation));
            bean.setFormationContinue("1".equals(formation.getFormationContinue()));
            bean.setTypeFormationContinue(processTypeFormationContinue(formation.getTypeFormationContinue(), ctx));
        }
        processAffichageOnglets(bean);
        ctx.getRequeteHTTP().setAttribute("formationBean", bean);
        return bean;
    }

    /**
     * Génère un {@link ImageViewBean}.
     * @param fiche la fiche contenant l'image
     * @param index l'index de l'image dans la fiche
     * @param defaultImage l'url de l'image par défaut
     * @return le bean
     */
    public ImageViewBean generateImageViewBean(FicheUniv fiche, String index, String defaultImage) {
        ImageViewBean imageViewBean = new ImageViewBean();
        imageViewBean.setRessourceBean(serviceRessource.getFile(fiche, index));
        if (imageViewBean.getRessourceBean() == null && StringUtils.isNotEmpty(defaultImage)) {
            imageViewBean.setBackupUrl(URLResolver.getAbsoluteUrl(defaultImage));
        }
        return imageViewBean;
    }

    /**
     *  Le bean d'affichage.
     * @param bean
     *
     * @throws NullPointerException
     * If bean is Null
     */
    protected void processAffichageOnglets(FormationJspBean bean) {
        bean.setAfficherPresentation(hasOngletPresentation(bean));
        bean.setAfficherAdmission(hasOngletAdmission(bean));
        bean.setAfficherProgramme(hasOngletProgramme(bean));
        bean.setAfficherEtApres(hasOngletEtApres(bean));
        bean.setAfficherInscriptions(hasOngletInscription(bean));
    }

    protected boolean hasOngletPresentation(FormationJspBean bean) {
        if (bean == null || bean.getFormationPresentation() == null) {
            return false;
        }
        FormationPresentationJspBean presentationJspBean = bean.getFormationPresentation();
        return !presentationJspBean.getImageViewBean().isBeanEmpty() || FrontUtil.isAnyNotBlank(presentationJspBean.getResume(), presentationJspBean.getDescription(), presentationJspBean.getSpecificites(), presentationJspBean.getLieux(), presentationJspBean.getLieuxComplement(), presentationJspBean.getResponsablesComplement(), presentationJspBean.getLaboratoiresPartenaires(), presentationJspBean.getEtablissementsPartenaires(), presentationJspBean.getEntreprisesPartenaires(), presentationJspBean.getConventionEtablissements()) || presentationJspBean.hasTemoignages() || presentationJspBean.hasResponsablesFiche();
    }

    protected boolean hasOngletAdmission(FormationJspBean bean) {
        if (bean == null || bean.getFormationAdmission() == null) {
            return false;
        }
        FormationAdmissionJspBean admissionJspBean = bean.getFormationAdmission();
        return !admissionJspBean.getImageViewBean().isBeanEmpty() || FrontUtil.isAnyNotBlank(admissionJspBean.getNiveauxRecrutements(), admissionJspBean.getFormationsRequises(), admissionJspBean.getPublicCible(), admissionJspBean.getModalitesCandidatures(), admissionJspBean.getModalitesCandidaturesSpecifiques(), admissionJspBean.getConditionsAdmission()) || admissionJspBean.hasFormationLiees();
    }

    protected boolean hasOngletProgramme(FormationJspBean bean) {
        if (bean == null || bean.getFormationProgramme() == null) {
            return false;
        }
        FormationProgrammeJspBean programmeJspBean = bean.getFormationProgramme();
        return !programmeJspBean.getImageViewBean().isBeanEmpty() || FrontUtil.isAnyNotBlank(programmeJspBean.getProgramme(), programmeJspBean.getContenuFormation(), programmeJspBean.getRythmePremiereAnnee(), programmeJspBean.getEquipePedagogique(), programmeJspBean.getStagesDescription(), programmeJspBean.getSejoursEtrangerDescription(), programmeJspBean.getProjetsTutoresComplement(), programmeJspBean.getTempsPleinDescription(), programmeJspBean.getTempsAmenageDescription(), programmeJspBean.getCompatibleProfessionnelleDescription(), programmeJspBean.getAlternanceDescription(), programmeJspBean.getModalitesEvaluation()) || (StringUtils.isNotBlank(programmeJspBean.getPresentielDescription()) && programmeJspBean.isPresentiel()) || (StringUtils.isNotBlank(programmeJspBean.getPartiellementDistanceDescription()) && programmeJspBean.isPartiellementDistance()) || (StringUtils.isNotBlank(programmeJspBean.getTotalementDistanceDescription()) && programmeJspBean.isTotalementDistance()) || (bean.getFormationListParcours() != null && bean.getFormationListParcours().hasParcoursDiplomant() && programmeJspBean.isProgrammeParcoursAffiche()) || programmeJspBean.isConstruireParcoursActive();
    }

    protected boolean hasOngletEtApres(FormationJspBean bean) {
        if (bean == null || bean.getFormationEtApres() == null) {
            return false;
        }
        FormationEtApresJspBean etApresJspBean = bean.getFormationEtApres();
        return !etApresJspBean.getImageViewBean().isBeanEmpty() || FrontUtil.isAnyNotBlank(etApresJspBean.getAnnePostbac(), etApresJspBean.getNiveauSortie(), etApresJspBean.getCompetencesAttestees(), etApresJspBean.getUrlFicheRNCP(), etApresJspBean.getPoursuiteEtudes(), etApresJspBean.getDebouches(), etApresJspBean.getInsertionProfessionnelle()) || etApresJspBean.hasParcoursPoursuite() || etApresJspBean.hasFormationsPoursuites() || etApresJspBean.hasMetiers();
    }

    protected boolean hasOngletInscription(FormationJspBean bean) {
        if (bean == null || bean.getFormationInscriptions() == null) {
            return false;
        }
        FormationInscriptionsJspBean inscriptionsJspBean = bean.getFormationInscriptions();
        return !inscriptionsJspBean.getImageViewBean().isBeanEmpty() || FrontUtil.isAnyNotBlank(inscriptionsJspBean.getTarifComplement(), inscriptionsJspBean.getModalitesInscriptions());
    }

    protected FormationInscriptionsJspBean processInscriptions(Formation formation) {
        FormationInscriptionsJspBean formationInscriptions = new FormationInscriptionsJspBean();
        formationInscriptions.setImageViewBean(generateImageViewBean(formation, "5", getProperty("fiche.FORMATION.images.inscriptions")));
        formationInscriptions.setTarifComplement(formation.getFormatedTarifComplement());
        formationInscriptions.setModalitesInscriptions(formation.getFormatedModalitesInscription());
        return formationInscriptions;
    }

    protected List<ProgrammeJspBean> toProgramme(List<Ueup> ueups) {
        if (ueups.isEmpty()) {
            //Condition sortante, on évite d'aller plus loin
            return Collections.emptyList();
        }
        List<ProgrammeJspBean> programmeJspBeans = new ArrayList<>();
        for (Ueup ueup : ueups) {
            ProgrammeJspBean programmeJspBean = new ProgrammeJspBean();
            programmeJspBean.setTitre(ueup.getLibelleLong());
            programmeJspBean.setUrl(urlProvider.provideAbsoluteUrl(ueup.getPersistanceBean()));
            programmeJspBeans.add(programmeJspBean);
        }
        return programmeJspBeans;
    }

    protected FormationEtApresJspBean processEtApres(Formation formation) {
        FormationEtApresJspBean formationEtApres = new FormationEtApresJspBean();
        formationEtApres.setImageViewBean(generateImageViewBean(formation, "4", getProperty("fiche.FORMATION.images.etapres")));
        formationEtApres.setAnnePostbac(formation.getLibelleAnneePostbac().replaceAll(";", "<br />"));
        formationEtApres.setNiveauSortie(formation.getLibelleNiveauSortie().replaceAll(";", ", "));
        formationEtApres.setCompetencesAttestees(formation.getFormatedCompetencesAttestees());
        formationEtApres.setUrlFicheRNCP(formation.getFormatedFicheRncp());
        formationEtApres.setPoursuiteEtudes(formation.getFormatedPoursuiteEtudes());
        try {
            formationEtApres.setParcoursPoursuites(processParcoursPoursuites(formation.getFichesParcoursPoursuite()));
        } catch (Exception e) {
            LOGGER.error("Erreur dans la récupération des fiches parcours poursuites", e);
            formationEtApres.setParcoursPoursuites(Collections.<ParcoursPoursuiteJspBean>emptyList());
        }
        try {
            formationEtApres.setFormationsPoursuites(processFormationsPoursuites(formation.getFichesFormationPoursuite()));
        } catch (Exception e) {
            LOGGER.error("Erreur dans la récupération des fiches formations poursuites", e);
            formationEtApres.setFormationsPoursuites(Collections.<FormationPoursuiteJspBean>emptyList());
        }
        formationEtApres.setDebouches(formation.getFormatedDebouches());
        formationEtApres.setMetiers(serviceLabel.getFromCodes(formation.getMetiers(), formation.getLangue(), ";", MetierSerializer.TYPE_LIBELLE_METIER));
        formationEtApres.setInsertionProfessionnelle(formation.getFormatedInsertionProfessionnelle());
        formationEtApres.setSecteurActivites(LabelUtils.getLibelleList(OffreFormationTypeLibelle.SECTEUR_ACTIVITE.getValeur(), formation.getSecteursActivites(), formation.getLangue()));
        return formationEtApres;
    }

    protected List<FormationPoursuiteJspBean> processFormationsPoursuites(final List<Formation> fichesFormationPoursuite) {
        if (fichesFormationPoursuite.isEmpty()) {
            return Collections.emptyList();
        }
        List<FormationPoursuiteJspBean> formationsPoursuites = new ArrayList<>();
        for (Formation fiche : fichesFormationPoursuite) {
            FormationPoursuiteJspBean formationPoursuite = new FormationPoursuiteJspBean();
            formationPoursuite.setLibelle(fiche.getLibelleAffichable());
            formationPoursuite.setUrl(urlProvider.provideAbsoluteUrl(fiche.getPersistanceBean()));
            formationsPoursuites.add(formationPoursuite);
        }
        return formationsPoursuites;
    }

    protected List<ParcoursPoursuiteJspBean> processParcoursPoursuites(final List<Parcours> fichesParcoursPoursuites) {
        if (fichesParcoursPoursuites.isEmpty()) {
            return Collections.emptyList();
        }
        List<ParcoursPoursuiteJspBean> parcoursPoursuites = new ArrayList<>();
        for (Parcours fiche : fichesParcoursPoursuites) {
            ParcoursPoursuiteJspBean parcoursPoursuite = new ParcoursPoursuiteJspBean();
            parcoursPoursuite.setUrl(urlProvider.provideAbsoluteUrl(fiche.getPersistanceBean()));
            parcoursPoursuite.setSurtitre(OffreFormationUtil.getMessage("FO.LIBELLE_PARCOURS") + " " + OffreFormationUtil.buildLibelleCourtAnneeSemestreParcours(fiche.getPersistenceBean()));
            parcoursPoursuite.setTitre(fiche.getLibelleAffichable());
            parcoursPoursuite.setCorps(fiche.getFormatedPresentation());
            parcoursPoursuite.setProgramme(toProgramme(fiche.getFicheUeups()));
            parcoursPoursuites.add(parcoursPoursuite);
        }
        return parcoursPoursuites;
    }

    protected FormationProgrammeJspBean processProgramme(Formation formation) {
        FormationProgrammeJspBean formationProgramme = new FormationProgrammeJspBean();
        formationProgramme.setImageViewBean(generateImageViewBean(formation, "3", getProperty("fiche.FORMATION.images.programme")));
        formationProgramme.setProgramme(formation.getFormatedProgramme());
        formationProgramme.setContenuFormation(formation.getFormatedContenuFormation());
        formationProgramme.setRythmePremiereAnnee(formation.getFormatedRythmePremiereAnnee());
        formationProgramme.setEquipePedagogique(formation.getFormatedEquipePedagogique());
        formationProgramme.setStagesDescription(formation.getFormatedStagesDescription());
        formationProgramme.setSejoursEtrangerDescription(formation.getFormatedSejoursEtrangerDescription());
        formationProgramme.setDomaineErasmus(formation.getLibelleDomaineErasmus().replaceAll(";", ", "));
        formationProgramme.setProjetsTutoresComplement(formation.getFormatedProjetsTutoresComplement());
        formationProgramme.setTempsPleinDescription(formation.getFormatedTempsPleinDescription());
        formationProgramme.setTempsAmenageDescription(formation.getFormatedTempsAmenageDescription());
        formationProgramme.setCompatibleProfessionnelleDescription(formation.getFormatedCompatibleProfessionnelleDescription());
        formationProgramme.setAlternanceDescription(formation.getFormatedAlternanceDescription());
        formationProgramme.setModalitesEvaluation(formation.getFormatedModalitesEvaluation());
        formationProgramme.setPresentiel("1".equals(formation.getPresentiel()));
        formationProgramme.setPresentielDescription(formation.getFormatedPresentielDescription());
        formationProgramme.setPartiellementDistance("1".equals(formation.getPartiellementDistance()));
        formationProgramme.setPartiellementDistanceDescription(formation.getFormatedPartiellementDistanceDescription());
        formationProgramme.setTotalementDistance("1".equals(formation.getTotalementDistance()));
        formationProgramme.setTotalementDistanceDescription(formation.getFormatedTotalementDistanceDescription());
        formationProgramme.setProgrammeParcoursAffiche("1".equals(formation.getAfficherProgramme()));
        formationProgramme.setConstruireParcoursActive("1".equals(formation.getActiverConstruireParcours()));
        return formationProgramme;
    }

    protected FormationAdmissionJspBean processAdmission(Formation formation) {
        FormationAdmissionJspBean formationAdmission = new FormationAdmissionJspBean();
        formationAdmission.setImageViewBean(generateImageViewBean(formation, "2", getProperty("fiche.FORMATION.images.admission")));
        formationAdmission.setNiveauxRecrutements(formation.getLibelleNiveauRecrutement().replaceAll(";", ", "));
        formationAdmission.setFormationsRequises(formation.getFormatedFormationsRequises());
        try {
            formationAdmission.setFormationsLiees(processFormationLiees(formation.getFichesFormationsLieesFiche()));
        } catch (Exception e) {
            LOGGER.error("Erreur dans la récupération des fiches formations liées", e);
            formationAdmission.setFormationsLiees(Collections.emptyList());
        }
        formationAdmission.setPublicCible(formation.getFormatedPublicCible());
        formationAdmission.setModalitesCandidatures(formation.getFormatedModalitesCandidatures());
        formationAdmission.setModalitesCandidaturesSpecifiques(formation.getFormatedModalitesCandidaturesSpecifiques());
        formationAdmission.setConditionsAdmission(formation.getFormatedConditionsAdmission());
        formationAdmission.setSerieBacsPreconises(LabelUtils.getLibelleList(OffreFormationTypeLibelle.SERIE_BAC.getValeur(), formation.getSerieBac(), formation.getLangue()));
        return formationAdmission;
    }

    protected List<FormationViewBean> processFormationLiees(final List<Formation> fichesFormationsLieesFiche) {
        if (fichesFormationsLieesFiche.isEmpty()) {
            return Collections.emptyList();
        }
        List<FormationViewBean> results = new ArrayList<>();
        for (Formation formationLiee : fichesFormationsLieesFiche) {
            FormationViewBean view = new FormationViewBean();
            view.setLibelle(formationLiee.getLibelleAffichable());
            view.setUrl(urlProvider.provideAbsoluteUrl(formationLiee.getPersistanceBean()));
            results.add(view);
        }
        return results;
    }

    protected FormationPresentationJspBean processPresentation(Formation formation) {
        FormationPresentationJspBean formationPresentation = new FormationPresentationJspBean();
        formationPresentation.setImageViewBean(generateImageViewBean(formation, "1", getProperty("fiche.FORMATION.images.presentation")));
        formationPresentation.setResume(formation.getFormatedResume());
        formationPresentation.setDescription(formation.getFormatedDescription());
        formationPresentation.setSpecificites(formation.getFormatedSpecificites());
        formationPresentation.setLieux(formation.getLibelleLieux().replaceAll(";", ", "));
        formationPresentation.setLieuxComplement(formation.getFormatedLieuxComplement());
        formationPresentation.setResponsableFiche(OffreFormationUtil.getResponsablesAvecFonctionEtUrl(formation.getPersistanceBean()));
        formationPresentation.setResponsablesComplement(formation.getFormatedResponsablesComplement());
        try {
            List<TemoignageJspBean> temoignages = new ArrayList<>();
            for (AnnuaireModele fiche : formation.getFichesTemoignages()) {
                TemoignageJspBean temoignage = new TemoignageJspBean();
                String urlTemoignage = urlProvider.provideAbsoluteUrl(fiche.getPersistanceBean());
                if (StringUtils.isNotBlank(urlTemoignage)) {
                    temoignage.setFiche(fiche);
                    temoignage.setPrenom(fiche.getPrenom());
                    temoignage.setNom(fiche.getNom());
                    temoignage.setFormation(StringUtils.EMPTY); // voir fiche.jsp pour le remplissage de ce champ
                    if (!"1".equals(fiche.getListeRouge())) {
                        temoignage.setImageViewBean(generateImageViewBean(fiche, "1", StringUtils.EMPTY));
                    }
                    temoignage.setQuote(StringUtils.EMPTY); // voir fiche.jsp pour le remplissage de ce champ
                    temoignage.setDetails(StringUtils.EMPTY); // voir fiche.jsp pour le remplissage de ce champ
                    temoignage.setLink(urlTemoignage);
                    temoignage.setMore(getMessage("FORMATION.FO.LIRE_LA_SUITE") + "<span class=\"sr-only\"> " + getMessage("FORMATION.FO.CARD.DE_LA_CITATION") + "</span>");
                    temoignages.add(temoignage);
                }
            }
            formationPresentation.setTemoignages(temoignages);
        } catch (Exception e) {
            LOGGER.error("Erreur dans la récupération des témoignages", e);
            formationPresentation.setTemoignages(Collections.<TemoignageJspBean>emptyList());
        }
        formationPresentation.setLaboratoiresPartenaires(formation.getFormatedLaboratoiresPartenaires());
        formationPresentation.setEtablissementsPartenaires(formation.getFormatedEtablissementsPartenaires());
        formationPresentation.setEntreprisesPartenaires(formation.getFormatedEntreprisesPartenaires());
        formationPresentation.setConventionEtablissements(formation.getFormatedConventionEtablissements());
        return formationPresentation;
    }

    public String titreTemoignage(final AnnuaireModele fiche, String nomFormation) {
        StringBuilder titre = new StringBuilder();
        if (StringUtils.isNotBlank(fiche.getPrenom())) {
            titre.append(fiche.getPrenom());
        }
        if (StringUtils.isNotBlank(titre) && StringUtils.isNotBlank(fiche.getNom())) {
            titre.append(" ");
        }
        if (StringUtils.isNotBlank(fiche.getNom())) {
            titre.append(fiche.getNom());
        }
        if (StringUtils.isNotBlank(titre) && StringUtils.isNotBlank(nomFormation)) {
            titre.append(", ");
        }
        if (StringUtils.isNotBlank(nomFormation)) {
            titre.append(nomFormation);
        }
        return titre.toString();
    }

    protected CallToActionJspBean processCallToActions(Formation formation) {
        CallToActionJspBean callToAction = new CallToActionJspBean();
        List<RessourceBean> listeFichiersJoints = serviceRessource.getFiles(formation);
        callToAction.setFichiers(getFichiersJoints(listeFichiersJoints));
        callToAction.setRenseignements(formation.getFormatedRenseignements());
        callToAction.setUrlExterneFormation(formation.getUrlExterneFormation());
        callToAction.setContactEmail(formation.getContactEmail());
        callToAction.setContactTelephone(formation.getContactTelephone());
        callToAction.setContactPlaceholder(String.format(getMessage("FORMATION.FO.__EXEMPLE_QUESTION"), formation.getIntituleCourt()));
        callToAction.setUrlInscription(formation.getUrlInscription());
        return callToAction;
    }

    protected String getMessage(String key) {
        return StringUtils.defaultString(OffreFormationUtil.getMessage(key));
    }

    protected String getProperty(String key) {
        return StringUtils.defaultString(OffreFormationUtil.getProperty(key));
    }

    protected List<FichierJspBean> getFichiersJoints(List<RessourceBean> listeFichiersJoints) {
        List<FichierJspBean> fichiers = new ArrayList<>();
        for (RessourceBean ressourceBean : listeFichiersJoints) {
            FichierJspBean fichier = new FichierJspBean();
            String classeFichier = "defaut";
            String format = RessourceUtils.getFormat(ressourceBean);
            if (ClasseFichier.getInstance().containsKey(format)) {
                classeFichier = ClasseFichier.getInstance().get(format);
            }
            fichier.setClasseFichier(classeFichier);
            fichier.setUrl(URLResolver.getRessourceUrl(RessourceUtils.getUrl(ressourceBean), ContexteUtil.getContexteUniv()));
            fichier.setTitre(StringUtils.defaultIfBlank(RessourceUtils.getTitre(ressourceBean), RessourceUtils.getSource(ressourceBean)));
            String type = classeFichier.toUpperCase();
            String extensionFichier = RessourceUtils.getExtension(ressourceBean);
            if (RessourceUtils.isLocal(ressourceBean) && StringUtils.isNotBlank(extensionFichier)) {
                type = extensionFichier.toUpperCase();
            } else if ("defaut".equals(classeFichier)) {
                type = getMessage("ST_EXTENSION_AUTRE");
            }
            fichier.setType(type);
            fichiers.add(fichier);
            final Long poids = RessourceUtils.getPoids(ressourceBean);
            if (poids != null && poids > 0L) {
                fichier.setPoids(RessourceUtils.getFileSize(ressourceBean));
            }
        }
        return fichiers;
    }

    /**alimentation de la liste de parcours à afficher dans la fiche formation
     *
     *
     */
    protected FormationListParcoursJspBean processFormationParcours(Formation formation) {
        FormationListParcoursJspBean formationParcoursJspBean = new FormationListParcoursJspBean();
        List<ParcoursViewBean> res = new ArrayList<>();
        // Liste des parcours diplomants de la fiche formation
        Set<ParcoursBean> listParcoursDiplomants = serviceRechercheParcours.getParcoursBeanFromFormationBean(formation.getPersistenceBean());
        Comparator<ParcoursDetailViewBean> trieurAffichage = new Comparator<ParcoursDetailViewBean>() {

            @Override
            public int compare(final ParcoursDetailViewBean p1, final ParcoursDetailViewBean p2) {
                //On ne teste pas la nullité des éléments car aucun élément ne doit être null (cela respecte le contrat du TreeSet)
                int result = Integer.compare(p1.getSemestreDebut(), p2.getSemestreDebut());
                if (result == 0) {
                    //Si les numéro de semestre sont les mêmes, on trie par ordre alphabétique
                    //Les titres ne peuvent pas être NULL (cf plus bas)
                    result = p1.getTitre().compareTo(p2.getTitre());
                }
                return result;
            }
        };
        for (ParcoursBean parcoursDiplomant : listParcoursDiplomants) {
            ParcoursViewBean parcoursView = new ParcoursViewBean();
            parcoursView.setParcoursDiplomant(parcoursDiplomant);
            ResponseRechercheParcoursBean respParcoursAmont = serviceRechercheParcours.findTousParcoursForProgramme(parcoursDiplomant);
            // map <semestre, List<ParcoursBean>
            Map<String, Set<ParcoursDetailViewBean>> parcoursSemestres = new TreeMap<>(); // liste de parcours du même semestre
            Set<ParcoursBean> parcoursAmont = respParcoursAmont.getParcours();
            for (ParcoursBean pa : parcoursAmont) {
                String semestre = StringUtils.EMPTY;
                //Semestre par défaut, atterit en début de la map
                if (StringUtils.isNotBlank(pa.getLibelleParcoursSemestre())) {
                    LabelDto labelDto = serviceLabel.getByTypeCodeLanguage(OffreFormationTypeLibelle.PARCOURS_SEMESTRE.getValeur(), pa.getLibelleParcoursSemestre(), pa.getLangue());
                    if (labelDto != null) {
                        semestre = StringUtils.defaultString(labelDto.getLibelle());
                    }
                }
                if (!parcoursSemestres.containsKey(semestre)) {
                    parcoursSemestres.put(semestre, new TreeSet<>(trieurAffichage));
                }
                Parcours currentParcours = new Parcours();
                currentParcours.setPersistenceBean(pa);
                currentParcours.setCtx(ContexteUtil.getContexteUniv());
                ParcoursDetailViewBean parcoursDetail = new ParcoursDetailViewBean();
                parcoursDetail.setSemestreDebut(currentParcours.getNumeroPremierSemestre());
                parcoursDetail.setCode(pa.getCode());
                parcoursDetail.setCorps(currentParcours.getFormatedPresentation());
                parcoursDetail.setSurtitre(OffreFormationUtil.buildLibelleCourtAnneeSemestreParcours(pa));
                String prefixcalcule = OffreFormationUtil.buildLibelleCourtAnneeSemestreParcours(pa);
                String prefix = StringUtils.isNotBlank(prefixcalcule) ? prefixcalcule + " : " : StringUtils.EMPTY;
                parcoursDetail.setTitre(prefix + currentParcours.getIntituleLong());
                parcoursDetail.setUrl(urlProvider.provideAbsoluteUrl(currentParcours.getPersistanceBean()));
                parcoursDetail.setProgramme(toProgramme(currentParcours.getFicheUeups()));
                parcoursSemestres.get(semestre).add(parcoursDetail);
            }
            parcoursView.setListeParcoursAmont(parcoursSemestres);
            // ajout des parcours de la fiche formation
            res.add(parcoursView);
        }
        formationParcoursJspBean.setListeParcoursDiplomant(res);
        return formationParcoursJspBean;
    }

    /**
     * Retourne le type de formation.
     * @param typeFormationContinue le code du type de formation
     * @param ctx
     * @return
     */
    protected String processTypeFormationContinue(final String typeFormationContinue, final ContexteUniv ctx) {
        String typeFormation = StringUtils.EMPTY;
        LabelDto formationContinueLibelle = serviceLabel.getByTypeCodeLanguage(OffreFormationTypeLibelle.TYPE_FORMATION_CONTINUE.getValeur(), typeFormationContinue, ctx.getLangue());
        if(formationContinueLibelle != null){
            typeFormation = formationContinueLibelle.getLibelle();
        }
        return typeFormation;
    }

    public void setServiceRessource(final ServiceRessource serviceRessource) {
        this.serviceRessource = serviceRessource;
    }

    public void setServiceRechercheParcours(final ServiceRechercheParcours serviceRechercheParcours) {
        this.serviceRechercheParcours = serviceRechercheParcours;
    }

    public void setServiceLabel(final ServiceLabel serviceLabel) {
        this.serviceLabel = serviceLabel;
    }

    @Override
    public void save(final FormationJspBean bean) {
        throw new NotImplementedException("Cette méthode n'est pas implémentée sur ce service");
    }

    @Override
    public void delete(final Long id) {
        throw new NotImplementedException("Cette méthode n'est pas implémentée sur ce service");
    }

    @Override
    public FormationJspBean getById(final Long id) {
        throw new NotImplementedException("Cette méthode n'est pas implémentée sur ce service");
    }

    public void setUrlProvider(final UrlProvider urlProvider) {
        this.urlProvider = urlProvider;
    }
}
