/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.misc;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.Objetpartage;
import com.kportal.core.config.MessageHelper;
import com.univ.objetspartages.om.AbstractRechercheExterne;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.utils.RechercheFicheHelper;

import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereChaineAvecLibelle;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVideFormater;

/**
 * The Class RechercheKosmosLien.
 */
public class RechercheAnnuaireSup extends AbstractRechercheExterne {

    /**
     * Prepare le formulaire de recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if preparer recherche
     *
     */
    @Override
    public boolean preparerRECHERCHE(final InfoBean infoBean, final OMContext ctx) {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        boolean rechercheEffectuee = false;
        StringUtils.defaultString(infoBean.getString("OBJET"), "");
        final Locale locale = ctx.getLocale();
        if (infoBean.getString("OBJET").equalsIgnoreCase("ANNUAIREKSUP")) {
            infoBean.set("LISTE_TYPE_POPULATIONS", LabelUtils.getLabelCombo("10", locale));
            infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo("02", locale));
            infoBean.set("LISTE_STRUCTURES", serviceStructure.getSubStructures(StringUtils.EMPTY, String.valueOf(LangueUtil.getIndiceLocale(locale)), true));
            rechercheEffectuee = true;
        }
        if (infoBean.getString("OBJET").equalsIgnoreCase("DEFAULTSTRUCTUREKSUP")) {
            infoBean.set("LISTE_TYPE_STRUCTURES", LabelUtils.getLabelCombo("60", locale));
            infoBean.set("LISTE_DOMAINE_FORMATION_ETABLISSEMENTS", LabelUtils.getLabelCombo("2004", locale));
            rechercheEffectuee = true;
        }
        return rechercheEffectuee;
    }

    /**
     * Traite la recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if traiter recherche
     *
     * @throws ErreurApplicative
     *             the exception
     */
    @Override
    public boolean traiterRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws ErreurApplicative {
        boolean rechercheEffectuee = false;
        final String objet = infoBean.getString("OBJET").toUpperCase();
        final List<CritereRecherche> criteres = new ArrayList<>();
        criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
        final Objetpartage module = ReferentielObjets.getObjetByNom(objet);
        if (objet.equals("ANNUAIREKSUP")) {
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "NOM"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "PRENOM"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "FONCTION"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "THEME_RECHERCHE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "DISCIPLINE_ENSEIGNEE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "THEME_RECHERCHE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE_POPULATION", "10"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DISCIPLINE", "02"));
            final String listeRouge = infoBean.getString("LISTE_ROUGE");
            if ("0".equals(listeRouge) || "1".equals(listeRouge)) {
                String libelleListeRouge = MessageHelper.getCoreMessage("JTF_OUI");
                if ("0".equals(listeRouge)) {
                    libelleListeRouge = MessageHelper.getCoreMessage("JTF_NON");
                }
                criteres.add(new CritereRecherche("LISTE_ROUGE", listeRouge, libelleListeRouge));
            }
            if ("0".equals(module.getProperty("fiche.ANNUAIREKSUP.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(module.getMessage(ctx.getLocale(), "ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        // defaultstructureksup
        if (objet.equals("DEFAULTSTRUCTUREKSUP")) {
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "LIBELLE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE_STRUCTURE", "60"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DOMAINE_FORMATION_ETABLISSEMENT", "50"));
            final String structureEnseignement = infoBean.getString("STRUCTURE_ENSEIGNEMENT");
            if (StringUtils.isNotBlank(structureEnseignement) && !"0000".equals(structureEnseignement)) {
                final String libelleStructureEnseignement = module.getMessage("JTF_DEFAULTSTRUCTUREKSUP_STRUCTURE_ENSEIGNEMENT_" + structureEnseignement);
                criteres.add(new CritereRecherche("STRUCTURE_ENSEIGNEMENT", structureEnseignement, StringUtils.defaultIfBlank(libelleStructureEnseignement, structureEnseignement)));
            }
            if ("0".equals(module.getProperty("fiche.DEFAULTSTRUCTUREKSUP.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(module.getMessage(ctx.getLocale(), "ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        if (objet.equals("STRUCTURE")) {
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "LIBELLE"));
            if ("0".equals(module.getProperty("fiche.STRUCTURE.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(module.getMessage(ctx.getLocale(), "ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        if (rechercheEffectuee) {
            // attribut commun à toute les fiches
            criteres.addAll(getCritereDefaut(infoBean));
            infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
            gestionAncienFront(infoBean, criteres);
        }
        return rechercheEffectuee;
    }

    @Override
    public List<String> getCriteresRequete(final String objet, final boolean listeIncluse) {
        final List<String> criteres = new ArrayList<>();
        if (objet.equals("ANNUAIREKSUP")) {
            criteres.add("TYPE_POPULATION");
            criteres.add("CODE_RUBRIQUE");
            criteres.add("CODE_RATTACHEMENT");
            criteres.add("DISCIPLINE");
            criteres.add("LISTE_ROUGE");
        }
        if (objet.equals("DEFAULTSTRUCTUREKSUP")) {
            criteres.add("CODE_RUBRIQUE");
            criteres.add("CODE_RATTACHEMENT");
            criteres.add("TYPE_STRUCTURE");
            criteres.add("STRUCTURE_ENSEIGNEMENT");
            criteres.add("DOMAINE_FORMATION_ETABLISSEMENT");
        }
        return criteres;
    }
}
