/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.misc;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.etudiantetalumni.util.EtudiantetalumniUtil;
import com.kportal.cms.objetspartages.Objetpartage;
import com.univ.objetspartages.om.AbstractRechercheExterne;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.RechercheFicheHelper;

import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereChaineAvecFichierDat;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereChaineAvecLibelle;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVideFormater;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereThematique;

public class RechercheEtudiantAlumni extends AbstractRechercheExterne {

    private static final Logger LOG = LoggerFactory.getLogger(RechercheEtudiantAlumni.class);

    /**
     * Prepare le formulaire de recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if preparer recherche
     *
     */
    @Override
    public boolean preparerRECHERCHE(final InfoBean infoBean, final OMContext ctx) {
        boolean rechercheEffectuee = false;
        StringUtils.defaultString(infoBean.getString("OBJET"), "");
        final Locale locale = ctx.getLocale();
        if (infoBean.getString("OBJET").equalsIgnoreCase("ASSOCIATIONETUDIANT")) {
            infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelCombo("04", locale));
            infoBean.set("LISTE_TYPE_ASSOCIATIONS", LabelUtils.getLabelCombo("34", locale));
            rechercheEffectuee = true;
        }
        if (infoBean.getString("OBJET").equalsIgnoreCase("ANNUAIREETUDIANTS")) {
            infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo("55", locale));
            final Map<String, String> hdiplome = LabelUtils.getLabelCombo("20", locale);
            int i = 1;
            while (hdiplome.get("#" + i + "0000#") != null) {
                hdiplome.put(i + "0000", "#GROUPE#" + hdiplome.get("#" + i + "0000#"));
                hdiplome.put(i + "ZZZZ", "#/GROUPE#");
                hdiplome.remove("#" + i + "0000#");
                i++;
            }
            infoBean.set("LISTE_TYPE_DIPLOMES", hdiplome);
            infoBean.set("LISTE_COMPETENCES_DIPLOMES", LabelUtils.getLabelCombo("51", locale));
            infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelCombo("65", locale));
            rechercheEffectuee = true;
        }
        if (infoBean.getString("OBJET").equalsIgnoreCase("ANNUAIREANCIENSETUDIANTS")) {
            infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelCombo("65", locale));
            rechercheEffectuee = true;
        }
        return rechercheEffectuee;
    }

    /**
     * Traite la recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if traiter recherche
     *
     * @throws ErreurApplicative
     *             the exception
     */
    @Override
    public boolean traiterRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws ErreurApplicative {
        boolean rechercheEffectuee = false;
        final List<CritereRecherche> criteres = new ArrayList<>();
        final String objet = infoBean.getString("OBJET").toUpperCase();
        criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
        final Objetpartage module = ReferentielObjets.getObjetByNom(objet);
        if (objet.equals("ANNUAIREANCIENSETUDIANTS")) {
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "NOM"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "PRENOM"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "FONCTION_ACTUELLE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "SECTEURS_ACTIVITES", "65"));
            final String diplomeObtenus = infoBean.getString("DIPLOMES_OBTENUS");
            if (StringUtils.isNotBlank(diplomeObtenus) && StringUtils.contains(diplomeObtenus, ",LANGUE=")) {
                String libelleDiplome = diplomeObtenus;
                final String[] codesFiche = StringUtils.split(diplomeObtenus, ",LANGUE=");
                try {
                    final FicheUniv ficheDiplomeObtenus = FicheUnivHelper.getFiche(EtudiantetalumniUtil.NOM_OBJET_FORMATION, codesFiche[0], codesFiche[1]);
                    libelleDiplome = ficheDiplomeObtenus.getLibelleAffichable();
                } catch (final Exception e) {
                    LOG.error("impossible de récuperer la fiche formation de code " + diplomeObtenus, e);
                } finally {
                    criteres.add(new CritereRecherche("LIBELLE_DIPLOMES_OBTENUS", libelleDiplome));
                    criteres.add(new CritereRecherche("DIPLOMES_OBTENUS", diplomeObtenus, libelleDiplome));
                }
            }
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecFichierDat(infoBean, "PROMOTION", "annee_universitaire", module.getIdExtension()));
            if ("0".equals(module.getProperty("fiche.ANNUAIREANCIENSETUDIANTS.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(module.getMessage(ctx.getLocale(), "ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        if (objet.equals("ANNUAIREETUDIANTS")) {
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "NOM"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "PRENOM"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "COMPETENCES_DIPLOME", "51"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "COMPETENCES_GENERALES"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "COMPETENCES_INFORMATIQUES"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "LANGUES_PARLEES"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DISCIPLINE", "55"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE_DIPLOME", "20"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "SECTEURS_ACTIVITES", "65"));
            final String formation = infoBean.getString("CODE_FORMATION");
            if (StringUtils.isNotBlank(formation) && StringUtils.contains(formation, ",LANGUE=")) {
                String libelleFormation = formation;
                final String[] codesFiche = StringUtils.split(formation, ",LANGUE=");
                try {
                    final FicheUniv ficheFormation = FicheUnivHelper.getFiche(EtudiantetalumniUtil.NOM_OBJET_FORMATION, codesFiche[0], codesFiche[1]);
                    libelleFormation = ficheFormation.getLibelleAffichable();
                } catch (final Exception e) {
                    LOG.error("impossible de récuperer la fiche formation de code " + formation, e);
                } finally {
                    criteres.add(new CritereRecherche("LIBELLE_CODE_FORMATION", libelleFormation));
                    criteres.add(new CritereRecherche("CODE_FORMATION", formation, libelleFormation));
                }
            }
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecFichierDat(infoBean, "NIVEAU_DIPLOME", "annee_postbac", module.getIdExtension()));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecFichierDat(infoBean, "PROMOTION", "annee_universitaire", module.getIdExtension()));
            if ("0".equals(module.getProperty("fiche.ANNUAIREETUDIANTS.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(module.getMessage(ctx.getLocale(), "ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        if (objet.equals("ASSOCIATIONETUDIANT")) {
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "NOM"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "MOTCLE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "MOTS_CLES"));
            CollectionUtils.addIgnoreNull(criteres, getCritereThematique(infoBean));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE_ASSOCIATION", "34"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "NOM_PRESIDENT"));
            if ("0".equals(module.getProperty("fiche.ASSOCIATIONETUDIANT.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(module.getMessage(ctx.getLocale(), "ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        if (rechercheEffectuee) {
            // attribut commun à toute les fiches
            criteres.addAll(getCritereDefaut(infoBean));
            infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
            gestionAncienFront(infoBean, criteres);
        }
        return rechercheEffectuee;
    }

    @Override
    public List<String> getCriteresRequete(final String objet, final boolean listeIncluse) {
        final List<String> criteres = new ArrayList<>();
        if (objet.equals("ANNUAIREANCIENSETUDIANTS")) {
            criteres.add("DIPLOMES_OBTENUS");
            criteres.add("PROMOTION");
            criteres.add("FONCTION_ACTUELLE");
            criteres.add("SECTEURS_ACTIVITES");
        }
        if (objet.equals("ANNUAIREETUDIANTS")) {
            criteres.add("CODE_FORMATION");
            criteres.add("PROMOTION");
            criteres.add("DISCIPLINE");
            criteres.add("NIVEAU_DIPLOME");
            criteres.add("TYPE_DIPLOME");
            criteres.add("COMPETENCES_DIPLOME");
            criteres.add("SECTEURS_ACTIVITES");
        }
        if (objet.equals("ASSOCIATIONETUDIANT")) {
            criteres.add("TYPE_ASSOCIATION");
            criteres.add("MOTCLE");
            criteres.add("THEMATIQUE");
        }
        return criteres;
    }
}
