/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.misc;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.Objetpartage;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.ksup.objetspartages.om.Ueup;
import com.univ.objetspartages.om.AbstractRechercheExterne;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.objetspartages.util.CritereRechercheUtil;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.RechercheFicheHelper;

import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereChaineAvecLibelle;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVide;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVideFormater;
// TODO: Auto-generated Javadoc

/**
 * The Class RechercheKosmosLien.
 */
public class RechercheOffreFormation extends AbstractRechercheExterne {

    /**
     * Prepare le formulaire de recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if preparer recherche
     *
     */
    @Override
    public boolean preparerRECHERCHE(final InfoBean infoBean, final OMContext ctx) {
        boolean rechercheEffectuee = false;
        final String objet = StringUtils.defaultString(infoBean.getString("OBJET"), "").toUpperCase();
        final Locale locale = ctx.getLocale();
        final Objetpartage module = ReferentielObjets.getObjetByNom(objet);
        if (objet.equals("FORMATION")) {
            infoBean.set("LISTE_CATALOGUE", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.CATALOGUE.getValeur(), locale));
            infoBean.set("LISTE_NATURE_FORMATION", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NATURE_FORMATION.getValeur(), locale));
            final Map<String, String> hdiplome = LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), locale);
            int i = 1;
            while (hdiplome.get("#" + i + "0000#") != null) {
                hdiplome.put(i + "0000", "#GROUPE#" + hdiplome.get("#" + i + "0000#"));
                hdiplome.put(i + "9999", "#/GROUPE#");
                hdiplome.remove("#" + i + "0000#");
                i++;
            }
            infoBean.set("LISTE_TYPE_DIPLOMES", hdiplome);
            infoBean.set("LISTE_GRADE", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.GRADE.getValeur(), locale));
            infoBean.set("LISTE_DOMAINE_LMD", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), locale));
            infoBean.set("LISTE_DISCIPLINE", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), locale));
            infoBean.set("LISTE_MENTION", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.MENTION.getValeur(), locale));
            infoBean.set("LISTE_PARCOURS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.PARCOURS.getValeur(), locale));
            infoBean.set("LISTE_NIVEAU_RECRUTEMENT", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), locale));
            infoBean.set("LISTE_ANNEE_POSTBAC", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.ANNEE_POSTBAC.getValeur(), locale));
            infoBean.set("LISTE_NIVEAU_SORTIE", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NIVEAUX_SORTIE.getValeur(), locale));
            rechercheEffectuee = true;
        }
        if (objet.equalsIgnoreCase("PARCOURS")) {
            infoBean.set("LISTE_NIVEAU_RECRUTEMENT", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), locale));
            infoBean.set("LISTE_NIVEAU_DU_DIPLOME", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), locale));
            infoBean.set("LISTE_ANNEE", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.ANNEE.getValeur(), locale));
            infoBean.set("LISTE_SECTEUR", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.SECTEUR.getValeur(), locale));
            rechercheEffectuee = true;
        }
        if (objet.equals("UEUP")) {
            infoBean.set("LISTE_TYPES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_ELEMENT.getValeur(), locale));
            infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), locale));
            infoBean.set("LISTE_LANGUES_ENSEIGNEMENTS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), locale));
            rechercheEffectuee = true;
        }
        if (objet.equals("COURS")) {
            infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), locale));
            rechercheEffectuee = true;
        }
        return rechercheEffectuee;
    }

    /**
     * Traite la recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if traiter recherche
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public boolean traiterRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
        boolean rechercheEffectuee = false;
        final String objet = infoBean.getString("OBJET").toUpperCase();
        final List<CritereRecherche> criteres = new ArrayList<>();
        criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
        final Objetpartage module = ReferentielObjets.getObjetByNom(objet);
        final String langueDeRecherche = StringUtils.defaultString(infoBean.getString("LANGUE"), ctx.getLocale().getLanguage());

        if (objet.equals("FORMATION")) {
            //Champ code_rattachement_autres
            final String codeCodeRattachementAutres = infoBean.getString("CODE_RATTACHEMENT_AUTRES");
            if (StringUtils.isNotBlank(codeCodeRattachementAutres)) {
                final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
                StructureModele structureModele = serviceStructure.getByCodeLanguage(codeCodeRattachementAutres, langueDeRecherche);
                if (structureModele != null) {
                    CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("CODE_RATTACHEMENT_AUTRES", codeCodeRattachementAutres, structureModele.getLibelleAffichable()));
                }
            }
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "CODE_SI_SCOLARITE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "LIBELLE"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "CATALOGUE", "2001"));
            //Champ code_formation_mere
            final String codeCodeFormationMere = infoBean.getString("CODE_FORMATION_MERE");
            if (StringUtils.isNotBlank(codeCodeFormationMere)) {
                CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("CODE_FORMATION_MERE", codeCodeFormationMere, Formation.getLibelleAffichable(codeCodeFormationMere, langueDeRecherche)));
            }
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "NATURE_FORMATION", "2002"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "LIEU", OffreFormationTypeLibelle.LIEUX.getValeur()));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "FORMATION_DIPLOMANTE"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "FORMATION_LMD"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "TYPE_DIPLOME", "20"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "GRADE", "2003"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "DOMAINE_LMD", "2004"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "DISCIPLINE", "55"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "MENTION", "2005"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "PARCOURS", "2006"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "NIVEAU_RECRUTEMENT", "2007"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "ANNEE_POSTBAC", "2008"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "NIVEAU_SORTIE", "2009"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "FORMATION_INITIALE"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "REPRISE_ETUDES"));
            if ("1".equals(infoBean.get("TEMPS_PLEIN"))) {
                criteres.add(new CritereRecherche("TEMPS_PLEIN", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("TEMPS_AMENAGE"))) {
                criteres.add(new CritereRecherche("TEMPS_AMENAGE", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("COMPATIBLE_ACTIVITE_PRO"))) {
                criteres.add(new CritereRecherche("COMPATIBLE_ACTIVITE_PRO", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("ALTERNANCE"))) {
                criteres.add(new CritereRecherche("ALTERNANCE", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("PARTIELLEMENT_DISTANCE"))) {
                criteres.add(new CritereRecherche("PARTIELLEMENT_DISTANCE", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("TOTALEMENT_DISTANCE"))) {
                criteres.add(new CritereRecherche("TOTALEMENT_DISTANCE", "1", module.getMessage("JTF_OUI")));
            }
            //Champ responsables_fiche
            final String codeResponsablesFiche = infoBean.getString("RESPONSABLES_FICHE");
            if (StringUtils.isNotBlank(codeResponsablesFiche)) {
                CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("RESPONSABLES_FICHE", codeResponsablesFiche, FicheAnnuaire.getLibelleAffichable(codeResponsablesFiche, langueDeRecherche)));
            }
            //Champ ids
            final String ids = infoBean.getString("IDS");
            if (StringUtils.isNotBlank(ids)) {
                CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("IDS", ids, ids));
            }
            rechercheEffectuee = true;
        }
        if (objet.equals("PARCOURS")) {
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "CODE_SI_SCOLARITE"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "INTITULE_COURT"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "INTITULE_LONG"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "INTITULE_PARCOURS_DIPLOMANT"));
            if ("1".equals(infoBean.get("PARCOURS_FICTIF"))) {
                criteres.add(new CritereRecherche("PARCOURS_FICTIF", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("DIPLOMANT_ETAPE"))) {
                criteres.add(new CritereRecherche("DIPLOMANT_ETAPE", "1", module.getMessage("JTF_OUI")));
            }
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "NIVEAU_RECRUTEMENT", "2007"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "NIVEAU_DU_DIPLOME", "20"));
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "ACCES_SOUS_CONDITION"));
            if ("1".equals(infoBean.get("TEMPS_PLEIN"))) {
                criteres.add(new CritereRecherche("TEMPS_PLEIN", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("TEMPS_AMENAGE"))) {
                criteres.add(new CritereRecherche("TEMPS_AMENAGE", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("COMPATIBLE_ACTIVITE_PRO"))) {
                criteres.add(new CritereRecherche("COMPATIBLE_ACTIVITE_PRO", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("ALTERNANCE"))) {
                criteres.add(new CritereRecherche("ALTERNANCE", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("PARTIELLEMENT_DISTANCE"))) {
                criteres.add(new CritereRecherche("PARTIELLEMENT_DISTANCE", "1", module.getMessage("JTF_OUI")));
            }
            if ("1".equals(infoBean.get("TOTALEMENT_DISTANCE"))) {
                criteres.add(new CritereRecherche("TOTALEMENT_DISTANCE", "1", module.getMessage("JTF_OUI")));
            }
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "ANNEE", "2031"));

            final String codeCodeRattachementAutres = infoBean.getString("CODE_RATTACHEMENT_AUTRES");
            if (StringUtils.isNotBlank(codeCodeRattachementAutres)) {
                final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
                StructureModele structureModele = serviceStructure.getByCodeLanguage(codeCodeRattachementAutres, langueDeRecherche);
                if (structureModele != null) {
                    CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("CODE_RATTACHEMENT_AUTRES", codeCodeRattachementAutres, structureModele.getLibelleAffichable()));
                }
            }
            //Champ semestre1
            final String codeSemestre1 = infoBean.getString("UEUPS");
            if (StringUtils.isNotBlank(codeSemestre1)) {
                CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("UEUPS", codeSemestre1, com.ksup.objetspartages.om.Ueup.getLibelleAffichable(codeSemestre1, StringUtils.EMPTY)));
            }

            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereChaineAvecLibelle(infoBean, "SECTEUR", "2032"));
            //Champ parcours_amont
            final String codeParcoursAmont = infoBean.getString("PARCOURS_AMONT");
            if (StringUtils.isNotBlank(codeParcoursAmont)) {
                CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("PARCOURS_AMONT", codeParcoursAmont, com.ksup.objetspartages.om.Parcours.getLibelleAffichable(codeParcoursAmont, StringUtils.EMPTY)));
            }
            //Champ parcours_aval
            final String codeParcoursAval = infoBean.getString("PARCOURS_AVAL");
            if (StringUtils.isNotBlank(codeParcoursAval)) {
                CollectionUtils.addIgnoreNull(criteres, new CritereRecherche("PARCOURS_AVAL", codeParcoursAval, com.ksup.objetspartages.om.Parcours.getLibelleAffichable(codeParcoursAval, StringUtils.EMPTY)));
            }
            rechercheEffectuee = true;
        }
        if (objet.equals("UEUP")) {
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "CODE_SI_SCOLARITE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "LIBELLE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE", "56"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "NATURE", "54"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "SEMESTRE", "57"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DOMAINES", OffreFormationTypeLibelle.DOMAINE_LMD.getValeur()));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DOMAINE_ERASMUS", OffreFormationTypeLibelle.DOMAINE_ERASMUS.getValeur()));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "GRADE", OffreFormationTypeLibelle.GRADE.getValeur()));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DISCIPLINE", "55"));
            final String codeFormation = infoBean.getString("CODE_FORMATION");
            if (StringUtils.isNotBlank(codeFormation)) {
                final String libelleFormation = Formation.getLibelleAffichable(infoBean.getString("CODE_FORMATION"), LangueUtil.getLangueLocale(ctx.getLocale()));
                criteres.add(new CritereRecherche("CODE_FORMATION", codeFormation, StringUtils.defaultString(libelleFormation, codeFormation)));
                criteres.add(new CritereRecherche("LIBELLE_CODE_FORMATION", libelleFormation));
            }
            final String codeUeup = infoBean.getString("CODE_UEUP");
            if (StringUtils.isNotBlank(codeUeup)) {
                final String libelleUeup = Ueup.getLibelleAffichable(infoBean.getString("CODE_UEUP"), LangueUtil.getLangueLocale(ctx.getLocale()));
                criteres.add(new CritereRecherche("CODE_UEUP", codeUeup, StringUtils.defaultString(libelleUeup, codeUeup)));
                criteres.add(new CritereRecherche("LIBELLE_CODE_UEUP", libelleUeup));
            }
            criteres.addAll(getModaliteUEUP(infoBean, module));
            final String codeResponsable = infoBean.getString("CODE_RESPONSABLE");
            if (StringUtils.isNotBlank(codeResponsable) && StringUtils.contains(codeResponsable, ",LANGUE=")) {
                String libelleResponsable = codeResponsable;
                final String[] codesFiche = StringUtils.split(libelleResponsable, ",LANGUE=");
                final FicheUniv ficheResponsable = FicheUnivHelper.getFiche("annuaireksup", codesFiche[0], codesFiche[1]);
                libelleResponsable = ficheResponsable.getLibelleAffichable();
                criteres.add(new CritereRecherche("CODE_RESPONSABLE", codeResponsable, libelleResponsable));
                criteres.add(new CritereRecherche("LIBELLE_CODE_RESPONSABLE", libelleResponsable));
            }
            if ("0".equals(module.getProperty("fiche.UEUP.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(OffreFormationUtil.getMessage("ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        if (objet.equals("COURS")) {
            CollectionUtils.addIgnoreNull(criteres, CritereRechercheUtil.getCritereTexteNonVideFormater(infoBean, "CODE_SI_SCOLARITE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVide(infoBean, "CODE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "LIBELLE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "LANGUES_ENSEIGNEMENT", OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur()));
            final String codeResponsable = infoBean.getString("CODE_RESPONSABLE");
            if (StringUtils.isNotBlank(codeResponsable) && StringUtils.contains(codeResponsable, ",LANGUE=")) {
                String libelleResponsable = codeResponsable;
                final String[] codesFiche = StringUtils.split(libelleResponsable, ",LANGUE=");
                final FicheUniv ficheResponsable = FicheUnivHelper.getFiche("ANNUAIREKSUP", codesFiche[0], codesFiche[1]);
                libelleResponsable = ficheResponsable.getLibelleAffichable();
                criteres.add(new CritereRecherche("CODE_RESPONSABLE", codeResponsable, libelleResponsable));
                criteres.add(new CritereRecherche("LIBELLE_CODE_RESPONSABLE", libelleResponsable));
            }
            if ("0".equals(module.getProperty("fiche.UEUP.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(OffreFormationUtil.getMessage("ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            final String codeUeup = infoBean.getString("CODE_UEUP");
            if (StringUtils.isNotBlank(codeUeup)) {
                final String libelleUeup = Ueup.getLibelleAffichable(infoBean.getString("CODE_UEUP"), LangueUtil.getLangueLocale(ctx.getLocale()));
                criteres.add(new CritereRecherche("CODE_UEUP", codeUeup, StringUtils.defaultString(libelleUeup, codeUeup)));
                criteres.add(new CritereRecherche("LIBELLE_CODE_UEUP", libelleUeup));
            }
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DISCIPLINE", "55"));
            if ("0".equals(module.getProperty("fiche.COURS.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(OffreFormationUtil.getMessage("ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
        }
        if (rechercheEffectuee) {
            // attribut commun à toute les fiches
            criteres.addAll(getCritereDefaut(infoBean));
            infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
            gestionAncienFront(infoBean, criteres);
        }
        return rechercheEffectuee;
    }

    /**
     * C'est historique...
     *
     * @param infoBean
     * @param module
     * @return
     */
    private Collection<CritereRecherche> getModaliteUEUP(final InfoBean infoBean, final Objetpartage module) {
        final Collection<CritereRecherche> criteresModalite = new ArrayList<>();
        final String modalite = infoBean.getString("MODALITES_ENSEIGNEMENT");
        if (StringUtils.isNotBlank(modalite)) {
            final String libelleModalite = OffreFormationUtil.getMessage("ST_FORMATION_" + modalite);
            criteresModalite.add(new CritereRecherche("MODALITES_ENSEIGNEMENT", modalite, StringUtils.defaultString(libelleModalite, modalite)));
        } else {
            final StringBuilder oldCritere = new StringBuilder();
            final String aDistance = infoBean.getString("A_DISTANCE");
            if ("0".equals(aDistance) || "1".equals(aDistance)) {
                String libelleADistance = OffreFormationUtil.getMessage("JTF_NON");
                if ("1".equals(aDistance)) {
                    libelleADistance = OffreFormationUtil.getMessage("JTF_OUI");
                    oldCritere.append("A_DISTANCE");
                }
                criteresModalite.add(new CritereRecherche("A_DISTANCE", aDistance, libelleADistance));
            }
            final String mixteDistance = infoBean.getString("MIXTE_DISTANCE");
            if ("0".equals(mixteDistance) || "1".equals(mixteDistance)) {
                String libelleMixteDistance = OffreFormationUtil.getMessage("JTF_NON");
                if ("1".equals(mixteDistance)) {
                    libelleMixteDistance = OffreFormationUtil.getMessage("JTF_OUI");
                    if (oldCritere.length() > 0) {
                        oldCritere.append(";");
                    }
                    oldCritere.append("MIXTE_DISTANCE");
                }
                criteresModalite.add(new CritereRecherche("MIXTE_DISTANCE", mixteDistance, libelleMixteDistance));
            }
            if (StringUtils.isNotBlank(oldCritere)) {
                criteresModalite.add(new CritereRecherche("MODALITES_ENSEIGNEMENT", oldCritere.toString()));
            }
        }
        return criteresModalite;
    }

    @Override
    public List<String> getCriteresRequete(final String objet, final boolean listeIncluse) {
        final List<String> criteres = new ArrayList<>();
        if (objet.equals("FORMATION")) {
            criteres.add("CODE_RATTACHEMENT_AUTRES");
            criteres.add("CODE_SI_SCOLARITE");
            criteres.add("INTITULE_COURT");
            criteres.add("INTITULE_LONG");
            criteres.add("CATALOGUE");
            criteres.add("CODE_FORMATION_MERE");
            criteres.add("NATURE_FORMATION");
            criteres.add("FORMATION_DIPLOMANTE");
            criteres.add("FORMATION_LMD");
            criteres.add("TYPE_DIPLOME");
            criteres.add("GRADE");
            criteres.add("DOMAINE_LMD");
            criteres.add("DISCIPLINE");
            criteres.add("MENTION");
            criteres.add("PARCOURS");
            criteres.add("NIVEAU_RECRUTEMENT");
            criteres.add("ANNEE_POSTBAC");
            criteres.add("NIVEAU_SORTIE");
            criteres.add("FORMATION_INITIALE");
            criteres.add("REPRISE_ETUDES");
            criteres.add("TEMPS_PLEIN");
            criteres.add("COMPATIBLE_ACTIVITE_PRO");
            criteres.add("TEMPS_AMENAGE");
            criteres.add("ALTERNANCE");
            criteres.add("PARTIELLEMENT_DISTANCE");
            criteres.add("TOTALEMENT_DISTANCE");
            criteres.add("RESPONSABLES_FICHE");
        }
        if (objet.equals("PARCOURS")) {
            criteres.add("PARCOURS_FICTIF");
            criteres.add("DIPLOMANT_ETAPE");
            criteres.add("NIVEAU_RECRUTEMENT");
            criteres.add("NIVEAU_DU_DIPLOME");
            criteres.add("ACCES_SOUS_CONDITION");
            criteres.add("RYTHME_ALTERNE");
            criteres.add("ANNEE");
            criteres.add("SEMESTRE1");
            criteres.add("SEMESTRE2");
            criteres.add("SECTEUR");
            criteres.add("PARCOURS_AMONT");
            criteres.add("PARCOURS_AVAL");
        }
        if (objet.equals("UEUP")) {
            criteres.add("TYPE");
            criteres.add("NATURE");
            criteres.add("SEMESTRE");
            criteres.add("GRADE");
            criteres.add("DOMAINES");
            criteres.add("DOMAINE_ERASMUS");
            criteres.add("DISCIPLINE");
            criteres.add("A_DISTANCE");
            criteres.add("MIXTE_DISTANCE");
            criteres.add("CODE_RESPONSABLE");
            criteres.add("CODE_FORMATION");
            criteres.add("CODE_UEUP");
            criteres.add("LANGUES_ENSEIGNEMENT");
        }
        if (objet.equals("COURS")) {
            criteres.add("DISCIPLINE");
            criteres.add("CODE_RESPONSABLE");
            criteres.add("CODE_UEUP");
        }
        return criteres;
    }
}