/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: offrestageemploi
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.misc;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.Objetpartage;
import com.kportal.core.config.MessageHelper;
import com.univ.objetspartages.om.AbstractRechercheExterne;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.utils.RechercheFicheHelper;

import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereChaineAvecFichierDat;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereChaineAvecLibelle;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereDate;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVideFormater;

public class RechercheOffrestageemploi extends AbstractRechercheExterne {

    /**
     * Prepare le formulaire de recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if preparer recherche
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public boolean preparerRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
        boolean rechercheEffectuee = false;
        StringUtils.defaultString(infoBean.getString("OBJET"), "");
        final Locale locale = ctx.getLocale();
        if (infoBean.getString("OBJET").equalsIgnoreCase("OFFRESSTAGESEMPLOIS")) {
            infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelCombo("65", locale));
            infoBean.set("LISTE_DUREES", LabelUtils.getLabelCombo("32", locale));
            infoBean.set("LISTE_DEBUTS", LabelUtils.getLabelCombo("33", locale));
            infoBean.set("LISTE_TYPE_STAGES", LabelUtils.getLabelCombo("35", locale));
            infoBean.set("LISTE_TYPE_JOBS", LabelUtils.getLabelCombo("36", locale));
            infoBean.set("LISTE_TYPE_EMPLOIS", LabelUtils.getLabelCombo("37", locale));
            rechercheEffectuee = true;
        }
        return rechercheEffectuee;
    }

    /**
     * Traite la recherche.
     *
     * @param infoBean
     *            the info bean
     * @param ctx
     *            the ctx
     *
     * @return true, if traiter recherche
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public boolean traiterRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        boolean rechercheEffectuee = false;
        final String objet = infoBean.getString("OBJET").toUpperCase();
        if (objet.equals("OFFRESSTAGESEMPLOIS")) {
            final List<CritereRecherche> criteres = new ArrayList<>();
            criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
            final Objetpartage module = ReferentielObjets.getObjetByNom(objet);
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "TITRE"));
            final String typeOffre = infoBean.getString("TYPE_OFFRE");
            //Type offre
            if (StringUtils.isNotBlank(typeOffre)) {
                CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "TYPE_OFFRE"));
            } else {
                final StringBuilder oldTypeOffre = new StringBuilder();
                if ("1".equals(infoBean.get("STAGE"))) {
                    oldTypeOffre.append("stage");
                    criteres.add(new CritereRecherche("STAGE", "1", module.getMessage("JTF_OUI")));
                }
                if ("1".equals(infoBean.get("JOB"))) {
                    if (oldTypeOffre.length() > 0) {
                        oldTypeOffre.append(";");
                    }
                    oldTypeOffre.append("job");
                    criteres.add(new CritereRecherche("JOB", "1", module.getMessage("JTF_OUI")));
                }
                if ("1".equals(infoBean.get("EMPLOI"))) {
                    if (oldTypeOffre.length() > 0) {
                        oldTypeOffre.append(";");
                    }
                    oldTypeOffre.append("emploi");
                    criteres.add(new CritereRecherche("EMPLOI", "1", module.getMessage("JTF_OUI")));
                }
                criteres.add(new CritereRecherche("TYPE_OFFRE", oldTypeOffre.toString()));
            }
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "RAISON_SOCIALE"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE_STAGE", "35"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE_JOB", "36"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "TYPE_EMPLOI", "37"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "SECTEURS_ACTIVITES", "65"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DUREE", "32"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecLibelle(infoBean, "DEBUT", "33"));
            CollectionUtils.addIgnoreNull(criteres, getCritereDate(infoBean, "DATE_DEBUT"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "VILLES"));
            CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "MOTS_CLES"));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecFichierDat(infoBean, "PAYS", "international_pays", module.getIdExtension()));
            CollectionUtils.addIgnoreNull(criteres, getCritereChaineAvecFichierDat(infoBean, "NIVEAU_ETUDE", "niveau_recrutement", module.getIdExtension()));
            final String remunerationStage = infoBean.getString("REMUNERATION_STAGE");
            if ("0".equals(remunerationStage) || "1".equals(remunerationStage)) {
                String libelleRemunerationStage = MessageHelper.getCoreMessage("JTF_OUI");
                if ("0".equals(remunerationStage)) {
                    libelleRemunerationStage = MessageHelper.getCoreMessage("JTF_NON");
                }
                criteres.add(new CritereRecherche("REMUNERATION_STAGE", remunerationStage, libelleRemunerationStage));
            }
            final String filiere = infoBean.getString("FILIERE");
            if (StringUtils.isNotBlank(filiere) && !"0000".equals(filiere)) {
                final String libelleFiliere = serviceStructure.getDisplayableLabel(filiere, LangueUtil.getLangueLocale(ctx.getLocale()));
                criteres.add(new CritereRecherche("FILIERE", filiere, libelleFiliere));
            }
            if ("0".equals(module.getProperty("fiche.OFFRESSTAGESEMPLOIS.recherche_vide")) && criteres.size() <= 1) {
                throw new ErreurApplicative(module.getMessage(ctx.getLocale(), "ST_RECHERCHE_SAISIR_UN_CRITERE"));
            }
            rechercheEffectuee = true;
            // attribut commun à toute les fiches
            criteres.addAll(getCritereDefaut(infoBean));
            infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
            gestionAncienFront(infoBean, criteres);
        }
        return rechercheEffectuee;
    }

    @Override
    public List<String> getCriteresRequete(final String objet, final boolean listeIncluse) {
        final List<String> criteres = new ArrayList<>();
        if (objet.equals("OFFRESSTAGESEMPLOIS")) {
            criteres.add("STAGE");
            criteres.add("JOB");
            criteres.add("EMPLOI");
            criteres.add("ENSEIG");
            criteres.add("THESE");
            criteres.add("POSTDC");
            criteres.add("TYPE_EMPLOI");
            criteres.add("TYPE_STAGE");
            criteres.add("TYPE_JOB");
            criteres.add("DATE_DEBUT");
            criteres.add("DATE_FIN");
            criteres.add("SECTEURS_ACTIVITES");
            criteres.add("DUREE");
            criteres.add("DEBUT");
            criteres.add("PAYS");
            criteres.add("FILIERE");
            criteres.add("SECTEURS_ACTIVITES");
            criteres.add("NIVEAU_ETUDE");
            criteres.add("REMUNERATION_STAGE");
            if (listeIncluse) {
                criteres.add("SELECTION");
                criteres.add("JOUR");
            }
        }
        return criteres;
    }
}
