/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.etudiantetalumni.util.EtudiantetalumniUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.ksup.objetspartages.bean.AnnuaireanciensetudiantsBean;
import com.ksup.objetspartages.service.ServiceAnnuaireAnciensEtudiants;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.EscapeString;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.Operateur;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;
import com.univ.utils.sql.operande.TypeOperande;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet annuaireanciensetudiants.
 */
@FicheAnnotation()
public class Annuaireanciensetudiants extends AbstractFicheRestriction<AnnuaireanciensetudiantsBean> implements FicheRattachementsSecondaires, AnnuaireModele {

	private static final Logger LOG = LoggerFactory.getLogger(Annuaireanciensetudiants.class);

	/**
	 * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
	 */
	public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
		String res = StringUtils.EMPTY;
		if (codes == null || codes.length() == 0) {
			return res;
		}
		final Annuaireanciensetudiants annuaireanciensetudiants = new Annuaireanciensetudiants();
		annuaireanciensetudiants.init();
		if (langue == null || langue.length() == 0) {
			langue = "0";
		}
		final StringTokenizer st = new StringTokenizer(codes, ";");
		while (st.hasMoreTokens()) {
			final String code = st.nextToken();
			// On cherche d'abord la version en ligne puis les autres versions
			int count = annuaireanciensetudiants.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.toString());
			if (count == 0) {
				count = annuaireanciensetudiants.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
			}
			if (count > 0) {
				if (res.length() > 1) {
					res += ";";
				}
				if (annuaireanciensetudiants.nextItem()) {
					res += annuaireanciensetudiants.getLibelleAffichable();
				}
			} else {
				res += "-";
			}
		}
		return res;
	}

	/**
	 * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
	 */
	public static Annuaireanciensetudiants getFicheAnnuaireanciensetudiants(final OMContext ctx, final String code, String langue) throws Exception {
		Annuaireanciensetudiants fiche = null;
		if (code != null && code.length() > 0) {
			if (langue == null || langue.length() == 0) {
				langue = "0"; // francais par defaut
			}
			fiche = new Annuaireanciensetudiants();
			fiche.init();
			int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.toString());
			if (count == 0) // si pas de version en ligne
			{
				count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
			}
			if (count == 0) // si pas de version dans la langue demandee
			{
				count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.toString());
			}
			if (count > 0) {
				fiche.nextItem();
			}
		}
		return fiche;
	}

	/**
	 * Initialise l'objet metier.
	 */
	@Override
	public void init() {
		super.init();
		setCivilite(StringUtils.EMPTY);
		setNom(StringUtils.EMPTY);
		setPrenom(StringUtils.EMPTY);
		setCodeRattachementAutres(StringUtils.EMPTY);
		setDiplomesObtenus(StringUtils.EMPTY);
		setAnneeDiplome(StringUtils.EMPTY);
		setFonctionActuelle(StringUtils.EMPTY);
		setAdresseMail(StringUtils.EMPTY);
		setContact(StringUtils.EMPTY);
		setListeRouge(StringUtils.EMPTY);
		setCv(StringUtils.EMPTY);
		setSecteursActivites(StringUtils.EMPTY);
		setAdresseSitePerso(StringUtils.EMPTY);
		setComplements(StringUtils.EMPTY);
		setEntreprise(StringUtils.EMPTY);
		setAccrocheTemoignage(StringUtils.EMPTY);
		setTemoignage(StringUtils.EMPTY);
		setUrlProfilLinkedin(StringUtils.EMPTY);
		setUrlProfilViadeo(StringUtils.EMPTY);
	}



	/**
	 * Renvoie le libelle a afficher.
	 */
	@Override
	public String getLibelleAffichable() {
		return getServiceFiche().getLibelleAffichable(persistenceBean);
	}

	public String getLibelleSansFallback() {
		return ((ServiceAnnuaireAnciensEtudiants) getServiceFiche()).getLibelleSansFallback(persistenceBean);
	}

	/**
	 * Renvoie le libelle pour l'attribut civilite.
	 */
	public String getLibelleCivilite() {
		return ((ServiceAnnuaireAnciensEtudiants) getServiceFiche()).getLibelleCivilite(persistenceBean);
	}

	/**
	 * Renvoie le libelle pour l'attribut codeRattachementAutres.
	 */
	public String getLibelleCodeRattachementAutres() throws Exception {
		final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
		return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
	}

	/**
	 * Renvoie la liste des structures correspondant a l'attribut codeRattachementAutres.
	 */
	public List<StructureModele> getFichesCodeRattachementAutres() throws Exception {
		final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
		final List<StructureModele> listCodeRattachementAutres = new ArrayList<>();
		final StringTokenizer stCodeRattachementAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
		while (stCodeRattachementAutres.hasMoreTokens()) {
			final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementAutres.nextToken(), getLangue());
			if(currentStructure != null) {
				listCodeRattachementAutres.add(currentStructure);
			}
		}
		return listCodeRattachementAutres;
	}

	/**
	 * Renvoie le libelle pour l'attribut diplomesObtenus.
	 */
	public String getLibelleDiplomesObtenus() throws Exception {
		String listeLibelles = StringUtils.EMPTY;
		final StringTokenizer stUe = new StringTokenizer(getDiplomesObtenus(), ";");
		String code;
		String langue;
		String annee;
		StringTokenizer stUeCourant;
		int token = 0, idxUe = 0;
		//la liste des formations obtenues est composée de la manière suivante :
		// code,langue,annee
		while (stUe.hasMoreTokens()) {
			code = StringUtils.EMPTY;
			langue = "0";
			annee = StringUtils.EMPTY;
			stUeCourant = new StringTokenizer(stUe.nextToken(), ",");
			token = 0;
			while (stUeCourant.hasMoreTokens()) {
				switch (token) {
					case 0:
						code = stUeCourant.nextToken();
						break;
					case 1:
						langue = stUeCourant.nextToken();
						break;
					case 2:
						annee = stUeCourant.nextToken();
						break;
					default:
						stUeCourant.nextToken();
						break;
				}
				token++;
			}
			int iLangue = 0;
			try {
				iLangue = new Integer(langue);
			} catch (final NumberFormatException e) {
				LOG.error("impossible de parser la langue en integer", e);
			}
			LangueUtil.getLocale(iLangue);
			if (idxUe > 0) {
				listeLibelles += ";";
			}
			if (!code.equals(StringUtils.EMPTY)) {
				listeLibelles += StringUtils.replace(FicheUnivHelper.getFiche(EtudiantetalumniUtil.NOM_OBJET_FORMATION, code, langue).getLibelleAffichable(), ";", "-");
			}
			if (!annee.equals(StringUtils.EMPTY)) {
				listeLibelles += " [" + FicheUnivHelper.getLibelle(this, "annee_universitaire", annee) + "]";
			}
			idxUe++;
		}
		String sCumullibelles = listeLibelles.replaceAll("\"", "\\\"");
		sCumullibelles = sCumullibelles.replaceAll("\'", "\\\'");
		return sCumullibelles;
	}

	/**
	 * Renvoie la liste des formations correspondant a l'attribut diplomesObtenus.
	 */
	public List<FicheUniv> getFichesFormation() throws Exception {
		final List<FicheUniv> listDiplomesObtenus = new ArrayList<>();
		final StringTokenizer stDiplomesObtenus = new StringTokenizer(getDiplomesObtenus(), ";");
		StringTokenizer stFormation;
		String code = StringUtils.EMPTY, langue = "0";
		int token;
		while (stDiplomesObtenus.hasMoreTokens()) {
			stFormation = new StringTokenizer(stDiplomesObtenus.nextToken(), ",");
			token = 0;
			while (stFormation.hasMoreTokens()) {
				switch (token) {
					case 0:
						code = stFormation.nextToken();
						break;
					case 1:
						langue = stFormation.nextToken();
						break;
					default:
						stFormation.nextToken();
						break;
				}
				token++;
			}
			listDiplomesObtenus.add(FicheUnivHelper.getFiche(EtudiantetalumniUtil.NOM_OBJET_FORMATION, code, langue));
		}
		return listDiplomesObtenus;
	}

	/**
	 * Renvoie le contenu formate en HTML pour l'attribut contact.
	 */
	public String getFormatedContact() throws Exception {
		return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getContact());
	}

	/**
	 * Renvoie le contenu formate en HTML pour l'attribut cv.
	 */
	public String getFormatedCv() throws Exception {
		return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCv());
	}

	/**
	 * Renvoie le libelle pour l'attribut secteursActivites.
	 */
	public String getLibelleSecteursActivites() {
		return LabelUtils.getLibelle("65", getSecteursActivites(), getLangue());
	}

	/**
	 * Renvoie le contenu formate en HTML pour l'attribut complements.
	 */
	public String getFormatedComplements() throws Exception {
		return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
	}

	/**
	 * Renvoie la structure de la fiche.
	 *
	 * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
	 */
	@Deprecated
	public StructureModele getInfosStructure() {
		final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
		return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
	}

	/**
	 * Renvoie les metadonnees de la fiche.
	 */
	public MetatagBean getMeta() throws Exception {
		return MetatagUtils.lireMeta(this);
	}

	/**
	 * Traitement d'une requete sur l'objet.
	 */
	@Override
	public int traiterRequete(final String requete) throws Exception {
		// Recuperation des parametres de la requete
		final String nom = RequeteUtil.renvoyerParametre(requete, "NOM");
		final String prenom = RequeteUtil.renvoyerParametre(requete, "PRENOM");
		final String diplomesObtenus = RequeteUtil.renvoyerParametre(requete, "DIPLOMES_OBTENUS");
		final String promotion = RequeteUtil.renvoyerParametre(requete, "PROMOTION");
		final String anneeDiplome = RequeteUtil.renvoyerParametre(requete, "ANNEE_DIPLOME");
		final String fonctionActuelle = RequeteUtil.renvoyerParametre(requete, "FONCTION_ACTUELLE");
		final String secteursActivites = RequeteUtil.renvoyerParametre(requete, "SECTEURS_ACTIVITES");
		final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
		final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
		final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
		final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
		final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
		final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
		final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
		final String codeFiche = RequeteUtil.renvoyerParametre(requete, "CODE");
		String order = "";
		if (selection.length() > 0) {
			// Dernières pages libres publiées
			if (selection.equals("0007")) {
				order = "META.META_DATE_MISE_EN_LIGNE DESC";
			}
		}
		return select(nom, prenom, diplomesObtenus, anneeDiplome, promotion, fonctionActuelle, secteursActivites, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, codeFiche, nombre, order);
	}

	/**
	 * Renvoie la liste des references a cette fiche.
	 */
	@Override
	public String getReferences() {
		return getServiceFiche().getReferences(persistenceBean);
	}

	/**
	 * Selection d'une instance de l'objet Annuaireanciensetudiants a partir de l'ensemble des criteres.
	 */
	public int select(final String nom, final String prenom, final String diplomesObtenus, final String anneeDiplome, final String promotion, final String fonctionActuelle, final String secteursActivites, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final String order) throws Exception {
		final ClauseWhere where = new ClauseWhere();
		if (isNotEmpty(code)) {
			where.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE", code));
		}
		if (isNotEmpty(nom)) {
			where.and(ConditionHelper.like("T1.NOM", nom, "%", "%"));
		}
		if (isNotEmpty(prenom)) {
			where.and(ConditionHelper.like("T1.PRENOM", prenom, "%", "%"));
		}
		boolean diplomeTraite = Boolean.FALSE;
		String critereDiplomeFormater = "";
		if (isNotEmpty(diplomesObtenus)) {
			if (isNotEmpty(promotion) && !"0000".equals(promotion)) {
				critereDiplomeFormater = "'" + EscapeString.escapeSql(diplomesObtenus) + ",%," + EscapeString.escapeSql(promotion) + "'";
				diplomeTraite = Boolean.TRUE;
			} else {
				critereDiplomeFormater = "'" + EscapeString.escapeSql(diplomesObtenus) + ",%,%'";
			}
		}
		if (!diplomeTraite && isNotEmpty(promotion) && !"0000".equals(promotion)) {
			critereDiplomeFormater = "'%,%," + EscapeString.escapeSql(promotion) + "'";
		}
		if (isNotEmpty(critereDiplomeFormater)) {
			where.and(ConditionHelper.genericConditionSurColonne("T1.DIPLOMES_OBTENUS", critereDiplomeFormater, TypeOperande.NON_ECHAPABLE, Operateur.LIKE));
		}
		if (isNotEmpty(anneeDiplome) && !"0000".equals(anneeDiplome)) {
			where.and(ConditionHelper.egalVarchar("T1.ANNEE_DIPLOME", anneeDiplome));
		}
		if (isNotEmpty(fonctionActuelle)) {
			where.and(ConditionHelper.like("T1.FONCTION_ACTUELLE", fonctionActuelle, "%", "%"));
		}
		if (isNotEmpty(secteursActivites) && !"0000".equals(secteursActivites)) {
			where.and(ConditionHelper.likePourValeursMultiple("T1.SECTEURS_ACTIVITES", secteursActivites));
		}
		if (isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
			final ConditionList conditionStructureAnnuaire = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
			conditionStructureAnnuaire.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
			where.and(conditionStructureAnnuaire);
		}
		if (isNotEmpty(langue) && !"0000".equals(langue)) {
			where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
		}
		if (isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
			where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
		}
		if (isNotEmpty(codeRedacteur)) {
			where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
		}
		final RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
		ClauseOrderBy orderBy = new ClauseOrderBy();
		if (StringUtils.isNotEmpty(order)) {
			orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
		} else {
			orderBy.orderBy("T1.NOM", SensDeTri.ASC);
		}
		requeteSelect.orderBy(orderBy);
		final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
		requeteSelect.limit(limite);
		return select(requeteSelect.formaterRequete());
	}

	public Long getIdAnnuaireanciensetudiants(){
		return persistenceBean.getId();
	}

	public void setIdAnnuaireanciensetudiants(Long id){
		persistenceBean.setId(id);
	}

	/**
	 * Gets the civilite.
	 *
	 * @return the civilite
	 */
	@Override
	public String getCivilite() {
		return persistenceBean.getCivilite();
	}

	/**
	 * Sets the civilite.
	 *
	 * @param civilite
	 *            the new civilite
	 */
	@Override
	public void setCivilite(String civilite) {
		persistenceBean.setCivilite(civilite);
	}

	/**
	 * Gets the nom.
	 *
	 * @return the nom
	 */
	@Override
	public String getNom() {
		return persistenceBean.getNom();
	}

	/**
	 * Sets the nom.
	 *
	 * @param nom
	 *            the new nom
	 */
	@Override
	public void setNom(String nom) {
		persistenceBean.setNom(nom);
	}

	/**
	 * Gets the prenom.
	 *
	 * @return the prenom
	 */
	@Override
	public String getPrenom() {
		return persistenceBean.getPrenom();
	}



	/**
	 * Sets the prenom.
	 *
	 * @param prenom
	 *            the new prenom
	 */
	@Override
	public void setPrenom(String prenom) {
		persistenceBean.setPrenom(prenom);
	}

	/**
	 * Gets the date naissance.
	 *
	 * @return the date naissance
	 */
	@Override
	public Date getDateNaissance() {
		return persistenceBean.getDateNaissance();
	}

	/**
	 * Sets the date naissance.
	 *
	 * @param dateNaissance
	 *            the new date naissance
	 */
	@Override
	public void setDateNaissance(Date dateNaissance) {
		persistenceBean.setDateNaissance(dateNaissance);
	}

	/**
	 * Gets the code rattachement autres.
	 *
	 * @return the code rattachement autres
	 */
	@Override
	public String getCodeRattachementAutres() {
		return persistenceBean.getCodeRattachementAutres();
	}

	/**
	 * Sets the code rattachement autres.
	 *
	 * @param codeRattachementAutres
	 *            the new code rattachement autres
	 */
	@Override
	public void setCodeRattachementAutres(String codeRattachementAutres) {
		persistenceBean.setCodeRattachementAutres(codeRattachementAutres);
	}

	/**
	 * Gets the diplomes obtenus.
	 *
	 * @return the diplomes obtenus
	 */
	public String getDiplomesObtenus() {
		return persistenceBean.getDiplomesObtenus();
	}

	/**
	 * Sets the diplomes obtenus.
	 *
	 * @param diplomesObtenus
	 *            the new diplomes obtenus
	 */
	public void setDiplomesObtenus(String diplomesObtenus) {
		persistenceBean.setDiplomesObtenus(diplomesObtenus);
	}

	/**
	 * Gets the annee diplome.
	 *
	 * @return the annee diplome
	 */
	public String getAnneeDiplome() {
		return persistenceBean.getAnneeDiplome();
	}

	/**
	 * Sets the annee diplome.
	 *
	 * @param anneeDiplome
	 *            the new annee diplome
	 */
	public void setAnneeDiplome(String anneeDiplome) {
		persistenceBean.setAnneeDiplome(anneeDiplome);
	}

	/**
	 * Gets the fonction actuelle.
	 *
	 * @return the fonction actuelle
	 */
	public String getFonctionActuelle() {
		return persistenceBean.getFonctionActuelle();
	}


	/**
	 * Sets the fonction actuelle.
	 *
	 * @param fonctionActuelle
	 *            the new fonction actuelle
	 */
	public void setFonctionActuelle(String fonctionActuelle) {
		persistenceBean.setFonctionActuelle(fonctionActuelle);
	}

	/**
	 * Gets the adresse mail.
	 *
	 * @return the adresse mail
	 */
	@Override
	public String getAdresseMail() {
		return persistenceBean.getAdresseMail();
	}

	/**
	 * Sets the adresse mail.
	 *
	 * @param adresseMail
	 *            the new adresse mail
	 */
	@Override
	public void setAdresseMail(String adresseMail) {
		persistenceBean.setAdresseMail(adresseMail);
	}

	/**
	 * Gets the contact.
	 *
	 * @return the contact
	 */
	@GetterAnnotation(isToolbox = true)
	public String getContact() {
		return persistenceBean.getContact();
	}

	/**
	 * Sets the contact.
	 *
	 * @param contact
	 *            the new contact
	 */
	@SetterAnnotation(isToolbox = true)
	public void setContact(String contact) {
		persistenceBean.setContact(contact);
	}

	/**
	 * Gets the liste rouge.
	 *
	 * @return the liste rouge
	 */
	@Override
	public String getListeRouge() {
		return persistenceBean.getListeRouge();
	}

	/**
	 * Sets the liste rouge.
	 *
	 * @param listeRouge
	 *            the new liste rouge
	 */
	@Override
	public void setListeRouge(String listeRouge) {
		persistenceBean.setListeRouge(listeRouge);
	}

	/**
	 * Gets the cv.
	 *
	 * @return the cv
	 */
	@Override
	public String getCv() {
		return persistenceBean.getCv();
	}

	/**
	 * Sets the cv.
	 *
	 * @param cv
	 *            the new cv
	 */
	@Override
	public void setCv(String cv) {
		persistenceBean.setCv(cv);
	}

	/**
	 * Gets the secteurs activites.
	 *
	 * @return the secteurs activites
	 */
	public String getSecteursActivites() {
		return persistenceBean.getSecteursActivites();
	}

	/**
	 * Sets the secteurs activites.
	 *
	 * @param secteursActivites
	 *            the new secteurs activites
	 */
	public void setSecteursActivites(String secteursActivites) {
		persistenceBean.setSecteursActivites(secteursActivites);
	}

	/**
	 * Gets the adresse site perso.
	 *
	 * @return the adresse site perso
	 */
	@Override
	public String getAdresseSitePerso() {
		return persistenceBean.getAdresseSitePerso();
	}

	/**
	 * Sets the adresse site perso.
	 *
	 * @param adresseSitePerso
	 *            the new adresse site perso
	 */
	@Override
	public void setAdresseSitePerso(String adresseSitePerso) {
		persistenceBean.setAdresseSitePerso(adresseSitePerso);
	}

	/**
	 * Gets the complements.
	 *
	 * @return the complements
	 */
	@Override
	public String getComplements() {
		return persistenceBean.getComplements();
	}

	/**
	 * Sets the complements.
	 *
	 * @param complements
	 *            the new complements
	 */
	@Override
	public void setComplements(String complements) {
		persistenceBean.setComplements(complements);
	}

	/**
	 * Gets the date suppression.
	 *
	 * @return the date suppression
	 */
	public Date getDateSuppression() {
		return persistenceBean.getDateSuppression();
	}

	/**
	 * Sets the date suppression.
	 *
	 * @param dateSuppression
	 *            the new date suppression
	 */
	public void setDateSuppression(Date dateSuppression) {
		persistenceBean.setDateSuppression(dateSuppression);
	}


	@Override
	public String getTypePopulation() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setTypePopulation(final String _typePopulation) {}

	@Override
	public String getFonction() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setFonction(final String _fonction) {}

	@Override
	public String getCorps() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setCorps(final String _corps) {}

	@Override
	public String getDiscipline() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setDiscipline(final String _discipline) {}

	@Override
	public String getAdresse() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setAdresse(final String _adresse) {}

	@Override
	public String getBureau1() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setBureau1(final String _bureau1) {}

	@Override
	public String getBureau2() {
		return StringUtils.EMPTY;
	}


	@Override
	public void setBureau2(final String _bureau2) {}

	@Override
	public String getTelephone1() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setTelephone1(final String _telephone1) {}

	@Override
	public String getTelephone2() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setTelephone2(final String _telephone2) {}

	@Override
	public String getTelecopie1() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setTelecopie1(final String _telecopie1) {}

	@Override
	public String getTelecopie2() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setTelecopie2(final String _telecopie2) {}

	@Override
	public String getDisciplineEnseignee() {
		return StringUtils.EMPTY;
	}

	@Override
	public void setDisciplineEnseignee(final String _disciplineEnseignee) {}

	/**
	 * Renvoie la rubrique de la fiche.
	 *
	 * @return the infos rubrique
	 *
	 * @throws Exception
	 *             the exception
	 */
	public InfosRubriques getInfosRubrique() throws Exception {
		InfosRubriques res = new InfosRubriques("");
		if (StringUtils.isNotBlank(getCodeRubrique())) {
			final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
			final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
			if (rubriqueBean != null) {
				res = new InfosRubriques(rubriqueBean);
			}
		}
		return res;
	}

	/**
	 * Gets the Entreprise.
	 *
	 * @return the Entreprise
	 */
	public String getEntreprise() {
		return persistenceBean.getEntreprise();
	}

	/**
	 * Sets the entreprise.
	 *
	 * @param entreprise
	 *            the new entreprise
	 */
	public void setEntreprise(String entreprise) {
		persistenceBean.setEntreprise(entreprise);
	}

	/**
	 * Gets the accrocheTemoignage.
	 *
	 * @return the accrocheTemoignage
	 */
	public String getAccrocheTemoignage() {
		return persistenceBean.getAccrocheTemoignage();
	}

	/**
	 * Sets the accrocheTemoignage.
	 *
	 * @param accrocheTemoignage
	 *            the new accrocheTemoignage
	 */
	public void setAccrocheTemoignage(String accrocheTemoignage) {
		persistenceBean.setAccrocheTemoignage(accrocheTemoignage);
	}

	/**
	 * Gets the Temoignage.
	 *
	 * @return the Temoignage
	 */
	public String getTemoignage() {
		return persistenceBean.getTemoignage();
	}

	/**
	 * Sets the temoignage.
	 *
	 * @param temoignage
	 *            the new temoignage
	 */
	public void setTemoignage(String temoignage) {
		persistenceBean.setTemoignage(temoignage);
	}

	/**
	 * Renvoie le contenu formate en HTML pour l'attribut temoignage.
	 */
	public String getFormatedTemoignage() throws Exception {
		return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getTemoignage());
	}


	/**
	 * Gets the urlLinkedin.
	 *
	 * @return the urlLinkedin
	 */
	public String getUrlProfilLinkedin() {
		return persistenceBean.getUrlProfilLinkedin();
	}

	/**
	 * Sets the urlLinkedin.
	 *
	 * @param urlLinkedin
	 *            the new urlLinkedin
	 */
	public void setUrlProfilLinkedin(String urlLinkedin) {
		persistenceBean.setUrlProfilLinkedin(urlLinkedin);
	}

	/**
	 * Gets the urlViadeo.
	 *
	 * @return the urlViadeo
	 */
	public String getUrlProfilViadeo() {
		return persistenceBean.getUrlProfilViadeo();
	}

	/**
	 * Sets the urlViadeo.
	 *
	 * @param urlViadeo
	 *            the new urlViadeo
	 */
	public void setUrlProfilViadeo(String urlViadeo) {
		persistenceBean.setUrlProfilViadeo(urlViadeo);
	}
}
