/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.etudiantetalumni.util.EtudiantetalumniUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.AnnuaireetudiantsBean;
import com.ksup.objetspartages.service.ServiceAnnuaireEtudiants;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.EscapeString;
import com.univ.utils.FicheUnivHelper;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.Operateur;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;
import com.univ.utils.sql.operande.TypeOperande;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet annuaireetudiants.
 */
@FicheAnnotation()
public class Annuaireetudiants extends AbstractFicheRestriction<AnnuaireetudiantsBean> implements AnnuaireModele, FicheRattachementsSecondaires {

    private final ServiceLabel serviceLabel;

    public Annuaireetudiants() {
        serviceLabel = ServiceManager.getService(ServiceLabel.class);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (codes == null || codes.length() == 0) {
            return res;
        }
        final Annuaireetudiants annuaireetudiants = new Annuaireetudiants();
        annuaireetudiants.init();
        if (langue == null || langue.length() == 0) {
            langue = "0";
        }
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = annuaireetudiants.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.toString());
            if (count == 0) {
                count = annuaireetudiants.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (annuaireetudiants.nextItem()) {
                    res += annuaireetudiants.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Annuaireetudiants getFicheAnnuaireetudiants(final OMContext ctx, final String code, String langue) throws Exception {
        Annuaireetudiants fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Annuaireetudiants();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.toString());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.toString());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        setCivilite(StringUtils.EMPTY);
        setNom(StringUtils.EMPTY);
        setPrenom(StringUtils.EMPTY);
        setCodeRattachementAutres(StringUtils.EMPTY);
        setCodeFormation(StringUtils.EMPTY);
        setPromotion(StringUtils.EMPTY);
        setAdresseMail(StringUtils.EMPTY);
        setContact(StringUtils.EMPTY);
        setListeRouge(StringUtils.EMPTY);
        setDiscipline(StringUtils.EMPTY);
        setNiveauDiplome(StringUtils.EMPTY);
        setTypeDiplome(StringUtils.EMPTY);
        setCv(StringUtils.EMPTY);
        setCompetencesDiplome(StringUtils.EMPTY);
        setCompetencesGenerales(StringUtils.EMPTY);
        setCompetencesInformatiques(StringUtils.EMPTY);
        setLanguesParlees(StringUtils.EMPTY);
        setObjectifProfessionnel(StringUtils.EMPTY);
        setSecteursActivites(StringUtils.EMPTY);
        setExperiencesProfessionnelles(StringUtils.EMPTY);
        setAdresseSitePerso(StringUtils.EMPTY);
        setCentresInterets(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    public String getLibelleSansFallback() {
        return ((ServiceAnnuaireEtudiants) getServiceFiche()).getLibelleSansFallback(persistenceBean);
    }

    /**
     * Renvoie le libelle pour l'attribut civilite.
     */
    public String getLibelleCivilite() {
        return ((ServiceAnnuaireEtudiants) getServiceFiche()).getLibelleCivilite(persistenceBean);
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementAutres.
     */
    public List<StructureModele> getFichesCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listCodeRattachementAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementAutres.nextToken(), getLangue());
            if(currentStructure != null) {
                listCodeRattachementAutres.add(currentStructure);
            }
        }
        return listCodeRattachementAutres;
    }

    /**
     * Renvoie le libelle pour l'attribut codeFormation.
     */
    public String getLibelleCodeFormation() throws Exception {
        String listeLibelles = StringUtils.EMPTY;
        final StringTokenizer stUe = new StringTokenizer(getCodeFormation(), ";");
        String code;
        String langue;
        String annee;
        StringTokenizer stUeCourant;
        int token = 0, idxUe = 0;
        //la liste des formations obtenues est composée de la manière suivante :
        // code,langue,annee
        while (stUe.hasMoreTokens()) {
            code = StringUtils.EMPTY;
            langue = "0";
            annee = StringUtils.EMPTY;
            stUeCourant = new StringTokenizer(stUe.nextToken(), ",");
            token = 0;
            while (stUeCourant.hasMoreTokens()) {
                switch (token) {
                    case 0:
                        code = stUeCourant.nextToken();
                        break;
                    case 1:
                        langue = StringUtils.replace(stUeCourant.nextToken(), "LANGUE=", StringUtils.EMPTY);
                        break;
                    case 2:
                        annee = stUeCourant.nextToken();
                        break;
                    default:
                        stUeCourant.nextToken();
                        break;
                }
                token++;
            }
            LangueUtil.getLocale(new Integer(langue));
            if (idxUe > 0) {
                listeLibelles += ";";
            }
            if (!code.equals(StringUtils.EMPTY)) {
                listeLibelles += StringUtils.replace(FicheUnivHelper.getFiche(EtudiantetalumniUtil.NOM_OBJET_FORMATION, code, langue).getLibelleAffichable(), ";", "-");
            }
            if (!annee.equals(StringUtils.EMPTY)) {
                listeLibelles += " [" + FicheUnivHelper.getLibelle(this, "annee_universitaire", annee) + "]";
            }
            idxUe++;
        }
        String sCumullibelles = listeLibelles.replaceAll("\"", "\\\"");
        sCumullibelles = sCumullibelles.replaceAll("\'", "\\\'");
        return sCumullibelles;
    }

    /**
     * Renvoie la liste des formations correspondant a l'attribut codeFormation.
     */
    public List<FicheUniv> getFichesFormation() throws Exception {
        final List<FicheUniv> listFormations = new ArrayList<>();
        final StringTokenizer stDiplomesObtenus = new StringTokenizer(getCodeFormation(), ";");
        StringTokenizer stFormation;
        String code = StringUtils.EMPTY, langue = "0";
        int token;
        while (stDiplomesObtenus.hasMoreTokens()) {
            stFormation = new StringTokenizer(stDiplomesObtenus.nextToken(), ",");
            token = 0;
            while (stFormation.hasMoreTokens()) {
                switch (token) {
                    case 0:
                        code = stFormation.nextToken();
                        break;
                    case 1:
                        langue = stFormation.nextToken();
                        break;
                    default:
                        stFormation.nextToken();
                        break;
                }
                token++;
            }
            listFormations.add(FicheUnivHelper.getFiche(EtudiantetalumniUtil.NOM_OBJET_FORMATION, code, langue));
        }
        return listFormations;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut contact.
     */
    public String getFormatedContact() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getContact());
    }

    /**
     * Renvoie le libelle pour l'attribut discipline.
     */
    public String getLibelleDiscipline() {
        return LabelUtils.getLibelle("55", getDiscipline(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut niveauDiplome.
     */
    public String getLibelleNiveauDiplome() {
        return FicheUnivHelper.getLibelle(this, "annee_postbac", getNiveauDiplome());
    }

    /**
     * Renvoie le libelle pour l'attribut typeDiplome.
     */
    public String getLibelleTypeDiplome() {
        return LabelUtils.getLibelle("20", getTypeDiplome(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut cv.
     */
    public String getFormatedCv() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCv());
    }

    /**
     * Renvoie le libelle pour l'attribut competencesDiplome.
     */
    public String getLibelleCompetencesDiplome() {
        return LabelUtils.getLibelle("51", getCompetencesDiplome(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut secteursActivites.
     */
    public String getLibelleSecteursActivites() {
        return LabelUtils.getLibelle("65", getSecteursActivites(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut secteursActivites. Si plusieurs libellés, ils sont renvoyés sous forme de listes à puces li
     */
    public String getLibelleSecteursActivitesListe() {
        final String _codes = getSecteursActivites();
        String code;
        String res = StringUtils.EMPTY;
        if (_codes != null && !_codes.equals(StringUtils.EMPTY) && !_codes.equals("0000")) {
            final StringTokenizer st = new StringTokenizer(_codes, ";");
            while (st.hasMoreTokens()) {
                code = st.nextToken();
                final LabelDto labelDto = serviceLabel.getByTypeCodeLanguage("65", code, getLangue());
                if (labelDto != null) {
                    res += "<li>" + serviceLabel.getDisplayableLibelle(labelDto) + "</li>";
                }
            }
        }
        return res;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String nom = RequeteUtil.renvoyerParametre(requete, "NOM");
        final String prenom = RequeteUtil.renvoyerParametre(requete, "PRENOM");
        final String codeFormation = RequeteUtil.renvoyerParametre(requete, "CODE_FORMATION");
        final String discipline = RequeteUtil.renvoyerParametre(requete, "DISCIPLINE");
        final String niveauDiplome = RequeteUtil.renvoyerParametre(requete, "NIVEAU_DIPLOME");
        final String typeDiplome = RequeteUtil.renvoyerParametre(requete, "TYPE_DIPLOME");
        final String competencesDiplome = RequeteUtil.renvoyerParametre(requete, "COMPETENCES_DIPLOME");
        final String competencesGenerales = RequeteUtil.renvoyerParametre(requete, "COMPETENCES_GENERALES");
        final String competencesInformatiques = RequeteUtil.renvoyerParametre(requete, "COMPETENCES_INFORMATIQUES");
        final String languesParlees = RequeteUtil.renvoyerParametre(requete, "LANGUES_PARLEES");
        final String secteursActivites = RequeteUtil.renvoyerParametre(requete, "SECTEURS_ACTIVITES");
        final String promotion = RequeteUtil.renvoyerParametre(requete, "PROMOTION");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String codeFiche = RequeteUtil.renvoyerParametre(requete, "CODE");
        String order = "";
        if (selection.length() > 0) {
            // Dernières pages libres publiées
            if (selection.equals("0007")) {
                order = "META.META_DATE_MISE_EN_LIGNE DESC";
            }
        }
        return select(nom, prenom, codeFormation, discipline, niveauDiplome, typeDiplome, competencesDiplome, competencesGenerales, competencesInformatiques, languesParlees, secteursActivites, promotion, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, codeFiche, nombre, order);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Annuaireetudiants a partir de l'ensemble des criteres.
     */
    public int select(final String nom, final String prenom, final String codeFormation, final String discipline, final String niveauDiplome, final String typeDiplome, final String competencesDiplome, final String competencesGenerales, final String competencesInformatiques, final String languesParlees, final String secteursActivites, final String promotion, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final String order) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (isNotEmpty(code)) {
            where.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        if (isNotEmpty(nom)) {
            where.and(ConditionHelper.like("T1.NOM", nom, "%", "%"));
        }
        if (isNotEmpty(prenom)) {
            where.and(ConditionHelper.like("T1.PRENOM", prenom, "%", "%"));
        }
        if (isNotEmpty(codeFormation)) {
            where.and(ConditionHelper.genericConditionSurColonne("T1.CODE_FORMATION", "'" + EscapeString.escapeSql(codeFormation) + ",%,%'", TypeOperande.NON_ECHAPABLE, Operateur.LIKE));
        }
        if (isNotEmpty(discipline) && !"0000".equals(discipline)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DISCIPLINE", discipline));
        }
        if (isNotEmpty(niveauDiplome) && !"0000".equals(niveauDiplome)) {
            where.and(ConditionHelper.egalVarchar("T1.NIVEAU_DIPLOME", niveauDiplome));
        }
        if (isNotEmpty(typeDiplome) && !"0000".equals(typeDiplome)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_DIPLOME", typeDiplome));
        }
        if (isNotEmpty(competencesDiplome) && !"0000".equals(competencesDiplome)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.COMPETENCES_DIPLOME", competencesDiplome));
        }
        if (isNotEmpty(competencesGenerales)) {
            where.and(ConditionHelper.like("T1.COMPETENCES_GENERALES", competencesGenerales, "%", "%"));
        }
        if (isNotEmpty(competencesInformatiques)) {
            where.and(ConditionHelper.like("T1.COMPETENCES_INFORMATIQUES", competencesInformatiques, "%", "%"));
        }
        if (isNotEmpty(languesParlees)) {
            where.and(ConditionHelper.like("T1.LANGUES_PARLEES", languesParlees, "%", "%"));
        }
        if (isNotEmpty(secteursActivites) && !"0000".equals(secteursActivites)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.SECTEURS_ACTIVITES", secteursActivites));
        }
        if (isNotEmpty(promotion) && !"0000".equals(promotion)) {
            where.and(ConditionHelper.genericConditionSurColonne("T1.CODE_FORMATION", "'%,%," + EscapeString.escapeSql(promotion) + "'", TypeOperande.NON_ECHAPABLE, Operateur.LIKE));
        }
        if (isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructureAnnuaire = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            conditionStructureAnnuaire.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
            where.and(conditionStructureAnnuaire);
        }
        if (isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.NOM", SensDeTri.ASC);
        }
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    public Long getIdAnnuaireetudiants(){
        return persistenceBean.getId();
    }

    public void setIdAnnuaireetudiants(Long id){
        persistenceBean.setId(id);
    }

    /**
     * Gets the civilite.
     *
     * @return the civilite
     */
    public String getCivilite() {
        return persistenceBean.getCivilite();
    }

    /**
     * Sets the civilite.
     *
     * @param civilite
     *            the new civilite
     */
    public void setCivilite(String civilite) {
        persistenceBean.setCivilite(civilite);
    }

    /**
     * Gets the nom.
     *
     * @return the nom
     */
    public String getNom() {
        return persistenceBean.getNom();
    }

    /**
     * Sets the nom.
     *
     * @param nom
     *            the new nom
     */
    public void setNom(String nom) {
        persistenceBean.setNom(nom);
    }

    /**
     * Gets the prenom.
     *
     * @return the prenom
     */
    public String getPrenom() {
        return persistenceBean.getPrenom();
    }

    /**
     * Sets the prenom.
     *
     * @param prenom
     *            the new prenom
     */
    public void setPrenom(String prenom) {
        persistenceBean.setPrenom(prenom);
    }

    /**
     * Gets the date naissance.
     *
     * @return the date naissance
     */
    public Date getDateNaissance() {
        return persistenceBean.getDateNaissance();
    }

    /**
     * Sets the date naissance.
     *
     * @param dateNaissance
     *            the new date naissance
     */
    public void setDateNaissance(Date dateNaissance) {
        persistenceBean.setDateNaissance(dateNaissance);
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param codeRattachementAutres
     *            the new code rattachement autres
     */
    public void setCodeRattachementAutres(String codeRattachementAutres) {
        persistenceBean.setCodeRattachementAutres(codeRattachementAutres);
    }

    /**
     * Gets the code formation.
     *
     * @return the code formation
     */
    public String getCodeFormation() {
        return persistenceBean.getCodeFormation();
    }

    /**
     * Sets the code formation.
     *
     * @param codeFormation
     *            the new code formation
     */
    public void setCodeFormation(String codeFormation) {
        persistenceBean.setCodeFormation(codeFormation);
    }

    /**
     * Gets the promotion.
     *
     * @return the promotion
     */
    public String getPromotion() {
        return persistenceBean.getPromotion();
    }

    /**
     * Sets the promotion.
     *
     * @param promotion
     *            the new promotion
     */
    public void setPromotion(String promotion) {
        persistenceBean.setPromotion(promotion);
    }

    /**
     * Gets the adresse mail.
     *
     * @return the adresse mail
     */
    public String getAdresseMail() {
        return persistenceBean.getAdresseMail();
    }

    /**
     * Sets the adresse mail.
     *
     * @param adresseMail
     *            the new adresse mail
     */
    public void setAdresseMail(String adresseMail) {
        persistenceBean.setAdresseMail(adresseMail);
    }

    /**
     * Gets the contact.
     *
     * @return the contact
     */
    public String getContact() {
        return persistenceBean.getContact();
    }

    /**
     * Sets the contact.
     *
     * @param contact
     *            the new contact
     */
    public void setContact(String contact) {
        persistenceBean.setContact(contact);
    }

    /**
     * Gets the liste rouge.
     *
     * @return the liste rouge
     */
    public String getListeRouge() {
        return persistenceBean.getListeRouge();
    }

    /**
     * Sets the liste rouge.
     *
     * @param listeRouge
     *            the new liste rouge
     */
    public void setListeRouge(String listeRouge) {
        persistenceBean.setListeRouge(listeRouge);
    }

    /**
     * Gets the discipline.
     *
     * @return the discipline
     */
    public String getDiscipline() {
        return persistenceBean.getDiscipline();
    }

    /**
     * Sets the discipline.
     *
     * @param discipline
     *            the new discipline
     */
    public void setDiscipline(String discipline) {
        persistenceBean.setDiscipline(discipline);
    }

    /**
     * Gets the niveau diplome.
     *
     * @return the niveau diplome
     */
    public String getNiveauDiplome() {
        return persistenceBean.getNiveauDiplome();
    }

    /**
     * Sets the niveau diplome.
     *
     * @param niveauDiplome
     *            the new niveau diplome
     */
    public void setNiveauDiplome(String niveauDiplome) {
        persistenceBean.setNiveauDiplome(niveauDiplome);
    }

    /**
     * Gets the type diplome.
     *
     * @return the type diplome
     */
    public String getTypeDiplome() {
        return persistenceBean.getTypeDiplome();
    }

    /**
     * Sets the type diplome.
     *
     * @param typeDiplome
     *            the new type diplome
     */
    public void setTypeDiplome(String typeDiplome) {
        persistenceBean.setTypeDiplome(typeDiplome);
    }

    /**
     * Gets the cv.
     *
     * @return the cv
     */
    public String getCv() {
        return persistenceBean.getCv();
    }

    /**
     * Sets the cv.
     *
     * @param cv
     *            the new cv
     */
    public void setCv(String cv) {
        persistenceBean.setCv(cv);
    }

    /**
     * Gets the competences diplome.
     *
     * @return the competences diplome
     */
    public String getCompetencesDiplome() {
        return persistenceBean.getCompetencesDiplome();
    }

    /**
     * Sets the competences diplome.
     *
     * @param competencesDiplome
     *            the new competences diplome
     */
    public void setCompetencesDiplome(String competencesDiplome) {
        persistenceBean.setCompetencesDiplome(competencesDiplome);
    }

    /**
     * Gets the competences generales.
     *
     * @return the competences generales
     */
    public String getCompetencesGenerales() {
        return persistenceBean.getCompetencesGenerales();
    }

    /**
     * Sets the competences generales.
     *
     * @param competencesGenerales
     *            the new competences generales
     */
    public void setCompetencesGenerales(String competencesGenerales) {
        persistenceBean.setCompetencesGenerales(competencesGenerales);
    }

    /**
     * Gets the competences informatiques.
     *
     * @return the competences informatiques
     */
    public String getCompetencesInformatiques() {
        return persistenceBean.getCompetencesInformatiques();
    }

    /**
     * Sets the competences informatiques.
     *
     * @param competencesInformatiques
     *            the new competences informatiques
     */
    public void setCompetencesInformatiques(String competencesInformatiques) {
        persistenceBean.setCompetencesInformatiques(competencesInformatiques);
    }

    /**
     * Gets the langues parlees.
     *
     * @return the langues parlees
     */
    public String getLanguesParlees() {
        return persistenceBean.getLanguesParlees();
    }

    /**
     * Sets the langues parlees.
     *
     * @param languesParlees
     *            the new langues parlees
     */
    public void setLanguesParlees(String languesParlees) {
        persistenceBean.setLanguesParlees(languesParlees);
    }

    /**
     * Gets the objectif professionnel.
     *
     * @return the objectif professionnel
     */
    public String getObjectifProfessionnel() {
        return persistenceBean.getObjectifProfessionnel();
    }

    /**
     * Sets the objectif professionnel.
     *
     * @param objectifProfessionnel
     *            the new objectif professionnel
     */
    public void setObjectifProfessionnel(String objectifProfessionnel) {
        persistenceBean.setObjectifProfessionnel(objectifProfessionnel);
    }

    /**
     * Gets the secteurs activites.
     *
     * @return the secteurs activites
     */
    public String getSecteursActivites() {
        return persistenceBean.getSecteursActivites();
    }

    /**
     * Sets the secteurs activites.
     *
     * @param secteursActivites
     *            the new secteurs activites
     */
    public void setSecteursActivites(String secteursActivites) {
        persistenceBean.setSecteursActivites(secteursActivites);
    }

    /**
     * Gets the experiences professionnelles.
     *
     * @return the experiences professionnelles
     */
    public String getExperiencesProfessionnelles() {
        return persistenceBean.getExperiencesProfessionnelles();
    }

    /**
     * Sets the experiences professionnelles.
     *
     * @param experiencesProfessionnelles
     *            the new experiences professionnelles
     */
    public void setExperiencesProfessionnelles(String experiencesProfessionnelles) {
        persistenceBean.setExperiencesProfessionnelles(experiencesProfessionnelles);
    }

    /**
     * Gets the photo.
     *
     * @return the photo
     */
    public Long getPhoto() {
        return persistenceBean.getPhoto();
    }

    /**
     * Sets the photo.
     *
     * @param photo
     *            the new photo
     */
    public void setPhoto(Long photo) {
        persistenceBean.setPhoto(photo);
    }

    /**
     * Gets the adresse site perso.
     *
     * @return the adresse site perso
     */
    public String getAdresseSitePerso() {
        return persistenceBean.getAdresseSitePerso();
    }

    /**
     * Sets the adresse site perso.
     *
     * @param adresseSitePerso
     *            the new adresse site perso
     */
    public void setAdresseSitePerso(String adresseSitePerso) {
        persistenceBean.setAdresseSitePerso(adresseSitePerso);
    }

    /**
     * Gets the centres interets.
     *
     * @return the centres interets
     */
    public String getCentresInterets() {
        return persistenceBean.getCentresInterets();
    }

    /**
     * Sets the centres interets.
     *
     * @param centresInterets
     *            the new centres interets
     */
    public void setCentresInterets(String centresInterets) {
        persistenceBean.setCentresInterets(centresInterets);
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    /**
     * Gets the date suppression.
     *
     * @return the date suppression
     */
    public Date getDateSuppression() {
        return persistenceBean.getDateSuppression();
    }

    /**
     * Sets the date suppression.
     *
     * @param dateSuppression
     *            the new date suppression
     */
    public void setDateSuppression(Date dateSuppression) {
        persistenceBean.setDateSuppression(dateSuppression);
    }
    
    @Override
    public String getTypePopulation() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setTypePopulation(final String _typePopulation) {
    }

    @Override
    public String getFonction() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setFonction(final String _fonction) {
    }

    @Override
    public String getCorps() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setCorps(final String _corps) {}

    @Override
    public String getAdresse() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setAdresse(final String _adresse) {
    }

    @Override
    public String getBureau1() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setBureau1(final String _bureau1) {
    }

    @Override
    public String getBureau2() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setBureau2(final String _bureau2) {
    }

    @Override
    public String getTelephone1() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setTelephone1(final String _telephone1) {
    }

    @Override
    public String getTelephone2() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setTelephone2(final String _telephone2) {
    }

    @Override
    public String getTelecopie1() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setTelecopie1(final String _telecopie1) {
    }

    @Override
    public String getTelecopie2() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setTelecopie2(final String _telecopie2) {
    }

    @Override
    public String getDisciplineEnseignee() {
        return StringUtils.EMPTY;
    }

    @Override
    public void setDisciplineEnseignee(final String _disciplineEnseignee) {
    }

    /**
     * Renvoie la rubrique de la fiche.
     *
     * @return the infos rubrique
     *
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }
}
