/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceFiche;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.AnnuaireksupBean;
import com.ksup.objetspartages.service.ServiceAnnuaireksup;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet annuaire.
 */
@FicheAnnotation()
public class Annuaireksup extends AbstractFicheRestriction<AnnuaireksupBean> implements AnnuaireModele, FicheRattachementsSecondaires {

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        // Temps systeme en ms
        final String timeStamp = Long.toString(System.currentTimeMillis());
        setCode(timeStamp);
        setCivilite(StringUtils.EMPTY);
        setNom(StringUtils.EMPTY);
        setPrenom(StringUtils.EMPTY);
        setPhoto((long) 0);
        setTypePopulation(StringUtils.EMPTY);
        setFonction(StringUtils.EMPTY);
        setCorps(StringUtils.EMPTY);
        setDiscipline(StringUtils.EMPTY);
        setDisciplineEnseignee(StringUtils.EMPTY);
        setThemeRecherche(StringUtils.EMPTY);
        setAdresse(StringUtils.EMPTY);
        setAdresseMail(StringUtils.EMPTY);
        setRattachementBandeau(StringUtils.EMPTY);
        setBureau1(StringUtils.EMPTY);
        setTelephone1(StringUtils.EMPTY);
        setTelecopie1(StringUtils.EMPTY);
        setBureau2(StringUtils.EMPTY);
        setTelephone2(StringUtils.EMPTY);
        setTelecopie2(StringUtils.EMPTY);
        setCodeRattachementAutres(StringUtils.EMPTY);
        setListeRouge(StringUtils.EMPTY);
        setAdresseSitePerso("http://");
        setCv(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
        setCodeStructure(StringUtils.EMPTY);
    }

    /**
     * Récupération du libellé à afficher
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(this.persistenceBean);
    }

    public String getLibelleSansFallback() {
        return ((ServiceAnnuaireksup) getServiceFiche()).getLibelleSansFallback(this.persistenceBean);
    }

    /**
     * Sélection à partir du rédacteur
     */
    public int selectParRedacteur(final String _codeRedacteur) throws Exception {
        final ClauseWhere whereCodeRedacteur = new ClauseWhere(ConditionHelper.egalVarchar("CODE_REDACTEUR", _codeRedacteur));
        return select(whereCodeRedacteur.formaterSQL());
    }

    /**
     * Renvoie le libelle pour l'attribut civilite.
     */
    public String getLibelleCivilite() {
        return ((ServiceAnnuaireksup) getServiceFiche()).getLibelleCivilite(this.persistenceBean);
    }

    /**
     * Renvoie le libelle pour l'attribut typePopulation.
     */
    public String getLibelleTypePopulation() {
        return LabelUtils.getLibelle("10", getTypePopulation(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut discipline.
     */
    public String getLibelleDiscipline() {
        return LabelUtils.getLibelle("02", getDiscipline(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementAutres.
     */
    public List<StructureModele> getFichesCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listCodeRattachementAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementAutres.nextToken(), getLangue());
            if(currentStructure != null) {
                listCodeRattachementAutres.add(currentStructure);
            }
        }
        return listCodeRattachementAutres;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut cv.
     */
    public String getFormatedCv() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCv());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut theme recherche.
     */
    public String getFormatedThemeRecherche() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getThemeRecherche());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut Discipline Enseignee.
     */
    public String getFormatedDisciplineEnseignee() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getDisciplineEnseignee());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie le libelle pour l'attribut codeStructure.
     */
    public String getLibelleCodeStructure() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeStructure(), getLangue());
    }

    /**
     * Renvoie la fiche structure correspondant a l'attribut codeStructure.
     */
    public FicheUniv getFicheCodeStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeStructure(), getLangue(), false);
    }

    /**
     * Renvoie la structure de la fiche.
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue(), false);
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String nom = RequeteUtil.renvoyerParametre(requete, "NOM");
        final String prenom = RequeteUtil.renvoyerParametre(requete, "PRENOM");
        final String typePopulation = RequeteUtil.renvoyerParametre(requete, "TYPE_POPULATION");
        final String fonction = RequeteUtil.renvoyerParametre(requete, "FONCTION");
        final String discipline = RequeteUtil.renvoyerParametre(requete, "DISCIPLINE");
        final String disciplineEnseignee = RequeteUtil.renvoyerParametre(requete, "DISCIPLINE_ENSEIGNEE");
        final String themeRecherche = RequeteUtil.renvoyerParametre(requete, "THEME_RECHERCHE");
        final String listeRouge = RequeteUtil.renvoyerParametre(requete, "LISTE_ROUGE");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_STRUCTURE");
        if (codeRattachement == null || codeRattachement.equals("")) {
            codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        }
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String codeFiche = RequeteUtil.renvoyerParametre(requete, "CODE");
        String order = "";
        if (selection.length() > 0) {
            // Dernières pages libres publiées
            if (selection.equals("0007")) {
                order = "META.META_DATE_MISE_EN_LIGNE DESC";
            }
        }
        return select(nom, prenom, typePopulation, fonction, discipline, disciplineEnseignee, themeRecherche, listeRouge, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, codeFiche, nombre, order);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Annuaire a partir de l'ensemble des criteres.
     */
    public int select(final String nom, final String prenom, final String typePopulation, final String fonction, final String discipline, final String disciplineEnseignee, final String themeRecherche, final String listeRouge, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final String order) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (isNotEmpty(nom)) {
            where.setPremiereCondition(ConditionHelper.like("T1.NOM", nom, "%", "%"));
        }
        if (isNotEmpty(prenom)) {
            where.and(ConditionHelper.like("T1.PRENOM", prenom, "%", "%"));
        }
        if (isNotEmpty(typePopulation) && !"0000".equals(typePopulation)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_POPULATION", typePopulation));
        }
        if (isNotEmpty(fonction)) {
            where.and(ConditionHelper.rechercheMots("T1.FONCTION", fonction));
        }
        if (isNotEmpty(discipline) && !"0000".equals(discipline)) {
            final ConditionList conditionDiscipline = new ConditionList(ConditionHelper.likePourValeursMultiple("T1.DISCIPLINE", discipline));
            if (isNotEmpty(disciplineEnseignee)) {
                conditionDiscipline.or(ConditionHelper.rechercheMots("T1.DISCIPLINE_ENSEIGNEE", LabelUtils.getLibelle("02", discipline, ctx.getLocale())));
            }
            where.and(conditionDiscipline);
        }
        if (isNotEmpty(disciplineEnseignee)) {
            where.and(ConditionHelper.rechercheMots("T1.DISCIPLINE_ENSEIGNEE", disciplineEnseignee));
        }
        if (isNotEmpty(themeRecherche)) {
            where.and(ConditionHelper.rechercheMots("T1.THEME_RECHERCHE", themeRecherche));
        }
        if (isNotEmpty(listeRouge)) {
            where.and(ConditionHelper.egalVarchar("T1.LISTE_ROUGE", listeRouge));
        }
        if (isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructureAnnuaire = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            conditionStructureAnnuaire.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
            where.and(conditionStructureAnnuaire);
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.NOM", ClauseOrderBy.SensDeTri.ASC).orderBy("T1.PRENOM", ClauseOrderBy.SensDeTri.ASC);
        }
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    /**
     * Renvoie la rubrique de la fiche.
     *
     * @return the infos rubrique
     *
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }


    public String getCodeStructure() {
        return persistenceBean.getCodeStructure();
    }

    public String getRattachementBandeau() {
        return persistenceBean.getRattachementBandeau();
    }

    public void setRattachementBandeau(String rattachementBandeau) {
        persistenceBean.setRattachementBandeau(rattachementBandeau);
    }


    public void setCodeStructure(String codeStructure) {
        persistenceBean.setCodeStructure(codeStructure);
    }

    @Override
    public String getCivilite() {
        return persistenceBean.getCivilite();
    }

    @Override
    public void setCivilite(String civilite) {
        persistenceBean.setCivilite(civilite);
    }

    @Override
    public String getNom() {
        return persistenceBean.getNom();
    }

    @Override
    public void setNom(String nom) {
        persistenceBean.setNom(nom);
    }

    @Override
    public String getPrenom() {
        return persistenceBean.getPrenom();
    }

    @Override
    public void setPrenom(String prenom) {
        persistenceBean.setPrenom(prenom);
    }

    @Override
    public String getTypePopulation() {
        return persistenceBean.getTypePopulation();
    }

    @Override
    public void setTypePopulation(String typePopulation) {
        persistenceBean.setTypePopulation(typePopulation);
    }

    @Override
    public String getFonction() {
        return persistenceBean.getFonction();
    }

    @Override
    public void setFonction(String fonction) {
        persistenceBean.setFonction(fonction);
    }

    @Override
    public String getCorps() {
        return persistenceBean.getCorps();
    }

    @Override
    public void setCorps(String corps) {
        persistenceBean.setCorps(corps);
    }

    @Override
    public String getDiscipline() {
        return persistenceBean.getDiscipline();
    }

    @Override
    public void setDiscipline(String discipline) {
        persistenceBean.setDiscipline(discipline);
    }

    @Override
    public String getAdresse() {
        return persistenceBean.getAdresse();
    }

    @Override
    public void setAdresse(String adresse) {
        persistenceBean.setAdresse(adresse);
    }

    @Override
    public String getAdresseMail() {
        return persistenceBean.getAdresseMail();
    }

    @Override
    public void setAdresseMail(String adresseMail) {
        persistenceBean.setAdresseMail(adresseMail);
    }

    @Override
    public String getBureau1() {
        return persistenceBean.getBureau1();
    }

    @Override
    public void setBureau1(String bureau1) {
        persistenceBean.setBureau1(bureau1);
    }

    @Override
    public String getBureau2() {
        return persistenceBean.getBureau2();
    }

    @Override
    public void setBureau2(String bureau2) {
        persistenceBean.setBureau2(bureau2);
    }

    @Override
    public String getTelephone1() {
        return persistenceBean.getTelephone1();
    }

    @Override
    public void setTelephone1(String telephone1) {
        persistenceBean.setTelephone1(telephone1);
    }

    @Override
    public String getTelephone2() {
        return persistenceBean.getTelephone2();
    }

    @Override
    public void setTelephone2(String telephone2) {
        persistenceBean.setTelephone2(telephone2);
    }

    @Override
    public String getTelecopie1() {
        return persistenceBean.getTelecopie1();
    }

    @Override
    public void setTelecopie1(String telecopie1) {
        persistenceBean.setTelecopie1(telecopie1);
    }

    @Override
    public String getTelecopie2() {
        return persistenceBean.getTelecopie2();
    }

    @Override
    public void setTelecopie2(String telecopie2) {
        persistenceBean.setTelecopie2(telecopie2);
    }

    @Override
    public String getListeRouge() {
        return persistenceBean.getListeRouge();
    }

    @Override
    public void setListeRouge(String listeRouge) {
        persistenceBean.setListeRouge(listeRouge);
    }

    @Override
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    @Override
    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    @Override
    public String getCv() {
        return persistenceBean.getCv();
    }

    @Override
    public void setCv(String cv) {
        persistenceBean.setCv(cv);
    }

    @Override
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    @Override
    public void setCodeRattachementAutres(String codeRattachement) {
        persistenceBean.setCodeRattachementAutres(codeRattachement);
    }

    @Override
    public Date getDateNaissance() {
        return persistenceBean.getDateNaissance();
    }

    @Override
    public void setDateNaissance(Date dateNaissance) {
        persistenceBean.setDateNaissance(dateNaissance);
    }

    @Override
    public String getDisciplineEnseignee() {
        return persistenceBean.getDisciplineEnseignee();
    }

    @Override
    public void setDisciplineEnseignee(String disciplineEnseignee) {
        persistenceBean.setDisciplineEnseignee(disciplineEnseignee);
    }

    @Override
    public String getAdresseSitePerso() {
        return persistenceBean.getAdresseSitePerso();
    }

    @Override
    public void setAdresseSitePerso(String adresseSitePerso) {
        persistenceBean.setAdresseSitePerso(adresseSitePerso);
    }

    public Long getPhoto() {
        return persistenceBean.getPhoto();
    }

    public void setPhoto(Long idPhoto) {
        persistenceBean.setPhoto(idPhoto);
    }

    public String getThemeRecherche() {
        return persistenceBean.getThemeRecherche();
    }

    public void setThemeRecherche(String themeRecherche) {
        persistenceBean.setThemeRecherche(themeRecherche);
    }

    public void setIdAnnuaireksup(long idAnnuaireksup) {
        persistenceBean.setId(idAnnuaireksup);
    }

    public Long getIdAnnuaireksup() {
        return persistenceBean.getId();
    }

    @Override
    public ServiceFiche getServiceFiche(){
        return ServiceManager.getServiceForBean(persistenceBean.getClass());
    }
}
