/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.AssociationetudiantBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet associationetudiant.
 */
@FicheAnnotation(isSaisieFrontOffice = false)
public class Associationetudiant extends AbstractFicheRestriction<AssociationetudiantBean> implements FicheRattachementsSecondaires {

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (codes == null || codes.length() == 0) {
            return res;
        }
        final Associationetudiant associationetudiant = new Associationetudiant();
        associationetudiant.init();
        if (langue == null || langue.length() == 0) {
            langue = "0";
        }
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = associationetudiant.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = associationetudiant.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (associationetudiant.nextItem()) {
                    res += associationetudiant.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Associationetudiant getFicheAssociationetudiant(final OMContext ctx, final String code, String langue) throws Exception {
        Associationetudiant fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Associationetudiant();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        setNom(StringUtils.EMPTY);
        setObjet(StringUtils.EMPTY);
        setTypeAssociation(StringUtils.EMPTY);
        setThematique(StringUtils.EMPTY);
        setCodeRattachementAutres(StringUtils.EMPTY);
        setAdresseEmail(StringUtils.EMPTY);
        setCoordonnees(StringUtils.EMPTY);
        setSiteWeb("http://");
        setNomPresident(StringUtils.EMPTY);
        setAdresseEmailPresident(StringUtils.EMPTY);
        setTelephonePresident(StringUtils.EMPTY);
        setTelecopiePresident(StringUtils.EMPTY);
        setCotisation(StringUtils.EMPTY);
        setActivites(StringUtils.EMPTY);
        setPlanAcces((long) 0);
        setCompositionBureau(StringUtils.EMPTY);
        setPermanences(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     *
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut domaineFormationEtablissement.
     */
    public String getLibelleTypeAssociation() {
        return LabelUtils.getLibelle("34", getTypeAssociation(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut domaineFormationEtablissement.
     */
    public String getLibelleThematique() {
        return LabelUtils.getLibelle("04", getThematique(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementAutres.
     */
    public List<StructureModele> getFichesCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listCodeRattachementAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementAutres.nextToken(), getLangue());
            if(currentStructure != null) {
                listCodeRattachementAutres.add(currentStructure);
            }
        }
        return listCodeRattachementAutres;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut activites.
     */
    public String getFormatedActivites() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getActivites());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut compositionBureau.
     */
    public String getFormatedCompositionBureau() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCompositionBureau());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut permanences.
     */
    public String getFormatedPermanences() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPermanences());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String nom = RequeteUtil.renvoyerParametre(requete, "NOM");
        final String typeAssociation = RequeteUtil.renvoyerParametre(requete, "TYPE_ASSOCIATION");
        final String objet = RequeteUtil.renvoyerParametre(requete, "MOTCLE");
        final String thematique = RequeteUtil.renvoyerParametre(requete, "THEMATIQUE");
        final String nomPresident = RequeteUtil.renvoyerParametre(requete, "NOM_PRESIDENT");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String codeFiche = RequeteUtil.renvoyerParametre(requete, "CODE");
        String order = StringUtils.EMPTY;
        if (selection.length() > 0) {
            // Dernières pages libres publiées
            if (selection.equals("0007")) {
                order = "META.META_DATE_MISE_EN_LIGNE DESC";
            }
        }
        return select(nom, typeAssociation, objet, thematique, nomPresident, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, codeFiche, nombre, order);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Associationetudiant a partir de l'ensemble des criteres.
     */
    public int select(final String nom, final String typeAssociation, final String objet, final String thematique, final String nomPresident, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final String order) throws Exception {
        RequeteSQL requeteSelect;
        final ClauseWhere where = new ClauseWhere();
        if (isNotEmpty(code)) {
            where.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        if (isNotEmpty(nom)) {
            where.and(ConditionHelper.like("T1.NOM", nom, "%", "%"));
        }
        if (isNotEmpty(typeAssociation) && !"0000".equals(typeAssociation)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.TYPE_ASSOCIATION", typeAssociation));
        }
        if (isNotEmpty(objet)) {
            final ConditionList conditionObjet = new ConditionList(ConditionHelper.rechercheMots("T1.OBJET", objet));
            conditionObjet.or(ConditionHelper.rechercheMots("T1.META_KEYWORDS", objet));
            where.and(conditionObjet);
        }
        if (isNotEmpty(thematique) && !"0000".equals(thematique)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.THEMATIQUE", thematique));
        }
        if (isNotEmpty(nomPresident)) {
            where.and(ConditionHelper.like("T1.NOM_PRESIDENT", nomPresident, "%", "%"));
        }
        if (isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructureAnnuaire = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            conditionStructureAnnuaire.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
            where.and(conditionStructureAnnuaire);
        }
        if (isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.NOM", SensDeTri.ASC);
        }
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }


    public Long getIdAssociationetudiant(){
        return persistenceBean.getId();
    }

    public void setIdAssociationetudiant(Long id) {
        persistenceBean.setId(id);
    }

    /**
     * Gets the nom.
     *
     * @return the nom
     */
    public String getNom() {
        return persistenceBean.getNom();
    }

    /**
     * Sets the nom.
     *
     * @param nom
     *            the new nom
     */
    public void setNom(String nom) {
        persistenceBean.setNom(nom);
    }

    /**
     * Gets the objet.
     *
     * @return the objet
     */
    public String getObjet() {
        return persistenceBean.getObjet();
    }

    /**
     * Sets the objet.
     *
     * @param objet
     *            the new objet
     */
    public void setObjet(String objet) {
        persistenceBean.setObjet(objet);
    }

    /**
     * Gets the type association.
     *
     * @return the type association
     */
    public String getTypeAssociation() {
        return persistenceBean.getTypeAssociation();
    }

    /**
     * Sets the type association.
     *
     * @param typeAssociation
     *            the new type association
     */
    public void setTypeAssociation(String typeAssociation) {
        persistenceBean.setTypeAssociation(typeAssociation);
    }

    /**
     * Gets the thematique.
     *
     * @return the thematique
     */
    public String getThematique() {
        return persistenceBean.getThematique();
    }

    /**
     * Sets the thematique.
     *
     * @param thematique
     *            the new thematique
     */
    public void setThematique(String thematique) {
        persistenceBean.setThematique(thematique);
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param codeRattachementAutres
     *            the new code rattachement autres
     */
    public void setCodeRattachementAutres(String codeRattachementAutres) {
        persistenceBean.setCodeRattachementAutres(codeRattachementAutres);
    }

    /**
     * Gets the adresse email.
     *
     * @return the adresse email
     */
    public String getAdresseEmail() {
        return persistenceBean.getAdresseEmail();
    }

    /**
     * Sets the adresse email.
     *
     * @param adresseEmail
     *            the new adresse email
     */
    public void setAdresseEmail(String adresseEmail) {
        persistenceBean.setAdresseEmail(adresseEmail);
    }

    /**
     * Gets the coordonnees.
     *
     * @return the coordonnees
     */
    public String getCoordonnees() {
        return persistenceBean.getCoordonnees();
    }

    /**
     * Sets the coordonnees.
     *
     * @param coordonnees
     *            the new coordonnees
     */
    public void setCoordonnees(String coordonnees) {
        persistenceBean.setCoordonnees(coordonnees);
    }

    /**
     * Gets the site web.
     *
     * @return the site web
     */
    public String getSiteWeb() {
        return persistenceBean.getSiteWeb();
    }

    /**
     * Sets the site web.
     *
     * @param siteWeb
     *            the new site web
     */
    public void setSiteWeb(String siteWeb) {
        persistenceBean.setSiteWeb(siteWeb);
    }

    /**
     * Gets the nom president.
     *
     * @return the nom president
     */
    public String getNomPresident() {
        return persistenceBean.getNomPresident();
    }

    /**
     * Sets the nom president.
     *
     * @param nomPresident
     *            the new nom president
     */
    public void setNomPresident(String nomPresident) {
        persistenceBean.setNomPresident(nomPresident);
    }

    /**
     * Gets the adresse email president.
     *
     * @return the adresse email president
     */
    public String getAdresseEmailPresident() {
        return persistenceBean.getAdresseEmailPresident();
    }

    /**
     * Sets the adresse email president.
     *
     * @param adresseEmailPresident
     *            the new adresse email president
     */
    public void setAdresseEmailPresident(String adresseEmailPresident) {
        persistenceBean.setAdresseEmailPresident(adresseEmailPresident);
    }

    /**
     * Gets the telephone president.
     *
     * @return the telephone president
     */
    public String getTelephonePresident() {
        return persistenceBean.getTelephonePresident();
    }

    /**
     * Sets the telephone president.
     *
     * @param telephonePresident
     *            the new telephone president
     */
    public void setTelephonePresident(String telephonePresident) {
        persistenceBean.setTelephonePresident(telephonePresident);
    }

    /**
     * Gets the telecopie president.
     *
     * @return the telecopie president
     */
    public String getTelecopiePresident() {
        return persistenceBean.getTelecopiePresident();
    }

    /**
     * Sets the telecopie president.
     *
     * @param telecopiePresident
     *            the new telecopie president
     */
    public void setTelecopiePresident(String telecopiePresident) {
        persistenceBean.setTelecopiePresident(telecopiePresident);
    }

    /**
     * Gets the cotisation.
     *
     * @return the cotisation
     */
    public String getCotisation() {
        return persistenceBean.getCotisation();
    }

    /**
     * Sets the cotisation.
     *
     * @param cotisation
     *            the new cotisation
     */
    public void setCotisation(String cotisation) {
        persistenceBean.setCotisation(cotisation);
    }

    /**
     * Gets the activites.
     *
     * @return the activites
     */
    public String getActivites() {
        return persistenceBean.getActivites();
    }

    /**
     * Sets the activites.
     *
     * @param activites
     *            the new activites
     */
    public void setActivites(String activites) {
        persistenceBean.setActivites(activites);
    }

    /**
     * Gets the plan acces.
     *
     * @return the plan acces
     */
    public Long getPlanAcces() {
        return persistenceBean.getPlanAcces();
    }

    /**
     * Sets the plan acces.
     *
     * @param planAcces
     *            the new plan acces
     */
    public void setPlanAcces(Long planAcces) {
        persistenceBean.setPlanAcces(planAcces);
    }

    /**
     * Gets the composition bureau.
     *
     * @return the composition bureau
     */
    public String getCompositionBureau() {
        return persistenceBean.getCompositionBureau();
    }

    /**
     * Sets the composition bureau.
     *
     * @param compositionBureau
     *            the new composition bureau
     */
    public void setCompositionBureau(String compositionBureau) {
        persistenceBean.setCompositionBureau(compositionBureau);
    }

    /**
     * Gets the permanences.
     *
     * @return the permanences
     */
    public String getPermanences() {
        return persistenceBean.getPermanences();
    }

    /**
     * Sets the permanences.
     *
     * @param permanences
     *            the new permanences
     */
    public void setPermanences(String permanences) {
        persistenceBean.setPermanences(permanences);
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    /**
     * Gets the date suppression.
     *
     * @return the date suppression
     */
    public Date getDateSuppression() {
        return persistenceBean.getDateSuppression();
    }

    /**
     * Sets the date suppression.
     *
     * @param dateSuppression
     *            the new date suppression
     */
    public void setDateSuppression(Date dateSuppression) {
        persistenceBean.setDateSuppression(dateSuppression);
    }
}
