/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Transformer;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;
import com.ksup.objetspartages.bean.CoursBean;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.UeupBean;
import com.ksup.objetspartages.service.ServiceCours;
import com.ksup.objetspartages.service.ServiceOffreFormation;
import com.ksup.objetspartages.service.ServiceUeup;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.FicheUnivMgr;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet cours.
 */
@FicheAnnotation()
public class Cours extends AbstractFicheRestriction<CoursBean> implements FicheRattachementsSecondaires {

    /**
     *
     */
    private static final long serialVersionUID = 352871955356206099L;

    private static Logger LOG = LoggerFactory.getLogger(Cours.class);

    private final ServiceRessource serviceRessource;

    public Cours() {
        serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     * @deprecated utilise le contexte uniquement pour la connexion à la bdd. Utiliser {@link Cours#getLibelleAffichable(String, String)}
     */
    @Deprecated
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        return getLibelleAffichable(codes, langue);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (StringUtils.isEmpty(codes)) {
            return res;
        }
        langue = StringUtils.defaultString(langue, "0");
        final Cours cours = new Cours();
        cours.init();
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            final String[] values = code.split(",");
            // On cherche d'abord la version en ligne puis les autres versions
            int count = cours.selectCodeLangueEtat(values[0], langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = cours.selectCodeLangueEtat(values[0], langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (cours.nextItem()) {
                    res += cours.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     * @deprecated utilise le contexte uniquement pour la connexion à la bdd. Utiliser {@link Cours#getFicheCours(String, String)}
     */
    public static Cours getFicheCours(final OMContext ctx, final String code, String langue) throws Exception {
        return getFicheCours(code, langue);
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Cours getFicheCours(final String code, String langue) throws Exception {
        Cours fiche = null;
        if (StringUtils.isNotEmpty(code)) {
            langue = StringUtils.defaultString(langue, "0");
            fiche = new Cours();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    private ServiceCours getServiceCours() {
        return ServiceManager.getServiceForBean(CoursBean.class);
    }

    @Override
    public void add() {
        getServiceCours().add(this.getPersistenceBean());
    }

    @Override
    public void addWithForcedId() throws Exception {
        getServiceCours().addWithForcedId(getPersistenceBean());
    }

    @Override
    public void update() {
        getServiceCours().update(getPersistenceBean());
    }

    @Override
    public void delete() throws Exception {
        getServiceCours().delete(getIdCours());
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        final String timeStamp = Long.toString(System.currentTimeMillis());
        setCode(timeStamp.substring(timeStamp.length() - 8, timeStamp.length()));
        setLibelleCourt(StringUtils.EMPTY);
        setLibelleLong(StringUtils.EMPTY);
        setCreditEcts(StringUtils.EMPTY);
        setDiscipline(StringUtils.EMPTY);
        setCodeRattachementAutres(StringUtils.EMPTY);
        setCodeResponsable(StringUtils.EMPTY);
        setComplementsResponsable(StringUtils.EMPTY);
        setCoordonnees(StringUtils.EMPTY);
        setVolumeHoraireCm(StringUtils.EMPTY);
        setVolumeHoraireTd(StringUtils.EMPTY);
        setVolumeHoraireTp(StringUtils.EMPTY);
        setVolumeHoraireProjet(StringUtils.EMPTY);
        setVolumeHoraireStage(StringUtils.EMPTY);
        setVolumeHoraireDs(StringUtils.EMPTY);
        setCoefTp(StringUtils.EMPTY);
        setCoefDs(StringUtils.EMPTY);
        setObjectifs(StringUtils.EMPTY);
        setCalendrier(StringUtils.EMPTY);
        setContenu(StringUtils.EMPTY);
        setBibliographie(StringUtils.EMPTY);
        setControleConnaissances(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
        setRattachementBandeau(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    public String getCodeSiScolarite() {
        return persistenceBean.getCodeSiScolarite();
    }

    public void setCodeSiScolarite(final String codeSIScolarite) {
        persistenceBean.setCodeSiScolarite(codeSIScolarite);
    }


    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     *
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie l'identifiant de la fiche.
     */
    @Override
    public Long getIdFiche() {
        return persistenceBean.getId();
    }

    /**
     * Valorise l'identifiant de la fiche.
     */
    @Override
    public void setIdFiche(final Long idFiche) {
        persistenceBean.setId(idFiche);
    }

    /**
     * Renvoie le libelle pour l'attribut discipline.
     */
    public String getLibelleDiscipline() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), persistenceBean.getDiscipline(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementsAutres.
     */
    public String getLibelleCodeRattachementsAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementsAutres.
     */
    public List<StructureModele> getFichesCodeRattachementsAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listCodeRattachementsAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementsAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementsAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementsAutres.nextToken(), getLangue());
            if (currentStructure != null) {
                listCodeRattachementsAutres.add(currentStructure);
            }
        }
        return listCodeRattachementsAutres;
    }

    /**
     * Renvoie le libelle pour l'attribut codeResponsable.
     */
    public String getLibelleCodeResponsable() throws Exception {
        return FicheAnnuaire.getLibelleAffichable(persistenceBean.getCodeResponsable(), getLangue());
    }

    /**
     * Renvoie la liste des annuaires correspondant a l'attribut codeResponsable.
     */
    public List<AnnuaireModele> getFichesCodeResponsable() throws Exception {
        final List<AnnuaireModele> listCodeResponsable = new ArrayList<>();
        final StringTokenizer stCodeResponsable = new StringTokenizer(persistenceBean.getCodeResponsable(), ";");
        while (stCodeResponsable.hasMoreTokens()) {
            listCodeResponsable.add(FicheAnnuaire.getFicheAnnuaire(stCodeResponsable.nextToken(), getLangue()));
        }
        return listCodeResponsable;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complementsResponsable.
     */
    public String getFormatedComplementsResponsable() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplementsResponsable());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut coordonnees.
     */
    public String getFormatedCoordonnees() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCoordonnees());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut objectifs.
     */
    public String getFormatedObjectifs() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getObjectifs());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut calendrier.
     */
    public String getFormatedCalendrier() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCalendrier());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut contenu.
     */
    public String getFormatedContenu() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getContenu());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut bibliographie.
     */
    public String getFormatedBibliographie() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getBibliographie());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut controleConnaissances.
     */
    public String getFormatedControleConnaissances() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getControleConnaissances());
    }

    /**
     * Renvoie la liste des fichiers joints de la fiche.
     */
    public List<RessourceBean> getListeFichiers() throws Exception {
        return serviceRessource.getFiles(this);
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Duplication dans une nouvelle langue, le nouvel objet devient l'objet courant.
     */
    @Override
    public void dupliquer(final String nouvelleLangue) throws Exception {
        // donnees a reinitialiser dans le nouvel objet
        FicheUnivMgr.dupliquer(this);
        // creation objet dans la langue demandee
        setIdFiche(0L);
        setLangue(nouvelleLangue);
        add();
        // recuperation
        selectCodeLangueEtat(getCode(), nouvelleLangue, EtatFiche.BROUILLON.getEtat());
        nextItem();
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String codeSiScolarite = RequeteUtil.renvoyerParametre(requete, "CODE_SI_SCOLARITE");
        final String libelle = RequeteUtil.renvoyerParametre(requete, "LIBELLE");
        final String discipline = RequeteUtil.renvoyerParametre(requete, "DISCIPLINE");
        final String codeResponsable = RequeteUtil.renvoyerParametre(requete, "CODE_RESPONSABLE");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String codeUeupParent = RequeteUtil.renvoyerParametre(requete, "CODE_UEUP");
        final String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        String order = StringUtils.EMPTY;
        if (selection.length() > 0) {
            // Dernières pages libres publiées
            if (selection.equals("0007")) {
                order = "META.META_DATE_MISE_EN_LIGNE DESC";
            }
        }
        return select(codeSiScolarite, libelle, discipline, codeResponsable, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, code, codeUeupParent, nombre, order);
    }

    /**
     * Selection d'une fiche a partir de son code, de sa langue et de son etat.
     */
    @Override
    public int selectCodeLangueEtat(final String code, final String langue, final String etatObjet) {
        final RequeteSQL codeLangueEtatOrdreSurLibelleLong = new RequeteSQL();
        final ClauseWhere whereCodeLangueEtat = ConditionHelper.whereCodeLangueEtat(code, langue, etatObjet);
        final ClauseOrderBy orderBy = new ClauseOrderBy();
        orderBy.orderBy("LIBELLE_LONG", SensDeTri.ASC);
        codeLangueEtatOrdreSurLibelleLong.where(whereCodeLangueEtat).orderBy(orderBy);
        return select(codeLangueEtatOrdreSurLibelleLong.formaterRequete());
    }

    /**
     * Renvoie les cours pour la rubrique demandee.
     */
    @Override
    @Deprecated
    public int selectParCodeRubrique(final String codeRubrique, final String langue) throws Exception {
        final RequeteSQL codeRubriqueLangueEnLigneOrdreLibelleCourt = new RequeteSQL();
        final ClauseWhere whereCodeRubriqueLangueEtat = new ClauseWhere();
        if (isNotEmpty(codeRubrique)) {
            whereCodeRubriqueLangueEtat.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE_RUBRIQUE", codeRubrique));
        }
        if (isNotEmpty(langue)) {
            whereCodeRubriqueLangueEtat.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        whereCodeRubriqueLangueEtat.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", "0003"));
        final ClauseOrderBy orderBy = new ClauseOrderBy();
        orderBy.orderBy("LIBELLE_COURT", SensDeTri.ASC);
        codeRubriqueLangueEnLigneOrdreLibelleCourt.where(whereCodeRubriqueLangueEtat).orderBy(orderBy);
        return select(codeRubriqueLangueEnLigneOrdreLibelleCourt.formaterRequete());
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Cours a partir de l'ensemble des criteres.
     */
    public int select(final String codeSiScolarite, final String libelle, final String discipline, final String codeResponsable, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String codeUeupParent, final String nombre, final String order) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (isNotEmpty(libelle)) {
            final ConditionList conditionLibelle = new ConditionList(ConditionHelper.rechercheMots("T1.LIBELLE_COURT", libelle));
            conditionLibelle.or(ConditionHelper.rechercheMots("T1.LIBELLE_LONG", libelle));
            where.and(conditionLibelle);
        }
        if (isNotEmpty(discipline) && !"0000".equals(discipline)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DISCIPLINE", discipline));
        }
        if (isNotEmpty(codeResponsable)) {
            where.and(ConditionHelper.likePourValeursMultipleAvecSeparateurInterne("T1.CODE_RESPONSABLE", codeResponsable, ","));
        }
        if (isNotEmpty(codeUeupParent)) {
            String codeUEUPFormater = codeUeupParent;
            String langueUEUP = "0";
            if (codeUeupParent.contains(",LANGUE=")) {
                codeUEUPFormater = StringUtils.substringBefore(codeUeupParent, ",LANGUE=");
                langueUEUP = StringUtils.substringAfter(codeUeupParent, ",LANGUE=");
            }
            ServiceUeup serviceUeup = ServiceManager.getServiceForBean(UeupBean.class);
            UeupBean ueup = serviceUeup.getFiche(codeUEUPFormater, langueUEUP);
            if (ueup != null) {
                final Collection<String> codesCours = CollectionUtils.collect(ueup.getCoursBeans(), new Transformer() {

                    @Override
                    public Object transform(final Object input) {
                        return ((CoursBean) input).getCode();
                    }
                });
                if (codesCours.isEmpty()) {
                    codesCours.add("ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ");
                }
                where.and(ConditionHelper.in("T1.CODE", codesCours));
            }
        }
        if (StringUtils.isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructureAnnuaire = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            conditionStructureAnnuaire.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
            where.and(conditionStructureAnnuaire);
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE_REDACTEUR", codeRedacteur));
        }
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.LIBELLE_LONG", ClauseOrderBy.SensDeTri.ASC);
        }
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    @Override
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    @Override
    public void setCodeRattachementAutres(String codesRattachement) {
        persistenceBean.setCodeRattachementAutres(codesRattachement);
    }

    /**
     * Gets the id cours.
     *
     * @return the id cours
     */
    public Long getIdCours() {
        return persistenceBean.getId();
    }

    /**
     * Sets the id cours.
     *
     * @param idCours
     *            the new id cours
     */
    public void setIdCours(Long idCours) {
        persistenceBean.setId(idCours);
    }

    /**
     * Gets the libelle court.
     *
     * @return the libelle court
     */
    public String getLibelleCourt() {
        return persistenceBean.getLibelleCourt();
    }

    /**
     * Sets the libelle court.
     *
     * @param libelleCourt
     *            the new libelle court
     */
    public void setLibelleCourt(String libelleCourt) {
        persistenceBean.setLibelleCourt(libelleCourt);
    }

    /**
     * Gets the libelle long.
     *
     * @return the libelle long
     */
    public String getLibelleLong() {
        return persistenceBean.getLibelleLong();
    }

    /**
     * Sets the libelle long.
     *
     * @param libelleLong
     *            the new libelle long
     */
    public void setLibelleLong(String libelleLong) {
        persistenceBean.setLibelleLong(libelleLong);
    }

    /**
     * Gets the credit ects.
     *
     * @return the credit ects
     */
    public String getCreditEcts() {
        return persistenceBean.getCreditEcts();
    }

    /**
     * Sets the credit ects.
     *
     * @param creditEcts
     *            the new credit ects
     */
    public void setCreditEcts(String creditEcts) {
        persistenceBean.setCreditEcts(creditEcts);
    }

    /**
     * Gets the discipline.
     *
     * @return the discipline
     */
    public String getDiscipline() {
        return persistenceBean.getDiscipline();
    }

    /**
     * Sets the discipline.
     *
     * @param discipline
     *            the new discipline
     */
    public void setDiscipline(String discipline) {
        persistenceBean.setDiscipline(discipline);
    }

    /**
     * Gets the code responsable.
     *
     * @return the code responsable
     */
    public String getCodeResponsable() {
        return persistenceBean.getCodeResponsable();
    }

    /**
     * Sets the code responsable.
     *
     * @param codeResponsable the new code responsable
     */
    public void setCodeResponsable(String codeResponsable) {
        persistenceBean.setCodeResponsable(codeResponsable);
    }

    /**
     * Gets the complements responsable.
     *
     * @return the complements responsable
     */
    @GetterAnnotation(isToolbox = true)
    public String getComplementsResponsable() {
        return persistenceBean.getComplementsResponsable();
    }

    /**
     * Sets the complements responsable.
     *
     * @param complementsResponsable
     *            the new complements responsable
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplementsResponsable(String complementsResponsable) {
        persistenceBean.setComplementsResponsable(complementsResponsable);
    }

    /**
     * Gets the coordonnees.
     *
     * @return the coordonnees
     */
    @GetterAnnotation(isToolbox = true)
    public String getCoordonnees() {
        return persistenceBean.getCoordonnees();
    }

    /**
     * Sets the coordonnees.
     *
     * @param coordonnees
     *            the new coordonnees
     */
    @SetterAnnotation(isToolbox = true)
    public void setCoordonnees(String coordonnees) {
        persistenceBean.setCoordonnees(coordonnees);
    }

    /**
     * Gets the volume horaire cm.
     *
     * @return the volume horaire cm
     */
    public String getVolumeHoraireCm() {
        return persistenceBean.getVolumeHoraireCm();
    }

    /**
     * Sets the volume horaire cm.
     *
     * @param volumeHoraireCm
     *            the new volume horaire cm
     */
    public void setVolumeHoraireCm(String volumeHoraireCm) {
        persistenceBean.setVolumeHoraireCm(volumeHoraireCm);
    }

    /**
     * Gets the volume horaire td.
     *
     * @return the volume horaire td
     */
    public String getVolumeHoraireTd() {
        return persistenceBean.getVolumeHoraireTd();
    }

    /**
     * Sets the volume horaire td.
     *
     * @param volumeHoraireTd
     *            the new volume horaire td
     */
    public void setVolumeHoraireTd(String volumeHoraireTd) {
        persistenceBean.setVolumeHoraireTd(volumeHoraireTd);
    }

    /**
     * Gets the volume horaire tp.
     *
     * @return the volume horaire tp
     */
    public String getVolumeHoraireTp() {
        return persistenceBean.getVolumeHoraireTp();
    }

    /**
     * Sets the volume horaire tp.
     *
     * @param volumeHoraireTp
     *            the new volume horaire tp
     */
    public void setVolumeHoraireTp(String volumeHoraireTp) {
        persistenceBean.setVolumeHoraireTp(volumeHoraireTp);
    }

    /**
     * Gets the volume horaire projet.
     *
     * @return the volume horaire projet
     */
    public String getVolumeHoraireProjet() {
        return persistenceBean.getVolumeHoraireProjet();
    }

    /**
     * Sets the volume horaire projet.
     *
     * @param volumeHoraireProjet the new volume horaire projet
     */
    public void setVolumeHoraireProjet(String volumeHoraireProjet) {
        persistenceBean.setVolumeHoraireProjet(volumeHoraireProjet);
    }

    /**
     * Gets the volume horaire stage.
     *
     * @return the volume horaire stage
     */
    public String getVolumeHoraireStage() {
        return persistenceBean.getVolumeHoraireStage();
    }

    /**
     * Sets the volume horaire stage.
     *
     * @param volumeHoraireStage the new volume horaire stage
     */
    public void setVolumeHoraireStage(String volumeHoraireStage) {
        persistenceBean.setVolumeHoraireStage(volumeHoraireStage);
    }

    /**
     * Gets the volume horaire ds.
     *
     * @return the volume horaire ds
     */
    public String getVolumeHoraireDs() {
        return persistenceBean.getVolumeHoraireDs();
    }

    /**
     * Sets the volume horaire ds.
     *
     * @param volumeHoraireDs
     *            the new volume horaire ds
     */
    public void setVolumeHoraireDs(String volumeHoraireDs) {
        persistenceBean.setVolumeHoraireDs(volumeHoraireDs);
    }

    /**
     * Gets the coef tp.
     *
     * @return the coef tp
     */
    public String getCoefTp() {
        return persistenceBean.getCoefTp();
    }

    /**
     * Sets the coef tp.
     *
     * @param coefTp
     *            the new coef tp
     */
    public void setCoefTp(String coefTp) {
        persistenceBean.setCoefTp(coefTp);
    }

    /**
     * Gets the coef ds.
     *
     * @return the coef ds
     */
    public String getCoefDs() {
        return persistenceBean.getCoefDs();
    }

    /**
     * Sets the coef ds.
     *
     * @param coefDs
     *            the new coef ds
     */
    public void setCoefDs(String coefDs) {
        persistenceBean.setCoefDs(coefDs);
    }

    /**
     * Gets the objectifs.
     *
     * @return the objectifs
     */
    @GetterAnnotation(isToolbox = true)
    public String getObjectifs() {
        return persistenceBean.getObjectifs();
    }

    /**
     * Sets the objectifs.
     *
     * @param objectifs the new objectifs
     */
    @SetterAnnotation(isToolbox = true)
    public void setObjectifs(String objectifs) {
        persistenceBean.setObjectifs(objectifs);
    }

    /**
     * Gets the calendrier.
     *
     * @return the calendrier
     */
    @GetterAnnotation(isToolbox = true)
    public String getCalendrier() {
        return persistenceBean.getCalendrier();
    }

    /**
     * Sets the calendrier.
     *
     * @param calendrier the new calendrier
     */
    @SetterAnnotation(isToolbox = true)
    public void setCalendrier(String calendrier) {
        persistenceBean.setCalendrier(calendrier);
    }

    /**
     * Gets the contenu.
     *
     * @return the contenu
     */
    @GetterAnnotation(isToolbox = true)
    public String getContenu() {
        return persistenceBean.getContenu();
    }

    /**
     * Sets the contenu.
     *
     * @param contenu the new contenu
     */
    @SetterAnnotation(isToolbox = true)
    public void setContenu(String contenu) {
        persistenceBean.setContenu(contenu);
    }

    /**
     * Gets the bibliographie.
     *
     * @return the bibliographie
     */
    @GetterAnnotation(isToolbox = true)
    public String getBibliographie() {
        return persistenceBean.getBibliographie();
    }

    /**
     * Sets the bibliographie.
     *
     * @param bibliographie
     *            the new bibliographie
     */
    @SetterAnnotation(isToolbox = true)
    public void setBibliographie(String bibliographie) {
        persistenceBean.setBibliographie(bibliographie);
    }

    /**
     * Gets the controle connaissances.
     *
     * @return the controle connaissances
     */
    @GetterAnnotation(isToolbox = true)
    public String getControleConnaissances() {
        return persistenceBean.getControleConnaissances();
    }

    /**
     * Sets the controle connaissances.
     *
     * @param controleConnaissances
     *            the new controle connaissances
     */
    @SetterAnnotation(isToolbox = true)
    public void setControleConnaissances(String controleConnaissances) {
        persistenceBean.setControleConnaissances(controleConnaissances);
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    @GetterAnnotation(isToolbox = true)
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    /**
     * Gets the rattachement bandeau.
     *
     * @return the rattachement bandeau
     */
    public String getRattachementBandeau() {
        return persistenceBean.getRattachementBandeau();
    }

    /**
     * Sets the rattachement bandeau.
     *
     * @param rattachementBandeau
     *            the new rattachement bandeau
     */
    public void setRattachementBandeau(String rattachementBandeau) {
        persistenceBean.setRattachementBandeau(rattachementBandeau);
    }

    public Set<FormationBean> getListeFormations() {
        ServiceOffreFormation serviceFormation = ServiceManager.getService(ServiceOffreFormation.class);
        return serviceFormation.findFormationByCours(getCode(), getLangue());

    }

    public Set<UeupBean> getListeUeups() {
        ServiceOffreFormation serviceUeup = ServiceManager.getService(ServiceOffreFormation.class);
        return serviceUeup.findUeupByCours(getCode(), getLangue());
    }
}
