/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.DefaultstructureksupBean;
import com.ksup.objetspartages.service.ServiceDefaultstructureksup;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet defaultstructureksup.
 */
@FicheAnnotation(isSaisieFrontOffice = false)
public class Defaultstructureksup extends AbstractFicheRestriction<DefaultstructureksupBean> implements StructureModele, FicheRattachementsSecondaires {

    private static final long serialVersionUID = 3423160501659896723L;

    private transient final ServiceRessource serviceRessource;

    public Defaultstructureksup() {
        serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
    }

    @Override
    public void addWithForcedId() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        serviceStructure.updateCache(this);
        super.addWithForcedId();
    }

    @Override
    public void add() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        serviceStructure.updateCache(this);
        super.add();
    }

    @Override
    public void delete() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final ServiceDefaultstructureksup serviceDefaultstructureksup = ServiceManager.getServiceForBean(DefaultstructureksupBean.class);
        serviceStructure.updateCache(this);
        serviceDefaultstructureksup.delete(this.getIdFiche());
    }

    @Override
    public void update() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        super.update();
        serviceStructure.updateCache(this);
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        // Temps systeme en ms
        final String timeStamp = Long.toString(System.currentTimeMillis());
        setCode(timeStamp);
        setLibelleCourt(StringUtils.EMPTY);
        setLibelleLong(StringUtils.EMPTY);
        setTypeStructure("0000");
        setStructureEnseignement("1");
        setIdBandeau((long) 0);
        setOnglets(StringUtils.EMPTY);
        setCouleurTitre(StringUtils.EMPTY);
        setCouleurFond(StringUtils.EMPTY);
        setAdresse(StringUtils.EMPTY);
        setCodePostal(StringUtils.EMPTY);
        setVille(StringUtils.EMPTY);
        setVilles(StringUtils.EMPTY);
        setTelephone(StringUtils.EMPTY);
        setTelecopie(StringUtils.EMPTY);
        setAdresseMail(StringUtils.EMPTY);
        setSiteWeb("http://");
        setIdPhoto((long) 0);
        setCodeResponsable(StringUtils.EMPTY);
        setOrganigramme(StringUtils.EMPTY);
        setContacts(StringUtils.EMPTY);
        setActivite(StringUtils.EMPTY);
        setChiffresCles(StringUtils.EMPTY);
        setDomaineFormationEtablissement(StringUtils.EMPTY);
        setFormations(StringUtils.EMPTY);
        setRecherche(StringUtils.EMPTY);
        setPartenariats(StringUtils.EMPTY);
        setComplementCoordonnees(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
        setIdPlanAcces((long) 0);
        setCodeRattachementAutres(StringUtils.EMPTY);
    }

    /**
     * Récupération du libellé à afficher (spécifique au back-office)
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Renvoie le libelle pour l'attribut typeStructure.
     */
    public String getLibelleTypeStructure() {
        return LabelUtils.getLibelle("60", getTypeStructure(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut onglets.
     */
    public String getFormatedOnglets() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getOnglets());
    }

    /**
     * Renvoie le libelle pour l'attribut villes.
     */
    public String getLibelleVilles() {
        return LabelUtils.getLibelle("22", getVilles(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut codeResponsable.
     */
    public String getLibelleCodeResponsable() throws Exception {
        String listeLibelles = StringUtils.EMPTY;
        final StringTokenizer stUe = new StringTokenizer(getCodeResponsable(), ";");
        int idxUe = 0;
        //la liste des ue pour un parcours est composée de la manière suivante :
        // code,langue,semestre,type,credits;code,langue,semestre,type,credits...
        while (stUe.hasMoreTokens()) {
            String code = StringUtils.EMPTY;
            String langue = "0";
            String titre = StringUtils.EMPTY;
            StringTokenizer stUeCourant = new StringTokenizer(stUe.nextToken(), ",");
            int token = 0;
            while (stUeCourant.hasMoreTokens()) {
                switch (token) {
                    case 0:
                        code = stUeCourant.nextToken();
                        break;
                    case 1:
                        langue = stUeCourant.nextToken();
                        break;
                    case 2:
                        titre = stUeCourant.nextToken();
                        break;
                    default:
                        stUeCourant.nextToken();
                        break;
                }
                token++;
            }
            if (idxUe > 0) {
                listeLibelles += ";";
            }
            if (!code.equals(StringUtils.EMPTY)) {
                final AnnuaireModele annuaire = FicheAnnuaire.getFicheAnnuaire(code, langue);
                if (annuaire != null) {
                    listeLibelles += annuaire.getLibelleAffichable();
                }
            }
            if (!titre.equals(StringUtils.EMPTY)) {
                listeLibelles += " [" + LabelUtils.getLibelle("64", titre, langue) + "]";
            }
            idxUe++;
        }
        String sCumullibelles = listeLibelles.replaceAll("\"", "\\\"");
        sCumullibelles = sCumullibelles.replaceAll("\'", "\\\'");
        return sCumullibelles;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut organigramme.
     */
    public String getFormatedOrganigramme() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getOrganigramme());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut contacts.
     */
    public String getFormatedContacts() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getContacts());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut activite.
     */
    public String getFormatedActivite() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getActivite());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut chiffresCles.
     */
    public String getFormatedChiffresCles() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getChiffresCles());
    }

    /**
     * Renvoie le libelle pour l'attribut domaineFormationEtablissement.
     */
    public String getLibelleDomaineFormationEtablissement() {
        return LabelUtils.getLibelle("2004", getDomaineFormationEtablissement(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut formations.
     */
    public String getFormatedFormations() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getFormations());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut recherche.
     */
    public String getFormatedRecherche() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getRecherche());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut partenariats.
     */
    public String getFormatedPartenariats() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPartenariats());
    }

    /**
     * Renvoie le fichier joint correspondant a l'attribut photo.
     */
    public RessourceBean getPhoto() throws Exception {
        return serviceRessource.getFile(this);
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complementCoordonnees.
     */
    public String getFormatedComplementCoordonnees() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplementCoordonnees());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementAutres.
     */
    public String getLibelleCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementAutres.
     */
    public List<StructureModele> getFichesCodeRattachementAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listCodeRattachementAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementAutres.nextToken(), getLangue());
            if (currentStructure != null) {
                listCodeRattachementAutres.add(currentStructure);
            }
        }
        return listCodeRattachementAutres;
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String libelle = RequeteUtil.renvoyerParametre(requete, "LIBELLE");
        final String typeStructure = RequeteUtil.renvoyerParametre(requete, "TYPE_STRUCTURE");
        final String formations = RequeteUtil.renvoyerParametre(requete, "DOMAINE_FORMATION_ETABLISSEMENT");
        final String structureEnseignement = RequeteUtil.renvoyerParametre(requete, "STRUCTURE_ENSEIGNEMENT");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String codeFiche = RequeteUtil.renvoyerParametre(requete, "CODE");
        String order = "";
        if (selection.length() > 0) {
            // Dernières pages libres publiées
            if (selection.equals("0007")) {
                order = "META.META_DATE_MISE_EN_LIGNE DESC";
            }
        }
        return select(libelle, typeStructure, formations, structureEnseignement, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, codeFiche, nombre, order);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Defaultstructureksup a partir de l'ensemble des criteres.
     */
    public int select(final String libelle, final String typeStructure, final String formations, final String structureEnseignement, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final String order) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (isNotEmpty(code)) {
            where.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        if (isNotEmpty(libelle)) {
            final ConditionList conditionLibelle = new ConditionList(ConditionHelper.rechercheMots("T1.LIBELLE_COURT", libelle));
            conditionLibelle.or(ConditionHelper.rechercheMots("T1.LIBELLE_LONG", libelle));
            where.and(conditionLibelle);
        }
        if (isNotEmpty(typeStructure) && !"0000".equals(typeStructure)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE_STRUCTURE", typeStructure));
        }
        if (isNotEmpty(formations) && !"0000".equals(formations)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DOMAINE_FORMATION_ETABLISSEMENT", formations));
        }
        if (isNotEmpty(structureEnseignement)) {
            where.and(ConditionHelper.egalVarchar("T1.STRUCTURE_ENSEIGNEMENT", structureEnseignement));
        }
        if (isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructureAnnuaire = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            conditionStructureAnnuaire.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
            where.and(conditionStructureAnnuaire);
        }
        if (isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE_REDACTEUR", codeRedacteur));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        } else {
            orderBy.orderBy("T1.LIBELLE_LONG", ClauseOrderBy.SensDeTri.ASC);
        }
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    @Override
    public String getLibelleCourt() {
        return persistenceBean.getLibelleCourt();
    }

    @Override
    public void setLibelleCourt(String libelle) {
        persistenceBean.setLibelleCourt(libelle);
    }

    @Override
    public String getLibelleLong() {
        return persistenceBean.getLibelleLong();
    }

    @Override
    public void setLibelleLong(String libelle) {
        persistenceBean.setLibelleLong(libelle);
    }

    @Override
    public String getOnglets() {
        return persistenceBean.getOnglets();
    }

    @Override
    public void setOnglets(String onglets) {
        persistenceBean.setOnglets(onglets);
    }

    @Override
    public String getTypeStructure() {
        return persistenceBean.getTypeStructure();
    }

    @Override
    public void setTypeStructure(String typeStructure) {
        persistenceBean.setTypeStructure(typeStructure);
    }

    @Override
    public Long getIdBandeau() {
        return persistenceBean.getIdBandeau();
    }

    @Override
    public void setIdBandeau(Long idBandeau) {
        persistenceBean.setIdBandeau(idBandeau);
    }

    @Override
    public String getCouleurTitre() {
        return persistenceBean.getCouleurTitre();
    }

    @Override
    public void setCouleurTitre(String couleurTitre) {
        persistenceBean.setCouleurTitre(couleurTitre);
    }

    @Override
    public String getCouleurFond() {
        return persistenceBean.getCouleurFond();
    }

    @Override
    public void setCouleurFond(String couleurFond) {
        persistenceBean.setCouleurFond(couleurFond);
    }

    @Override
    public String getAdresse() {
        return persistenceBean.getAdresse();
    }

    @Override
    public void setAdresse(String adresse) {
        persistenceBean.setAdresse(adresse);
    }

    @Override
    public String getCodePostal() {
        return persistenceBean.getCodePostal();
    }

    @Override
    public void setCodePostal(String codePostal) {
        persistenceBean.setCodePostal(codePostal);
    }

    @Override
    public String getVille() {
        return persistenceBean.getVille();
    }

    @Override
    public void setVille(String ville) {
        persistenceBean.setVille(ville);
    }

    @Override
    public String getTelephone() {
        return persistenceBean.getTelephone();
    }

    @Override
    public void setTelephone(String telephone) {
        persistenceBean.setTelephone(telephone);
    }

    @Override
    public String getTelecopie() {
        return persistenceBean.getTelecopie();
    }

    @Override
    public void setTelecopie(String telecopie) {
        persistenceBean.setTelecopie(telecopie);
    }

    @Override
    public String getAdresseMail() {
        return persistenceBean.getAdresseMail();
    }

    @Override
    public void setAdresseMail(String adresseMail) {
        persistenceBean.setAdresseMail(adresseMail);
    }

    @Override
    public String getSiteWeb() {
        return persistenceBean.getSiteWeb();
    }

    @Override
    public void setSiteWeb(String siteWeb) {
        persistenceBean.setSiteWeb(siteWeb);
    }

    @Override
    public Long getIdPlanAcces() {
        return persistenceBean.getIdPlanAcces();
    }

    /* (non-Javadoc)
         * @see com.univ.objetspartages.om.StructureModele#getAttributSpecifique1()
         */
    @Override
    public String getAttributSpecifique1() {
        return getStructureEnseignement();
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.om.StructureModele#getAttributSpecifique2()
     */
    @Override
    public String getAttributSpecifique2() {
        return StringUtils.EMPTY;
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.om.StructureModele#getAttributSpecifique3()
     */
    @Override
    public String getAttributSpecifique3() {
        return StringUtils.EMPTY;
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.om.StructureModele#getAttributSpecifique4()
     */
    @Override
    public String getAttributSpecifique4() {
        return StringUtils.EMPTY;
    }

    /* (non-Javadoc)
     * @see com.univ.objetspartages.om.StructureModele#getAttributSpecifique5()
     */
    @Override
    public String getAttributSpecifique5() {
        return StringUtils.EMPTY;
    }

    @Override
    public String getComplementCoordonnees() {
        return persistenceBean.getComplementCoordonnees();
    }

    @Override
    public void setComplementCoordonnees(String complements) {
        persistenceBean.setComplementCoordonnees(complements);
    }

    @Override
    public String getActivite() {
        return persistenceBean.getActivite();
    }

    @Override
    public void setActivite(String activite) {
        persistenceBean.setActivite(activite);
    }

    @Override
    public String getCodeResponsable() {
        return persistenceBean.getCodeResponsable();
    }

    @Override
    public void setCodeResponsable(String codeResponsable) {
        persistenceBean.setCodeResponsable(codeResponsable);
    }

    @Override
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    @Override
    public void setCodeRattachementAutres(String codesRattachement) {
        persistenceBean.setCodeRattachementAutres(codesRattachement);
    }

    public void setIdDefaultstructureksup(long idDefaultstructure) {
        persistenceBean.setId(idDefaultstructure);
    }

    public Long getIdDefaultstructureksup() {
        return persistenceBean.getId();
    }

    public void setVilles(String villes) {
        persistenceBean.setVilles(villes);
    }

    public void setIdPhoto(long idPhoto) {
        persistenceBean.setIdPhoto(idPhoto);
    }

    public void setIdPlanAcces(long idPlanAcces) {
        persistenceBean.setIdPlanAcces(idPlanAcces);
    }

    public void setOrganigramme(String organigramme) {
        persistenceBean.setOrganigramme(organigramme);
    }

    public void setChiffresCles(String chiffresCles) {
        persistenceBean.setChiffresCles(chiffresCles);
    }

    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    public String getVilles() {
        return persistenceBean.getVilles();
    }

    public String getOrganigramme() {
        return persistenceBean.getOrganigramme();
    }

    public String getChiffresCles() {
        return persistenceBean.getChiffresCles();
    }

    public String getComplements() {
        return persistenceBean.getComplements();
    }

    public Long getIdPhoto() {
        return persistenceBean.getIdPhoto();
    }

    public void setStructureEnseignement(String structureEnseignement) {
        persistenceBean.setStructureEnseignement(structureEnseignement);
    }

    public void setContacts(String contacts) {
        persistenceBean.setContacts(contacts);
    }

    public void setDomaineFormationEtablissement(String domaineFormationEtablissement) {
        persistenceBean.setDomaineFormationEtablissement(domaineFormationEtablissement);
    }

    public void setFormations(String formations) {
        persistenceBean.setFormations(formations);
    }

    public void setRecherche(String recherche) {
        persistenceBean.setRecherche(recherche);
    }

    public void setPartenariats(String partenariats) {
        persistenceBean.setPartenariats(partenariats);
    }

    public String getContacts() {
        return persistenceBean.getContacts();
    }

    public String getDomaineFormationEtablissement() {
        return persistenceBean.getDomaineFormationEtablissement();
    }

    public String getFormations() {
        return persistenceBean.getFormations();
    }

    public String getRecherche() {
        return persistenceBean.getRecherche();
    }

    public String getPartenariats() {
        return persistenceBean.getPartenariats();
    }

    public String getStructureEnseignement() {
        return persistenceBean.getStructureEnseignement();
    }

    /**
     * FIXME : correctif pour CORE-1746 à corriger en suppprimant ce genre de cache
     * @param o
     * @throws IOException
     */
    private void writeObject(final ObjectOutputStream o) throws IOException {
        o.writeObject(this.getPersistenceBean());
    }

    /**
     * FIXME : correctif pour CORE-1746 à corriger en suppprimant ce genre de cache
     * @param o
     * @throws IOException
     * @throws ClassNotFoundException
     */
    private void readObject(final ObjectInputStream o) throws IOException, ClassNotFoundException {
        this.persistenceBean = (DefaultstructureksupBean) o.readObject();
    }
}
