/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.om;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.bean.UeupBean;
import com.ksup.objetspartages.service.ServiceOffreFormation;
import com.ksup.objetspartages.service.ServiceUeup;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.AnnuaireModele;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.FicheRattachementsSecondaires;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.ContexteUtil;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.Condition;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.OrderByHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;

import static org.apache.commons.lang3.StringUtils.isNotBlank;
import static org.apache.commons.lang3.StringUtils.isNotEmpty;

/**
 * Classe representant un objet ueup.
 */
@FicheAnnotation()
public class Ueup extends AbstractFicheRestriction<UeupBean> implements FicheRattachementsSecondaires {

    private static final Logger LOG = LoggerFactory.getLogger(Ueup.class);

    private final ServiceRessource serviceRessource;

    public Ueup() {
        serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     * @deprecated utilise le contexte uniquement pour la connexion à la bdd. Utiliser {@link Ueup#getLibelleAffichable(String, String)}
     */
    @Deprecated
    public static String getLibelleAffichable(final OMContext ctx, final String codes, final String langues) throws Exception {
        return getLibelleAffichable(codes, langues);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(final String codes, final String langues) throws Exception {
        String res = StringUtils.EMPTY;
        if (StringUtils.isEmpty(codes)) {
            return res;
        }
        String langue = StringUtils.defaultString(langues, StringUtils.EMPTY);
        final Ueup ueup = new Ueup();
        ueup.init();
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            String code = st.nextToken();
            final String[] tab = code.split(",");
            // CODE,LANGUE=0
            if (tab.length == 2) {
                code = tab[0];
                langue = tab[1].substring(7);
            }
            // CODE,LANGUE,TITRE
            if (tab.length == 3) {
                code = tab[0];
                langue = tab[1];
            }
            // On cherche d'abord la version en ligne puis les autres versions
            int count = ueup.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = ueup.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (ueup.nextItem()) {
                    res += ueup.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     * @deprecated utilise le contexte uniquement pour la connexion à la bdd. Utiliser {@link Ueup#getFicheUeup(String, String)}
     */
    @Deprecated
    public static Ueup getFicheUeup(final OMContext ctx, final String code, String langue) throws Exception {
        return getFicheUeup(code, langue);
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Ueup getFicheUeup(final String code, String langue) throws Exception {
        Ueup fiche = null;
        if (StringUtils.isNotBlank(code)) {
            langue = StringUtils.defaultString(langue, "0");
            fiche = new Ueup();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * AM 200501 : pour le LMD Retourne sous la forme d'une arraylist la liste des fiches ueup trouvées par rapport au code: &lt;a href="lien fiche ueup"
     * title="libellé long"&gt;libellé long fiche ueup&lt;/a&gt;
     *
     * @param ctx
     * @param codes
     * @param langue
     * @return
     * @throws Exception
     */
    public static ArrayList<String> getLibellesLiensUeups(final OMContext ctx, final String codes, final String langue) throws Exception {
        final ArrayList<String> listeUeups = new ArrayList<>();
        UrlProvider urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
        String computedLangue = langue;
        if (computedLangue == null || computedLangue.length() == 0) {
            computedLangue = "0";
        }
        if (codes == null || codes.length() == 0) {
            return listeUeups;
        }
        final Ueup ueup = new Ueup();
        ueup.init();
        final StringTokenizer st = new StringTokenizer(codes, ";");
        String code;
        while (st.hasMoreTokens()) {
            code = st.nextToken();
            if (code.startsWith("0/")) {
                code = code.substring(2);
            }
            if (code.contains(",LANGUE=")) {
                code = StringUtils.split(code, ",")[0];
            }
            // On cherche d'abord la version en ligne puis les autres
            // versions
            int count = ueup.selectCodeLangueEtat(code, computedLangue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = ueup.selectCodeLangueEtat(code, computedLangue, StringUtils.EMPTY);
            }
            if (count > 0) {
                while (ueup.nextItem()) {
                    String url = urlProvider.provideAbsoluteUrl(ueup.getPersistanceBean());
                    listeUeups.add("<a href=\"" + url + "\" title=\"" + ueup.getLibelleLong() + "\">" + ueup.getLibelleLong() + "</a>");
                }
            }
        }
        return listeUeups;
    }

    private ServiceUeup getServiceUeup() {
        return ServiceManager.getServiceForBean(UeupBean.class);
    }

    @Override
    public void add() {
        getServiceUeup().add(this.getPersistenceBean());
    }

    @Override
    public void addWithForcedId() throws Exception {
        getServiceUeup().addWithForcedId(getPersistenceBean());
    }

    @Override
    public void update() {
        getServiceUeup().update(getPersistenceBean());
    }

    @Override
    public void delete() throws Exception {
        getServiceUeup().delete(getIdUeup());
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        final String timeStamp = Long.toString(System.currentTimeMillis());
        setCode(timeStamp.substring(timeStamp.length() - 8, timeStamp.length()));
        setLibelleCourt(StringUtils.EMPTY);
        setLibelleLong(StringUtils.EMPTY);
        setCreditEcts(StringUtils.EMPTY);
        setSemestre(StringUtils.EMPTY);
        setType(StringUtils.EMPTY);
        setNature(StringUtils.EMPTY);
        setNbMaxiAchoix(StringUtils.EMPTY);
        setNbMiniAchoix(StringUtils.EMPTY);
        setDisciplines(StringUtils.EMPTY);
        setCodeRattachementAutres(StringUtils.EMPTY);
        setVille(StringUtils.EMPTY);
        setaDistance("0");
        setMixteDistance("0");
        setCodeResponsables(StringUtils.EMPTY);
        setComplementsResponsable(StringUtils.EMPTY);
        setCoordonnees(StringUtils.EMPTY);
        setVolumeHoraireTotal(StringUtils.EMPTY);
        setVolumeHoraireCm(StringUtils.EMPTY);
        setVolumeHoraireTd(StringUtils.EMPTY);
        setVolumeHoraireTp(StringUtils.EMPTY);
        setVolumeHoraireProjet(StringUtils.EMPTY);
        setVolumeHoraireStage(StringUtils.EMPTY);
        setVolumeHoraireDs(StringUtils.EMPTY);
        setVolumeHoraireFormationDistance(StringUtils.EMPTY);
        setVolumeHorairePresentiel(StringUtils.EMPTY);
        setPrerequis(StringUtils.EMPTY);
        setCodeUeupsRequises(StringUtils.EMPTY);
        setObjectifs(StringUtils.EMPTY);
        setCalendrier(StringUtils.EMPTY);
        setContenu(StringUtils.EMPTY);
        setUeupfils(StringUtils.EMPTY);
        setCours(StringUtils.EMPTY);
        setBibliographie(StringUtils.EMPTY);
        setControleConnaissances(StringUtils.EMPTY);
        setDomaines(StringUtils.EMPTY);
        setDomaineErasmus(StringUtils.EMPTY);
        setGrade(StringUtils.EMPTY);
        setTypeDiplome(StringUtils.EMPTY);
        setLanguesEnseignement(StringUtils.EMPTY);
        setComplements(StringUtils.EMPTY);
        setRattachementBandeau(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }



    public String getCodeSiScolarite() {
        return persistenceBean.getCodeSiScolarite();
    }

    public void setCodeSiScolarite(final String codeSIScolarite) {
        persistenceBean.setCodeSiScolarite(codeSIScolarite);
    }


    /**
     * Récupération du libellé à afficher (spécifique au back-office).
     *
     * @return the libelle back office
     *
     */
    public String getLibelleBackOffice() {
        return getLibelleAffichable();
    }

    /**
     * Renvoie le libelle pour l'attribut semestre.
     */
    public String getLibelleSemestre() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.SEMESTRE.getValeur(), getSemestre(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut type.
     */
    public String getLibelleType() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.TYPE_ELEMENT.getValeur(), getType(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut nature.
     */
    public String getLibelleNature() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.NATURE.getValeur(), getNature(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut discipline.
     */
    public String getLibelleDiscipline() {
        return getConcatenatedListeLabel(LabelUtils.getLibelleList(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), getDisciplines(), getLangue()));
    }

    private String getConcatenatedListeLabel(List<String> labels) {
        return StringUtils.join(labels, ", ");
    }

    /**
     * Renvoie le libelle pour l'attribut codeRattachementsAutres.
     */
    public String getLibelleCodeRattachementsAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getDisplayableLabel(getCodeRattachementAutres(), getLangue());
    }

    /**
     * Renvoie la liste des structures correspondant a l'attribut codeRattachementsAutres.
     */
    public List<StructureModele> getFichesCodeRattachementsAutres() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        final List<StructureModele> listCodeRattachementsAutres = new ArrayList<>();
        final StringTokenizer stCodeRattachementsAutres = new StringTokenizer(getCodeRattachementAutres(), ";");
        while (stCodeRattachementsAutres.hasMoreTokens()) {
            final StructureModele currentStructure = serviceStructure.getByCodeLanguage(stCodeRattachementsAutres.nextToken(), getLangue());
            if (currentStructure != null) {
                listCodeRattachementsAutres.add(currentStructure);
            }
        }
        return listCodeRattachementsAutres;
    }

    /**
     * Renvoie le libelle pour l'attribut ville.
     */
    public String getLibelleVille() {
        return LabelUtils.getLibelle("22", getVille(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut codeResponsable.
     */
    public String getLibelleCodeResponsable() throws Exception {
        return FicheAnnuaire.getLibelleAffichable(getCodeResponsables(), getLangue());
    }

    /**
     * Renvoie la liste des annuaires correspondant a l'attribut codeResponsable.
     */
    public List<AnnuaireModele> getFichesCodeResponsable() throws Exception {
        final List<AnnuaireModele> listCodeResponsable = new ArrayList<>();
        final StringTokenizer stCodeResponsable = new StringTokenizer(getCodeResponsables(), ";");
        while (stCodeResponsable.hasMoreTokens()) {
            listCodeResponsable.add(FicheAnnuaire.getFicheAnnuaire(stCodeResponsable.nextToken(), getLangue()));
        }
        return listCodeResponsable;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complementsResponsable.
     */
    public String getFormatedComplementsResponsable() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplementsResponsable());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut coordonnees.
     */
    public String getFormatedCoordonnees() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCoordonnees());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut prerequis.
     */
    public String getFormatedPrerequis() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getPrerequis());
    }

    /**
     * Renvoie le libelle pour l'attribut codeUeupsRequises.
     */
    public String getLibelleCodeUeupsRequises() throws Exception {
        return Ueup.getLibelleAffichable(getCodeUeupsRequises(), getLangue());
    }

    /**
     * Renvoie la liste des ueups correspondant a l'attribut codeUeupsRequises.
     */
    public List<Ueup> getFichesCodeUeupsRequises() throws Exception {
        final List<Ueup> listCodeUeupsRequises = new ArrayList<>();
        final StringTokenizer stCodeUeupsRequises = new StringTokenizer(getCodeUeupsRequises(), ";");
        while (stCodeUeupsRequises.hasMoreTokens()) {
            listCodeUeupsRequises.add(Ueup.getFicheUeup(stCodeUeupsRequises.nextToken(), getLangue()));
        }
        return listCodeUeupsRequises;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut objectifs.
     */
    public String getFormatedObjectifs() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getObjectifs());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut calendrier.
     */
    public String getFormatedCalendrier() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getCalendrier());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut contenu.
     */
    public String getFormatedContenu() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getContenu());
    }

    /**
     * Renvoie le libelle pour l'attribut codeUeupfils.
     */
    public String getLibelleCodeUeupfils() throws Exception {
        return Ueup.getLibelleAffichable(getUeupfils(), getLangue());
    }

    /**
     * Renvoie la liste des ueups correspondant a l'attribut codeUeupfils.
     */
    public List<Ueup> getFichesCodeUeupfils() throws Exception {
        final List<Ueup> listCodeUeupfils = new ArrayList<>();
        String[] listeCodes = StringUtils.split(getUeupfils(), ";");
        if (ArrayUtils.isNotEmpty(listeCodes)) {
            for (String codeLangue : listeCodes) {
                String[] codeParts = StringUtils.split(codeLangue, ",");
                if (ArrayUtils.isNotEmpty(codeParts)) {
                    listCodeUeupfils.add(Ueup.getFicheUeup(codeParts[0], getLangue()));
                }
            }
        }
        return listCodeUeupfils;
    }

    /**
     * Renvoie le libelle pour l'attribut codeCours.
     */
    public String getLibelleCodeCours() throws Exception {
        return Cours.getLibelleAffichable(getCours(), getLangue());
    }

    /**
     * Renvoie la liste des cours correspondant a l'attribut codeCours.
     */
    public List<Cours> getFichesCodeCours() throws Exception {
        final List<Cours> listCodeCours = new ArrayList<>();
        String[] listeCodes = StringUtils.split(getCours(), ";");
        if (ArrayUtils.isNotEmpty(listeCodes)) {
            for (String codeLangue : listeCodes) {
                String[] codeParts = StringUtils.split(codeLangue, ",");
                if (ArrayUtils.isNotEmpty(codeParts)) {
                    listCodeCours.add(Cours.getFicheCours(codeParts[0], getLangue()));
                }
            }
        }
        return listCodeCours;
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut bibliographie.
     */
    public String getFormatedBibliographie() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getBibliographie());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut controleConnaissances.
     */
    public String getFormatedControleConnaissances() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getControleConnaissances());
    }

    /**
     * Renvoie la liste des fichiers joints de la fiche.
     */
    public List<RessourceBean> getListeFichiers() throws Exception {
        return serviceRessource.getFiles(this);
    }

    /**
     * Renvoie le libelle pour l'attribut domaines.
     */
    public String getLibelleDomaines() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), getDomaines(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut domaineErasmus. Si plusieurs villes, chaque libellé est séparé par ", "
     */
    public String getLibelleDomaineErasmus() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.DOMAINE_ERASMUS.getValeur(), getDomaineErasmus(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut grade.
     */
    public String getLibelleGrade() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.GRADE.getValeur(), getGrade(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut typeDiplome.
     */
    public String getLibelleTypeDiplome() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), getTypeDiplome(), getLangue());
    }

    /**
     * Renvoie le libelle pour l'attribut languesEnseignement.
     */
    public String getLibelleLanguesEnseignement() {
        return LabelUtils.getLibelle(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), getLanguesEnseignement(), getLangue());
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut complements.
     */
    public String getFormatedComplements() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getComplements());
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String codeSiScolarite = RequeteUtil.renvoyerParametre(requete, "CODE_SI_SCOLARITE");
        final String libelle = RequeteUtil.renvoyerParametre(requete, "LIBELLE");
        final String type = RequeteUtil.renvoyerParametre(requete, "TYPE");
        final String nature = RequeteUtil.renvoyerParametre(requete, "NATURE");
        final String semestre = RequeteUtil.renvoyerParametre(requete, "SEMESTRE");
        final String grade = RequeteUtil.renvoyerParametre(requete, "GRADE");
        final String domaines = RequeteUtil.renvoyerParametre(requete, "DOMAINES");
        final String domaineErasmus = RequeteUtil.renvoyerParametre(requete, "DOMAINE_ERASMUS");
        final String discipline = RequeteUtil.renvoyerParametre(requete, "DISCIPLINE");
        final String modalitesEnseignement = RequeteUtil.renvoyerParametre(requete, "MODALITES_ENSEIGNEMENT");
        final String languesEnseignement = RequeteUtil.renvoyerParametre(requete, "LANGUES_ENSEIGNEMENT");
        final String codeResponsable = RequeteUtil.renvoyerParametre(requete, "CODE_RESPONSABLE");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String codeUeupParent = RequeteUtil.renvoyerParametre(requete, "CODE_UEUP");
        final String codeFormation = RequeteUtil.renvoyerParametre(requete, "CODE_FORMATION");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String selection = RequeteUtil.renvoyerParametre(requete, "SELECTION");
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        String order = StringUtils.EMPTY;
        if (selection.length() > 0) {
            // Dernières pages libres publiées
            if (selection.equals("0007")) {
                order = "META.METADATEMISEENLIGNE DESC";
            }
        }
        return select(codeSiScolarite, libelle, type, nature, semestre, grade, domaines, domaineErasmus, discipline, modalitesEnseignement, languesEnseignement, codeResponsable, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, code, "", codeUeupParent, codeFormation, nombre, order);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return getServiceFiche().getReferences(persistenceBean);
    }

    /**
     * Selection d'une instance de l'objet Ueup a partir de l'ensemble des criteres.
     */
    public int select(final String codeSiScolarite, final String libelle, final String type, final String nature, final String semestre, final String grade, final String domaines, final String domaineErasmus, final String discipline, final String modalitesEnseignement, final String languesEnseignement, final String codeResponsable, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String codeCours, final String codeUeupParent, final String codeFormation, final String nombre, final String order) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        if (isNotEmpty(code)) {
            where.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        if (StringUtils.isNotEmpty(codeSiScolarite)) {
            where.and(ConditionHelper.rechercheMots("T1.CODE_SI_SCOLARITE", codeSiScolarite));
        }
        if (isNotEmpty(libelle) && isNotBlank(libelle)) {
            final ConditionList conditionLibelle = new ConditionList(ConditionHelper.rechercheMots("T1.LIBELLE_COURT", libelle));
            conditionLibelle.or(ConditionHelper.rechercheMots("T1.LIBELLE_LONG", libelle));
            where.and(conditionLibelle);
        }
        if (isNotEmpty(type) && !"0000".equals(type)) {
            where.and(ConditionHelper.egalVarchar("T1.TYPE", type));
        }
        if (isNotEmpty(nature) && !"0000".equals(nature)) {
            where.and(ConditionHelper.egalVarchar("T1.NATURE", nature));
        }
        if (isNotEmpty(semestre) && !"0000".equals(semestre)) {
            where.and(ConditionHelper.egalVarchar("T1.SEMESTRE", semestre));
        }
        if (isNotEmpty(grade) && !"0000".equals(grade)) {
            where.and(ConditionHelper.egalVarchar("T1.GRADE", grade));
        }
        if (isNotEmpty(domaines) && !"0000".equals(domaines)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DOMAINES", domaines));
        }
        if (isNotEmpty(domaineErasmus) && !"0000".equals(domaineErasmus)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DOMAINE_ERASMUS", domaineErasmus));
        }
        if (isNotEmpty(discipline) && !"0000".equals(discipline)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.DISCIPLINES", discipline));
        }
        if (isNotEmpty(modalitesEnseignement) && !"0000".equals(modalitesEnseignement)) {
            where.and(traiterModalitesEnseignements(modalitesEnseignement));
        }
        if (isNotEmpty(codeResponsable) && !"0000".equals(codeResponsable)) {
            where.and(ConditionHelper.likePourValeursMultipleAvecSeparateurInterne("T1.CODE_RESPONSABLES", codeResponsable, ","));
        }
        if (isNotEmpty(codeRattachement) && !"0000".equals(codeRattachement)) {
            final ConditionList conditionStructure = new ConditionList(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
            conditionStructure.or(ConditionHelper.getConditionStructureMultiple("T1.CODE_RATTACHEMENT_AUTRES", codeRattachement));
            where.and(conditionStructure);
        }
        if (isNotEmpty(codeCours)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.CODE_COURS", codeCours));
        }
        if (isNotEmpty(languesEnseignement)) {
            where.and(ConditionHelper.likePourValeursMultiple("T1.LANGUES_ENSEIGNEMENT", languesEnseignement));
        }
        if (isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("CODE_REDACTEUR", codeRedacteur));
        }
        if (isNotEmpty(codeUeupParent)) {
            ServiceUeup serviceUeup = ServiceManager.getServiceForBean(UeupBean.class);
            Set<String> childsCode = serviceUeup.findAllChild(codeUeupParent);
            if (childsCode.isEmpty()) {
                childsCode.add("ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ");
            }
            where.and(ConditionHelper.in("T1.CODE", childsCode));
        }
        if (isNotEmpty(codeFormation)) {
            ServiceOffreFormation serviceOffreFormation = ServiceManager.getService(ServiceOffreFormation.class);
            Set<String> codesUeups = serviceOffreFormation.findUeupByFormation(StringUtils.substringBefore(codeFormation, ",LANGUE="), StringUtils.substringAfter(codeFormation, ",LANGUE="));
            if (codesUeups.isEmpty()) {
                codesUeups.add("ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ");
            }
            where.and(ConditionHelper.in("T1.CODE", codesUeups));
        }
        RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        ClauseOrderBy orderBy = new ClauseOrderBy();
        if (StringUtils.isNotEmpty(order)) {
            orderBy = OrderByHelper.reconstruireClauseOrderBy(order);
        }
        orderBy.orderBy("T1.LIBELLE_LONG", SensDeTri.ASC);
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        return select(requeteSelect.formaterRequete());
    }

    private Condition traiterModalitesEnseignements(final String modalitesEnseignement) {
        final ConditionList conditionSurModalites = new ConditionList();
        final String[] requetesSurModalite = StringUtils.split(modalitesEnseignement, ";");
        for (final String requeteModalite : requetesSurModalite) {
            if (ModalitesUEUP.contains(requeteModalite)) {
                conditionSurModalites.and(ConditionHelper.egalVarchar("T1." + requeteModalite, "1"));
            }
        }
        return conditionSurModalites;
    }

    /**
     * Selection d'une instance de l'objet Ueup a partir de l'ensemble des criteres.
     */
    public int selectParCours(final String codeCours, final String langue) throws Exception {
        return select(StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, langue, "0003", StringUtils.EMPTY, StringUtils.EMPTY, codeCours, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY);
    }

    /**
     * Selection d'une instance de l'objet Ueup a partir de l'ensemble des criteres.
     */
    public int selectParUeup(final String codeUeup, final String langue) throws Exception {
        return select(StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, langue, "0003", StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, codeUeup, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY);
    }

    /**
     * Selection d'une instance de l'objet Ueup a partir de l'ensemble des criteres.
     */
    public int selectParFormation(final String codeFormation, final String langue) throws Exception {
        return select(StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, langue, "0003", StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY, codeFormation, StringUtils.EMPTY, StringUtils.EMPTY);
    }

    public Long getIdUeup() {
        return persistenceBean.getId();
    }

    public void setIdUeup(Long id) {
        persistenceBean.setId(id);
    }

    /**
     * Gets the libelle court.
     *
     * @return the libelle court
     */
    public String getLibelleCourt() {
        return persistenceBean.getLibelleCourt();
    }

    /**
     * Sets the libelle court.
     *
     * @param libelleCourt
     *            the new libelle court
     */
    public void setLibelleCourt(String libelleCourt) {
        persistenceBean.setLibelleCourt(libelleCourt);
    }

    /**
     * Gets the libelle long.
     *
     * @return the libelle long
     */
    public String getLibelleLong() {
        return persistenceBean.getLibelleLong();
    }

    /**
     * Sets the libelle long.
     *
     * @param libelleLong
     *            the new libelle long
     */
    public void setLibelleLong(String libelleLong) {
        persistenceBean.setLibelleLong(libelleLong);
    }

    /**
     * Gets the credit ects.
     *
     * @return the credit ects
     */
    public String getCreditEcts() {
        return persistenceBean.getCreditEcts();
    }

    /**
     * Sets the credit ects.
     *
     * @param creditEcts
     *            the new credit ects
     */
    public void setCreditEcts(String creditEcts) {
        persistenceBean.setCreditEcts(creditEcts);
    }

    /**
     * Gets the semestre.
     *
     * @return the semestre
     */
    public String getSemestre() {
        return persistenceBean.getSemestre();
    }

    /**
     * Sets the semestre.
     *
     * @param semestre
     *            the new semestre
     */
    public void setSemestre(String semestre) {
        persistenceBean.setSemestre(semestre);
    }

    /**
     * Gets the type.
     *
     * @return the type
     */
    public String getType() {
        return persistenceBean.getType();
    }

    /**
     * Sets the type.
     *
     * @param type
     *            the new type
     */
    public void setType(String type) {
        persistenceBean.setType(type);
    }

    /**
     * Gets the nature.
     *
     * @return the nature
     */
    public String getNature() {
        return persistenceBean.getNature();
    }

    /**
     * Sets the nature.
     *
     * @param nature
     *            the new nature
     */
    public void setNature(String nature) {
        persistenceBean.setNature(nature);
    }

    /**
     * Gets the nb maxi achoix.
     *
     * @return the nb maxi achoix
     */
    public String getNbMaxiAchoix() {
        return persistenceBean.getNbMaxiAchoix();
    }

    /**
     * Sets the nb maxi achoix.
     *
     * @param nbMaxiAchoix
     *            the new nb maxi achoix
     */
    public void setNbMaxiAchoix(String nbMaxiAchoix) {
        persistenceBean.setNbMaxiAchoix(nbMaxiAchoix);
    }

    /**
     * Gets the nb mini achoix.
     *
     * @return the nb mini achoix
     */
    public String getNbMiniAchoix() {
        return persistenceBean.getNbMiniAchoix();
    }

    /**
     * Sets the nb mini achoix.
     *
     * @param nbMiniAchoix
     *            the new nb mini achoix
     */
    public void setNbMiniAchoix(String nbMiniAchoix) {
        persistenceBean.setNbMiniAchoix(nbMiniAchoix);
    }

    /**
     * Gets the disciplines.
     *
     * @return the disciplines
     */
    public String getDisciplines() {
        return persistenceBean.getDisciplines();
    }

    /**
     * Sets the disciplines.
     *
     * @param disciplines
     *            the new disciplines
     */
    public void setDisciplines(String disciplines) {
        persistenceBean.setDisciplines(disciplines);
    }

    /**
     * Gets the code rattachement autres.
     *
     * @return the code rattachement autres
     */
    @Override
    public String getCodeRattachementAutres() {
        return persistenceBean.getCodeRattachementAutres();
    }

    /**
     * Sets the code rattachement autres.
     *
     * @param codeRattachementsAutres
     *            the new code rattachement autres
     */
    @Override
    public void setCodeRattachementAutres(String codeRattachementsAutres) {
        persistenceBean.setCodeRattachementAutres(codeRattachementsAutres);
    }

    /**
     * Gets the ville.
     *
     * @return the ville
     */
    public String getVille() {
        return persistenceBean.getVille();
    }

    /**
     * Sets the ville.
     *
     * @param ville
     *            the new ville
     */
    public void setVille(String ville) {
        persistenceBean.setVille(ville);
    }

    /**
     * Gets the a distance.
     *
     * @return the a distance
     */
    public String getaDistance() {
        return persistenceBean.getaDistance();
    }

    /**
     * Sets the a distance.
     *
     * @param aDistance
     *            the new a distance
     */
    public void setaDistance(String aDistance) {
        persistenceBean.setaDistance(aDistance);
    }

    /**
     * Gets the mixte distance.
     *
     * @return the mixte distance
     */
    public String getMixteDistance() {
        return persistenceBean.getMixteDistance();
    }

    /**
     * Sets the mixte distance.
     *
     * @param mixteDistance
     *            the new mixte distance
     */
    public void setMixteDistance(String mixteDistance) {
        persistenceBean.setMixteDistance(mixteDistance);
    }

    /**
     * Gets the code responsables.
     *
     * @return the code responsables
     */
    public String getCodeResponsables() {
        return persistenceBean.getCodeResponsables();
    }

    /**
     * Sets the code responsables.
     *
     * @param codeResponsables
     *            the new code responsables
     */
    public void setCodeResponsables(String codeResponsables) {
        persistenceBean.setCodeResponsables(codeResponsables);
    }

    /**
     * Gets the complements responsable.
     *
     * @return the complements responsable
     */
    public String getComplementsResponsable() {
        return persistenceBean.getComplementsResponsable();
    }

    /**
     * Sets the complements responsable.
     *
     * @param complementsResponsable
     *            the new complements responsable
     */
    public void setComplementsResponsable(String complementsResponsable) {
        persistenceBean.setComplementsResponsable(complementsResponsable);
    }

    /**
     * Gets the coordonnees.
     *
     * @return the coordonnees
     */
    public String getCoordonnees() {
        return persistenceBean.getCoordonnees();
    }

    /**
     * Sets the coordonnees.
     *
     * @param coordonnees
     *            the new coordonnees
     */
    public void setCoordonnees(String coordonnees) {
        persistenceBean.setCoordonnees(coordonnees);
    }

    /**
     * Gets the volume horaire total.
     *
     * @return the volume horaire total
     */
    public String getVolumeHoraireTotal() {
        return persistenceBean.getVolumeHoraireTotal();
    }

    /**
     * Sets the volume horaire total.
     *
     * @param volumeHoraireTotal
     *            the new volume horaire total
     */
    public void setVolumeHoraireTotal(String volumeHoraireTotal) {
        persistenceBean.setVolumeHoraireTotal(volumeHoraireTotal);
    }

    /**
     * Gets the volume horaire cm.
     *
     * @return the volume horaire cm
     */
    public String getVolumeHoraireCm() {
        return persistenceBean.getVolumeHoraireCm();
    }

    /**
     * Sets the volume horaire cm.
     *
     * @param volumeHoraireCm
     *            the new volume horaire cm
     */
    public void setVolumeHoraireCm(String volumeHoraireCm) {
        persistenceBean.setVolumeHoraireCm(volumeHoraireCm);
    }

    /**
     * Gets the volume horaire td.
     *
     * @return the volume horaire td
     */
    public String getVolumeHoraireTd() {
        return persistenceBean.getVolumeHoraireTd();
    }

    /**
     * Sets the volume horaire td.
     *
     * @param volumeHoraireTd
     *            the new volume horaire td
     */
    public void setVolumeHoraireTd(String volumeHoraireTd) {
        persistenceBean.setVolumeHoraireTd(volumeHoraireTd);
    }

    /**
     * Gets the volume horaire tp.
     *
     * @return the volume horaire tp
     */
    public String getVolumeHoraireTp() {
        return persistenceBean.getVolumeHoraireTp();
    }

    /**
     * Sets the volume horaire tp.
     *
     * @param volumeHoraireTp
     *            the new volume horaire tp
     */
    public void setVolumeHoraireTp(String volumeHoraireTp) {
        persistenceBean.setVolumeHoraireTp(volumeHoraireTp);
    }

    /**
     * Gets the volume horaire projet.
     *
     * @return the volume horaire projet
     */
    public String getVolumeHoraireProjet() {
        return persistenceBean.getVolumeHoraireProjet();
    }

    /**
     * Sets the volume horaire projet.
     *
     * @param volumeHoraireProjet
     *            the new volume horaire projet
     */
    public void setVolumeHoraireProjet(String volumeHoraireProjet) {
        persistenceBean.setVolumeHoraireProjet(volumeHoraireProjet);
    }

    /**
     * Gets the volume horaire stage.
     *
     * @return the volume horaire stage
     */
    public String getVolumeHoraireStage() {
        return persistenceBean.getVolumeHoraireStage();
    }

    /**
     * Sets the volume horaire stage.
     *
     * @param volumeHoraireStage
     *            the new volume horaire stage
     */
    public void setVolumeHoraireStage(String volumeHoraireStage) {
        persistenceBean.setVolumeHoraireStage(volumeHoraireStage);
    }

    /**
     * Gets the volume horaire ds.
     *
     * @return the volume horaire ds
     */
    public String getVolumeHoraireDs() {
        return persistenceBean.getVolumeHoraireDs();
    }

    /**
     * Sets the volume horaire ds.
     *
     * @param volumeHoraireDs
     *            the new volume horaire ds
     */
    public void setVolumeHoraireDs(String volumeHoraireDs) {
        persistenceBean.setVolumeHoraireDs(volumeHoraireDs);
    }

    /**
     * Gets the volume horaire formation distance.
     *
     * @return the volume horaire formation distance
     */
    public String getVolumeHoraireFormationDistance() {
        return persistenceBean.getVolumeHoraireFormationDistance();
    }

    /**
     * Sets the volume horaire formation distance.
     *
     * @param volumeHoraireFormationDistance
     *            the new volume horaire formation distance
     */
    public void setVolumeHoraireFormationDistance(String volumeHoraireFormationDistance) {
        persistenceBean.setVolumeHoraireFormationDistance(volumeHoraireFormationDistance);
    }

    /**
     * Gets the volume horaire presentiel.
     *
     * @return the volume horaire presentiel
     */
    public String getVolumeHorairePresentiel() {
        return persistenceBean.getVolumeHorairePresentiel();
    }

    /**
     * Sets the volume horaire presentiel.
     *
     * @param volumeHorairePresentiel
     *            the new volume horaire presentiel
     */
    public void setVolumeHorairePresentiel(String volumeHorairePresentiel) {
        persistenceBean.setVolumeHorairePresentiel(volumeHorairePresentiel);
    }

    /**
     * Gets the prerequis.
     *
     * @return the prerequis
     */
    public String getPrerequis() {
        return persistenceBean.getPrerequis();
    }

    /**
     * Sets the prerequis.
     *
     * @param prerequis
     *            the new prerequis
     */
    public void setPrerequis(String prerequis) {
        persistenceBean.setPrerequis(prerequis);
    }

    /**
     * Gets the code ueups requises.
     *
     * @return the code ueups requises
     */
    public String getCodeUeupsRequises() {
        return persistenceBean.getUeupsRequises();
    }

    /**
     * Sets the code ueups requises.
     *
     * @param codeUeupsRequises
     *            the new code ueups requises
     */
    public void setCodeUeupsRequises(String codeUeupsRequises) {
        persistenceBean.setUeupsRequises(codeUeupsRequises);
    }

    /**
     * Gets the objectifs.
     *
     * @return the objectifs
     */
    public String getObjectifs() {
        return persistenceBean.getObjectifs();
    }

    /**
     * Sets the objectifs.
     *
     * @param objectifs
     *            the new objectifs
     */
    public void setObjectifs(String objectifs) {
        persistenceBean.setObjectifs(objectifs);
    }

    /**
     * Gets the calendrier.
     *
     * @return the calendrier
     */
    public String getCalendrier() {
        return persistenceBean.getCalendrier();
    }

    /**
     * Sets the calendrier.
     *
     * @param calendrier
     *            the new calendrier
     */
    public void setCalendrier(String calendrier) {
        persistenceBean.setCalendrier(calendrier);
    }

    /**
     * Gets the contenu.
     *
     * @return the contenu
     */
    public String getContenu() {
        return persistenceBean.getContenu();
    }

    /**
     * Sets the contenu.
     *
     * @param contenu
     *            the new contenu
     */
    public void setContenu(String contenu) {
        persistenceBean.setContenu(contenu);
    }

    /**
     * Gets the code ueupfils.
     *
     * @return the code ueupfils
     */
    public String getUeupfils() {
        return persistenceBean.getUeupfils();
    }

    /**
     * Sets the code ueupfils.
     *
     * @param codeUeupfils
     *            the new code ueupfils
     */
    public void setUeupfils(String codeUeupfils) {
        persistenceBean.setUeupfils(codeUeupfils);
    }

    /**
     * Gets the code cours.
     *
     * @return the code cours
     */
    public String getCours() {
        return persistenceBean.getCours();
    }

    /**
     * Sets the code cours.
     *
     * @param codeCours
     *            the new code cours
     */
    public void setCours(String codeCours) {
        persistenceBean.setCours(codeCours);
    }

    /**
     * Gets the bibliographie.
     *
     * @return the bibliographie
     */
    public String getBibliographie() {
        return persistenceBean.getBibliographie();
    }

    /**
     * Sets the bibliographie.
     *
     * @param bibliographie
     *            the new bibliographie
     */
    public void setBibliographie(String bibliographie) {
        persistenceBean.setBibliographie(bibliographie);
    }

    /**
     * Gets the controle connaissances.
     *
     * @return the controle connaissances
     */
    public String getControleConnaissances() {
        return persistenceBean.getControleConnaissances();
    }

    /**
     * Sets the controle connaissances.
     *
     * @param controleConnaissances
     *            the new controle connaissances
     */
    public void setControleConnaissances(String controleConnaissances) {
        persistenceBean.setControleConnaissances(controleConnaissances);
    }

    /**
     * Gets the domaines.
     *
     * @return the domaines
     */
    public String getDomaines() {
        return persistenceBean.getDomaines();
    }

    /**
     * Sets the domaines.
     *
     * @param domaines
     *            the new domaines
     */
    public void setDomaines(String domaines) {
        persistenceBean.setDomaines(domaines);
    }

    /**
     * Gets the domaine erasmus.
     *
     * @return the domaine erasmus
     */
    public String getDomaineErasmus() {
        return persistenceBean.getDomaineErasmus();
    }

    /**
     * Sets the domaine erasmus.
     *
     * @param domaineErasmus
     *            the new domaine erasmus
     */
    public void setDomaineErasmus(String domaineErasmus) {
        persistenceBean.setDomaineErasmus(domaineErasmus);
    }

    /**
     * Gets the grade.
     *
     * @return the grade
     */
    public String getGrade() {
        return persistenceBean.getGrade();
    }

    /**
     * Sets the grade.
     *
     * @param grade
     *            the new grade
     */
    public void setGrade(String grade) {
        persistenceBean.setGrade(grade);
    }

    /**
     * Gets the type diplome.
     *
     * @return the type diplome
     */
    public String getTypeDiplome() {
        return persistenceBean.getTypeDiplome();
    }

    /**
     * Sets the type diplome.
     *
     * @param typeDiplome
     *            the new type diplome
     */
    public void setTypeDiplome(String typeDiplome) {
        persistenceBean.setTypeDiplome(typeDiplome);
    }

    /**
     * Gets the langues enseignement.
     *
     * @return the langues enseignement
     */
    public String getLanguesEnseignement() {
        return persistenceBean.getLanguesEnseignement();
    }

    /**
     * Sets the langues enseignement.
     *
     * @param languesEnseignement the new langues enseignement
     */
    public void setLanguesEnseignement(String languesEnseignement) {
        persistenceBean.setLanguesEnseignement(languesEnseignement);
    }

    /**
     * Gets the complements.
     *
     * @return the complements
     */
    public String getComplements() {
        return persistenceBean.getComplements();
    }

    /**
     * Sets the complements.
     *
     * @param complements
     *            the new complements
     */
    public void setComplements(String complements) {
        persistenceBean.setComplements(complements);
    }

    /**
     * Gets the rattachement bandeau.
     *
     * @return the rattachement bandeau
     */
    public String getRattachementBandeau() {
        return persistenceBean.getRattachementBandeau();
    }

    /**
     * Sets the rattachement bandeau.
     *
     * @param rattachementBandeau
     *            the new rattachement bandeau
     */
    public void setRattachementBandeau(String rattachementBandeau) {
        persistenceBean.setRattachementBandeau(rattachementBandeau);
    }

    public Set<FormationBean> getListeFormation() {
        ServiceOffreFormation serviceOffreFormation = ServiceManager.getService(ServiceOffreFormation.class);
        return serviceOffreFormation.findAllFormationByCodeUeup(getCode(), getLangue());
    }
}
