/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.Date;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.om.Annuaireanciensetudiants;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;

/**
 * Processus de gestion des annuaireanciensetudiantss.
 */
public class SaisieAnnuaireanciensetudiants extends SaisieFiche {

	protected Annuaireanciensetudiants annuaireanciensetudiants = null;

	/**
	 * Constructeur.
	 *
	 * @param infoBean
	 */
	public SaisieAnnuaireanciensetudiants(final InfoBean infoBean) {
		super(infoBean);
	}

	/**
	 * Point d'entree du processus.
	 */
	@Override
	public boolean traiterAction() throws Exception {
		infoBean.set("CODE_OBJET", "0013");
		annuaireanciensetudiants = new Annuaireanciensetudiants();
		annuaireanciensetudiants.init();
		traiterActionParDefaut(annuaireanciensetudiants);
		infoBean.set("NOM_ONGLET", "annuaireanciensetudiants");
		// on continue si on n'est pas a la FIN !!!
		return etat == FIN;
	}

	@Override
	protected void preparerRECHERCHE() {
		infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
		infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelCombo("65", this.getLocale()));
	}

	@Override
	protected void prepareRestrictedSearch(final String sectionCode) {
		infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
		infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelComboWithRestrictionInformation("65", this.getLocale()));
	}

	/**
	 * Affichage de l'ecran de saisie d'un(e) annuaireanciensetudiants.
	 */
	@Override
	protected void preparerPRINCIPAL() throws Exception {
		ecranLogique = ECRAN_PRINCIPAL;
		infoBean.set("ID_ANNUAIREANCIENSETUDIANTS", annuaireanciensetudiants.getIdAnnuaireanciensetudiants().toString());
		infoBean.set("CIVILITE", annuaireanciensetudiants.getCivilite());
		infoBean.set("NOM", annuaireanciensetudiants.getNom());
		infoBean.set("PRENOM", annuaireanciensetudiants.getPrenom());
		infoBean.set("DATE_NAISSANCE", annuaireanciensetudiants.getDateNaissance());
		infoBean.set("CODE_RATTACHEMENT_AUTRES", annuaireanciensetudiants.getCodeRattachementAutres());
		infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", annuaireanciensetudiants.getLibelleCodeRattachementAutres());
		infoBean.set("DIPLOMES_OBTENUS", annuaireanciensetudiants.getDiplomesObtenus());
		infoBean.set("LIBELLE_DIPLOMES_OBTENUS", annuaireanciensetudiants.getLibelleDiplomesObtenus());
		infoBean.set("ANNEE_DIPLOME", annuaireanciensetudiants.getAnneeDiplome());
		infoBean.set("FONCTION_ACTUELLE", annuaireanciensetudiants.getFonctionActuelle());
		infoBean.set("ENTREPRISE", annuaireanciensetudiants.getEntreprise());
		infoBean.set("ADRESSE_MAIL", annuaireanciensetudiants.getAdresseMail());
		infoBean.set("CONTACT", annuaireanciensetudiants.getContact());
		infoBean.set("LISTE_ROUGE", annuaireanciensetudiants.getListeRouge());
		infoBean.set("CV", annuaireanciensetudiants.getCv());
		infoBean.set("SECTEURS_ACTIVITES", annuaireanciensetudiants.getSecteursActivites());
		infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelForComboForSection("65", annuaireanciensetudiants.getLangue(), getCurrentSection(annuaireanciensetudiants.getPersistanceBean())));
		infoBean.set("ACCROCHE_TEMOIGNAGE", annuaireanciensetudiants.getAccrocheTemoignage());
		infoBean.set("TEMOIGNAGE", annuaireanciensetudiants.getTemoignage());
		infoBean.set("PROFIL_LINKEDIN", annuaireanciensetudiants.getUrlProfilLinkedin());
		infoBean.set("PROFIL_VIADEO", annuaireanciensetudiants.getUrlProfilViadeo());
		infoBean.set("LIBELLE_SECTEURS_ACTIVITES", LabelUtils.getLibelleWithSection("65", annuaireanciensetudiants.getLibelleSecteursActivites(), annuaireanciensetudiants.getLangue(), getCurrentSection(annuaireanciensetudiants.getPersistanceBean())));
		infoBean.set("ADRESSE_SITE_PERSO", annuaireanciensetudiants.getAdresseSitePerso());
		infoBean.set("COMPLEMENTS", annuaireanciensetudiants.getComplements());
		infoBean.set("CODE_RATTACHEMENT", annuaireanciensetudiants.getCodeRattachement());
		final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
		infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(annuaireanciensetudiants.getCodeRattachement(), getCurrentSection(annuaireanciensetudiants.getPersistanceBean())));
		infoBean.set("LIBELLE_AFFICHABLE", annuaireanciensetudiants.getLibelleSansFallback());
		if (infoBean.get("SAISIE_FRONT") == null) {
			infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
		}
		ControleurUniv.preparerPRINCIPAL(infoBean, annuaireanciensetudiants, this);
	}

	@Override
	protected void alimenterDonneesFiche() throws Exception {
		annuaireanciensetudiants.setCivilite((String) infoBean.get("CIVILITE"));
		annuaireanciensetudiants.setNom((String) infoBean.get("NOM"));
		annuaireanciensetudiants.setPrenom((String) infoBean.get("PRENOM"));
		annuaireanciensetudiants.setDateNaissance((Date) infoBean.get("DATE_NAISSANCE"));
		annuaireanciensetudiants.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
		annuaireanciensetudiants.setDiplomesObtenus((String) infoBean.get("DIPLOMES_OBTENUS"));
		annuaireanciensetudiants.setAnneeDiplome((String) infoBean.get("ANNEE_DIPLOME"));
		annuaireanciensetudiants.setFonctionActuelle((String) infoBean.get("FONCTION_ACTUELLE"));
		annuaireanciensetudiants.setEntreprise((String) infoBean.get("ENTREPRISE"));
		annuaireanciensetudiants.setAdresseMail((String) infoBean.get("ADRESSE_MAIL"));
		annuaireanciensetudiants.setContact((String) infoBean.get("CONTACT"));
		annuaireanciensetudiants.setListeRouge((String) infoBean.get("LISTE_ROUGE"));
		annuaireanciensetudiants.setCv((String) infoBean.get("CV"));
		annuaireanciensetudiants.setSecteursActivites((String) infoBean.get("SECTEURS_ACTIVITES"));
		annuaireanciensetudiants.setAccrocheTemoignage((String) infoBean.get("ACCROCHE_TEMOIGNAGE"));
		annuaireanciensetudiants.setTemoignage((String) infoBean.get("TEMOIGNAGE"));
		annuaireanciensetudiants.setUrlProfilLinkedin((String) infoBean.get("PROFIL_LINKEDIN"));
		annuaireanciensetudiants.setUrlProfilViadeo((String) infoBean.get("PROFIL_VIADEO"));
		annuaireanciensetudiants.setAdresseSitePerso((String) infoBean.get("ADRESSE_SITE_PERSO"));
		annuaireanciensetudiants.setComplements((String) infoBean.get("COMPLEMENTS"));
		annuaireanciensetudiants.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
		annuaireanciensetudiants.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
	}

	/**
	 * Traite l'ecran de saisie d'une fiche.
	 */
	@Override
	protected void traiterPRINCIPAL() throws Exception {
		// chargement de l'objet
		if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
			annuaireanciensetudiants.init();
		} else {
			annuaireanciensetudiants.setIdAnnuaireanciensetudiants(new Long(infoBean.getString("ID_ANNUAIREANCIENSETUDIANTS")));
			annuaireanciensetudiants.retrieve();
		}
		// changement d'onglet
		if (action.equals(InfoBean.ACTION_ONGLET)) {
			infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
		}
		// validation de l'ecran
		else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
			alimenteCodeSiVide(infoBean);
			alimenteDonneesCreation(annuaireanciensetudiants, true);
			alimenterDonneesFiche();
		}
		// appel au traitement general
		ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, annuaireanciensetudiants, this);
		if (ecranLogique.length() == 0) {
			etat = FIN;
		}
	}
}
