/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.om.Annuaireetudiants;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;

/**
 * Processus de gestion des annuaireetudiantss.
 */
public class SaisieAnnuaireetudiants extends SaisieFiche {

    protected Annuaireetudiants annuaireetudiants = null;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieAnnuaireetudiants(final InfoBean infoBean) {
        super(infoBean);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0007");
        annuaireetudiants = new Annuaireetudiants();
        annuaireetudiants.init();
        traiterActionParDefaut(annuaireetudiants);
        infoBean.set("NOM_ONGLET", "annuaireetudiants");
        // on continue si on n'est pas a la FIN !!!
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo("55", this.getLocale()));
        final Map<String, String> hdiplome = LabelUtils.getLabelCombo("20", this.getLocale());
        infoBean.set("LISTE_TYPE_DIPLOMES", generateDiplomesMap(hdiplome));
        infoBean.set("LISTE_COMPETENCES_DIPLOMES", LabelUtils.getLabelCombo("51", this.getLocale()));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelCombo("65", this.getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelComboWithRestrictionInformation("55", this.getLocale()));
        final Map<String, String> hdiplome = LabelUtils.getLabelComboWithRestrictionInformation("20", this.getLocale());
        infoBean.set("LISTE_TYPE_DIPLOMES", generateDiplomesMap(hdiplome));
        infoBean.set("LISTE_COMPETENCES_DIPLOMES", LabelUtils.getLabelComboWithRestrictionInformation("51", this.getLocale()));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelComboWithRestrictionInformation("65", this.getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) annuaireetudiants.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_ANNUAIREETUDIANTS", annuaireetudiants.getIdAnnuaireetudiants().toString());
        infoBean.set("CIVILITE", annuaireetudiants.getCivilite());
        infoBean.set("NOM", annuaireetudiants.getNom());
        infoBean.set("PRENOM", annuaireetudiants.getPrenom());
        infoBean.set("DATE_NAISSANCE", annuaireetudiants.getDateNaissance());
        infoBean.set("CODE_RATTACHEMENT_AUTRES", annuaireetudiants.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", annuaireetudiants.getLibelleCodeRattachementAutres());
        infoBean.set("CODE_FORMATION", annuaireetudiants.getCodeFormation());
        infoBean.set("LIBELLE_CODE_FORMATION", annuaireetudiants.getLibelleCodeFormation());
        infoBean.set("PROMOTION", annuaireetudiants.getPromotion());
        infoBean.set("ADRESSE_MAIL", annuaireetudiants.getAdresseMail());
        infoBean.set("CONTACT", annuaireetudiants.getContact());
        infoBean.set("LISTE_ROUGE", annuaireetudiants.getListeRouge());
        infoBean.set("DISCIPLINE", annuaireetudiants.getDiscipline());
        infoBean.set("LIBELLE_DISCIPLINE", LabelUtils.getLibelleWithSection("55", annuaireetudiants.getDiscipline(), annuaireetudiants.getLangue(), getCurrentSection(annuaireetudiants.getPersistanceBean())));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelForComboForSection("55", annuaireetudiants.getLangue(), getCurrentSection(annuaireetudiants.getPersistanceBean())));
        infoBean.set("NIVEAU_DIPLOME", annuaireetudiants.getNiveauDiplome());
        infoBean.set("TYPE_DIPLOME", annuaireetudiants.getTypeDiplome());
        final Map<String, String> hdiplome = LabelUtils.getLabelForComboForSection("20", annuaireetudiants.getLangue(), getCurrentSection(annuaireetudiants.getPersistanceBean()));
        int i = 1;
        while (hdiplome.get("#" + i + "0000#") != null) {
            hdiplome.remove("#" + i + "0000#");
            i++;
        }
        infoBean.set("LISTE_TYPE_DIPLOMES", hdiplome);
        infoBean.set("CV", annuaireetudiants.getCv());
        infoBean.set("COMPETENCES_DIPLOME", annuaireetudiants.getCompetencesDiplome());
        infoBean.set("LIBELLE_COMPETENCES_DIPLOME", LabelUtils.getLibelleWithSection("51", annuaireetudiants.getLibelleCompetencesDiplome(), annuaireetudiants.getLangue(), getCurrentSection(annuaireetudiants.getPersistanceBean())));
        infoBean.set("LISTE_COMPETENCES_DIPLOMES", LabelUtils.getLabelForComboForSection("51", annuaireetudiants.getLangue(), getCurrentSection(annuaireetudiants.getPersistanceBean())));
        infoBean.set("COMPETENCES_GENERALES", annuaireetudiants.getCompetencesGenerales());
        infoBean.set("COMPETENCES_INFORMATIQUES", annuaireetudiants.getCompetencesInformatiques());
        infoBean.set("LANGUES_PARLEES", annuaireetudiants.getLanguesParlees());
        infoBean.set("OBJECTIF_PROFESSIONNEL", annuaireetudiants.getObjectifProfessionnel());
        infoBean.set("SECTEURS_ACTIVITES", annuaireetudiants.getSecteursActivites());
        infoBean.set("LIBELLE_SECTEURS_ACTIVITES", LabelUtils.getLibelleWithSection("65", annuaireetudiants.getLibelleSecteursActivites(), annuaireetudiants.getLangue(), getCurrentSection(annuaireetudiants.getPersistanceBean())));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelForComboForSection("65", annuaireetudiants.getLangue(), getCurrentSection(annuaireetudiants.getPersistanceBean())));
        infoBean.set("EXPERIENCES_PROFESSIONNELLES", annuaireetudiants.getExperiencesProfessionnelles());
        infoBean.set("PHOTO", annuaireetudiants.getPhoto().toString());
        infoBean.set("ADRESSE_SITE_PERSO", annuaireetudiants.getAdresseSitePerso());
        infoBean.set("CENTRES_INTERETS", annuaireetudiants.getCentresInterets());
        infoBean.set("COMPLEMENTS", annuaireetudiants.getComplements());
        infoBean.set("DATE_SUPPRESSION", annuaireetudiants.getDateSuppression());
        infoBean.set("CODE_RATTACHEMENT", annuaireetudiants.getCodeRattachement());
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(annuaireetudiants.getCodeRattachement(), getCurrentSection(annuaireetudiants.getPersistanceBean())));
        infoBean.set("LIBELLE_AFFICHABLE", annuaireetudiants.getLibelleSansFallback());
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        ControleurUniv.preparerPRINCIPAL(infoBean, annuaireetudiants, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        annuaireetudiants.setCivilite((String) infoBean.get("CIVILITE"));
        annuaireetudiants.setNom((String) infoBean.get("NOM"));
        annuaireetudiants.setPrenom((String) infoBean.get("PRENOM"));
        annuaireetudiants.setDateNaissance((Date) infoBean.get("DATE_NAISSANCE"));
        annuaireetudiants.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
        annuaireetudiants.setCodeFormation((String) infoBean.get("CODE_FORMATION"));
        annuaireetudiants.setPromotion((String) infoBean.get("PROMOTION"));
        annuaireetudiants.setAdresseMail((String) infoBean.get("ADRESSE_MAIL"));
        annuaireetudiants.setContact((String) infoBean.get("CONTACT"));
        annuaireetudiants.setListeRouge((String) infoBean.get("LISTE_ROUGE"));
        annuaireetudiants.setDiscipline((String) infoBean.get("DISCIPLINE"));
        annuaireetudiants.setNiveauDiplome((String) infoBean.get("NIVEAU_DIPLOME"));
        annuaireetudiants.setTypeDiplome((String) infoBean.get("TYPE_DIPLOME"));
        annuaireetudiants.setCv((String) infoBean.get("CV"));
        annuaireetudiants.setCompetencesDiplome((String) infoBean.get("COMPETENCES_DIPLOME"));
        annuaireetudiants.setCompetencesGenerales((String) infoBean.get("COMPETENCES_GENERALES"));
        annuaireetudiants.setCompetencesInformatiques((String) infoBean.get("COMPETENCES_INFORMATIQUES"));
        annuaireetudiants.setLanguesParlees((String) infoBean.get("LANGUES_PARLEES"));
        annuaireetudiants.setObjectifProfessionnel((String) infoBean.get("OBJECTIF_PROFESSIONNEL"));
        annuaireetudiants.setSecteursActivites((String) infoBean.get("SECTEURS_ACTIVITES"));
        annuaireetudiants.setExperiencesProfessionnelles((String) infoBean.get("EXPERIENCES_PROFESSIONNELLES"));
        annuaireetudiants.setPhoto(new Long(infoBean.getString("PHOTO")));
        annuaireetudiants.setAdresseSitePerso((String) infoBean.get("ADRESSE_SITE_PERSO"));
        annuaireetudiants.setCentresInterets((String) infoBean.get("CENTRES_INTERETS"));
        annuaireetudiants.setComplements((String) infoBean.get("COMPLEMENTS"));
        annuaireetudiants.setDateSuppression((Date) infoBean.get("DATE_SUPPRESSION"));
        annuaireetudiants.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        annuaireetudiants.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            annuaireetudiants.init();
        } else {
            annuaireetudiants.setIdAnnuaireetudiants(new Long(infoBean.getString("ID_ANNUAIREETUDIANTS")));
            annuaireetudiants.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteCodeSiVide(infoBean);
            alimenteDonneesCreation(annuaireetudiants, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, annuaireetudiants, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }

    /**
     * Génère la map des diplômes à faire apparaître dans la recherche.
     * @param hdiplome la map des diplômes non travaillées
     * @return la map des diplômes agrémentée
     */
    private Map<String, String> generateDiplomesMap(final Map<String, String> hdiplome) {
        final Map<String, String> outputMap = new HashMap<>(hdiplome);
        int i = 1;
        while (outputMap.get("#" + i + "0000#") != null) {
            outputMap.put(i + "0000", "#GROUPE#" + hdiplome.get("#" + i + "0000#"));
            outputMap.put(i + "9999", "#/GROUPE#");
            outputMap.remove("#" + i + "0000#");
            i++;
        }
        return outputMap;
    }
}
