/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.Date;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.om.Annuaireksup;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.utils.UnivFmt;

/**
 * Processus de gestion des annuaires.
 */
public class SaisieAnnuaireksup extends SaisieFiche {

    protected Annuaireksup annuaire = null;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieAnnuaireksup(final InfoBean infoBean) {
        super(infoBean);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0006");
        annuaire = new Annuaireksup();
        annuaire.init();
        traiterActionParDefaut(annuaire);
        infoBean.set("NOM_ONGLET", "annuaireksup");
        // on continue si on n'est pas a la FIN !!!
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_TYPE_POPULATIONS", LabelUtils.getLabelCombo("10", this.getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo("02", this.getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_TYPE_POPULATIONS", LabelUtils.getLabelComboWithRestrictionInformation("10", this.getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelComboWithRestrictionInformation("02", this.getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) annuaire.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_ANNUAIREKSUP", annuaire.getIdAnnuaireksup().toString());
        infoBean.set("DATE_NAISSANCE", annuaire.getDateNaissance());
        infoBean.set("CIVILITE", annuaire.getCivilite());
        infoBean.set("NOM", annuaire.getNom());
        infoBean.set("PRENOM", annuaire.getPrenom());
        infoBean.set("PHOTO", annuaire.getPhoto().toString());
        infoBean.set("TYPE_POPULATION", annuaire.getTypePopulation());
        infoBean.set("LISTE_TYPE_POPULATIONS", LabelUtils.getLabelForComboForSection("10", annuaire.getLangue(), getCurrentSection(annuaire.getPersistanceBean())));
        infoBean.set("FONCTION", annuaire.getFonction());
        infoBean.set("CORPS", annuaire.getCorps());
        infoBean.set("DISCIPLINE", annuaire.getDiscipline());
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelForComboForSection("02", annuaire.getLangue(), getCurrentSection(annuaire.getPersistanceBean())));
        infoBean.set("LIBELLE_DISCIPLINE", LabelUtils.getLibelleWithSection("02", annuaire.getLibelleDiscipline(), annuaire.getLangue(), getCurrentSection(annuaire.getPersistanceBean())));
        infoBean.set("INFOBULLE_DISCIPLINE", LabelUtils.getLibelleWithSection("02", annuaire.getLibelleDiscipline(), annuaire.getLangue(), getCurrentSection(annuaire.getPersistanceBean())));
        infoBean.set("DISCIPLINE_ENSEIGNEE", annuaire.getDisciplineEnseignee());
        infoBean.set("THEME_RECHERCHE", annuaire.getThemeRecherche());
        infoBean.set("ADRESSE", annuaire.getAdresse());
        infoBean.set("ADRESSE_MAIL", annuaire.getAdresseMail());
        infoBean.set("RATTACHEMENT_BANDEAU", annuaire.getRattachementBandeau());
        infoBean.set("BUREAU_1", annuaire.getBureau1());
        infoBean.set("TELEPHONE_1", annuaire.getTelephone1());
        infoBean.set("TELECOPIE_1", annuaire.getTelecopie1());
        infoBean.set("BUREAU_2", annuaire.getBureau2());
        infoBean.set("TELEPHONE_2", annuaire.getTelephone2());
        infoBean.set("TELECOPIE_2", annuaire.getTelecopie2());
        infoBean.set("CODE_RATTACHEMENT_AUTRES", annuaire.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", annuaire.getLibelleCodeRattachementAutres());
        infoBean.set("LISTE_ROUGE", annuaire.getListeRouge());
        infoBean.set("ADRESSE_SITE_PERSO", annuaire.getAdresseSitePerso());
        infoBean.set("CV", annuaire.getCv());
        infoBean.set("COMPLEMENTS", annuaire.getComplements());
        infoBean.set("CODE_RATTACHEMENT", annuaire.getCodeRattachement());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(annuaire.getCodeRattachement(), annuaire.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", annuaire.getLibelleSansFallback());
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        ControleurUniv.preparerPRINCIPAL(infoBean, annuaire, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.CIVILITE")) {
            annuaire.setCivilite((String) infoBean.get("CIVILITE"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.NOM")) {
            annuaire.setNom((String) infoBean.get("NOM"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.PRENOM")) {
            annuaire.setPrenom((String) infoBean.get("PRENOM"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.TYPE_POPULATION")) {
            annuaire.setTypePopulation((String) infoBean.get("TYPE_POPULATION"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.CORPS")) {
            annuaire.setCorps((String) infoBean.get("CORPS"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.DISCIPLINE")) {
            annuaire.setDiscipline((String) infoBean.get("DISCIPLINE"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.CODE_RATTACHEMENT")) {
            annuaire.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.CODE_RUBRIQUE")) {
            annuaire.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.TELEPHONE_1")) {
            annuaire.setTelephone1((String) infoBean.get("TELEPHONE_1"));
        }
        if (UnivFmt.estSaisissable(infoBean, "ANNUAIREKSUP.TELECOPIE_1")) {
            annuaire.setTelecopie1((String) infoBean.get("TELECOPIE_1"));
        }
        annuaire.setDateNaissance((Date) infoBean.get("DATE_NAISSANCE"));
        annuaire.setPhoto(new Long(infoBean.getString("PHOTO")));
        annuaire.setFonction((String) infoBean.get("FONCTION"));
        annuaire.setDisciplineEnseignee((String) infoBean.get("DISCIPLINE_ENSEIGNEE"));
        annuaire.setThemeRecherche((String) infoBean.get("THEME_RECHERCHE"));
        annuaire.setAdresse((String) infoBean.get("ADRESSE"));
        annuaire.setAdresseMail((String) infoBean.get("ADRESSE_MAIL"));
        annuaire.setRattachementBandeau((String) infoBean.get("RATTACHEMENT_BANDEAU"));
        annuaire.setBureau1((String) infoBean.get("BUREAU_1"));
        annuaire.setBureau2((String) infoBean.get("BUREAU_2"));
        annuaire.setTelephone2((String) infoBean.get("TELEPHONE_2"));
        annuaire.setTelecopie2((String) infoBean.get("TELECOPIE_2"));
        annuaire.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
        annuaire.setListeRouge((String) infoBean.get("LISTE_ROUGE"));
        annuaire.setAdresseSitePerso((String) infoBean.get("ADRESSE_SITE_PERSO"));
        annuaire.setCv((String) infoBean.get("CV"));
        annuaire.setComplements((String) infoBean.get("COMPLEMENTS"));
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            annuaire.init();
        } else {
            annuaire.setIdAnnuaireksup(new Long(infoBean.getString("ID_ANNUAIREKSUP")));
            annuaire.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteDonneesCreation(annuaire, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, annuaire, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }
}
