/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.om.Associationetudiant;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MediaUtils;

/**
 * Processus de gestion des associationetudiants.
 */
public class SaisieAssociationetudiant extends SaisieFiche {

    private final ServiceMedia serviceMedia;

    protected Associationetudiant associationetudiant = null;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieAssociationetudiant(final InfoBean infoBean) {
        super(infoBean);
        serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0012");
        associationetudiant = new Associationetudiant();
        associationetudiant.init();
        traiterActionParDefaut(associationetudiant);
        infoBean.set("NOM_ONGLET", "associationetudiant");
        // on continue si on n'est pas a la FIN !!!
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelCombo("04", this.getLocale()));
        infoBean.set("LISTE_TYPE_ASSOCIATIONS", LabelUtils.getLabelCombo("34", this.getLocale()));
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelComboWithRestrictionInformation("04", this.getLocale()));
        infoBean.set("LISTE_TYPE_ASSOCIATIONS", LabelUtils.getLabelComboWithRestrictionInformation("34", this.getLocale()));
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) associationetudiant.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_ASSOCIATIONETUDIANT", associationetudiant.getIdAssociationetudiant().toString());
        infoBean.set("NOM", associationetudiant.getNom());
        infoBean.set("OBJET_ASSO", associationetudiant.getObjet());
        infoBean.set("TYPE_ASSOCIATION", associationetudiant.getTypeAssociation());
        infoBean.set("LIBELLE_TYPE_ASSOCIATION", LabelUtils.getLibelleWithSection("34", associationetudiant.getLibelleTypeAssociation(), associationetudiant.getLangue(), getCurrentSection(associationetudiant.getPersistanceBean())));
        infoBean.set("LISTE_TYPE_ASSOCIATIONS", LabelUtils.getLabelForComboForSection("34", associationetudiant.getLangue(), getCurrentSection(associationetudiant.getPersistanceBean())));
        infoBean.set("THEMATIQUE", associationetudiant.getThematique());
        infoBean.set("LIBELLE_THEMATIQUE", LabelUtils.getLibelleWithSection("04", associationetudiant.getLibelleThematique(), associationetudiant.getLangue(), getCurrentSection(associationetudiant.getPersistanceBean())));
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelForComboForSection("04", associationetudiant.getLangue(), getCurrentSection(associationetudiant.getPersistanceBean())));
        infoBean.set("CODE_RATTACHEMENT_AUTRES", associationetudiant.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", associationetudiant.getLibelleCodeRattachementAutres());
        infoBean.set("ADRESSE_EMAIL", associationetudiant.getAdresseEmail());
        infoBean.set("COORDONNEES", associationetudiant.getCoordonnees());
        infoBean.set("SITE_WEB", associationetudiant.getSiteWeb());
        infoBean.set("NOM_PRESIDENT", associationetudiant.getNomPresident());
        infoBean.set("ADRESSE_EMAIL_PRESIDENT", associationetudiant.getAdresseEmailPresident());
        infoBean.set("TELEPHONE_PRESIDENT", associationetudiant.getTelephonePresident());
        infoBean.set("TELECOPIE_PRESIDENT", associationetudiant.getTelecopiePresident());
        infoBean.set("COTISATION", associationetudiant.getCotisation());
        infoBean.set("ACTIVITES", associationetudiant.getActivites());
        infoBean.set("PLAN_ACCES", associationetudiant.getPlanAcces().toString());
        final MediaBean media = serviceMedia.getById(associationetudiant.getPlanAcces());
        infoBean.set("LIBELLE_PLAN_ACCES", media != null ? StringUtils.defaultIfBlank(media.getTitre(), StringUtils.EMPTY) : StringUtils.EMPTY);
        infoBean.set("LISTE_PLANS_ACCES", MediaUtils.getMediasAsMap(serviceMedia.getByTypeAndTypeMedia("photo", "0003")));
        infoBean.set("COMPOSITION_BUREAU", associationetudiant.getCompositionBureau());
        infoBean.set("PERMANENCES", associationetudiant.getPermanences());
        infoBean.set("COMPLEMENTS", associationetudiant.getComplements());
        infoBean.set("CODE_RATTACHEMENT", associationetudiant.getCodeRattachement());
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(associationetudiant.getCodeRattachement(), associationetudiant.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", associationetudiant.getNom());
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        ControleurUniv.preparerPRINCIPAL(infoBean, associationetudiant, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        associationetudiant.setNom((String) infoBean.get("NOM"));
        associationetudiant.setObjet((String) infoBean.get("OBJET_ASSO"));
        associationetudiant.setTypeAssociation((String) infoBean.get("TYPE_ASSOCIATION"));
        associationetudiant.setThematique((String) infoBean.get("THEMATIQUE"));
        associationetudiant.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
        associationetudiant.setAdresseEmail((String) infoBean.get("ADRESSE_EMAIL"));
        associationetudiant.setCoordonnees((String) infoBean.get("COORDONNEES"));
        associationetudiant.setSiteWeb((String) infoBean.get("SITE_WEB"));
        associationetudiant.setNomPresident((String) infoBean.get("NOM_PRESIDENT"));
        associationetudiant.setAdresseEmailPresident((String) infoBean.get("ADRESSE_EMAIL_PRESIDENT"));
        associationetudiant.setTelephonePresident((String) infoBean.get("TELEPHONE_PRESIDENT"));
        associationetudiant.setTelecopiePresident((String) infoBean.get("TELECOPIE_PRESIDENT"));
        associationetudiant.setCotisation((String) infoBean.get("COTISATION"));
        associationetudiant.setActivites((String) infoBean.get("ACTIVITES"));
        associationetudiant.setPlanAcces(new Long(infoBean.getString("PLAN_ACCES")));
        associationetudiant.setCompositionBureau((String) infoBean.get("COMPOSITION_BUREAU"));
        associationetudiant.setPermanences((String) infoBean.get("PERMANENCES"));
        associationetudiant.setComplements((String) infoBean.get("COMPLEMENTS"));
        associationetudiant.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        associationetudiant.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            associationetudiant.init();
        } else {
            associationetudiant.setIdAssociationetudiant(new Long(infoBean.getString("ID_ASSOCIATIONETUDIANT")));
            associationetudiant.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteDonneesCreation(associationetudiant, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, associationetudiant, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }
}
