/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.bean.UeupBean;
import com.ksup.objetspartages.om.Cours;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.ksup.objetspartages.service.ServiceOffreFormation;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;

/**
 * Processus de gestion des courss.
 */
public class SaisieCours extends SaisieFiche {

    protected Cours cours = null;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieCours(final InfoBean infoBean) {
        super(infoBean);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0004");
        cours = new Cours();
        cours.init();
        traiterActionParDefaut(cours);
        infoBean.set("NOM_ONGLET", "cours");
        // on continue si on n'est pas a la FIN !!!
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), this.getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), this.getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) cours.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_COURS", cours.getIdCours().toString());
        infoBean.set("CODE_SI_SCOLARITE", cours.getCodeSiScolarite());
        infoBean.set("LIBELLE_COURT", cours.getLibelleCourt());
        infoBean.set("LIBELLE_LONG", cours.getLibelleLong());
        infoBean.set("CREDIT_ECTS", cours.getCreditEcts());
        infoBean.set("DISCIPLINE", cours.getDiscipline());
        infoBean.set("LIBELLE_DISCIPLINE", LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), cours.getLibelleDiscipline(), cours.getLangue(), getCurrentSection(cours.getPersistanceBean())));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), cours.getLangue(), getCurrentSection(cours.getPersistanceBean())));
        infoBean.set("CODE_RATTACHEMENTS_AUTRES", cours.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENTS_AUTRES", cours.getLibelleCodeRattachementsAutres());
        infoBean.set("CODE_RESPONSABLE", cours.getCodeResponsable());
        infoBean.set("LIBELLE_CODE_RESPONSABLE", cours.getLibelleCodeResponsable());
        infoBean.set("COMPLEMENTS_RESPONSABLE", cours.getComplementsResponsable());
        infoBean.set("COORDONNEES", cours.getCoordonnees());
        infoBean.set("VOLUME_HORAIRE_CM", cours.getVolumeHoraireCm());
        infoBean.set("VOLUME_HORAIRE_TD", cours.getVolumeHoraireTd());
        infoBean.set("VOLUME_HORAIRE_TP", cours.getVolumeHoraireTp());
        infoBean.set("VOLUME_HORAIRE_PROJET", cours.getVolumeHoraireProjet());
        infoBean.set("VOLUME_HORAIRE_STAGE", cours.getVolumeHoraireStage());
        infoBean.set("VOLUME_HORAIRE_DS", cours.getVolumeHoraireDs());
        infoBean.set("COEF_TP", cours.getCoefTp());
        infoBean.set("COEF_DS", cours.getCoefDs());
        infoBean.set("OBJECTIFS", cours.getObjectifs());
        infoBean.set("CALENDRIER", cours.getCalendrier());
        infoBean.set("CONTENU", cours.getContenu());
        infoBean.set("BIBLIOGRAPHIE", cours.getBibliographie());
        infoBean.set("CONTROLE_CONNAISSANCES", cours.getControleConnaissances());
        infoBean.set("COMPLEMENTS", cours.getComplements());
        infoBean.set("RATTACHEMENT_BANDEAU", cours.getRattachementBandeau());
        infoBean.set("CODE_RATTACHEMENT", cours.getCodeRattachement());
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(cours.getCodeRattachement(), cours.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", cours.getLibelleLong());
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        infoBean.set("LIBELLES_UEUP_PARENT", getLibellesUeupParent());
        ControleurUniv.preparerPRINCIPAL(infoBean, cours, this);
    }

    /**
     * Récupère la liste des libellés des ueup parents.
     * @return la liste des libellés
     */
    private List<String> getLibellesUeupParent() {
        if (!StringUtils.isNoneBlank(cours.getCode(), cours.getLangue())) {
            return Collections.emptyList();
        }
        final ServiceOffreFormation serviceUeup = ServiceManager.getService(ServiceOffreFormation.class);
        return serviceUeup.findUeupByCours(cours.getCode(), cours.getLangue()).stream()
            .map(UeupBean::getLibelleLong)
            .collect(Collectors.toList());
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        cours.setCodeSiScolarite((String) infoBean.get("CODE_SI_SCOLARITE"));
        cours.setLibelleCourt((String) infoBean.get("LIBELLE_COURT"));
        cours.setLibelleLong((String) infoBean.get("LIBELLE_LONG"));
        cours.setCreditEcts((String) infoBean.get("CREDIT_ECTS"));
        cours.setDiscipline((String) infoBean.get("DISCIPLINE"));
        cours.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENTS_AUTRES"));
        cours.setCodeResponsable((String) infoBean.get("CODE_RESPONSABLE"));
        cours.setComplementsResponsable((String) infoBean.get("COMPLEMENTS_RESPONSABLE"));
        cours.setCoordonnees((String) infoBean.get("COORDONNEES"));
        cours.setVolumeHoraireCm((String) infoBean.get("VOLUME_HORAIRE_CM"));
        cours.setVolumeHoraireTd((String) infoBean.get("VOLUME_HORAIRE_TD"));
        cours.setVolumeHoraireTp((String) infoBean.get("VOLUME_HORAIRE_TP"));
        cours.setVolumeHoraireProjet((String) infoBean.get("VOLUME_HORAIRE_PROJET"));
        cours.setVolumeHoraireStage((String) infoBean.get("VOLUME_HORAIRE_STAGE"));
        cours.setVolumeHoraireDs((String) infoBean.get("VOLUME_HORAIRE_DS"));
        cours.setCoefTp((String) infoBean.get("COEF_TP"));
        cours.setCoefDs((String) infoBean.get("COEF_DS"));
        cours.setObjectifs((String) infoBean.get("OBJECTIFS"));
        cours.setCalendrier((String) infoBean.get("CALENDRIER"));
        cours.setContenu((String) infoBean.get("CONTENU"));
        cours.setBibliographie((String) infoBean.get("BIBLIOGRAPHIE"));
        cours.setControleConnaissances((String) infoBean.get("CONTROLE_CONNAISSANCES"));
        cours.setComplements((String) infoBean.get("COMPLEMENTS"));
        cours.setRattachementBandeau((String) infoBean.get("RATTACHEMENT_BANDEAU"));
        cours.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        cours.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            cours.init();
        } else {
            cours.setIdCours(Long.valueOf(infoBean.getString("ID_COURS")));
            cours.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteDonneesCreation(cours, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, cours, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }
}
