/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.om.Defaultstructureksup;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheAnnuaire;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MediaUtils;
import com.univ.utils.UnivFmt;

/**
 * Processus de gestion des defaultstructureksups.
 */
public class SaisieDefaultstructureksup extends SaisieFiche {

    private final ServiceMedia serviceMedia;

    protected Defaultstructureksup defaultstructureksup = null;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieDefaultstructureksup(final InfoBean infoBean) {
        super(infoBean);
        serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0001");
        defaultstructureksup = new Defaultstructureksup();
        defaultstructureksup.init();
        final AutorisationBean autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
        if (autorisations != null) {
            this.getDatas().put("AUTORISATIONS", autorisations);
        }
        traiterActionParDefaut(defaultstructureksup);
        infoBean.set("NOM_ONGLET", "defaultstructureksup");
        // on continue si on n'est pas a la FIN !!!
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_TYPE_STRUCTURES", LabelUtils.getLabelCombo("60", this.getLocale()));
        infoBean.set("LISTE_VILLESS", LabelUtils.getLabelCombo("22", this.getLocale()));
        infoBean.set("LISTE_DOMAINE_FORMATION_ETABLISSEMENTS", LabelUtils.getLabelCombo("2004", this.getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_TYPE_STRUCTURES", LabelUtils.getLabelComboWithRestrictionInformation("60", this.getLocale()));
        infoBean.set("LISTE_VILLESS", LabelUtils.getLabelComboWithRestrictionInformation("22", this.getLocale()));
        infoBean.set("LISTE_DOMAINE_FORMATION_ETABLISSEMENTS", LabelUtils.getLabelComboWithRestrictionInformation("2004", this.getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) defaultstructureksup.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_DEFAULTSTRUCTUREKSUP", defaultstructureksup.getIdDefaultstructureksup().toString());
        infoBean.set("LIBELLE_COURT", defaultstructureksup.getLibelleCourt());
        infoBean.set("LIBELLE_LONG", defaultstructureksup.getLibelleLong());
        infoBean.set("TYPE_STRUCTURE", defaultstructureksup.getTypeStructure());
        infoBean.set("LISTE_TYPE_STRUCTURES", LabelUtils.getLabelForComboForSection("60", defaultstructureksup.getLangue(), getCurrentSection(defaultstructureksup.getPersistanceBean())));
        infoBean.set("STRUCTURE_ENSEIGNEMENT", defaultstructureksup.getStructureEnseignement());
        infoBean.set("ADRESSE", defaultstructureksup.getAdresse());
        infoBean.set("CODE_POSTAL", defaultstructureksup.getCodePostal());
        infoBean.set("VILLE", defaultstructureksup.getVille());
        infoBean.set("VILLES", defaultstructureksup.getVilles());
        infoBean.set("LIBELLE_VILLES", LabelUtils.getLibelleWithSection("22", defaultstructureksup.getVilles(), defaultstructureksup.getLangue(), getCurrentSection(defaultstructureksup.getPersistanceBean())));
        infoBean.set("LISTE_VILLESS", LabelUtils.getLabelForComboForSection("22", defaultstructureksup.getLangue(), getCurrentSection(defaultstructureksup.getPersistanceBean())));
        infoBean.set("TELEPHONE", defaultstructureksup.getTelephone());
        infoBean.set("TELECOPIE", defaultstructureksup.getTelecopie());
        infoBean.set("ADRESSE_MAIL", defaultstructureksup.getAdresseMail());
        infoBean.set("SITE_WEB", defaultstructureksup.getSiteWeb());
        infoBean.set("ID_PHOTO", defaultstructureksup.getIdPhoto().toString());
        infoBean.set("CODE_RESPONSABLE", defaultstructureksup.getCodeResponsable());
        infoBean.set("LIBELLE_CODE_RESPONSABLE", prepareLibelleReponsables());
        infoBean.set("ORGANIGRAMME", defaultstructureksup.getOrganigramme());
        infoBean.set("CONTACTS", defaultstructureksup.getContacts());
        infoBean.set("ACTIVITE", defaultstructureksup.getActivite());
        infoBean.set("CHIFFRES_CLES", defaultstructureksup.getChiffresCles());
        infoBean.set("DOMAINE_FORMATION_ETABLISSEMENT", defaultstructureksup.getDomaineFormationEtablissement());
        infoBean.set("LIBELLE_DOMAINE_FORMATION_ETABLISSEMENT", LabelUtils.getLibelleWithSection("2004", defaultstructureksup.getDomaineFormationEtablissement(), defaultstructureksup.getLangue(), getCurrentSection(defaultstructureksup.getPersistanceBean())));
        infoBean.set("LISTE_DOMAINE_FORMATION_ETABLISSEMENTS", LabelUtils.getLabelForComboForSection("2004", defaultstructureksup.getLangue(), getCurrentSection(defaultstructureksup.getPersistanceBean())));
        infoBean.set("FORMATIONS", defaultstructureksup.getFormations());
        infoBean.set("RECHERCHE", defaultstructureksup.getRecherche());
        infoBean.set("PARTENARIATS", defaultstructureksup.getPartenariats());
        infoBean.set("COMPLEMENT_COORDONNEES", defaultstructureksup.getComplementCoordonnees());
        infoBean.set("COMPLEMENTS", defaultstructureksup.getComplements());
        infoBean.set("ID_PLAN_ACCES", defaultstructureksup.getIdPlanAcces().toString());
        infoBean.set("LISTE_PLANS_ACCES", MediaUtils.getMediasAsMap(serviceMedia.getByTypeAndTypeMedia("photo", EtatFiche.EN_LIGNE.toString())));
        infoBean.set("CODE_RATTACHEMENT_AUTRES", defaultstructureksup.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", defaultstructureksup.getLibelleCodeRattachementAutres());
        infoBean.set("CODE_RATTACHEMENT", defaultstructureksup.getCodeRattachement());
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(defaultstructureksup.getCodeRattachement(), defaultstructureksup.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", defaultstructureksup.getLibelleLong());
        final MediaBean mediaBean = serviceMedia.getById(defaultstructureksup.getIdPlanAcces());
        if (mediaBean != null) {
            infoBean.set("LIBELLE_ID_PLAN_ACCES", StringUtils.defaultIfBlank(mediaBean.getTitre(), StringUtils.EMPTY));
        } else {
            infoBean.set("LIBELLE_ID_PLAN_ACCES", StringUtils.EMPTY);
        }
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        ControleurUniv.preparerPRINCIPAL(infoBean, defaultstructureksup, this);
    }

    private String prepareLibelleReponsables() throws Exception {
        final Collection<String> labels = new ArrayList<>();
        final String[] codes = defaultstructureksup.getCodeResponsable().split(";");
        for(String currentCode : codes)  {
            final String[] split = currentCode.split(",");
            final StringBuilder currentLabel = new StringBuilder(FicheAnnuaire.getLibelleAffichable(currentCode, defaultstructureksup.getLangue()));
            if (split.length == 3 && !"0000".equals(split[2])) {
                currentLabel.append(String.format(" [%s]", LabelUtils.getLibelleWithSection("64", split[2], defaultstructureksup.getLangue(), getCurrentSection(defaultstructureksup.getPersistanceBean()))));
            }
            labels.add(currentLabel.toString());
        }
        return StringUtils.join(labels, ";");
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        if (UnivFmt.estSaisissable(infoBean, "DEFAULTSTRUCTURE.LIBELLE_COURT")) {
            defaultstructureksup.setLibelleCourt((String) infoBean.get("LIBELLE_COURT"));
        }
        if (UnivFmt.estSaisissable(infoBean, "DEFAULTSTRUCTURE.LIBELLE_LONG")) {
            defaultstructureksup.setLibelleLong((String) infoBean.get("LIBELLE_LONG"));
        }
        defaultstructureksup.setTypeStructure((String) infoBean.get("TYPE_STRUCTURE"));
        defaultstructureksup.setStructureEnseignement((String) infoBean.get("STRUCTURE_ENSEIGNEMENT"));
        defaultstructureksup.setAdresse((String) infoBean.get("ADRESSE"));
        defaultstructureksup.setCodePostal((String) infoBean.get("CODE_POSTAL"));
        defaultstructureksup.setVille((String) infoBean.get("VILLE"));
        defaultstructureksup.setVilles((String) infoBean.get("VILLES"));
        defaultstructureksup.setTelephone((String) infoBean.get("TELEPHONE"));
        defaultstructureksup.setTelecopie((String) infoBean.get("TELECOPIE"));
        defaultstructureksup.setAdresseMail((String) infoBean.get("ADRESSE_MAIL"));
        defaultstructureksup.setSiteWeb((String) infoBean.get("SITE_WEB"));
        defaultstructureksup.setIdPhoto(new Long(infoBean.getString("ID_PHOTO")));
        defaultstructureksup.setCodeResponsable((String) infoBean.get("CODE_RESPONSABLE"));
        defaultstructureksup.setOrganigramme((String) infoBean.get("ORGANIGRAMME"));
        defaultstructureksup.setContacts((String) infoBean.get("CONTACTS"));
        defaultstructureksup.setActivite((String) infoBean.get("ACTIVITE"));
        defaultstructureksup.setChiffresCles((String) infoBean.get("CHIFFRES_CLES"));
        defaultstructureksup.setDomaineFormationEtablissement((String) infoBean.get("DOMAINE_FORMATION_ETABLISSEMENT"));
        defaultstructureksup.setFormations((String) infoBean.get("FORMATIONS"));
        defaultstructureksup.setRecherche((String) infoBean.get("RECHERCHE"));
        defaultstructureksup.setPartenariats((String) infoBean.get("PARTENARIATS"));
        defaultstructureksup.setComplementCoordonnees((String) infoBean.get("COMPLEMENT_COORDONNEES"));
        defaultstructureksup.setComplements((String) infoBean.get("COMPLEMENTS"));
        defaultstructureksup.setIdPlanAcces(new Long(infoBean.getString("ID_PLAN_ACCES")));
        defaultstructureksup.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
        defaultstructureksup.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        if (infoBean.getString("CODE_RATTACHEMENT").length() > 0) {
            defaultstructureksup.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        } else {
            defaultstructureksup.setCodeRattachement("");
        }
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            defaultstructureksup.init();
        } else {
            defaultstructureksup.setIdDefaultstructureksup(new Long(infoBean.getString("ID_DEFAULTSTRUCTUREKSUP")));
            defaultstructureksup.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteDonneesCreation(defaultstructureksup, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, defaultstructureksup, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }
}
