/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: offrestageemploi
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.List;
import java.util.Map.Entry;

import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;
import javax.mail.internet.AddressException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.ApplicationContextManager;
import com.jsbsoft.jtf.core.CodeLibelle;
import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.email.JSBMailbox;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.exception.ErreurTechnique;
import com.jsbsoft.jtf.upload.UploadedFile;
import com.kosmos.offrestageemploi.util.OffrestageemploiUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.ksup.objetspartages.om.Offresstagesemplois;
import com.univ.objetspartages.dto.LabelDto;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceLabel;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.url.provider.UrlProvider;
import com.univ.utils.EscapeString;
import com.univ.utils.MailUtil;

/**
 * Processus de gestion des offresstagesemploiss.
 */
public class SaisieOffresstagesemplois extends SaisieFiche {

    private static final Logger LOG = LoggerFactory.getLogger(SaisieOffresstagesemplois.class);

    private static final String ECRAN_REPONSE = "REPONSE";

    private static final String ECRAN_CONFIRMATION = "CONFIRMATION";

    protected Offresstagesemplois offresstagesemplois = null;

    private final ServiceLabel serviceLabel;

    private final UrlProvider urlProvider;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieOffresstagesemplois(final InfoBean infoBean) {
        super(infoBean);
        serviceLabel = ServiceManager.getService(ServiceLabel.class);
        urlProvider = ApplicationContextManager.getCoreContextBean(UrlProvider.ID_BEAN, UrlProvider.class);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0011");
        offresstagesemplois = new Offresstagesemplois();
        offresstagesemplois.init();
        action = infoBean.getString("ACTION");
        ecranLogique = infoBean.getEcranLogique();
        if (ECRAN_REPONSE.equals(ecranLogique)) {
            traiterReponse();
        } else if ("SAISIR_REPONSE".equals(action)) {
            preparerReponse();
        } else {
            traiterActionParDefaut(offresstagesemplois);
        }
        infoBean.set("NOM_ONGLET", "offresstagesemplois");
        // on continue si on n'est pas a la FIN !!!
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelCombo("65", this.getLocale()));
        infoBean.set("LISTE_DUREES", LabelUtils.getLabelCombo("32", this.getLocale()));
        infoBean.set("LISTE_DEBUTS", LabelUtils.getLabelCombo("33", this.getLocale()));
        infoBean.set("LISTE_TYPE_STAGES", LabelUtils.getLabelCombo("35", this.getLocale()));
        infoBean.set("LISTE_TYPE_JOBS", LabelUtils.getLabelCombo("36", this.getLocale()));
        infoBean.set("LISTE_TYPE_EMPLOIS", LabelUtils.getLabelCombo("37", this.getLocale()));
        infoBean.set("LISTE_TYPE_OFFRE", CodeLibelle.lireTable(infoBean.getNomExtension(), "type_offre", this.getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelComboWithRestrictionInformation("65", this.getLocale()));
        infoBean.set("LISTE_DUREES", LabelUtils.getLabelComboWithRestrictionInformation("32", this.getLocale()));
        infoBean.set("LISTE_DEBUTS", LabelUtils.getLabelComboWithRestrictionInformation("33", this.getLocale()));
        infoBean.set("LISTE_TYPE_STAGES", LabelUtils.getLabelComboWithRestrictionInformation("35", this.getLocale()));
        infoBean.set("LISTE_TYPE_JOBS", LabelUtils.getLabelComboWithRestrictionInformation("36", this.getLocale()));
        infoBean.set("LISTE_TYPE_EMPLOIS", LabelUtils.getLabelComboWithRestrictionInformation("37", this.getLocale()));
        infoBean.set("LISTE_TYPE_OFFRE", CodeLibelle.lireTable(infoBean.getNomExtension(), "type_offre", this.getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) offresstagesemplois.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_OFFRESSTAGESEMPLOIS", offresstagesemplois.getIdOffresstagesemplois().toString());
        infoBean.set("TITRE", offresstagesemplois.getTitre());
        infoBean.set("TYPE_OFFRE", offresstagesemplois.getTypeOffre());
        final Hashtable<String, String> hTypeOffre = CodeLibelle.lireTable(infoBean.getNomExtension(), "type_offre", this.getLocale());
        int compteur = hTypeOffre.size() - 1;
        final String code = "";
        for (final Entry<String, String> typeOffre : hTypeOffre.entrySet()) {
            infoBean.set("TYPE_OFFRE" + Integer.toString(compteur), typeOffre.getValue() + "#" + typeOffre.getKey());
            infoBean.set("CODE_OFFRE" + Integer.toString(compteur), code);
            compteur--;
        }
        infoBean.set("TYPE_STAGE", offresstagesemplois.getTypeStage());
        final List<LabelDto> stageLabels = serviceLabel.getByTypeLanguageSection("35", offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean()));
        List<String> listeTypesStage = new ArrayList<>();
        for (LabelDto currentStageLabel : stageLabels) {
            listeTypesStage.add(currentStageLabel.getLibelle() + "#" + currentStageLabel.getCode());
        }
        infoBean.set("LISTE_TYPE_STAGE", listeTypesStage);
        infoBean.set("TYPE_JOB", offresstagesemplois.getTypeJob());
        final List<LabelDto> jobLabels = serviceLabel.getByTypeLanguageSection("36", offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean()));
        List<String> listeTypesJob = new ArrayList<>();
        for (LabelDto currentLabelJob : jobLabels) {
            listeTypesJob.add(currentLabelJob.getLibelle() + "#" + currentLabelJob.getCode());
        }
        infoBean.set("LISTE_TYPE_JOB", listeTypesJob);
        infoBean.set("TYPE_EMPLOI", offresstagesemplois.getTypeEmploi());
        final List<LabelDto> typeLabels = serviceLabel.getByTypeLanguageSection("37", offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean()));
        List<String> listeTypesEmploi = new ArrayList<>();
        for (LabelDto currentType : typeLabels) {
            listeTypesEmploi.add(currentType.getLibelle() + "#" + currentType.getCode());
        }
        infoBean.set("LISTE_TYPE_EMPLOI", listeTypesEmploi);
        infoBean.set("CODE_RATTACHEMENT_AUTRES", offresstagesemplois.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", offresstagesemplois.getLibelleCodeRattachementAutres());
        infoBean.set("SECTEURS_ACTIVITES", offresstagesemplois.getSecteursActivites());
        infoBean.set("LIBELLE_SECTEURS_ACTIVITES", LabelUtils.getLibelleWithSection("65", offresstagesemplois.getLibelleSecteursActivites(), offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean())));
        infoBean.set("LISTE_SECTEURS_ACTIVITESS", LabelUtils.getLabelForComboForSection("65", offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean())));
        infoBean.set("DESCRIPTIF", offresstagesemplois.getDescriptif());
        infoBean.set("DUREE", offresstagesemplois.getDuree());
        infoBean.set("LIBELLE_DUREE", LabelUtils.getLibelleWithSection("32", offresstagesemplois.getLibelleDuree(), offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean())));
        infoBean.set("LISTE_DUREES", LabelUtils.getLabelForComboForSection("32", offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean())));
        infoBean.set("COMMENTAIRE_DUREE", offresstagesemplois.getCommentaireDuree());
        infoBean.set("DEBUT", offresstagesemplois.getDebut());
        infoBean.set("LIBELLE_DEBUT", LabelUtils.getLibelleWithSection("33", offresstagesemplois.getLibelleDebut(), offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean())));
        infoBean.set("LISTE_DEBUTS", LabelUtils.getLabelForComboForSection("33", offresstagesemplois.getLangue(), getCurrentSection(offresstagesemplois.getPersistanceBean())));
        infoBean.set("COMMENTAIRE_DEBUT", offresstagesemplois.getCommentaireDebut());
        infoBean.set("LIEU", offresstagesemplois.getLieu());
        infoBean.set("PAYS", offresstagesemplois.getPays());
        infoBean.set("VILLES", offresstagesemplois.getVilles());
        infoBean.set("REMUNERATION_STAGE", offresstagesemplois.getRemunerationStage());
        infoBean.set("COMMENTAIRE_REMUNERATION_STAGE", offresstagesemplois.getCommentaireRemunerationStage());
        infoBean.set("DATE_LIMITE_OFFRE", offresstagesemplois.getDateLimiteOffre());
        infoBean.set("URL_DETAIL_OFFRE", offresstagesemplois.getUrlDetailOffre());
        infoBean.set("FILIERE", offresstagesemplois.getFiliere());
        infoBean.set("LIBELLE_FILIERE", offresstagesemplois.getLibelleFiliere());
        infoBean.set("NIVEAU_ETUDE", offresstagesemplois.getNiveauEtude());
        infoBean.set("FORMATIONS_REQUISES", offresstagesemplois.getFormationsRequises());
        infoBean.set("COMPETENCES_REQUISES", offresstagesemplois.getCompetencesRequises());
        infoBean.set("RAISON_SOCIALE", offresstagesemplois.getRaisonSociale());
        infoBean.set("ACTIVITE", offresstagesemplois.getActivite());
        infoBean.set("RESPONSABLE", offresstagesemplois.getResponsable());
        infoBean.set("COORDONNEES", offresstagesemplois.getCoordonnees());
        infoBean.set("CODE_POSTAL", offresstagesemplois.getCodePostal());
        infoBean.set("VILLE", offresstagesemplois.getVille());
        infoBean.set("TELEPHONE", offresstagesemplois.getTelephone());
        infoBean.set("TELECOPIE", offresstagesemplois.getTelecopie());
        infoBean.set("EMAIL_RESPONSABLE", offresstagesemplois.getEmailResponsable());
        infoBean.set("SITE_WEB", offresstagesemplois.getSiteWeb());
        infoBean.set("ACTIVATION_REPONSE", offresstagesemplois.getActivationReponse());
        infoBean.set("COMPLEMENTS", offresstagesemplois.getComplements());
        infoBean.set("THEME", offresstagesemplois.getTheme());
        infoBean.set("DATE_SUPPRESSION", offresstagesemplois.getDateSuppression());
        infoBean.set("CODE_RATTACHEMENT", offresstagesemplois.getCodeRattachement());
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(offresstagesemplois.getCodeRattachement(), offresstagesemplois.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", offresstagesemplois.getTitre());
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        ControleurUniv.preparerPRINCIPAL(infoBean, offresstagesemplois, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        offresstagesemplois.setTitre((String) infoBean.get("TITRE"));
        offresstagesemplois.setTypeOffre((String) infoBean.get("TYPE_OFFRE"));
        offresstagesemplois.setTypeStage((String) infoBean.get("TYPE_STAGE"));
        offresstagesemplois.setTypeEmploi((String) infoBean.get("TYPE_EMPLOI"));
        offresstagesemplois.setTypeJob((String) infoBean.get("TYPE_JOB"));
        offresstagesemplois.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENT_AUTRES"));
        offresstagesemplois.setSecteursActivites((String) infoBean.get("SECTEURS_ACTIVITES"));
        offresstagesemplois.setDescriptif((String) infoBean.get("DESCRIPTIF"));
        offresstagesemplois.setDuree((String) infoBean.get("DUREE"));
        offresstagesemplois.setCommentaireDuree((String) infoBean.get("COMMENTAIRE_DUREE"));
        offresstagesemplois.setDebut((String) infoBean.get("DEBUT"));
        offresstagesemplois.setCommentaireDebut((String) infoBean.get("COMMENTAIRE_DEBUT"));
        offresstagesemplois.setLieu((String) infoBean.get("LIEU"));
        offresstagesemplois.setPays((String) infoBean.get("PAYS"));
        offresstagesemplois.setVilles((String) infoBean.get("VILLES"));
        offresstagesemplois.setRemunerationStage((String) infoBean.get("REMUNERATION_STAGE"));
        offresstagesemplois.setCommentaireRemunerationStage((String) infoBean.get("COMMENTAIRE_REMUNERATION_STAGE"));
        offresstagesemplois.setUrlDetailOffre((String) infoBean.get("URL_DETAIL_OFFRE"));
        offresstagesemplois.setFiliere((String) infoBean.get("FILIERE"));
        offresstagesemplois.setNiveauEtude((String) infoBean.get("NIVEAU_ETUDE"));
        offresstagesemplois.setFormationsRequises((String) infoBean.get("FORMATIONS_REQUISES"));
        offresstagesemplois.setCompetencesRequises((String) infoBean.get("COMPETENCES_REQUISES"));
        offresstagesemplois.setRaisonSociale((String) infoBean.get("RAISON_SOCIALE"));
        offresstagesemplois.setActivite((String) infoBean.get("ACTIVITE"));
        offresstagesemplois.setResponsable((String) infoBean.get("RESPONSABLE"));
        offresstagesemplois.setCoordonnees((String) infoBean.get("COORDONNEES"));
        offresstagesemplois.setCodePostal((String) infoBean.get("CODE_POSTAL"));
        offresstagesemplois.setVille((String) infoBean.get("VILLE"));
        offresstagesemplois.setTelephone((String) infoBean.get("TELEPHONE"));
        offresstagesemplois.setTelecopie((String) infoBean.get("TELECOPIE"));
        offresstagesemplois.setEmailResponsable((String) infoBean.get("EMAIL_RESPONSABLE"));
        offresstagesemplois.setSiteWeb((String) infoBean.get("SITE_WEB"));
        offresstagesemplois.setActivationReponse((String) infoBean.get("ACTIVATION_REPONSE"));
        offresstagesemplois.setComplements((String) infoBean.get("COMPLEMENTS"));
        offresstagesemplois.setTheme((String) infoBean.get("THEME"));
        offresstagesemplois.setDateSuppression((Date) infoBean.get("DATE_SUPPRESSION"));
        offresstagesemplois.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        offresstagesemplois.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        Date dateLimite = (Date) infoBean.get("DATE_LIMITE_OFFRE");
        final GregorianCalendar cal = new GregorianCalendar();
        boolean dateLimiteSaisie = false;
        //duree par défaut en jours
        int nbJour = -1;
        // si pas de date limite de l'offre précisée
        if (!Formateur.estSaisie(dateLimite)) {
            final String nbJoursDefaut = OffrestageemploiUtil.getProperty("offresstagesemplois.duree_" + offresstagesemplois.getTypeOffre());
            if (nbJoursDefaut != null) {
                try {
                    nbJour = Integer.parseInt(nbJoursDefaut);
                } catch (final Exception e) {
                }
            }
            if (nbJour != -1) {
                final Date today = new Date(System.currentTimeMillis());
                cal.setTime(today);
                cal.add(Calendar.DAY_OF_MONTH, nbJour);
                dateLimite = new Date(cal.getTimeInMillis());
            }
        } else {
            dateLimiteSaisie = true;
            //si la date de mise en ligne est saisie
            if (Formateur.estSaisie((Date) infoBean.get("DATE_MISE_EN_LIGNE"))) {
                cal.setTime((Date) infoBean.get("DATE_MISE_EN_LIGNE"));
                final GregorianCalendar calLimite = new GregorianCalendar();
                calLimite.setTime(dateLimite);
                //si la date de limite de l'offre est avant la date de mise en ligne
                if (calLimite.before(cal)) {
                    throw new ErreurApplicative("La date limite de l'offre doit être supérieure à la date de mise en ligne");
                }
            }
        }
        // mise à jour automatique : valeurs possibles : -1 pour ne rien faire, 0 pour archiver, 1 pour supprimer
        String actionFin = OffrestageemploiUtil.getProperty("offresstagesemplois.fin_" + offresstagesemplois.getTypeOffre());
        if (actionFin == null) {
            actionFin = "0";
        }
        //si l'utilisateur a les droits de suppression
        if (infoBean.get("GRS_AUTORISATION_SUPPRESSION").equals("1")) {
            //si l'action de fin est l'archivage et que la date limite de réponse a été saisie, on positionne la date d'archivage
            if (actionFin.equals("0") && (!Formateur.estSaisie((Date) infoBean.get("DATE_ARCHIVAGE")) || dateLimiteSaisie)) {
                infoBean.set("DATE_ARCHIVAGE", dateLimite);
                infoBean.set("GRS_SAUVEGARDE_DATE_ARCHIVAGE", dateLimite);
            }
            //si l'action de fin est la suppression et que la date limite de réponse a été saisie, on positionne la date de suppression
            else if (actionFin.equals("1") && (!Formateur.estSaisie((Date) infoBean.get("DATE_SUPPRESSION")) || dateLimiteSaisie)) {
                infoBean.set("DATE_SUPPRESSION", dateLimite);
                infoBean.set("GRS_SAUVEGARDE_DATE_SUPPRESSION", dateLimite);
            }
        }
        //on renseigne la date limite avec la valeur saisie ou celle configurée par défaut
        offresstagesemplois.setDateLimiteOffre(dateLimite);
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            offresstagesemplois.init();
        } else {
            offresstagesemplois.setIdOffresstagesemplois(new Long(infoBean.getString("ID_OFFRESSTAGESEMPLOIS")));
            offresstagesemplois.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteCodeSiVide(infoBean);
            alimenteDonneesCreation(offresstagesemplois, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, offresstagesemplois, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }

    private void traiterReponse() throws Exception {
        infoBean.set("ETAT_OFFRE", "EN_ENVOI");
        if (infoBean.getString("ID_OFFRESSTAGESEMPLOIS") != null) {
            offresstagesemplois.setIdOffresstagesemplois(new Long(infoBean.getString("ID_OFFRESSTAGESEMPLOIS")));
            offresstagesemplois.retrieve();
        }
        final String urlFicheOffre = urlProvider.provideAbsoluteUrl(offresstagesemplois.getPersistanceBean());
        final String nom = infoBean.getString("NOM");
        final String prenom = infoBean.getString("PRENOM");
        final String adresse = infoBean.getString("ADRESSE");
        final String tel = infoBean.getString("TELEPHONE");
        final String mail = infoBean.getString("EMAIL");
        if (!MailUtil.verifMail(mail)) {
            throw new ErreurApplicative(MessageHelper.getCoreMessage("ST_ERR_DEMANDE_MDP_EMAIL"));
        }
        // récupération des fichiers uploadés
        final String workDefaultDir = WebAppUtil.getWorkDefaultPath();
        // CV
        final UploadedFile fileCv = (UploadedFile) infoBean.get("CV_FILE");
        final File fCv = fileCv.getTemporaryFile();
        final String serverFileNameCV = fileCv.getContentFilename();
        final String pathFichierDestCv = workDefaultDir + File.separator + serverFileNameCV;
        // controle des extensions
        controlerFichiersJoints(serverFileNameCV);
        controlerFichiersJoints(pathFichierDestCv);
        // duplication du fichier source vers un fichier portant le nom spécifié côté client
        FileUtils.moveFile(fCv, new File(pathFichierDestCv));
        // motivations
        final UploadedFile fileMotivation = (UploadedFile) infoBean.get("MOTIVATIONS_FILE");
        final File fMotiv = fileMotivation.getTemporaryFile();
        final String serverFileNameMotiv = fileMotivation.getContentFilename();
        final String pathFichierDestMotiv = workDefaultDir + File.separator + serverFileNameMotiv;
        // duplication du fichier source vers un fichier portant le nom spécifié côté client
        FileUtils.moveFile(fMotiv, new File(pathFichierDestMotiv));
        final String[] filesAEnvoyer = {pathFichierDestCv, pathFichierDestMotiv};
        try {
            final JSBMailbox mailBox = new JSBMailbox(false);
            // envoi du mail de réponse au responsable de l'annonce
            String corpsMail = OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_MAIL_CORPS");
            corpsMail = StringUtils.replace(corpsMail, "[nom]", nom);
            corpsMail = StringUtils.replace(corpsMail, "[prenom]", prenom);
            corpsMail = StringUtils.replace(corpsMail, "[mail]", mail);
            corpsMail = StringUtils.replace(corpsMail, "[adresse]", adresse);
            corpsMail = StringUtils.replace(corpsMail, "[telephone]", tel);
            corpsMail = StringUtils.replace(corpsMail, "[code]", offresstagesemplois.getCode());
            corpsMail = StringUtils.replace(corpsMail, "[titre]", offresstagesemplois.getTitre());
            corpsMail = StringUtils.replace(corpsMail, "[url_offre]", urlFicheOffre);
            String sujet = OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_MAIL_SUJET");
            sujet = StringUtils.replace(sujet, "[code]", offresstagesemplois.getCode());
            sujet = StringUtils.replace(sujet, "[titre]", offresstagesemplois.getTitre());
            final String destinataire = StringUtils.defaultString(OffrestageemploiUtil.getProperty("offresstagesemplois.mail_to"));
            if (corpsMail.length() > 0) {
                final String mailResponsable = offresstagesemplois.getEmailResponsable();
                if (mailResponsable == null || "".equals(mailResponsable)) {
                    final String messageErreur = "L'email du destinataire de la réponse à l'offre d'emploi n'est pas renseigné";
                    LOG.error(messageErreur);
                    throw new ErreurTechnique(0, messageErreur);
                }
                mailBox.sendTxtMsgWithAttachedFiles(mail, mailResponsable, sujet, corpsMail, filesAEnvoyer);
                Thread.sleep(1000); //tempo de 1s
            } else {
                throw new ErreurApplicative("Aucun mail de réponse configuré.");
            }
            // envoi de l'accuse de réception à l'utilisateur
            String sujetAccuse = OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_ACCUSE_RECEPTION_SUJET");
            sujetAccuse = StringUtils.replace(sujetAccuse, "[code]", offresstagesemplois.getCode());
            sujetAccuse = StringUtils.replace(sujetAccuse, "[titre]", offresstagesemplois.getTitre());
            String corpsMailAccuse = OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_ACCUSE_RECEPTION_CORPS");
            corpsMailAccuse = StringUtils.replace(corpsMailAccuse, "[nom]", nom);
            corpsMailAccuse = StringUtils.replace(corpsMailAccuse, "[prenom]", prenom);
            corpsMailAccuse = StringUtils.replace(corpsMailAccuse, "[date]", new SimpleDateFormat("dd/MM/yyyy").format(new java.util.Date()));
            corpsMailAccuse = StringUtils.replace(corpsMailAccuse, "[code]", offresstagesemplois.getCode());
            corpsMailAccuse = StringUtils.replace(corpsMailAccuse, "[titre]", offresstagesemplois.getTitre());
            corpsMailAccuse = StringUtils.replace(corpsMailAccuse, "[url_offre]", urlFicheOffre);
            if (corpsMailAccuse.length() > 0) {
                mailBox.sendSystemMsg(mail, sujetAccuse, corpsMailAccuse);
                Thread.sleep(1000); //tempo de 1s
            } else {
                throw new ErreurApplicative("Aucun mail d'accusé de réception configuré.");
            }
            // envoi d'une copie en spécifique
            if (destinataire.length() > 0) {
                String corpsMailCC = OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_CC_CORPS");
                corpsMailCC = StringUtils.replace(corpsMailCC, "[nom]", nom);
                corpsMailCC = StringUtils.replace(corpsMailCC, "[prenom]", prenom);
                corpsMailCC = StringUtils.replace(corpsMailCC, "[date]", new SimpleDateFormat("dd/MM/yyyy").format(new java.util.Date()));
                corpsMailCC = StringUtils.replace(corpsMailCC, "[code]", offresstagesemplois.getCode());
                corpsMailCC = StringUtils.replace(corpsMailCC, "[titre]", offresstagesemplois.getTitre());
                corpsMailCC = StringUtils.replace(corpsMailCC, "[url_offre]", urlFicheOffre);
                String sujetCC = OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_CC_SUJET");
                sujetCC = StringUtils.replace(sujetCC, "[code]", offresstagesemplois.getCode());
                sujetCC = StringUtils.replace(sujetCC, "[titre]", offresstagesemplois.getTitre());
                if (corpsMailCC.length() > 0) {
                    mailBox.sendSystemMsg(destinataire, sujetCC, corpsMailCC);
                }
            }
        } catch (final AddressException e) {
            LOG.error("adresse mail incorrecte", e);
            throw new ErreurTechnique(0, "La fonctionnalité est momentanément indisponible, merci de réessayer plus tard.");
        } catch (final NoSuchProviderException e) {
            LOG.error("pas de smtp", e);
            throw new ErreurTechnique(0, "La fonctionnalité est momentanément indisponible, merci de réessayer plus tard.");
        } catch (final MessagingException e) {
            LOG.error("impossible d'envoyer le mail", e);
            throw new ErreurTechnique(0, "La fonctionnalité est momentanément indisponible, merci de réessayer plus tard.");
        } catch (final ErreurApplicative e) {
            LOG.error("erreur metier...", e);
            throw new ErreurTechnique(0, "La fonctionnalité est momentanément indisponible merci de réessayer plus tard.");
        } finally {
            infoBean.setEcranLogique(ECRAN_CONFIRMATION);
            infoBean.set("ID_OFFRESSTAGESEMPLOIS", offresstagesemplois.getIdFiche().toString());
            infoBean.set("CODE", offresstagesemplois.getCode());
            infoBean.set("TITRE", offresstagesemplois.getTitre());
            //on supprime les fichiers temporaires, et on conserve les fichiers originaux
            fileCv.deleteTemporaryFile();
            fileMotivation.deleteTemporaryFile();
        }
        String messConfirmation = OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_CONFIRMATION");
        if (messConfirmation.length() == 0) {
            messConfirmation = "Votre demande a bien &eacute;t&eacute; prise en compte, une r&eacute;ponse vous sera adress&eacute;e ult&eacute;rieurement.";
        }
        infoBean.set("MESSAGE_CONFIRMATION", messConfirmation);
        infoBean.set("URL_REDIRECT", urlFicheOffre);
    }

    private void preparerReponse() throws Exception {
        if (StringUtils.isNotEmpty(infoBean.get("CODE", String.class))) {
            final String langue = StringUtils.defaultString(infoBean.get("LANGUE", String.class), LangueUtil.getLangueLocale(LangueUtil.getDefaultLocale()));
            int nbResultats = offresstagesemplois.selectCodeLangueEtat(infoBean.get("CODE", String.class), langue, EtatFiche.EN_LIGNE.getEtat());
            if (nbResultats > 0) {
                offresstagesemplois.nextItem();
                infoBean.set("ID_OFFRESSTAGESEMPLOIS", offresstagesemplois.getIdFiche().toString());
                infoBean.set("TITRE", offresstagesemplois.getTitre());
                infoBean.set("CODE", offresstagesemplois.getCode());
                infoBean.set("NOM", "");
                infoBean.set("PRENOM", "");
                infoBean.set("ADRESSE", "");
                infoBean.set("TELEPHONE", "");
                infoBean.set("EMAIL", "");
                infoBean.set("CV", "");
                infoBean.set("MOTIVATIONS", "");
                infoBean.set("MODE_PAGE", "REPONSE");
                //placer l'état dans le composant d'infoBean
                infoBean.setTitreEcran(OffrestageemploiUtil.getMessage("ST_OFFRESSTAGESEMPLOIS_REPONSE_DEBUT_TITRE") + " " + offresstagesemplois.getTitre());
                infoBean.setEcranLogique(ECRAN_REPONSE);
            } else {
                final String messErr = "La réponse à l'offre de référence '" + EscapeString.escapeHtml(infoBean.getString("CODE")) + "' est indisponible.";
                throw new ErreurApplicative(messErr);
            }
        } else {
            final String messErr = "Aucune offre associée à la réponse.";
            throw new ErreurApplicative(messErr);
        }
    }

    /**
     * Méthode contrôlant si l'extension du fichier joint dont le nom est transmis en paramètre est d'un format accepté
     *
     * @param pathFile
     *            chemin vers le fichier joint dont on souhaite tester l'extension
     */
    private void controlerFichiersJoints(final String pathFile) throws ErreurApplicative {
        final String extensionsAcceptees = OffrestageemploiUtil.getProperty("offresstagesemplois.extensions_fichiers_joints");
        if (extensionsAcceptees != null && !"".equals(extensionsAcceptees)) {
            //ttes les extensions sont acceptées
            final String[] extensions = extensionsAcceptees.split(";");
            boolean extensionTrouvee = false;
            int i = 0;
            while (!extensionTrouvee && i < extensions.length) {
                extensionTrouvee = pathFile.toLowerCase().endsWith(extensions[i].toLowerCase());
                i++;
            }
            if (!extensionTrouvee) {
                final String messErr = "Seuls les fichiers de type '" + extensionsAcceptees + "' sont acceptés.";
                throw new ErreurApplicative(messErr);
            }
        }
    }
}
