/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.core.config.MessageHelper;
import com.ksup.objetspartages.bean.ParcoursBean;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.ksup.objetspartages.om.Parcours;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.util.LabelUtils;

/**
 * Processus de gestion des fiches Parcours.
 */
public class SaisieParcours extends SaisieFiche {

    private static final Logger LOG = LoggerFactory.getLogger(SaisieParcours.class);

    protected Parcours parcours = null;

    /**
     * Constructeur.
     * @param infoBean the infobean
     */
    public SaisieParcours(InfoBean infoBean) {
        super(infoBean);
    }

    /**
     * Point d'entrée du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0005");
        parcours = new Parcours();
        parcours.init();
        traiterActionParDefaut(parcours);
        return (etat == FIN);
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(getLocale()));
        infoBean.set("LISTE_NIVEAU_RECRUTEMENTS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), getLocale()));
        infoBean.set("LISTE_NIVEAU_DU_DIPLOMES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), getLocale()));
        infoBean.set("LISTE_LIBELLE_PARCOURS_SEMESTRES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.PARCOURS_SEMESTRE.getValeur(), getLocale()));
        infoBean.set("LISTE_ANNEES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.ANNEE.getValeur(), getLocale()));
        infoBean.set("LISTE_SECTEURS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.SECTEUR.getValeur(), getLocale()));
        try {
            infoBean.set("LIBELLE_UEUPS", com.ksup.objetspartages.om.Ueup.getLibelleAffichable((String) infoBean.get("UEUPS"), StringUtils.EMPTY));
            infoBean.set("LIBELLE_PARCOURS_AMONT", com.ksup.objetspartages.om.Parcours.getLibelleAffichable((String) infoBean.get("PARCOURS_AMONT"), StringUtils.EMPTY));
            infoBean.set("LIBELLE_PARCOURS_AVAL", com.ksup.objetspartages.om.Parcours.getLibelleAffichable((String) infoBean.get("PARCOURS_AVAL"), StringUtils.EMPTY));
        } catch (Exception e) {
            LOG.error("Erreur de récupération de libellés", e);
        }
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(getLocale()));
        infoBean.set("LISTE_NIVEAU_RECRUTEMENTS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), getLocale()));
        infoBean.set("LISTE_NIVEAU_DU_DIPLOMES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), getLocale()));
        infoBean.set("LISTE_LIBELLE_PARCOURS_SEMESTRES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.PARCOURS_SEMESTRE.getValeur(), getLocale()));
        infoBean.set("LISTE_ANNEES", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.ANNEE.getValeur(), getLocale()));
        infoBean.set("LISTE_SECTEURS", LabelUtils.getLabelComboWithRestrictionInformation(OffreFormationTypeLibelle.SECTEUR.getValeur(), getLocale()));
        try {
            infoBean.set("LIBELLE_UEUPS", com.ksup.objetspartages.om.Ueup.getLibelleAffichable((String) infoBean.get("UEUPS"), StringUtils.EMPTY));
            infoBean.set("LIBELLE_PARCOURS_AMONT", com.ksup.objetspartages.om.Parcours.getLibelleAffichable((String) infoBean.get("PARCOURS_AMONT"), StringUtils.EMPTY));
            infoBean.set("LIBELLE_PARCOURS_AVAL", com.ksup.objetspartages.om.Parcours.getLibelleAffichable((String) infoBean.get("PARCOURS_AVAL"), StringUtils.EMPTY));
        } catch (Exception e) {
            LOG.error("Erreur de récupération de libellés", e);
        }
    }

    /**
     * Affichage de l'écran de saisie d'une fiche Parcours.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_PARCOURS", parcours.getIdParcours().toString());
        infoBean.set("CODE_SI_SCOLARITE", parcours.getCodeSiScolarite());
        infoBean.set("DUREE_SEMESTRE", parcours.getDureeSemestre());
        infoBean.set("INTITULE_COURT", parcours.getIntituleCourt());
        infoBean.set("INTITULE_LONG", parcours.getIntituleLong());
        infoBean.set("CODE_RATTACHEMENT_AUTRES", parcours.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT_AUTRES", parcours.getLibelleCodeRattachementAutres());
        infoBean.set("DIPLOMANT_ETAPE", parcours.getDiplomantEtape());
        infoBean.set("INTITULE_PARCOURS_DIPLOMANT", parcours.getIntituleParcoursDiplomant());
        infoBean.set("NIVEAU_RECRUTEMENT", parcours.getNiveauRecrutement());
        infoBean.set("LIBELLE_NIVEAU_RECRUTEMENT", LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), parcours.getLibelleNiveauRecrutement(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean())));
        infoBean.set("LISTE_NIVEAU_RECRUTEMENTS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.NIVEAU_RECRUTEMENT.getValeur(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean())));
        infoBean.set("NIVEAU_DU_DIPLOME", parcours.getNiveauDuDiplome());
        infoBean.set("LISTE_NIVEAU_DU_DIPLOMES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean())));
        infoBean.set("TEMPS_PLEIN", parcours.getTempsPlein());
        infoBean.set("COMPATIBLE_ACTIVITE_PRO", parcours.getCompatibleActivitePro());
        infoBean.set("TEMPS_AMENAGE", parcours.getTempsAmenage());
        infoBean.set("ALTERNANCE", parcours.getAlternance());
        infoBean.set("HORS_CONTRAT_ALTERNANCE", parcours.getHorsContratAlternance());
        infoBean.set("CONTRAT_APPRENTISSAGE", parcours.getContratApprentissage());
        infoBean.set("CONTRAT_PRO", parcours.getContratPro());
        infoBean.set("LIBELLE_PARCOURS_SEMESTRE", LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.PARCOURS_SEMESTRE.getValeur(), parcours.getLibelleParcoursSemestre(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean())));
        infoBean.set("LISTE_LIBELLE_PARCOURS_SEMESTRES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.PARCOURS_SEMESTRE.getValeur(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean())));
        infoBean.set("ANNEE", parcours.getAnnee());
        infoBean.set("LISTE_ANNEES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.ANNEE.getValeur(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean())));
        infoBean.set("UEUPS", parcours.getUeups());
        infoBean.set("LIBELLE_UEUPS", parcours.getLibelleUeups());
        infoBean.set("PRESENTATION", parcours.getPresentation());
        infoBean.set("SECTEUR", parcours.getSecteur());
        infoBean.set("LISTE_SECTEURS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.SECTEUR.getValeur(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean())));
        infoBean.set("PARCOURS_AMONT", parcours.getParcoursAmont());
        infoBean.set("LIBELLE_PARCOURS_AMONT", prepareLibelleParcoursAmont());
        infoBean.set("PARCOURS_AVAL", parcours.getParcoursAval());
        infoBean.set("LIBELLE_PARCOURS_AVAL", prepareLibelleParcoursAval());
        infoBean.set("CODE_RATTACHEMENT", parcours.getCodeRattachement());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", parcours.getInfosStructure() != null ? parcours.getInfosStructure().getLibelleAffichable() : "");
        if(OffreFormationUtil.isConstructionParcoursPresent()) {
            infoBean.set("ACCES_SOUS_CONDITION", parcours.getAccesSousCondition());
            infoBean.set("PARCOURS_FICTIF", parcours.getParcoursFictif());
            infoBean.set("LISTE_DUREE_PARCOURS", getListeDureesSemestresPossibles());
            infoBean.set("LISTE_DEPART_PARCOURS", getListeSemestresDepartPossibles());
            infoBean.set("DUREE_PARCOURS", parcours.getDureeSemestre());
            infoBean.set("DEPART_PARCOURS", parcours.getNumeroPremierSemestre());
        }
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("LIBELLE_AFFICHABLE", parcours.getLibelleAffichable());
            if (infoBean.get("SOUS_ONGLET") == null) {
                // DSI sur les onglets, on affiche l'onglet suivant si on a pas les droits
                if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_CARACTERISTIQUE)) {
                    infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
                } else if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_AFFICHAGE)) {
                    infoBean.set("SOUS_ONGLET", "AFFICHAGE"); // onglet par defaut
                } else if (StringUtils.isBlank(infoBean.getString("GRS_SAISIE_ENCADRES")) || StringUtils.equals(infoBean.getString("GRS_SAISIE_ENCADRES"), "0")) {
                    //Cas pas de droits sur les premiers onglets on affiche les encadrés si possibles.
                    infoBean.set("SOUS_ONGLET", "encadres");
                } else {
                    throw new ErreurApplicative(MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ONGLET_AUCUN_DROIT"));
                }
            }
        }
        infoBean.set("LISTE_ONGLET_PARCOURS", getListeOnglet());
        infoBean.set("PRINCIPAL", OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_CARACTERISTIQUE));
        infoBean.set("AFFICHAGE", OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_AFFICHAGE));
        ControleurUniv.preparerPRINCIPAL(infoBean, parcours, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        parcours.setCodeRattachementAutres(infoBean.getString("CODE_RATTACHEMENT_AUTRES"));
        parcours.setCodeSiScolarite(infoBean.getString("CODE_SI_SCOLARITE"));
        parcours.setLibelleParcoursSemestre(infoBean.getString("LIBELLE_PARCOURS_SEMESTRE"));
        parcours.setIntituleCourt(infoBean.getString("INTITULE_COURT"));
        parcours.setIntituleLong(infoBean.getString("INTITULE_LONG"));
        parcours.setDiplomantEtape(infoBean.getString("DIPLOMANT_ETAPE"));
        parcours.setIntituleParcoursDiplomant(infoBean.getString("INTITULE_PARCOURS_DIPLOMANT"));
        parcours.setNiveauRecrutement(infoBean.getString("NIVEAU_RECRUTEMENT"));
        parcours.setNiveauDuDiplome(infoBean.getString("NIVEAU_DU_DIPLOME"));
        parcours.setTempsPlein(infoBean.getString("TEMPS_PLEIN"));
        parcours.setCompatibleActivitePro(infoBean.getString("COMPATIBLE_ACTIVITE_PRO"));
        parcours.setTempsAmenage(infoBean.getString("TEMPS_AMENAGE"));
        parcours.setAlternance(infoBean.getString("ALTERNANCE"));
        parcours.setHorsContratAlternance(infoBean.getString("HORS_CONTRAT_ALTERNANCE"));
        parcours.setContratApprentissage(infoBean.getString("CONTRAT_APPRENTISSAGE"));
        parcours.setContratPro(infoBean.getString("CONTRAT_PRO"));
        parcours.setLibelleParcoursSemestre(infoBean.getString("LIBELLE_PARCOURS_SEMESTRE"));
        parcours.setAnnee(infoBean.getString("ANNEE"));
        parcours.setUeups(infoBean.getString("UEUPS"));
        parcours.setPresentation(infoBean.getString("PRESENTATION"));
        parcours.setSecteur(infoBean.getString("SECTEUR"));
        parcours.setParcoursAmont(infoBean.getString("PARCOURS_AMONT"));
        parcours.setParcoursAval(infoBean.getString("PARCOURS_AVAL"));
        parcours.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        parcours.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        if(OffreFormationUtil.isConstructionParcoursPresent()) {
            parcours.setParcoursFictif(infoBean.getString("PARCOURS_FICTIF"));
            parcours.setAccesSousCondition(infoBean.getString("ACCES_SOUS_CONDITION"));
            try {
                Integer dureeParcours;
                // Dans le cas où l'onglet n'est pas affiché on récupère un int
                if(infoBean.get("DUREE_PARCOURS").getClass().isAssignableFrom(String.class)){
                    dureeParcours = infoBean.getInteger("DUREE_PARCOURS");
                }else{
                    dureeParcours = infoBean.getInt("DUREE_PARCOURS");
                }
                parcours.setDureeSemestre(dureeParcours);
            } catch (NumberFormatException nfe) {
                LOG.debug("valeurs incorrecte pour le champ durée", nfe);
                throw new ErreurApplicative(OffreFormationUtil.getMessage("PARCOURS.BO.ERREUR.DUREE"));
            }
            try {
                Integer departParcours;
                // Dans le cas où l'onglet n'est pas affiché on récupère un int
                if(infoBean.get("DEPART_PARCOURS").getClass().isAssignableFrom(String.class)){
                    departParcours = infoBean.getInteger("DEPART_PARCOURS");
                }else{
                    departParcours = infoBean.getInt("DEPART_PARCOURS");
                }
                parcours.setNumeroPremierSemestre(departParcours);
            } catch (NumberFormatException nfe) {
                LOG.debug("valeurs incorrecte pour le champ départ", nfe);
                throw new ErreurApplicative(OffreFormationUtil.getMessage("PARCOURS.BO.ERREUR.DEPART"));
            }
        }
    }

    /**
     * Traite l'écran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            parcours.init();
        } else {
            parcours.setIdParcours(Long.valueOf(infoBean.getString("ID_PARCOURS")));
            parcours.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            //si le code doit etre controle car saisi manuellement ou bien
            //parce que l'objet implemente l'interface Structure ou Annuaire : passer la valeur true
            alimenteDonneesCreation(parcours, false);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, parcours, this);
        if (StringUtils.isEmpty(ecranLogique)) {
            etat = FIN;
        }
    }

    private String prepareLibelleParcoursAval() throws Exception {
        final Collection<String> labels = new ArrayList<>();
        final String[] codes = StringUtils.split(parcours.getParcoursAval(), ";");
        for (String currentCode : codes) {
            final String[] split = currentCode.split(",");
            final StringBuilder currentLabel = new StringBuilder(Parcours.getLibelleAffichable(currentCode, parcours.getLangue()));
            if (split.length == 3 && !"0000".equals(split[2])) {
                currentLabel.append(String.format(" [%s]", LabelUtils.getLibelleWithSection("2034", split[2], parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean()))));
            }
            labels.add(currentLabel.toString());
        }
        return StringUtils.join(labels, ";");
    }

    private String prepareLibelleParcoursAmont() throws Exception {
        final Collection<String> labels = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(parcours.getPersistenceBean().getParcoursAmontBeans())) {
            for (Pair<ParcoursBean, String> currentParcours : parcours.getPersistenceBean().getParcoursAmontBeans()) {
                final StringBuilder currentLabel = new StringBuilder(Parcours.getLibelleAffichable(currentParcours.getLeft().getCode(), parcours.getLangue()));
                if (!"0000".equals(currentParcours.getRight())) {
                    currentLabel.append(String.format(" [%s]", LabelUtils.getLibelleWithSection("2034", currentParcours.getRight(), parcours.getLangue(), getCurrentSection(parcours.getPersistanceBean()))));
                }
                labels.add(currentLabel.toString());
            }
        }
        return StringUtils.join(labels, ";");
    }

    private Map<String, String> getListeDureesSemestresPossibles() {
        Map<String, String> resultats = new HashMap<>();
        resultats.put("1", OffreFormationUtil.getMessage("LIBELLE_DUREE_SEMESTRE1"));
        resultats.put("2", OffreFormationUtil.getMessage("LIBELLE_DUREE_SEMESTRE2"));
        resultats.put("3", OffreFormationUtil.getMessage("LIBELLE_DUREE_SEMESTRE3"));
        resultats.put("4", OffreFormationUtil.getMessage("LIBELLE_DUREE_SEMESTRE4"));
        resultats.put("5", OffreFormationUtil.getMessage("LIBELLE_DUREE_SEMESTRE5"));
        resultats.put("6", OffreFormationUtil.getMessage("LIBELLE_DUREE_SEMESTRE6"));
        return resultats;
    }

    private Map<String, String> getListeSemestresDepartPossibles() {
        Map<String, String> resultats = new HashMap<>();
        resultats.put("1", OffreFormationUtil.getMessage("LIBELLE_SEMESTRE1_DEPART"));
        resultats.put("2", OffreFormationUtil.getMessage("LIBELLE_SEMESTRE2_DEPART"));
        resultats.put("3", OffreFormationUtil.getMessage("LIBELLE_SEMESTRE3_DEPART"));
        resultats.put("4", OffreFormationUtil.getMessage("LIBELLE_SEMESTRE4_DEPART"));
        resultats.put("5", OffreFormationUtil.getMessage("LIBELLE_SEMESTRE5_DEPART"));
        resultats.put("6", OffreFormationUtil.getMessage("LIBELLE_SEMESTRE6_DEPART"));
        return resultats;
    }

    private Map<String, String> getListeOnglet() {
        final Map<String, String> listeOnglet = new HashMap<>();
        // Si droit "onglet caracteristiques
        if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_CARACTERISTIQUE)) {
            listeOnglet.put("PRINCIPAL", MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ONGLET_CARACTERISTIQUES"));
        }
        // Si droit "onglet Affichage (affichage + call + action + encadre)
        if (OffreFormationUtil.getAfficherOnglet(OffreFormationUtil.CODE_DROIT_ONGLET_AFFICHAGE)) {
            listeOnglet.put("AFFICHAGE", MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ONGLET_AFFICHAGE"));
        }
        return listeOnglet;
    }
}




