/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.ksup.objetspartages.om.Ueup;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.processus.ControleurUniv;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;

/**
 * Processus de gestion des ueups.
 */
public class SaisieUeup extends SaisieFiche {

    protected Ueup ueup = null;

    /**
     * Constructeur.
     *
     * @param infoBean
     */
    public SaisieUeup(final InfoBean infoBean) {
        super(infoBean);
    }

    /**
     * Point d'entree du processus.
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0003");
        ueup = new Ueup();
        ueup.init();
        traiterActionParDefaut(ueup);
        infoBean.set("NOM_ONGLET", "ueup");
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_SEMESTRES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.SEMESTRE.getValeur(), this.getLocale()));
        infoBean.set("LISTE_TYPES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_ELEMENT.getValeur(), this.getLocale()));
        infoBean.set("LISTE_NATURES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.NATURE.getValeur(), this.getLocale()));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), this.getLocale()));
        infoBean.set("LISTE_VILLES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.LIEUX.getValeur(), this.getLocale()));
        infoBean.set("LISTE_DOMAINESS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), this.getLocale()));
        infoBean.set("LISTE_TYPE_DIPLOMES", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), this.getLocale()));
        infoBean.set("LISTE_LANGUES_ENSEIGNEMENTS", LabelUtils.getLabelCombo(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), this.getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(this.getLocale()));
        infoBean.set("LISTE_SEMESTRES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.SEMESTRE.getValeur(), this.getLocale(), sectionCode));
        infoBean.set("LISTE_TYPES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.TYPE_ELEMENT.getValeur(), this.getLocale(), sectionCode));
        infoBean.set("LISTE_NATURES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.NATURE.getValeur(), this.getLocale(), sectionCode));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), this.getLocale(), sectionCode));
        infoBean.set("LISTE_VILLES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.LIEUX.getValeur(), this.getLocale(), sectionCode));
        infoBean.set("LISTE_DOMAINESS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), this.getLocale(), sectionCode));
        infoBean.set("LISTE_TYPE_DIPLOMES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), this.getLocale(), sectionCode));
        infoBean.set("LISTE_LANGUES_ENSEIGNEMENTS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), this.getLocale(), sectionCode));
    }

    /**
     * Affichage de l'ecran de saisie d'un(e) ueup.
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_UEUP", ueup.getIdUeup().toString());
        infoBean.set("CODE_SI_SCOLARITE", ueup.getCodeSiScolarite());
        infoBean.set("LIBELLE_COURT", ueup.getLibelleCourt());
        infoBean.set("LIBELLE_LONG", ueup.getLibelleLong());
        infoBean.set("CREDIT_ECTS", ueup.getCreditEcts());
        infoBean.set("SEMESTRE", ueup.getSemestre());
        infoBean.set("LISTE_SEMESTRES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.SEMESTRE.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("TYPE", ueup.getType());
        infoBean.set("LISTE_TYPES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.TYPE_ELEMENT.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("NATURE", ueup.getNature());
        infoBean.set("LISTE_NATURES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.NATURE.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("NB_MAXI_ACHOIX", ueup.getNbMaxiAchoix());
        infoBean.set("NB_MINI_ACHOIX", ueup.getNbMiniAchoix());
        infoBean.set("DISCIPLINE", ueup.getDisciplines());
        infoBean.set("LIBELLE_DISCIPLINE", LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), ueup.getDisciplines(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("LISTE_DISCIPLINES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.DISCIPLINE.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("CODE_RATTACHEMENTS_AUTRES", ueup.getCodeRattachementAutres());
        infoBean.set("LIBELLE_CODE_RATTACHEMENTS_AUTRES", ueup.getLibelleCodeRattachementsAutres());
        infoBean.set("VILLE", ueup.getVille());
        infoBean.set("LIBELLE_VILLE", LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.LIEUX.getValeur(), ueup.getVille(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("LISTE_VILLES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.LIEUX.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("A_DISTANCE", ueup.getaDistance());
        infoBean.set("MIXTE_DISTANCE", ueup.getMixteDistance());
        infoBean.set("CODE_RESPONSABLE", ueup.getCodeResponsables());
        infoBean.set("LIBELLE_CODE_RESPONSABLE", ueup.getLibelleCodeResponsable());
        infoBean.set("COMPLEMENTS_RESPONSABLE", ueup.getComplementsResponsable());
        infoBean.set("COORDONNEES", ueup.getCoordonnees());
        infoBean.set("VOLUME_HORAIRE_TOTAL", ueup.getVolumeHoraireTotal());
        infoBean.set("VOLUME_HORAIRE_CM", ueup.getVolumeHoraireCm());
        infoBean.set("VOLUME_HORAIRE_TD", ueup.getVolumeHoraireTd());
        infoBean.set("VOLUME_HORAIRE_TP", ueup.getVolumeHoraireTp());
        infoBean.set("VOLUME_HORAIRE_PROJET", ueup.getVolumeHoraireProjet());
        infoBean.set("VOLUME_HORAIRE_STAGE", ueup.getVolumeHoraireStage());
        infoBean.set("VOLUME_HORAIRE_DS", ueup.getVolumeHoraireDs());
        infoBean.set("VOLUME_HORAIRE_FORMATION_DISTANCE", ueup.getVolumeHoraireFormationDistance());
        infoBean.set("VOLUME_HORAIRE_PRESENTIEL", ueup.getVolumeHorairePresentiel());
        infoBean.set("PREREQUIS", ueup.getPrerequis());
        infoBean.set("CODE_UEUPS_REQUISES", ueup.getCodeUeupsRequises());
        infoBean.set("LIBELLE_CODE_UEUPS_REQUISES", ueup.getLibelleCodeUeupsRequises());
        infoBean.set("OBJECTIFS", ueup.getObjectifs());
        infoBean.set("CALENDRIER", ueup.getCalendrier());
        infoBean.set("CONTENU", ueup.getContenu());
        infoBean.set("CODE_UEUPFILS", ueup.getUeupfils());
        infoBean.set("LIBELLE_CODE_UEUPFILS", ueup.getLibelleCodeUeupfils());
        infoBean.set("CODE_COURS", ueup.getCours());
        infoBean.set("LIBELLE_CODE_COURS", ueup.getLibelleCodeCours());
        infoBean.set("BIBLIOGRAPHIE", ueup.getBibliographie());
        infoBean.set("CONTROLE_CONNAISSANCES", ueup.getControleConnaissances());
        infoBean.set("DOMAINES", ueup.getDomaines());
        infoBean.set("LIBELLE_DOMAINES", LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), ueup.getDomaines(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("LISTE_DOMAINESS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.DOMAINE_LMD.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("DOMAINE_ERASMUS", ueup.getDomaineErasmus());
        infoBean.set("LISTE_DOMAINE_ERASMUSS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.DOMAINE_ERASMUS.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("LIBELLE_DOMAINE_ERASMUS", LabelUtils.getLibelleWithSection(OffreFormationTypeLibelle.DOMAINE_ERASMUS.getValeur(), ueup.getDomaineErasmus(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("GRADE", ueup.getGrade());
        infoBean.set("TYPE_DIPLOME", ueup.getTypeDiplome());
        infoBean.set("LISTE_TYPE_DIPLOMES", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.TYPE_DIPLOME.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("LANGUES_ENSEIGNEMENT", ueup.getLanguesEnseignement());
        infoBean.set("LIBELLE_LANGUES_ENSEIGNEMENT", ueup.getLibelleLanguesEnseignement());
        infoBean.set("LISTE_LANGUES_ENSEIGNEMENTS", LabelUtils.getLabelForComboForSection(OffreFormationTypeLibelle.LANGUES_ENSEIGNEMENT.getValeur(), ueup.getLangue(), getCurrentSection(ueup.getPersistanceBean())));
        infoBean.set("COMPLEMENTS", ueup.getComplements());
        infoBean.set("RATTACHEMENT_BANDEAU", ueup.getRattachementBandeau());
        infoBean.set("CODE_RATTACHEMENT", ueup.getCodeRattachement());
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(ueup.getCodeRattachement(), ueup.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", ueup.getLibelleLong());
        if (infoBean.get("SAISIE_FRONT") == null) {
            infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        }
        ControleurUniv.preparerPRINCIPAL(infoBean, ueup, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        ueup.setCodeSiScolarite((String) infoBean.get("CODE_SI_SCOLARITE"));
        ueup.setLibelleCourt((String) infoBean.get("LIBELLE_COURT"));
        ueup.setLibelleLong((String) infoBean.get("LIBELLE_LONG"));
        ueup.setCreditEcts((String) infoBean.get("CREDIT_ECTS"));
        ueup.setSemestre((String) infoBean.get("SEMESTRE"));
        ueup.setType((String) infoBean.get("TYPE"));
        ueup.setNature((String) infoBean.get("NATURE"));
        ueup.setNbMaxiAchoix((String) infoBean.get("NB_MAXI_ACHOIX"));
        ueup.setNbMiniAchoix((String) infoBean.get("NB_MINI_ACHOIX"));
        ueup.setDisciplines((String) infoBean.get("DISCIPLINE"));
        ueup.setCodeRattachementAutres((String) infoBean.get("CODE_RATTACHEMENTS_AUTRES"));
        ueup.setVille((String) infoBean.get("VILLE"));
        ueup.setaDistance((String) infoBean.get("A_DISTANCE"));
        ueup.setMixteDistance((String) infoBean.get("MIXTE_DISTANCE"));
        ueup.setCodeResponsables((String) infoBean.get("CODE_RESPONSABLE"));
        ueup.setComplementsResponsable((String) infoBean.get("COMPLEMENTS_RESPONSABLE"));
        ueup.setCoordonnees((String) infoBean.get("COORDONNEES"));
        ueup.setVolumeHoraireTotal((String) infoBean.get("VOLUME_HORAIRE_TOTAL"));
        ueup.setVolumeHoraireCm((String) infoBean.get("VOLUME_HORAIRE_CM"));
        ueup.setVolumeHoraireTd((String) infoBean.get("VOLUME_HORAIRE_TD"));
        ueup.setVolumeHoraireTp((String) infoBean.get("VOLUME_HORAIRE_TP"));
        ueup.setVolumeHoraireProjet((String) infoBean.get("VOLUME_HORAIRE_PROJET"));
        ueup.setVolumeHoraireStage((String) infoBean.get("VOLUME_HORAIRE_STAGE"));
        ueup.setVolumeHoraireDs((String) infoBean.get("VOLUME_HORAIRE_DS"));
        ueup.setVolumeHoraireFormationDistance((String) infoBean.get("VOLUME_HORAIRE_FORMATION_DISTANCE"));
        ueup.setVolumeHorairePresentiel((String) infoBean.get("VOLUME_HORAIRE_PRESENTIEL"));
        ueup.setPrerequis((String) infoBean.get("PREREQUIS"));
        ueup.setCodeUeupsRequises((String) infoBean.get("CODE_UEUPS_REQUISES"));
        ueup.setObjectifs((String) infoBean.get("OBJECTIFS"));
        ueup.setCalendrier((String) infoBean.get("CALENDRIER"));
        ueup.setContenu((String) infoBean.get("CONTENU"));
        ueup.setUeupfils((String) infoBean.get("CODE_UEUPFILS"));
        ueup.setCours((String) infoBean.get("CODE_COURS"));
        ueup.setBibliographie((String) infoBean.get("BIBLIOGRAPHIE"));
        ueup.setControleConnaissances((String) infoBean.get("CONTROLE_CONNAISSANCES"));
        ueup.setDomaines((String) infoBean.get("DOMAINES"));
        ueup.setDomaineErasmus((String) infoBean.get("DOMAINE_ERASMUS"));
        ueup.setGrade((String) infoBean.get("GRADE"));
        ueup.setTypeDiplome((String) infoBean.get("TYPE_DIPLOME"));
        ueup.setLanguesEnseignement((String) infoBean.get("LANGUES_ENSEIGNEMENT"));
        ueup.setComplements((String) infoBean.get("COMPLEMENTS"));
        ueup.setRattachementBandeau((String) infoBean.get("RATTACHEMENT_BANDEAU"));
        ueup.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        ueup.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            ueup.init();
        } else {
            ueup.setIdUeup(Long.valueOf(infoBean.getString("ID_UEUP")));
            ueup.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteDonneesCreation(ueup, true);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, ueup, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }
}
