/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.processus;

import java.util.StringTokenizer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.processus.SaisieFiche;
import com.univ.objetspartages.util.LabelUtils;

/**
 * processus saisie Ueup
 */
public class TraitementParcours extends SaisieFiche {

    private static final Logger LOG = LoggerFactory.getLogger(TraitementParcours.class);

    public TraitementParcours(final InfoBean ciu) {
        super(ciu);
    }

    @Override
    protected void preparerLISTE() throws Exception {}

    @Override
    protected void preparerPRINCIPAL() throws Exception {
        String code = "", langue = "";
        //Modification d'un parcours amont ou aval
        if (infoBean.get("CODE_PARCOURS_TITRE") != null) {
            final String codeComplet = (String) infoBean.get("CODE_PARCOURS_TITRE");
            final StringTokenizer stCode = new StringTokenizer(codeComplet, ",");
            int token = 0;
            String titre = "";
            while (stCode.hasMoreTokens()) {
                switch (token) {
                    case 0:
                        code = stCode.nextToken();
                        break;
                    case 1:
                        langue = stCode.nextToken();
                        break;
                    case 2:
                        titre = stCode.nextToken();
                        break;
                    default:
                        stCode.nextToken();
                        break;
                }
                token++;
            }
            infoBean.set("PARCOURS", titre);
        } else {
            //ajout d'un parcours amont ou aval, on vient de l'écran de recherche d'un parcours
            //!\\ Contournement produit. Ticket N° FORMATION-207 , on passe par le code_ueup qui est écrit en dur dans LienPopupServlet du produit au lieu de passer normalement par lien le code_parcours
            code = (String) infoBean.get("CODE_UEUP");
            langue = code.substring(code.indexOf("=") + 1);
            code = code.substring(0, code.indexOf(","));
        }
        infoBean.set("CODE_PARCOURS", code);
        infoBean.set("LANGUE_PARCOURS", langue);
        infoBean.set("LIBELLE_PARCOURS", com.ksup.objetspartages.om.Parcours.getLibelleAffichable(code, langue));
        infoBean.set("LISTE_ACCES_SOUS_CONDITION", LabelUtils.getLabelCombo("2034", LangueUtil.getDefaultLocale()));
        ecranLogique = ECRAN_PRINCIPAL;
    }

    @Override
    protected void preparerRECHERCHE() {}

    @Override
    protected void prepareRestrictedSearch(final String sectionCode) {
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {}

        @Override
    public boolean traiterAction() throws Exception {
        etat = EN_COURS;
        final Object o = getGp().getSessionUtilisateur().getInfos().get("AUTORISATIONS");
        if (o == null) {
            infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
            infoBean.setEcranLogique("LOGIN");
        } else {
            try {
                ecranLogique = infoBean.getEcranLogique();
                action = infoBean.getActionUtilisateur();
                etat = EN_COURS;
                /* Entrée par lien hyper-texte */
                if (ecranLogique == null) {
                    if (action.equals("AJOUTER")) {
                        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
                        preparerPRINCIPAL();
                    } else if (action.equals("MODIFIER")) {
                        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_MODIF);
                        preparerPRINCIPAL();
                    }
                } else {
                    if (ecranLogique.indexOf(ECRAN_PRINCIPAL) == 0) {
                        traiterPRINCIPAL();
                    } else if (ecranLogique.equals(ECRAN_LISTE)) {
                        traiterLISTE();
                    }
                }
                //placer l'état dans le composant d'infoBean
                infoBean.setEcranLogique(ecranLogique);
            } catch (final Exception e) {
                LOG.error("erreur lors du traitement du processus", e);
                infoBean.addMessageErreur(e.toString());
            }
        }
        infoBean.set("NOM_ONGLET", "formation");
        // On continue si on n'est pas à la FIN !!!
        return etat == FIN;
    }

    protected void traiterLISTE() throws Exception {
        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
        preparerPRINCIPAL();
    }

    @Override
    protected void traiterPRINCIPAL() throws Exception {
        if (action.equals(InfoBean.ACTION_ANNULER)) {
            ecranLogique = "FIN_TOOLBOX";
        }
    }

    /**
     * Traitement associé à l'écran de saisie des critères
     */
    @Override
    protected void traiterRECHERCHE() throws Exception {}

}
