/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.reference;

import java.util.ArrayList;
import java.util.Arrays;

import com.kosmos.usinesite.reference.DelimitationPattern;
import com.kosmos.usinesite.reference.impl.DefaultModuleReference;
import com.kportal.cms.objetspartages.Objetpartage;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.Parcours;
import com.ksup.objetspartages.om.Ueup;
import com.univ.objetspartages.om.ReferentielObjets;

/**
 * Mise à jour des champs : - LISTE_UEUP sur PARCOURS - CODE_UEUPS_REQUISES sur FORMATION - CODE_UEUPS_REQUISES sur UEUP - CODE_UEUPFILS sur UEUP
 */
public class UeupReference extends DefaultModuleReference {

    @Override
    public void init() {
        Objetpartage ueup = ReferentielObjets.getObjetByCode(ReferentielObjets.getCodeObjetParClasse(Ueup.class.getName()));
        if (ueup != null) {
            modules = new ArrayList<>();
            modules.add(ueup);
            // "code_ueups_requises":"CODE" ou "code_ueups_requises":"CODE;... ou "code_ueups_requises":"...;CODE;... ou "code_ueups_requises":"...;CODE"
            addModulesBySearchStringsForCode("\"code_ueups_requises\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.SEMICOLON_DELIMITER, Arrays.asList(new String[] {ueup.getId()}));
            // "code_ueup_fils":"CODE" ou "code_ueup_fils":"CODE;... ou "code_ueup_fils":"...;CODE;... ou "code_ueup_fils":"...;CODE"
            addModulesBySearchStringsForCode("\"code_ueupfils\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.SEMICOLON_DELIMITER, Arrays.asList(new String[] {ueup.getId()}));
            Objetpartage formation = ReferentielObjets.getObjetByCode(ReferentielObjets.getCodeObjetParClasse(Formation.class.getName()));
            if (formation != null) {
                // "code_ueups_requises":"CODE" ou "code_ueups_requises":"CODE;... ou "code_ueups_requises":"...;CODE;... ou "code_ueups_requises":"...;CODE"
                addModulesBySearchStringsForCode("\"code_ueups_requises\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.SEMICOLON_DELIMITER, Arrays.asList(new String[] {formation.getId()}));
            }
            Objetpartage parcours = ReferentielObjets.getObjetByCode(ReferentielObjets.getCodeObjetParClasse(Parcours.class.getName()));
            if (parcours != null) {
                // "liste_ueup":"CODE,0,S1,O,1" ou "liste_ueup":"CODE,0,S1,O,1;... ou "liste_ueup":"...;CODE,0,S1,O,1;... ou "liste_ueup":"...;CODE,0,S1,O,1"
                addModulesBySearchStringsForCode("\"semestre1\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.COMMA_DELIMITER, Arrays.asList(new String[] {parcours.getId()}));
                addModulesBySearchStringsForCode("\"semestre2\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.COMMA_DELIMITER, Arrays.asList(new String[] {parcours.getId()}));
                addModulesBySearchStringsForCode("\"semestre3\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.COMMA_DELIMITER, Arrays.asList(new String[] {parcours.getId()}));
                addModulesBySearchStringsForCode("\"semestre4\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.COMMA_DELIMITER, Arrays.asList(new String[] {parcours.getId()}));
                addModulesBySearchStringsForCode("\"semestre5\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.COMMA_DELIMITER, Arrays.asList(new String[] {parcours.getId()}));
                addModulesBySearchStringsForCode("\"semestre6\":\"" + DelimitationPattern.QUOTE_DELIMITER + "\"", ";?" + DelimitationPattern.COMMA_DELIMITER, Arrays.asList(new String[] {parcours.getId()}));
            }
        }
    }
}
