/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: etudiantetalumni
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.CodeLibelle;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.etudiantetalumni.util.EtudiantetalumniUtil;
import com.kosmos.service.impl.ServiceFiche;
import com.ksup.objetspartages.bean.AnnuaireetudiantsBean;
import com.ksup.objetspartages.dao.impl.AnnuaireEtudiantsDAO;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.utils.FicheUnivMgr;

/**
 * Created on 28/09/17.
 */
public class ServiceAnnuaireEtudiants extends ServiceFiche<AnnuaireetudiantsBean, AnnuaireEtudiantsDAO, FicheDefaultViewModel> {

    public String getLibelleAffichable(AnnuaireetudiantsBean bean) {
        return EtudiantetalumniUtil.getLibelleAffichableEtudiant(bean.getNom(), bean.getPrenom(), getLibelleCivilite(bean), true);
    }

    public String getLibelleSansFallback(AnnuaireetudiantsBean bean) {
        return EtudiantetalumniUtil.getLibelleAffichableEtudiant(bean.getNom(), bean.getPrenom(), getLibelleCivilite(bean), false);
    }

    /**
     * Renvoie le libelle pour l'attribut civilite.
     */
    public String getLibelleCivilite(AnnuaireetudiantsBean bean) {
        return CodeLibelle.lireLibelle(EtudiantetalumniUtil.ID_EXTENSION, "civilite", LangueUtil.getLocale(Integer.parseInt(bean.getLangue())), bean.getCivilite());
    }

    @Override
    public String getReferences(AnnuaireetudiantsBean bean){
        String references;
        references = FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement());
        references += FicheUnivMgr.getReferenceParPhoto(bean.getPhoto()) + StringUtils.defaultString(bean.getContact());
        references += StringUtils.defaultString(bean.getCv());
        references += StringUtils.defaultString(bean.getAdresseSitePerso());
        references += StringUtils.defaultString(bean.getComplements());
        if(StringUtils.isNotEmpty(bean.getCodeFormation())) {
            for (String codeFormation : bean.getCodeFormation().split(";")){
                references += FicheUnivMgr.getReferenceParJointure("formation", codeFormation.substring(2, codeFormation.length()));
            }
        }
        return references;
    }
}
