/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.CodeLibelle;
import com.jsbsoft.jtf.core.LangueUtil;
import com.kosmos.annuairesup.util.AnnuaireSupUtil;
import com.ksup.objetspartages.bean.AnnuaireksupBean;
import com.ksup.objetspartages.dao.impl.AnnuaireksupDAO;
import com.univ.annuaire.service.AbstractServiceAnnuaire;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.utils.FicheUnivMgr;

public class ServiceAnnuaireksup extends AbstractServiceAnnuaire<AnnuaireksupBean, AnnuaireksupDAO, FicheDefaultViewModel> {

    /**
     * @inheritDoc
     */
    @Override
    public String getLibelleAffichable(AnnuaireksupBean bean) {
        String res = getLibelleSansFallback(bean);
        if (res.equals(StringUtils.EMPTY)) {
            res = "-";
        }
        return res;
    }

    /**
     * Calcul du libellé affichable
     * @param bean
     * @return
     */
    public String getLibelleSansFallback(AnnuaireksupBean bean) {
        final StringBuilder res = new StringBuilder();
        if (StringUtils.isNotBlank(bean.getCivilite()) && !bean.getCivilite().equals("0000")) {
            res.append(getLibelleCivilite(bean)).append(" ");
        }
        if (StringUtils.isNotBlank(bean.getPrenom())) {
            res.append(bean.getPrenom()).append(" ");
        }
        if (StringUtils.isNotBlank(bean.getNom())) {
            res.append(bean.getNom().toUpperCase());
        }
        return res.toString();
    }

    /**
     * Renvoie le libelle pour l'attribut civilite.
     */
    public String getLibelleCivilite(AnnuaireksupBean bean) {
        return CodeLibelle.lireLibelle(AnnuaireSupUtil.ID_EXTENSION, "civilite", LangueUtil.getLocale(Integer.parseInt(bean.getLangue())), bean.getCivilite());
    }

    @Override
    public String getReferences(AnnuaireksupBean bean){
        String references = FicheUnivMgr.getReferenceParPhoto(bean.getPhoto());
        references += StringUtils.defaultString(bean.getAdresseSitePerso());
        references += StringUtils.defaultString(bean.getCv());
        references += FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement());
        references += StringUtils.defaultString(bean.getComplements());
        return references;
    }

    @Override
    public List<AnnuaireksupBean> getByStructure(final String codeStructure) {
        return dao.getByStructure(codeStructure);
    }
}
