/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Caching;

import com.kosmos.service.impl.ServiceFiche;
import com.ksup.objetspartages.bean.CoursBean;
import com.ksup.objetspartages.dao.impl.CoursDAO;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.utils.FicheUnivMgr;

/**
 * Created by alexandre.baillif on 15/02/17.
 */
public class ServiceCours extends ServiceFiche<CoursBean, CoursDAO, FicheDefaultViewModel> {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceCours.class);

    @Caching(evict = {@CacheEvict(value = "ServiceOffreFormation.findUeupByCours", key = "#coursBean.code + #coursBean.langue"), @CacheEvict(value = "ServiceOffreFormation.findFormationByCours", key = "#coursBean.code + #coursBean.langue")})
    public void updateCacheExterne(final CoursBean coursBean) {
        LOG.debug("Eviction du cache pour le cours [" + coursBean.getLibelleLong() + ", " + coursBean.getCode() + "]");
    }

    public void add(CoursBean coursBean) {
        dao.add(coursBean);
    }

    public void addWithForcedId(CoursBean coursBean) {
        dao.addWithForcedId(coursBean);
    }

    public void update(CoursBean coursBean) {
        dao.update(coursBean);
        updateCacheExterne(coursBean);
    }

    /**
     *
     * @param id
     *
     * @deprecated use {@link #delete(CoursBean)} instead
     */
    @Deprecated
    public void delete(Long id) {
        CoursBean coursBean = getById(id);
        delete(coursBean);
    }

    public void delete(CoursBean coursBean) {
        if (coursBean != null) {
            updateCacheExterne(coursBean);
            dao.delete(coursBean.getId());
        }
    }

    @Override
    public String getLibelleAffichable(CoursBean bean) {
        String res = bean.getLibelleLong();
        if (res.equals(StringUtils.EMPTY)) {
            res = "-";
        }
        return res;
    }

    @Override
    public String getReferences(CoursBean bean){
        String references = StringUtils.defaultString(bean.getComplementsResponsable());
        references += StringUtils.defaultString(bean.getCoordonnees());
        references += StringUtils.defaultString(bean.getObjectifs());
        references += StringUtils.defaultString(bean.getCalendrier());
        references += StringUtils.defaultString(bean.getContenu());
        references += StringUtils.defaultString(bean.getBibliographie());
        references += StringUtils.defaultString(bean.getControleConnaissances());
        references += StringUtils.defaultString(bean.getComplements());
        references += FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement());
        return references;
    }
}
