/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Caching;

import com.kosmos.service.impl.ServiceFiche;
import com.kportal.core.config.Langue;
import com.ksup.objetspartages.bean.FormationBean;
import com.ksup.objetspartages.dao.impl.FormationDAO;
import com.ksup.objetspartages.dao.impl.FormationJointuresDAO;
import com.ksup.objetspartages.om.Formation;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.utils.FicheUnivMgr;

public class ServiceFormation extends ServiceFiche<FormationBean, FormationDAO, FicheDefaultViewModel> {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceFormation.class);

    protected FormationJointuresDAO formationJointuresDAO;

    public void setFormationJointuresDAO(FormationJointuresDAO formationJointuresDAO) {
        this.formationJointuresDAO = formationJointuresDAO;
    }

    public void add(FormationBean formationBean) {
        dao.add(formationBean);
        saveDonneesJointures(formationBean);
        updateCacheLies();
    }

    public void addWithForcedId(FormationBean formationBean) {
        dao.addWithForcedId(formationBean);
        saveDonneesJointures(formationBean);
        updateCacheLies();
    }

    public void update(FormationBean formationBean) {
        dao.update(formationBean);
        saveDonneesJointures(formationBean);
        updateCacheLies();
    }

    /**
     * @param id
     * @deprecated use {@link #delete(FormationBean)} instead
     */
    @Deprecated
    public void delete(Long id) {
        FormationBean formationBean = getById(id);
        delete(formationBean);
    }

    public void delete(FormationBean formationBean) {
        dao.delete(formationBean.getId());
        formationJointuresDAO.deleteFormationToFormationsLiens(formationBean);
        formationJointuresDAO.deleteFormationToParcoursLiens(formationBean);
        updateCacheLies();
    }

    protected void saveDonneesJointures(FormationBean formationBean) {
        formationJointuresDAO.saveFormationsLiees(formationBean);
        formationJointuresDAO.saveFormationsPoursuite(formationBean);
        formationJointuresDAO.saveParcoursPoursuite(formationBean);
        formationJointuresDAO.saveParcoursDiplomant(formationBean);
    }

    /**
     * Met à jour les caches qui contient des formationbean mais qui n'ont pas de key utilisable à cet endroit. Cela fait que quand une formation est modifiée, les caches qui la contiennent sont mis à jour
     */
    @Caching(evict = {@CacheEvict(value = "ServiceOffreFormation.findFormationByCours", allEntries = true), @CacheEvict(value = "ServiceOffreFormation.findFormationByUeup", allEntries = true)})
    public void updateCacheLies() {
        LOG.debug("Eviction du cache ServiceFormation.findFormationByCours");
    }

    /**
     * Retourne la liste de toutes les formations pour une langue et un état donnés.
     * @param langue la langue
     * @param  etatFiche l'état
     * @return une liste de {@link FormationBean}
     */
    public List<FormationBean> getAll(Langue langue, EtatFiche etatFiche) {
        return dao.selectCodeLangueEtat(StringUtils.EMPTY, String.valueOf(langue.getIndice()), etatFiche.getEtat());
    }

    public boolean isChooseMyProgramActivated(final String id) {
        String construireMaFormation = dao.chooseMyProgram(id);
        return "1".equals(construireMaFormation);
    }

    @Override
    public String getLibelleAffichable(FormationBean bean){
        String res = bean.getIntituleLong();
        if (res.equals(StringUtils.EMPTY)) {
            res = "-";
        }
        return res;
    }

    @Override
    public String getReferences(FormationBean bean){
        String references = StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachementAutres()));
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Formation", bean.getCodeFormationMere()));
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Formation", bean.getFormationsLiees()));
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Formation", bean.getFormationPoursuite()));
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Parcours", bean.getParcoursPoursuite()));
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Parcours", bean.getParcoursFiche()));
        references += StringUtils.defaultString(bean.getResume());
        references += StringUtils.defaultString(bean.getDescription());
        references += StringUtils.defaultString(bean.getSpecificites());
        references += StringUtils.defaultString(bean.getLieuxComplement());
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("Annuaireksup", bean.getResponsablesFiche()));
        references += StringUtils.defaultString(bean.getResponsablesComplement());
        references += StringUtils.defaultString(bean.getLaboratoiresPartenaires());
        references += StringUtils.defaultString(bean.getEtablissementsPartenaires());
        references += StringUtils.defaultString(bean.getEntreprisesPartenaires());
        references += StringUtils.defaultString(bean.getConventionEtablissements());
        references += StringUtils.defaultString(bean.getFormationsRequises());
        references += StringUtils.defaultString(bean.getPublicCible());
        references += StringUtils.defaultString(bean.getModalitesCandidatures());
        references += StringUtils.defaultString(bean.getModalitesCandidaturesSpecifiques());
        references += StringUtils.defaultString(bean.getConditionsAdmission());
        references += StringUtils.defaultString(bean.getProgramme());
        references += StringUtils.defaultString(bean.getContenuFormation());
        references += StringUtils.defaultString(bean.getRythmePremiereAnnee());
        references += StringUtils.defaultString(bean.getEquipePedagogique());
        references += StringUtils.defaultString(bean.getTempsPleinDescription());
        references += StringUtils.defaultString(bean.getCompatibleProfessionnelleDescription());
        references += StringUtils.defaultString(bean.getTempsAmenageDescription());
        references += StringUtils.defaultString(bean.getAlternanceDescription());
        references += StringUtils.defaultString(bean.getPresentielDescription());
        references += StringUtils.defaultString(bean.getPartiellementDistanceDescription());
        references += StringUtils.defaultString(bean.getTotalementDistanceDescription());
        references += StringUtils.defaultString(bean.getStagesDescription());
        references += StringUtils.defaultString(bean.getSejoursEtrangerDescription());
        references += StringUtils.defaultString(bean.getProjetsTutoresComplement());
        references += StringUtils.defaultString(bean.getModalitesEvaluation());
        references += StringUtils.defaultString(bean.getFicheRncp());
        references += StringUtils.defaultString(bean.getCompetencesAttestees());
        references += StringUtils.defaultString(bean.getPoursuiteEtudes());
        references += StringUtils.defaultString(bean.getParcoursPoursuite());
        references += StringUtils.defaultString(bean.getFormationPoursuite());
        references += StringUtils.defaultString(bean.getDebouches());
        references += StringUtils.defaultString(bean.getInsertionProfessionnelle());
        references += StringUtils.defaultString(bean.getTarifComplement());
        references += StringUtils.defaultString(bean.getModalitesInscription());
        references += StringUtils.defaultString(bean.getRenseignements());
        references += StringUtils.defaultString(FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement()));
        return references;
    }
}


