/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: rechercheetlabo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.service;

import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.service.impl.ServiceFiche;
import com.kosmos.service.impl.ServiceManager;
import com.ksup.objetspartages.bean.LaboratoireBean;
import com.ksup.objetspartages.dao.impl.LaboratoireDAO;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.structure.service.AbstractServiceStructure;
import com.univ.utils.FicheUnivMgr;

/**
 * Created on 28/09/17.
 */
public class ServiceLaboratoire extends AbstractServiceStructure<LaboratoireBean, LaboratoireDAO, FicheDefaultViewModel> {

    @Override
    public String getLibelleAffichable(LaboratoireBean bean) {
        String res;
        res = bean.getLibelleLong();
        if (res.equals(StringUtils.EMPTY)) {
            res = bean.getLibelleCourt();
        }
        if (res.equals(StringUtils.EMPTY)) {
            res = "-";
        }
        return res;
    }

    @Override
    public String getReferences(LaboratoireBean bean){
        String references = StringUtils.defaultString(bean.getOnglets());
        references += FicheUnivMgr.getReferenceParJointure("annuaire", bean.getCodeResponsable());
        references += StringUtils.defaultString(bean.getComposition());
        references += StringUtils.defaultString(bean.getAxesRecherche());
        references += StringUtils.defaultString(bean.getFormations());
        references += StringUtils.defaultString(bean.getPublications());
        references += StringUtils.defaultString(bean.getEquipements());
        references += StringUtils.defaultString(bean.getCooperationsInternationales());
        references += StringUtils.defaultString(bean.getCollaborationsUniversitaires());
        references += StringUtils.defaultString(bean.getCooperationsIndustrielles());
        references += StringUtils.defaultString(bean.getChiffresCles());
        references += StringUtils.defaultString(bean.getComplementCoordonnees());
        references += StringUtils.defaultString(bean.getComplements());
        references += FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement());
        return references;
    }

    @Override
    public boolean isContentVisible(LaboratoireBean bean) {
        ServiceMetatag serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
        MetatagBean metatagBean = serviceMetatag.getByCodeAndIdFiche(ReferentielObjets.getCodeObjetParClasseBean(bean.getClass()), bean.getId());
        return !"0".equals(metatagBean.getMetaInTree());
    }

    @Override
    public List<LaboratoireBean> getByCodeRattachementAndState(String parentStructureCode, final EtatFiche etatFiche) {
        return dao.getByCodeRattachementAndState(parentStructureCode, etatFiche);
    }

    @Override
    protected Class<LaboratoireBean> getBeanClass() {
        return LaboratoireBean.class;
    }

    @Override
    public LaboratoireBean getByCodeAndState(final String code, final EtatFiche etatFiche) {
        return dao.getByCodeAndState(code, etatFiche);
    }

}
