/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.util;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.univ.objetspartages.bean.AbstractRestrictionFicheBean;

/**
 * Classe utilitaire pour transformer des chaines de concatenation produit (code,langue)  sous forme de bean et inversement
 *
 * Created by alexandre.baillif on 05/04/17.
 */
public final class FicheBeanUtils {

    private static Logger LOG = LoggerFactory.getLogger(FicheBeanUtils.class);

    public static String convertListBeanToStringCodesLangues(List<? extends AbstractRestrictionFicheBean> beans) {
        StringBuilder codesLangues = new StringBuilder();
        if(CollectionUtils.isNotEmpty(beans)) {
            for (AbstractRestrictionFicheBean myBean : beans) {
                if (StringUtils.isNoneBlank(myBean.getCode(), myBean.getLangue())) {
                    codesLangues.append(myBean.getCode()).append(",LANGUE=").append(myBean.getLangue()).append(";");
                }
            }
        }
        return codesLangues.toString();
    }

    public static <T extends AbstractRestrictionFicheBean> List<T> convertStringCodesLanguesToListBean(String codesLangues, Class<T> clazz) {
        List<T> beans = new ArrayList<>();
        String[] codesAndLangues = StringUtils.split(codesLangues, ";");
        if(codesLangues != null) {
            for (String codeLangue : codesAndLangues) {
                String[] codeAndLangue = StringUtils.split(codeLangue, ",");
                if (codeAndLangue != null) {
                    T bean;
                    try {
                        bean = clazz.newInstance();
                        bean.setCode(codeAndLangue[0]);
                        if (codeAndLangue.length > 1) {
                            //Ce test n'est normalement pas nécessaire mais bon au cas on reste conforme à certains endroits du produit
                            String langue = StringUtils.replace(codeAndLangue[1], "LANGUE=", StringUtils.EMPTY);
                            bean.setLangue(langue);
                        }
                        beans.add(bean);
                    } catch (InstantiationException | IllegalAccessException e) {
                        LOG.error(String.format("unable to instanciate a bean of type %s", clazz.getCanonicalName()), e);
                    }
                }
            }
        }
        return beans;
    }

    public static <T extends AbstractRestrictionFicheBean> String convertListOptionAndBeanToStringCodesLanguesOptions(final List<Pair<T, String>> beansWithOptions) {
        final StringBuilder codesLangues = new StringBuilder();
        if(CollectionUtils.isNotEmpty(beansWithOptions)) {
            for (final Pair<T, String> myBeanWithOption : beansWithOptions) {
                final String option = myBeanWithOption.getRight();
                final AbstractRestrictionFicheBean myBean = myBeanWithOption.getLeft();
                if (StringUtils.isNoneBlank(myBean.getCode(), myBean.getLangue(), option)) {
                    //Attention traite sans "LANGUE="
                    codesLangues.append(myBean.getCode()).append(",").append(myBean.getLangue()).append(",").append(option).append(";");
                }
            }
        }
        return codesLangues.toString();
    }

    public static <T extends AbstractRestrictionFicheBean> List<Pair<T, String>> convertStringCodesLanguesOptionsToListOptionAndBeans(String codesLanguesOptions, Class<T> clazz) {
        List<Pair<T, String>> result = new ArrayList<>();
        String[] codesAndLanguesAndOptions = StringUtils.split(codesLanguesOptions, ";");
        if(codesAndLanguesAndOptions != null) {
            for (String codeLangueOption : codesAndLanguesAndOptions) {
                String[] codeAndLangueAndOption = StringUtils.split(codeLangueOption, ",");
                if (codeAndLangueAndOption != null) {
                    T bean;
                    try {
                        bean = clazz.newInstance();
                        bean.setCode(codeAndLangueAndOption[0]);
                        if (codeAndLangueAndOption.length > 2 && StringUtils.isNotBlank(codeAndLangueAndOption[2])) {
                            //Ce test n'est normalement pas nécessaire mais bon au cas on reste conforme à certains endroits du produit
                            String langue = StringUtils.replace(codeAndLangueAndOption[1], "LANGUE=", StringUtils.EMPTY);
                            bean.setLangue(langue);
                            String options = codeAndLangueAndOption[2];
                            Pair<T, String> myPair = Pair.of((T) bean, options);
                            result.add(myPair);
                        }
                    } catch (InstantiationException | IllegalAccessException e) {
                        LOG.error("Erreur dans la création d'un ficheBean à partir du code", e);
                    }
                }
            }
        }
        return result;
    }
}
