/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.util;

import java.util.Hashtable;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.ksup.objetspartages.om.Formation;
import com.ksup.objetspartages.om.OffreFormationTypeLibelle;
import com.univ.objetspartages.util.LabelUtils;

public class FormationHelper {

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     * @deprecated utiliser {@link #getFicheFormation(String, String)}
     */
    @Deprecated
    public static Formation getFicheFormation(OMContext ctx, String code, String langue) throws Exception {
        Formation fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Formation();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, "0003");
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, "");
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, "", "0003");
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Formation getFicheFormation(String code, String langue) throws Exception {
        Formation fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Formation();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, "0003");
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, "");
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, "", "0003");
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    public static String getFormatedStage(Formation formation) {
        if (StringUtils.isBlank(formation.getStages())) {
            return StringUtils.EMPTY;
        } else if (!"1".equals(formation.getStages())) {
            return OffreFormationUtil.getMessage("FORMATION.FO.CARD.NON");
        } else {
            StringBuilder sb = new StringBuilder(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OUI"));
            if ("1".equals(formation.getStagesObligatoire())) {
                sb.append(", ").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OBLIGATOIRES"));
                if ("1".equals(formation.getStagesOptionnelMultiple()) || "1".equals(formation.getStagesOptionnelEtranger())) {
                    sb.append(" (");
                    if ("1".equals(formation.getStagesOptionnelMultiple())) {
                        sb.append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.MULTIPLES"));
                    }
                    if ("1".equals(formation.getStagesOptionnelMultiple()) || "1".equals(formation.getStagesOptionnelEtranger())) {
                        sb.append(", ");
                    }
                    if ("1".equals(formation.getStagesOptionnelEtranger())) {
                        sb.append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.A_L_ETRANGER"));
                    }
                    sb.append(")");
                }
            }
            if ("1".equals(formation.getStagesOptionnel())) {
                sb.append(", ").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OPTIONNELS"));
                if ("1".equals(formation.getStagesObligatoireMultiple()) || "1".equals(formation.getStagesObligatoireEtranger())) {
                    sb.append(" (");
                    if ("1".equals(formation.getStagesObligatoireMultiple())) {
                        sb.append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.MULTIPLES"));
                    }
                    if ("1".equals(formation.getStagesObligatoireMultiple()) || "1".equals(formation.getStagesObligatoireEtranger())) {
                        sb.append(", ");
                    }
                    if ("1".equals(formation.getStagesObligatoireEtranger())) {
                        sb.append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.A_L_ETRANGER"));
                    }
                    sb.append(")");
                }
            }
            return sb.toString();
        }
    }

    public static String getFormatedSejoursALEtranger(Formation formation) {
        if (StringUtils.isBlank(formation.getSejoursEtranger())) {
            return StringUtils.EMPTY;
        } else if (!"1".equals(formation.getSejoursEtranger())) {
            return OffreFormationUtil.getMessage("FORMATION.FO.CARD.NON");
        } else {
            StringBuilder sb = new StringBuilder(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OUI"));
            if ("1".equals(formation.getSejoursEtrangerObligatoire())) {
                sb.append(", ").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OBLIGATOIRES"));
                if ("1".equals(formation.getSejoursEtrangerObligatoireMultiples())) {
                    sb.append(" (").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.MULTIPLES")).append(")");
                }
            }
            if ("1".equals(formation.getSejoursEtrangerOptionnel())) {
                sb.append(", ").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OPTIONNELS"));
                if ("1".equals(formation.getSejoursEtrangerOptionnelMultiples())) {
                    sb.append(" (").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.MULTIPLES")).append(")");
                }
            }
            return sb.toString();
        }
    }

    public static String getFormatedProjetsTutores(Formation formation) {
        if (StringUtils.isBlank(formation.getProjetsTutores())) {
            return StringUtils.EMPTY;
        } else if (!"1".equals(formation.getProjetsTutores())) {
            return OffreFormationUtil.getMessage("FORMATION.FO.CARD.NON");
        } else {
            StringBuilder sb = new StringBuilder(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OUI"));
            if ("1".equals(formation.getProjetsTutoresObligatoire())) {
                sb.append(", ").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OBLIGATOIRES"));
                if ("1".equals(formation.getProjetsTutoresObligatoireMultiples())) {
                    sb.append(" (").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.MULTIPLES")).append(")");
                }
            }
            if ("1".equals(formation.getProjetsTutoresOptionnel())) {
                sb.append(", ").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.OPTIONNELS"));
                if ("1".equals(formation.getProjetsTutoresOptionnelMultiples())) {
                    sb.append(" (").append(OffreFormationUtil.getMessage("FORMATION.FO.CARD.MULTIPLES")).append(")");
                }
            }
            return sb.toString();
        }
    }
}
