/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ofin
 * Version: 2.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ksup.objetspartages.util;

import org.apache.commons.lang3.StringUtils;

import com.kosmos.controllers.SGController;
import com.kosmos.offreformation.util.OffreFormationUtil;
import com.kportal.core.config.PropertyHelper;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.PermissionBean;
import com.univ.utils.ContexteUtil;
import com.univ.utils.URLResolver;

public class FormationSaisieBOUtils {

    public static String getLienCreationParcours() {
        String url = StringUtils.EMPTY;
        final AutorisationBean autorisationsUser = ContexteUtil.getContexteUniv().getAutorisation();
        PermissionBean perm = new PermissionBean("FICHE/0005/C");
        if (autorisationsUser != null && autorisationsUser.possedePermission(perm)) {
            url = URLResolver.getAbsoluteUrl(PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?EXT=" + OffreFormationUtil.ID_EXTENSION + "&PROC=SAISIE_PARCOURS&ACTION=AJOUTER", ContexteUtil.getContexteUniv());
        }
        return url;
    }


    /**
     * formate les informations de stage sous la forme "Oui, obligatoires (multiples) et optionnel...."
     * @param stages
     * @param obligatoire
     * @param optionnel
     * @param obligatoireMultiple
     * @param obligatoireEtranger
     * @param optionnelMultiple
     * @param optionnelEtranger
     * @return les infos sur le stages
     */
    public static String getStagesInfos(String stages, String obligatoire, String optionnel, String obligatoireMultiple, String obligatoireEtranger, String optionnelMultiple, String optionnelEtranger) {
        StringBuilder stagesInfos = new StringBuilder();
        if (StringUtils.equals("1", stages)) {
            stagesInfos.append(OffreFormationUtil.getMessage("FORMATION.STAGE_OUI"));
            // Stage obligatoire
            if (StringUtils.equals("1", obligatoire)) {
                stagesInfos.append(", ").append(OffreFormationUtil.getMessage("FORMATION.STAGE_OBLIGATOIRE"));
                stagesInfos.append(getStagesInfosDetail(obligatoireMultiple, obligatoireEtranger));
            }
            // Stage optionnel
            if (StringUtils.equals("1", optionnel)) {
                stagesInfos.append(", ").append(OffreFormationUtil.getMessage("FORMATION.STAGE_OPTIONNEL"));
                stagesInfos.append(getStagesInfosDetail(optionnelMultiple, optionnelEtranger));
            }
        } else {
            stagesInfos.append(OffreFormationUtil.getMessage("FORMATION.STAGE_NON"));
        }
        return stagesInfos.toString();
    }

    private static String getStagesInfosDetail(String multiple, String etranger) {
        StringBuilder stagesInfosDetail = new StringBuilder();
        if (StringUtils.equals("1", multiple) || StringUtils.equals("1", etranger)) {
            stagesInfosDetail.append(" (");
            if (StringUtils.equals("1", multiple)) {
                stagesInfosDetail.append(OffreFormationUtil.getMessage("FORMATION.STAGE_MULTIPLES"));
            }
            if (StringUtils.equals("1", etranger)) {
                if (StringUtils.equals("1", multiple)) {
                    stagesInfosDetail.append(", ");
                }
                stagesInfosDetail.append(OffreFormationUtil.getMessage("FORMATION.STAGE_ETRANGER"));
            }
            stagesInfosDetail.append(")");
        }
        return stagesInfosDetail.toString();
    }

    /**
     * formate les informations de sejour a l'etranger sous la forme "Oui, obligatoires (multiples) et optionnel...."
     * @param etranger
     * @param obligatoire
     * @param optionnel
     * @param obligatoireMultiple
     * @param optionnelMultiple
     * @return les infos du stage sous forme d'une chaine de caractère
     */
    public static String getSejourEtrangerInfos(String etranger, String obligatoire, String optionnel, String obligatoireMultiple, String optionnelMultiple) {
        StringBuilder sejourEtranger = new StringBuilder();
        if (StringUtils.equals("1", etranger)) {
            sejourEtranger.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.OUI"));
            // Stage obligatoire
            if (StringUtils.equals("1", obligatoire)) {
                sejourEtranger.append(", ").append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.OBLIGATOIRE"));
                sejourEtranger.append(getSejourInfosDetail(obligatoireMultiple, ""));
            }
            // Stage optionnel
            if (StringUtils.equals("1", optionnel)) {
                sejourEtranger.append(", ").append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.OPTIONNEL"));
                sejourEtranger.append(getSejourInfosDetail("", optionnelMultiple));
            }
        } else {
            sejourEtranger.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.NON"));
        }
        return sejourEtranger.toString();
    }

    private static String getSejourInfosDetail(String obligatoireMultiple, String optionnelMultiple) {
        StringBuilder sejourInfosDetail = new StringBuilder();
        if (StringUtils.equals("1", optionnelMultiple) || StringUtils.equals("1", obligatoireMultiple)) {
            sejourInfosDetail.append(" (");
            if (StringUtils.equals("1", optionnelMultiple)) {
                sejourInfosDetail.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.MULTIPLES"));
            }
            if (StringUtils.equals("1", obligatoireMultiple)) {
                sejourInfosDetail.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.MULTIPLES"));
            }
            sejourInfosDetail.append(")");
        }
        return sejourInfosDetail.toString();
    }

    /**
     * formate les informations du projet tutore sous la forme "Oui, obligatoires (multiples) et optionnel...."
     * @param tutore
     * @param obligatoire
     * @param optionnel
     * @param obligatoireMultiple
     * @param optionnelMultiple
     * @return les informations du projet tutore sous forme d'une chaine de caractère
     */
    public static String getProjetTutoreInfos(String tutore, String obligatoire, String optionnel, String obligatoireMultiple, String optionnelMultiple) {
        StringBuilder projetTutore = new StringBuilder();
        if (StringUtils.equals("1", tutore)) {
            projetTutore.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.OUI"));
            // obligatoire
            if (StringUtils.equals("1", obligatoire)) {
                projetTutore.append(", ").append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.OBLIGATOIRE"));
                projetTutore.append(getProjetTutoreInfosDetail(obligatoireMultiple, ""));
            }
            // optionnel
            if (StringUtils.equals("1", optionnel)) {
                projetTutore.append(", ").append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.OPTIONNEL"));
                projetTutore.append(getProjetTutoreInfosDetail("", optionnelMultiple));
            }
        } else {
            projetTutore.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.NON"));
        }
        return projetTutore.toString();
    }

    private static String getProjetTutoreInfosDetail(String obligatoireMultiple, String optionnelMultiple) {
        StringBuilder projetTutoreInfosDetail = new StringBuilder();
        if (StringUtils.equals("1", optionnelMultiple) || StringUtils.equals("1", obligatoireMultiple)) {
            projetTutoreInfosDetail.append(" (");
            if (StringUtils.equals("1", optionnelMultiple)) {
                projetTutoreInfosDetail.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.MULTIPLES"));
            }
            if (StringUtils.equals("1", obligatoireMultiple)) {
                projetTutoreInfosDetail.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.MULTIPLES"));
            }
            projetTutoreInfosDetail.append(")");
        }
        return projetTutoreInfosDetail.toString();
    }

    /**
     * Affiche le libelle Oui ou non pour les checkbox
     * @param valeur
     * @return libelle Oui / Non
     */
    public static String getCaseCochee(String valeur) {
        StringBuilder reponse = new StringBuilder();
        if (StringUtils.equals("1", valeur)) {
            reponse.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.OUI"));
        } else {
            reponse.append(OffreFormationUtil.getMessage("BO.CARD.GENERAL.NON"));
        }
        return reponse.toString();
    }

    /**
     * recupere le libelle de la case a cocher
     * @param nomCaseCochee
     * @param valeur
     * @return le libelle de la case a cocher
     */
    public static String getlibelleCaseACocher(String nomCaseCochee, String valeur) {
        String libelle = "";
        if (StringUtils.equals("1", valeur)) {
            libelle = StringUtils.defaultString(OffreFormationUtil.getMessage("FORMATION." + nomCaseCochee), "");
        }
        return libelle;
    }
}
