/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: actualite
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.bean;

import java.io.Serializable;
import java.util.Date;

import com.kportal.cms.objetspartages.annotation.Aggregation;
import com.kportal.cms.objetspartages.annotation.GetterAnnotation;
import com.kportal.cms.objetspartages.annotation.Label;
import com.kportal.cms.objetspartages.annotation.SetterAnnotation;

/**
 * Classe d'acces aux donnees pour actualite.
 */
public class ActualiteBean extends AbstractRestrictionFicheBean implements Serializable {

    private static final long serialVersionUID = 4803968170869317538L;

    /** The resume. */
    protected String resume = null;

    /** The date evenement. */
    protected Date dateEvenement = null;

    /** The lieu. */
    protected String lieu = null;

    /** The id plan acces. */
    protected Long idPlanAcces = null;

    /** The type evenement. */
    @Label(contexte="actualite", type="03")
    protected String typeEvenement = null;

    /** The thematique. */
    @Label(type = "04")
    @Aggregation(name = "thematique")
    protected String thematique = null;

    /** The description. */
    protected String description = null;

    /** The responsable evenement. */
    protected String responsableEvenement = null;

    /** The email responsable evenement. */
    protected String emailResponsableEvenement = null;

    /** The rattachement bandeau. */
    protected String rattachementBandeau = null;

    /** The date suppression. */
    protected Date dateSuppression = null;

    /** The complement date. */
    protected String complementDate = null;

    /** The site evenement. */
    @Label(contexte = "actualite", type = "08")
    @Aggregation(name = "lieu")
    protected String siteEvenement = null;

    /** The date fin evenement. */
    protected Date dateFinEvenement = null;

    /** The date archivage. */
    protected Date dateArchivage = null;

    /** The id vignette. */
    protected Long idVignette = null;

    /** The date declenchement. */
    protected Date dateDeclenchement = null;

    /** The partenaires. */
    protected String partenaires = null;

    /** The insertion agenda. */
    protected String insertionAgenda = null;

    /**
     * Instantiates a new actualite bean.
     */
    public ActualiteBean() {
        super();
    }

    /**
     * Inits the.
     *
     * @param bean
     *            the bean
     */
    public void init(ActualiteBean bean) {
        this.titre = bean.titre;
        this.resume = bean.resume;
        this.dateEvenement = bean.dateEvenement;
        this.lieu = bean.lieu;
        this.idPlanAcces = bean.idPlanAcces;
        this.typeEvenement = bean.typeEvenement;
        this.thematique = bean.thematique;
        this.description = bean.description;
        this.responsableEvenement = bean.responsableEvenement;
        this.emailResponsableEvenement = bean.emailResponsableEvenement;
        this.rattachementBandeau = bean.rattachementBandeau;
        this.dateSuppression = bean.dateSuppression;
        this.complementDate = bean.complementDate;
        this.siteEvenement = bean.siteEvenement;
        this.dateFinEvenement = bean.dateFinEvenement;
        this.dateArchivage = bean.dateArchivage;
        this.idVignette = bean.idVignette;
        this.dateDeclenchement = bean.dateDeclenchement;
        this.partenaires = bean.partenaires;
        this.insertionAgenda = bean.insertionAgenda;
        this.code = bean.code;
        this.codeRubrique = bean.codeRubrique;
        this.codeRattachement = bean.codeRattachement;
        this.metaKeywords = bean.metaKeywords;
        this.metaDescription = bean.metaDescription;
        this.titreEncadre = bean.titreEncadre;
        this.contenuEncadre = bean.contenuEncadre;
        this.encadreRecherche = bean.encadreRecherche;
        this.encadreRechercheBis = bean.encadreRechercheBis;
        this.dateAlerte = bean.dateAlerte;
        this.messageAlerte = bean.messageAlerte;
        this.dateCreation = bean.dateCreation;
        this.dateProposition = bean.dateProposition;
        this.dateValidation = bean.dateValidation;
        this.dateModification = bean.dateModification;
        this.codeRedacteur = bean.codeRedacteur;
        this.codeValidation = bean.codeValidation;
        this.langue = bean.langue;
        this.etatObjet = bean.etatObjet;
        this.nbHits = bean.nbHits;
        this.diffusionPublicVise = bean.diffusionPublicVise;
        this.diffusionModeRestriction = bean.diffusionModeRestriction;
        this.diffusionPublicViseRestriction = bean.diffusionPublicViseRestriction;
    }

    /**
     * Gets the complement date.
     *
     * @return the complementDate
     */
    @GetterAnnotation(isToolbox = true)
    public String getComplementDate() {
        return complementDate;
    }

    /**
     * Sets the complement date.
     *
     * @param complementDate
     *            the complementDate to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setComplementDate(String complementDate) {
        this.complementDate = complementDate;
    }

    /**
     * Gets the date archivage.
     *
     * @return the dateArchivage
     */
    public Date getDateArchivage() {
        return dateArchivage;
    }

    /**
     * Sets the date archivage.
     *
     * @param dateArchivage
     *            the dateArchivage to set
     */
    public void setDateArchivage(Date dateArchivage) {
        this.dateArchivage = dateArchivage;
    }

    /**
     * Gets the date declenchement.
     *
     * @return the dateDeclenchement
     */
    public Date getDateDeclenchement() {
        return dateDeclenchement;
    }

    /**
     * Sets the date declenchement.
     *
     * @param dateDeclenchement
     *            the dateDeclenchement to set
     */
    public void setDateDeclenchement(Date dateDeclenchement) {
        this.dateDeclenchement = dateDeclenchement;
    }

    /**
     * Gets the date evenement.
     *
     * @return the dateEvenement
     */
    public Date getDateEvenement() {
        return dateEvenement;
    }

    /**
     * Sets the date evenement.
     *
     * @param dateEvenement
     *            the dateEvenement to set
     */
    public void setDateEvenement(Date dateEvenement) {
        this.dateEvenement = dateEvenement;
    }

    /**
     * Gets the date fin evenement.
     *
     * @return the dateFinEvenement
     */
    public Date getDateFinEvenement() {
        return dateFinEvenement;
    }

    /**
     * Sets the date fin evenement.
     *
     * @param dateFinEvenement
     *            the dateFinEvenement to set
     */
    public void setDateFinEvenement(Date dateFinEvenement) {
        this.dateFinEvenement = dateFinEvenement;
    }

    /**
     * Gets the date suppression.
     *
     * @return the dateSuppression
     */
    public Date getDateSuppression() {
        return dateSuppression;
    }

    /**
     * Sets the date suppression.
     *
     * @param dateSuppression
     *            the dateSuppression to set
     */
    public void setDateSuppression(Date dateSuppression) {
        this.dateSuppression = dateSuppression;
    }

    /**
     * Gets the description.
     *
     * @return the description
     */
    @GetterAnnotation(isToolbox = true)
    public String getDescription() {
        return description;
    }

    /**
     * Sets the description.
     *
     * @param description
     *            the description to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Gets the email responsable evenement.
     *
     * @return the emailResponsableEvenement
     */
    public String getEmailResponsableEvenement() {
        return emailResponsableEvenement;
    }

    /**
     * Sets the email responsable evenement.
     *
     * @param emailResponsableEvenement
     *            the emailResponsableEvenement to set
     */
    public void setEmailResponsableEvenement(String emailResponsableEvenement) {
        this.emailResponsableEvenement = emailResponsableEvenement;
    }

    /**
     * Gets the id plan acces.
     *
     * @return the idPlanAcces
     */
    @GetterAnnotation(isIdMedia = true)
    public Long getIdPlanAcces() {
        return idPlanAcces;
    }

    /**
     * Sets the id plan acces.
     *
     * @param idPlanAcces
     *            the idPlanAcces to set
     */
    public void setIdPlanAcces(Long idPlanAcces) {
        this.idPlanAcces = idPlanAcces;
    }

    /**
     * Gets the id vignette.
     *
     * @return the idVignette
     */
    public Long getIdVignette() {
        return idVignette;
    }

    /**
     * Sets the id vignette.
     *
     * @param idVignette
     *            the idVignette to set
     */
    public void setIdVignette(Long idVignette) {
        this.idVignette = idVignette;
    }

    /**
     * Gets the insertion agenda.
     *
     * @return the insertionAgenda
     */
    public String getInsertionAgenda() {
        return insertionAgenda;
    }

    /**
     * Sets the insertion agenda.
     *
     * @param insertionAgenda
     *            the insertionAgenda to set
     */
    public void setInsertionAgenda(String insertionAgenda) {
        this.insertionAgenda = insertionAgenda;
    }

    /**
     * Gets the lieu.
     *
     * @return the lieu
     */
    @GetterAnnotation(isToolbox = true)
    public String getLieu() {
        return lieu;
    }

    /**
     * Sets the lieu.
     *
     * @param lieu
     *            the lieu to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setLieu(String lieu) {
        this.lieu = lieu;
    }

    /**
     * Gets the partenaires.
     *
     * @return the partenaires
     */
    @GetterAnnotation(isToolbox = true)
    public String getPartenaires() {
        return partenaires;
    }

    /**
     * Sets the partenaires.
     *
     * @param partenaires
     *            the partenaires to set
     */
    @SetterAnnotation(isToolbox = true)
    public void setPartenaires(String partenaires) {
        this.partenaires = partenaires;
    }

    /**
     * Gets the rattachement bandeau.
     *
     * @return the rattachementBandeau
     */
    public String getRattachementBandeau() {
        return rattachementBandeau;
    }

    /**
     * Sets the rattachement bandeau.
     *
     * @param rattachementBandeau
     *            the rattachementBandeau to set
     */
    public void setRattachementBandeau(String rattachementBandeau) {
        this.rattachementBandeau = rattachementBandeau;
    }

    /**
     * Gets the responsable evenement.
     *
     * @return the responsableEvenement
     */
    public String getResponsableEvenement() {
        return responsableEvenement;
    }

    /**
     * Sets the responsable evenement.
     *
     * @param responsableEvenement
     *            the responsableEvenement to set
     */
    public void setResponsableEvenement(String responsableEvenement) {
        this.responsableEvenement = responsableEvenement;
    }

    /**
     * Gets the resume.
     *
     * @return the resume
     */
    public String getResume() {
        return resume;
    }

    /**
     * Sets the resume.
     *
     * @param resume
     *            the resume to set
     */
    public void setResume(String resume) {
        this.resume = resume;
    }

    /**
     * Gets the site evenement.
     *
     * @return the siteEvenement
     */
    public String getSiteEvenement() {
        return siteEvenement;
    }

    /**
     * Sets the site evenement.
     *
     * @param siteEvenement
     *            the siteEvenement to set
     */
    public void setSiteEvenement(String siteEvenement) {
        this.siteEvenement = siteEvenement;
    }

    /**
     * Gets the thematique.
     *
     * @return the thematique
     */
    public String getThematique() {
        return thematique;
    }

    /**
     * Sets the thematique.
     *
     * @param thematique
     *            the thematique to set
     */
    public void setThematique(String thematique) {
        this.thematique = thematique;
    }

     /**
     * Gets the type evenement.
     *
     * @return the typeEvenement
     */
    public String getTypeEvenement() {
        return typeEvenement;
    }

    /**
     * Sets the type evenement.
     *
     * @param typeEvenement
     *            the typeEvenement to set
     */
    public void setTypeEvenement(String typeEvenement) {
        this.typeEvenement = typeEvenement;
    }
}
