/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: actualite
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.dao.impl;

import java.util.List;

import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.univ.objetspartages.bean.ActualiteBean;
import com.univ.objetspartages.dao.AbstractFicheDAO;

/**
 * DAO de l'objet ActualiteBean
 */
public class ActualiteDAO extends AbstractFicheDAO<ActualiteBean> {

    public ActualiteDAO() {
        tableName = "ACTUALITE";
    }

    public List<ActualiteBean> getByAccesPlanId(final Long accesPlanId) throws DataSourceException {
        List<ActualiteBean> result;
        try {
            SqlParameterSource namedParameters = new MapSqlParameterSource("id", accesPlanId);
            result = namedParameterJdbcTemplate.query("select * from ACTUALITE T1 WHERE ID_PLAN_ACCES = :id", namedParameters, rowMapper);
        } catch (DataAccessException dae) {
            throw new DataSourceException(String.format("An error occured retrieving object with id %d from table %s", accesPlanId, tableName), dae);
        }
        return result;
    }
}
