/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: actualite
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.processus;

import java.util.Date;

import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.MessageHelper;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.om.Actualite;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.LabelUtils;
import com.univ.objetspartages.util.MediaUtils;

/**
 * Processus de gestion des actualites.
 */
public class SaisieActualite extends SaisieFiche {

    private final ServiceMedia serviceMedia;

    /**
     * The actualite.
     */
    protected Actualite actualite = null;

    /**
     * Constructeur.
     *
     * @param infoBean the info bean
     */
    public SaisieActualite(final InfoBean infoBean) {
        super(infoBean);
        serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
    }

    /**
     * Point d'entree du processus.
     *
     * @return true, if traiter action
     * @throws Exception the exception
     */
    @Override
    public boolean traiterAction() throws Exception {
        infoBean.set("CODE_OBJET", "0008");
        actualite = new Actualite();
        actualite.init();
        final AutorisationBean autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
        if (autorisations != null) {
            this.getDatas().put("AUTORISATIONS", autorisations);
        }
        traiterActionParDefaut(actualite);
        infoBean.set("NOM_ONGLET", "actualite");
        // on continue si on n'est pas a la FIN !!!
        return etat == FIN;
    }

    @Override
    protected void preparerRECHERCHE() {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(getLocale()));
        infoBean.set("LISTE_TYPE_EVENEMENT", LabelUtils.getLabelCombo("03", getLocale()));
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelCombo("04", getLocale()));
        infoBean.set("LISTE_SITES", LabelUtils.getLabelCombo("08", getLocale()));
    }

    @Override
    protected void prepareRestrictedSearch(String sectionCode) {
        infoBean.set("LISTE_LANGUES", LangueUtil.getListeLangues(getLocale()));
        infoBean.set("LISTE_TYPE_EVENEMENT", LabelUtils.getLabelComboWithRestrictionInformation("03", getLocale()));
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelComboWithRestrictionInformation("04", getLocale()));
        infoBean.set("LISTE_SITES", LabelUtils.getLabelComboWithRestrictionInformation("08", getLocale()));
    }

    /**
     * Affichage de l'ecran de saisie d'une actualite.
     *
     * @throws Exception the exception
     */
    @Override
    protected void preparerPRINCIPAL() throws Exception {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        ecranLogique = ECRAN_PRINCIPAL;
        infoBean.set("ID_ACTUALITE", actualite.getIdActualite().toString());
        infoBean.set("TITRE", actualite.getTitre());
        infoBean.set("DATE_EVENEMENT", actualite.getDateEvenement());
        infoBean.set("DATE_FIN_EVENEMENT", actualite.getDateFinEvenement());
        infoBean.set("CORPS", actualite.getDescription());
        infoBean.set("THEMATIQUE", actualite.getThematique());
        infoBean.set("LIBELLE_THEMATIQUE", LabelUtils.getLibelleWithSection("04", actualite.getThematique(), actualite.getLangue(), getCurrentSection(actualite.getPersistanceBean())));
        infoBean.set("LISTE_THEMATIQUES", LabelUtils.getLabelForComboForSection("04", actualite.getLangue(), getCurrentSection(actualite.getPersistanceBean())));
        infoBean.set("SITE_EVENEMENT", actualite.getSiteEvenement());
        infoBean.set("LIBELLE_SITE_EVENEMENT", LabelUtils.getLibelleWithSection("08", actualite.getSiteEvenement(), actualite.getLangue(), getCurrentSection(actualite.getPersistanceBean())));
        infoBean.set("LISTE_SITES_EVENEMENT", LabelUtils.getLabelForComboForSection("08", actualite.getLangue(), getCurrentSection(actualite.getPersistanceBean())));
        infoBean.set("COMPLEMENT_DATE", actualite.getComplementDate());
        infoBean.set("RESUME", actualite.getResume());
        infoBean.set("PARTENAIRES", actualite.getPartenaires());
        infoBean.set("RATTACHEMENT_BANDEAU", actualite.getRattachementBandeau());
        infoBean.set("LIEU", actualite.getLieu());
        infoBean.set("ID_PLAN_ACCES", actualite.getIdPlanAcces());
        infoBean.set("LISTE_PLANS_ACCES", MediaUtils.getMediasAsMap(serviceMedia.getByTypeAndTypeMedia("photo", "0003")));
        infoBean.set("TYPE_EVENEMENT", actualite.getTypeEvenement());
        infoBean.set("LIBELLE_TYPE_EVENEMENT", LabelUtils.getLibelleWithSection("03", actualite.getTypeEvenement(), actualite.getLangue(), getCurrentSection(actualite.getPersistanceBean())));
        infoBean.set("LISTE_TYPE_EVENEMENT", LabelUtils.getLabelForComboForSection("03", actualite.getLangue(), getCurrentSection(actualite.getPersistanceBean())));
        infoBean.set("RESPONSABLE_EVENEMENT", actualite.getResponsableEvenement());
        infoBean.set("EMAIL_RESPONSABLE_EVENEMENT", actualite.getEmailResponsableEvenement());
        infoBean.set("CODE_RATTACHEMENT", actualite.getCodeRattachement());
        infoBean.set("LIBELLE_CODE_RATTACHEMENT", serviceStructure.getDisplayableLabel(actualite.getCodeRattachement(), actualite.getLangue()));
        infoBean.set("LIBELLE_AFFICHABLE", actualite.getLibelleAffichable());
        infoBean.set("SOUS_ONGLET", "PRINCIPAL"); // onglet par defaut
        ControleurUniv.preparerPRINCIPAL(infoBean, actualite, this);
    }

    @Override
    protected void alimenterDonneesFiche() throws Exception {
        actualite.setTitre(infoBean.getString("TITRE"));
        actualite.setDateEvenement(infoBean.getDate("DATE_EVENEMENT"));
        actualite.setCodeRattachement(infoBean.getString("CODE_RATTACHEMENT"));
        if (!"0000".equals(infoBean.getString("CODE_RUBRIQUE"))) {
            actualite.setCodeRubrique(infoBean.getString("CODE_RUBRIQUE"));
        } else {
            actualite.setCodeRubrique("");
        }
        final Date dateFinSaisie = infoBean.getDate("DATE_FIN_EVENEMENT");
        if (Formateur.estSaisie(dateFinSaisie)) {
            if(actualite.getDateEvenement() == null){
                throw new ErreurApplicative(MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ACTUALITE_ERREUR_DATE_DEBUT_NON_SAISIE"));
            }
            if (dateFinSaisie.before(actualite.getDateEvenement())) {
                throw new ErreurApplicative(MessageHelper.getMessage(infoBean.getNomExtension(), "BO_ACTUALITE_ERREUR_DATE_FIN"));
            }
            actualite.setDateFinEvenement(infoBean.getDate("DATE_FIN_EVENEMENT"));
        } else {
            if (Formateur.estSaisie(infoBean.getDate("DATE_EVENEMENT"))) {
                actualite.setDateFinEvenement(infoBean.getDate("DATE_EVENEMENT"));
            } else {
                actualite.setDateFinEvenement(null);
            }
        }
        actualite.setThematique(infoBean.getString("THEMATIQUE"));
        actualite.setResume(infoBean.getString("RESUME"));
        actualite.setRattachementBandeau(infoBean.getString("RATTACHEMENT_BANDEAU"));
        actualite.setLieu(infoBean.getString("LIEU"));
        if (infoBean.get("ID_PLAN_ACCES") instanceof String) {
            //si l'onglet en cours ne contient pas le plan d'accès
            actualite.setIdPlanAcces(Long.valueOf(infoBean.getString("ID_PLAN_ACCES")));
        } else {
            //si l'onglet en cours contient le plan d'accès
            actualite.setIdPlanAcces((Long) infoBean.get("ID_PLAN_ACCES"));
        }
        actualite.setEmailResponsableEvenement(infoBean.getString("EMAIL_RESPONSABLE_EVENEMENT"));
        actualite.setResponsableEvenement(infoBean.getString("RESPONSABLE_EVENEMENT"));
        actualite.setDescription(infoBean.getString("CORPS"));
        actualite.setTypeEvenement(infoBean.getString("TYPE_EVENEMENT"));
        actualite.setSiteEvenement(infoBean.getString("SITE_EVENEMENT"));
        actualite.setPartenaires(infoBean.getString("PARTENAIRES"));
        actualite.setComplementDate(infoBean.getString("COMPLEMENT_DATE"));
    }

    /**
     * Traite l'ecran de saisie d'une fiche.
     *
     * @throws Exception the exception
     */
    @Override
    protected void traiterPRINCIPAL() throws Exception {
        // chargement de l'objet
        if (infoBean.getEtatObjet().equals(InfoBean.ETAT_OBJET_CREATION)) {
            actualite.init();
        } else {
            actualite.setIdActualite(Long.valueOf(infoBean.getString("ID_ACTUALITE")));
            actualite.retrieve();
        }
        // changement d'onglet
        if (action.equals(InfoBean.ACTION_ONGLET)) {
            infoBean.set("SOUS_ONGLET", infoBean.getString("SOUS_ONGLET_DEMANDE"));
        }
        // validation de l'ecran
        else if (action.equals(InfoBean.ACTION_ENREGISTRER)) {
            alimenteDonneesCreation(actualite, false);
            alimenterDonneesFiche();
        }
        // appel au traitement general
        ecranLogique = ControleurUniv.traiterPRINCIPAL(infoBean, actualite, this);
        if (ecranLogique.length() == 0) {
            etat = FIN;
        }
    }
}
