/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: actualite
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.service;

import java.text.DateFormat;
import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.datasource.exceptions.DataSourceException;
import com.kosmos.service.impl.ServiceFiche;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.ActualiteBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.dao.impl.ActualiteDAO;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.view.model.ActualiteViewModel;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;
import com.univ.utils.FicheUnivMgr;

/**
 * Created on 28/09/17.
 */
public class ServiceActualite extends ServiceFiche<ActualiteBean, ActualiteDAO, ActualiteViewModel> {

    /**
     * Service de gestion des {@link com.univ.objetspartages.bean.MediaBean}.
     */
    private ServiceMedia serviceMedia;

    /**
     * Service de gestion des {@link RessourceBean}.
     */
    private ServiceRessource serviceRessource;

    private static final Logger LOG = LoggerFactory.getLogger(ServiceActualite.class);

    /**
     * {@inheritDoc}
     */
    @Override
    public Optional<ActualiteViewModel> getCommonViewModel(final ActualiteBean bean) {
        Optional<ActualiteViewModel> optViewModel = super.getCommonViewModel(bean);
        if (optViewModel.isPresent()) {
            final ActualiteViewModel viewModel = optViewModel.get();
            final ContexteUniv contexteUniv = ContexteUtil.getContexteUniv();
            // Date
            if(bean.getDateEvenement() != null) {
                final DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.LONG, contexteUniv.getLocale());
                viewModel.setDate(dateFormat.format(bean.getDateEvenement()));
            }
            // Photo
            RessourceBean photoJointe = serviceRessource.getFile(bean.getId(), ReferentielObjets.getCodeObjetParClasseBean(bean.getClass()));
            if (photoJointe != null) {
                viewModel.setPhoto(serviceMedia.getById(photoJointe.getIdMedia()));
            }
        }
        return optViewModel;
    }

    @Override
    protected ActualiteViewModel getViewModel(final String nomObjet, final String titre, final String urlFiche) {
        return new ActualiteViewModel(nomObjet, titre, urlFiche);
    }

    public void setServiceMedia(final ServiceMedia serviceMedia) {
        this.serviceMedia = serviceMedia;
    }

    public void setServiceRessource(final ServiceRessource serviceRessource) {
        this.serviceRessource = serviceRessource;
    }

    @Override
    public String getReferences(ActualiteBean bean) {
        String res = StringUtils.defaultString(bean.getLieu());
        res += FicheUnivMgr.getReferenceParPhoto(bean.getIdPlanAcces());
        res += StringUtils.defaultString(bean.getDescription());
        res += StringUtils.defaultString(bean.getComplementDate());
        res += FicheUnivMgr.getReferenceParPhoto(bean.getIdVignette());
        res += StringUtils.defaultString(bean.getPartenaires());
        res += FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement());
        try {
            ServiceRessource serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
            res += FicheUnivMgr.getReferenceParListePhoto(serviceRessource.getFiles(bean));
            res += FicheUnivMgr.getReferenceParListePhoto(serviceRessource.getFilesOrderBy(bean, "CODE_PARENT"));
        } catch (final DataSourceException e) {
            LOG.debug("unable to query on files list", e);
        }
        return res;
    }

    public List<ActualiteBean> getByAccesPlanId(final Long accesPlanId) throws DataSourceException {
        return dao.getByAccesPlanId(accesPlanId);
    }
}
