/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: ged
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.service;

import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.service.ServiceContentRedirect;
import com.kosmos.service.impl.ServiceFiche;
import com.kportal.core.config.PropertyHelper;
import com.univ.objetspartages.bean.DocumentBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.dao.impl.DocumentDAO;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.util.RessourceUtils;
import com.univ.objetspartages.view.model.FicheDefaultViewModel;
import com.univ.utils.ContexteUtil;
import com.univ.utils.FicheUnivMgr;
import com.univ.utils.URLResolver;

/**
 * Created on 28/09/17.
 */
public class ServiceDocument extends ServiceFiche<DocumentBean, DocumentDAO, FicheDefaultViewModel> implements ServiceContentRedirect<DocumentBean> {

    private static final Logger LOG = LoggerFactory.getLogger(ServiceDocument.class);

    ServiceMetatag serviceMetatag;

    ServiceRessource serviceRessource;

    @Override
    public String getReferences(DocumentBean bean) {
        String references = StringUtils.defaultString(bean.getSourceSupport());
        references += StringUtils.defaultString(bean.getAuteurs());
        references += StringUtils.defaultString(bean.getEditeur());
        references += StringUtils.defaultString(bean.getDescription());
        references += StringUtils.defaultString(bean.getEditeur());
        references += StringUtils.defaultString(bean.getReferencesIsdn());
        references += FicheUnivMgr.getReferenceParJointure("structure", bean.getCodeRattachement());
        references += StringUtils.defaultString(bean.getComplements());
        return references;
    }

    @Override
    public String getUrlRedirect(final DocumentBean documentBean) {
        String urlRedirect = StringUtils.EMPTY;
        final RessourceBean ressourceBean = getSingleRessourceForDownload(documentBean);
        if (ressourceBean != null) {
            urlRedirect = URLResolver.getAbsoluteUrl(RessourceUtils.getUrl(ressourceBean), ContexteUtil.getContexteUniv());
            final String typeDisposition = StringUtils.defaultIfEmpty(PropertyHelper.getCoreProperty("telechargement.disposition"), "attachment");
            if (typeDisposition.equalsIgnoreCase("attachment")) {
                urlRedirect += (urlRedirect.contains("?") ? "&" : "?") + "INLINE=FALSE";
            }
        }
        return urlRedirect;
    }

    public void setServiceMetatag(final ServiceMetatag serviceMetatag) {
        this.serviceMetatag = serviceMetatag;
    }

    public void setServiceRessource(final ServiceRessource serviceRessource) {
        this.serviceRessource = serviceRessource;
    }

    /**
     * Renvoie la première ressource associée à une fiche document si la fiche est uniquement accessible en téléchargement.
     * @param documentBean {@link DocumentBean} Le document traité
     * @return null si le document est null, si on a aucune ou plus d'une ressource associée ou bien si le document n'est pas accessible en téléchargement
     * Retourne {@link RessourceBean} dans le cas échéant
     */
    public RessourceBean getSingleRessourceForDownload(final DocumentBean documentBean) {
        RessourceBean ressourceBean = null;
        if (documentBean != null) {
            final MetatagBean meta = serviceMetatag.getByCodeAndIdFiche("0017", documentBean.getId());
            if (meta != null && "1".equals(meta.getMetaDocumentFichiergw())) {
                final List<RessourceBean> fichiersJoint = serviceRessource.getFiles(documentBean);
                if (CollectionUtils.size(fichiersJoint) == 1) {
                    ressourceBean = fichiersJoint.get(0);
                }
            }
        }
        return ressourceBean;
    }
}
