/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: seo
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.servlet;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.kosmos.seo.util.SEOUtil;

@Controller
public class DownloaderSitemapServlet {

    public final static String URL_PROPERTY = "sitemap_downloader.mapping";

    @RequestMapping(value = "${" + URL_PROPERTY + ":/servlet/sitemap}", method = RequestMethod.GET)
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        final String submit = StringUtils.defaultIfBlank(req.getParameter("submit"), StringUtils.EMPTY);
        final String alias = StringUtils.defaultIfBlank(req.getParameter("alias"), StringUtils.EMPTY);
        final String mode = StringUtils.defaultIfBlank(req.getParameter("mode"), StringUtils.EMPTY);
        if (StringUtils.isNotBlank(alias)) {
            if (StringUtils.isNotBlank(mode) && "XML".equals(mode)) {
                try {
                    final String xmlString = SEOUtil.getGenerator().genererSitemap(alias);
                    resp.setCharacterEncoding("UTF-8");
                    resp.setContentType("application/xml");
                    resp.getWriter().write(xmlString);
                } catch (Exception e) {
                    resp.sendError(500, e.getMessage());
                }
            } else if (StringUtils.isNotBlank(submit)) {
                final HttpClient client = new HttpClient();
                final HttpMethod httpMethod = new GetMethod(submit);
                try {
                    resp.setStatus(client.executeMethod(httpMethod));
                } catch (IOException e) {
                    resp.sendError(500, e.getMessage());
                }
            } else {
                resp.sendError(404);
            }
        }
    }
}
