/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.objetspartages.util;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.beanutils.Converter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class XmlDateConverter implements Converter {

    private static final Logger LOGGER = LoggerFactory.getLogger(XmlDateConverter.class);

    private final List<DateFormat> formats = new ArrayList<>();

    public Date convert(Class clazz, Object value) {
        if (Date.class.isInstance(value)) {
            return (Date) value;
        }
        if (String.class.isInstance(value)) {
            for (DateFormat currentFormat : formats) {
                try {
                    final Date date = currentFormat.parse((String) value);
                    if (date != null) {
                        return new Date(date.getTime());
                    }
                } catch (ParseException e) {
                    LOGGER.debug(String.format("Impossible de parser la date : %s", value), e);
                }
            }
        }
        return null;
    }

    public void registerPattern(String pattern, final Locale locale, final boolean lenient) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern, locale);
        simpleDateFormat.setLenient(lenient);
        formats.add(simpleDateFormat);
    }
}