/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws;

import java.io.File;
import java.io.StringWriter;
import java.net.URL;
import java.util.Vector;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xmlrpc.AuthenticatedXmlRpcHandler;
import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Node;

import com.jsbsoft.jtf.identification.GestionnaireIdentification;
import com.jsbsoft.jtf.identification.SourceAuthDefautKbd;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.kosmos.importexport.util.ExtensionUtil;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.utils.ContexteUniv;
import com.univ.utils.ContexteUtil;

public class ExportFicheHandler implements AuthenticatedXmlRpcHandler {

    public static String remoteProcedure = "ExportFicheHandler.execute";

    private String remoteHost = null;

    private Logger logger = null;

    public ExportFicheHandler() {
        super();
    }

    public ExportFicheHandler(final String host) {
        super();
        this.remoteHost = host;
    }

    /**
     * execute pour les variables d'authentification
     */
    @Override
    public Object execute(final String methodName, final Vector parameters, final String user, final String password) throws Exception {
        final String requete = (String) parameters.firstElement();
        if (logger == null) {
            logger = LoggerFactory.getLogger(ExportFicheHandler.class);
        }
        return execute(user, password, requete);
    }

    public String execute(final String user, final String password, final String requete) throws XmlRpcException {
        String out = null;
        final ContexteUniv ctx = ContexteUtil.getContexteUniv();
        //
        try {
            if (remoteHost != null) {
                // appel distant
                final URL urlClient = new URL(remoteHost + ExtensionUtil.getProperty(SynchroXmlRpcServlet.URL_PROPERTY));
                // creation du client xmlrpc pour cet url
                final XmlRpcClient xmlrpc = new XmlRpcClient(urlClient);
                xmlrpc.setBasicAuthentication(user, password);
                // ajout de la requete en parametre
                final Vector<String> paramHandler = new Vector<String>();
                paramHandler.addElement(requete);
                logger.info("Url : " + urlClient.toString());
                logger.info("Requête : " + requete);
                out = (String) xmlrpc.execute(remoteProcedure, paramHandler);
            } else {
                logger.info("Export des fiches en ws");
                // création de la session utilisateur
                if ((new SourceAuthDefautKbd()).connecte(user, password, ctx)) {
                    final UtilisateurBean utilisateur = GestionnaireIdentification.getInstance().estUtilisateurKportal(user, password, ctx);
                    ctx.setCode(utilisateur.getCode());
                    ctx.setProfilDsi(utilisateur.getProfilDsi());
                }
                final javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
                final javax.xml.parsers.DocumentBuilder builder = factory.newDocumentBuilder();
                final TransformerFactory tFactory = TransformerFactory.newInstance();
                Transformer transformer = null;
                final org.w3c.dom.Document document = builder.newDocument();
                final org.w3c.dom.Element rootElement = document.createElement("ROOT");
                // Génération d'un dom d'export par instrospection
                logger.info("Requête : " + requete);
                final Node resultat = ExportFicheIntrospection.executeExport(ctx, rootElement, requete);
                rootElement.appendChild(resultat);
                document.appendChild(rootElement);
                Node nodeOut = document;
                // si la personne appartient à un seul groupe DSI, et qu'on a un xsl pour ce groupe, alors on fait une transfo
                boolean transfoXSL = false;
                final String xslPath = ExtensionUtil.getXslPath();
                File xslFile = null;
                if (xslPath != null) {
                    xslFile = new File(xslPath + File.separator + "PWS_" + user.toUpperCase() + ".xsl");
                    if (xslFile.exists()) {
                        transfoXSL = true;
                    }
                }
                if (transfoXSL) {
                    logger.info("Transformation XSL");
                    // on transforme les fiches en DOM
                    final DOMResult domResult = new DOMResult();
                    transformer = tFactory.newTransformer(new StreamSource(xslFile));
                    transformer.setOutputProperty(OutputKeys.ENCODING, CharEncoding.DEFAULT);
                    transformer.setOutputProperty(OutputKeys.METHOD, "xml");
                    transformer.transform(new DOMSource(document), domResult);
                    nodeOut = domResult.getNode();
                }
                // puis on transforme en texte
                logger.info("Sauvegarde en XML");
                transformer = tFactory.newTransformer();
                final DOMSource source = new DOMSource(nodeOut);
                final StringWriter sw = new StringWriter();
                final StreamResult result = new StreamResult(sw);
                // on force l'export en iso pour la compatibilité
                transformer.setOutputProperty(OutputKeys.ENCODING, org.apache.commons.lang3.CharEncoding.ISO_8859_1);
                transformer.transform(source, result);
                out = sw.toString();
            }
            return out;
        } catch (final Exception e) {
            if (e instanceof XmlRpcException) {
                logger.error(e.getMessage(), e);
                throw (XmlRpcException) e;
            } else {
                logger.error(e.getMessage(), e);
                throw new XmlRpcException(0, e.getMessage());
            }
        }
    }

    public void setLogger(final Logger logger) {
        this.logger = logger;
    }
}
