/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws;

import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Node;

import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.textsearch.ResultatRecherche;
import com.kosmos.importexport.util.ExtensionUtil;
import com.kportal.extension.module.plugin.objetspartages.IPluginFiche;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.kportal.extension.module.plugin.objetspartages.om.PluginBean;
import com.univ.mediatheque.utils.MediathequeHelper;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.Chaine;
import com.univ.utils.ContexteUniv;
import com.univ.utils.IntrospectionWrapper;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.xml.NodeUtil;

/**
 * Classe d'export par introspection
 */
public class ExportFicheIntrospection {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportFicheIntrospection.class);

    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    protected ContexteUniv ctx;

    public ExportFicheIntrospection() {
        super();
    }

    /**
     * A partir d'une requete standard : Generation d'un DOM <LISTE_OBJETS> <OBJET> contenant les tags issus de l'instrospection
     *
     * @param _ctx
     * @param _rootElement
     * @param _requete
     * @return
     * @throws Exception
     */
    public static Node executeExport(final OMContext _ctx, final Node _rootElement, final String _requete) throws Exception {
        String REQUETE = StringUtils.replace(_requete.toUpperCase(), "%20", " ");
        final String objet = RequeteUtil.renvoyerParametre(REQUETE, "OBJET");
        FicheUniv ficheUniv = ReferentielObjets.instancierFiche(objet);
        if (ficheUniv == null) {
            return _rootElement;
        }
        ficheUniv.setCtx(_ctx);
        ficheUniv.init();
        final Map<String, Method> lstGetter = IntrospectionWrapper.getIntrospectionWrapper(ficheUniv.getClass()).getLstGetter();
        final Node liste = _rootElement.getOwnerDocument().createElement("LISTE_OBJETS");
        try {
            // EXECUTION DE LA REQUETE ET RECUPERATION DE TOUTES LES FICHES RESULTATS
            final List<ResultatRecherche> listeFiches = RequeteUtil.traiterRequete(_ctx, _requete);
            for (ResultatRecherche res : listeFiches) {
                try {
                    Method getter;
                    Object returnParam;
                    String valeur;
                    FicheUniv fiche = RequeteUtil.lireFiche(res);
                    Node listeResultat = liste.getOwnerDocument().createElement(objet);
                    for (final String libelleParametre : lstGetter.keySet()) {
                        getter = lstGetter.get(libelleParametre);
                        returnParam = getter.invoke(fiche);
                        if (returnParam != null) {
                            if (returnParam instanceof java.sql.Date) {
                                valeur = sdf.format(new Date(((java.sql.Date) returnParam).getTime()));
                                NodeUtil.addNode(listeResultat, formatNodeLibelle(libelleParametre), valeur);
                            } else {
                                valeur = returnParam.toString();
                                if (valeur.indexOf('[') != -1) {
                                    valeur = UnivWebFmt.formaterEnHTML((ContexteUniv) _ctx, valeur);
                                }
                                if (valeur.indexOf('<') != -1) {
                                    saveNodeCData(listeResultat, formatNodeLibelle(libelleParametre), Chaine.encodeISO88591(valeur));
                                } else {
                                    NodeUtil.addNode(listeResultat, formatNodeLibelle(libelleParametre), Chaine.encodeISO88591(valeur));
                                }
                            }
                        }
                    }
                    /* récupération des données Metatag */
                    final MetatagBean meta = MetatagUtils.lireMeta(fiche);
                    if (meta.getId() > 0L && meta.getMetaDateSuppression() != null) {
                        returnParam = meta.getMetaDateSuppression();
                        if (returnParam instanceof java.sql.Date) {
                            valeur = sdf.format(new java.util.Date(((java.sql.Date) returnParam).getTime()));
                        } else {
                            valeur = returnParam.toString();
                        }
                        NodeUtil.addNode(listeResultat, "META_DATE_SUPPRESSION", valeur);
                    }
                    final List<String> classNames = new ArrayList<String>();
                    for (final IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
                        if (plugin.isActive(fiche.getClass().getName())) {
                            if (plugin.getObjets() != null) {
                                classNames.addAll(plugin.getObjets());
                            }
                        }
                    }
                    // On boucle sur tous les sous-objets de la fiche courante
                    for (final String name : classNames) {
                        final int indexDebutNomClasse = name.lastIndexOf(".");
                        String nomSousObjet = name.substring(indexDebutNomClasse + 1).toUpperCase();
                        Node listeSousObjets = liste.getOwnerDocument().createElement(nomSousObjet + "S");
                        final List<PluginBean> lstObjets = PluginFicheHelper.lirePlugins(fiche, meta, name);
                        for (final PluginBean objetPluginContenu : lstObjets) {
                            Node sousObjet = liste.getOwnerDocument().createElement(nomSousObjet);
                            final Map<String, Method> lstGetterSousObjet = IntrospectionWrapper.getIntrospectionWrapper(objetPluginContenu.getClass()).getLstGetter();
                            for (final String libelleParametre : lstGetterSousObjet.keySet()) {
                                getter = lstGetterSousObjet.get(libelleParametre);
                                returnParam = getter.invoke(objetPluginContenu);
                                if (returnParam != null) {
                                    valeur = null;
                                    if (returnParam instanceof java.sql.Date) {
                                        valeur = sdf.format(new Date(((java.sql.Date) returnParam).getTime()));
                                    } else {
                                        valeur = returnParam.toString();
                                        if (valeur.indexOf('[') != -1) {
                                            valeur = UnivWebFmt.formaterEnHTML((ContexteUniv) _ctx, valeur);
                                        }
                                    }
                                    // Insertion de ce noeud ssi il a une valeur
                                    NodeUtil.addNode(sousObjet, formatNodeLibelle(libelleParametre), Chaine.encodeISO88591(valeur));
                                }
                            }
                            listeSousObjets.appendChild(sousObjet);
                        }
                        listeResultat.appendChild(listeSousObjets);
                    }
                    if ("1".equals(ExtensionUtil.getProperty(ExtensionUtil.PROP_HWS_EXPORT_MEDIA))) {
                        final Node listeMediathequePaths = liste.getOwnerDocument().createElement("listeMediathequePaths");
                        listeResultat.appendChild(listeMediathequePaths);
                        NodeUtil.addNode(listeMediathequePaths, "mediathequePath", MediathequeHelper.getUrlRelativePath());
                        final Node listeMedias = liste.getOwnerDocument().createElement("listeMediaRessources");
                        listeResultat.appendChild(listeMedias);
                        exportMedias(_ctx, fiche.getIdFiche().toString(), ReferentielObjets.getCodeObjet(objet), listeMedias);
                    }
                    liste.appendChild(listeResultat);
                } catch (final Exception ex) {
                    LOGGER.error("Erreur lors de l'export de la fiche : requete = " + _requete + ", id_fiche = " + ficheUniv.getIdFiche(), ex);
                }
            }
        } catch (final Exception e) {
            LOGGER.error("Echec de l'introspection", e);
        }
        return liste;
    }

    private static void exportMedias(final OMContext _ctx, final String idFiche, final String typeFiche, final Node destNode) throws Exception {
        try(final ResultSet rs = _ctx.getConnection().createStatement().executeQuery("SELECT * FROM MEDIA M, RESSOURCE R WHERE M.ID_MEDIA=R.ID_MEDIA AND (R.CODE_PARENT like '" + idFiche + ",TYPE=FICHIER_" + typeFiche + "%' OR R.CODE_PARENT like '" + idFiche + ",TYPE=" + typeFiche + "%')")) {
            while (rs.next()) {
                final Node mediaNode = destNode.getOwnerDocument().createElement("mediaRessource");
                destNode.appendChild(mediaNode);
                for (int i = 1; i < rs.getMetaData().getColumnCount() + 1; i++) {
                    final String columnName = rs.getMetaData().getColumnName(i);
                    final String columnValue = rs.getString(columnName);
                    if (StringUtils.isNotBlank(columnValue)) {
                        NodeUtil.addNode(mediaNode, columnName, Chaine.encodeISO88591(columnValue));
                    }
                }
            }
        }
    }

    public static String formatNodeLibelle(final String libelle) {
        String res = "";
        for (int j = 0; j < libelle.length(); j++) {
            if (j != 0 && Character.isUpperCase(libelle.charAt(j))) {
                res += "_";
            }
            res += libelle.charAt(j);
        }
        return res.toUpperCase();
    }

    /**
     *
     * @param aNode
     * @param nomDonnee
     * @param cdatavaleur
     */
    public static void saveNodeCData(final Node aNode, final String nomDonnee, final String cdatavaleur) {
        final Node attribut = aNode.getOwnerDocument().createElement(nomDonnee);
        attribut.appendChild(aNode.getOwnerDocument().createCDATASection(cdatavaleur));
        aNode.appendChild(attribut);
    }
}
