/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Serializable;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.beanutils.ConversionException;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ReflectionUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import com.jsbsoft.jtf.database.OMContext;
import com.kportal.extension.module.plugin.objetspartages.IPluginFiche;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.pws.om.FicheUnivImport;
import com.univ.pws.om.RessourceEtMediaAssocie;
import com.univ.utils.Chaine;
import com.univ.utils.ContexteUniv;
import com.univ.utils.IntrospectionWrapper;
import com.univ.utils.UnivWebFmt;
import com.univ.xml.TraitementImport;
import com.univ.xml.util.InfosImport;

public class FicheUnivTransformer {

    private static final Logger LOGGER = LoggerFactory.getLogger(FicheUnivTransformer.class);

    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    private static Object[] argVide = new Object[0];

    public static Vector<Hashtable<String, Serializable>> listeFiche2RPC(final ContexteUniv ctx, final Vector<?> lstFiche) throws Exception {
        // Structure principale de la liste de fiche : vector = <array>
        final Vector<Hashtable<String, Serializable>> lstRpc = new Vector<Hashtable<String, Serializable>>();
        final Enumeration<?> e = lstFiche.elements();
        while (e.hasMoreElements()) {
            final Object o = e.nextElement();
            // Structure secondaire par fiche : hashtable = <struct>
            final Hashtable<String, Serializable> ficheRPC = fiche2RPC(ctx, (FicheUniv) o);
            lstRpc.add(ficheRPC);
        }
        return lstRpc;
    }

    private static Hashtable<String, Serializable> fiche2RPC(final ContexteUniv ctx, final FicheUniv fiche) throws Exception {
        final Hashtable<String, Serializable> ficheRPC = new Hashtable<String, Serializable>();
        // première clé : nomFiche <string>,<string>
        final String codeObjet = ReferentielObjets.getCodeObjetParClasse(fiche.getClass().getName());
        ficheRPC.put("nomFiche", ReferentielObjets.getNomObjet(codeObjet));
        // deuxième clé : lstPropriete <string>,<struct>
        final Hashtable<String, String> lstPropriete = new Hashtable<String, String>();
        ficheRPC.put("lstPropriete", lstPropriete);
        final Map<String, Method> lstGetter = IntrospectionWrapper.getIntrospectionWrapper(fiche.getClass()).getLstGetter();
        for (final String nomParametre : lstGetter.keySet()) {
            final Method getter = lstGetter.get(nomParametre);
            final Object returnParam = getter.invoke(fiche, argVide);
            if (returnParam != null) {
                String valeur;
                if (returnParam instanceof java.sql.Date) {
                    valeur = sdf.format(new Date(((java.sql.Date) returnParam).getTime()));
                } else {
                    valeur = returnParam.toString();
                    if (valeur.indexOf('[') != -1) {
                        valeur = UnivWebFmt.formaterEnHTML(ctx, valeur);
                    }
                }
                lstPropriete.put(nomParametre, valeur);
            }
        }
        return ficheRPC;
    }

    public static Node listeFiche2DOM(final ContexteUniv ctx, final Vector<?> lstFiche) throws Exception {
        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        final DocumentBuilder builder = factory.newDocumentBuilder();
        final org.w3c.dom.Document document = builder.newDocument();
        final org.w3c.dom.Element noeudListe = document.createElement("ListeFiche");
        document.appendChild(noeudListe);
        // traitement de la structure principale
        final Enumeration<?> e = lstFiche.elements();
        while (e.hasMoreElements()) {
            final Object o = e.nextElement();
            // traitement de la structure secondaire par liste
            noeudListe.appendChild(fiche2DOM(ctx, (FicheUniv) o, noeudListe, ""));
        }
        return document;
    }

    private static Node fiche2DOM(final ContexteUniv ctx, final FicheUniv ficheUniv, final Node liste, final String formatExport) throws Exception {
        final String codeObjet = ReferentielObjets.getCodeObjetParClasse(ficheUniv.getClass().getName());
        final String nomFiche = ReferentielObjets.getNomObjet(codeObjet);
        final org.w3c.dom.Document document = liste.getOwnerDocument();
        final Node fiche = document.createElement(nomFiche);
        // Node attribut = null;
        final Map<String, Method> lstGetter = IntrospectionWrapper.getIntrospectionWrapper(ficheUniv.getClass()).getLstGetter();
        for (final String nomParametre : lstGetter.keySet()) {
            final Method getter = lstGetter.get(nomParametre);
            final Object returnParam = getter.invoke(ficheUniv, argVide);
            if (returnParam != null) {
                String valeur;
                if (returnParam instanceof java.sql.Date) {
                    valeur = sdf.format(new Date(((java.sql.Date) returnParam).getTime()));
                } else {
                    valeur = returnParam.toString();
                    if (valeur.indexOf('[') != -1) {
                        valeur = UnivWebFmt.formaterEnHTML(ctx, valeur);
                    }
                }
                final Element element = document.createElement(nomParametre);
                final Text texte = document.createTextNode(valeur);
                element.appendChild(texte);
                fiche.appendChild(element);
            }
        }
        return fiche;
    }

    public static Vector<Hashtable<String, Serializable>> listeDOM2RPC(final Node lstFiche) throws Exception {
        // Structure principale de la liste de fiche : vector = <array>
        final Vector<Hashtable<String, Serializable>> lstRpc = new Vector<Hashtable<String, Serializable>>();
        final NodeList nl = lstFiche.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            final Node fiche = nl.item(i);
            // Structure secondaire par fiche : hashtable = <struct>
            final Hashtable<String, Serializable> ficheRPC = DOM2RPC(fiche);
            lstRpc.add(ficheRPC);
        }
        return lstRpc;
    }

    private static Hashtable<String, Serializable> DOM2RPC(final Node fiche) throws Exception {
        final Hashtable<String, Serializable> ficheRPC = new Hashtable<String, Serializable>();
        // première clé : nomFiche <string>,<string>
        ficheRPC.put("nomFiche", fiche.getNodeName());
        // deuxième clé : lstPropriete <string>,<struct>
        final Hashtable<String, String> lstPropriete = new Hashtable<String, String>();
        ficheRPC.put("lstPropriete", lstPropriete);
        final NodeList nl = fiche.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            final Node propriete = nl.item(i);
            String valeurPropriete = "";
            final NodeList noeuds = propriete.getChildNodes();
            for (int k = 0; k < noeuds.getLength(); k++) {
                valeurPropriete += noeuds.item(k).getNodeValue();
            }
            lstPropriete.put(propriete.getNodeName(), valeurPropriete);
        }
        return ficheRPC;
    }

    public static Document listeRPC2DOM(final Vector<?> lstFicheRPC) throws Exception {
        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        final DocumentBuilder builder = factory.newDocumentBuilder();
        final org.w3c.dom.Document document = builder.newDocument();
        final org.w3c.dom.Element noeudListe = document.createElement("ListeFiche");
        document.appendChild(noeudListe);
        // traitement de la structure principale
        final Enumeration<?> e = lstFicheRPC.elements();
        while (e.hasMoreElements()) {
            final Object o = e.nextElement();
            // traitement de la structure secondaire par liste
            noeudListe.appendChild(RPC2DOM((Hashtable<?, ?>) o, noeudListe, ""));
        }
        return document;
    }

    private static Node RPC2DOM(final Hashtable<?, ?> hashtable, final Node liste, final String formatExport) throws Exception {
        final String nomFiche = (String) hashtable.get("nomFiche");
        final org.w3c.dom.Document document = liste.getOwnerDocument();
        final Node fiche = document.createElement(nomFiche);
        // Node attribut = null;
        final Hashtable<String, String> lstPropriete = (Hashtable<String, String>) hashtable.get("lstPropriete");
        final Enumeration<String> keys = lstPropriete.keys();
        while (keys.hasMoreElements()) {
            final String nomPropriete = keys.nextElement();
            final String valeur = lstPropriete.get(nomPropriete);
            final Element element = document.createElement(nomPropriete);
            final Text texte = document.createTextNode(valeur);
            element.appendChild(texte);
            fiche.appendChild(element);
        }
        return fiche;
    }

    public static Vector<FicheUnivImport> listeDOM2Fiche(final Node document, final OMContext _ctx, final String _sourceSynchronisation) throws Exception {
        final Vector<FicheUnivImport> lstInstanceFiche = new Vector<FicheUnivImport>();
        final NodeList lstNoeudFiche = document.getFirstChild().getChildNodes();
        // Parcours des noeuds de fiches
        for (int i = 0; i < lstNoeudFiche.getLength(); i++) {
            final Node noeudFiche = lstNoeudFiche.item(i);
            final FicheUnivImport ficheUnivAvecRessourcesAssociees = DOM2Fiche(noeudFiche, _ctx, _sourceSynchronisation);
            if (ficheUnivAvecRessourcesAssociees != null) {
                lstInstanceFiche.add(ficheUnivAvecRessourcesAssociees);
            }
        }
        // renvoie la liste des fiches instanciées
        return lstInstanceFiche;
    }

    private static FicheUnivImport DOM2Fiche(final Node noeudFiche, final OMContext _ctx, final String _sourceSynchronisation) throws Exception {
        final Object[] lstParam = new Object[1];
        String nomPropriete = "", nomSousObjet = "", nomAccesseur, classeXMLSousObjet, nomDeclarationObjet;
        String[] tokensSousObjet;
        final Class<?>[] classesParametres = new Class[1];
        Class<?> classeDeclarationObjet;
        classesParametres[0] = ArrayList.class;
        final String nomObjet = noeudFiche.getNodeName().toUpperCase();
        // instanciation de la fiche
        final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(nomObjet);
        final FicheUnivImport ficheUnivImport = new FicheUnivImport();
        String mediathequePath = "";
        if (ficheUniv != null) {
            ficheUniv.init();
            ficheUnivImport.setFicheUniv(ficheUniv);
            // récupération des informations sur la fiche
            final IntrospectionWrapper wrapper = IntrospectionWrapper.getIntrospectionWrapper(ficheUniv.getClass());
            // Parcours des proprietes de la fiche
            final NodeList lstNoeudPropriete = noeudFiche.getChildNodes();
            for (int j = 0; j < lstNoeudPropriete.getLength(); j++) {
                // récupération des propriétés de la fiche
                final Node noeudPropriete = lstNoeudPropriete.item(j);
                nomPropriete = noeudPropriete.getNodeName();
                nomAccesseur = ImportFicheIntrospection.formatNomTag(nomPropriete);
                // Si au moins un des noeuds fils n'est pas de type TEXT, alors il s'agit d'une sous-arborescence et donc d'un sous-objet
                nomSousObjet = "";
                Node fils = noeudPropriete.getFirstChild();
                while (fils != null) {
                    if (fils.getNodeType() != Node.TEXT_NODE && fils.getNodeType() != Node.CDATA_SECTION_NODE) {
                        nomSousObjet = fils.getNodeName();
                    }
                    fils = fils.getNextSibling();
                }
                if (nomSousObjet.length() > 0) {
                    // cas particulier de la liste de médias et ressourcesrattachés à la fiche
                    if (nomSousObjet.equalsIgnoreCase("mediathequePath")) {
                        mediathequePath = noeudPropriete.getChildNodes().item(0).getTextContent();
                    } else if (nomSousObjet.equalsIgnoreCase("mediaRessource")) {
                        final NodeList mediaNodesList = noeudPropriete.getChildNodes();
                        for (int i = 0; i < mediaNodesList.getLength(); i++) {
                            final MediaBean media = new MediaBean();
                            final RessourceBean ressource = new RessourceBean();
                            final Node mediaNode = mediaNodesList.item(i);
                            final NodeList mediaElements = mediaNode.getChildNodes();
                            for (int k = 0; k < mediaElements.getLength(); k++) {
                                final Node mediaElement = mediaElements.item(k);
                                final String setterName = "set" + ImportFicheIntrospection.formatNomTag(mediaElement.getNodeName());
                                final Method mediaSetter = findSetter(MediaBean.class, setterName);
                                final Method ressourceSetter = findSetter(RessourceBean.class, setterName);
                                if (mediaSetter != null && StringUtils.isNotBlank(mediaElement.getTextContent())) {
                                    invokeSetter(media, mediaSetter, Chaine.encodeUTF8(mediaElement.getTextContent()));
                                }
                                if (ressourceSetter != null && StringUtils.isNotBlank(mediaElement.getTextContent())) {
                                    invokeSetter(ressource, ressourceSetter, Chaine.encodeUTF8(mediaElement.getTextContent()));
                                }
                            }
                            final RessourceEtMediaAssocie ressourceEtMediaAssocie = new RessourceEtMediaAssocie();
                            ressourceEtMediaAssocie.setMedia(media);
                            ressourceEtMediaAssocie.setRessource(ressource);
                            ficheUnivImport.getRessourcesEtMediasAssocies().add(ressourceEtMediaAssocie);
                        }
                    } else {
                        // On détermine la classe du sous-objet
                        final List<String> classNames = new ArrayList<>();
                        for (final IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
                            if (plugin.isActive(ficheUniv.getClass().getName())) {
                                if (plugin.getObjets() != null) {
                                    classNames.addAll(plugin.getObjets());
                                }
                            }
                        }
                        for (final String name : classNames) {
                            classeDeclarationObjet = Class.forName(name);
                            // Le nom de la classe doit correspondre au nom dans le fichier XML
                            tokensSousObjet = nomSousObjet.split("_", -2);
                            classeXMLSousObjet = "";
                            for (final String element : tokensSousObjet) {
                                classeXMLSousObjet += element.substring(0, 1).toUpperCase() + element.substring(1).toLowerCase();
                            }
                            nomDeclarationObjet = classeDeclarationObjet.getName();
                            if (nomDeclarationObjet.lastIndexOf('.') != -1) {
                                nomDeclarationObjet = nomDeclarationObjet.substring(nomDeclarationObjet.lastIndexOf('.') + 1);
                            }
                            if (nomDeclarationObjet.equals(classeXMLSousObjet)) {
                                // Appel de l'import du sous-objet
                                final XMLInputFactory xmlif = XMLInputFactory.newInstance();
                                xmlif.setProperty(XMLInputFactory.IS_COALESCING, true);
                                // Create dom source for the document
                                final DOMSource domSource = new DOMSource(noeudPropriete);
                                // Create the result stream for the transform
                                final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
                                final StreamResult result = new StreamResult(outputStream);
                                // Create a Transformer to serialize the document
                                final TransformerFactory tFactory = TransformerFactory.newInstance();
                                final Transformer transformer = tFactory.newTransformer();
                                transformer.transform(domSource, result);
                                final XMLEventReader xmlr = xmlif.createXMLEventReader(new ByteArrayInputStream(outputStream.toByteArray()));
                                final TraitementImport traitementImport = new TraitementImport(new InfosImport());
                                ficheUnivImport.getPlugins().addAll(traitementImport.importSousObjet(_ctx, nomObjet, nomSousObjet, classeDeclarationObjet, xmlr));
                            }
                        }
                    }
                } else {
                    final NodeList noeuds = noeudPropriete.getChildNodes();
                    String valeurPropriete = "";
                    for (int k = 0; k < noeuds.getLength(); k++) {
                        valeurPropriete += noeuds.item(k).getNodeValue();
                    }
                    if (nomPropriete.equals("META_DATE_SUPPRESSION")) {
                        final java.util.Date d = new SimpleDateFormat("yyyy-MM-dd").parse(valeurPropriete);
                        ficheUnivImport.setDateSuppression(d);
                    } else {
                        // s'il y a incohérence entre les propriétés reçues et celles des fiches, on sort
                        final Method setter = wrapper.getSetter(nomAccesseur);
                        if (setter == null) {
                            continue;
                        }
                        final Class<?>[] lstClassParam = setter.getParameterTypes();
                        if (lstClassParam.length != 1) {
                            continue;
                        }
                        // Mapping des valeurs reçues vers les types de propriétés des fiches
                        if (lstClassParam[0].getName().equals("java.lang.Long")) {
                            lstParam[0] = new Long(valeurPropriete);
                        } else if (lstClassParam[0].getName().equals("java.lang.Integer")) {
                            lstParam[0] = new Integer(valeurPropriete);
                        } else if (lstClassParam[0].getName().equals("java.sql.Date")) {
                            try {
                                lstParam[0] = sdf.parse(valeurPropriete);
                            } catch (final Exception e) {
                                LOGGER.error("Echec parsing date", e);
                                lstParam[0] = new Date(0);
                            }
                        } else if (lstClassParam[0].getName().equals("java.lang.String")) {
                            // traitement de l'encodage
                            lstParam[0] = Chaine.encodeUTF8(valeurPropriete);
                        } else {
                            continue;
                        }
                        // insertion des propriétés
                        if (lstParam[0] != null) {
                            setter.invoke(ficheUniv, lstParam);
                        }
                    }
                }
            }
        }
        ficheUnivImport.setMediathequeDistantePath(mediathequePath);
        // renvoie la liste des fiches instanciées
        return ficheUnivImport;
    }

    private static void invokeSetter(final Object instance, final Method setter, final String value) {
        try {
            LOGGER.debug(instance.getClass().getName() + " " + setter.getName() + " : " + value);
            ReflectionUtils.invokeMethod(setter, instance, new Object[] {ConvertUtils.convert(value, setter.getParameterTypes()[0])});
        } catch (final ConversionException e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    private static Method findSetter(final Class<?> clazz, final String setterName) {
        Method result = null;
        for (final Class<?> paramType : Arrays.asList(new Class[] {String.class, Long.class})) {
            result = ReflectionUtils.findMethod(clazz, setterName, new Class[] {paramType});
            if (result != null) {
                break;
            }
        }
        return result;
    }

    /**
     * Formate la chaine String au format XML (Ote les /)
     *
     * @param _nomTag
     *            : nom du tag vérifier
     * @return Renvoi la chaine modifiée
     */
    public static Vector<FicheUniv> listeRPC2Fiche(final Vector<?> lstFicheRPC) throws Exception {
        final Vector<FicheUniv> lstInstanceFiche = new Vector<FicheUniv>();
        final Enumeration<?> e = lstFicheRPC.elements();
        while (e.hasMoreElements()) {
            final Object o = e.nextElement();
            // traitement de la structure secondaire par liste
            final FicheUniv ficheUniv = RPC2Fiche((Hashtable<?, ?>) o);
            if (ficheUniv != null) {
                lstInstanceFiche.add(ficheUniv);
            }
        }
        return lstInstanceFiche;
    }

    private static FicheUniv RPC2Fiche(final Hashtable<?, ?> hashtable) throws Exception {
        final Object[] lstParam = new Object[1];
        final String nomFiche = (String) (hashtable.get("nomFiche"));
        // instanciation de la fiche
        final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(nomFiche);
        if (ficheUniv == null) {
            LOGGER.error("FicheUnivTransformer.RPC2Fiche -> Fiche non instanciée : " + nomFiche);
            return null;
        }
        ficheUniv.init();
        // récupération des informations sur la fiche
        final IntrospectionWrapper wrapper = IntrospectionWrapper.getIntrospectionWrapper(ficheUniv.getClass());
        final Hashtable<String, String> lstPropriete = (Hashtable<String, String>) hashtable.get("lstPropriete");
        final Enumeration<String> keys = lstPropriete.keys();
        while (keys.hasMoreElements()) {
            final String nomPropriete = keys.nextElement();
            final String valeurPropriete = lstPropriete.get(nomPropriete);
            // s'il y a incohérence entre les propriétés reçues et celles des
            // fiches, on sort
            final Method setter = wrapper.getSetter(nomPropriete);
            if (setter == null) {
                break;
            }
            final Class<?>[] lstClassParam = setter.getParameterTypes();
            if (lstClassParam.length != 1) {
                break;
            }
            // Mapping des String reçues vers les types de propriétés des fiches
            if (lstClassParam[0].getName().equals("java.lang.Long")) {
                lstParam[0] = new Long(valeurPropriete);
            } else if (lstClassParam[0].getName().equals("java.lang.Integer")) {
                lstParam[0] = new Integer(valeurPropriete);
            } else if (lstClassParam[0].getName().equals("java.sql.Date")) {
                try {
                    lstParam[0] = sdf.parse(valeurPropriete);
                } catch (final Exception e) {
                    LOGGER.error("Echec transformation date", e);
                    lstParam[0] = new Date(0);
                }
            } else if (lstClassParam[0].getName().equals("java.lang.String")) {
                lstParam[0] = valeurPropriete;
            } else {
                break;
            }
            // insertion des propriétés
            setter.invoke(ficheUniv, lstParam);
        }
        return ficheUniv;
    }
}
