/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Node;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.controllers.SGController;
import com.kosmos.importexport.util.ExtensionUtil;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.config.PropertyHelper;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.kportal.extension.module.plugin.objetspartages.om.PluginBean;
import com.univ.objetspartages.bean.MediaBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.om.FicheObjet;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.services.ServiceMedia;
import com.univ.objetspartages.services.ServiceRessource;
import com.univ.objetspartages.util.MediaUtils;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.pws.om.FicheUnivImport;
import com.univ.pws.om.RessourceEtMediaAssocie;
import com.univ.pws.om.Synchronisation;
import com.univ.utils.FicheUnivMgr;
import com.univ.utils.URLResolver;

public class ImportFicheIntrospection {

    private static final Logger LOGGER = LoggerFactory.getLogger(ImportFicheIntrospection.class);

    public ImportFicheIntrospection() {
        super();
    }

    /*
     * IMPORT des fiches recues dans le DOM XML Via le WEB Service
     */
    public static String importNode(final OMContext _ctx, final Node _rootElement, final Synchronisation synchronisation) throws Exception {
        Vector<FicheUnivImport> vFiche = null;
        int sNbreElementNonImporte = 0; // Nombre d'élement Traité
        int sNbreElementAjoute = 0; // Nombre d'élement Traité
        int sNbreElementUpdate = 0; // Nombre d'élement Traité
        int sNbreElementSynchronis = 0; // Nombre d'élement Stynchronise
        String inTree = ExtensionUtil.getProperty("import.structures.intree");
        if (inTree == null || inTree.equals("")) {
            inTree = "1";
        }
        String requeteUpperCase = synchronisation.getRequete().toUpperCase();
        requeteUpperCase = StringUtils.replace(requeteUpperCase, "%20", " ");
        final String sourceSynchronisation = synchronisation.getSource();
        // Parcours des fiches pour insertion en base
        vFiche = FicheUnivTransformer.listeDOM2Fiche(_rootElement, _ctx, sourceSynchronisation);
        // Calcul du cookie de session
        String authCookie = null;
        try {
            authCookie = getAuthCookie(synchronisation.getSourceSynchronisation().getUrl(), synchronisation.getSourceSynchronisation().getLogin(), synchronisation.getSourceSynchronisation().getPassword(), _ctx);
        } catch (final IOException e) {
            LOGGER.error(e.getMessage(), e);
        }
        final ArrayList<String> lstFicheExistante = new ArrayList<String>();
        FicheUniv fiche, ficheUniv2;
        String nomObjet;
        String sourceFiche = sourceSynchronisation;
        String metaDescription;
        boolean ficheExiste, ficheUpdatee;
        final Enumeration<FicheUnivImport> e = vFiche.elements();
        while (e.hasMoreElements()) {
            sNbreElementSynchronis++; // Nombre d'élement Traité
            final FicheUnivImport ficheUnivImport = e.nextElement();
            fiche = ficheUnivImport.getFicheUniv();
            fiche.setCtx(_ctx);
            nomObjet = ReferentielObjets.getNomObjet(ReferentielObjets.getCodeObjetParClasse(fiche.getClass().getName()));
            // Le parametre HWS_ORIGINE doit être le même que le préfixe du code.
            // si l'un des deux n'est pas renseigné on le renseigne avec la source de synchronisation
            metaDescription = fiche.getMetaDescription();
            final int idx = metaDescription.indexOf("[HWS_ORIGINE=");
            if (idx != -1) {
                sourceFiche = metaDescription.substring(idx + 13, metaDescription.indexOf("]", idx + 13));
            }
            if (metaDescription.indexOf("[HWS_ORIGINE=" + sourceFiche + "]") == -1) {
                fiche.setMetaDescription("[HWS_ORIGINE=" + sourceFiche + "]" + metaDescription);
            }
            if (fiche.getCode().indexOf(sourceFiche + "_") == -1) {
                fiche.setCode(sourceFiche + "_" + fiche.getCode());
            }
            fiche.setIdFiche((long) 0);// nouvelle fiche
            ficheUniv2 = (fiche.getClass().newInstance());
            ficheUniv2.init();
            ficheUniv2.setCtx(_ctx);
            ficheExiste = false;
            ficheUpdatee = false;
            // Si il y a d'autre fiche dans d'autre ETAT
            if (ficheUniv2.selectCodeLangueEtat(fiche.getCode(), fiche.getLangue(), fiche.getEtatObjet()) > 0) {
                while (ficheUniv2.nextItem()) {
                    if (ficheUniv2.getDateModification().getTime() > fiche.getDateModification().getTime()) {
                        // fiche existante + récente : on ne fait rien, mais on note qu'elle existe
                        ficheExiste = true;
                        lstFicheExistante.add(nomObjet + ";" + ficheUniv2.getIdFiche());
                        sNbreElementNonImporte++;
                    } else {
                        ficheUpdatee = true;
                        // suppression de l'ancienne fiche, elle sera réajoutée ensuite (et aussi réindexée)
                        FicheUnivMgr.supprimerFiche(ficheUniv2, true);
                    }
                }
            }
            if (!ficheExiste) {
                fiche.setIdFiche((long) 0);
                fiche.add();
                if (ficheUpdatee) {
                    sNbreElementUpdate++;
                } else {
                    sNbreElementAjoute++;
                }
                final String etatFiche = fiche.getEtatObjet();
                final MetatagBean meta = MetatagUtils.lireMeta(fiche);
                if (etatFiche.equals("0003")) {
                    if (meta.getId() == 0L) {
                        meta.setMetaInTree(inTree);
                    }
                    meta.setMetaDateSuppression(ficheUnivImport.getDateSuppression());
                    meta.setMetaSourceImport(sourceFiche);
                    MetatagUtils.getMetatagService().save(meta);
                    FicheUnivMgr.synchroniserMetaTag(fiche, "");
                }
                // remplacement du code des fiches dans les objets liés
                for (final PluginBean objet : ficheUnivImport.getPlugins()) {
                    if (objet instanceof FicheObjet) {
                        if (((FicheObjet) objet).getCode().indexOf(sourceFiche + "_") == -1) {
                            ((FicheObjet) objet).setCode(sourceFiche + "_" + ((FicheObjet) objet).getCode());
                        }
                    }
                }
                // enregistrement des objets liés
                PluginFicheHelper.ajouterPlugins(fiche, meta, ficheUnivImport.getPlugins(), null);
                lstFicheExistante.add(nomObjet + ";" + fiche.getIdFiche());
                importRessourcesEtMedias(ficheUnivImport, synchronisation.getSourceSynchronisation().getUrl(), authCookie);
            }
        }
        String sMsg = "Synchronisation de " + sNbreElementSynchronis + " fiche(s)";
        sMsg += "\r\n\t > " + sNbreElementAjoute + " fiche(s) ajoutée(s).";
        sMsg += "\r\n\t > " + sNbreElementUpdate + " fiche(s) mise(s) à jour.";
        sMsg += "\r\n\t > " + sNbreElementNonImporte + " fiche(s) ignorée(s) (fiche existante plus récente).";
        // suppression des fiches non synchronisées
        if (vFiche.size() > 0) {
            final FicheUnivImport ficheUnivAvecRessourcesAssociees = vFiche.firstElement();
            fiche = ficheUnivAvecRessourcesAssociees.getFicheUniv();
            nomObjet = ReferentielObjets.getNomObjet(ReferentielObjets.getCodeObjetParClasse(fiche.getClass().getName()));
            sMsg += "\r\n\t > " + deleteFicheNonSyncho(_ctx, synchronisation, lstFicheExistante, nomObjet) + " fiche(s) supprimée(s).";
        }
        return sMsg;
    }

    /**
     * Insere en base un media et sa ressource provenant du site de synchronisation source et télécharge le fichier correspondant
     *
     * @param ficheUnivAvecRessourcesAssociees
     *            Aggrégat contenant la fiche univ, ses ressources et leurs medias
     */
    private static void importRessourcesEtMedias(final FicheUnivImport ficheUnivAvecRessourcesAssociees, final String urlSynchro, final String authCookie) {
        final ServiceMedia serviceMedia = ServiceManager.getServiceForBean(MediaBean.class);
        final ServiceRessource serviceRessource = ServiceManager.getServiceForBean(RessourceBean.class);
        for (final RessourceEtMediaAssocie ressourceEtMediaAssocie : ficheUnivAvecRessourcesAssociees.getRessourcesEtMediasAssocies()) {
            try {
                // téléchargement du média sur le serveur source
                final int retour = telechargerFicherMedia(ressourceEtMediaAssocie.getMedia(), ressourceEtMediaAssocie.getRessource(), urlSynchro, authCookie, ficheUnivAvecRessourcesAssociees.getMediathequeDistantePath());
                // import de la ressource et du média en base
                if (retour != -1) {
                    if (retour == 1) {
                        ressourceEtMediaAssocie.getMedia().setId((long) 0);
                        serviceMedia.add(ressourceEtMediaAssocie.getMedia());
                    }
                    ressourceEtMediaAssocie.getRessource().setId((long) 0);
                    ressourceEtMediaAssocie.updateRessourceIdMedia(ressourceEtMediaAssocie.getMedia().getId());
                    ressourceEtMediaAssocie.updateRessourceCodeParent(ficheUnivAvecRessourcesAssociees.getFicheUniv().getIdFiche());
                    serviceRessource.save(ressourceEtMediaAssocie.getRessource());
                }
            } catch (final Exception e) {
                LOGGER.error(e.getMessage(), e);
            }
        }
    }

    /**
     * Telecharge un fichier média depuis la mediateque du site source, dans la mediatheque de ce site
     *
     * @param media
     *            L'objet media
     * @param ressource
     *            L'objet ressource lié à l'objet media
     * @param sourceHostUrl
     *            L'url du site source scheme://hostname:port/
     * @param authCookie
     *            Un string contenant le cookie d'authentification au site distant
     * @throws IOException
     */
    private static int telechargerFicherMedia(final MediaBean media, final RessourceBean ressource, final String sourceHostUrl, final String authCookie, final String mediathequeDistantePath) throws IOException {
        // -1 erreur
        // 0 existe non telecharge
        // 1 telecharge
        if (!(ressource.getCodeParent().equals("") || ressource.getCodeParent().startsWith("TYPE") || ressource.getEtat().equals("0") || ressource.getEtat().equals("2"))) {
            final File fichierDestination = new File(MediaUtils.getPathAbsolu(media));
            if (!fichierDestination.exists()) {
                final URL url = new URL(sourceHostUrl + MediaUtils.getUrlAbsolue(media));
                final URLConnection urlConnection = url.openConnection();
                if (StringUtils.isNotBlank(authCookie)) {
                    urlConnection.setRequestProperty("Cookie", authCookie);
                }
                final int responseCode = ((HttpURLConnection) urlConnection).getResponseCode();
                if (responseCode == HttpURLConnection.HTTP_OK) {
                    LOGGER.debug("Synchronisation du fichier : " + url.toString());
                    final BufferedInputStream bis = new BufferedInputStream(urlConnection.getInputStream());
                    FileUtils.writeByteArrayToFile(fichierDestination, IOUtils.toByteArray(bis));
                    return 1;
                } else {
                    LOGGER.error("Erreur téléchargement du fichier : " + url.toString() + ", code=" + responseCode);
                    return -1;
                }
            }
        }
        return 0;
    }

    /**
     * Retourne un cookie d'authentification pour le site source de la synchronisation
     *
     * @param host
     *            L'url du site source (scheme://hostname:port/)
     * @param user
     *            Le nom d'utilisateur avec lequel s'authentifier
     * @param password
     *            Le password avec lequel s'authentifier
     * @param ctx
     *            Le contexte
     * @return Un string contenant le cookie d'authentification
     * @throws IOException
     */
    private static String getAuthCookie(final String host, final String user, final String password, final OMContext ctx) throws IOException {
        final String sUrl = URLResolver.getAbsoluteUrl(host + PropertyHelper.getCoreProperty(SGController.URL_PROPERTY) + "?PROC=IDENTIFICATION&ACTION=VALIDER&KUSER_CONNECTOR=1&LOGIN=" + user + "&PASSWORD=" + password, ctx);
        final URL url = new URL(sUrl);
        final URLConnection urlConnection = url.openConnection();
        final InputStream is = urlConnection.getInputStream();
        final String leCookie = urlConnection.getHeaderField("Set-Cookie");
        is.close();
        return leCookie;
    }

    /**
     *
     * @param _ctx
     */
    private static int deleteFicheNonSyncho(final OMContext _ctx, final Synchronisation synchronisation, final ArrayList<String> lstFicheExistante, final String nomObjet) throws Exception {
        int nbSuppression = 0;
        final FicheUniv myFiche = ReferentielObjets.instancierFiche(nomObjet);
        myFiche.setCtx(_ctx);
        myFiche.init();
        // si on synchronise les autres sources, on récupère toutes les fiches
        // issues de hws sauf notre source actuelle
        // sinon on récupère uniquement la source en cours
        String sourceLocale = ExtensionUtil.getProperty("hws.source");
        if (sourceLocale == null) {
            sourceLocale = "";
        }
        String myClause = "";
        if (synchronisation.getIndicateurSynchrosAutres().equals("0")) {
            myClause = "WHERE META_DESCRIPTION LIKE '%[HWS_ORIGINE=" + synchronisation.getSource() + "]%';";
        } else {
            myClause = "WHERE META_DESCRIPTION LIKE '%[HWS_ORIGINE=%' AND META_DESCRIPTION NOT LIKE '%[HWS_ORIGINE=" + sourceLocale + "]%';";
        }
        try {
            if (myFiche.select(myClause) > 0) {
                while (myFiche.nextItem()) {
                    if (lstFicheExistante.contains(nomObjet + ";" + myFiche.getIdFiche()) == false) {
                        FicheUnivMgr.supprimerFiche(myFiche, true);
                        nbSuppression++;
                    }
                }
            }
        } catch (final Exception e) {
            LOGGER.error("Erreur lors de la synchronisation", e);
        }
        return nbSuppression;
    }

    /**
     * Formate le nom du TAG
     *
     * @param _nomTag
     *            : nom du tag vérifier
     * @return Renvoi la chaine modifiée
     */
    public static String formatNomTag(final String _nomTag) {
        String nomTag = _nomTag;
        nomTag = nomTag.replace("__", "_");
        String newTag = new String("");
        for (int i = 0; i < nomTag.length(); i++) {
            if (i == 0) {
                newTag = newTag.concat(nomTag.substring(i, i + 1).toUpperCase());
            } else {
                if (nomTag.substring(i, i + 1).compareTo("_") == 0) {
                    newTag = newTag.concat(nomTag.substring(i + 1, i + 2).toUpperCase());
                    i++;
                } else {
                    newTag = newTag.concat(nomTag.substring(i, i + 1).toLowerCase());
                }
            }
        }
        return newTag;
    }
}
