/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws.client;

import java.io.File;

import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kosmos.importexport.util.ExtensionUtil;
import com.univ.pws.SynchroXmlRpcClient;
import com.univ.pws.om.SourceSynchronisation;
import com.univ.pws.om.Synchronisation;
import com.univ.utils.ContexteDao;

/**
 * Classe helper permettant de lancer la synchronisation
 */
public class SynchronisationClient implements Job {

    private static final Logger LOGGER = LoggerFactory.getLogger(SynchronisationClient.class);

    /**
     * Lance la synchronisation
     *
     * @throws Exception
     */
    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        Synchronisation synchronisation = new Synchronisation();
        synchronisation.init();
        try (ContexteDao contexteDao = new ContexteDao()) {
            synchronisation.setCtx(contexteDao);
            if (synchronisation.select("") > 0) {
                // Première étape, appel du webservice pour récupérer les
                // fichiers d'import
                while (synchronisation.nextItem()) {
                    if ("1".equals(synchronisation.getIndicateurBatch())) {
                        SynchroXmlRpcClient pwsClient = new SynchroXmlRpcClient();
                        pwsClient.setSynchronisation(synchronisation);
                        Thread sync = new Thread(pwsClient);
                        sync.start();
                        try {
                            sync.join();
                        } catch (Exception e) {
                            LOGGER.error("Erreur lors de la synchronisation de contenu " + synchronisation.getLibelle(), e);
                        }
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
        // deuxième étape, import des fichiers
        File repertoireImport = new File(ExtensionUtil.getHwsDefaultPath());
        if (repertoireImport.exists() && repertoireImport.isDirectory()) {
            for (File repertoireSource : repertoireImport.listFiles()) {
                // boucle sur les dossier source
                if (repertoireSource.isDirectory()) {
                    String source = repertoireSource.getName();
                    File[] lstFichierImport = repertoireSource.listFiles();
                    // boucle sur les fichiers de chaque source
                    for (File fichierImport : lstFichierImport) {
                        synchronisation.setSource(source);
                        SourceSynchronisation sourceSynchronisation = synchronisation.getSourceSynchronisation();
                        if (sourceSynchronisation == null) {
                            LOGGER.warn("Source de synchronisation de contenu ignorée : " + source);
                        } else {
                            synchronisation.setFichierSynchro(fichierImport);
                            synchronisation.setLibelle(synchronisation.getSourceSynchronisation().getLibelle() + " : Fichier " + fichierImport.getName());
                            SynchroXmlRpcClient pwsClient = new SynchroXmlRpcClient();
                            pwsClient.setSynchronisation(synchronisation);
                            Thread sync = new Thread(pwsClient);
                            sync.start();
                            try {
                                sync.join();
                            } catch (Exception e) {
                                LOGGER.error("Erreur lors de la synchronisation du fichier " + fichierImport.getName(), e);
                            }
                        }
                    }
                }
            }
        }
    }
}
