/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws.processus;

import java.util.Hashtable;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.kosmos.importexport.util.ExtensionUtil;
import com.kportal.core.config.MessageHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.pws.SynchroXmlRpcClient;
import com.univ.pws.om.SourceSynchronisation;
import com.univ.pws.om.Synchronisation;

/**
 * processus saisie Synchronisation
 */
public class SaisieSynchronisation extends ProcessusBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(SaisieSynchronisation.class);

    private static final int MODE_AJOUT = 0;

    private static final int MODE_MODIFICATION = 1;

    private static final String ECRAN_PRINCIPAL = "PRINCIPAL";

    private static final String ECRAN_ACCUEIL = "LISTE";

    private static final String ACTION_ACCUEIL = "ACCUEIL";

    private static final String ACTION_RECHARGER = "RECHARGER";

    private static Hashtable<String, String> lstTypeSource = null;

    private static Hashtable<String, SourceSynchronisation> lstSource = null;

    private int mode = -1;

    private Synchronisation synchronisation = null;

    private AutorisationBean autorisations = null;

    /**
     * processus saisie Synchronisation
     *
     * @param ciu
     *            com.jsbsoft.jtf.core.InfoBean
     */
    public SaisieSynchronisation(InfoBean ciu) {
        super(ciu);
    }

    /**
     * @return Returns the lstSource.
     */
    public static Hashtable<String, SourceSynchronisation> getLstSource() {
        checkInit();
        return lstSource;
    }

    /**
     * @return Returns the lstTypeSource.
     */
    public static Hashtable<String, String> getLstTypeSource() {
        checkInit();
        return lstTypeSource;
    }

    private static void checkInit() {
        if (lstTypeSource == null) {
            lstTypeSource = new Hashtable<String, String>();
            lstTypeSource.put(" ", "");
            lstSource = new Hashtable<String, SourceSynchronisation>();
            int i = 1;
            String param = null;
            while ((param = ExtensionUtil.getProperty("synchroxmlrpc.source" + i)) != null) {
                try {
                    String[] lstParam = param.split(";", -2);
                    SourceSynchronisation source = new SourceSynchronisation();
                    source.setCode(lstParam[0]);
                    source.setUrl(lstParam[1]);
                    source.setLibelle(lstParam[2]);
                    source.setLogin(lstParam[3]);
                    source.setPassword(lstParam[4]);
                    i++;
                    lstTypeSource.put(source.getCode(), source.getLibelle());
                    lstSource.put(source.getCode(), source);
                } catch (Exception e) {
                    LOGGER.error("Erreur dans le paramétrage des actions formulaire : " + param, e);
                }
            }
        }
    }

    /**
     * Point d'entree du processus
     */
    @Override
    public boolean traiterAction() throws Exception {
        autorisations = (AutorisationBean) getGp().getSessionUtilisateur().getInfos().get(SessionUtilisateur.AUTORISATIONS);
        if (autorisations == null) {
            infoBean.setEcranRedirection(WebAppUtil.getBoConnectionUrl());
            infoBean.setEcranLogique("LOGIN");
        } else {
            try {
                ecranLogique = infoBean.getEcranLogique();
                action = infoBean.getActionUtilisateur();
                etat = EN_COURS;
                if (ecranLogique == null) {
                    if (action.equals(ACTION_ACCUEIL)) {
                        preparerLISTE();
                    }
                    if (action.equals(ACTION_RECHARGER)) {
                        // on vide la table
                        lstTypeSource = null;
                        preparerLISTE();
                    }
                    if (action.equals("AJOUTER")) {
                        mode = MODE_AJOUT;
                        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_CREATION);
                        preparerPRINCIPAL();
                    }
                    if (action.equals("MODIFIER")) {
                        mode = MODE_MODIFICATION;
                        infoBean.setEtatObjet(InfoBean.ETAT_OBJET_MODIF);
                        preparerPRINCIPAL();
                    }
                    if (action.equals("SUPPRIMER")) {
                        initSynchronisation();
                        synchronisation.delete();
                        infoBean.addMessageConfirmation(MessageHelper.getExtensionMessage(infoBean.getNomExtension(), "CONFIRMATION_SUPPRESSION_SOURCE_SYNCHRONISATION"));
                        etat = FIN;
                    }
                    if (action.equals("LANCER")) {
                        traiterLANCEMENT();
                    }
                } else if (ecranLogique.equals(ECRAN_PRINCIPAL)) {
                    traiterPRINCIPAL();
                }
                // placer l'etat dans le composant d'infoBean
                infoBean.setEcranLogique(ecranLogique);
            } catch (Exception e) {
                LOGGER.error("Erreur lors du traitement de synchronisation", e);
                infoBean.addMessageErreur(e.toString());
            }
        }
        // On continue si on n'est pas Ã  la FIN !!!
        return etat == FIN;
    }

    /**
     * Affichage de la liste des Synchronisations
     */
    private void preparerLISTE() throws Exception {
        int i = 0;
        initSynchronisation();
        synchronisation.select("");
        while (synchronisation.nextItem()) {
            infoBean.set("ID#" + i, synchronisation.getIdSynchronisation());
            infoBean.set("LIBELLE#" + i, synchronisation.getLibelle());
            infoBean.set("REQUETE#" + i, synchronisation.getRequete());
            infoBean.set("SOURCE#" + i, synchronisation.getSource());
            infoBean.set("INDICATEUR_BATCH#" + i, synchronisation.getIndicateurBatch());
            infoBean.set("INDICATEUR_SYNCHROS_AUTRES#" + i, synchronisation.getIndicateurSynchrosAutres());
            if (synchronisation.getEtat() == Synchronisation.ETAT_EN_COURS) {
                infoBean.set("ITEM_EN_COURS", i);
            }
            i++;
        }
        infoBean.set("LISTE_NB_ITEMS", new Integer(i));
        ecranLogique = ECRAN_ACCUEIL;
    }

    /**
     * Affichage de l'écran de saisie d'un Synchronisation
     */
    private void preparerPRINCIPAL() throws Exception {
        initSynchronisation();
        infoBean.set("ID_SYNCHRONISATION", synchronisation.getIdSynchronisation());
        infoBean.set("LIBELLE", synchronisation.getLibelle());
        infoBean.set("REQUETE", synchronisation.getRequete());
        infoBean.set("SOURCE", synchronisation.getSource());
        infoBean.set("INDICATEUR_BATCH", synchronisation.getIndicateurBatch());
        infoBean.set("INDICATEUR_SYNCHROS_AUTRES", synchronisation.getIndicateurSynchrosAutres());
        ecranLogique = ECRAN_PRINCIPAL;
    }

    private void initSynchronisation() throws Exception {
        // initalisation objet
        synchronisation = new Synchronisation();
        synchronisation.setCtx(this);
        synchronisation.init();
        if (StringUtils.isNotEmpty(infoBean.getString("ID_SYNCHRONISATION"))) {
            synchronisation.setIdSynchronisation(new Long(infoBean.getString("ID_SYNCHRONISATION")));
            try {
                synchronisation.retrieve();
            } catch (Exception e) {
                throw new ErreurApplicative("Veuillez sélectionner une source valide");
            }
        }
    }

    /**
     * Gestion de l'écran de saisie d'un Synchronisation
     */
    private void traiterPRINCIPAL() throws Exception {
        if (action.equals(InfoBean.ACTION_VALIDER)) {
            synchronisation.setLibelle((String) infoBean.get("LIBELLE"));
            synchronisation.setRequete((String) infoBean.get("REQUETE"));
            synchronisation.setSource((String) infoBean.get("SOURCE"));
            synchronisation.setIndicateurBatch((String) infoBean.get("INDICATEUR_BATCH"));
            synchronisation.setIndicateurSynchrosAutres((String) infoBean.get("INDICATEUR_SYNCHROS_AUTRES"));
            if (mode == MODE_AJOUT) {
                synchronisation.add();
            }
            if (mode == MODE_MODIFICATION) {
                synchronisation.update();
            }
            infoBean.addMessageConfirmation(MessageHelper.getExtensionMessage(infoBean.getNomExtension(), "CONFIRMATION_VALIDATION_SOURCE_SYNCHRONISATION"));
        } else if (action.equals(InfoBean.ACTION_SUPPRIMER)) {
            synchronisation.delete();
            infoBean.addMessageConfirmation(MessageHelper.getExtensionMessage(infoBean.getNomExtension(), "CONFIRMATION_SUPPRESSION_SOURCE_SYNCHRONISATION"));
        }
        etat = FIN;
    }

    private void traiterLANCEMENT() throws Exception {
        SynchroXmlRpcClient client = new SynchroXmlRpcClient();
        initSynchronisation();
        client.setSynchronisation(this.synchronisation);
        Thread thread = new Thread(client);
        thread.start();
        etat = FIN;
    }
}
