/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.pws.sgbd;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

import com.jsbsoft.jtf.database.OMContext;

public class SynchronisationDB {

    protected String qualifier = null;

    protected OMContext ctx = null;

    private java.sql.PreparedStatement _stmt = null;

    private ResultSet _rs = null;

    private Long idSynchronisation = null;

    private String libelle = null;

    private String requete = null;

    private String source = null;

    private String indicateurBatch = null;

    private String indicateurSynchrosAutres = null;

    private int etat = 0;

    public SynchronisationDB() {
    }

    public void add() throws Exception {
        try {
            _stmt = getConnection().prepareStatement(" INSERT INTO " + qualifier + "SYNCHRONISATION (ID_SYNCHRONISATION  ,   LIBELLE  ,   REQUETE  ,   SOURCE  ,   INDICATEUR_BATCH  ,   INDICATEUR_SYNCHROS_AUTRES ,   ETAT )              VALUES (?  ,?  ,?  ,?  ,?  ,? ,?  ) ", Statement.RETURN_GENERATED_KEYS);
            _stmt.setObject(1, getIdSynchronisation(), Types.BIGINT);
            _stmt.setObject(2, getLibelle(), Types.VARCHAR);
            _stmt.setObject(3, getRequete(), Types.LONGVARCHAR);
            _stmt.setObject(4, getSource(), Types.VARCHAR);
            _stmt.setObject(5, getIndicateurBatch(), Types.VARCHAR);
            _stmt.setObject(6, getIndicateurSynchrosAutres(), Types.VARCHAR);
            _stmt.setObject(7, getEtat(), Types.INTEGER);
            int rowsAffected = _stmt.executeUpdate();
            if (rowsAffected == 0) {
                throw new Exception("METHOD_AFFCTD_NOROWS");
            }
            if (rowsAffected > 1) {
                throw new Exception("METHOD_AFFCTD_MULTROWS");
            }
            _rs = _stmt.getGeneratedKeys();
            _rs.next();
            setIdSynchronisation(_rs.getLong(1));
        } catch (SQLException exc) {
            throw new Exception("ERROR_IN_METHOD add() " + exc);
        } finally {
            if (_stmt != null) {
                _stmt.close();
            }
        }
    }

    public void delete() throws Exception {
        try {
            _stmt = getConnection().prepareStatement("DELETE FROM " + qualifier + "SYNCHRONISATION WHERE " + "             ID_SYNCHRONISATION = ? ");
            // put parameters into statement
            _stmt.setObject(1, getIdSynchronisation(), Types.BIGINT);
            int rowsAffected = _stmt.executeUpdate();
            if (rowsAffected == 0) {
                _stmt.close();
                throw new Exception("METHOD_AFFCTD_NOROWS");
            }
            if (rowsAffected > 1) {
                _stmt.close();
                throw new Exception("METHOD_AFFCTD_MULTROWS");
            }
            _stmt.close();
        } catch (SQLException exc) {
            throw new Exception("ERROR_IN_METHOD delete() " + exc);
        } finally {
            _stmt = null;
        }
    }

    private Connection getConnection() {
        return ctx.getConnection();
    }

    public Long getIdSynchronisation() {
        return idSynchronisation;
    }

    public void setIdSynchronisation(Long _idSynchronisation) {
        idSynchronisation = _idSynchronisation;
    }

    public String getLibelle() {
        return libelle;
    }

    public void setLibelle(String _libelle) {
        libelle = _libelle;
    }

    public String getRequete() {
        return requete;
    }

    public void setRequete(String _requete) {
        requete = _requete;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String _source) {
        source = _source;
    }

    public String getIndicateurBatch() {
        return indicateurBatch;
    }

    public void setIndicateurBatch(String _indicateurBatch) {
        indicateurBatch = _indicateurBatch;
    }

    public String getIndicateurSynchrosAutres() {
        return indicateurSynchrosAutres;
    }

    public void setIndicateurSynchrosAutres(String _indicateurSynchrosAutres) {
        indicateurSynchrosAutres = _indicateurSynchrosAutres;
    }

    public String getSQLBaseQuery() {
        return "SELECT DISTINCT " + "T1.ID_SYNCHRONISATION ,  " + "T1.LIBELLE ,  " + "T1.REQUETE ,  " + "T1.SOURCE ,  " + "T1.INDICATEUR_BATCH ,  " + "T1.INDICATEUR_SYNCHROS_AUTRES ,  " + "T1.ETAT " + "         FROM " + qualifier + "SYNCHRONISATION T1 ";
    }

    public boolean nextItem() throws Exception {
        boolean res = false;
        try {
            if (_rs.next()) {
                retrieveFromRS(_rs);
                res = true;
            }
        } catch (Exception e) {
            throw new Exception("ERROR_IN_METHOD nextItem() " + e);
        }
        return res;
    }

    /**
     * Récupération d'une ligne de la base de données.
     */
    public void retrieve() throws Exception {
        try {
            _stmt = getConnection().prepareStatement("SELECT " + "             ID_SYNCHRONISATION," + "             LIBELLE," + "             REQUETE," + "             SOURCE," + "             INDICATEUR_BATCH," + "             INDICATEUR_SYNCHROS_AUTRES," + "             ETAT" + "         FROM " + qualifier + "SYNCHRONISATION WHERE " + "             ID_SYNCHRONISATION = ? ");
            // put parameters into statement
            _stmt.setObject(1, getIdSynchronisation(), Types.BIGINT);
            ResultSet rs = _stmt.executeQuery();
            if (!rs.next()) {
                _stmt.close();
                throw new Exception("retrieve  : METHOD_NO_RESULTS");
            }
            // get output from result set
            retrieveFromRS(rs);
            _stmt.close();
        } catch (SQLException exc) {
            throw new Exception("ERROR_IN_METHOD retrieve() " + exc);
        } finally {
            _stmt = null;
        }
    }

    private void retrieveFromRS(ResultSet _rs) throws Exception {
        try {
            // get output from result set
            setIdSynchronisation(new Long(_rs.getLong(1)));
            setLibelle(_rs.getString(2));
            setRequete(_rs.getString(3));
            setSource(_rs.getString(4));
            setIndicateurBatch(_rs.getString(5));
            setIndicateurSynchrosAutres(_rs.getString(6));
            setEtat(_rs.getInt(7));
        } catch (SQLException exc) {
            throw new Exception("ERROR_IN_METHOD retrieveFromRS() " + exc);
        }
    }

    /**
     * cette méthode optimise l'exécution de la requete (pas de count)
     *
     * @param sqlSuffix
     * @throws Exception
     */
    public void selectNoCount(String sqlSuffix) throws Exception {
        try {
            if (sqlSuffix == null) {
                sqlSuffix = "";
            }
            _stmt = getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
            _rs = _stmt.executeQuery();
        } catch (SQLException exc) {
            throw new Exception("SELECT_FAILED " + exc);
        } finally {
            _stmt = null;
        }
    }

    public int select(String sqlSuffix) throws Exception {
        int count = 0;
        try {
            if (sqlSuffix == null) {
                sqlSuffix = "";
            }
            String query = "SELECT COUNT(*)   FROM " + qualifier + "SYNCHRONISATION T1 " + sqlSuffix;
            /* récupération nombre de lignes */
            _stmt = getConnection().prepareStatement(query);
            _rs = _stmt.executeQuery(query);
            _rs.next();
            count = _rs.getInt(1);
			/* éxécution requete */
            query = getSQLBaseQuery() + sqlSuffix;
            _stmt = getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
            _rs = _stmt.executeQuery();
        } catch (SQLException exc) {
            throw new Exception("SELECT_FAILED " + exc);
        } finally {
            _stmt = null;
        }
        return count;
    }

    public void setParam(OMContext _ctx, String _qualifier) {
        ctx = _ctx;
        qualifier = "";
        if (_qualifier.equals("") == false) {
            if (_qualifier.substring(_qualifier.length() - 1).equals(".") == false) {
                qualifier = _qualifier + ".";
            } else {
                qualifier = _qualifier;
            }
        }
    }

    @Override
    public String toString() {
        return toString(".");
    }

    public String toString(String aSeparator) {
        return String.valueOf("" + aSeparator + getIdSynchronisation() + aSeparator + getLibelle() + aSeparator + getRequete() + aSeparator + getSource() + aSeparator + getIndicateurBatch() + aSeparator + getIndicateurSynchrosAutres());
    }

    /**
     * Mise à jour d'une ligne de la base de données.
     */
    public void update() throws Exception {
        try {
            _stmt = getConnection().prepareStatement("UPDATE " + qualifier + "SYNCHRONISATION SET " + "ID_SYNCHRONISATION = ?, " + "LIBELLE = ?, " + "REQUETE = ?, " + "SOURCE = ?, " + "INDICATEUR_BATCH = ?, " + "INDICATEUR_SYNCHROS_AUTRES = ?, " + "ETAT = ? " + "         WHERE " + "             ID_SYNCHRONISATION = ? ");
            // put parameters into statement
            _stmt.setObject(1, getIdSynchronisation(), Types.BIGINT);
            _stmt.setObject(2, getLibelle(), Types.VARCHAR);
            _stmt.setObject(3, getRequete(), Types.LONGVARCHAR);
            _stmt.setObject(4, getSource(), Types.VARCHAR);
            _stmt.setObject(5, getIndicateurBatch(), Types.VARCHAR);
            _stmt.setObject(6, getIndicateurSynchrosAutres(), Types.VARCHAR);
            _stmt.setObject(7, getEtat(), Types.INTEGER);
            _stmt.setObject(8, getIdSynchronisation(), Types.BIGINT);
            _stmt.executeUpdate();
            _stmt.close();
        } catch (SQLException exc) {
            throw new Exception("ERROR_IN_METHOD update() " + exc);
        } finally {
            _stmt = null;
        }
    }

    public void setCtx(OMContext _ctx) {
        setParam(_ctx, "");
    }

    public int getEtat() {
        return etat;
    }

    public void setEtat(int etat) {
        this.etat = etat;
    }
}
