/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.synchro;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.ContexteUniv;
import com.univ.utils.IntrospectionWrapper;
// TODO: Auto-generated Javadoc

/**
 * The Class ExportFicheHandler.
 */
public class ExportFicheHandler extends KPortalExporter {

    /** The Constant FORMAT_FICHE. */
    public static final int FORMAT_FICHE = 0;

    /** The Constant FORMAT_DOM. */
    public static final int FORMAT_DOM = 1;

    /** The remote procedure. */
    public static String remoteProcedure = "ExportFicheHandler.execute";

    /** The remote servlet. */
    public static String remoteServlet = "/servlet/com.univ.synchro.SynchroXmlRpcServlet";

    /** The remote host. */
    private String remoteHost = null;

    /** The format. */
    private int format = FORMAT_FICHE;

    /**
     * Instantiates a new export fiche handler.
     */
    public ExportFicheHandler() {
        super();
    }

    /**
     * Instantiates a new export fiche handler.
     *
     * @param host
     *            the host
     */
    public ExportFicheHandler(final String host) {
        //constructeur pour un appel distant, autrement c'est local
        super();
        remoteHost = host;
    }

    /**
     * Instantiates a new export fiche handler.
     *
     * @param host
     *            the host
     * @param newFormat
     *            the new format
     */
    public ExportFicheHandler(final String host, final int newFormat) {
        //constructeur pour un appel distant, autrement c'est local
        super();
        remoteHost = host;
        format = newFormat;
    }

    /**
     * Execute.
     *
     * @param user
     *            the user
     * @param password
     *            the password
     * @param requete
     *            the requete
     *
     * @return the vector
     *
     * @throws XmlRpcException
     *             the xml rpc exception
     */
    public Vector<Object> execute(final String user, final String password, final String requete) throws XmlRpcException {
        Vector<Object> out = null;
        try {
            if (remoteHost != null) {
                //si c'est un appel distant, ben j'appelle le distant avec xml-rpc
                final XmlRpcClient xmlrpc = new XmlRpcClient(remoteHost + remoteServlet);
                final Vector<String> paramHandler = new Vector<String>();
                paramHandler.addElement(user);
                paramHandler.addElement(password);
                paramHandler.addElement(requete);
                final Vector<?> result = (Vector<?>) xmlrpc.execute(remoteProcedure, paramHandler);
                //Transformation RPC -> DOM
                final Document document = RPCTransformer.transformToDOM(result);
                if (format == FORMAT_DOM) {
                    out = new Vector<Object>();
                    out.add(document);
                } else {
                    //Transformation DOM -> lstFiche
                    out = instancierLstFiche(document);
                }
            } else {
                ctx = new ContexteUniv("");
                //si c'est local, j'effectue l'export
                final Vector<?> lstFiche = export(user, password, requete);
                //transformation lstFiche -> RPC
                out = RPCTransformer.transformFromFiche(ctx, lstFiche);
                ctx.release();
            }
            return out;
        } catch (final Exception e) {
            throw new XmlRpcException(0, e.getMessage());
        }
    }

    /**
     * Instancier lst fiche.
     *
     * @param document
     *            the document
     *
     * @return the vector
     *
     * @throws Exception
     *             the exception
     */
    private Vector<Object> instancierLstFiche(final Document document) throws Exception {
        final Object[] lstParam = new Object[1];
        final Vector<Object> lstInstanceFiche = new Vector<Object>();
        final NodeList lstNoeudFiche = document.getFirstChild().getChildNodes();
        // Parcours des noeuds de fiches
        for (int i = 0; i < lstNoeudFiche.getLength(); i++) {
            final Node noeudFiche = lstNoeudFiche.item(i);
            //récupération du nom de la fiche
            final String nomFiche = noeudFiche.getNodeName();
            //instanciation de la fiche
            final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(nomFiche.toUpperCase());
            //récupération des informations sur la fiche
            final IntrospectionWrapper wrapper = IntrospectionWrapper.getIntrospectionWrapper(ficheUniv.getClass());
            // Parcours des proprietes de la fiche
            final NodeList lstNoeudPropriete = noeudFiche.getChildNodes();
            for (int j = 0; j < lstNoeudPropriete.getLength(); j++) {
                //récupération des propriétés de la fiche
                final Node noeudPropriete = lstNoeudPropriete.item(j);
                final String nomPropriete = noeudPropriete.getNodeName();
                final String valeurPropriete = noeudPropriete.getFirstChild().getNodeValue();
                //s'il y a incohérence entre les propriétés reçues et celles des fiches, on sort
                final Method setter = wrapper.getSetter(nomPropriete);
                if (setter == null) {
                    break;
                }
                final Class<?>[] lstClassParam = setter.getParameterTypes();
                if (lstClassParam.length != 1) {
                    break;
                }
                //Mapping des String reçues vers les types de propriétés des fiches
                if (lstClassParam[0].getName().equals("java.lang.Long")) {
                    lstParam[0] = new Long(valeurPropriete);
                } else if (lstClassParam[0].getName().equals("java.lang.Integer")) {
                    lstParam[0] = new Integer(valeurPropriete);
                } else if (lstClassParam[0].getName().equals("java.sql.Date")) {
                    lstParam[0] = new SimpleDateFormat("yyyy-MM-dd").parse(valeurPropriete);
                } else if (lstClassParam[0].getName().equals("java.lang.String")) {
                    lstParam[0] = valeurPropriete;
                } else {
                    break;
                }
                //insertion des propriétés
                setter.invoke(ficheUniv, lstParam);
            }
            //ajout de la fiche
            lstInstanceFiche.add(ficheUniv);
        }
        // renvoie la liste des fiches instanciées
        return lstInstanceFiche;
    }
}
