/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.synchro;

import java.util.Collection;
import java.util.List;
import java.util.TreeSet;
import java.util.Vector;

import com.jsbsoft.jtf.identification.GestionnaireIdentification;
import com.jsbsoft.jtf.identification.SourceAuthDefautKbd;
import com.jsbsoft.jtf.textsearch.ResultatRecherche;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.GroupeUtilisateurBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.services.ServiceGroupeUtilisateur;
import com.univ.utils.ContexteUniv;
import com.univ.utils.RequeteUtil;

/**
 * The Class KPortalExporter.
 */
public class KPortalExporter {

    /**
     * The ctx.
     */
    protected ContexteUniv ctx;

    /**
     * Instantiates a new k portal exporter.
     */
    public KPortalExporter() {
        super();
    }

    /**
     * Export.
     *
     * @param user     the user
     * @param password the password
     * @param requete  the requete
     * @return the vector
     * @throws Exception the exception
     */
    public Vector<FicheUniv> export(final String user, final String password, final String requete) throws Exception {
        UtilisateurBean utilisateur = null;
        if (new SourceAuthDefautKbd().connecte(user, password, ctx)) {
            utilisateur = GestionnaireIdentification.getInstance().estUtilisateurKportal(user, "", ctx);
        }
        //création de la session utilisateur
        if (utilisateur != null) {
            ctx.setCode(utilisateur.getCode());
            final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
            Collection<String> groupesUtilisateurs = serviceGroupeUtilisateur.getAllGroupsCodesByUserCode(utilisateur.getCode());
            ctx.setGroupesDsi(new TreeSet<>(groupesUtilisateurs));
            ctx.calculerGroupesDsiAvecAscendants();
            ctx.initialiserListeServices();
        }
        //exécution de la requête
        ctx.setCalculListeResultatsFront(true);
        List<ResultatRecherche> lstRes = RequeteUtil.traiterRequete(ctx, requete);
        ctx.setCalculListeResultatsFront(false);
        Vector<FicheUniv> lstFiche = new Vector<>();
        for (ResultatRecherche res : lstRes) {
            final FicheUniv fiche = RequeteUtil.lireFiche(res);
            lstFiche.add(fiche);
        }
        return lstFiche;
    }
}
