/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.synchro;

import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

import com.jsbsoft.jtf.webutils.ContextePage;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.GroupeUtilisateurBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.services.ServiceGroupeUtilisateur;
import com.univ.objetspartages.services.ServiceUser;
// TODO: Auto-generated Javadoc

/**
 * The Class KPortalImporter.
 */
public class KPortalImporter {

    private final ServiceUser serviceUser;

    /** The remote host. */
    private String remoteHost;

    /** The redacteur. */
    private String redacteur;

    /** The liste fiches. */
    private Vector<Object> listeFiches;

    /** The ctx. */
    private ContextePage ctx;

    /** The autorisation. */
    private AutorisationBean autorisation;

    public KPortalImporter() {
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
    }

    /**
     * Inits the.
     *
     * @throws Exception
     *             the exception
     */
    public void init() throws Exception {
        //création du contexte de page lié à l'utilisateur
        ctx = new ContextePage("");
        final UtilisateurBean utilisateur = serviceUser.getByCode(redacteur);
        // JSS 20040409 : relation groupes utilisateurs
        final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
        Collection<String> groupesUtilisateurs = serviceGroupeUtilisateur.getAllGroupsCodesByUserCode(utilisateur.getCode());
        //Création de la gestion des autorisations
        autorisation = new AutorisationBean(utilisateur, new Vector<>(groupesUtilisateurs), ctx);
    }

    /**
     * Execute.
     *
     * @throws Exception
     *             the exception
     */
    public void execute() throws Exception {
        //Parcours des fiches pour insertion en base
        final Enumeration<Object> en = listeFiches.elements();
        while (en.hasMoreElements()) {
            final FicheUniv fiche = (FicheUniv) en.nextElement();
            fiche.setCtx(ctx);
            //attribution d'un code lié au rédacteur de l'import
            //(permet de faire de tests en local sans rajouter
            // systématiquement le préfixe)
            final String prefixeRedacteur = redacteur + "_";
            if (!fiche.getCode().startsWith(prefixeRedacteur)) {
                fiche.setCode(prefixeRedacteur + fiche.getCode());
            }
            //définition du droit du rédacteur à mettre en ligne les fiches
            // JSS 2004-04-19 : délégation
            //boolean droitValidation = autorisation.getAutorisation(codeTypeObjet,AutorisationBean.INDICE_VALIDATION);
            final boolean droitValidation = autorisation.getAutorisationParFiche(fiche, AutorisationBean.INDICE_VALIDATION);
            boolean ficheExiste = false;
            // Dans tous les cas, suppression des fiches avec etat != "0003"
            // Si on n'a pas le droit de validation, création de la fiche brouillon avec date_maj = aujourd'hui
            // Si on a le droit de validation, comparaison de la fiche en ligne.
            //	- si il n'y a pas de fiche en ligne, on importe avec date_maj = aujourd'hui
            //	- s'il y a une fiche en ligne, comparaison de la date de modification
            //		- si la date est plus récente, on ne remplace pas
            //		- si la date est plus vieille, on importe avec date_maj = aujourd'hui
            final FicheUniv ficheUniv2 = fiche.getClass().newInstance();
            ficheUniv2.setCtx(ctx);
            if (ficheUniv2.selectCodeLangueEtat(fiche.getCode(), fiche.getLangue(), "") > 0) {
                while (ficheUniv2.nextItem()) {
                    if (!ficheUniv2.getEtatObjet().equals("0003")) {
                        ficheUniv2.delete();
                    } else if (droitValidation) {
                        if (ficheUniv2.getDateModification().before(fiche.getDateModification())) {
                            ficheUniv2.delete();
                        } else {
                            ficheExiste = true;
                        }
                    }
                }
            }
            if (!ficheExiste) {
                //si le rédacteur n'a pas le droit de mettre en ligne,
                //création de la fiche brouillon
                if (!droitValidation) {
                    //création de la fiche brouillon
                    fiche.setEtatObjet("0002");
                    //si le rédacteur a le droit de mettre en ligne,
                    //création de la fiche en ligne
                } else {
                    fiche.setEtatObjet("0003");
                }
                //insertion de la fiche
                fiche.setIdFiche((long) 0);
                fiche.setDateModification(new Date(System.currentTimeMillis()));
                fiche.add();
            }
        }
        ctx.release();
    }

    /**
     * Sets the redacteur.
     *
     * @param string
     *            the string
     */
    public void setRedacteur(final String string) {
        redacteur = string;
    }

    /**
     * Sets the liste fiches.
     *
     * @param vector
     *            the vector
     */
    public void setListeFiches(final Vector<Object> vector) {
        listeFiches = vector;
    }

    /**
     * Gets the remote host.
     *
     * @return the remote host
     */
    public String getRemoteHost() {
        return remoteHost;
    }

    /**
     * Sets the remote host.
     *
     * @param string
     *            the string
     */
    public void setRemoteHost(final String string) {
        remoteHost = string;
    }
}
