/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.synchro;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.ContexteUniv;
import com.univ.utils.IntrospectionWrapper;
import com.univ.utils.UnivWebFmt;
// TODO: Auto-generated Javadoc

/**
 * The Class RPCTransformer.
 */
public class RPCTransformer {

    /** The sdf. */
    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    /** The arg vide. */
    private static Object[] argVide = new Object[0];

    /**
     * Transform from fiche.
     *
     * @param ctx
     *            the ctx
     * @param lstFiche
     *            the lst fiche
     *
     * @return the vector
     *
     * @throws Exception
     *             the exception
     */
    public static Vector<Object> transformFromFiche(final ContexteUniv ctx, final Vector<?> lstFiche) throws Exception {
        // Structure principale de la liste de fiche : vector = <array>
        final Vector<Object> lstRpc = new Vector<Object>();
        final Enumeration<?> en = lstFiche.elements();
        while (en.hasMoreElements()) {
            final Object o = en.nextElement();
            // Structure secondaire par fiche : hashtable = <struct>
            final Hashtable<String, Serializable> ficheRPC = transformFromFiche(ctx, (FicheUniv) o);
            lstRpc.add(ficheRPC);
        }
        return lstRpc;
    }

    /**
     * Transform from fiche.
     *
     * @param ctx
     *            the ctx
     * @param fiche
     *            the fiche
     *
     * @return the hashtable
     *
     * @throws Exception
     *             the exception
     */
    private static Hashtable<String, Serializable> transformFromFiche(final ContexteUniv ctx, final FicheUniv fiche) throws Exception {
        final Hashtable<String, Serializable> ficheRPC = new Hashtable<String, Serializable>();
        //première clé : nomFiche <string>,<string>
        final String codeObjet = ReferentielObjets.getCodeObjetParClasse(fiche.getClass().getName());
        ficheRPC.put("nomFiche", ReferentielObjets.getNomObjet(codeObjet));
        //deuxième clé : lstPropriete <string>,<struct>
        final Hashtable<String, String> lstPropriete = new Hashtable<String, String>();
        ficheRPC.put("lstPropriete", lstPropriete);
        final Map<String, Method> lstGetter = IntrospectionWrapper.getIntrospectionWrapper(fiche.getClass()).getLstGetter();
        for (final String nomParametre : lstGetter.keySet()) {
            final Method getter = lstGetter.get(nomParametre);
            final Object returnParam = getter.invoke(fiche, argVide);
            if (returnParam != null) {
                String valeur;
                if (returnParam instanceof java.sql.Date) {
                    valeur = ((SimpleDateFormat) sdf.clone()).format(new Date(((java.sql.Date) returnParam).getTime()));
                } else {
                    valeur = returnParam.toString();
                    if (valeur.indexOf('[') != -1) {
                        valeur = UnivWebFmt.formaterEnHTML(ctx, valeur);
                    }
                }
                lstPropriete.put(nomParametre, valeur);
            }
        }
        return ficheRPC;
    }

    /**
     * Transform to dom.
     *
     * @param lstFicheRPC
     *            the lst fiche rpc
     *
     * @return the document
     *
     * @throws Exception
     *             the exception
     */
    public static Document transformToDOM(final Vector<?> lstFicheRPC) throws Exception {
        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        final DocumentBuilder builder = factory.newDocumentBuilder();
        final org.w3c.dom.Document document = builder.newDocument();
        final org.w3c.dom.Element noeudListe = document.createElement("ListeFiche");
        document.appendChild(noeudListe);
        // traitement de la structure principale
        final Enumeration<?> en = lstFicheRPC.elements();
        while (en.hasMoreElements()) {
            final Object o = en.nextElement();
            // traitement de la structure secondaire par liste
            noeudListe.appendChild(transformToDOM((Hashtable<?, ?>) o, noeudListe, ""));
        }
        return document;
    }

    /**
     * Transform to dom.
     *
     * @param hashtable
     *            the hashtable
     * @param liste
     *            the liste
     * @param formatExport
     *            the format export
     *
     * @return the node
     *
     * @throws Exception
     *             the exception
     */
    private static Node transformToDOM(final Hashtable<?, ?> hashtable, final Node liste, final String formatExport) throws Exception {
        final Enumeration<?> en = (hashtable).keys();
        while (en.hasMoreElements()) {
            final String nomFiche = (String) hashtable.get("nomFiche");
            final org.w3c.dom.Document document = liste.getOwnerDocument();
            final Node fiche = document.createElement(nomFiche);
            final Hashtable<String, String> lstPropriete = (Hashtable<String, String>) hashtable.get("lstPropriete");
            final Enumeration<String> keys = lstPropriete.keys();
            while (keys.hasMoreElements()) {
                final String nomPropriete = keys.nextElement();
                final String valeur = lstPropriete.get(nomPropriete);
                final Element element = document.createElement(nomPropriete);
                final Text texte = document.createTextNode(valeur);
                element.appendChild(texte);
                fiche.appendChild(element);
            }
            return fiche;
        }
        return null;
    }
}