/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.synchro;

import java.util.Vector;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
// TODO: Auto-generated Javadoc

/**
 * The Class SyndicXmlRpcClient.
 *
 * @author sanka
 *
 *         To change the template for this generated type comment go to Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class SyndicXmlRpcClient extends Thread {

    private static final Logger LOG = LoggerFactory.getLogger(SyndicXmlRpcClient.class);

    /** The usage. */
    private static String usage = "SynchroXmlRpcClient --host=<remote host name> --user=<remote user> --password=<remote password> --request=<request to execute> --redacteur=<local user>";

    /** The host name. */
    private static String hostName = null;

    /** The redacteur. */
    private static String redacteur = null;

    /** The user. */
    private static String user = null;

    /** The password. */
    private static String password = null;

    /** The requete. */
    private static String requete = null;

    /**
     * The main method.
     *
     * @param args
     *            the arguments
     */
    public static void main(final String[] args) {
        for (final String arg : args) {
            if ((hostName == null) && (arg.startsWith("--host="))) {
                hostName = arg.substring(7);
            } else if ((user == null) && (arg.startsWith("--user="))) {
                user = arg.substring(7);
            } else if ((password == null) && (arg.startsWith("--password="))) {
                password = arg.substring(11);
            } else if ((requete == null) && (arg.startsWith("--request="))) {
                requete = arg.substring(10);
            } else if ((redacteur == null) && (arg.startsWith("--redacteur="))) {
                redacteur = arg.substring(12);
            }
        }
        if ((hostName == null) || (user == null) || (password == null) || (requete == null) || (redacteur == null)) {
            System.out.println(usage);
            return;
        }
        final SyndicXmlRpcClient rpcClient = new SyndicXmlRpcClient();
        rpcClient.start();
    }

    /* (non-Javadoc)
     * @see java.lang.Thread#run()
     */
    @Override
    public void run() {
        try {
            //initialisation de l'import avec le rédacteur
            final KPortalImporter importer = new KPortalImporter();
            importer.setRedacteur(redacteur);
            importer.init();
            //appel procédure distante
            final Vector<Object> result = new ExportFicheHandler(hostName, ExportFicheHandler.FORMAT_DOM).execute(user, password, requete);
            final Document doc = (Document) result.firstElement();
            // Affichage
            /**************************************************/
            /*    Génération du fichier XML                   */
            /**************************************************/
            final TransformerFactory transFactory = TransformerFactory.newInstance();
            final Transformer transformer = transFactory.newTransformer();
			/*
			Using the DOM tree root node, the following line of code constructs a DOMSource object as the source
			of the transformation.
			 */
            final DOMSource source = new DOMSource(doc);
			/*
			The following code fragment creates a StreamResult object to take the results of the transformation
			and transforms the tree to XML.
			 */
            final StreamResult stream = new StreamResult(System.out);
            transformer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
            transformer.transform(source, stream);
            return;
        } catch (final Exception e) {
            LOG.error("erreur lors de la génération du fichier XML", e);
        }
    }
}
