/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.importexport.util.ExtensionUtil;
import com.kportal.extension.module.plugin.objetspartages.om.ObjetPluginContenu;
import com.univ.utils.IntrospectionWrapper;
import com.univ.xml.util.InfosImport;

import java.io.StringWriter;
import java.lang.reflect.Method;
import java.util.ArrayList;

import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.XMLEvent;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.database.OMContext;
import com.kportal.extension.module.plugin.objetspartages.om.ObjetPluginContenu;
import com.univ.objetspartages.util.TraitementImportUtil;
import com.univ.utils.IntrospectionWrapper;
import com.univ.xml.util.InfosImport;

public class TraitementImport implements ITraitementImport {

    protected Logger logger = LoggerFactory.getLogger(TraitementImport.class);

    protected String inTree = "1";

    protected final ResultatImport resultatImport;

    protected InfosImport infosImport;

	public TraitementImport(final InfosImport infosImport) {
		super();
		this.infosImport = infosImport;
		this.resultatImport = new ResultatImport(infosImport.isSynchronizedImport(), infosImport.getSourceImport());
	}

    @Override
    public ResultatImport execute(final OMContext ctx, final XMLEventReader xmlr) throws Exception {
        return getResultatImport();
    }

	/**
	 * Ajout d'un noeud de sous-objets du type
	 * <p>
	 * <SESSIONFORMATIONS> <SESSIONFORMATION> <CODE>A21</CODE> <INTITULE>Session n°21</INTITULE> </SESSIONFORMATION> <SESSIONFORMATION> <CODE>A23</CODE> <INTITULE>Session
	 * n°23</INTITULE> </SESSIONFORMATION> </SESSIONFORMATIONS>.
	 *
	 * @param ctx
	 * 		the ctx
	 * @param nomObjet
	 * 		nom de l'objet principal dans le fichier XML
	 * @param nomSousObjet
	 * 		nom du sous-objet dans le fichier XML
	 * @param classeSousObjet
	 * 		classe à instancier
	 * @param xmlr
	 * 		the xmlr
	 * @return the array list
	 * @throws Exception
	 * 		the exception
	 */
	@SuppressWarnings("deprecation")
	public ArrayList<ObjetPluginContenu> importSousObjet(final OMContext ctx, final String nomObjet, final String nomSousObjet, final Class<?> classeSousObjet, final XMLEventReader xmlr) throws Exception {
		final IntrospectionWrapper wrapper = new IntrospectionWrapper(classeSousObjet);
		final ArrayList<ObjetPluginContenu> nouvellesValeurs = new ArrayList<ObjetPluginContenu>();
		logger.debug("importSousObjet " + classeSousObjet + " tag <" + nomObjet + "><" + nomSousObjet + ">");
		// on est dans la balise <SOUS_OBJETSS>, on passe au tag suivant
		XMLEvent xmlEvent = xmlr.nextEvent();
		// tant que l'on reste dans l'entete </SOUS_OBJETSS>
		while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equals(nomSousObjet.toUpperCase() + "S"))) {
			// on cherche la balise <SOUS-OBJETS>
			if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equals(nomSousObjet.toUpperCase())) {
				final ObjetPluginContenu nouvelObjet = (ObjetPluginContenu) classeSousObjet.newInstance();
				nouvelObjet.init();
				nouvelObjet.setCtx(ctx);
				// valeurs par défaut
				TraitementImportHelper.initialiserValeursImport(nouvelObjet, nomObjet + "." + nomSousObjet, infosImport.getSourceImport());
				// on est dans la balise <SOUS_OBJETS>, on passe au tag suivant
				xmlEvent = xmlr.nextEvent();
				String nodename = "";
				String value = "";
				// tant que l'on reste dans l'entete </SOUS_OBJETS>
				while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equals(nomSousObjet.toUpperCase()))) {
					if (xmlEvent.isStartElement()) {
						nodename = xmlEvent.asStartElement().getName().getLocalPart();
						// on regarde la valeur de l'élement
						xmlEvent = xmlr.peek();
						if (xmlEvent != null && xmlEvent.isCharacters()) {
							value = xmlEvent.asCharacters().getData();
						}
						String nomAccesseur = "";
						// on met à jour l'attribut seulement si paramétré
						final String importDonnee = TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), nomObjet + "." + nomSousObjet + "." + nodename);
						if (importDonnee == null || importDonnee.equals("") || !importDonnee.equals("0")) {
							// nom accesseur : ID_FORMATION devent idFormation
							final String tokens[] = nodename.split("_", -2);
							for (final String token : tokens) {
								nomAccesseur += token.substring(0, 1).toUpperCase() + token.substring(1).toLowerCase();
							}
							// on détermine la méthode à appeler
							Method accesseur = null;
							final Object o[] = new Object[1];
							accesseur = wrapper.getSetter(nomAccesseur);
							if (accesseur != null) {
								final Class<?>[] parametres = accesseur.getParameterTypes();
								o[0] = TraitementImportHelper.format(parametres[0], value);
								// mise à jour de l'attribut
								accesseur.invoke(nouvelObjet, o);
							}
						}
					}
					// on passe au tag suivant
					xmlEvent = xmlr.nextEvent();
				}
				// remise à zéro de l'id si importé
				nouvelObjet.setIdObjet(new Long(0));
				// ajout du sous-objet
				nouvellesValeurs.add(nouvelObjet);
			}
			// on passe au tag suivant
			xmlEvent = xmlr.nextEvent();
		}
		return nouvellesValeurs;
	}

    /**
     * Renvoie le flux original (Chaîne de caractères) d'une liste d'items.
     * <p>
     * <pre>
     * &lt;PluginClass&gt;
     *   &lt;item&gt;&lt;/item&gt;
     *   &lt;item&gt;&lt;/item&gt;
     * &lt;/PluginClass&gt;
     * </pre>
     *
     * @param eventReader Lecture du flux de base.
     * @return le flux XML de base.
     * @throws XMLStreamException
     */
    public static String readElementBody(XMLEventReader eventReader) throws XMLStreamException {
        StringWriter buf = new StringWriter(1024);
        int depth = 0;
        while (eventReader.hasNext()) {
            //
            XMLEvent xmlEvent = eventReader.peek();
            if (xmlEvent.isStartElement()) {
                ++depth;
            } else if (xmlEvent.isEndElement()) {
                --depth;
            }
            // On consomme l'évènement XML
            xmlEvent = eventReader.nextEvent();
            // On le met dans le buffer
            xmlEvent.writeAsEncodedUnicode(buf);
            // On remonte l'élément.
            if (depth <= 0) {
                break;
            }
        }
        return buf.getBuffer().toString();
    }

    public void setLogger(final Logger logger) {
        this.logger = logger;
    }

    public InfosImport getInfosImport() {
        return infosImport;
    }

    public ResultatImport getResultatImport() {
        return resultatImport;
    }

    public String getInTree() {
        return inTree;
    }

    public void setInTree(final String inTree) {
        this.inTree = inTree;
    }
}
