/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.XMLEvent;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.extension.module.plugin.objetspartages.IPluginFiche;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.kportal.extension.module.plugin.objetspartages.om.PluginBean;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceMetatag;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.objetspartages.util.TraitementImportUtil;
import com.univ.synchro.SynchroGroupedsiStructure;
import com.univ.xml.imports.EnteteImport.Action;
import com.univ.xml.util.InfosImport;

/**
 * Traitement d'import (gestion de la création / mise à jour de fiche).
 */
public class TraitementImportEvent extends TraitementImport {

	private static final Logger LOG = LoggerFactory.getLogger(TraitementImportEvent.class);

    private final ServiceMetatag serviceMetatag;

    public TraitementImportEvent(final InfosImport infosImport) {
        super(infosImport);
        serviceMetatag = ServiceManager.getServiceForBean(MetatagBean.class);
    }

    /**
     * Cette méthode réalise l'import
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public ResultatImport execute(final OMContext ctx, final XMLEventReader xmlr) throws Exception {
        // paramètre permettant d'activer la visibilité des structures importées dans l'arbre front office
        final String szTree = TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), "structures.intree");
        if ("0".equals(szTree)) {
			setInTree("0");
		}
		FicheUniv ficheUniv = null;
		// l'objet à importer est spécifié hors contenu du fichier d'import
		/* on itère sur tous les élements
		 * <OBJET>
		 * 		<CODE>code</CODE>
		 * 		<LANGUE>langue</LANGUE>
		 * 		<ATTRIBUT1>att</ATTRIBUT1>
		 * 		<ATTRIBUT2>att</ATTRIBUT2>
		 * 		...
		 * </OBJET>
		 * */
		final String objet = getInfosImport().getTypeImport();
		String sourceImport;
		// boucle sur les éléments
		while (xmlr.hasNext()) {
			// premier tag (un tag = une balise ouvrante/fermante d'un élément)
			XMLEvent xmlEvent = xmlr.nextEvent();
			sourceImport = "";
			// on cherche la balise <OBJET>
			if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase(objet)) {
				final Iterator<Attribute> it = xmlEvent.asStartElement().getAttributes();
				while (it.hasNext()) {
					final Attribute attr = it.next();
					if (attr.getName().getLocalPart().equals("source-import")) {
						if (getInfosImport().isAllSourceImport() && !attr.getValue().equalsIgnoreCase(getInfosImport().getSourceImport())) {
							sourceImport = attr.getValue();
						}
					}
				}
				getResultatImport().incrementeNombreEnregistrements(objet);
				// tant que ne change pas d'item </OBJET>
				while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equalsIgnoreCase(objet))) {
					// pour chaque OBJET on va traiter les données
					try {
						// instanciation de la fiche
						ficheUniv = ReferentielObjets.instancierFiche(objet);
						if (ficheUniv != null) {
							// L'initialisation de la fiche "tampon" est désactivé, pour laisser les champs à null
							// Seuls les champs non null seront synchronisés (car présence du champ dans le flux XML)
							// import de la fiche
							xmlEvent = importItemFiche(ctx, ficheUniv, sourceImport, xmlr);
						} else {
							logger.warn("Objet " + objet + " non instanciable");
							getResultatImport().incrementeNombreErreurs(objet);
						}
						// mise à jour des groupes sur la structure
						if (ficheUniv != null && ficheUniv instanceof StructureModele) {
							SynchroGroupedsiStructure.synchroniserGroupeDsi(ctx, (StructureModele) ficheUniv);
						}
					} catch (final Exception e) {
						logger.error(e.getMessage(), e);
						getResultatImport().incrementeNombreErreurs(objet);
					}
				}
			}
		}
		return getResultatImport();
	}

    /**
     * Import item fiche.
	 *
	 * <p>
	 *     Utilisation d'une fiche "tampon" (buffer) qui permet de recevoir les champs de la fiche XML, avant synchronisation avec la fiche.
	 * </p>
     *
     * @param ficheTampon
     *            the _fiche a importer
     * @param source
     *            the _source import
     *
     * @return the XML event
     *
     * @throws Exception
     *             the exception
     */
	private XMLEvent importItemFiche(final OMContext ctx, final FicheUniv ficheTampon, final String source, final XMLEventReader xmlr) throws Exception {
		MetatagBean meta;
		final ArrayList<PluginBean> lstObjets = new ArrayList<>();
		final String nomObjet = ReferentielObjets.getNomObjet(ReferentielObjets.getCodeObjet(ficheTampon));

		// La fiche cible peut être une nouvelle fiche ou une fiche existante.
		FicheUniv ficheCible = ReferentielObjets.instancierFiche(nomObjet);
		ficheCible.init();
		ficheCible.setCtx(ctx);
		String sourceImport = getInfosImport().getSourceImport();
		if (source.length() > 0) {
			sourceImport = source;
		}
		// traitement des données
		final XMLEvent xmlEvent = importItemFicheSetDonnees(ctx, ficheTampon, lstObjets, xmlr);
		// Le code doit toujours être présent dans l'import
		String code = ficheTampon.getCode();
		if (sourceImport.length() > 0) {
			code = sourceImport + "_" + code;
		}
		// selection des fiches (car elles peuvent avoir plusieurs états) à modifier ou a supprimer
		if (ficheCible.selectCodeLangueEtat(code, ficheTampon.getLangue(), "0003") > 0) {
			ficheCible.nextItem();
			meta = MetatagUtils.lireMeta(ficheCible);
			ficheCible.selectCodeLangueEtat(code, ficheTampon.getLangue(), "");
		} else {
			ficheCible.selectCodeLangueEtat(code, ficheTampon.getLangue(), "");
			ficheCible.nextItem();
			meta = MetatagUtils.lireMeta(ficheCible);
		}
		// si le meta n'existe pas ou n'est pas initialisé (on ne regarde pas la valeur de la fiche, qu'on considère comme cohérent avec le meta).
		if (meta == null || 0L == meta.getIdMetatag()) {
			// Il est possible de créer une fiche en état brouillon ou en ligne. Par défaut en ligne
			if (ficheCible.getEtatObjet() != null && ficheCible.getEtatObjet().equals("")) {
				ficheCible.setEtatObjet("0003");
			}
			ficheCible.setCode(code);
			ficheCible = (FicheUniv) TraitementImportUtil.synchronizeObject(ficheCible, ficheTampon, Action.AJOUTER, infosImport);
			// ajout de la fiche
			ficheCible.add();
			meta = MetatagUtils.initMeta(ficheCible);
			meta.setMetaSourceImport(sourceImport);
			meta.setMetaInTree(getInTree());
			serviceMetatag.save(meta);
			// ajout des objets techniques
			PluginFicheHelper.ajouterPlugins(ficheCible, meta, lstObjets, null);
			getResultatImport().enregistrerAjoutFiche(nomObjet, ficheCible);
		}
		// le meta existe : synchronisation avec la source d'import
		else {
			// source d'import identique, on synchronise l'import avec la fiche existante
			if (meta.getMetaSourceImport().equalsIgnoreCase(sourceImport)) {
				ficheCible = (FicheUniv) TraitementImportUtil.synchronizeObject(ficheCible, ficheTampon, Action.IMPORTER, infosImport);
				// mise a jour du premier état de la fiche
				ficheCible.update();
				serviceMetatag.synchroniser(meta, ficheCible.getPersistanceBean(), false);
				serviceMetatag.save(meta);
				// synchronise la liste des sous objets
				PluginFicheHelper.synchroniserListePlugins(ficheCible, meta, lstObjets);
				getResultatImport().enregistrerModificationFiche(nomObjet, ficheTampon);
				// on boucle sur la fiche pour mettre a jour tous les etats
				while (ficheCible.nextItem()) {
					if (!"0003".equals(ficheCible.getEtatObjet())) {
						// on synchronise les autres fiches
						ficheCible = (FicheUniv) TraitementImportUtil.synchronizeObject(ficheCible, ficheTampon, Action.IMPORTER, infosImport);
						ficheCible.update();
						meta = MetatagUtils.lireMeta(ficheCible);
						serviceMetatag.synchroniser(meta, ficheCible.getPersistanceBean(), false);
						if (meta.getId() == 0) {
							meta.setMetaInTree(getInTree());
						}
						serviceMetatag.save(meta);
						// synchronise la liste des sous objets
						PluginFicheHelper.synchroniserListePlugins(ficheCible, meta, lstObjets);
					}
				}
			} else {
				LOG.info("Synchronisation sur la fiche '{}' (ID = {}) désactivée, la source d'import '{}' est incompatible avec la source d'import du fichier XML '{}'", ficheCible.getClass().getName(), ficheCible.getIdFiche(), meta.getMetaSourceImport(), sourceImport);
			}
		}
		return xmlEvent;
	}

    /**
     * Import item fiche set donnees.
     *
     * @param _fiche
     *            the _fiche
     *
     * @return the xML event
     *
     * @throws Exception
     *             the exception
     */
	private XMLEvent importItemFicheSetDonnees(final OMContext ctx, final FicheUniv _fiche, final List<PluginBean> lstObjets, final XMLEventReader xmlr) throws Exception {
		/*
		 * valeurs par défaut pour les champs non saisis (qui ne sont pas passés
		 * dans la boucle) qui ont une valeur par défaut dans le properties
		 */
		final String typeImport = getInfosImport().getTypeImport();
		TraitementImportHelper.initialiserValeursImport(_fiche, typeImport, infosImport.getSourceImport());
		// on est dans la balise <OBJET>, on lit le tag suivant
		XMLEvent xmlEvent = xmlr.nextEvent();
		// tant que l'on reste dans l'entete </OBJET>
		while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equalsIgnoreCase(typeImport))) {
			if (xmlEvent.isStartElement()) {
				final String nodeName = xmlEvent.asStartElement().getName().getLocalPart();
				logger.debug("NodeName : " + nodeName);
				String nomAccesseurSET = "set";
				// nom accesseur : ID_FORMATION devient idFormation
				final String tokens[] = nodeName.split("_", -2);
				for (final String token : tokens) {
					nomAccesseurSET += token.substring(0, 1).toUpperCase() + token.substring(1).toLowerCase();
				}
				logger.debug("nomAccesseur : " + nomAccesseurSET);
				// on regarde la valeur du noeud suivant
				String nomSousObjet = "";
				xmlEvent = xmlr.peek();
				/* Si au moins un des noeuds fils n'est pas de type TEXT, alors
				 * il s'agit d'une sous-arborescence et donc d'un sous-objet
				 */
                if (xmlEvent != null && xmlEvent.isStartElement()) {
                    nomSousObjet = xmlEvent.asStartElement().getName().getLocalPart();
                }
				/*Sous-objets techniques
				 * <OBJET>
				 *	<PARCOURSS>
				 * 		<PARCOURS>
				 * 			...
				 * 		</PARCOURS>
				 * 		<PARCOURS>
				 * 			...
				 * 		</PARCOURS>
				 * 	</PARCOURSS>
				 * </OBJET>
				 */
                if (nomSousObjet.length() > 0) {
                    final List<String> classNames = new ArrayList<>();
                    for (final IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
                        if (plugin.isActive(_fiche.getClass().getName())) {
                            if (plugin.getObjets() != null) {
                                classNames.addAll(plugin.getObjets());
                            }
                        }
                    }
                    Class<?> classeDeclarationObjet = null;
                    for (final String name : classNames) {
                        final Class<?> classeTemp = Class.forName(name);
                        // Le nom de la classe doit correspondre au nom dans le fichier XML
                        final String tokensSousObjet[] = nomSousObjet.split("_", -2);
                        String classeXMLSousObjet = "";
                        for (final String element : tokensSousObjet) {
                            classeXMLSousObjet += element.substring(0, 1).toUpperCase() + element.substring(1).toLowerCase();
                        }
                        String nomDeclarationObjet = classeTemp.getName();
                        if (nomDeclarationObjet.lastIndexOf('.') != -1) {
                            nomDeclarationObjet = nomDeclarationObjet.substring(nomDeclarationObjet.lastIndexOf('.') + 1);
                        }
                        // si correspondance on sort
                        if (nomDeclarationObjet.equals(classeXMLSousObjet)) {
                            classeDeclarationObjet = classeTemp;
                            break;
                        }
                    }
                    if (classeDeclarationObjet != null) {
                        // Appel de l'import du sous-objet
						lstObjets.addAll(importSousObjet(ctx, typeImport, nomSousObjet, classeDeclarationObjet, xmlr));
					}
					// attributs simples
				} else {
					String valeur = "";
					if (xmlEvent != null && xmlEvent.isCharacters()) {
						valeur = xmlEvent.asCharacters().getData();
					}
					if (valeur.length() > 0) {
						/* On détermine la méthode à appeler */
                        Method accesseur = null;
                        final Object o[] = new Object[1];
                        final Method[] methodes = _fiche.getClass().getMethods();
                        for (final Method methode : methodes) {
                            if (methode.getName().equals(nomAccesseurSET)) {
                                accesseur = methode;
                                final Class<?>[] parametres = accesseur.getParameterTypes();
                                o[0] = TraitementImportHelper.format(parametres[0], valeur);
                            }
                        }
                        // si l'accesseur a été trouvé
                        if (accesseur != null) {
							final String importDonnee = TraitementImportUtil.getPropertyImport(infosImport.getSourceImport() + "." + typeImport.toUpperCase(), nodeName);
							if (importDonnee == null || importDonnee.equals("") || !importDonnee.equals("0")) {
								accesseur.invoke(_fiche, o);
							}
						}
					}
				}// fin type attribut simple
			}
			xmlEvent = xmlr.nextEvent();
		}// fin </OBJET>
		// return le tag </OBJET>
		return xmlEvent;
	}
}
