/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.univ.objetspartages.bean.GroupeDsiBean;
import com.univ.objetspartages.bean.GroupeUtilisateurBean;
import com.univ.objetspartages.services.ServiceGroupeDsi;
import com.univ.objetspartages.services.ServiceGroupeUtilisateur;
import com.univ.xml.util.InfosImport;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.XMLEvent;
import org.apache.commons.lang3.StringUtils;

public class TraitementImportGroupe extends TraitementImport implements ITraitementImport {

    public TraitementImportGroupe(final InfosImport infosImport) {
        super(infosImport);
    }

    /**
     * Import des groupes à partir d'un fichier XML.
     *
     * @param _ctx
     *            the _ctx
     * @param _xmlr
     *            the _xmlr
     *
     * @return the string
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public ResultatImport execute(final OMContext _ctx, final XMLEventReader _xmlr) throws Exception {
        logger.info("Import de groupe");
        // synchronisation par rapport à la source
        String sourceImport = "xml";
        final String nomObjet = getInfosImport().getTypeImport();
        boolean synchronisation = true;
        final Collection<String> listeGroupes = new ArrayList<>();
        final Map<String, List<String>> hGroupesParents = new HashMap<>();
        XMLEvent xmlEvent = _xmlr.peek();
        // on cherche les attributs de l'entete <GROUPES>
        if (xmlEvent.isStartElement() && "GROUPES".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
            final Iterator<Attribute> it = xmlEvent.asStartElement().getAttributes();
            while (it.hasNext()) {
                final Attribute attr = it.next();
                if ("source-import".equals(attr.getName().getLocalPart())) {
                    sourceImport = attr.getValue();
                }
                if ("synchro".equals(attr.getName().getLocalPart()) && "false".equals(attr.getValue())) {
                    synchronisation = false;
                }
            }
        }
        if (synchronisation) {
            logger.info("Synchronisation active, source d'import : " + sourceImport);
        }
        final ServiceGroupeUtilisateur serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
        final ServiceGroupeDsi serviceGroupeDsi = ServiceManager.getServiceForBean(GroupeDsiBean.class);
        while (_xmlr.hasNext()) {
            // on avance
            xmlEvent = _xmlr.nextEvent();
            try {
                if (xmlEvent.isStartElement() && "GROUPE".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                    getResultatImport().incrementeNombreEnregistrements(nomObjet);
                    final GroupeDsiBean groupeImport = new GroupeDsiBean();
                    final List<String> groupesParents = new ArrayList<>();
                    final List<String> vlisteCodesUtilisateursImportSynchro = new ArrayList<>();
                    final List<String> vlisteCodesUtilisateursImportNonSynchro = new ArrayList<>();
                    Boolean importUtilisateurs = false;
                    // tant que l'on reste dans l'entete </GROUPE>
                    while (!(xmlEvent.isEndElement() && "GROUPE".equalsIgnoreCase(xmlEvent.asEndElement().getName().getLocalPart()))) {
                        // on cherche la balise <CODE>
                        if (xmlEvent.isStartElement() && "CODE".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                groupeImport.setCode(xmlEvent.asCharacters().getData().trim());
                                listeGroupes.add(groupeImport.getCode().trim());
                            }
                        } else if (xmlEvent.isStartElement() && "INTITULE".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on cherche la balise <INTITULE>
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                groupeImport.setLibelle(xmlEvent.asCharacters().getData().trim());
                            }
                        } else if (xmlEvent.isStartElement() && "TYPE".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on cherche la balise <TYPE>
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                groupeImport.setType(xmlEvent.asCharacters().getData().trim());
                            }
                        } else if (xmlEvent.isStartElement() && "CODE_STRUCTURE".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on cherche la balise <CODE_STRUCTURE>
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                groupeImport.setCodeStructure(xmlEvent.asCharacters().getData().trim());
                            }
                        } else if (xmlEvent.isStartElement() && "REQUETE_GROUPE".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on cherche la balise <REQUETE_GROUPE>
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                groupeImport.setRequeteGroupe(xmlEvent.asCharacters().getData().trim());
                            }
                        } else if (xmlEvent.isStartElement() && "REQUETE_LDAP".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on cherche la balise <REQUETE_LDAP>
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                groupeImport.setRequeteLdap(xmlEvent.asCharacters().getData().trim());
                            }
                        } else if (xmlEvent.isStartElement() && "GROUPES_PARENTS".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on cherche la balise <GROUPES_PARENTS>
                            xmlEvent = _xmlr.nextEvent();
                            // tant que l'on reste dans l'entete </GROUPES_PARENTS>
                            while (!(xmlEvent.isEndElement() && "GROUPES_PARENTS".equalsIgnoreCase(xmlEvent.asEndElement().getName().getLocalPart()))) {
                                // On mémorise tous les groupes parents afin d'éliminer, dans une deuxième
                                // temps les groupes redondants (c'est le cas des groupes exportés de e-sup)
                                if (xmlEvent.isStartElement()) {
                                    // on regarde la valeur de l'élement
                                    xmlEvent = _xmlr.peek();
                                    if (xmlEvent != null && xmlEvent.isCharacters()) {
                                        groupesParents.add(xmlEvent.asCharacters().getData().trim());
                                    }
                                }
                                xmlEvent = _xmlr.nextEvent();
                            }
                        } else if (xmlEvent.isStartElement() && "UTILISATEURS".equalsIgnoreCase(xmlEvent.asStartElement().getName().getLocalPart())) {
                            // on cherche la balise <UTILISATEURS>
                            importUtilisateurs = true;
                            xmlEvent = _xmlr.nextEvent();
                            // tant que l'on reste dans l'entete </UTILISATEURS>
                            while (!(xmlEvent.isEndElement() && "UTILISATEURS".equalsIgnoreCase(xmlEvent.asEndElement().getName().getLocalPart()))) {
                                boolean synchro = true;
                                if (xmlEvent.isStartElement()) {
                                    final Iterator<Attribute> it = xmlEvent.asStartElement().getAttributes();
                                    while (it.hasNext()) {
                                        final Attribute attr = it.next();
                                        if ("synchro".equals(attr.getName().getLocalPart()) && "false".equals(attr.getValue())) {
                                            synchro = false;
                                        }
                                    }
                                    xmlEvent = _xmlr.peek();
                                    if (xmlEvent != null && xmlEvent.isCharacters()) {
                                        if (synchro) {
                                            vlisteCodesUtilisateursImportSynchro.add(xmlEvent.asCharacters().getData().trim());
                                        } else {
                                            vlisteCodesUtilisateursImportNonSynchro.add(xmlEvent.asCharacters().getData().trim());
                                        }
                                    }
                                }
                                xmlEvent = _xmlr.nextEvent();
                            }
                        }
                        // on avance
                        xmlEvent = _xmlr.nextEvent();
                    }
                    if (StringUtils.isEmpty(groupeImport.getCode())) {
                        continue;
                    }
                    // mise à jour des groupes parents sur ce groupe
                    hGroupesParents.put(groupeImport.getCode(), groupesParents);
                    // rapprochement par rapport au code
                    Long idGroupe = (long) 0;
                    GroupeDsiBean groupe = serviceGroupeDsi.getByCode(groupeImport.getCode());
                    if (groupe != null) {
                        idGroupe = groupe.getIdGroupedsi();
                    } else {
                        groupe = new GroupeDsiBean();
                        groupe.setCode(groupeImport.getCode());
                    }
                    // Mise à jour de la source
                    if (synchronisation) {
                        groupe.setSourceImport(sourceImport);
                    }
                    if (groupeImport.getLibelle() != null) {
                        groupe.setLibelle(groupeImport.getLibelle());
                    }
                    if (groupeImport.getType() != null) {
                        groupe.setType(groupeImport.getType());
                    }
                    if (groupeImport.getCodeStructure() != null) {
                        groupe.setCodeStructure(groupeImport.getCodeStructure());
                    }
                    if (groupeImport.getRequeteGroupe() != null) {
                        groupe.setRequeteGroupe(groupeImport.getRequeteGroupe());
                    }
                    if (groupeImport.getRequeteLdap() != null) {
                        groupe.setRequeteLdap(groupeImport.getRequeteLdap());
                    }
                    // mise à jour du groupe
                    if (!idGroupe.equals(0L)) {
                        getResultatImport().incrementeNombreElementsModifies(nomObjet);
                    } else {
                        getResultatImport().incrementeNombreElementsAjoutes(nomObjet);
                    }
                    serviceGroupeDsi.save(groupe);
                    // mise à jour des relations groupes-utilisateurs
                    if (importUtilisateurs) {
                        //relations synchronisés
                        serviceGroupeUtilisateur.synchronizeUsersForGroup(vlisteCodesUtilisateursImportSynchro, groupe.getCode(), sourceImport);
                        //relations non synchronisés
                        serviceGroupeUtilisateur.addUsersToGroup(vlisteCodesUtilisateursImportNonSynchro, groupe.getCode());
                    }
                }
            } catch (final Exception e) {
                logger.error("Erreur traitement import groupe", e);
                getResultatImport().incrementeNombreErreurs(nomObjet);
            }
        }
        /* Il faut rebalayer tous les groupes pour mettre à jour le
		 * groupe père
		 * On élimine les groupes parents redondants (traitement nécessaire car e-sup exporte
		 * toute l'ascendance
		 */
        for (final String codeGroupe : hGroupesParents.keySet()) {
            final List<String> vGroupesParents = hGroupesParents.get(codeGroupe);
            final GroupeDsiBean groupe = serviceGroupeDsi.getByCode(codeGroupe);
            if (groupe != null) {
                final List<String> parentsIndirects = calculerGroupesParentsIndirect(codeGroupe, hGroupesParents);
                // On regarde pour chaque parent s'il ne s'agit pas d'un parent indirect
                int nbGroupesPeres = 0;
                for (final String groupeTmp : vGroupesParents) {
                    if (!parentsIndirects.contains(groupeTmp)) {
                        groupe.setCodeGroupePere(groupeTmp);
                        nbGroupesPeres++;
                    }
                }
                if (nbGroupesPeres > 1) {
                    logger.info("Avertissement : le groupe '" + groupe.getCode() + "' contient " + nbGroupesPeres + " groupes directs. Seul le premier est importé.");
                }
                serviceGroupeDsi.save(groupe);
            }
        }
        // Purge des anciens groupes de la meme source
        if (synchronisation) {
            final List<GroupeDsiBean> allGroups = serviceGroupeDsi.getAll();
            for (final GroupeDsiBean groupe : allGroups) {
                // Si le  groupe a été précédemment importé avec la meme source et pas ce coup-ci
                // il faut le supprimer
                if ((groupe.getSourceImport().equals(sourceImport)) && (!listeGroupes.contains(groupe.getCode()))) {
                    serviceGroupeDsi.delete(groupe.getIdGroupedsi());
                    serviceGroupeUtilisateur.deleteByGroup(groupe.getCode());
                    getResultatImport().incrementeNombreElementsSupprimes(nomObjet);
                }
            }
            logger.info("Synchronisation des groupes de la meme source");
        }
        logger.info("Rechargement des groupes");
        return getResultatImport();
    }

    /**
     * Elimine les groupes parents indirects par transitivité (fait pour les imports e-sup ou figurent tous les ascendants).
     *
     * @param codeGroupe
     *            the code groupe
     * @param hGroupesParents
     *            the h groupes parents
     *
     * @return the vector
     */
    public List<String> calculerGroupesParentsIndirect(final String codeGroupe, final Map<String, List<String>> hGroupesParents) {
        final List<String> res = new ArrayList<>();
        // Les parents indirects sont les parents des parents et leurs parents indirects
        final List<String> groupesParents = hGroupesParents.get(codeGroupe);
        if (groupesParents != null) {
            for (final String groupeParent : groupesParents) {
                final List<String> groupesGrandsParents = hGroupesParents.get(groupeParent);
                if (groupesGrandsParents != null) {
                    for (final String groupeGrandParent : groupesGrandsParents) {
                        res.add(groupeGrandParent);
                    }
                }
                res.addAll(calculerGroupesParentsIndirect(groupeParent, hGroupesParents));
            }
        }
        return res;
    }
}
