/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.XMLEvent;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.core.security.MySQLHelper;
import com.univ.objetspartages.bean.GroupeUtilisateurBean;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.objetspartages.services.ServiceGroupeUtilisateur;
import com.univ.objetspartages.services.ServiceUser;
import com.univ.objetspartages.util.TraitementImportUtil;
import com.univ.xml.util.InfosImport;

public class TraitementImportUtilisateur extends TraitementImport implements ITraitementImport {

    private final ServiceUser serviceUser;
    private final ServiceGroupeUtilisateur serviceGroupeUtilisateur;

    public TraitementImportUtilisateur(final InfosImport infosImport) {
        super(infosImport);
        serviceUser = ServiceManager.getServiceForBean(UtilisateurBean.class);
        serviceGroupeUtilisateur = ServiceManager.getServiceForBean(GroupeUtilisateurBean.class);
    }

    /**
     * Importation des données à partir d'un fichier XML.
     *
     * @param _ctx
     *            the _ctx
     * @param _xmlr
     *            the _xmlr
     *
     * @return the string
     *
     * @throws Exception
     *             the exception
     */
    @Override
    public ResultatImport execute(final OMContext _ctx, final XMLEventReader _xmlr) throws Exception {
        logger.info("Import d'utilisateur");
        // synchronisation par rapport à la source
        String sourceImport = "xml";
        final String nomObjet = getInfosImport().getTypeImport();
        boolean synchronisationListe = true;
        boolean majUtilisateur = true;
        final Collection<String> listeUsers = new ArrayList<>();
        boolean importGroupes = false;
        XMLEvent xmlEvent = _xmlr.peek();
        // on cherche les attributs de l'entete <UTILISATEURS>
        if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("UTILISATEURS")) {
            final Iterator<Attribute> it = xmlEvent.asStartElement().getAttributes();
            while (it.hasNext()) {
                final Attribute attr = it.next();
                if (attr.getName().getLocalPart().equals("source-import")) {
                    sourceImport = attr.getValue();
                }
                if (attr.getName().getLocalPart().equals("synchro")) {
                    if (attr.getValue().equals("false")) {
                        synchronisationListe = false;
                    }
                }
                if (attr.getName().getLocalPart().equals("maj-utilisateur")) {
                    if (attr.getValue().equals("false")) {
                        majUtilisateur = false;
                    }
                }
            }
        }
        if (synchronisationListe) {
            logger.info("Synchronisation active, source d'import : " + sourceImport);
        }
        if (!majUtilisateur) {
            logger.info("Mise à jour des rattachements Groupes/Utilisateurs exclusivement");
        }
        UtilisateurBean user = null;
        while (_xmlr.hasNext()) {
            // on avance
            xmlEvent = _xmlr.nextEvent();
            importGroupes = false;
            final Collection<String> vlisteGroupesDsiImportNonSynchro = new ArrayList<>();
            final Collection<String> vlisteGroupesDsiImportSynchro = new ArrayList<>();
            try {
                if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("UTILISATEUR")) {
                    user = new UtilisateurBean();
                    getResultatImport().incrementeNombreEnregistrements(nomObjet);
                    // tant que l'on reste dans l'entete </UTILISATEUR>
                    while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equalsIgnoreCase("UTILISATEUR"))) {
                        // on cherche la balise <CODE>
                        if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("CODE")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setCode(xmlEvent.asCharacters().getData().trim());
                                listeUsers.add(user.getCode());
                            }
                        }
                        // on cherche la balise <MOT_DE_PASSE>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("MOT_DE_PASSE")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setMotDePasse(MySQLHelper.encodePassword(xmlEvent.asCharacters().getData().trim()));
                            }
                        }
                        // on cherche la balise <CODE_LDAP>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("CODE_LDAP")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setCodeLdap(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <CIVILITE>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("CIVILITE")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setCivilite(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <NOM>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("NOM")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setNom(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <PRENOM>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("PRENOM")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setPrenom(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <CODE_RATTACHEMENT>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("CODE_RATTACHEMENT")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setCodeRattachement(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <ADRESSE_MAIL>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("ADRESSE_MAIL")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setAdresseMail(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <EXTENSION_MODIFICATION>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("EXTENSION_MODIFICATION")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setExtensionModification(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <RESTRICTION_VALIDATION>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("RESTRICTION_VALIDATION")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setRestrictionValidation(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <PROFIL_DSI>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("PROFIL_DSI")) {
                            // on regarde la valeur de l'élement
                            xmlEvent = _xmlr.peek();
                            if (xmlEvent != null && xmlEvent.isCharacters()) {
                                user.setProfilDefaut(xmlEvent.asCharacters().getData().trim());
                            }
                        }
                        // on cherche la balise <CENTRES_INTERETS>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("CENTRES_INTERETS")) {
                            String listeCentres = "";
                            xmlEvent = _xmlr.nextEvent();
                            // tant que l'on reste dans l'entete </CENTRES_INTERETS>
                            while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equalsIgnoreCase("CENTRES_INTERETS"))) {
                                // On mémorise tous les groupes parents afin d'éliminer, dans une deuxième
                                // temps les groupes redondants (c'est le cas des groupes exportés de e-sup)
                                if (xmlEvent.isStartElement()) {
                                    // on regarde la valeur de l'élement
                                    xmlEvent = _xmlr.peek();
                                    if (xmlEvent != null && xmlEvent.isCharacters()) {
                                        listeCentres += ";" + xmlEvent.asCharacters().getData().trim();
                                    }
                                }
                                _xmlr.nextEvent();
                            }
                            user.setCentresInteret(listeCentres);
                        }
                        // on cherche la balise <GROUPES>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("GROUPES")) {
                            String listeGroupes = "";
                            xmlEvent = _xmlr.nextEvent();
                            // tant que l'on reste dans l'entete </CENTRES_INTERETS>
                            while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equalsIgnoreCase("GROUPES"))) {
                                // On mémorise tous les groupes parents afin d'éliminer, dans une deuxième
                                // temps les groupes redondants (c'est le cas des groupes exportés de e-sup)
                                if (xmlEvent.isStartElement()) {
                                    // on regarde la valeur de l'élement
                                    xmlEvent = _xmlr.peek();
                                    if (xmlEvent != null && xmlEvent.isCharacters()) {
                                        listeGroupes += ";" + xmlEvent.asCharacters().getData().trim();
                                    }
                                }
                                xmlEvent = _xmlr.nextEvent();
                            }
                            user.setGroupes(listeGroupes);
                        }
                        // on cherche la balise <GROUPES_DSI>
                        else if (xmlEvent.isStartElement() && xmlEvent.asStartElement().getName().getLocalPart().equalsIgnoreCase("GROUPES_DSI")) {
                            importGroupes = true;
                            xmlEvent = _xmlr.nextEvent();
                            // tant que l'on reste dans l'entete </GROUPES_DSI>
                            while (!(xmlEvent.isEndElement() && xmlEvent.asEndElement().getName().getLocalPart().equalsIgnoreCase("GROUPES_DSI"))) {
                                boolean synchro = true;
                                if (xmlEvent.isStartElement()) {
                                    final Iterator<Attribute> it = xmlEvent.asStartElement().getAttributes();
                                    while (it.hasNext()) {
                                        final Attribute attr = it.next();
                                        if (attr.getName().getLocalPart().equals("synchro")) {
                                            if (attr.getValue().equals("false")) {
                                                synchro = false;
                                            }
                                        }
                                    }
                                    xmlEvent = _xmlr.peek();
                                    if (xmlEvent != null && xmlEvent.isCharacters()) {
                                        if (synchro) {
                                            vlisteGroupesDsiImportSynchro.add(xmlEvent.asCharacters().getData().trim());
                                        } else {
                                            vlisteGroupesDsiImportNonSynchro.add(xmlEvent.asCharacters().getData().trim());
                                        }
                                    }
                                }
                                xmlEvent = _xmlr.nextEvent();
                            }
                        }
                        // on avance
                        xmlEvent = _xmlr.nextEvent();
                    }
                    // mise à jour de la source d'import pour l'utilisateur
                    if (synchronisationListe) {
                        user.setSourceImport(sourceImport);
                    }
                    final UtilisateurBean oldUser = serviceUser.getByCode(user.getCode());
                    if ((majUtilisateur || oldUser != null) && importGroupes) {
                        // synchronisation des relations groupes/utilisateur pour la source
                        serviceGroupeUtilisateur.synchronizeGroupsForUser(vlisteGroupesDsiImportSynchro, user.getCode(), sourceImport);
                        // ajout des relations non synchronisés
                        serviceGroupeUtilisateur.addGroupsToUser(vlisteGroupesDsiImportNonSynchro, user.getCode());
                    }
                    if (oldUser != null) {
                        user.setId(oldUser.getId());
                        if (StringUtils.isEmpty(user.getCentresInteret()) || !majUtilisateur) {
                            user.setCentresInteret(oldUser.getCentresInteret());
                        }
                        if (StringUtils.isEmpty(user.getCivilite()) || !majUtilisateur) {
                            user.setCivilite(oldUser.getCivilite());
                        }
                        if (StringUtils.isEmpty(user.getCodeRattachement()) || !majUtilisateur) {
                            user.setCodeRattachement(oldUser.getCodeRattachement());
                        }
                        if (StringUtils.isEmpty(user.getExtensionModification()) || !majUtilisateur) {
                            user.setExtensionModification(oldUser.getExtensionModification());
                        }
                        if (StringUtils.isEmpty(user.getMotDePasse()) || !majUtilisateur) {
                            user.setMotDePasse(oldUser.getMotDePasse());
                        }
                        if (StringUtils.isEmpty(user.getNom()) || !majUtilisateur) {
                            user.setNom(oldUser.getNom());
                        }
                        if (StringUtils.isEmpty(user.getPrenom()) || !majUtilisateur) {
                            user.setPrenom(oldUser.getPrenom());
                        }
                        if (StringUtils.isEmpty(user.getRestrictionValidation()) || !majUtilisateur) {
                            user.setRestrictionValidation(oldUser.getRestrictionValidation());
                        }
                        if (StringUtils.isEmpty(user.getAdresseMail()) || !majUtilisateur) {
                            user.setAdresseMail(oldUser.getAdresseMail());
                        }
                        if (StringUtils.isEmpty(user.getRoles()) || !majUtilisateur) {
                            user.setRoles(oldUser.getRoles());
                        }
                        if (StringUtils.isEmpty(user.getCodeLdap()) || !majUtilisateur) {
                            user.setCodeLdap(oldUser.getCodeLdap());
                        }
                        if (StringUtils.isEmpty(user.getSourceImport()) || !majUtilisateur) {
                            user.setSourceImport(oldUser.getSourceImport());
                        }
                        logger.debug(String.format("Utilisateur mis à jour : %s %s", user.getNom(), user.getPrenom()));
                        getResultatImport().incrementeNombreElementsModifies(nomObjet);
                    }
                    if (majUtilisateur) {
                        serviceUser.save(user);
                    }
                }
            } catch (final Exception e) {
                logger.error("Erreur traitement import utilisateur", e);
                getResultatImport().incrementeNombreErreurs(nomObjet);
            }
        }
        String suppression = TraitementImportUtil.getPropertyImport(infosImport.getSourceImport(), "utilisateur.suppression");
        if (suppression == null) {
            suppression = "0";
        }
        if (suppression.equals("1") && majUtilisateur) {
            final List<UtilisateurBean> users = serviceUser.getAllUsers();
            for (final UtilisateurBean currentUser : users) {
                if (!listeUsers.contains(user.getCode())) {
                    serviceUser.delete(currentUser.getId());
                    serviceGroupeUtilisateur.deleteByUser(user.getCode());
                    logger.debug(String.format("Utilisateur supprimé : %s %s", user.getNom(), user.getPrenom()));
                    getResultatImport().incrementeNombreElementsSupprimes(nomObjet);
                }
            }
        } else {
            // ajout de la synchronisation par rapport à la source
            if (synchronisationListe && majUtilisateur) {
                final List<UtilisateurBean> users = serviceUser.getByImportSource(sourceImport);
                for (final UtilisateurBean currentUser : users) {
                    if (!listeUsers.contains(user.getCode())) {
                        serviceUser.delete(currentUser.getId());
                        serviceGroupeUtilisateur.deleteByUser(user.getCode());
                        logger.debug(String.format("Utilisateur supprimé : %s %s", user.getNom(), user.getPrenom()));
                        getResultatImport().incrementeNombreElementsSupprimes(nomObjet);
                    }
                }
            }
        }
        return getResultatImport();
    }
}
