/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Created on 23 juil. 07
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package com.univ.xml;

import java.io.File;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;
// TODO: Auto-generated Javadoc

/**
 * This is a simple example command line utility, that shows how to use the new Stax2 validation API to validate input documents against specific DTD instance.
 */
public class XMLValidator {

    /** The Constant JAXP_SCHEMA_LANGUAGE. */
    static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";

    /** The Constant W3C_XML_SCHEMA. */
    static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";

    /** The Constant JAXP_SCHEMA_SOURCE. */
    static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";

    private static final Logger LOG = LoggerFactory.getLogger(XMLValidator.class);

    /** The is valid dtd. */
    static boolean isValidDTD;

    /** The is valid xsd. */
    static boolean isValidXSD;

    static {
        isValidDTD = false;
        isValidXSD = false;
    }

    /**
     * Validate xsd.
     *
     * @param f
     *            the f
     * @param XSDf
     *            the xS df
     *
     * @return true, if successful
     */
    public static boolean validateXSD(final File f, final File XSDf) {
        isValidXSD = true;
        try {
            validate(f, XSDf);
        } catch (final Exception e) {
            LOG.error("erreur pendant la validation de la XSD", e);
            return false;
        }
        return isValidXSD;
    }

    /**
     * Validate xsd.
     *
     * @param f
     *            the f
     * @param XSDf
     *            the xS df
     *
     * @return true, if successful
     */
    public static void validate(final File f, final File XSDf) throws Exception {
        final SAXParserFactory spf = SAXParserFactory.newInstance();
        spf.setNamespaceAware(true);
        spf.setValidating(true);
        final SAXParser sp = spf.newSAXParser();
        // Implémentation spécifique qui permet de savoir si il y a un doctype
        final LexicalHandlerImpl lexicalHandler = new LexicalHandlerImpl();
        sp.setProperty("http://xml.org/sax/properties/lexical-handler", lexicalHandler);
        if (XSDf != null) {
            sp.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
            sp.setProperty(JAXP_SCHEMA_SOURCE, XSDf);
        }
        final DefaultHandler dh = new MyErrorHandler();
        sp.parse(f, dh);
        if (XSDf != null) {
            LOG.debug("Document '" + f + "' well-formated and succesfully validated with the schema '" + XSDf + "'.");
        } else if (lexicalHandler.hasDTD()) {
            LOG.debug("Document '" + f + "' well-formated and succesfully validated");
        } else {
            LOG.debug("Document '" + f + "' well-formated but DOCTYPE is missing");
        }
    }

    /**
     * The Class MyErrorHandler.
     */
    private static class MyErrorHandler extends DefaultHandler {

        /* (non-Javadoc)
         * @see org.xml.sax.helpers.DefaultHandler#warning(org.xml.sax.SAXParseException)
         */
        @Override
        public void warning(final SAXParseException e) throws SAXException {
			LOG.warn("Warning: {}", getInfos(e));
		}

        /* (non-Javadoc)
         * @see org.xml.sax.helpers.DefaultHandler#error(org.xml.sax.SAXParseException)
         */
        @Override
        public void error(final SAXParseException e) throws SAXException {
			LOG.error("Error: {}", getInfos(e));
		}

        /* (non-Javadoc)
         * @see org.xml.sax.helpers.DefaultHandler#fatalError(org.xml.sax.SAXParseException)
         */
        @Override
        public void fatalError(final SAXParseException e) throws SAXException {
			LOG.error("Fattal error: {}", getInfos(e));
		}

        /**
		 * Retourne les informations sur l'exception passée an paramètre.
		 *
		 * @param e
		 *            l'exception
		 */
		private String getInfos(final SAXParseException e) {

			final StringBuilder infos = new StringBuilder();

			infos.append("   Public ID: ").append(e.getPublicId());
			infos.append("   System ID: ").append(e.getSystemId());
			infos.append("   Line number: ").append(e.getLineNumber());
			infos.append("   Column number: ").append(e.getColumnNumber());
			infos.append("   Message: ").append(e.getMessage());

			return infos.toString();
		}
	}
}
