/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml.exports;

import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.xmlrpc.XmlRpcServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.jsbsoft.jtf.database.OMContext;
import com.univ.utils.ContexteUniv;
import com.univ.xml.IXmlCstes;
import com.univ.xml.ImportExportManager;
import com.univ.xml.util.ControleurAccesURL;

/**
 * Servlet appelee par XML-RPC permettant l'execution d'un export.
 *
 * @author jbiard
 */
@Controller
public class ExportXMLRPCServlet implements IXmlCstes {

    public final static String URL_PROPERTY = "export_xml_rpc.mapping";

    /**
     * The LOG.
     */
    private static final Logger LOG = LoggerFactory.getLogger(ExportXMLRPCServlet.class);

    /**
     * The rpc server.
     */
    private transient XmlRpcServer rpcServer;


    public ExportXMLRPCServlet() {
        //création du serveur xmlrpc avec le handler
        rpcServer = new XmlRpcServer();
    }

    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServlet#service(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    @RequestMapping("${" + URL_PROPERTY + ":/servlet/com.univ.xml.exports.ExportXMLRPCServlet}")
    public void service(final HttpServletRequest request, final HttpServletResponse response) throws IOException {
        final OMContext ctx = new ContexteUniv("");
        try {
            if (!new ControleurAccesURL().accesAutorise(request, ctx)) {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
                return;
            }
        } catch (final Exception e) {
            LOG.error("Erreur d'acces", e);
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
            return;
        }
        rpcServer.addHandler(XMLRPC_OBJET_EXPORT, new Exporter(ImportExportManager.getInstance(), request.getSession()));
        // execution de la requete xmlrpc
        final byte[] result = rpcServer.execute(request.getInputStream());
        // ecriture de la reponse
        response.setContentType(RESPONSE_CONTENT_TYPE);
        response.setContentLength(result.length);
        OutputStream output = null;
        try {
            output = response.getOutputStream();
            output.write(result);
            output.flush();
        } finally {
            if (output != null) {
                output.close();
            }
        }
    }
}
