/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Created on 26 juil. 07
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package com.univ.xml.extraction;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.univ.xml.extraction.impl.ExtracteurCSV;
import com.univ.xml.extraction.impl.ExtracteurPDF;
import com.univ.xml.extraction.impl.ExtracteurXML;
// TODO: Auto-generated Javadoc

/**
 * A factory for creating Extracteur objects.
 */
public class ExtracteurFactory {

    private static final Logger LOG = LoggerFactory.getLogger(ExtracteurFactory.class);

    /**
     * Méthode permettant de construire les extracteurs en fonction des types de sérialisation voulu.
     *
     * @param typeSerialisation type de sérialisation voulu
     * @return extracteur
     * @throws ErreurApplicative si le type n'est pas supporté
     */
    public static IExtractor newInstance(final String typeSerialisation) throws ErreurApplicative {
        IExtractor extracteur = null;
        if ("PDF".equals(typeSerialisation)) {
            extracteur = new ExtracteurPDF();
        } else if ("XML".equals(typeSerialisation)) {
            extracteur = new ExtracteurXML();
        } else if ("CSV".equals(typeSerialisation)) {
            extracteur = new ExtracteurCSV();
        } else if ("RTF".equals(typeSerialisation)) {
            final String messErr = "Le portail ne supporte pas ce type d'extraction: '" + typeSerialisation + "'.";
            LOG.error(messErr);
            throw new ErreurApplicative(messErr);
        } else {
            final String messErr = "Le portail ne supporte pas ce type d'extraction: '" + typeSerialisation + "'.";
            LOG.error(messErr);
            throw new ErreurApplicative(messErr);
        }
        return extracteur;
    }
}
