/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml.extraction.impl;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.kosmos.importexport.util.ExtensionUtil;
import com.univ.xml.IXmlCstes;
import com.univ.xml.extraction.IExtractor;
import com.univ.xml.util.Entete;
import com.univ.xml.util.HandlerCSV;
import com.univ.xml.util.Item;
import com.univ.xml.util.ItemOT;
// TODO: Auto-generated Javadoc

/**
 * Extracteur CSV.
 *
 * @author EBO
 */
public class ExtracteurCSV implements IExtractor, IXmlCstes {

    /** Logger. */
    private static Logger _log = LoggerFactory.getLogger(ExtracteurCSV.class);

    /**
     * initialisation d'un buffer contenant des libellés.
     *
     * @param listeLibelles
     *            the liste libelles
     *
     * @return the string
     */
    private String initBuffer(final List<String> listeLibelles) {
        final StringBuffer sbf = new StringBuffer();
        //insertion de l'entête du fichier csv
        String libelle = null;
        for (final Iterator<String> it = listeLibelles.iterator(); it.hasNext(); ) {
            libelle = it.next();
            sbf.append(libelle + ";");
        }
        return sbf.toString();
    }

    /**
     * initialisations d'un buffer à partir des libellés des différentes éléments à afficher en csv.
     *
     * @param listeLibellesFiche
     *            liste des libellés de la fiche
     * @param lstLibelleOT
     *            liste des libellés des sous objets techniques
     * @param lsLibelleFW
     *            liste des libellés des fichiers GW
     * @param lstLibelleMetier
     *            listes des libellés METIER
     *
     * @return buffer initialisé (première ligne entête saisie)
     */
    private StringBuilder initBuffer(final List<String> listeLibellesFiche, final List<String> lstLibelleOT, final List<String> lsLibelleFW, final List<String> lstLibelleMetier) {
        final StringBuilder sb = new StringBuilder();
        sb.append(ENTETE_CSV);
        sb.append(initBuffer(listeLibellesFiche));
        sb.append(initBuffer(lstLibelleOT));
        sb.append(initBuffer(lsLibelleFW));
        sb.append(initBuffer(lstLibelleMetier));
        sb.append("\n");
        return sb;
    }

    /**
     * Méthode permettant d'écrire une ligne dans le buffer.
     *
     * @param entete
     *            entête de la ligne
     * @param valeurs
     *            valeurs à insérer
     * @param libelles
     *            libellés correspondant
     * @param decalage
     *            décalage à appliquer
     *
     * @return the line
     */
    private StringBuffer getLine(final String entete, final Map<String, String> valeurs, final List<String> libelles, final int decalage) {
        final StringBuffer sbf = new StringBuffer();
        if (valeurs.size() > 0) {
            sbf.append(entete);
            for (int i = 0; i < decalage; i++) {
                sbf.append(";");
            }
            String val = null;
            for (final String string : libelles) {
                val = valeurs.get(string);
                if (val == null) {
                    val = "";
                }
                sbf.append(convertirVal(val) + ";");
            }
            sbf.append("\n");
        }
        return sbf;
    }

    /**
     * Méthode permettant de convertir les ; et \n.
     *
     * @param val
     *            texte à convertir
     *
     * @return texte converti
     */
    private String convertirVal(final String val) {
        String res = val;
        if (res.indexOf("\n") != -1 || res.indexOf(";") != -1) {
            res = "\"" + val + "\"";
        }
        return res;
    }

    /**
     * Extraire.
     *
     * @param input
     *            the input
     *
     * @return the export result
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     *
     * @see IExtractor#extraire(InputStream)
     */
    @Override
    public void writeToFile(final InputStream input, final File file) throws ErreurApplicative {
        final HandlerCSV handlerCSV = new HandlerCSV();
        List<Item> lstItems = null;
        List<String> lstLibellesDonnees = null;
        List<String> lstLibelleOT = null;
        List<String> lstLibelleFW = null;
        List<String> lstLibelleMetier = null;
        try {
            final XMLReader rdr = XMLReaderFactory.createXMLReader(SAXPARSER);
            rdr.setContentHandler(handlerCSV);
            final InputStreamReader isr = new InputStreamReader(input, "utf-8");
            rdr.parse(new InputSource(isr));
            lstItems = handlerCSV.getListeItems();
            lstLibellesDonnees = handlerCSV.getLstLibellesDonnees();
            lstLibelleOT = handlerCSV.getLstLibellesOT();
            lstLibelleFW = handlerCSV.getLstLibellesFW();
            lstLibelleMetier = handlerCSV.getLstLibellesMetier();
            //écriture de l'entête du fichier csv
            final StringBuilder sb = initBuffer(lstLibellesDonnees, lstLibelleOT, lstLibelleFW, lstLibelleMetier);
            Item item = null;
            Entete entete = null;
            final int decalageOT = lstLibellesDonnees.size();
            final int decalageFW = decalageOT + lstLibelleOT.size();
            final int decalageMetier = decalageFW + lstLibelleFW.size();
            //insertion des valeurs pour chaque item
            String chaineEntete = null;
            for (final Object element : lstItems) {
                item = (Item) element;
                entete = item.getEntete();
                //insertion des valeurs
                chaineEntete = entete.getObjet() + ";" + entete.getCode() + ";" + entete.getLangue() + ";";
                //fiche
                sb.append(getLine(chaineEntete, item.getValeursDonnees(), lstLibellesDonnees, 0));
                //ot
                for (final Iterator<ItemOT> it = item.getValeursOT().iterator(); it.hasNext(); ) {
                    final ItemOT itemOT = it.next();
                    chaineEntete = entete.getObjet() + "#" + itemOT.getNomItem() + ";" + entete.getCode() + ";" + entete.getLangue() + ";";
                    sb.append(getLine(chaineEntete, itemOT.getDonnees(), lstLibelleOT, decalageOT));
                }
                //fw
                for (final Iterator<Map<String, String>> it = item.getValeursFW().iterator(); it.hasNext(); ) {
                    chaineEntete = entete.getObjet() + "#" + "FICHIERGW" + ";" + entete.getCode() + ";" + entete.getLangue() + ";";
                    sb.append(getLine(chaineEntete, it.next(), lstLibelleFW, decalageFW));
                }
                //metier
                chaineEntete = entete.getObjet() + "#" + "METIER" + ";" + entete.getCode() + ";" + entete.getLangue() + ";";
                sb.append(getLine(chaineEntete, item.getValeursMetier(), lstLibelleMetier, decalageMetier));
            }
            FileUtils.writeStringToFile(file, sb.toString());
        } catch (final SAXException e) {
            final String messErr = "Erreur SAX lors de la conversion en csv";
            _log.error(messErr, e);
            throw new ErreurApplicative(messErr);
        } catch (final IOException e) {
            final String messErr = "Erreur à l'ouverture du flux à interpréter en csv";
            _log.error(messErr, e);
            throw new ErreurApplicative(messErr);
        } finally {
            try {
                input.close();
            } catch (final IOException e) {
                _log.error("Echec fermeture flux", e);
            }
        }
    }

    @Override
    public String getDisposition() {
        // lecture de la disposition
        String disposition = ExtensionUtil.getProperty(IXmlCstes.PARAM_JTF_DISPOSITION_PDF);
        if (disposition == null) {
            disposition = IXmlCstes.DISPOSITION_PDF_DEFAUT;
        }
        return disposition;
    }

    @Override
    public String getFileName() {
        return IXmlCstes.DISPOSITION_FILENAME_DEFAUT + System.currentTimeMillis() + ".csv";
    }

    @Override
    public String getContentType() {
        return "application/csv";
    }
}
