/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml.processus;

import java.io.File;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.AbstractProcessusBean;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.ProcessusBean;
import com.kportal.core.config.PropertyHelper;
import com.kportal.core.webapp.WebAppUtil;
import com.univ.utils.FileReaderServlet;
import com.univ.utils.URLResolver;
import com.univ.xml.IXmlCstes;
import com.univ.xml.ImportExportManager;
import com.univ.xml.exports.Exporter;
import com.univ.xml.extraction.IExtractor;
import com.univ.xml.util.InfosExport;

/**
 * Ce processus permet de générer un export pdf d'un objet (ex catalogue de formation) à partir d'une requête simple &OBJET=xxx&CODE=xxx
 * Le code d'export CODE_EXPORT permet de retrouver le fichier xsl de tranformation xsl-fo to pdf
 */
public class TraitementPDF extends ProcessusBean implements IXmlCstes {

    protected static final Logger LOG = LoggerFactory.getLogger(TraitementPDF.class);

    private static final String ECRAN_REDIRECTION = "REDIRECTION";

    public TraitementPDF(InfoBean infoBean) {
        super(infoBean);
    }

    private void traiterEXPORT() throws Exception {
        ImportExportManager iem = ImportExportManager.getInstance();
        String codeExport = infoBean.getString("CODE_EXPORT");
        InfosExport ie = new InfosExport();
        ie.setCode(codeExport);
        if (StringUtils.isNotEmpty(infoBean.getString("OBJET"))) {
            ie.addParametre("OBJET", infoBean.getString("OBJET"));
        }
        if (StringUtils.isNotEmpty(infoBean.getString("CODE"))) {
            ie.addParametre("CODE", infoBean.getString("CODE"));
        }
        ie.setSerialisation("PDF");
        ie.setFormaterEnHtml(true);
        ie.setInclusionOT(true);
        ie.setFormaterEnTexte(true);
        // Pour simplifier, le code export est également le nom du fichier xsl de transformation (en minuscule)
        ie.addTransformation(codeExport.toLowerCase());
        iem.setInfosExport(ie);
        // Génération du fichier d'export
        IExtractor extracteur = iem.getExportManager(codeExport).getExtracteur();
        File exportPdfFile = new File(WebAppUtil.getDownloadPrivatePath(), extracteur.getFileName());
        new Exporter(iem).execute(infoBean, this, exportPdfFile);
        // Ecran de redirection vers le téléchargement automatique
        final String url = PropertyHelper.getCoreProperty(FileReaderServlet.URL_PROPERTY) + "?FILE=" + exportPdfFile.getName() + "&TYPE=" + extracteur.getContentType();
        infoBean.setEcranRedirection(URLResolver.getAbsoluteUrl(url, this));
        ecranLogique = ECRAN_REDIRECTION;
    }

    /**
     * @see AbstractProcessusBean#traiterAction()
     */
    @Override
    protected boolean traiterAction() throws Exception {
        try {
            ecranLogique = infoBean.getEcranLogique();
            action = infoBean.getActionUtilisateur();
            if (ACTION_EXPORTER.equals(action)) {
                traiterEXPORT();
            }
            infoBean.setEcranLogique(ecranLogique);
        } catch (Exception e) {
            infoBean.addMessageErreur(e.getMessage());
        }
        return etat == FIN;
    }
}
