/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml.util;

import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.database.OMContext;
import com.jsbsoft.jtf.identification.GestionnaireIdentification;
import com.jsbsoft.jtf.identification.SourceAuthDefautKbd;
import com.jsbsoft.jtf.identification.ValidateurCAS;
import com.kosmos.importexport.util.ExtensionUtil;
import com.univ.objetspartages.bean.UtilisateurBean;
import com.univ.xml.IXmlCstes;
// TODO: Auto-generated Javadoc

/**
 * Controle l'acces par URL (WS et servlet).
 *
 * @author jbiard
 */
public class ControleurAccesURL implements IXmlCstes {

    /**
     * The _log.
     */
    private static Logger _log = LoggerFactory.getLogger(ControleurAccesURL.class);

    /**
     * Verifie l'acces.
     *
     * @param request requete
     * @param ctx     contexte
     * @return vrai si l'acces est autorise
     * @throws Exception the exception
     */
    public boolean accesAutorise(HttpServletRequest request, OMContext ctx) throws Exception {
        // restriction sur IP
        String ip = request.getRemoteAddr();
        if (!ipAutorisee(ip)) {
            _log.error("L'IP " + ip + " n'est pas autorisee a realiser un export.");
            return false;
        }
        GestionnaireIdentification gI = GestionnaireIdentification.getInstance();
        // recuperation du ticket CAS proxy si specifie
        String ticket = request.getParameter("ticket");
        if ((ticket != null) && gI.getSourceAuth().getCodeSource().equals(ValidateurCAS.SOURCE_LIBELLE_CAS)) {
            try {
                // on precise le port si celui-ci n'est pas standard
                String port = "" + request.getServerPort();
                if (!("80".equals(port) || "443".equals(port))) {
                    port = ":" + port;
                } else {
                    port = "";
                }
                // recuperation de l'URL appelee pour validation du service en mode proxy
                // celle-ci doit etre identique a celle specifiee lors de la demande du ticket
                String urlAppelee = request.getScheme() + "://" + request.getServerName() + port + request.getRequestURI();
                UtilisateurBean utilisateur = gI.getValidateurCAS().validation(ctx, urlAppelee, ticket, false);
                if (utilisateur == null) {
                    return false;
                }
                return gI.chargeInfoUser(utilisateur.getCode(), ctx, request.getSession(), request.getHeader("user-agent"));
            } catch (Exception e) {
                _log.error("Echec acces", e);
            }
        } else {
            // appel XML-RPC
            // recuperation des elements permettant d'evaluer le controle d'acces
            String auth = request.getHeader("Authorization");
            if (auth == null) {
                return false;
            }
            // recupere le nom d'utilisateur et mot de passe encodés
            String userPassEncoded = auth.substring(6);
            // decode les deonnees
            //Ancien traitement, migré en 5.2 (l'api est propriétaire de sun)
            // sun.misc.BASE64Decoder dec = new BASE64Decoder( );
            // String userPassDecoded = new String( dec.decodeBuffer( userPassEncoded ) );
            String userPassDecoded = new String(Base64.decodeBase64(userPassEncoded));
            // separe le nom d'utilisateur et mot de passe
            StringTokenizer userAndPass = new StringTokenizer(userPassDecoded, ":");
            String codeUtilisateur = userAndPass.nextToken();
            String mdp = userAndPass.nextToken();
            // verification du compte
            if ((codeUtilisateur != null) && (mdp != null)) {
                if (new SourceAuthDefautKbd().connecte(codeUtilisateur, mdp, ctx)) {
                    return GestionnaireIdentification.getInstance().chargeInfoUser(codeUtilisateur, ctx, request.getSession(), request.getHeader("user-agent"));
                }
                _log.error("Utilisateur " + codeUtilisateur + " invalide.");
            }
        }
        return false;
    }

    /**
     * Determine si l'IP du client est valide.
     *
     * @param ip
     *            ip du client
     *
     * @return vrai si demande acceptee
     */
    public boolean ipAutorisee(String ip) {
        Set<String> setWsIpsExport;
        // lecture des IPs autorisees
        String wsIpsExport = ExtensionUtil.getProperty(PARAM_JTF_RESTRICTION_IPS);
        if (wsIpsExport != null) {
            StringTokenizer st = new StringTokenizer(wsIpsExport, ";");
            setWsIpsExport = new HashSet<String>(st.countTokens());
            while (st.hasMoreElements()) {
                setWsIpsExport.add(st.nextToken());
            }
        } else {
            setWsIpsExport = new HashSet<String>(1);
            setWsIpsExport.add(IP_AUTORISEE_DEFAUT);
        }
        return setWsIpsExport.contains(ip);
    }
}
