/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.kportal.extension.module.plugin.objetspartages.IPluginFiche;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.univ.xml.IXmlCstes;
// TODO: Auto-generated Javadoc

/**
 * Handler CSV. Cette classe sert à parser le flux XML pour en extraire les informations à destination de l'extraction CSV
 *
 * @author EBO
 */
public class HandlerCSV extends DefaultHandler implements IXmlCstes {

    /** The lst ot. */
    private final List<String> lstOT = new ArrayList<String>();

    /** The liste items. */
    private final List<Item> listeItems = new ArrayList<Item>();

    /** The lst libelles donnees. */
    private final List<String> lstLibellesDonnees = new ArrayList<String>();

    /** The lst libelles ot. */
    private final List<String> lstLibellesOT = new ArrayList<String>();

    /** The lst libelles fw. */
    private final List<String> lstLibellesFW = new ArrayList<String>();

    /** The lst libelles metier. */
    private final List<String> lstLibellesMetier = new ArrayList<String>();

    /** The item. */
    private Item item = null;

    /** The item ot. */
    private ItemOT itemOT = null;

    /** The is item. */
    private boolean isItem = false;

    /** The is entete. */
    private boolean isEntete = false;

    /** The is entete objet. */
    private boolean isEnteteObjet = false;

    /** The is entete langue. */
    private boolean isEnteteLangue = false;

    /** The is entete code. */
    private boolean isEnteteCode = false;

    /** The is donnee. */
    private boolean isDonnee = false;

    /** The is ot. */
    private boolean isOT = false;

    /** The is fw. */
    private boolean isFW = false;

    /** The is metier. */
    private boolean isMetier = false;

    /** The libelle en cours. */
    private String libelleEnCours = null;

    /** The donnee fw. */
    private Map<String, String> donneeFW = null;

    /**
     * Instantiates a new handler csv.
     */
    public HandlerCSV() {
        String nomClasse;
        for (IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
            if (plugin.getObjets() != null) {
                for (String objet : plugin.getObjets()) {
                    nomClasse = objet.substring(objet.lastIndexOf(".") + 1);
                    lstOT.add(nomClasse.toUpperCase());
                }
            }
        }
    }

    /* (non-Javadoc)
     * @see org.xml.sax.helpers.DefaultHandler#startDocument()
     */
    @Override
    public void startDocument() throws SAXException {
        item = new Item();
    }

    /* (non-Javadoc)
     * @see org.xml.sax.helpers.DefaultHandler#endDocument()
     */
    @Override
    public void endDocument() throws SAXException {
        // sax2Dom.endDocument();
    }

    /* (non-Javadoc)
     * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attribs) {
        if (ITEM.equals(localName)) {
            isItem = true;
            item = new Item();
        } else if (ENTETE.equals(localName)) {
            isEntete = true;
        } else if (isEntete) {
            if (CODE.equals(localName)) {
                isEnteteCode = true;
            } else if (LANGUE.equals(localName)) {
                isEnteteLangue = true;
            } else if (OBJET.equals(localName)) {
                isEnteteObjet = true;
            }
        } else if (isItem && DONNEE.equals(localName)) {
            isDonnee = true;
        } else if (isItem && FW.equals(localName)) {
            donneeFW = new HashMap<String, String>();
            isFW = true;
        } else if (isItem && METIER.equals(localName)) {
            isMetier = true;
        } else if (isItem && lstOT.contains(localName.toUpperCase())) {
            itemOT = new ItemOT();
            itemOT.setNomItemOT(localName);
            isOT = true;
        } else {
            if (isOT || isFW || isDonnee || isMetier) {
                libelleEnCours = localName;
            }
        }
    }

    // -----
    /* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
    @Override
    public void endElement(String uri, String localName, String qName) {
        if (ITEM.equals(localName)) {
            isItem = false;
            listeItems.add(item);
            item = null;
        } else if (ENTETE.equals(localName)) {
            isEntete = false;
        } else if (isEntete && CODE.equals(localName)) {
            isEnteteCode = false;
        } else if (isEntete && LANGUE.equals(localName)) {
            isEnteteLangue = false;
        } else if (isEntete && OBJET.equals(localName)) {
            isEnteteObjet = false;
        } else if (isItem && DONNEE.equals(localName)) {
            isDonnee = false;
        } else if (isItem && lstOT.contains(localName.toUpperCase())) {
            item.ajouterDonneeOT(itemOT);
            isOT = false;
            itemOT = null;
            libelleEnCours = null;
        } else if (isItem && FW.equals(localName)) {
            item.ajouterDonneeFW(donneeFW);
            isFW = false;
            donneeFW = null;
            libelleEnCours = null;
        } else if (isItem && METIER.equals(localName)) {
            isMetier = false;
            libelleEnCours = null;
        } else {
            libelleEnCours = null;
        }
    }

    // -----
	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
    @Override
    public void characters(char[] data, int start, int length) {
        if (isEnteteCode) {
            item.setCode(getDonnee(data, start, length));
        } else if (isEnteteLangue) {
            item.setLangue(getDonnee(data, start, length));
        } else if (isEnteteObjet) {
            item.setObjet(getDonnee(data, start, length));
        } else if (isDonnee) {
            if (isOT) {
                if (!lstLibellesOT.contains(libelleEnCours)) {
                    lstLibellesOT.add(libelleEnCours);
                }
                itemOT.ajouterDonnee(libelleEnCours, getDonnee(data, start, length));
            } else {
                if (!lstLibellesDonnees.contains(libelleEnCours)) {
                    lstLibellesDonnees.add(libelleEnCours);
                }
                item.ajouterDonnee(Item.DONNEE, libelleEnCours, getDonnee(data, start, length));
            }
        } else if (isMetier) {
            if (!lstLibellesMetier.contains(libelleEnCours)) {
                lstLibellesMetier.add(libelleEnCours);
            }
            item.ajouterDonnee(Item.METIER, libelleEnCours, getDonnee(data, start, length));
        } else if (isFW) {
            if (!lstLibellesFW.contains(libelleEnCours)) {
                lstLibellesFW.add(libelleEnCours);
            }
            ajouterDonneeFW(libelleEnCours, getDonnee(data, start, length));
        }
    }

    /**
     * Gets the liste items.
     *
     * @return the liste items
     */
    public List<Item> getListeItems() {
        return listeItems;
    }

    /**
     * Gets the lst libelles donnees.
     *
     * @return the lst libelles donnees
     */
    public List<String> getLstLibellesDonnees() {
        return lstLibellesDonnees;
    }

    /**
     * Gets the lst libelles fw.
     *
     * @return the lst libelles fw
     */
    public List<String> getLstLibellesFW() {
        return lstLibellesFW;
    }

    /**
     * Gets the lst libelles metier.
     *
     * @return the lst libelles metier
     */
    public List<String> getLstLibellesMetier() {
        return lstLibellesMetier;
    }

    /**
     * Gets the lst libelles ot.
     *
     * @return the lst libelles ot
     */
    public List<String> getLstLibellesOT() {
        return lstLibellesOT;
    }

    /**
     * Gets the donnee.
     *
     * @param data
     *            the data
     * @param start
     *            the start
     * @param length
     *            the length
     *
     * @return the donnee
     */
    private String getDonnee(char[] data, int start, int length) {
        String donnee = "";
        for (int i = start; i < start + length; i++) {
            donnee += data[i];
        }
        return donnee;
    }

    /**
     * Ajouter donnee fw.
     *
     * @param libelle
     *            the libelle
     * @param val
     *            the val
     */
    private void ajouterDonneeFW(String libelle, String val) {
        donneeFW.put(libelle, val);
    }
}
