/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Classe abstraite des paramètres de configuration.
 *
 * @author EBO
 */
public abstract class InfosExportImport {

    /** The code. */
    private String code = null;

    /** The libelle. */
    private String libelle = null;

    /** The format. */
    private String format = "";

    /** The parametres. */
    private Map<String, String> parametres = null;

    /** The transformations. */
    private List<String> transformations = null;

    /** The inclusion fichier gw. */
    private boolean inclusionFichierGW = true;

    /** The inclusion ot. */
    private boolean inclusionOT = true;

    /**
     * Inits the.
     */
    public void init() {
        code = "";
        libelle = "";
        parametres = new HashMap<String, String>();
        transformations = new ArrayList<String>();
        inclusionFichierGW = true;
        inclusionOT = true;
    }

    /**
     * Gets the code.
     *
     * @return the code
     */
    public String getCode() {
        return code;
    }

    /**
     * Sets the code.
     *
     * @param code
     *            the new code
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Gets the format.
     *
     * @return the format
     */
    public String getFormat() {
        return format;
    }

    /**
     * Sets the format.
     *
     * @param format
     *            the new format
     */
    public void setFormat(String format) {
        this.format = format;
    }

    /**
     * Checks if is inclusion fichier gw.
     *
     * @return true, if is inclusion fichier gw
     */
    public boolean isInclusionFichierGW() {
        return inclusionFichierGW;
    }

    /**
     * Sets the inclusion fichier gw.
     *
     * @param inclusionFichierGW
     *            the new inclusion fichier gw
     */
    public void setInclusionFichierGW(boolean inclusionFichierGW) {
        this.inclusionFichierGW = inclusionFichierGW;
    }

    /**
     * Checks if is inclusion ot.
     *
     * @return true, if is inclusion ot
     */
    public boolean isInclusionOT() {
        return inclusionOT;
    }

    /**
     * Sets the inclusion ot.
     *
     * @param inclusionOT
     *            the new inclusion ot
     */
    public void setInclusionOT(boolean inclusionOT) {
        this.inclusionOT = inclusionOT;
    }

    /**
     * Gets the libelle.
     *
     * @return the libelle
     */
    public String getLibelle() {
        return libelle;
    }

    /**
     * Sets the libelle.
     *
     * @param libelle
     *            the new libelle
     */
    public void setLibelle(String libelle) {
        this.libelle = libelle;
    }

    /**
     * Gets the parametres.
     *
     * @return the parametres
     */
    public Map<String, String> getParametres() {
        return parametres;
    }

    /**
     * Sets the parametres.
     *
     * @param parametres
     *            the new parametres
     */
    public void setParametres(Map<String, String> parametres) {
        this.parametres = parametres;
    }

    /**
     * Gets the transformations.
     *
     * @return the transformations
     */
    public List<String> getTransformations() {
        return transformations;
    }

    /**
     * Adds the parametre.
     *
     * @param cle
     *            the cle
     * @param valeur
     *            the valeur
     */
    public void addParametre(String cle, String valeur) {
        this.parametres.put(cle, valeur);
    }

    /**
     * Adds the transformation.
     *
     * @param transformation
     *            the transformation
     */
    public void addTransformation(String transformation) {
        this.transformations.add(transformation);
    }
}
