/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.xml.util;

/**
 * The Class InfosImport.
 */
public class InfosImport extends InfosExportImport {

    /**
     * The type generation.
     */
    private String typeGeneration = null;

    /**
     * The type import.
     */
    private String typeImport = null;

    /**
     * The source synchronization.
     */
    private String sourceImport = null;

    /**
     * Import des noeuds vides (peut être basé sur la source d'import).
     */
    private boolean importNoeudsVides;

    /**
     * The source batch.
     */
    private String sourceBatch = null;

    /**
     * The synchronized import.
     */
    private boolean synchronizedImport = false;

    /**
     * The all source import.
     */
    private boolean allSourceImport = false;

    /**
     * Path d'un fichier local
     */
    private String pathFichier = null;

    /**
     * Nom du fichier local
     */
    private String nameFichier = null;

    /**
     * Pattern des fichiers à prendre en compte
     */
    private String patternFichiers = null;

    /**
     * Instantiates a new infos import.
     */
    public InfosImport() {
        // init
        init();
    }

    /* (non-Javadoc)
     * @see com.univ.xml.util.InfosExportImport#init()
     */
    @Override
    public void init() {
        super.init();
        typeGeneration = "";
        typeImport = "";
        sourceImport = "";
        sourceBatch = "";
        pathFichier = "";
        patternFichiers = "";
    }

    /**
     * Gets the source synchronization.
     *
     * @return the source synchronization
     */
    public String getSourceImport() {
        return sourceImport;
    }

    /**
     * Sets the source synchronization.
     *
     * @param sourceSynchronization the new source synchronization
     */
    public void setSourceImport(String sourceSynchronization) {
        this.sourceImport = sourceSynchronization;
    }

    /**
     * Gets the type generation.
     *
     * @return the type generation
     */
    public String getTypeGeneration() {
        return typeGeneration;
    }

    /**
     * Sets the type generation.
     *
     * @param typeGeneration the new type generation
     */
    public void setTypeGeneration(String typeGeneration) {
        this.typeGeneration = typeGeneration;
    }

    /**
     * Retourne le nom du fichier local d'upload uniquement si le type est égal à "UPLOAD".
     *
     * @return the path fichier
     */
    public String getPathFichier() {
        return pathFichier;
    }

    /**
     * Sets the path fichier.
     *
     * @param pathFichier the new path fichier
     */
    public void setPathFichier(String pathFichier) {
        this.pathFichier = pathFichier;
    }

    /**
     * Gets the type import.
     *
     * @return the type import
     */
    public String getTypeImport() {
        return typeImport;
    }

    /**
     * Sets the type import.
     *
     * @param typeImport the new type import
     */
    public void setTypeImport(String typeImport) {
        this.typeImport = typeImport;
    }

    /**
     * Checks if is synchronized import.
     *
     * @return true, if is synchronized import
     */
    public boolean isSynchronizedImport() {
        return synchronizedImport;
    }

    /**
     * Sets the synchronized import.
     *
     * @param synchronizedImport the new synchronized import
     */
    public void setSynchronizedImport(boolean synchronizedImport) {
        this.synchronizedImport = synchronizedImport;
    }

    /**
     * Checks if is all source import.
     *
     * @return true, if is all source import
     */
    public boolean isAllSourceImport() {
        return allSourceImport;
    }

    /**
     * Sets the all source import.
     *
     * @param allSourceImport the new all source import
     */
    public void setAllSourceImport(boolean allSourceImport) {
        this.allSourceImport = allSourceImport;
    }

    public String getNameFichier() {
        return nameFichier;
    }

    public void setNameFichier(final String nameFichier) {
        this.nameFichier = nameFichier;
    }

    public String getPatternFichiers() {
        return patternFichiers;
    }

    public void setPatternFichiers(final String patternFichiers) {
        this.patternFichiers = patternFichiers;
    }

    public boolean isImportNoeudsVides() {
        return importNoeudsVides;
    }

    public void setImportNoeudsVides(final boolean importNoeudsVides) {
        this.importNoeudsVides = importNoeudsVides;
    }

    public String getSourceBatch() {
        return sourceBatch;
    }

    public void setSourceBatch(final String sourceBatch) {
        this.sourceBatch = sourceBatch;
    }
}
