/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
// TODO: Auto-generated Javadoc

/**
 * The Class Item.
 *
 * @author Romain To change the template for this generated type comment go to Window - Preferences - Java - Code Generation - Code and Comments
 */
public class Item {

    /** The OT. */
    public static int OT = 0;

    /** The FW. */
    public static int FW = 1;

    /** The DONNEE. */
    public static int DONNEE = 2;

    /** The METIER. */
    public static int METIER = 3;

    /** The valeurs donnees. */
    Map<String, String> valeursDonnees = new HashMap<String, String>();

    /** The valeurs ot. */
    List<ItemOT> valeursOT = new ArrayList<ItemOT>();

    /** The valeurs fw. */
    List<Map<String, String>> valeursFW = new ArrayList<Map<String, String>>();

    /** The valeurs metier. */
    Map<String, String> valeursMetier = new HashMap<String, String>();

    /** The entete. */
    Entete entete = new Entete();

    /**
     * Gets the dONNEE.
     *
     * @return the dONNEE
     */
    public static int getDONNEE() {
        return DONNEE;
    }

    /**
     * Gets the fW.
     *
     * @return the fW
     */
    public static int getFW() {
        return FW;
    }

    /**
     * Gets the mETIER.
     *
     * @return the mETIER
     */
    public static int getMETIER() {
        return METIER;
    }

    /**
     * Gets the oT.
     *
     * @return the oT
     */
    public static int getOT() {
        return OT;
    }

    /**
     * Ajouter donnee ot.
     *
     * @param donnee
     *            the donnee
     */
    public void ajouterDonneeOT(final ItemOT donnee) {
        this.valeursOT.add(donnee);
    }

    /**
     * Ajouter donnee fw.
     *
     * @param donnee
     *            the donnee
     */
    public void ajouterDonneeFW(final Map<String, String> donnee) {
        this.valeursFW.add(donnee);
    }

    /**
     * Ajouter donnee.
     *
     * @param typeDonnee
     *            the type donnee
     * @param libelle
     *            the libelle
     * @param val
     *            the val
     */
    public void ajouterDonnee(final int typeDonnee, final String libelle, String val) {
        if (typeDonnee == DONNEE) {
            if (valeursDonnees.containsKey(libelle)) {
                val = valeursDonnees.get(libelle) + val;
            }
            valeursDonnees.put(libelle, val);
        } else if (typeDonnee == METIER) {
            if (valeursMetier.containsKey(libelle)) {
                val = valeursMetier.get(libelle) + val;
            }
            valeursMetier.put(libelle, val);
        }
    }

    /**
     * Clear.
     */
    public void clear() {
        valeursDonnees.clear();
        valeursOT.clear();
        valeursFW.clear();
        valeursMetier.clear();
        entete.clear();
    }

    /**
     * Gets the entete.
     *
     * @return the entete
     */
    public Entete getEntete() {
        return entete;
    }

    /**
     * Gets the valeurs donnees.
     *
     * @return the valeurs donnees
     */
    public Map<String, String> getValeursDonnees() {
        return valeursDonnees;
    }

    /**
     * Gets the valeurs fw.
     *
     * @return the valeurs fw
     */
    public List<Map<String, String>> getValeursFW() {
        return valeursFW;
    }

    /**
     * Gets the valeurs metier.
     *
     * @return the valeurs metier
     */
    public Map<String, String> getValeursMetier() {
        return valeursMetier;
    }

    /**
     * Gets the valeurs ot.
     *
     * @return the valeurs ot
     */
    public List<ItemOT> getValeursOT() {
        return valeursOT;
    }

    /**
     * Gets the code.
     *
     * @return the code
     */
    public String getCode() {
        return entete.getCode();
    }

    /**
     * Sets the code.
     *
     * @param val
     *            the new code
     */
    public void setCode(final String val) {
        entete.setCode(val);
    }

    /**
     * Gets the langue.
     *
     * @return the langue
     */
    public String getLangue() {
        return entete.getLangue();
    }

    /**
     * Sets the langue.
     *
     * @param val
     *            the new langue
     */
    public void setLangue(final String val) {
        entete.setLangue(val);
    }

    /**
     * Gets the objet.
     *
     * @return the objet
     */
    public String getObjet() {
        return entete.getObjet();
    }

    /**
     * Sets the objet.
     *
     * @param val
     *            the new objet
     */
    public void setObjet(final String val) {
        entete.setObjet(val);
    }

    /**
     * Retirer guillemets.
     *
     * @param val
     *            the val
     *
     * @return the string
     */
    private String retirerGuillemets(String val) {
        if (val.startsWith("\"")) {
            val = val.substring(1);
        }
        if (val.endsWith("\"")) {
            val = val.substring(0, val.length() - 1);
        }
        return val;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        final StringBuffer sbf = new StringBuffer();
        String cle = null;
        String val = null;
        sbf.append("<ITEM>");
        //on écrit l'entête
        sbf.append(entete.toString());
        //on écrit les données
        if (valeursDonnees.size() > 0) {
            sbf.append("<DONNEES>");
            for (final Iterator<String> it = valeursDonnees.keySet().iterator(); it.hasNext(); ) {
                cle = it.next();
                val = valeursDonnees.get(cle);
                sbf.append("<" + cle.toUpperCase() + ">" + retirerGuillemets(val) + "</" + cle.toUpperCase() + ">");
            }
            //on écrit les OT
            ItemOT itemOt = null;
            String nomOT = null;
            for (final Iterator<ItemOT> it = valeursOT.iterator(); it.hasNext(); ) {
                itemOt = it.next();
                if (nomOT == null) {
                    nomOT = itemOt.getNomItem();
                    sbf.append("<" + nomOT + "S" + ">");
                } else if (!itemOt.getNomItem().equalsIgnoreCase(nomOT)) {
                    sbf.append("</" + nomOT + "S" + ">");
                    nomOT = itemOt.getNomItem();
                    sbf.append("<" + nomOT + "S" + ">");
                }
                sbf.append(itemOt.toString());
            }
            if (nomOT != null) {
                sbf.append("</" + nomOT + "S" + ">");
            }
            sbf.append("</DONNEES>");
        }
        //les FW
        Map<String, String> tableDonneesFW = null;
        if (valeursFW.size() > 0) {
            sbf.append("<FICHIERSGW>");
        }
        for (final Map<String, String> map : valeursFW) {
            tableDonneesFW = map;
            for (final Iterator<String> itFW = tableDonneesFW.keySet().iterator(); itFW.hasNext(); ) {
                cle = itFW.next();
                val = tableDonneesFW.get(cle);
                sbf.append("<" + cle.toUpperCase() + ">" + retirerGuillemets(val) + "</" + cle.toUpperCase() + ">");
            }
        }
        if (valeursFW.size() > 0) {
            sbf.append("</FICHIERSGW>");
        }
        //les Metier
        //on écrit les données
        if (valeursMetier.size() > 0) {
            sbf.append("<METIER>");
        }
        for (final Iterator<String> it = valeursMetier.keySet().iterator(); it.hasNext(); ) {
            cle = it.next();
            val = valeursMetier.get(cle);
            sbf.append("<" + cle.toUpperCase() + ">" + retirerGuillemets(val) + "</" + cle.toUpperCase() + ">");
        }
        if (valeursMetier.size() > 0) {
            sbf.append("</METIER>");
        }
        sbf.append("</ITEM>");
        return sbf.toString();
    }
}
