/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: importExport
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * 
 */
package com.univ.xml.util;

import com.jsbsoft.jtf.exception.ErreurApplicative;
import com.jsbsoft.jtf.lang.CharEncoding;
import com.kportal.extension.module.plugin.objetspartages.IPluginFiche;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.univ.objetspartages.bean.RessourceBean;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.ReferentielObjets;
import com.univ.utils.IntrospectionWrapper;
import com.univ.xml.IXmlCstes;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
// TODO: Auto-generated Javadoc

/**
 * The Class ListeItem.
 *
 * @author Romain To change the template for this generated type comment go to Window - Preferences - Java - Code Generation - Code and Comments
 */
public class ListeItem implements IXmlCstes {

    /** The nom tag fichier gw. */
    private static String nomTagFichierGW = null;

    /** Logger. */
    private static Logger log = LoggerFactory.getLogger(ListeItem.class);

    static {
        init();
    }

    /** The table attribs ot. */
    private Map<String, Set<String>> tableAttribsOT = null;

    /** The items. */
    private List<Item> items = new ArrayList<Item>();

    /** The item en cours. */
    private Item itemEnCours = null;

    /** The liste attribs item. */
    private List<String> listeAttribsItem = null;

    /** The liste attributs fichier gw. */
    private List<String> listeAttributsFichierGW = null;

    /** The col names. */
    private String[] colNames = null;

    /** The nb cols fiche. */
    private int nbColsFiche = 0;

    /** The nb cols ot. */
    private int nbColsOT = 0;

    /** The nb cols fw. */
    private int nbColsFw = 0;

    /** The ot compte. */
    private boolean otCompte = false;

    /** The fw compte. */
    private boolean fwCompte = false;

    /**
     * Instantiates a new liste item.
     */
    public ListeItem() {
        items = new ArrayList<Item>();
    }

    /**
     * Inits the.
     */
    private static void init() {
        String nomTagGW = RessourceBean.class.getName();
        nomTagGW = nomTagGW.substring(nomTagGW.lastIndexOf(".") + 1);
        nomTagGW = nomTagGW.toUpperCase();
        nomTagFichierGW = nomTagGW;
    }

    /**
     * Sets the col names.
     *
     * @param colNames
     *            the new col names
     */
    public void setColNames(final String[] colNames) {
        this.colNames = colNames;
    }

    /**
     * Méthode retournant la liste des attributs des objets techniques d'un objet pour lesquels un getter ainsi qu'un setter a été définis La table contient des éléments du style:
     * <code>clé</code>:<code>valeur</code> où clé représente le nom du sous OT et valeur une liste d'éléments.
     *
     * @param objet
     *            the objet
     *
     * @return table pré-remplie contenant des éléments du style: <code>clé</code>:<code>valeur</code> où clé représente le nom du sous OT et valeur une liste d'éléments throws
     *         Exception exception liée à l'instanciation de l'objet
     *
     * @throws Exception
     *             the exception
     */
    private Map<String, Set<String>> getAttributsOT(final String objet) throws Exception {
        final Map<String, Set<String>> table = new HashMap<String, Set<String>>();
        final FicheUniv fiche = ReferentielObjets.instancierFiche(objet);
        final List<String> classNames = new ArrayList<String>();
        for (final IPluginFiche plugin : PluginFicheHelper.getPlugins()) {
            if (plugin.isActive(fiche.getClass().getName())) {
                if (plugin.getObjets() != null) {
                    classNames.addAll(plugin.getObjets());
                }
            }
        }
        String nomClasse = null;
        for (final String classe : classNames) {
            nomClasse = classe.substring(classe.lastIndexOf(".") + 1);
            final IntrospectionWrapper reflect = IntrospectionWrapper.getIntrospectionWrapper(Class.forName(classe));
            // on obtient la liste des getters
            final Map<String, Method> tableGetters = reflect.getLstGetter();
            table.put(nomClasse.toLowerCase(), tableGetters.keySet());
        }
        return table;
    }

    /**
     * Traiter ligne.
     *
     * @param values
     *            the values
     *
     * @throws ErreurApplicative
     *             the erreur applicative
     */
    public void traiterLigne(final String[] values) throws ErreurApplicative {
        final String objet = values[0];
        final String code = values[1];
        final String langue = values[2];
        try {
            if (itemEnCours == null || !code.equals(itemEnCours.getCode())) {
                if (itemEnCours == null || (itemEnCours != null && objet != null && !objet.equals(itemEnCours.getObjet()))) {
                    //récupération des attributs de la fiche Univ traitée
                    listeAttribsItem = getListeAttributsFicheUniv(objet);
                    //récupération des attributs des objets techniques de cette classe
                    tableAttribsOT = getAttributsOT(objet);
                }
                // nouvel item
                if (itemEnCours != null) {
                    items.add(itemEnCours);
                }
                itemEnCours = new Item();
                itemEnCours.setCode(code);
                itemEnCours.setLangue(langue);
                itemEnCours.setObjet(objet);
                //on change d'objet métier alors on réinitialise les compteurs
                nbColsOT = 0;
                nbColsFw = 0;
                nbColsFiche = 0;
                // traitement ligne item
                for (int i = 3; i < values.length; i++) {
                    if (listeAttribsItem.contains(formatNomObjetFromTag(colNames[i]))) {
                        nbColsFiche++;
                        itemEnCours.ajouterDonnee(Item.DONNEE, colNames[i], values[i]);
                    }
                }
            } else if (objet.indexOf("#") != -1) {
                // sous objet. on récupère le nom de l'objet et/ou son type
                final String sousObjet = objet.substring(objet.indexOf("#") + 1);
                if (nomTagFichierGW.equals(sousObjet)) {
                    if (listeAttributsFichierGW == null) {
                        listeAttributsFichierGW = getListeAttributsFicheUniv(nomTagFichierGW);
                        nbColsFw = listeAttributsFichierGW.size();
                    }
                    // FichierGW
                    //calcul du nombre de colonnes à éviter (ficheUniv)
                    final int nbColsIgnore = 3 + nbColsFiche + nbColsOT;
                    final Map<String, String> tableGw = new HashMap<String, String>();
                    for (int i = nbColsIgnore; i < colNames.length; i++) {
                        if (listeAttributsFichierGW.contains(formatNomObjetFromTag(colNames[i]))) {
                            tableGw.put(colNames[i], values[i]);
                            if (!fwCompte) {
                                nbColsFw++;
                            }
                        }
                    }
                    fwCompte = true;
                    itemEnCours.ajouterDonneeFW(tableGw);
                } else if (tableAttribsOT.get(sousObjet.toLowerCase()) != null) {
                    // OT
                    final Collection<String> listAttribsOt = tableAttribsOT.get(sousObjet.toLowerCase());
                    nbColsOT = listAttribsOt.size();
                    final ItemOT itemOt = new ItemOT();
                    itemOt.setNomItemOT(sousObjet.toUpperCase());
                    // calcul du nombre de colonnes à éviter (ficheUniv)
                    final int nbColsIgnore = 3 + nbColsFiche;
                    for (int i = nbColsIgnore; i < colNames.length; i++) {
                        if (listAttribsOt.contains(formatNomObjetFromTag(colNames[i]))) {
                            itemOt.ajouterDonnee(colNames[i], values[i]);
                            if (!otCompte) {
                                nbColsOT++;
                            }
                        }
                    }
                    otCompte = true;
                    itemEnCours.ajouterDonneeOT(itemOt);
                } else if (NOM_TAG_METIER.equals(sousObjet)) {
                    int nbColsIgnore = 0;
                    //calcul des colonnes objet à ne pas prendre en compte
                    //On ne prend pas en compte les 3 premières colonnes (ie: OBJET;CODE;LANGUE)
                    //et on ne prend pas en compte les colonnes concernant les OT, FW et FicheUniv calculés précédemment
                    nbColsIgnore = nbColsFiche + nbColsFw + nbColsOT + 3;
                    for (int i = nbColsIgnore; i < colNames.length; i++) {
                        itemEnCours.ajouterDonnee(Item.METIER, colNames[i], values[i]);
                    }
                }
            }
        } catch (final Exception e) {
            final String messErr = "Erreur lors de la récupération des attributs des objets traités";
            log.error(messErr, e);
            throw new ErreurApplicative(messErr);
        }
    }

    /**
     * Méthode convertissant un tag vers un nom de méthode.
     *
     * @param nomTag
     *            the nom tag
     *
     * @return the string
     */
    private String formatNomObjetFromTag(final String nomTag) {
        String nomObjet = "";
        String car = "";
        for (int i = 0; i < nomTag.length(); i++) {
            car = Character.toString(nomTag.charAt(i));
            if (i == 0) {
                nomObjet += car.toUpperCase();
            } else if ("_".equals(car)) {
                i++;
                if (i < nomTag.length()) {
                    nomObjet += Character.toString(nomTag.charAt(i)).toUpperCase();
                }
            } else {
                nomObjet += car.toLowerCase();
            }
        }
        return nomObjet;
    }

    /**
     * Gets the liste attributs fiche univ.
     *
     * @param objet
     *            the objet
     *
     * @return the liste attributs fiche univ
     *
     * @throws Exception
     *             the exception
     */
    private List<String> getListeAttributsFicheUniv(final String objet) throws Exception {
        final FicheUniv ficheUniv = ReferentielObjets.instancierFiche(objet);
        final IntrospectionWrapper reflect = IntrospectionWrapper.getIntrospectionWrapper(ficheUniv.getClass());
        // on obtient la liste des getters
        final Map<String, Method> tableGetters = reflect.getLstGetter();
        return new ArrayList<String>(tableGetters.keySet());
    }

    /**
     * Méthode permettant de retourner la liste d'Items récupérés depuis le csv en XML.
     *
     * @return chaine XML
     */
    @Override
    public String toString() {
        if (itemEnCours != null) {
            items.add(itemEnCours);
        }
        final StringBuffer sbf = new StringBuffer();
        sbf.append("<?xml version=\"1.0\" encoding=\"" + CharEncoding.DEFAULT + "\" ?>");
        sbf.append("<LISTE_ITEMS>");
        Item item = null;
        for (final Iterator<Item> it = items.iterator(); it.hasNext(); ) {
            item = it.next();
            sbf.append(item.toString());
        }
        sbf.append("</LISTE_ITEMS>");
        return sbf.toString();
    }
}
