/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: annuairesup
 * Version: 1.08.00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.univ.autocomplete.processus;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.session.SessionUtilisateur;
import com.ksup.objetspartages.bean.AnnuaireksupBean;
import com.ksup.objetspartages.dao.impl.AnnuaireksupDAO;
import com.univ.autocomplete.bean.AutoCompleteSuggestion;
import com.univ.objetspartages.om.AutorisationBean;
import com.univ.utils.ContexteUtil;
import com.univ.utils.SessionUtil;

public class AnnuaireksupAutoCompletion extends GestionAutoCompletion {

    public static final String ID_BEAN = "userAutoComplete";

    private static final Logger LOG = LoggerFactory.getLogger(AnnuaireksupAutoCompletion.class);

    private AnnuaireksupDAO annuaireksupDAO;

    public void setAnnuaireksupDAO(final AnnuaireksupDAO annuaireksupDAO) {
        this.annuaireksupDAO = annuaireksupDAO;
    }

    /**
     * A partir de l'objet FicheUniv, on retrouve l'ensemble des fiches et on les map dans une liste de résultat exploitable
     * @return
     */
    private List<AutoCompleteSuggestion> getResults(final AutorisationBean autorisations, final List<AnnuaireksupBean> fiches) {
        final List<AutoCompleteSuggestion> resultats = new ArrayList<>();
        for (final AnnuaireksupBean currentFiche : fiches) {
            final AutoCompleteSuggestion auto = buildSuggestion(currentFiche);
            if(auto != null) {
                resultats.add(auto);
            }
        }
        return resultats;
    }

    private AutoCompleteSuggestion buildSuggestion(AnnuaireksupBean annuaireksupBean) {
        if(hasMandatoryInfos(annuaireksupBean)) {
            final AutoCompleteSuggestion suggestion = new AutoCompleteSuggestion();
            suggestion.setData(annuaireksupBean.getAdresseMail());
            suggestion.setValue(String.format("%s %s (%s)", annuaireksupBean.getPrenom(), annuaireksupBean.getNom(), annuaireksupBean.getAdresseMail()));
            return suggestion;
        }
        return null;
    }

    private boolean hasMandatoryInfos(final AnnuaireksupBean annuaireksupBean) {
        return StringUtils.isNotBlank(annuaireksupBean.getAdresseMail()) &&
               StringUtils.isNotBlank(annuaireksupBean.getNom());
    }

    @Override
    public List<AutoCompleteSuggestion> traiterRechercheDepuisRequete(final HttpServletRequest req) {
        List<AutoCompleteSuggestion> resultats = new ArrayList<>();
        final AutorisationBean autorisations = (AutorisationBean) SessionUtil.getInfosSession(req).get(SessionUtilisateur.AUTORISATIONS);
        if (autorisations == null) {
            return resultats;
        }
        try {
            resultats = getResults(autorisations, annuaireksupDAO.select(""));
        } catch (final Exception e) {
            LOG.error("Impossible d'executer la recherche", e);
            resultats = Collections.emptyList();
        }
        return resultats;
    }

    @Override
    public String getKeyForCache() {
        String code = "anonyme";
        if (ContexteUtil.getContexteUniv() != null) {
            if (StringUtils.isNotEmpty(ContexteUtil.getContexteUniv().getCode())) {
                code = ContexteUtil.getContexteUniv().getCode();
            }
        }
        return String.format("%s_%s", ID_BEAN, code);
    }
}
